@@ -34,6 +34,7 @@
 #include "gpt_control.h"
 #include "lib_private.h"
 #include "cmdstack.h"
+#include "winstack.h"
 
 typedef struct CalcChild_ CalcChild;
 typedef struct test_t_ test_t;
@@ -596,11 +597,8 @@ static void htest_graph (int d, double x, double *parms)
 
     fclose(fp);
 
-    if (gnuplot_make_graph()) {
-	errbox(_("gnuplot command failed"));
-    } else {
-	register_graph(NULL);
-    }
+    err = gnuplot_make_graph();
+    gui_graph_handler(err);
 }
 
 static void dist_graph (int d, double *parms)
@@ -686,11 +684,8 @@ static void dist_graph (int d, double *parms)
 
     fclose(fp);
 
-    if (gnuplot_make_graph()) {
-	errbox(_("gnuplot command failed"));
-    } else {
-	register_graph(NULL);
-    }
+    err = gnuplot_make_graph();
+    gui_graph_handler(err);
 }
 
 #define ALT_CHI MAX_DIST
@@ -1316,7 +1311,6 @@ static void get_pvalue (GtkWidget *w, CalcChild *child)
 
 static int calc_finish_genr (void)
 {
-    char *cmdline = get_lib_cmdline();
     PRN *prn;
     int err = 0;
 
@@ -1324,12 +1318,12 @@ static int calc_finish_genr (void)
 	return 1;
     }
 
-    err = generate(cmdline, dataset, OPT_NONE, prn); 
+    err = generate(get_lib_cmdline(), dataset, OPT_NONE, prn); 
 
     if (err) {
 	errbox(gretl_print_get_buffer(prn));
-	delete_last_command();
     } else {
+	record_command_verbatim();
 	populate_varlist();
 	mark_dataset_as_modified();
     }
@@ -1371,47 +1365,43 @@ static void get_random (GtkWidget *w, CalcChild *child)
 
     switch (d) {
     case UNIFORM_DIST:
-	gretl_command_sprintf("genr %s = randgen(u,%g,%g)", vname,
-			      x[0], x[1]);
+	lib_command_sprintf("genr %s = randgen(u,%g,%g)", vname,
+			    x[0], x[1]);
 	break;
     case NORMAL_DIST:
-	gretl_command_sprintf("genr %s = randgen(N,%g,%g)", vname,
-			      x[0], x[1]);
+	lib_command_sprintf("genr %s = randgen(N,%g,%g)", vname,
+			    x[0], x[1]);
 	break;
     case T_DIST: 
-	gretl_command_sprintf("genr %s = randgen(t,%g)", vname, x[0]);
+	lib_command_sprintf("genr %s = randgen(t,%g)", vname, x[0]);
 	break;
     case CHISQ_DIST:
-	gretl_command_sprintf("genr %s = randgen(X,%g)", vname, x[0]);
+	lib_command_sprintf("genr %s = randgen(X,%g)", vname, x[0]);
 	break;
     case F_DIST:
-	gretl_command_sprintf("genr %s = randgen(F,%g,%g)", vname, 
-			      x[0], x[1]);
+	lib_command_sprintf("genr %s = randgen(F,%g,%g)", vname, 
+			    x[0], x[1]);
 	break;
     case GAMMA_DIST:
-	gretl_command_sprintf("genr %s = randgen(G,%g,%g)", vname, 
-			      x[0], x[1]);
+	lib_command_sprintf("genr %s = randgen(G,%g,%g)", vname, 
+			    x[0], x[1]);
 	break;
     case WEIBULL_DIST:
-	gretl_command_sprintf("genr %s = randgen(W,%g,%g)", vname, 
-			      x[0], x[1]);
+	lib_command_sprintf("genr %s = randgen(W,%g,%g)", vname, 
+			    x[0], x[1]);
 	break;
     case BINOMIAL_DIST: 
-	gretl_command_sprintf("genr %s = randgen(B,%g,%g)", vname, 
-			      x[0], x[1]);
+	lib_command_sprintf("genr %s = randgen(B,%g,%g)", vname, 
+			    x[0], x[1]);
 	break;
     case POISSON_DIST: 
 	/* FIXME allow variable as param? */
-	gretl_command_sprintf("genr %s = randgen(P,%g)", vname, x[0]);
+	lib_command_sprintf("genr %s = randgen(P,%g)", vname, x[0]);
 	break;
     }
 
     gretl_pop_c_numeric_locale();
 
-    if (check_and_record_command()) {
-	return;
-    }
-
     calc_finish_genr();
 }
 
@@ -3066,17 +3056,11 @@ static void real_stats_calculator (int code, gpointer data)
     /* Help button? */
     hcode = calc_help_code(code);
     if (hcode) { 
-	tmp = gtk_button_new_from_stock(GTK_STOCK_HELP);
-	gtk_container_add(GTK_CONTAINER(child->bbox), tmp);
-	gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(child->bbox),
-					   tmp, TRUE);
-	g_signal_connect(G_OBJECT(tmp), "clicked", 
-			 G_CALLBACK(context_help), 
-			 GINT_TO_POINTER(hcode));
+	context_help_button(child->bbox, hcode);
     }
 
     gtk_widget_show_all(child->dlg);
-    add_window_list_item(child->dlg, STAT_TABLE);
+    window_list_add(child->dlg, STAT_TABLE);
 }
 
 /* for gnuplot: convert '^' to '**' for exponentiation */
@@ -3160,10 +3144,9 @@ static void do_plot_curve (GtkWidget *w, struct curve_plotter *p)
     fclose(fp);
 
     err = gnuplot_make_graph();
-    if (err) {
-	gui_errmsg(err);
-    } else {
-	register_graph(NULL);
+    gui_graph_handler(err);
+
+    if (!err) {
 	gtk_widget_destroy(p->dlg);
     }
 }
@@ -3230,7 +3213,7 @@ static void plot_curve (void)
     hbox = gtk_dialog_get_action_area(GTK_DIALOG(dialog));
     
     /* "Cancel" button */
-    cancel_delete_button(hbox, dialog, NULL);
+    cancel_delete_button(hbox, dialog);
 
     /* "OK" button */
     button = ok_button(hbox);
@@ -3301,11 +3284,7 @@ static void do_plot_cdf (GtkWidget *w, GtkWidget *dlg)
     fclose(fp);
 
     err = gnuplot_make_graph();
-    if (err) {
-	gui_errmsg(err);
-    } else {
-	register_graph(NULL);
-    }
+    gui_graph_handler(err);
 
     gtk_widget_destroy(dlg);
 }
@@ -3362,7 +3341,7 @@ static void plot_cdf (GtkWidget *parent)
     hbox = gtk_dialog_get_action_area(GTK_DIALOG(dialog));
     
     /* "Cancel" button */
-    cancel_delete_button(hbox, dialog, NULL);
+    cancel_delete_button(hbox, dialog);
 
     /* "OK" button */
     button = ok_button(hbox);
