@@ -31,6 +31,7 @@
 #include "ssheet.h"
 #include "treeutils.h"
 #include "datawiz.h"
+#include "winstack.h"
 
 #include "boxplots.h"
 
@@ -196,21 +197,20 @@ static int open_data_code (const gchar *s)
 
 void open_data (GtkAction *action)
 {
-    int code;
+    if (!dataset_locked()) {
+	int code = open_data_code(gtk_action_get_name(action));
 
-    if (dataset_locked()) {
-	return;
-    }
+	if (code == OPEN_CSV || code == APPEND_CSV) {
+	    /* allow specification of separator */
+	    int resp = csv_options_dialog(code, GRETL_OBJ_DSET, NULL);
 
-    code = open_data_code(gtk_action_get_name(action));
-
-    if (code == OPEN_CSV || code == APPEND_CSV) {
-	if (csv_options_dialog(NULL)) {
-	    return;
+	    if (canceled(resp)) {
+		return;
+	    }
 	}
-    }
 
-    file_selector(code, FSEL_DATA_NONE, NULL);
+	file_selector(code, FSEL_DATA_NONE, NULL);
+    }
 }
 
 void file_save (windata_t *vwin, int ci)
@@ -309,8 +309,7 @@ static int model_action_code (GtkAction *action)
 void fit_resid_callback (GtkAction *action, gpointer data)
 {
     const gchar *s = gtk_action_get_name(action);
-    windata_t *mydata = (windata_t *) data; 
-    MODEL *pmod = mydata->data;
+    windata_t *vwin = (windata_t *) data; 
     int code = M_UHAT; 
 
     if (!strcmp(s, "yhat")) {
@@ -325,7 +324,7 @@ void fit_resid_callback (GtkAction *action, gpointer data)
 	code = M_AHAT;
     }
 
-    save_fit_resid(pmod, code);
+    save_fit_resid(vwin, code);
 }
 
 /* callback for adding a scalar from a model */
@@ -357,7 +356,7 @@ void model_stat_callback (GtkAction *action, gpointer data)
 	code = M_HQC;
     }
 
-    add_model_stat(pmod, code);
+    add_model_stat(pmod, code, vwin);
 }
 
 void model_callback (GtkAction *action, gpointer data) 
@@ -371,10 +370,11 @@ void model_genr_callback (GtkAction *action, gpointer data)
 {
     windata_t *vwin = (windata_t *) data;
 
-    edit_dialog(_("gretl: add var"), 
+    edit_dialog(MODEL_GENR, _("gretl: add var"), 
 		_("Enter formula for new variable:"),
 		"", do_model_genr, vwin, 
-		MODEL_GENR, VARCLICK_INSERT_NAME, NULL);   
+		VARCLICK_INSERT_NAME, 
+		vwin_toplevel(vwin));   
 }
 
 static int selector_callback_code (const gchar *s)
@@ -407,6 +407,10 @@ static int selector_callback_code (const gchar *s)
 	return ELLIPSE;
     if (!strcmp(s, "VarOmit"))
 	return VAROMIT;
+    if (!strcmp(s, "loess"))
+	return LOESS;
+    if (!strcmp(s, "nadarwat"))
+	return NADARWAT;
 
     return 0;
 }
@@ -419,17 +423,13 @@ void selector_callback (GtkAction *action, gpointer data)
 
     ci = selector_callback_code(s);
 
-    if (ci == ADD || ci == OMIT || ci == COEFFSUM || ci == ELLIPSE) {
-	set_window_busy(vwin);
-    }
-
     if (ci == COINT || ci == COINT2) {
-	selection_dialog(_("gretl: cointegration test"), do_coint, ci);
+	selection_dialog(ci, _("gretl: cointegration test"), do_coint);
     } else if (ci == VAR || ci == VECM) {
-	selection_dialog((ci == VAR)? _("gretl: VAR") : _("gretl: VECM"),
-			 do_vector_model, ci);
+	selection_dialog(ci, (ci == VAR)? _("gretl: VAR") : _("gretl: VECM"),
+			 do_vector_model);
     } else if (ci == VLAGSEL) {
-	selection_dialog(_("gretl: VAR lag selection"), do_vector_model, ci);
+	selection_dialog(ci, _("gretl: VAR lag selection"), do_vector_model);
     } else if (ci == GR_XY || ci == GR_IMP || ci == GR_DUMMY ||
 	       ci == SCATTERS || ci == GR_3D || ci == GR_XYZ ||
 	       ci == GR_FBOX) {
@@ -458,25 +458,36 @@ void selector_callback (GtkAction *action, gpointer data)
 	default:
 	    return;
 	}
-	selection_dialog(_("gretl: define graph"), selfunc, ci);
+	selection_dialog(ci, _("gretl: define graph"), selfunc);
     } else if (ci == ADD || ci == OMIT) {
-	simple_selection(_("gretl: model tests"), do_add_omit, ci, vwin);
+	simple_selection_for_viewer(ci, _("gretl: model tests"), 
+				    do_add_omit, vwin);
     } else if (ci == VAROMIT) {
-	simple_selection(_("gretl: model tests"), do_VAR_omit, ci, vwin);
+	simple_selection_for_viewer(ci, _("gretl: model tests"), 
+				    do_VAR_omit, vwin);
     } else if (ci == COEFFSUM) {
-	simple_selection(_("gretl: model tests"), do_coeff_sum, ci, vwin);
+	simple_selection_for_viewer(ci, _("gretl: model tests"), 
+				    do_coeff_sum, vwin);
     } else if (ci == ELLIPSE) {
-	simple_selection(_("gretl: model tests"), do_confidence_region, ci, vwin);
+	simple_selection_for_viewer(ci, _("gretl: model tests"), 
+				    do_confidence_region, vwin);
     } else if (ci == GR_PLOT) {
-	simple_selection(_("gretl: define graph"), do_graph_from_selector, ci, NULL);
+	simple_selection(ci, _("gretl: define graph"), do_graph_from_selector, 
+			 NULL);
     } else if (ci == TSPLOTS) {
-	simple_selection(_("gretl: define graph"), do_scatters, ci, vwin);
+	simple_selection(ci, _("gretl: define graph"), do_scatters, NULL);
     } else if (ci == SPEARMAN) {
 	char title[64];
 	
 	strcpy(title, "gretl: ");
 	strcat(title, _("rank correlation"));
-	simple_selection(title, do_rankcorr, ci, vwin);
+	simple_selection(ci, title, do_rankcorr, NULL);
+    } else if (ci == LOESS || ci == NADARWAT) {
+	char title[64];
+	
+	strcpy(title, "gretl: ");
+	strcat(title, (ci == LOESS)? _("Loess") : _("Nadaraya-Watson"));
+	selection_dialog(ci, title, do_nonparam_model);
     } else {
 	errbox("selector_callback: code was not recognized");
     }
@@ -514,17 +525,19 @@ static int gretl_callback_code (const gchar *s)
 
 void gretl_callback (GtkAction *action, gpointer data)
 {
+    windata_t *vwin = (windata_t *) data;
+    GtkWidget *parent = NULL;
     const char *title = NULL;
     const char *query = NULL;
     const char *defstr = NULL;
     gchar *dynquery = NULL;
     void (*okfunc)() = NULL;
-    guint varclick = VARCLICK_NONE;
-    int cmd, cancel = 0;
+    Varclick click = VARCLICK_NONE;
+    int ci;
 
-    cmd = gretl_callback_code(gtk_action_get_name(action));
+    ci = gretl_callback_code(gtk_action_get_name(action));
 
-    switch (cmd) {
+    switch (ci) {
     case GENR:
 	title = N_("gretl: add var");
 	if (dataset->n > 5000) {
@@ -534,7 +547,7 @@ void gretl_callback (GtkAction *action, gpointer data)
 		       "(or just a name, to enter data manually)");
 	}
 	okfunc = do_genr;
-	varclick = VARCLICK_INSERT_NAME;
+	click = VARCLICK_INSERT_NAME;
 	break;
     case VSETMISS:
 	title = N_("gretl: missing code");
@@ -552,28 +565,28 @@ void gretl_callback (GtkAction *action, gpointer data)
 	title = N_("gretl: boxplots");
 	query = N_("Specify variables to plot:");
 	okfunc = do_box_graph;
-	varclick = VARCLICK_INSERT_NAME;
+	click = VARCLICK_INSERT_NAME;
 	defstr = get_last_boxplots_string();
 	break;
     case GMM:
 	title = N_("gretl: GMM");
 	query = N_("GMM: Specify function and orthogonality conditions:");
 	okfunc = do_gmm_model;
-	varclick = VARCLICK_INSERT_TEXT;
+	click = VARCLICK_INSERT_TEXT;
 	data = NULL;
 	break;	
     case MLE:
 	title = N_("gretl: maximum likelihood");
 	query = N_("MLE: Specify function, and derivatives if possible:");
 	okfunc = do_mle_model;
-	varclick = VARCLICK_INSERT_TEXT;
+	click = VARCLICK_INSERT_TEXT;
 	data = NULL;
 	break;	
     case NLS:
 	title = N_("gretl: nonlinear least squares");
 	query = N_("NLS: Specify function, and derivatives if possible:");
 	okfunc = do_nls_model;
-	varclick = VARCLICK_INSERT_TEXT;
+	click = VARCLICK_INSERT_TEXT;
 	data = NULL;
 	break;	
     case SYSTEM:
@@ -581,12 +594,13 @@ void gretl_callback (GtkAction *action, gpointer data)
 	query = N_("Specify simultaneous equations:");
 	data = NULL;
 	okfunc = do_eqn_system;
-	varclick = VARCLICK_INSERT_TEXT;
+	click = VARCLICK_INSERT_TEXT;
 	break;
     case RESTRICT:
 	title = N_("gretl: linear restrictions");
 	query = N_("Specify restrictions:");
 	okfunc = do_restrict;
+	parent = vwin_toplevel(vwin);
 	break;	
     case MINIBUF:
 	title = N_("gretl: command entry");
@@ -600,12 +614,12 @@ void gretl_callback (GtkAction *action, gpointer data)
     }
 
     if (dynquery != NULL) {
-	edit_dialog(_(title), dynquery, defstr, okfunc, data, 
-		    cmd, varclick, &cancel);
+	edit_dialog(ci, _(title), dynquery, defstr, okfunc, data, 
+		    click, parent);
 	g_free(dynquery);
     } else {
-	edit_dialog(_(title), _(query), defstr, okfunc, data, 
-		    cmd, varclick, &cancel);
+	edit_dialog(ci, _(title), _(query), defstr, okfunc, data, 
+		    click, parent);
     }
 }
 
@@ -619,7 +633,7 @@ void menu_boxplot_callback (int varnum)
     int ret;
 
     ret = radio_dialog(_("gretl: define graph"), NULL, 
-		       opts, 3, 0, 0);
+		       opts, 3, 0, 0, NULL);
 
     if (ret == 0) {
 	do_boxplot_var(varnum, OPT_NONE);
@@ -627,18 +641,22 @@ void menu_boxplot_callback (int varnum)
 	do_boxplot_var(varnum, OPT_O);
     } else if (ret == 2) {
 	selector_set_varnum(varnum);
-	selection_dialog(_("gretl: define graph"), 
-			 do_factorized_boxplot, GR_FBOX);
+	selection_dialog(GR_FBOX, _("gretl: define graph"), 
+			 do_factorized_boxplot);
     }
 }
 
-void revise_nl_model (MODEL *pmod)
+void boxplot_callback (void)
+{
+    menu_boxplot_callback(mdata_active_var());
+}
+
+void revise_nl_model (MODEL *pmod, GtkWidget *parent)
 {
     const char *title = NULL;
     const char *query = NULL;
     const char *defstr;
     void (*okfunc)() = NULL;
-    int cancel = 0;
 
     switch (pmod->ci) {
     case GMM:
@@ -660,23 +678,19 @@ void revise_nl_model (MODEL *pmod)
 
     defstr = gretl_model_get_data(pmod, "nlinfo");
 
-    edit_dialog(_(title), _(query), defstr, okfunc, pmod, 
-		pmod->ci, VARCLICK_INSERT_TEXT, &cancel);   
+    edit_dialog(pmod->ci, _(title), _(query), defstr, okfunc, pmod, 
+		VARCLICK_INSERT_TEXT, parent);   
 }
 
-void revise_system_model (void *ptr)
+void revise_system_model (void *ptr, GtkWidget *parent)
 {
     equation_system *sys = (equation_system *) ptr;
-    int cancel = 0;
 
-    edit_dialog(_("gretl: simultaneous equations system"), 
-		_("Specify simultaneous equations:"),
-		NULL,
-		do_eqn_system,
-		sys,
-		SYSTEM,
+    edit_dialog(SYSTEM, _("gretl: simultaneous equations system"), 
+		_("Specify simultaneous equations:"), NULL,
+		do_eqn_system, sys,
 		VARCLICK_INSERT_TEXT,
-		&cancel);
+		parent);
 }
 
 void genr_callback (void)
@@ -690,17 +704,17 @@ void genr_callback (void)
 		 "(or just a name, to enter data manually)");
     }
 
-    edit_dialog(_("gretl: add var"), _(msg),
+    edit_dialog(GENR, _("gretl: add var"), _(msg),
 		NULL, do_genr, NULL, 
-		GENR, VARCLICK_INSERT_NAME, NULL);   
+		VARCLICK_INSERT_NAME, NULL);   
 }
 
 void minibuf_callback (void)
 {
-    edit_dialog(_("gretl: command entry"),
-		_("Type a command:"),
-		NULL, do_minibuf, NULL, 
-		MINIBUF, VARCLICK_NONE, NULL);   
+    edit_dialog(MINIBUF, _("gretl: command entry"),
+		_("Type a command:"), NULL, 
+		do_minibuf, NULL, 
+		VARCLICK_NONE, NULL);   
 }
 
 void newdata_callback (void) 
@@ -713,7 +727,7 @@ void newdata_callback (void)
 
     resp = spin_dialog(_("gretl: create data set"), NULL, &n, 
 		       _("Number of observations:"), 
-		       2, 1000000, 0);
+		       2, 1000000, 0, NULL);
 
     if (resp < 0) {
 	/* canceled */
@@ -737,7 +751,7 @@ void xcorrgm_callback (void)
 
 	strcpy(title, "gretl: ");
 	strcat(title, _("cross-correlogram"));
-	simple_selection(title, do_xcorrgm, XCORRGM, NULL);
+	simple_selection(XCORRGM, title, do_xcorrgm, NULL);
     }
 }
 
@@ -779,68 +793,10 @@ void do_nistcheck (GtkAction *action)
     g_free(fname);
 }
 
-#if defined (ENABLE_MAILER) && !defined(G_OS_WIN32)
-
-#if 0
-
-static gchar *my_url_encode (gchar const *text, int type)
-{
-    char const *good;
-    static char const hex[16] = "0123456789ABCDEF";
-    GString* result;
-
-    g_return_val_if_fail(text != NULL, NULL);
-    g_return_val_if_fail(*text != '\0', NULL);
-
-    switch (type) {
-    case 0: /* mailto: */
-	good = ".-_@";
-	break;
-    case 1: /* file: or http: */
-	good = "!$&'()*+,-./:=@_";
-	break;
-    default:
-	return NULL;
-    }
-
-    result = g_string_new(NULL);
-
-    while (*text) {
-	unsigned char c = *text++;
-	if (g_ascii_isalnum(c) || strchr(good, c))
-	    g_string_append_c(result, c);
-	else {
-	    g_string_append_c(result, '%');
-	    g_string_append_c(result, hex[c >> 4]);
-	    g_string_append_c(result, hex[c & 0xf]);
-	}
-    }
-
-    return g_string_free(result, FALSE);
-}
-
-#endif
+#if defined(ENABLE_MAILER) && !defined(G_OS_WIN32)
 
 void send_file (char *fullname)
 {
-#if 0 /* if GTK_MAJOR_VERSION > 2 || GTK_MINOR_VERSION >= 14 */
-    GError *error = NULL;
-    gchar *uri, *tmp;
-
-    tmp = my_url_encode(fullname, 0);
-    /* this idea is from gnumeric, but it doesn't work with thunderbird */
-    uri = g_strdup_printf("mailto:?subject=gretl data file&"
-			  "body=Attached&attachment=%s", tmp);
-    fprintf(stderr, "uri: '%s'\n", uri);
-    g_free(tmp);
-
-    gtk_show_uri(NULL, uri, GDK_CURRENT_TIME, &error);
-    if (error != NULL) {
-        errbox(error->message);
-        g_error_free(error);
-    }
-    g_free(uri);
-# else
     int (*email_file) (const char *, const char *);
     void *handle;
 
@@ -851,7 +807,6 @@ void send_file (char *fullname)
     
     email_file(fullname, gretl_dotdir());
     close_plugin(handle);
-# endif
 }
 
 #endif
