@@ -194,16 +194,12 @@ static void graph_dbdata (DATASET *dbset)
 	err = gnuplot(list, NULL, dbset, OPT_G | OPT_O | OPT_T);
     }
 
-    if (err) {
-	gui_errmsg(err);
-    } else {
-	register_graph(NULL);
-    }
-
     free(list);
+    gui_graph_handler(err);
 }
 
-static int expand_data_dialog (int src_pd, int targ_pd, int *interpol)
+static int expand_data_dialog (int src_pd, int targ_pd, int *interpol,
+			       GtkWidget *parent)
 {
     int mult = targ_pd / src_pd;
     int resp;
@@ -218,7 +214,7 @@ static int expand_data_dialog (int src_pd, int targ_pd, int *interpol)
 
 	resp = radio_dialog("gretl", _("Adding a lower frequency series to a\n"
 				       "higher frequency dataset"),
-			    opts, 2, 0, 0);
+			    opts, 2, 0, 0, parent);
 	if (resp == 0) {
 	    *interpol = 1;
 	    resp = GRETL_YES;
@@ -305,36 +301,34 @@ static void record_db_open_command (dbwrapper *dw)
 {
     if (dw->fname != NULL) {
 	int quotes = (strchr(dw->fname, ' ') != NULL);
-	gchar *cmdline;
 
 	if (dw->dbtype == GRETL_PCGIVE_DB) {
 	    if (quotes) {
-		cmdline = g_strdup_printf("open \"%s.bn7\"", dw->fname);
+		lib_command_sprintf("open \"%s.bn7\"", dw->fname);
 	    } else {
-		cmdline = g_strdup_printf("open %s.bn7", dw->fname);
+		lib_command_sprintf("open %s.bn7", dw->fname);
 	    }
 	} else if (dw->dbtype == GRETL_NATIVE_DB) {
 	    const char *tmp = trimmed_db_name(dw->fname);
 
 	    if (tmp != NULL) {
-		cmdline = g_strdup_printf("open %s.bin", tmp);
+		lib_command_sprintf("open %s.bin", tmp);
 	    } else if (quotes) {
-		cmdline = g_strdup_printf("open \"%s.bin\"", dw->fname);
+		lib_command_sprintf("open \"%s.bin\"", dw->fname);
 	    } else {
-		cmdline = g_strdup_printf("open %s.bin", dw->fname);
+		lib_command_sprintf("open %s.bin", dw->fname);
 	    }
 	} else if (dw->dbtype == GRETL_NATIVE_DB_WWW) {
-	    cmdline = g_strdup_printf("open %s --www", dw->fname);
+	    lib_command_sprintf("open %s --www", dw->fname);
 	} else {
 	    if (quotes) {
-		cmdline = g_strdup_printf("open \"%s\"", dw->fname);
+		lib_command_sprintf("open \"%s\"", dw->fname);
 	    } else {
-		cmdline = g_strdup_printf("open %s", dw->fname);
+		lib_command_sprintf("open %s", dw->fname);
 	    }
 	}
 
-	record_command_verbatim(cmdline);
-	g_free(cmdline);
+	record_command_verbatim();
     }
 }
 
@@ -343,7 +337,6 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 {
     SERIESINFO *sinfo;
     CompactMethod method = COMPACT_AVG;
-    gchar *cmdline = NULL;
     int resp, warned = 0, chosen = 0;
     int i, t, err = 0;
 
@@ -370,7 +363,8 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	}
 
 	if (sinfo->pd < dataset->pd && !warned) {
-	    resp = expand_data_dialog(sinfo->pd, dataset->pd, &interpol);
+	    resp = expand_data_dialog(sinfo->pd, dataset->pd, &interpol,
+				      vwin->main);
 	    if (resp != GRETL_YES) {
 		return 0;
 	    }
@@ -407,8 +401,8 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	    /* the series needs to be compacted */
 	    compact = 1;
 	    if (!chosen) {
-		data_compact_dialog(vwin->main, sinfo->pd, &dataset->pd, NULL, 
-				    &method, NULL);
+		data_compact_dialog(sinfo->pd, &dataset->pd, NULL, 
+				    &method, NULL, vwin->main);
 		if (method == COMPACT_NONE) {
 		    if (!overwrite) {
 			dataset_drop_last_variables(1, dataset);
@@ -439,15 +433,14 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 
 	/* record successful importation in command log */
 	if (compact && (cstr = compact_method_string(method)) != NULL) {
-	    cmdline = g_strdup_printf("data (compact=%s) %s", cstr,
-				      sinfo->varname);
+	    lib_command_sprintf("data (compact=%s) %s", cstr,
+				sinfo->varname);
 	} else {
 	    /* FIXME: handle expand/interpolate option */
-	    cmdline = g_strdup_printf("data %s", sinfo->varname);
+	    lib_command_sprintf("data %s", sinfo->varname);
 	}
 
-	record_command_verbatim(cmdline);
-	g_free(cmdline);
+	record_command_verbatim();
 
 	/* common stuff for adding a var */
 	strcpy(dataset->varname[dbv], sinfo->varname);
@@ -500,8 +493,6 @@ static void add_dbdata (windata_t *vwin, DATASET *dbset,
 	add_db_series_to_dataset(vwin, dbset->Z, dw);
     } else {  
 	/* no data open: start new data set from db */
-	gchar *cmdline;
-
 	destroy_dataset(dataset);
 	dataset = dbset;
 	*freeit = 0;
@@ -514,9 +505,8 @@ static void add_dbdata (windata_t *vwin, DATASET *dbset,
 	    strcpy(dataset->varname[i], sinfo->varname);
 	    strcpy(VARLABEL(dataset, i), sinfo->descrip);
 	    
-	    cmdline = g_strdup_printf("data %s", sinfo->varname);
-	    record_command_verbatim(cmdline);
-	    g_free(cmdline);
+	    lib_command_sprintf("data %s", sinfo->varname);
+	    record_command_verbatim();
 	}
 	
 	data_status |= (GUI_DATA | MODIFIED_DATA);
@@ -547,14 +537,10 @@ void sync_db_windows (void)
     const char *dname = get_db_name();
 
     if (*dname != '\0') {
-	GtkWidget *w = match_db_window_by_filename(dname);
+	windata_t *vwin = get_browser_for_gretl_database(dname);
 
-	if (w != NULL) {
-	    windata_t *vwin = g_object_get_data(G_OBJECT(w), "vwin");
-
-	    if (vwin != NULL) {
-		add_local_db_series_list(vwin);
-	    }
+	if (vwin != NULL) {
+	    add_local_db_series_list(vwin);
 	}
     }
 }
@@ -821,6 +807,7 @@ static GretlToolItem db_items[] = {
     { N_("Add to dataset"), GTK_STOCK_ADD,    G_CALLBACK(db_import_series), 0 },
     { N_("Delete"),         GTK_STOCK_DELETE, G_CALLBACK(db_delete_callback), DEL_BTN },
     { N_("Codebook"),       GRETL_STOCK_BOOK, G_CALLBACK(db_view_codebook), CB_BTN },
+    { N_("Windows"),        GRETL_STOCK_COMPASS, GNULL, 0 },
     { N_("Close"),          GTK_STOCK_CLOSE,  G_CALLBACK(db_close), 0 }
 };
 
@@ -844,7 +831,11 @@ static void make_db_toolbar (windata_t *vwin, int cb, int del)
 	} if (!cb && item->flag == CB_BTN) {
 	    continue;
 	}
-	gretl_toolbar_insert(vwin->mbar, item, item->func, vwin, -1);
+	if (winlist_item(item)) {
+	    vwin_toolbar_insert_winlist(vwin);
+	} else {
+	    gretl_toolbar_insert(vwin->mbar, item, item->func, vwin, -1);
+	}
     }
 
     gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
@@ -852,16 +843,6 @@ static void make_db_toolbar (windata_t *vwin, int cb, int del)
     gtk_widget_show_all(hbox);
 }
 
-static void destroy_db_win (GtkWidget *w, windata_t *vwin)
-{
-    if (vwin != NULL) {
-	if (vwin->popup != NULL) {
-	    gtk_widget_destroy(vwin->popup);
-	}
-	free(vwin);
-    }
-}
-
 static int db_has_codebook (const char *fname)
 {
     char testname[MAXLEN];
@@ -944,19 +925,19 @@ maybe_adjust_descrip_column (windata_t *vwin)
 }
 
 static int 
-make_db_series_window (int action, char *fname, char *buf)
+make_db_index_window (int action, char *fname, char *buf)
 {
-    GtkWidget *w, *listbox;
+    GtkWidget *listbox;
     gchar *title;
     windata_t *vwin;
     int db_width = 700, db_height = 420;
     int cb = 0, del = 0;
-    int record, err = 0;
+    int err = 0;
 
-    w = match_db_window_by_filename(fname);
+    vwin = get_browser_for_database(fname);
 
-    if (w != NULL) {
-	gtk_window_present(GTK_WINDOW(w));
+    if (vwin != NULL) {
+	gtk_window_present(GTK_WINDOW(vwin->main));
 	return 0;
     }
 
@@ -970,9 +951,7 @@ make_db_series_window (int action, char *fname, char *buf)
 	title = fname;
     } 
 
-    record = (action == NATIVE_SERIES);
-
-    vwin = gretl_browser_new(action, title, record);
+    vwin = gretl_browser_new(action, title);
     if (vwin == NULL) {
 	return 1;
     }
@@ -981,11 +960,6 @@ make_db_series_window (int action, char *fname, char *buf)
     db_height *= gui_scale;
     gtk_window_set_default_size(GTK_WINDOW(vwin->main), db_width, db_height);
     
-    if (action != NATIVE_SERIES) {
-	g_signal_connect(G_OBJECT(vwin->main), "destroy", 
-			 G_CALLBACK(destroy_db_win), vwin);
-    }
-
     if (action == NATIVE_SERIES || action == PCGIVE_SERIES) {
 	strip_extension(fname);
     }
@@ -1039,12 +1013,12 @@ make_db_series_window (int action, char *fname, char *buf)
 
 void open_rats_window (char *fname)
 {
-    make_db_series_window(RATS_SERIES, fname, NULL);
+    make_db_index_window(RATS_SERIES, fname, NULL);
 }
 
 void open_bn7_window (char *fname)
 {
-    make_db_series_window(PCGIVE_SERIES, fname, NULL);
+    make_db_index_window(PCGIVE_SERIES, fname, NULL);
 }
 
 static int check_serinfo (char *str, char *sername, int *nobs)
@@ -1622,7 +1596,7 @@ void open_named_db_index (char *dbname)
 	file_read_errbox(dbname);
     } else {
 	fclose(fp);
-	make_db_series_window(action, dbname, NULL);
+	make_db_index_window(action, dbname, NULL);
     } 
 }
 
@@ -1638,7 +1612,7 @@ void open_named_remote_db_index (char *dbname)
     } else if (getbuf != NULL && !strncmp(getbuf, "Couldn't open", 13)) {
 	errbox(getbuf);
     } else {
-	make_db_series_window(REMOTE_SERIES, dbname, getbuf);
+	make_db_index_window(REMOTE_SERIES, dbname, getbuf);
 	/* check for error */
     }
 
@@ -1667,7 +1641,7 @@ void open_db_index (GtkWidget *w, gpointer data)
     g_free(fname);
     g_free(dbdir);
 
-    make_db_series_window(action, dbfile, NULL); 
+    make_db_index_window(action, dbfile, NULL); 
 }
 
 void open_remote_db_index (GtkWidget *w, gpointer data)
@@ -1698,7 +1672,7 @@ void open_remote_db_index (GtkWidget *w, gpointer data)
 	show_network_error(vwin);
     } else {
 	update_statusline(vwin, "OK");
-	make_db_series_window(REMOTE_SERIES, fname, getbuf);
+	make_db_index_window(REMOTE_SERIES, fname, getbuf);
     }
 
     g_free(fname);
@@ -1891,35 +1865,63 @@ static void offer_db_open (char *target)
     }
 }
 
-#ifdef OSX_BUILD
-
 static int get_target_in_home (char *targ, int code, 
 			       const char *objname,
 			       const char *ext)
 {
-    const char *suppdir = gretl_app_support_dir();
+#ifdef OSX_BUILD
+    const char *savedir = gretl_app_support_dir();
+#else
+    const char *savedir = gretl_dotdir();
+#endif
     int err = 0;
 
-    fprintf(stderr, "suppdir='%s'\n", suppdir);
-
-    if (suppdir == NULL || *suppdir == '\0') {
+    if (savedir == NULL || *savedir == '\0') {
 	err = E_FOPEN;
     } else {
+	int subdir = 0;
+
 	if (code == REMOTE_FUNC_FILES) {
-	    sprintf(targ, "%sfunctions/%s%s", suppdir, objname, ext);
+	    sprintf(targ, "%sfunctions", savedir);
+	    subdir = 1;
 	} else if (code == REMOTE_DB) {
-	    sprintf(targ, "%sdb/%s%s", suppdir, objname, ext);
+	    sprintf(targ, "%sdb", savedir);
+	    subdir = 1;
 	} else if (code == REMOTE_DATA_PKGS) {
-	    sprintf(targ, "%sdata/%s%s", suppdir, objname, ext);
+	    sprintf(targ, "%sdata", savedir);
+	    subdir = 1;
 	} else {
-	    sprintf(targ, "%s%s%s", suppdir, objname, ext);
+	    sprintf(targ, "%s%s%s", savedir, objname, ext);
+	}
+
+	if (subdir) {
+	    err = gretl_mkdir(targ);
+	    if (!err) {
+		strcat(targ, SLASHSTR);
+		strcat(targ, objname);
+		strcat(targ, ext);
+	    }
 	}
     }
 
     return err;
 }
 
-#endif
+static void get_system_target (char *targ, int code, 
+			       const char *objname,
+			       const char *ext)
+{
+    if (code == REMOTE_DB) {
+	get_default_dir(targ, SAVE_REMOTE_DB);
+    } else if (code == REMOTE_DATA_PKGS) {
+	get_default_dir(targ, SAVE_DATA_PKG);
+    } else if (code == REMOTE_FUNC_FILES) {
+	get_default_dir(targ, SAVE_FUNCTIONS);
+    }
+
+    strcat(targ, objname);
+    strcat(targ, ext);
+}
 
 enum {
     REAL_INSTALL,
@@ -1931,20 +1933,25 @@ enum {
    data files, or function package 
 */
 
-static char *get_writable_target (int code, int op, char *objname,
+static char *get_writable_target (int code, char *objname,
 				  gboolean zipfile)
 {
     const char *ext;
     char *targ;
+    int done_home = 0;
     int err = 0;
 
+#if 0
     fprintf(stderr, "get_writable_target\n");
+#endif
 
     targ = mymalloc(MAXLEN);
     if (targ == NULL) {
 	return NULL;
     }
 
+    *targ = '\0';
+
     if (code == REMOTE_DB) {
 	ext = ".ggz";
     } else if (code == REMOTE_FUNC_FILES) {
@@ -1957,60 +1964,29 @@ static char *get_writable_target (int code, int op, char *objname,
     /* we prefer writing to ~/Library/Application Support
        rather than /Applications/Gretl.app 
     */
-    if (op != TMP_INSTALL) {
-	err = get_target_in_home(targ, code, objname, ext);
-	goto path_done;
-    }
+    err = get_target_in_home(targ, code, objname, ext);
+    done_home = 1;
+#else
+    get_system_target(targ, code, objname, ext);
 #endif
 
-    if (code == REMOTE_DB) {
-	char dbdir[MAXLEN];
-
-	build_path(dbdir, gretl_home(), "db", NULL);
-	build_path(targ, dbdir, objname, ext);
-    } else if (code == REMOTE_DATA_PKGS) {
-	char pkgdir[MAXLEN];
-
-	get_default_dir(pkgdir, SAVE_DATA_PKG);
-	build_path(targ, pkgdir, objname, ext);
-    } else if (code == REMOTE_FUNC_FILES) {
-	if (op == TMP_INSTALL) {
-	    build_path(targ, gretl_dotdir(), "dltmp", NULL);
-	    err = gretl_tempname(targ);
-	} else {
-	    char pkgdir[MAXLEN];
-
-	    get_default_dir(pkgdir, SAVE_FUNCTIONS);
-	    build_path(targ, pkgdir, objname, ext);
+    if (!err) {
+	err = gretl_test_fopen(targ, "w");
+	if (err == EACCES && !done_home) { 
+	    /* permissions problem: write to home dir instead */
+	    err = get_target_in_home(targ, code, objname, ext);
 	}
-    } 
-
-#ifdef OSX_BUILD
- path_done:
-#endif
+    }
 
     if (err) {
+	file_write_errbox(targ);
 	free(targ);
 	targ = NULL;
-    } else {
-	FILE *fp;
+    } 
 
-	errno = 0;
-	fp = gretl_fopen(targ, "w");
-
-	if (fp == NULL) {
-	    if (errno == EACCES) { 
-		/* write to working dir instead? */
-		build_path(targ, gretl_workdir(), objname, ext);
-	    } else {
-		file_write_errbox(targ);
-		free(targ);
-		targ = NULL;
-	    }
-	} else {
-	    fclose(fp);
-	}
-    }
+#if 0
+    fprintf(stderr, "writable targ: '%s'\n", targ);
+#endif
 
     return targ;
 }
@@ -2088,6 +2064,9 @@ int unzip_package_file (const char *zipname, const char *path)
     return err;
 }
 
+#define STATUS_COLUMN  4
+#define ZIPFILE_COLUMN 5
+
 /* note : 'vwin' here is the source viewer window displaying the
    remote file (database or datafiles package or function package)
    that is being installed onto the local machine.
@@ -2115,7 +2094,7 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 			     vwin->active_var, 0, &objname);
 	if (vwin->role == REMOTE_FUNC_FILES) {
 	    tree_view_get_bool(GTK_TREE_VIEW(vwin->listbox), 
-			       vwin->active_var, 4, &zipfile);
+			       vwin->active_var, ZIPFILE_COLUMN, &zipfile);
 	}
     }
 
@@ -2124,7 +2103,7 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 	return;
     } 
 
-    path = get_writable_target(vwin->role, REAL_INSTALL, objname, zipfile);
+    path = get_writable_target(vwin->role, objname, zipfile);
     if (path == NULL) {
 	g_free(objname);
 	return;
@@ -2161,8 +2140,9 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 	windata_t *local = get_local_viewer(vwin->role);
 
 	if (vwin->role == REMOTE_FUNC_FILES) {
+	    infobox(_("Installed"));
 	    list_store_set_string(GTK_TREE_VIEW(vwin->listbox),
-				  vwin->active_var, 3,
+				  vwin->active_var, STATUS_COLUMN,
 				  _("Up to date"));
 	    if (local != NULL) {
 		populate_filelist(local, NULL);
@@ -2202,19 +2182,13 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 
 void pkg_info_from_server (GtkWidget *w, windata_t *vwin)
 {
-    gchar *objname = NULL;
-    char *path = NULL;
+    static int idx;
+    gchar *path, *objname = NULL;
     int err = 0;
 
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			 vwin->active_var, 0, &objname);
-
-    path = get_writable_target(vwin->role, TMP_INSTALL, objname, FALSE);
-    if (path == NULL) {
-	g_free(objname);
-	return;
-    }
-
+    path = g_strdup_printf("%sdltmp.%d", gretl_dotdir(), idx++);
     err = retrieve_remote_function_package(objname, path);
 
     if (err) {
@@ -2224,7 +2198,7 @@ void pkg_info_from_server (GtkWidget *w, windata_t *vwin)
     }
 
     g_free(objname);
-    free(path);
+    g_free(path);
 }
 
 static gchar *
@@ -2335,21 +2309,23 @@ read_db_files_in_dir (DIR *dir, int dbtype, const char *path,
 {
     struct dirent *dirent;
     const char *fname;
-    char *descrip;
+    gchar *name, *descrip;
     int len, ndb = 0;
 
     while ((dirent = readdir(dir)) != NULL) {
 	fname = dirent->d_name;
 	len = strlen(fname);
 	if (!g_ascii_strcasecmp(fname + len - 4, ".bin")) {
+	    name = g_strndup(fname, len - 4);
 	    descrip = real_get_db_description(NULL, fname, path);
-	    if (descrip != NULL) {
+	    if (name != NULL && descrip != NULL) {
 		gtk_list_store_append(store, iter);
-		gtk_list_store_set(store, iter, 0, fname, 1, descrip, 
+		gtk_list_store_set(store, iter, 0, name, 1, descrip, 
 				   2, path, -1);
-		g_free(descrip);
 		ndb++;
 	    }
+	    g_free(name);
+	    g_free(descrip);
 	}
     }
 
@@ -2817,6 +2793,7 @@ gint populate_remote_func_list (windata_t *vwin)
     while (bufgets(line, sizeof line, getbuf)) {
 	char *descrip = NULL;
 	char *version = NULL;
+	char *author = NULL;
 	gboolean zipfile;
 
 	if (read_remote_filetime(line, fname, &remtime, NULL)) {
@@ -2831,30 +2808,41 @@ gint populate_remote_func_list (windata_t *vwin)
 	if (bufgets(line, sizeof line, getbuf)) {
 	    tailstrip(line);
 	    utf8_correct(line);
-	    descrip = gretl_strdup(line + 2);
+	    if (strlen(line) > 62) {
+		descrip = gretl_strndup(line + 2, 60);
+		descrip[56] = '\0';
+		strncat(descrip, "...", 3);
+	    } else {
+		descrip = gretl_strdup(line + 2);
+	    }
 	} 
 
 	if (bufgets(line, sizeof line, getbuf)) {
 	    tailstrip(line);
-	    version = line + 2;
+	    version = gretl_strdup(line + 2);
 	} 
 
-	if (descrip == NULL || version == NULL) {
-	    free(descrip);
-	    continue;
+	if (bufgets(line, sizeof line, getbuf)) {
+	    tailstrip(line);
+	    author = gretl_strdup(line + 2);
 	}
 
-	gtk_list_store_append(store, &iter);
-	gtk_list_store_set(store, &iter, 
+	if (descrip != NULL && version != NULL && author != NULL) {
+	    gtk_list_store_append(store, &iter);
+	    gtk_list_store_set(store, &iter, 
 			   0, basename, 
 			   1, version,
-			   2, descrip, 
-			   3, _(status),
-			   4, zipfile,
+			   2, author,
+			   3, descrip, 
+			   4, _(status),
+			   5, zipfile,
 			   -1);
+	    n++;
+	}
 
 	free(descrip);
-	n++;
+	free(version);
+	free(author);
     }
 
     bufgets_finalize(getbuf);
@@ -2980,6 +2968,8 @@ gint populate_dbfilelist (windata_t *vwin)
     if (ndb == 0) {
 	errbox(_("No database files found"));
 	err = 1;
+    } else {
+	presort_treelist(vwin);
     }
 
     return err;
@@ -3016,6 +3006,19 @@ static void set_compact_info_from_default (int method)
     }
 }
 
+static const char *method_string (CompactMethod method)
+{
+    if (method == COMPACT_SUM) {
+	return "sum";
+    } else if (method == COMPACT_SOP) {
+	return "first";
+    } else if (method == COMPACT_EOP) {
+	return "last";
+    } else {
+	return NULL;
+    }
+}
+
 void do_compact_data_set (void)
 {
     CompactMethod method = COMPACT_AVG;
@@ -3031,8 +3034,8 @@ void do_compact_data_set (void)
 	pmonstart = &monstart;
     }
 
-    data_compact_dialog(mdata->main, dataset->pd, &newpd, pmonstart, 
-			&method, &repday);
+    data_compact_dialog(dataset->pd, &newpd, pmonstart, 
+			&method, &repday, mdata->main);
 
     if (method == COMPACT_NONE) {
 	/* the user cancelled */
@@ -3044,6 +3047,15 @@ void do_compact_data_set (void)
     if (err) {
 	gui_errmsg(err);
     } else {
+	const char *mstr = method_string(method);
+
+	if (mstr != NULL) {
+	    lib_command_sprintf("dataset compact %d %s", newpd, mstr);
+	} else {
+	    lib_command_sprintf("dataset compact %d", newpd);
+	}
+	record_command_verbatim();
+
 	mark_dataset_as_modified();
 	set_compact_info_from_default(method);
     }
@@ -3060,7 +3072,7 @@ void do_expand_data_set (void)
     /* supported: annual to quarterly or quarterly to monthly */
     newpd = (dataset->pd == 1)? 4 : 12;
 
-    data_expand_dialog(mdata->main, dataset->pd, &interpol);
+    data_expand_dialog(dataset->pd, &interpol, mdata->main);
 
     if (interpol < 0) {
 	/* canceled */
