@@ -144,13 +144,14 @@ static void prep_spreadsheet (GtkWidget *widget, dialog_t *dlg)
     buf = edit_dialog_get_text(dlg);
 
     if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_SERIES)) {
+	edit_dialog_reset(dlg);
 	return;
     }
 
     dataset->varname[1][0] = 0;
     strncat(dataset->varname[1], buf, VNAMELEN - 1);
 
-    close_dialog(dlg);
+    edit_dialog_close(dlg);
 
     /* blank out the auto "index" variable */
     for (t=0; t<dataset->n; t++) {
@@ -165,12 +166,11 @@ static void maybe_start_editing (void)
 {
     int cancel = 0;
 
-    edit_dialog(_("gretl: name variable"), 
-		_("Enter name for first variable\n"
-		  "(max. 15 characters)"),
-		NULL, prep_spreadsheet, NULL, 
-		CREATE_DATASET, VARCLICK_NONE, 
-		&cancel);
+    blocking_edit_dialog(CREATE_DATASET, _("gretl: name variable"), 
+			 _("Enter name for first variable\n"
+			   "(max. 15 characters)"),
+			 NULL, prep_spreadsheet, NULL, 
+			 VARCLICK_NONE, NULL, &cancel);
 
     if (cancel) {
 	/* accept the default blank dataset */
@@ -361,8 +361,8 @@ static int dwiz_make_changes (DATASET *dwinfo, dw_opts *opts,
 	    maybe_start_editing();
 	} else {
 	    register_data(NULLDATA_STARTED);
-	    sprintf(setline, "nulldata %d", dataset->n);
-	    record_command_verbatim(setline);
+	    lib_command_sprintf("nulldata %d", dataset->n);
+	    record_command_verbatim();
 	}
     } else {
 	if (delete_markers) {
@@ -372,7 +372,8 @@ static int dwiz_make_changes (DATASET *dwinfo, dw_opts *opts,
     }
 
     if (!err && *record != '\0') {
-	record_command_verbatim(record);
+	lib_command_strcpy(record);
+	record_command_verbatim();
     }
 
 #if DWDEBUG
@@ -414,13 +415,19 @@ struct panel_info pan_info[] = {
     { PANEL_UNKNOWN,         N_("Use index variables") }
 };
 
-static const char *ts_frequency_string (int pd)
+static const char *ts_frequency_string (const DATASET *dwinfo)
 {
-    int i;
+    int i, pd = dwinfo->pd;
 
-    for (i=0; i<TS_INFO_MAX; i++) {
-	if (ts_info[i].pd == pd) {
-	    return ts_info[i].label;
+    if (pd == PD_SPECIAL) {
+	return N_("Other");
+    } else if (dwinfo->structure == SPECIAL_TIME_SERIES) {
+	return N_("Time series");
+    } else {
+	for (i=0; i<TS_INFO_MAX; i++) {
+	    if (ts_info[i].pd == pd) {
+		return ts_info[i].label;
+	    }
 	}
     }
 
@@ -557,7 +564,7 @@ static void make_confirmation_text (char *ctxt, DATASET *dwinfo, gretlopt *flags
 	char endobs[OBSLEN];
 	const char *tslabel;
 
-	tslabel = _(ts_frequency_string(dwinfo->pd));
+	tslabel = _(ts_frequency_string(dwinfo));
 
 	if (lastobs > dwinfo->n - 1) {
 	    dwinfo->n = lastobs + 1;
@@ -1102,7 +1109,7 @@ static GtkWidget *dwiz_spinner (GtkWidget *hbox, DATASET *dwinfo, int step)
     int spinmin, spinmax, spinstart;
 
     if (step == DW_STARTING_OBS) {
-	GtkWidget *label = gtk_label_new(_(ts_frequency_string(dwinfo->pd)));
+	GtkWidget *label = gtk_label_new(_(ts_frequency_string(dwinfo)));
 
 	gtk_widget_show(label);
 	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
