@@ -22,28 +22,13 @@
 #include "gretl.h"
 #include "textbuf.h"
 #include "menustate.h"
+#include "tabwin.h"
 #include "dlgutils.h"
 
 #include "libset.h"
 #include "system.h"
 #include "gretl_bfgs.h"
 
-void set_window_busy (windata_t *vwin)
-{
-    vwin->flags |= VWIN_BUSY;
-    if (vwin->mbar != NULL) {
-	gtk_widget_set_sensitive(vwin->mbar, FALSE);
-    }
-}
-
-void unset_window_busy (windata_t *vwin)
-{
-    vwin->flags &= ~VWIN_BUSY;
-    if (vwin->mbar != NULL) {
-	gtk_widget_set_sensitive(vwin->mbar, TRUE);
-    }
-}
-
 dialog_opts *dialog_opts_new (int n, int type, 
 			      gretlopt *optp,
 			      const gretlopt *vals,
@@ -89,58 +74,13 @@ void pack_in_hbox (GtkWidget *w, GtkWidget *vbox, int vspace)
 
 /* Various buttons, usable in several sorts of dialogs */
 
-GtkWidget *context_help_button (GtkWidget *hbox, int cmdcode)
-{
-    GtkWidget *button;
-
-    button = gtk_button_new_from_stock(GTK_STOCK_HELP);
-    gtk_widget_set_can_default(button, TRUE);
-    gtk_container_add(GTK_CONTAINER(hbox), button);
-    gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbox),
-				       button, TRUE);
-
-    if (cmdcode >= 0) {
-	g_signal_connect(G_OBJECT(button), "clicked", 
-			 G_CALLBACK(context_help), 
-			 GINT_TO_POINTER(cmdcode));
-    }
-
-    return button;
-}
-
-static void set_canceled (GtkWidget *w, int *c)
-{
-    if (c != NULL) {
-	*c = -1;
-    }
-}
-
-static void maybe_set_canceled (GtkDialog *d, int resp, int *c)
-{
-    if (resp == GTK_RESPONSE_DELETE_EVENT ||
-	resp == GTK_RESPONSE_CANCEL) {
-	*c = -1;
-    }
-}
-
-GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ,
-				 int *canceled)
+GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ)
 {
     GtkWidget *button;
 
     button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
     gtk_widget_set_can_default(button, TRUE);
     gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-
-    if (canceled != NULL) {
-	g_signal_connect(G_OBJECT(button), "clicked", 
-			 G_CALLBACK(set_canceled), 
-			 canceled);
-	g_signal_connect(GTK_DIALOG(targ), "response", 
-			 G_CALLBACK(maybe_set_canceled), 
-			 canceled);
-    }
-
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(delete_widget), 
 		     targ);
@@ -148,45 +88,6 @@ GtkWidget *cancel_delete_button (GtkWidget *hbox, GtkWidget *targ,
     return button;
 }
 
-static void invalidate_opt (GtkWidget *w, int *opt)
-{
-    *opt = -1;
-}
-
-static void maybe_invalidate_opt (GtkDialog *d, int resp, int *opt)
-{
-    if (resp == GTK_RESPONSE_NONE || 
-	resp == GTK_RESPONSE_DELETE_EVENT ||
-	resp == GTK_RESPONSE_CANCEL) {
-	*opt = -1;
-    }
-}
-
-GtkWidget *cancel_options_button (GtkWidget *hbox, GtkWidget *targ,
-				  int *opt)
-{
-    GtkWidget *button;
-
-    button = gtk_button_new_from_stock(GTK_STOCK_CANCEL);
-    gtk_widget_set_can_default(button, TRUE);
-    gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, TRUE, 0);
-
-    if (opt != NULL) {
-	g_signal_connect(G_OBJECT(button), "clicked", 
-			 G_CALLBACK(invalidate_opt), 
-			 opt);
-	g_signal_connect(GTK_DIALOG(targ), "response", 
-			 G_CALLBACK(maybe_invalidate_opt), 
-			 opt);
-    }
-
-    g_signal_connect(G_OBJECT(button), "clicked", 
-		     G_CALLBACK(delete_widget), 
-		     targ);
-
-    return button;
-}
-
 GtkWidget *ok_button (GtkWidget *hbox)
 {
     GtkWidget *w;
@@ -198,6 +99,34 @@ GtkWidget *ok_button (GtkWidget *hbox)
     return w;
 }
 
+static void set_valid_response (GtkButton *b, int *valptr)
+{
+    int *retptr = g_object_get_data(G_OBJECT(b), "retptr");
+
+    if (valptr == NULL) {
+	*retptr = 0;
+    } else {
+	*retptr = *valptr;
+    }
+}
+
+/* on "OK": if @valptr is non-NULL, copy the valid value from 
+   @valptr to @retptr; otherwise signal all-clear by copying
+   0 to @retptr
+*/
+
+GtkWidget *ok_validate_button (GtkWidget *hbox, int *retptr,
+			       int *valptr)
+{
+    GtkWidget *button = ok_button(hbox);
+
+    g_object_set_data(G_OBJECT(button), "retptr", retptr);
+    g_signal_connect(G_OBJECT(button), "clicked", 
+		     G_CALLBACK(set_valid_response), valptr);
+
+    return button;
+}
+
 GtkWidget *apply_button (GtkWidget *hbox)
 {
     GtkWidget *w;
@@ -253,15 +182,13 @@ GtkWidget *back_button (GtkWidget *hbox)
     return w;
 }
 
-static void set_dialog_border_widths (GtkWidget *dlg)
+static void set_dialog_border_widths (GtkWidget *ca, GtkWidget *aa)
 {
-    GtkWidget *box = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
     int w1 = 10, w2 = 5;
 
-    gtk_container_set_border_width(GTK_CONTAINER(box), w1);
-    gtk_box_set_spacing(GTK_BOX(box), w2);
-    box = gtk_dialog_get_action_area(GTK_DIALOG(dlg));
-    gtk_container_set_border_width(GTK_CONTAINER(box), w2);
+    gtk_container_set_border_width(GTK_CONTAINER(ca), w1);
+    gtk_box_set_spacing(GTK_BOX(ca), w2);
+    gtk_container_set_border_width(GTK_CONTAINER(aa), w2);
 }
 
 static void gretl_dialog_set_resizeable (GtkWidget *w, gboolean s)
@@ -294,11 +221,15 @@ static gint dialog_unblock (GtkWidget *w, gpointer p)
     return FALSE;
 }
 
-static gint dialog_set_destruction (GtkWidget *w, gpointer p)
+gint gretl_dialog_set_destruction (GtkWidget *w, gpointer p)
 {
     gtk_window_set_transient_for(GTK_WINDOW(w), GTK_WINDOW(p));
     gtk_window_set_destroy_with_parent(GTK_WINDOW(w), TRUE);
 
+    if (g_object_get_data(G_OBJECT(p), "tabwin") != NULL) {
+	tabwin_register_dialog(w, p);
+    }
+
     return FALSE;
 }
 
@@ -306,7 +237,7 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
 			     unsigned char flags)
 {
     GtkWidget *d = gtk_dialog_new();
-    GtkWidget *aa;
+    GtkWidget *ca, *aa;
 
     if (title != NULL) {
 	gtk_window_set_title(GTK_WINDOW(d), title);
@@ -314,9 +245,10 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
 	gtk_window_set_title(GTK_WINDOW(d), "gretl");
     }
 
+    ca = gtk_dialog_get_content_area(GTK_DIALOG(d));
     aa = gtk_dialog_get_action_area(GTK_DIALOG(d));
     gtk_button_box_set_layout(GTK_BUTTON_BOX(aa), GTK_BUTTONBOX_END);
-    set_dialog_border_widths(d);
+    set_dialog_border_widths(ca, aa);
     gtk_window_set_position(GTK_WINDOW(d), GTK_WIN_POS_MOUSE);
 
     if (flags & GRETL_DLG_BLOCK) {
@@ -338,11 +270,14 @@ GtkWidget *gretl_dialog_new (const char *title, GtkWidget *parent,
 			 G_CALLBACK(dialog_unblock), NULL);
     }
 
-    if (parent != NULL) {
-	g_signal_connect(G_OBJECT(d), "show", 
-			 G_CALLBACK(dialog_set_destruction), parent);
+    if (parent == NULL) {
+	parent = mdata->main;
     }
 
+    g_signal_connect(G_OBJECT(d), "show", 
+		     G_CALLBACK(gretl_dialog_set_destruction), 
+		     parent);
+
     if (flags & GRETL_DLG_BLOCK) {
 	g_signal_connect(G_OBJECT(d), "show", 
 			 G_CALLBACK(gtk_main), NULL);
@@ -480,16 +415,20 @@ void set_active_edit_name (GtkWidget *w)
 }
 
 struct dialog_t_ {
+    int ci;
+    void (*okfunc)();
+    void *data;
+    int blocking;
+    int *cancel;
+    gretlopt opt;
     GtkWidget *dialog;
+    GtkWidget *vbox;
+    GtkWidget *bbox;
     GtkWidget *edit;
     GtkWidget *popup;
-    gpointer data;
-    gint ci;
-    gint blocking;
-    gretlopt opt;
 };
 
-static void destroy_dialog_data (GtkWidget *w, gpointer data) 
+static void destroy_edit_dialog (GtkWidget *w, gpointer data) 
 {
     dialog_t *d = (dialog_t *) data;
 
@@ -506,21 +445,10 @@ static void destroy_dialog_data (GtkWidget *w, gpointer data)
     if (active_edit_text) active_edit_text = NULL;
 }
 
-static gboolean cancel_on_delete (GtkDialog *d, int resp, int *c)
-{
-    if (resp == GTK_RESPONSE_NONE || 
-	resp == GTK_RESPONSE_DELETE_EVENT ||
-	resp == GTK_RESPONSE_CANCEL) {
-	*c = 1;
-    }
-
-    return FALSE;
-}
-
 gboolean esc_kills_window (GtkWidget *w, GdkEventKey *key, 
-			   gpointer unused)
+			   gpointer p)
 {
-    if (key->keyval == GDK_Escape) { 
+    if (key->keyval == GDK_Escape) {
         gtk_widget_destroy(w);
 	return TRUE;
     } else {
@@ -528,30 +456,32 @@ gboolean esc_kills_window (GtkWidget *w, GdkEventKey *key,
     }
 }
 
-static dialog_t *dialog_data_new (gpointer p, int ci, 
-				  const char *title,
+static dialog_t *edit_dialog_new (int ci, const char *title,
+				  void (*okfunc)(), void *data,
+				  int helpcode, GtkWidget *parent,
 				  int *canceled)
 {
     dialog_t *d = mymalloc(sizeof *d);
-    GtkWidget *aa;
 
     if (d == NULL) {
 	return NULL;
     }
 
-    d->data = p;
     d->ci = ci;
+    d->okfunc = okfunc;
+    d->data = data;
     d->opt = OPT_NONE;
-    d->dialog = gtk_dialog_new();
     d->popup = NULL;
+    d->blocking = 0;
+    d->cancel = canceled;
+
+    d->dialog = gtk_dialog_new();
+    d->vbox = gtk_dialog_get_content_area(GTK_DIALOG(d->dialog));
+    d->bbox = gtk_dialog_get_action_area(GTK_DIALOG(d->dialog));
 
     if (canceled != NULL) {
+	*canceled = 1; /* will be undone by "OK" */
 	d->blocking = 1;
-	g_signal_connect(G_OBJECT(d->dialog), "delete-event",
-			 G_CALLBACK(cancel_on_delete), 
-			 canceled);
-    } else {
-	d->blocking = 0;
     }
 
     if (!strncmp(title, "gretl", 5)) {
@@ -563,19 +493,28 @@ static dialog_t *dialog_data_new (gpointer p, int ci,
 	g_free(tmp);
     }
 
-    aa = gtk_dialog_get_action_area(GTK_DIALOG(d->dialog));
-    gtk_box_set_homogeneous(GTK_BOX(aa), TRUE); 
+    gtk_box_set_homogeneous(GTK_BOX(d->bbox), TRUE); 
     gtk_window_set_position(GTK_WINDOW(d->dialog), GTK_WIN_POS_MOUSE);
 
+    if (parent == NULL) {
+	parent = mdata->main;
+    }
+
     g_signal_connect(G_OBJECT(d->dialog), "destroy", 
-		     G_CALLBACK(destroy_dialog_data), d);
+		     G_CALLBACK(destroy_edit_dialog), d);
     g_signal_connect(G_OBJECT(d->dialog), "key-press-event", 
 		     G_CALLBACK(esc_kills_window), NULL);
+    g_signal_connect(G_OBJECT(d->dialog), "show", 
+		     G_CALLBACK(gretl_dialog_set_destruction), parent);
+    if (d->blocking) {
+	g_signal_connect(G_OBJECT(d->dialog), "show", 
+			 G_CALLBACK(gtk_main), NULL);
+    }
 
     return d;
 }
 
-void close_dialog (dialog_t *dlg)
+void edit_dialog_close (dialog_t *dlg)
 {
     gtk_widget_destroy(dlg->dialog);
 }
@@ -693,12 +632,11 @@ gpointer edit_dialog_get_data (dialog_t *dlg)
 
 static void dialog_table_setup (dialog_t *dlg, int hsize)
 {
-    GtkWidget *vbox, *sw;
+    GtkWidget *sw;
 
     sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_widget_set_size_request(sw, hsize, 200);
-    vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg->dialog));
-    gtk_box_pack_start(GTK_BOX(vbox), sw, TRUE, TRUE, FALSE);
+    gtk_box_pack_start(GTK_BOX(dlg->vbox), sw, TRUE, TRUE, FALSE);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (sw),
 				   GTK_POLICY_AUTOMATIC,
 				   GTK_POLICY_AUTOMATIC);
@@ -1020,7 +958,7 @@ static void mle_gmm_iters_dialog (GtkWidget *w, dialog_t *d)
 {
     int maxit, lmem = 0, optim = BFGS_MAX;
     double tol;
-    int cancel = 0;
+    int resp;
 
     BFGS_defaults(&maxit, &tol, d->ci);
     lmem = libset_get_int(LBFGS_MEM);
@@ -1033,10 +971,10 @@ static void mle_gmm_iters_dialog (GtkWidget *w, dialog_t *d)
 	optim = LBFGS_MAX;
     }
 
-    iter_control_dialog(&optim, &maxit, &tol, &lmem, &cancel,
-			d->dialog);
+    resp = iter_control_dialog(&optim, &maxit, &tol, &lmem,
+			       d->dialog);
 
-    if (!cancel) {
+    if (!canceled(resp)) {
 	int err;
 
 	err = libset_set_int(BFGS_MAXITER, maxit);
@@ -1128,8 +1066,8 @@ static void system_estimator_list (GtkWidget *vbox, dialog_t *d,
     equation_system *sys = NULL;
     GtkWidget *w, *hbox;
     gchar *str;
-    int method = 0;
-    int i;
+    int active = 0;
+    int i, j;
 
     if (d->data != NULL) {
 	sys = (equation_system *) d->data;
@@ -1145,13 +1083,17 @@ static void system_estimator_list (GtkWidget *vbox, dialog_t *d,
 
     w = gtk_combo_box_text_new();
 
+    j = 0;
     for (i=SYS_METHOD_SUR; i<SYS_METHOD_MAX; i++) {
-	str = g_strdup_printf("%s (%s)", _(system_method_full_string(i)),
-			      system_method_short_string(i));
-	combo_box_append_text(w, str);
-	g_free(str);
-	if (sys != NULL && sys->method == i) {
-	    method = i;
+	if (system_supports_method(sys, i)) {
+	    str = g_strdup_printf("%s (%s)", _(system_method_full_string(i)),
+				  system_method_short_string(i));
+	    combo_box_append_text(w, str);
+	    g_free(str);
+	    if (sys != NULL && sys->method == i) {
+		active = j;
+	    }
+	    j++;
 	}
     }
 
@@ -1160,7 +1102,7 @@ static void system_estimator_list (GtkWidget *vbox, dialog_t *d,
     g_signal_connect(G_OBJECT(w), "changed",
 		     G_CALLBACK(set_sys_method), d);
 
-    gtk_combo_box_set_active(GTK_COMBO_BOX(w), method);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(w), active);
 
     gtk_box_pack_start(GTK_BOX(hbox), w, TRUE, TRUE, 5);
     gtk_widget_show(w);  
@@ -1174,12 +1116,9 @@ static void dlg_display_sys (dialog_t *d)
 
     if (d->ci == ESTIMATE) {
 	/* estimating an existing system, not editing */
-	GtkWidget *vbox;
-
 	w = gtk_label_new(sys->name);
 	gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
-	vbox = gtk_dialog_get_content_area(GTK_DIALOG(d->dialog));
-	gtk_box_pack_start(GTK_BOX(vbox), w, TRUE, TRUE, 10);
+	gtk_box_pack_start(GTK_BOX(d->vbox), w, TRUE, TRUE, 10);
 	gtk_widget_show(w);
     }
 
@@ -1229,16 +1168,11 @@ static void clear_dlg_previous (GtkWidget *w, dialog_t *d)
 
 static void edit_dialog_ok (GtkWidget *w, dialog_t *d)
 {
-    gtk_widget_destroy(d->dialog);
-}
-
-static gboolean cancel_vwin_edit (GtkWidget *w, gpointer p)
-{
-    if (open_edit_dialog != NULL) {
-	gtk_widget_destroy(open_edit_dialog);
+    if (d->okfunc != NULL) {
+	d->okfunc(w, d);
+    } else {
+	gtk_widget_destroy(d->dialog);
     }
-
-    return FALSE;
 }
 
 static int ols_model_window (windata_t *vwin)
@@ -1267,20 +1201,22 @@ static void edit_dialog_add_note (const char *s, GtkWidget *vbox)
     lbl = g_strdup_printf("%s\n%s", s, _("(Please refer to Help for guidance)"));
     w = gtk_label_new(lbl);
     gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_CENTER);
-    gtk_box_pack_start(GTK_BOX(vbox), w, TRUE, TRUE, 10);
+    gtk_box_pack_start(GTK_BOX(vbox), w, FALSE, FALSE, 10);
     gtk_widget_show(w);
     g_free(lbl);
 }
 
-void edit_dialog (const char *title, const char *info, const char *deflt, 
-		  void (*okfunc)(), void *okptr, guint ci, 
-		  guint varclick, int *canceled)
+void 
+blocking_edit_dialog (int ci, const char *title, 
+		      const char *info, const char *deflt, 
+		      void (*okfunc)(), void *okptr,
+		      Varclick click, GtkWidget *parent,
+		      int *canceled)
 {
     dialog_t *d;
     GtkWidget *w;
-    GtkWidget *top_vbox, *button_box;
     MODEL *pmod = NULL;
-    int hlpcode, modal = 0;
+    int helpcode;
     int clear = 0;
 
     if (open_edit_dialog != NULL && ci != MINIBUF) {
@@ -1288,32 +1224,31 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	return;
     }
 
-    d = dialog_data_new(okptr, ci, title, canceled);
+    helpcode = edit_dialog_help_code(ci, okptr);
+    d = edit_dialog_new(ci, title, okfunc, okptr, helpcode,
+			parent, canceled);
     if (d == NULL) return;
 
     open_edit_dialog = d->dialog;
-    set_dialog_border_widths(d->dialog);
+    set_dialog_border_widths(d->vbox, d->bbox);
     gretl_dialog_set_resizeable(d->dialog, FALSE);
 
-    /* convenience pointers */
-    top_vbox = gtk_dialog_get_content_area(GTK_DIALOG(d->dialog));
-    button_box = gtk_dialog_get_action_area(GTK_DIALOG(d->dialog));
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(button_box), 
+    gtk_button_box_set_layout(GTK_BUTTON_BOX(d->bbox), 
 			      GTK_BUTTONBOX_END);
 
     if (ci == ESTIMATE) {
 	/* estimating saved equation system */
 	dlg_display_sys(d);
     } else if (ci == SYSTEM && d->data != NULL) {
-	/* repecifying equation system */
-	edit_dialog_add_note(info, top_vbox);
+	/* respecifying equation system */
+	edit_dialog_add_note(info, d->vbox);
 	dlg_display_sys(d);
 	clear = 1;
     } else if (ci == NLS || ci == MLE || ci == GMM ||
 	       ci == SYSTEM || ci == RESTRICT) {
 	int hsize = 62;
 
-	edit_dialog_add_note(info, top_vbox);
+	edit_dialog_add_note(info, d->vbox);
 	d->edit = dlg_text_edit_new(&hsize, TRUE);
 	dialog_table_setup(d, hsize);
 	gretl_dialog_set_resizeable(d->dialog, TRUE);
@@ -1341,17 +1276,16 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	if (info != NULL) {
 	    w = gtk_label_new(info);
 	    gtk_label_set_justify(GTK_LABEL(w), GTK_JUSTIFY_LEFT);
-	    gtk_box_pack_start(GTK_BOX(top_vbox), w, TRUE, TRUE, 5);
+	    gtk_box_pack_start(GTK_BOX(d->vbox), w, TRUE, TRUE, 5);
 	    gtk_widget_show(w);
 	}
 
 	d->edit = gtk_entry_new();
-	gtk_box_pack_start(GTK_BOX(top_vbox), d->edit, TRUE, TRUE, 5);
+	gtk_box_pack_start(GTK_BOX(d->vbox), d->edit, TRUE, TRUE, 5);
 
 	/* make the Enter key do the business */
 	if (okfunc != NULL) {
-	    g_signal_connect(G_OBJECT(d->edit), "activate", 
-			     G_CALLBACK(okfunc), d);
+	    gtk_entry_set_activates_default(GTK_ENTRY(d->edit), TRUE);
 	}
 
 	if (deflt != NULL && *deflt != '\0') {
@@ -1359,8 +1293,6 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
 	    gtk_editable_select_region(GTK_EDITABLE(d->edit), 0, strlen(deflt));
 	} 
 
-	gtk_widget_show(d->edit);
-
 	g_signal_connect(G_OBJECT(GTK_EDITABLE(d->edit)), "changed", 
 			 G_CALLBACK(raise_and_focus_dialog), d->dialog);
     }
@@ -1368,94 +1300,83 @@ void edit_dialog (const char *title, const char *info, const char *deflt,
     if (ci == SYSTEM || ci == ESTIMATE) {
 	GtkWidget *bt, *bv;
 
-	bt = dialog_option_switch(top_vbox, d, OPT_T, NULL);
-	bv = dialog_option_switch(top_vbox, d, OPT_V, NULL);
-	system_estimator_list(top_vbox, d, bt, bv);
+	bt = dialog_option_switch(d->vbox, d, OPT_T, NULL);
+	bv = dialog_option_switch(d->vbox, d, OPT_V, NULL);
+	system_estimator_list(d->vbox, d, bt, bv);
     } else if (ci == NLS || ci == MLE) {
-	dialog_option_switch(top_vbox, d, OPT_V, pmod);
-	dialog_option_switch(top_vbox, d, OPT_R, pmod);
+	dialog_option_switch(d->vbox, d, OPT_V, pmod);
+	dialog_option_switch(d->vbox, d, OPT_R, pmod);
 	if (ci == MLE) {
-	    iter_control_button(top_vbox, d, pmod);
+	    iter_control_button(d->vbox, d, pmod);
 	}
     } else if (ci == GMM) {
-	dialog_option_switch(top_vbox, d, OPT_V, pmod);
-	build_gmm_combo(top_vbox, d, pmod);
+	dialog_option_switch(d->vbox, d, OPT_V, pmod);
+	build_gmm_combo(d->vbox, d, pmod);
     } else if (ci == RESTRICT && ols_model_window(okptr)) {
-	dialog_option_switch(top_vbox, d, OPT_B, NULL);
+	dialog_option_switch(d->vbox, d, OPT_B, NULL);
     } else if (ci == RESTRICT && vecm_model_window(okptr)) {
-	dialog_option_switch(top_vbox, d, OPT_F, NULL);
+	dialog_option_switch(d->vbox, d, OPT_F, NULL);
     } else if (ci == GR_BOX) {
-	dialog_option_switch(top_vbox, d, OPT_O, NULL);
+	dialog_option_switch(d->vbox, d, OPT_O, NULL);
     }
 
-    if (varclick == VARCLICK_INSERT_ID) { 
+    if (click == VARCLICK_INSERT_ID) { 
 	active_edit_id = d->edit; 
-    } else if (varclick == VARCLICK_INSERT_NAME) {
+    } else if (click == VARCLICK_INSERT_NAME) {
 	active_edit_name = d->edit;
-    } else if (varclick == VARCLICK_INSERT_TEXT) { 
+    } else if (click == VARCLICK_INSERT_TEXT) { 
 	active_edit_text = d->edit;
-    } else if (ci != ESTIMATE) {
-	modal = 1;
-    }
+    } 
 
-    hlpcode = edit_dialog_help_code(ci, okptr);
-
-    if (varclick == VARCLICK_NONE && !hlpcode) {
+    if (click == VARCLICK_NONE && helpcode == 0) {
 	gtk_window_set_keep_above(GTK_WINDOW(d->dialog), TRUE);
-    }
+    } 
 
     gtk_widget_grab_focus(d->edit);
 
     /* "Clear" button? */
     if (clear) {
 	w = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
-	gtk_container_add(GTK_CONTAINER(button_box), w);
+	gtk_container_add(GTK_CONTAINER(d->bbox), w);
 	g_signal_connect(G_OBJECT(w), "clicked", 
 			 G_CALLBACK(clear_dlg_previous), d);
     }    
 
     /* "Cancel" button */
-    cancel_delete_button(button_box, d->dialog, canceled);
+    cancel_delete_button(d->bbox, d->dialog);
 
     /* "OK" button */
-    w = ok_button(button_box);
-    if (okfunc != NULL) {
-	g_signal_connect(G_OBJECT(w), "clicked", 
-			 G_CALLBACK(okfunc), d);
+    if (canceled != NULL) {
+	w = ok_validate_button(d->bbox, canceled, NULL);
     } else {
-	g_signal_connect(G_OBJECT(w), "clicked", 
-			 G_CALLBACK(edit_dialog_ok), d);
+	w = ok_button(d->bbox);
     }
+    g_signal_connect(G_OBJECT(w), "clicked", 
+		     G_CALLBACK(edit_dialog_ok), d);
     gtk_widget_grab_default(w);
 
     /* "Help" button, if wanted */
-    if (hlpcode > 0) {
-	context_help_button(button_box, hlpcode);
-	modal = 0;
-    }
-
-    gtk_widget_show_all(button_box); 
-
-    if (ci == MODEL_GENR || ci == RESTRICT) {
-	windata_t *vwin = (windata_t *) okptr;
-
-	/* schedule destruction */
-	g_signal_connect(G_OBJECT(vwin->main), "destroy",
-			 G_CALLBACK(cancel_vwin_edit), 
-			 NULL);
+    if (helpcode > 0) {
+	context_help_button(d->bbox, helpcode);
     } 
 
-    if (modal) {
-	g_signal_connect(G_OBJECT(d->dialog), "show", 
-			 G_CALLBACK(gretl_set_window_modal), NULL);
-    }    
+    gtk_widget_show_all(d->dialog); 
+}
 
-    if (d->blocking) {
-	g_signal_connect(G_OBJECT(d->dialog), "show", 
-			 G_CALLBACK(gtk_main), NULL);
+void edit_dialog (int ci, const char *title, 
+		  const char *info, const char *deflt, 
+		  void (*okfunc)(), void *okptr, 
+		  Varclick click, GtkWidget *parent)
+{
+    blocking_edit_dialog(ci, title, info, deflt, okfunc, okptr,
+			 click, parent, NULL);
+}
+
+void edit_dialog_reset (dialog_t *dlg)
+{
+    if (dlg->cancel != NULL) {
+	*dlg->cancel = 1;
     }
-
-    gtk_widget_show(d->dialog); 
 }
 
 gchar *entry_box_get_trimmed_text (GtkWidget *w)
@@ -1593,13 +1514,7 @@ GdkModifierType parent_get_pointer_mask (GtkWidget *w)
 gboolean widget_get_pointer_info (GtkWidget *w, gint *x, gint *y,
 				  GdkModifierType *mask)
 {
-    GdkWindow *window;
-
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 14)    
-    window = w->window;
-#else
-    window = gtk_widget_get_window(w);
-#endif
+    GdkWindow *window = gtk_widget_get_window(w);
 
     if (window == NULL) {
 	return FALSE;
@@ -1633,7 +1548,9 @@ void gretl_emulated_dialog_add_structure (GtkWidget *dlg,
     gtk_container_set_border_width(GTK_CONTAINER(*pvbox), 5);
     gtk_box_set_spacing(GTK_BOX(*pvbox), 5);
 
+#if 0
     vbox_add_hsep(base);
+#endif
 
     *pbbox = gtk_hbutton_box_new();
     gtk_button_box_set_layout(GTK_BUTTON_BOX(*pbbox), 
