@@ -434,7 +434,9 @@ static void open_file_from_filelist (GtkAction *action)
     if (!strcmp(ftype, "Data")) {
 	strcpy(tryfile, datap[i]);
 	if (strstr(tryfile, ".csv")) {
-	    if (csv_options_dialog(NULL)) {
+	    int resp = csv_options_dialog(OPEN_CSV, GRETL_OBJ_DSET, NULL);
+
+	    if (canceled(resp)) {
 		return;
 	    }
 	}
@@ -526,7 +528,7 @@ static void real_add_files_to_menus (int ftype)
 		alabel = g_strdup_printf("%d. %s", k+1, endbit(tmp, fname, 1));
 		entry.name = aname;
 		entry.label = alabel;
-		id[i] = vwin_menu_add_item(mdata, mpath[j], &entry);
+		id[i] = vwin_menu_add_item_unique(mdata, aname, mpath[j], &entry);
 		apath = g_strdup_printf("%s/%s", mpath[j], aname);
 		w = gtk_ui_manager_get_widget(mdata->ui, apath);
 		if (w != NULL) {
