@@ -28,7 +28,10 @@
 #include "fnsave.h"
 #include "database.h"
 #include "datafiles.h"
+#include "guiprint.h"
 #include "graphics.h"
+#include "winstack.h"
+#include "tabwin.h"
 #include "bootstrap.h"
 
 #include <sys/stat.h>
@@ -237,6 +240,9 @@ static void script_window_update (windata_t *vwin, const char *fname)
     gchar *trfname, *title;
     const char *p = strrchr(fname, SLASH);
 
+    /* update internal filename record */
+    strcpy(vwin->fname, fname);
+
     /* ensure UTF-8 filename for display */
     if (p != NULL) {
 	trfname = my_filename_to_utf8(p + 1);
@@ -244,14 +250,17 @@ static void script_window_update (windata_t *vwin, const char *fname)
 	trfname = my_filename_to_utf8(fname);
     }
 
-    /* update the window title */
-    title = g_strdup_printf("gretl: %s", trfname);
-    gtk_window_set_title(GTK_WINDOW(vwin->main), title);
-    g_free(trfname);
-    g_free(title);
+    if (window_is_tab(vwin)) {
+	/* update the tab label */
+	tabwin_tab_set_title(vwin, trfname);
+    } else {
+	/* update the window title */
+	title = g_strdup_printf("gretl: %s", trfname);
+	gtk_window_set_title(GTK_WINDOW(vwin->main), title);
+	g_free(title);
+    }
 
-    /* and update internal filename record */
-    strcpy(vwin->fname, fname);
+    g_free(trfname);
 
     if (vwin->role == VIEW_LOG || vwin->role == VIEW_SCRIPT ||
 	vwin->role == VIEW_PKG_CODE) {
@@ -266,13 +275,14 @@ static void script_window_update (windata_t *vwin, const char *fname)
 
     /* make the window editable */
     if (!gtk_text_view_get_editable(GTK_TEXT_VIEW(vwin->text))) {
-	view_window_set_editable(vwin);
+	viewer_set_editable(vwin);
     }
 }
 
 static void 
 save_editable_content (int action, const char *fname, windata_t *vwin)
 {
+    const gchar *cset;
     FILE *fp;
     gchar *buf;
     gchar *trbuf;
@@ -290,9 +300,13 @@ save_editable_content (int action, const char *fname, windata_t *vwin)
 	return;
     }
 
+    if (!g_get_charset(&cset)) {
+	/* UTF-8 minuses not wanted for locale */
+	strip_unicode_minus(buf);
+    }
+
     trbuf = my_locale_from_utf8(buf);
     if (trbuf != NULL) {
-	/* UTF minuses? */
 	system_print_buf(trbuf, fp);
 	g_free(trbuf);
     }
@@ -328,21 +342,22 @@ static void filesel_save_prn_buffer (PRN *prn, const char *fname)
 
 static void filesel_open_script (const char *fname, windata_t *vwin)
 {
-    if (vwin != NULL) {
-	/* we're called from an existing script editor window */
+    if (vwin != NULL && !window_is_tab(vwin)) {
+	/* we're called from an existing (and untabbed) script 
+	   editor window */
 	strcpy(tryfile, fname);
 	sourceview_insert_file(vwin, fname);
     } else if (has_suffix(fname, ".R")) {
-	view_file(fname, 1, 0, 78, 370, EDIT_R);
+	view_script(fname, 1, EDIT_R);
     } else if (has_suffix(fname, ".plt")) {
-	view_file(fname, 1, 0, 78, 370, EDIT_GP);
+	view_script(fname, 1, EDIT_GP);
     } else if (ox_support && has_suffix(fname, ".ox")) {
-	view_file(fname, 1, 0, 78, 370, EDIT_OX);
+	view_script(fname, 1, EDIT_OX);
     } else if (has_suffix(fname, ".m")) {
-	view_file(fname, 1, 0, 78, 370, EDIT_OCTAVE);
+	view_script(fname, 1, EDIT_OCTAVE);
     } else {
 	strcpy(tryfile, fname);
-	if (view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT) != NULL) {
+	if (view_script(tryfile, 1, EDIT_SCRIPT) != NULL) {
 	    strcpy(scriptfile, tryfile);
 	    mkfilelist(FILE_LIST_SCRIPT, scriptfile);
 	    gretl_set_current_dir(scriptfile);
@@ -361,9 +376,9 @@ static void filesel_open_session (const char *fname)
 	windata_t *vwin;
 
 	if (has_system_prefix(tryfile, SCRIPT_SEARCH)) {
-	    vwin = view_file(tryfile, 0, 0, 78, 370, VIEW_SCRIPT);
+	    vwin = view_script(tryfile, 0, VIEW_SCRIPT);
 	} else {
-	    vwin = view_file(tryfile, 1, 0, 78, 370, EDIT_SCRIPT);
+	    vwin = view_script(tryfile, 1, EDIT_SCRIPT);
 	}
 
 	if (vwin != NULL) {
@@ -475,10 +490,11 @@ static int overwrite_stop (const char *fname)
 }
 
 static void
-file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
-			      gpointer data)
+file_selector_process_result (const char *in_fname, int action, 
+			      FselDataSrc src, gpointer data)
 {
     char fname[FILENAME_MAX];
+    int quit = 0;
     int err = 0;
 
     *fname = 0;
@@ -538,9 +554,14 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
     if (EXPORT_ACTION(action, src) || action == EXPORT_GDT ||
 	(action > END_SAVE_DATA && action < END_SAVE_OTHER)) {
 	/* saving CSV, graphs etc.; check overwrite */
-	if (overwrite_stop(fname)) {
-	    return;
-	}
+	quit = overwrite_stop(fname);
+    } else if (action == SAVE_DATA_AS && strcmp(fname, datafile)) {
+	/* "saving as" over an existing gdt file */
+	quit = overwrite_stop(fname);
+    }
+
+    if (quit) {
+	return;
     }
 
     if (action == SAVE_TEX) {
@@ -552,7 +573,7 @@ file_selector_process_result (const char *in_fname, int action, FselDataSrc src,
     } else if (src == FSEL_DATA_PRN) {
 	filesel_save_prn_buffer((PRN *) data, fname);
     } else if (SAVE_DATA_ACTION(action)) {
-	err = do_store(fname, action);
+	err = do_store(fname, action, data);
     } else if (action == SAVE_GNUPLOT) {
 	save_graph_to_file(data, fname);
     } else if (action == SAVE_GRAPHIC) {
@@ -643,6 +664,9 @@ static GtkFileFilter *get_file_filter (int action, gpointer data)
 
 #ifdef G_OS_WIN32
 
+/* On getting an output filename @s from the GTK file chooser, 
+   convert to the locale encoding */
+
 static gchar *inplace_windows_filename (gchar *s)
 {
     char tmp[MAXLEN];
@@ -658,6 +682,9 @@ static gchar *inplace_windows_filename (gchar *s)
     return s;
 }
 
+/* Having constructed a path @s for input use with the GTK 
+   file chooser, ensure that it's in UTF-8 */
+
 static char *win32_correct_path (char *s)
 {
     if (!g_utf8_validate(s, -1, NULL)) {
@@ -926,7 +953,7 @@ void file_selector (int action, FselDataSrc src, gpointer data)
     if (src == FSEL_DATA_VWIN) {
 	windata_t *vwin = (windata_t *) data;
 
-	w = vwin->main;
+	w = vwin_toplevel(vwin);
     }
 
     gtk_file_selector(action, src, data, w);
