@@ -924,7 +924,7 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
 	fprintf(fp, "plot '-' using 1:2 title '%s' w lines\n", title);
 	print_gp_data(finfo, obs, u, fp);
 	fputs("e\n", fp);
-	fputs("set nomultiplot\n", fp);
+	fputs("unset multiplot\n", fp);
     } else if (finfo->ftype == FILTER_FD) {
 	sprintf(ztitle, "fracdiff(%s, %g)", finfo->vname, finfo->lambda);
 	fprintf(fp, "plot '-' using 1:2 title '%s' w lines\n", ztitle);
@@ -961,11 +961,7 @@ do_filter_graph (filter_info *finfo, const double *fx, const double *u)
     fclose(fp);
 
     err = gnuplot_make_graph();
-    if (err) {
-	gui_errmsg(err);
-    } else {
-	register_graph(NULL);
-    }
+    gui_graph_handler(err);
 
     return err;
 }
@@ -1022,11 +1018,7 @@ static void butterworth_poles_graph (GtkWidget *button, filter_info *finfo)
     fclose(fp);
 
     err = gnuplot_make_graph();
-    if (err) {
-	gui_errmsg(err);
-    } else {
-	register_graph(NULL);
-    }
+    gui_graph_handler(err);
 }
 
 static void weights_shape_graph (GtkWidget *button, filter_info *finfo)
@@ -1068,11 +1060,7 @@ static void weights_shape_graph (GtkWidget *button, filter_info *finfo)
 	destroy_private_matrices();
     }
 
-    if (err) {
-	gui_errmsg(err);
-    } else {
-	register_graph(NULL);
-    }
+    gui_graph_handler(err);
 }
 
 static int do_filter_response_graph (filter_info *finfo)
@@ -1141,11 +1129,7 @@ static int do_filter_response_graph (filter_info *finfo)
     fclose(fp);
 
     err = gnuplot_make_graph();
-    if (err) {
-	gui_errmsg(err);
-    } else {
-	register_graph(NULL);
-    }
+    gui_graph_handler(err);
 
     return err;
 }
@@ -1371,12 +1355,11 @@ static int filter_code (GtkAction *action)
 void filter_callback (GtkAction *action)
 {
     filter_info finfo;
+    int code = filter_code(action);
     int v = mdata_active_var();
     int t1 = dataset->t1;
     int t2 = dataset->t2;
-    int code, err = 0;
-
-    code = filter_code(action);
+    int err = 0;
 
     err = series_adjust_sample(dataset->Z[v], &t1, &t2);
 
@@ -1386,10 +1369,8 @@ void filter_callback (GtkAction *action)
 
     if (err) {
 	gui_errmsg(err);
-	return;
+    } else {
+	filter_info_init(&finfo, code, v, t1, t2);
+	filter_dialog(&finfo);
     }
-
-    filter_info_init(&finfo, code, v, t1, t2);
-
-    filter_dialog(&finfo);
 }
