@@ -580,7 +580,7 @@ static void update_combo_selectors (call_info *cinfo,
     g_list_free(newlist);
 }
 
-int do_make_list (selector *sr)
+static int do_make_list (selector *sr)
 {
     const char *buf = selector_list(sr);
     const char *lname = selector_entry_text(sr);
@@ -666,20 +666,20 @@ int do_make_list (selector *sr)
     return err;
 } 
 
-static void launch_list_maker (GtkWidget *w, GtkWidget *entry)
+static void launch_list_maker (GtkWidget *button, GtkWidget *entry)
 {
-    selector *sr;
-    GtkWidget *dlg;
+    call_info *cinfo;
 
-    sr = simple_selection(_("Define list"), do_make_list, DEFINE_LIST, 
-			  entry);
-    dlg = selector_get_window(sr);
-    gtk_window_set_keep_above(GTK_WINDOW(dlg), TRUE);
+    cinfo = g_object_get_data(G_OBJECT(button), "cinfo");
+    simple_selection_with_data(DEFINE_LIST, _("Define list"), 
+			       do_make_list, cinfo->dlg, 
+			       entry);
 }
 
 void gui_define_list (void)
 {
-    launch_list_maker(NULL, NULL);
+    simple_selection_with_data(DEFINE_LIST, _("Define list"), 
+			       do_make_list, NULL, NULL);
 }
 
 static void launch_matrix_maker (GtkWidget *button, call_info *cinfo)
@@ -718,20 +718,20 @@ static void launch_series_maker (GtkWidget *button, call_info *cinfo)
 {
     GtkWidget *combo = g_object_get_data(G_OBJECT(button), "combo");
 
-    edit_dialog(_("add series"), 
-		_("Enter name=formula for new series"),
-		NULL, do_fncall_genr, combo, 
-		GENR, VARCLICK_INSERT_NAME, NULL);  
+    edit_dialog(GENR, _("add series"), 
+		_("Enter name=formula for new series"), NULL,
+		do_fncall_genr, combo, 
+		VARCLICK_INSERT_NAME, cinfo->dlg);  
 }
 
 static void launch_scalar_maker (GtkWidget *button, call_info *cinfo)
 {
     GtkWidget *combo = g_object_get_data(G_OBJECT(button), "combo");
 
-    edit_dialog(_("add scalar"), 
-		_("Enter name=formula for new scalar"),
-		NULL, do_fncall_genr, combo, 
-		GENR, VARCLICK_INSERT_NAME, NULL);  
+    edit_dialog(GENR, _("add scalar"), 
+		_("Enter name=formula for new scalar"), NULL,
+		do_fncall_genr, combo, 
+		VARCLICK_INSERT_NAME, cinfo->dlg);  
 }
 
 static GtkWidget *bool_arg_selector (call_info *cinfo, int i)
@@ -1288,6 +1288,7 @@ static void function_call_dialog (call_info *cinfo)
 	    button = add_object_button(ptype, sel);
 	    add_table_cell(tbl, button, 2, 3, row);
 	    widget_set_int(entry, "argnum", i);
+	    g_object_set_data(G_OBJECT(button), "cinfo", cinfo);
 	    g_signal_connect(G_OBJECT(button), "clicked", 
 			     G_CALLBACK(launch_list_maker),
 			     entry);
@@ -1530,6 +1531,7 @@ static void set_genr_model_from_vwin (windata_t *vwin)
 static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 {
     ExecState state;
+    GtkWidget *hbox;
     char fnline[MAXLINE];
     char *tmpname = NULL;
     const char *funname;
@@ -1590,17 +1592,17 @@ static int real_GUI_function_call (call_info *cinfo, PRN *prn)
 	set_genr_model_from_vwin(cinfo->vwin);
     }
 
-#if USE_GTK_SPINNER
-    if (cinfo->top_hbox != NULL) {
-	start_wait_for_output(cinfo->top_hbox, 0);
+    hbox = cinfo->top_hbox;
+
+    if (hbox != NULL) {
+	start_wait_for_output(hbox, FALSE);
     }
-#endif
+
     err = gui_exec_line(&state, dataset);
-#if USE_GTK_SPINNER
-    if (cinfo->top_hbox != NULL) {
-	stop_wait_for_output(cinfo->top_hbox);
+
+    if (hbox != NULL) {
+	stop_wait_for_output(hbox);
     }
-#endif
 
     if (cinfo->flags & MODEL_CALL) {
 	unset_genr_model();
@@ -1685,7 +1687,7 @@ static void select_interface (call_info *cinfo, int npub)
     } else if (radios) {
 	resp = radio_dialog("gretl", "select function", 
 			    (const char **) opts, 
-			    nopts, 0, 0);
+			    nopts, 0, 0, cinfo->dlg);
 	if (resp >= 0) {
 	    cinfo->iface = cinfo->publist[resp+1];
 	} else {
@@ -1693,7 +1695,8 @@ static void select_interface (call_info *cinfo, int npub)
 	}
 	free_strings_array(opts, nopts);
     } else {
-	resp = combo_selector_dialog(ilist, "select function", 0);
+	resp = combo_selector_dialog(ilist, "select function", 
+				     0, cinfo->dlg);
 	if (resp >= 0) {
 	    cinfo->iface = cinfo->publist[resp+1];
 	} else {
@@ -2278,19 +2281,29 @@ static void gfn_menu_callback (GtkAction *action, windata_t *vwin)
     }
 }
 
-static int read_addons_info (void)
+static int package_is_unseen (const char *name, int n)
+{
+    int i;
+
+    for (i=0; i<n; i++) {
+	if (!strcmp(name, addons[i].pkgname)) {
+	    return 0;
+	}
+    }
+
+    return 1;
+}
+
+static int real_read_packages_file (const char *fname, int *pn,
+				    int optional)
 {
-    char fname[FILENAME_MAX];
     xmlDocPtr doc = NULL;
     xmlNodePtr cur = NULL;
-    int err, n = 0;
-
-    sprintf(fname, "%sfunctions%cpackages.xml", 
-	    gretl_home(), SLASH);
+    int err, n = *pn;
 
     err = gretl_xml_open_doc_root(fname, "gretl-package-info", &doc, &cur);
     if (err) {
-	return err;
+	return optional ? 0 : err;
     } 
 
     cur = cur->xmlChildrenNode;
@@ -2300,19 +2313,20 @@ static int read_addons_info (void)
 	    int mw = gretl_xml_get_prop_as_bool(cur, "model-window");
 	    int top = gretl_xml_get_prop_as_bool(cur, "toplev");
 	    xmlChar *name, *desc, *path;
+	    int freeit = 1;
 
 	    name = xmlGetProp(cur, (XUC) "name");
 	    desc = xmlGetProp(cur, (XUC) "label");
 	    path = xmlGetProp(cur, (XUC) "path");
 
 	    if (name == NULL || desc == NULL || path == NULL) {
-		err = 1;
-	    } else {
+		err = E_DATA;
+	    } else if (package_is_unseen((const char *) name, n)) {
 		addons = myrealloc(addons, (n+1) * sizeof *addons);
-
 		if (addons == NULL) {
-		    err = 1;
+		    err = E_ALLOC;
 		} else {
+		    freeit = 0;
 		    addons[n].pkgname = (char *) name;
 		    addons[n].label = (char *) desc;
 		    addons[n].menupath = (char *) path;
@@ -2322,29 +2336,49 @@ static int read_addons_info (void)
 		    n++;
 		} 
 	    }
-
-	    if (err) {
+	    if (freeit) {
 		free(name);
 		free(desc);
 		free(path);
 	    }		
 	}
-
 	if (!err) {
 	    cur = cur->next;
 	}
     }
 
+    if (doc != NULL) {
+	xmlFreeDoc(doc);
+    }    
+
+    *pn = n;
+
+    return err;
+}
+
+/* read "packages.xml" to find out what's what among packages
+   that offer to place themselves in the gretl menu system
+*/
+
+static int read_addons_info (void)
+{
+    char fname[FILENAME_MAX];
+    int err, n = 0;
+
+    sprintf(fname, "%sfunctions%cpackages.xml", gretl_home(), SLASH);
+    err = real_read_packages_file(fname, &n, 0);
+
+    if (!err) {
+	sprintf(fname, "%sfunctions%cpackages.xml", gretl_dotdir(), SLASH);
+	err = real_read_packages_file(fname, &n, 1);
+    }
+
     if (err) {
 	destroy_addons_info();
     } else {
 	n_addons = n;
     }
 
-    if (doc != NULL) {
-	xmlFreeDoc(doc);
-    }
-
     return err;
 }
 
@@ -2461,7 +2495,6 @@ static int maybe_add_model_pkg (addon_info *addon,
 	    ufunc *func = get_function_from_package(precheck, pkg);
 	    
 	    if (func == NULL || precheck_error(func, vwin)) {
-		fprintf(stderr, "precheck failed\n");
 		err = 1;
 	    }
 	} 
