@@ -20,6 +20,11 @@
 #ifndef FNCALL_H
 #define FNCALL_H
 
+enum {
+    FN_NO_LOAD,
+    FN_NO_DATA
+} fncall_errors;
+
 void call_function_package (const char *fname, windata_t *vwin,
 			    int *loaderr);
 
@@ -39,4 +44,8 @@ void maybe_add_packages_to_model_menus (windata_t *vwin);
 
 void maybe_add_packages_to_menus (windata_t *vwin);
 
+int query_addons (void);
+
+int download_addon (const char *pkgname, char **local_path);
+
 #endif /* FNCALL_H */
