@@ -113,6 +113,7 @@ function_info *finfo_new (void)
     finfo->active = NULL;
     finfo->samplewin = NULL;
     finfo->codewins = NULL;
+    finfo->codesel = NULL;
     finfo->popup = NULL;
 
     finfo->help = NULL;
@@ -206,7 +207,7 @@ static void login_init_or_free (login_info *linfo, int freeit)
     } else {
 	linfo->login = (login == NULL)? NULL : g_strdup(login);
 	linfo->pass = (pass == NULL)? NULL : g_strdup(pass);
-	linfo->canceled = 0;
+	linfo->canceled = 1;
     }
 }
 
@@ -366,12 +367,6 @@ static void finfo_destroy (GtkWidget *w, function_info *finfo)
     finfo_free(finfo);
 }
 
-static void login_cancel (GtkWidget *w, login_info *linfo)
-{
-    linfo->canceled = 1;
-    gtk_widget_destroy(linfo->dlg);
-}
-
 static gboolean update_active_func (GtkComboBox *menu, 
 				    function_info *finfo)
 {
@@ -490,7 +485,11 @@ static void edit_code_callback (GtkWidget *w, function_info *finfo)
 
     fun = get_function_from_package(finfo->active, finfo->pkg);
     if (fun == NULL) {
-	errbox(_("Can't find the function '%s'"), finfo->active);
+	/* the package may not be saved yet */
+	fun = get_user_function_by_name(finfo->active);
+	if (fun == NULL) {
+	    errbox(_("Can't find the function '%s'"), finfo->active);
+	}
     }
 
     if (bufopen(&prn)) {
@@ -798,10 +797,11 @@ static int finfo_set_function_names (function_info *finfo,
 static void func_selector_set_strings (function_info *finfo, 
 				       GtkWidget *ifmenu)
 {
-    int i;
+    int i, n = 0;
 
     for (i=0; i<finfo->n_pub; i++) {
 	combo_box_append_text(ifmenu, finfo->pubnames[i]);
+	n++;
     }
 
     for (i=0; i<finfo->n_priv; i++) {
@@ -809,9 +809,11 @@ static void func_selector_set_strings (function_info *finfo,
 
 	combo_box_append_text(ifmenu, s);
 	g_free(s);
+	n++;
     }
 
     gtk_combo_box_set_active(GTK_COMBO_BOX(ifmenu), 0);
+    gtk_widget_set_sensitive(ifmenu, n > 1);
 }
 
 static GtkWidget *active_func_selector (function_info *finfo)
@@ -1068,7 +1070,7 @@ static void finfo_dialog (function_info *finfo)
     gtk_window_set_default_size(GTK_WINDOW(finfo->dlg), 640, 500);
 
     if (finfo->fname != NULL) {
-	gchar *title = title_from_filename(finfo->fname);
+	gchar *title = title_from_filename(finfo->fname, TRUE);
 
 	gtk_window_set_title(GTK_WINDOW(finfo->dlg), title);
 	g_free(title);
@@ -1077,6 +1079,7 @@ static void finfo_dialog (function_info *finfo)
 			     _("gretl: function package editor"));
     } 
 
+    gtk_widget_set_name(finfo->dlg, "pkg-editor");
     g_signal_connect(G_OBJECT(finfo->dlg), "delete-event",
 		     G_CALLBACK(query_save_package), finfo);
     g_signal_connect(G_OBJECT(finfo->dlg), "destroy", 
@@ -1156,12 +1159,10 @@ static void finfo_dialog (function_info *finfo)
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(edit_code_callback), finfo);
 
-    if (finfo->n_pub + finfo->n_priv > 1) {
-	finfo->codesel = active_func_selector(finfo);
-	gtk_box_pack_start(GTK_BOX(hbox), finfo->codesel, FALSE, FALSE, 5);
-	g_signal_connect(G_OBJECT(finfo->codesel), "changed",
-			 G_CALLBACK(update_active_func), finfo);
-    } 
+    finfo->codesel = active_func_selector(finfo);
+    gtk_box_pack_start(GTK_BOX(hbox), finfo->codesel, FALSE, FALSE, 5);
+    g_signal_connect(G_OBJECT(finfo->codesel), "changed",
+		     G_CALLBACK(update_active_func), finfo);
 
     update_active_func(NULL, finfo);
 
@@ -1238,7 +1239,7 @@ static void finfo_dialog (function_info *finfo)
     finfo_set_modified(finfo, finfo->fname == NULL);
 
     gtk_widget_show_all(finfo->dlg);
-    add_window_list_item(finfo->dlg, SAVE_FUNCTIONS);
+    window_list_add(finfo->dlg, SAVE_FUNCTIONS);
 }
 
 static void web_get_login (GtkWidget *w, gpointer p)
@@ -1246,7 +1247,7 @@ static void web_get_login (GtkWidget *w, gpointer p)
     browser_open("http://gretl.ecn.wfu.edu/apply/");
 }
 
-static void login_dialog (login_info *linfo)
+static void login_dialog (login_info *linfo, GtkWidget *parent)
 {
     GtkWidget *button, *label;
     GtkWidget *tbl, *vbox, *hbox;
@@ -1254,11 +1255,10 @@ static void login_dialog (login_info *linfo)
 
     login_init(linfo);
 
-    linfo->dlg = gretl_dialog_new(_("gretl: upload"), NULL, GRETL_DLG_BLOCK);
+    linfo->dlg = gretl_dialog_new(_("gretl: upload"), parent, GRETL_DLG_BLOCK);
     vbox = gtk_dialog_get_content_area(GTK_DIALOG(linfo->dlg));
 
     hbox = label_hbox(vbox, _("Upload function package"));
-    gtk_widget_show(hbox);
 
     tbl = gtk_table_new(2, 2, FALSE);
     gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 5);
@@ -1271,7 +1271,6 @@ static void login_dialog (login_info *linfo)
 	gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, i, i+1,
 			 GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
 			 5, 5);
-	gtk_widget_show(label);
 
 	entry = gtk_entry_new();
 	gtk_entry_set_width_chars(GTK_ENTRY(entry), 34);
@@ -1280,7 +1279,6 @@ static void login_dialog (login_info *linfo)
 	if (src != NULL) {
 	    gtk_entry_set_text(GTK_ENTRY(entry), src);
 	}
-	gtk_widget_show(entry); 
 
 	if (i == 0) {
 	    linfo->login_entry = entry;
@@ -1290,14 +1288,11 @@ static void login_dialog (login_info *linfo)
 	}
     }
 
-    gtk_widget_show(tbl);
-
     hbox = label_hbox(vbox, 
 		      _("If you don't have a login to the gretl server\n"
 			"please see http://gretl.ecn.wfu.edu/apply/.\n"
 			"The 'Website' button below should open this page\n"
 			"in your web browser."));
-    gtk_widget_show(hbox);
 
     /* control button area */
 
@@ -1306,15 +1301,13 @@ static void login_dialog (login_info *linfo)
     /* Cancel */
     button = cancel_button(hbox);
     g_signal_connect(G_OBJECT(button), "clicked", 
-		     G_CALLBACK(login_cancel), linfo);
-    gtk_widget_show(button);
+		     G_CALLBACK(delete_widget), linfo->dlg);
 
     /* OK */
-    button = ok_button(hbox);
+    button = ok_validate_button(hbox, &linfo->canceled, NULL);
     g_signal_connect(G_OBJECT(button), "clicked",
 		     G_CALLBACK(login_finalize), linfo);
     gtk_widget_grab_default(button);
-    gtk_widget_show(button);
 
     /* Website */
     button = gtk_button_new_with_label("Website");
@@ -1324,9 +1317,8 @@ static void login_dialog (login_info *linfo)
 				       button, TRUE);
     g_signal_connect(G_OBJECT(button), "clicked", 
 		     G_CALLBACK(web_get_login), NULL);
-    gtk_widget_show(button);
 
-    gtk_widget_show(linfo->dlg);
+    gtk_widget_show_all(linfo->dlg);
 }
 
 /* Check the package file against gretlfunc.dtd. We call
@@ -1403,7 +1395,7 @@ static int validate_package_file (const char *fname, int verbose)
     return err;
 }
 
-static void do_upload (const char *fname)
+static void do_pkg_upload (function_info *finfo, const char *fname)
 {
     gchar *buf = NULL;
     char *retbuf = NULL;
@@ -1420,7 +1412,7 @@ static void do_upload (const char *fname)
 	return;
     }
 
-    login_dialog(&linfo);
+    login_dialog(&linfo, finfo->dlg);
 
     if (linfo.canceled) {
 	linfo_free(&linfo);
@@ -1602,7 +1594,7 @@ int save_function_package (const char *fname, gpointer p)
 	gtk_widget_set_sensitive(finfo->validate, TRUE);
 	maybe_update_func_files_window(EDIT_FN_PKG);
 	if (finfo->upload) {
-	    do_upload(fname);
+	    do_pkg_upload(finfo, fname);
 	}
     }
 
@@ -1835,6 +1827,9 @@ void revise_function_package (void *p)
 	finfo_set_modified(finfo, TRUE);
     }
 
+    fprintf(stderr, "finfo->n_pub=%d, finfo->n_priv=%d\n", 
+	    finfo->n_pub, finfo->n_priv);
+
     free(publist);
     free(privlist);
 }
