@@ -45,6 +45,8 @@
 #include "dlgutils.h"
 #include "fncall.h"
 #include "selector.h"
+#include "guiprint.h"
+#include "tabwin.h"
 
 #include <dirent.h>
 
@@ -74,7 +76,7 @@ static void make_main_window (void);
 
 static gboolean main_popup_handler (GtkWidget *w, GdkEventButton *event,
 				    gpointer data);
-static int set_up_main_menu (void);
+static GtkWidget *make_main_menu (void);
 static void start_R_callback (void);
 static void auto_store (void);
 static void restore_sample_callback (void);
@@ -82,6 +84,7 @@ static void show_sample_callback (void);
 static void mdata_select_all (void);
 static void mdata_select_list (void);
 static int gui_query_stop (void);
+static void mdata_handle_paste (void);
 
 GtkTargetEntry gretl_drag_targets[] = {
     { "text/uri-list",  0, GRETL_FILENAME },
@@ -169,7 +172,7 @@ char Rcommand[MAXSTR] = "/usr/X11R6/bin/xterm -e R";
 #else
 char Browser[MAXSTR] = "mozilla";
 char calculator[MAXSTR] = "xcalc";
-char latex[MAXSTR] = "latex";
+char latex[MAXSTR] = "pdflatex";
 char viewdvi[MAXSTR] = "xdvi";
 char viewpdf[MAXSTR] = "acroread";
 char viewps[MAXSTR] = "gv";
@@ -434,7 +437,8 @@ int main (int argc, char **argv)
     *filearg = '\0';
 
 #if GUI_DEBUG
-    fprintf(stderr, "starting gretl %s, %s\n", GRETL_VERSION, BUILD_DATE);
+    fprintf(stderr, "starting gretl %s, build date %s\n", GRETL_VERSION, 
+	    BUILD_DATE);
 #endif
 
     gtk_init_with_args(&argc, &argv, _(param_msg), options, "gretl", &opterr);
@@ -630,7 +634,7 @@ int main (int argc, char **argv)
 
     session_menu_state(FALSE);
     restore_sample_state(FALSE);
-    main_menubar_state(FALSE);
+    dataset_menubar_state(FALSE);
 
 #if GUI_DEBUG
     fprintf(stderr, "done setting GUI state\n");
@@ -754,8 +758,16 @@ static void mdata_avoid_zero (GtkTreeView *view, gpointer p)
 static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 {
     GdkModifierType mods = widget_get_pointer_mask(w);
+    int Ctrl = (mods & GDK_CONTROL_MASK);
+    int Alt = (mods & GDK_MOD1_MASK);
     int k = key->keyval;
 
+    if (Ctrl && k == GDK_v) {
+	/* Ctrl-V for paste */
+	mdata_handle_paste();
+	return TRUE;
+    }
+
     if (k == GDK_h || k == GDK_F1) {
 	/* invoke help */
 	plain_text_cmdref(NULL);
@@ -768,14 +780,14 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 	/* launch the console */
 	gretl_console();
 	return TRUE;
-    } else if (mods & GDK_MOD1_MASK) {
+    } else if (Alt) {
 	if (k == GDK_x) {
 	    /* Alt-x: invoke command minibuffer */
 	    minibuf_callback();
 	    return TRUE;
 	} else if (k == GDK_w) {
 	    /* Alt-w: window list */
-	    window_list_popup(vwin->main);
+	    window_list_popup(w, NULL, vwin->main);
 	    return TRUE;
 	}
     }
@@ -785,7 +797,7 @@ static gint catch_mdata_key (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
     }
 
 #if defined(HAVE_FLITE) || defined(G_OS_WIN32)
-    if (!(mods & GDK_MOD1_MASK)) {
+    if (!Ctrl && !Alt) {
 	if (k == GDK_a) {
 	    audio_render_window(vwin, AUDIO_LISTBOX);
 	    return TRUE;
@@ -1083,11 +1095,11 @@ static void mdata_select_list (void)
 	return;
     } else {
 	char lname[32];
-	int cancel = 0;
+	int resp;
 
-	select_list_dialog(lname, &cancel);
+	resp = select_list_dialog(lname);
 
-	if (!cancel) {
+	if (!canceled(resp)) {
 	    int *list = get_list_by_name(lname);
 
 	    if (list != NULL) {
@@ -1179,6 +1191,7 @@ static void make_main_window (void)
     GtkWidget *main_vbox;
     GtkWidget *box, *dlabel;
     GtkWidget *align;
+    GtkWidget *menu;
     const char *titles[] = {
 	N_("ID #"), 
 	N_("Variable name"), 
@@ -1190,7 +1203,7 @@ static void make_main_window (void)
 	G_TYPE_STRING
     };
 
-    mdata = gretl_viewer_new(MAINWIN, "gretl", NULL, 0);
+    mdata = gretl_viewer_new(MAINWIN, "gretl", NULL);
     if (mdata == NULL) {
 	noalloc();
     }
@@ -1205,6 +1218,7 @@ static void make_main_window (void)
     }
 
     mdata->main = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    g_object_set_data(G_OBJECT(mdata->main), "vwin", mdata);
 
 #ifdef G_OS_WIN32
     set_up_windows_look();
@@ -1230,12 +1244,13 @@ static void make_main_window (void)
     gtk_container_add(GTK_CONTAINER(mdata->main), main_vbox);
     g_object_set_data(G_OBJECT(mdata->main), "vbox", main_vbox);
 
-    if (set_up_main_menu()) {
+    menu = make_main_menu();
+    if (menu == NULL) {
 	exit(EXIT_FAILURE);
     }
 
     /* put the main menu bar in place */
-    gtk_box_pack_start(GTK_BOX(main_vbox), mdata->mbar, FALSE, TRUE, 0);
+    gtk_box_pack_start(GTK_BOX(main_vbox), menu, FALSE, TRUE, 0);
 
     /* label for name of datafile */
     dlabel = gtk_label_new(_(" No datafile loaded ")); 
@@ -1304,7 +1319,7 @@ static void make_main_window (void)
     fprintf(stderr, "  add_mainwin_toolbar done\n");
 #endif
 
-    gtk_widget_show_all(mdata->main); 
+    gtk_widget_show_all(mdata->main);
 
 #if GUI_DEBUG
     fprintf(stderr, "  gtk_widget_show_all done\n");
@@ -1314,6 +1329,8 @@ static void make_main_window (void)
 	gtk_window_move(GTK_WINDOW(mdata->main), main_x, main_y);
     }
 
+    window_list_add(mdata->main, MAINWIN);
+
 #if GUI_DEBUG
     fprintf(stderr, "  possible window_move done\n");
 #endif
@@ -1416,6 +1433,7 @@ GtkActionEntry main_entries[] = {
     /* Data */
     { "Data", NULL, N_("_Data"), NULL, NULL, NULL },
     { "DataSelectAll", NULL, N_("Select _all"), "<control>A", NULL, G_CALLBACK(mdata_select_all) },
+    { "VarFind", GTK_STOCK_FIND, N_("_Find variable..."), NULL, NULL, G_CALLBACK(listbox_find) },
     { "DefineList", NULL, N_("Define or edit _list..."), NULL, NULL, G_CALLBACK(gui_define_list) },
     { "SelectList", NULL, N_("_Set selection from list..."), NULL, NULL, G_CALLBACK(mdata_select_list) },
     { "DisplayValues", NULL, N_("_Display values"), NULL, NULL, G_CALLBACK(display_selected) },
@@ -1431,14 +1449,10 @@ GtkActionEntry main_entries[] = {
     { "DataTranspose", NULL, N_("_Transpose data..."), NULL, NULL, G_CALLBACK(gui_transpose_data) },
     { "DataSort", NULL, N_("_Sort data..."), NULL, NULL, G_CALLBACK(gui_sort_data) },
     { "GSETMISS", NULL, N_("Set missing _value code..."), NULL, NULL, G_CALLBACK(gretl_callback) },
-    { "VarFind", GTK_STOCK_FIND, N_("_Find variable..."), NULL, NULL, G_CALLBACK(listbox_find) },
-    { "DataRefresh", NULL, N_("_Refresh window"), NULL, NULL, G_CALLBACK(refresh_data) },
 
     /* View */
     { "View", NULL, N_("_View"), NULL, NULL, NULL },
     { "IconView", NULL, N_("_Icon view"), NULL, NULL, G_CALLBACK(view_session) },
-    { "EditScalars", NULL, N_("_Scalars"), NULL, NULL, G_CALLBACK(edit_scalars) },   
-    { "Windows", NULL, N_("_Windows"), NULL, NULL, NULL },
     { "GraphVars", NULL, N_("_Graph specified vars"), NULL, NULL, NULL },
     { "TSPlot", NULL, N_("_Time series plot..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "ScatterPlot", NULL, N_("X-Y _scatter..."), NULL, NULL, G_CALLBACK(selector_callback) },
@@ -1501,6 +1515,7 @@ GtkActionEntry main_entries[] = {
     { "normtest", NULL, N_("_Normality test"), NULL, NULL, G_CALLBACK(menu_op_action) },
     { "FreqDist", NULL, N_("_Frequency distribution..."), NULL, NULL, G_CALLBACK(do_freq_dist) },
     { "Density", NULL, N_("Estimated _density plot..."), NULL, NULL, G_CALLBACK(do_kernel) },
+    { "boxplot", NULL, N_("_Boxplot"), NULL, NULL, G_CALLBACK(boxplot_callback) },
     { "qqplot", NULL, N_("Normal _Q-Q plot..."), NULL, NULL, G_CALLBACK(do_qqplot) },
     { "Gini", NULL, N_("_Gini coefficient"), NULL, NULL, G_CALLBACK(do_gini) },
     { "rmplot", NULL, N_("_Range-mean graph"), NULL, NULL, G_CALLBACK(do_range_mean) },
@@ -1584,7 +1599,9 @@ GtkActionEntry main_entries[] = {
     { "RobustModels", NULL, N_("_Robust estimation"), NULL, NULL, NULL },
     { "lad", NULL, N_("Least _Absolute Deviation..."), NULL, NULL, G_CALLBACK(model_callback) }, 
     { "quantreg", NULL, N_("_Quantile regression..."), NULL, NULL, G_CALLBACK(model_callback) }, 
-    { "spearman", NULL, N_("_Rank correlation..."), NULL, NULL, G_CALLBACK(selector_callback) }, 
+    { "spearman", NULL, N_("_Rank correlation..."), NULL, NULL, G_CALLBACK(selector_callback) },
+    { "loess", NULL, N_("_Loess..."), NULL, NULL, G_CALLBACK(selector_callback) }, 
+    { "nadarwat", NULL, N_("_Nadaraya-Watson..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "mle", NULL, N_("_Maximum likelihood..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
     { "gmm", NULL, N_("_GMM..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
     { "system", NULL, N_("_Simultaneous equations..."), NULL, NULL, G_CALLBACK(gretl_callback) }, 
@@ -1595,6 +1612,7 @@ GtkActionEntry main_entries[] = {
     { "FuncRef",   NULL, N_("_Function reference"), NULL, NULL, G_CALLBACK(genr_funcs_ref) },
     { "UserGuide", GRETL_STOCK_PDF, N_("_User's guide"), NULL, NULL, G_CALLBACK(display_pdf_help) },
     { "PDFCmdRef", GRETL_STOCK_PDF, N_("_Command reference"), NULL, NULL, G_CALLBACK(display_pdf_help) },
+    { "KbdRef", GRETL_STOCK_PDF, N_("_Keyboard shortcuts"), NULL, NULL, G_CALLBACK(display_pdf_help) },
     { "UpdateCheck", GTK_STOCK_NETWORK, N_("Check for _updates"), NULL, NULL, G_CALLBACK(update_query) },
     { "SFAddons", NULL, N_("Check for _addons"), NULL, NULL, G_CALLBACK(show_files) },
     { "About", GTK_STOCK_ABOUT, N_("_About gretl"), NULL, NULL, G_CALLBACK(about_dialog) }
@@ -1665,16 +1683,16 @@ static gchar *get_main_ui (void)
     return err ? NULL : main_ui;
 }
 
-static int set_up_main_menu (void)
+static GtkWidget *make_main_menu (void)
 {
+    GtkWidget *menu = NULL;
     GtkActionGroup *actions;
     gchar *main_ui = NULL;
     GError *error = NULL;
-    int err = 0;
 
     main_ui = get_main_ui();
     if (main_ui == NULL) {
-	return 1;
+	return NULL;
     }
 
     mdata->ui = gtk_ui_manager_new();
@@ -1692,20 +1710,18 @@ static int set_up_main_menu (void)
     if (!gtk_ui_manager_add_ui_from_string(mdata->ui, main_ui, -1, &error)) {
 	g_message("building menus failed: %s", error->message);
 	g_error_free(error);
-	err = 1;
     } else {
 	add_conditional_items(mdata);
-	mdata->mbar = gtk_ui_manager_get_widget(mdata->ui, "/menubar");
-	if (mdata->mbar == NULL) {
+	menu = gtk_ui_manager_get_widget(mdata->ui, "/menubar");
+	if (menu == NULL) {
 	    fprintf(stderr, "/menubar widget is NULL!\n");
-	    err = 1;
 	} else {
 	    GtkWidget *dataitem;
 
 	    dataitem = gtk_ui_manager_get_widget(mdata->ui, "/menubar/Data");
 	    if (dataitem == NULL) {
 		fprintf(stderr, "/menubar/Data widget is NULL!\n");
-		err = 1;
+		menu = NULL;
 	    } else {
 		g_signal_connect(G_OBJECT(dataitem), "activate",
 				 G_CALLBACK(check_var_labels_state), mdata);
@@ -1714,231 +1730,74 @@ static int set_up_main_menu (void)
 	}
     }
 
-    return err;
+    return menu;
 }
 
-void raise_main_window (void)
+static void name_new_list (GtkWidget *widget, dialog_t *dlg) 
 {
-    gtk_window_present(GTK_WINDOW(mdata->main));
+    char *lname = (char *) edit_dialog_get_data(dlg);
+    const gchar *buf = edit_dialog_get_text(dlg);
+
+    if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_LIST)) {
+	return;
+    }
+
+    strncat(lname, buf, VNAMELEN - 1);
+    edit_dialog_close(dlg);
 }
 
-static void gretl_window_raise (GtkAction *action, gpointer data)
+static void real_make_mainwin_list (const int *list,
+				    const char *lname)
 {
-    const gchar *s = gtk_action_get_name(action);
-    unsigned long lptr = strtoul(s, NULL, 16);
-    GtkWidget *wptr = (GtkWidget *) lptr;
+    int err = remember_list(list, lname, NULL);
+    char *lstr = NULL;
 
-    if (GTK_IS_WINDOW(wptr)) {
-	gtk_window_present(GTK_WINDOW(wptr));
+    if (err) {
+	gui_errmsg(err);
+    } else {
+	lstr = gretl_list_to_string(list);
+    }
+
+    if (lstr != NULL) {
+	/* record to command log */
+	lib_command_sprintf("list %s =%s", lname, lstr);
+	record_command_verbatim();
+	free(lstr);
     }
 }
 
-static const gchar *window_list_icon (int role)
+/* respond to "Define list", selected from main window
+   right-click popup menu when two or more series are
+   selected 
+*/
+
+void make_list_from_main (void)
 {
-    const gchar *id = NULL;
+    int *list = main_window_selection_as_list();
 
-    if (role == VIEW_MODEL) {
-	id = GRETL_STOCK_MODEL;
-    } else if (role == CONSOLE) {
-	id = GRETL_STOCK_CONSOLE;
-    } else if (role >= EDIT_HEADER && 
-	       role < EDIT_MAX) {
-	id = GTK_STOCK_EDIT;
-    } else if (role == GNUPLOT) {
-	id = GRETL_STOCK_SCATTER;
-    } else if (BROWSER_ROLE(role)) {
-	id = GTK_STOCK_INDEX;
-    } else if (HELP_ROLE(role)) {
-	id = GRETL_STOCK_BOOK;
-    } else if (role == STAT_TABLE) {
-	id = GRETL_STOCK_CALC;
-    } else if (role == VIEW_SCRIPT) {
-	id = GTK_STOCK_EXECUTE;
-    } else if (role == OPEN_SESSION) {
-	id = GRETL_STOCK_ICONS;
-    }
+    if (list != NULL) {
+	char lname[VNAMELEN];
+	int cancel = 0;
+	gchar *msg;
 
-    return id;
-}
+	*lname = '\0';
+	msg = g_strdup_printf("%s\n (%s%s%s)",
+			      _("Enter name for list of series"),
+			      dataset->varname[list[1]],
+			      (list[0] == 2)? " " : " ... ",
+			      dataset->varname[list[list[0]]]);
 
-static int n_listed_windows;
-static GtkActionGroup *window_list;
+	blocking_edit_dialog(0, _("gretl: name list"), msg,
+			     NULL, name_new_list, lname, 
+			     VARCLICK_NONE, mdata->main,
+			     &cancel);
+	g_free(msg);
 
-int get_n_listed_windows (void)
-{
-    return n_listed_windows;
-}
-
-static const gchar *get_window_title (GtkWidget *w)
-{
-    const gchar *s = NULL;
-
-    if (GTK_IS_WINDOW(w)) {
-	s = gtk_window_get_title(GTK_WINDOW(w));
-
-	if (s != NULL && !strncmp(s, "gretl", 5)) {
-	    s += 5;
-	    s += strspn(s, " ");
-	    if (*s == ':') {
-		s++;
-		s += strspn(s, " ");
-	    }
-	}
-    }
-
-    return s;
-}
-
-static void destroy_window_action (GtkWidget *w, GtkActionGroup *group)
-{
-    guint merge_id = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "merge_id"));
-
-    if (merge_id > 0) {
-	GtkAction *action;
-	gchar *aname;
-
-	/* remove the ui element */
-	gtk_ui_manager_remove_ui(mdata->ui, merge_id);
-	n_listed_windows--;
-	if (n_listed_windows == 0) {
-	    window_list_state(FALSE);
+	if (!cancel && *lname != '\0') {
+	    real_make_mainwin_list(list, lname);
 	}
 
-	/* and also the underlying action */
-	aname = g_strdup_printf("%p", (void *) w);
-	action = gtk_action_group_get_action(group, aname);
-	if (action != NULL) {
-	    gtk_action_group_remove_action(group, action);
-	}
-	g_free(aname);
-    }    
-}
-
-static gchar *get_parent_path (GtkWidget *w)
-{
-    windata_t *vwin = g_object_get_data(G_OBJECT(w), "vwin");
-    gchar *path = NULL;
-
-    if (vwin != NULL && vwin->gretl_parent != NULL) {
-	/* got a viewer with a 'parent' */
-	path = g_strdup_printf("/menubar/View/Windows/%p", 
-			       vwin->gretl_parent->main);
-    } 
-    
-    return path;
-}
-
-void add_window_list_item (GtkWidget *w, int role)
-{
-    GtkActionEntry entry = { 
-	NULL, NULL, NULL, NULL, NULL, G_CALLBACK(gretl_window_raise) 
-    };
-    const gchar *s = get_window_title(w);
-    gchar *aname;
-    guint merge_id;
-
-    if (window_list == NULL) {
-	/* create the window_list group */
-	window_list = gtk_action_group_new("WindowList");
-	gtk_ui_manager_insert_action_group(mdata->ui, window_list, 0);
-    }
-
-    /* set up an action entry */
-    aname = g_strdup_printf("%p", (void *) w);
-    entry.name = aname;
-    entry.stock_id = window_list_icon(role);
-    entry.label = s;
-
-    /* add new action entry to group, and to ui */
-    gtk_action_group_add_actions(window_list, &entry, 1, mdata);
-    merge_id = gtk_ui_manager_new_merge_id(mdata->ui);
-
-    if (merge_id > 0) {
-	gchar *apath = get_parent_path(w);
-
-	gtk_ui_manager_add_ui(mdata->ui, merge_id, 
-			      (apath != NULL)? apath : "/menubar/View/Windows",
-			      aname, aname,
-			      GTK_UI_MANAGER_MENUITEM, FALSE);
-	g_object_set_data(G_OBJECT(w), "merge_id", GINT_TO_POINTER(merge_id));
-	g_signal_connect(G_OBJECT(w), "destroy", 
-			 G_CALLBACK(destroy_window_action), 
-			 window_list);
-
-	window_list_state(TRUE);
-	n_listed_windows++;
-	g_free(apath);
-    }
-
-    g_free(aname);
-}
-
-static gint sort_window_items (gconstpointer a, gconstpointer b)
-{
-    const gchar *aname = gtk_action_get_name((GtkAction *) a);
-    const gchar *bname = gtk_action_get_name((GtkAction *) b);
-    GtkWidget *wa = (GtkWidget *) strtol(aname, NULL, 16);
-    GtkWidget *wb = (GtkWidget *) strtol(bname, NULL, 16);
-    windata_t *va = g_object_get_data(G_OBJECT(wa), "vwin");
-    windata_t *vb = g_object_get_data(G_OBJECT(wb), "vwin");
-    gint ret = 0;
-
-    if (va != NULL && vb != NULL) {
-	if (va->gretl_parent == vb) {
-	    ret = 1;
-	} else if (vb->gretl_parent == va) {
-	    ret = -1;
-	}
-    }
-    
-    return ret;
-}
-
-void window_list_popup (GtkWidget *src)
-{
-    static GtkWidget *menu;    
-
-    if (menu != NULL) {
-	gtk_widget_destroy(menu);
-	menu = NULL;
-    }    
-
-    if (n_listed_windows > 0) {
-	GList *wlist = gtk_action_group_list_actions(window_list);
-	GList *list = g_list_sort(wlist, sort_window_items);
-	GtkAction *a;
-	GtkWidget *w;
-	int n_items = 0;
-
-	menu = gtk_menu_new();
-
-	while (list) {
-	    if (n_items == 0) {
-		/* at top: item for main window */
-		w = gtk_menu_item_new_with_label(_("Main window"));
-		g_signal_connect(G_OBJECT(w), "activate",
-				 G_CALLBACK(raise_main_window), NULL);
-		gtk_widget_show(w);
-		gtk_menu_shell_append(GTK_MENU_SHELL(menu), w);
-		n_items++;
-	    }
-	    a = (GtkAction *) list->data;
-	    w = gtk_action_create_menu_item(a);
-	    gtk_widget_show(w);
-	    gtk_menu_shell_append(GTK_MENU_SHELL(menu), w);
-	    n_items++;
-	    list = list->next;
-	}
-
-	g_list_free(wlist);
-
-	if (n_items > 0) {
-	    gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL,
-			   0, gtk_get_current_event_time());
-	} else {
-	    gtk_widget_destroy(menu);
-	    menu = NULL;
-	}
+	free(list);
     }
 }
 
@@ -1952,6 +1811,7 @@ int gui_restore_sample (DATASET *dset)
 	    gui_errmsg(err);
 	} else {
 	    restore_sample_state(FALSE);
+	    mark_session_changed();
 	}
     }
 
@@ -1964,8 +1824,8 @@ static void restore_sample_callback (void)
 
     if (!err) {
 	set_sample_label(dataset);    
-	gretl_command_strcpy("smpl --full");
-	check_and_record_command();
+	lib_command_strcpy("smpl --full");
+	record_command_verbatim();
     }
 }
 
@@ -2162,13 +2022,53 @@ static void auto_store (void)
 
 	if ((data_status & USER_DATA) && has_suffix(datafile, ".gdt")) {
 	    /* bypass filename selection */
-	    do_store(datafile, SAVE_DATA);
+	    do_store(datafile, SAVE_DATA, NULL);
 	} else {
 	    file_selector(SAVE_DATA, FSEL_DATA_NONE, NULL);
 	}
     }	
 }
 
+static void mdata_text_received (GtkClipboard *cb,
+				 const gchar *text,
+				 gpointer data)
+{
+    if (text != NULL) {
+	char fullname[FILENAME_MAX];
+	PRN *prn = NULL;
+	int append = 0;
+	int err, resp;
+
+	resp = paste_data_dialog(&append);
+	if (canceled(resp)) {
+	    return;
+	}
+	
+	err = user_fopen(CLIPTEMP, fullname, &prn);
+
+	if (!err) {
+	    int ci = append ? APPEND_CSV : OPEN_CSV;
+
+	    pputs(prn, text);
+	    gretl_print_destroy(prn);
+	    strcpy(tryfile, fullname);
+	    do_open_data(NULL, ci);
+	    gretl_remove(fullname);
+	}
+    }
+}
+
+static void mdata_handle_paste (void)
+{
+    static GtkClipboard *cb;
+
+    if (cb == NULL) {
+	cb = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+    }
+
+    gtk_clipboard_request_text(cb, mdata_text_received, NULL);
+}
+
 int mdata_selection_count (void)
 {
     return vwin_selection_count(mdata, NULL);
