@@ -56,17 +56,6 @@
 
 /* remedial macros for old GTK installations */
 
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 14)
-# define gtk_dialog_get_content_area(d) ((d)->vbox)
-# define gtk_dialog_get_action_area(d) ((d)->action_area)
-# define gtk_widget_get_window(w) ((w)->window)
-# define gtk_selection_data_get_data_type(s) ((s)->type)
-# define gtk_selection_data_get_data(s) ((s)->data)
-# define gtk_adjustment_get_upper(a) ((a)->upper)
-# define gtk_adjustment_set_upper(a,v) ((a)->upper = v)
-# define gtk_color_selection_dialog_get_color_selection(d) ((d)->colorsel)
-#endif
-
 #if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 18)
 # define gtk_widget_is_sensitive(w) GTK_WIDGET_IS_SENSITIVE(w)
 # define gtk_widget_has_focus(w) GTK_WIDGET_HAS_FOCUS(w)
@@ -95,6 +84,12 @@
 
 #define GRETL_BUFSIZE 8192
 #define MAXSTR FILENAME_MAX
+#define CLIPTEMP "cliptmp.txt"
+
+#define SCRIPT_WIDTH 78
+#define SCRIPT_HEIGHT 370
+#define MODEL_WIDTH 72
+#define MODEL_HEIGHT 420 
 
 /* basic global program vars */
 extern DATASET *dataset;
@@ -156,11 +151,7 @@ void populate_varlist (void);
 void clear_varlist (GtkWidget *widget);
 void mdata_select_last_var (void);
 int gui_restore_sample (DATASET *dset);
-
-void add_window_list_item (GtkWidget *w, int role);
-void window_list_popup (GtkWidget *src);
-void raise_main_window (void);
-int get_n_listed_windows (void);
+void make_list_from_main (void);
 void do_stop_script (GtkWidget *w, windata_t *vwin);
 
 /* functions defined in files other than gretl.c */
