@@ -38,8 +38,6 @@ int copyfile (const char *src, const char *dest);
 
 FILE *gretl_tempfile_open (char *fname);
 
-int gretl_tempname (char *fname);
-
 void delete_widget (GtkWidget *widget, gpointer data);
 
 void *mymalloc (size_t size); 
@@ -95,12 +93,13 @@ view_file_with_title (const char *filename, int editable, int del_file,
 		      int hsize, int vsize, int role, 
 		      const char *given_title);
 
-windata_t *view_formatted_text_buffer (const gchar *title, const char *buf, 
-				       int width, int height);
+windata_t *view_formatted_text_buffer (const gchar *title, 
+				       const char *buf, 
+				       int hsize, int vsize);
 
-gchar *gretl_window_title (const char *s1, const char *s2);
+gchar *gretl_window_title (const char *s);
 
-gchar *title_from_filename (const char *fname);
+gchar *title_from_filename (const char *fname, gboolean prepend);
 
 windata_t *console_window (int hsize, int vsize);
 
@@ -111,10 +110,17 @@ windata_t *edit_buffer (char **pbuf, int hsize, int vsize,
 
 windata_t *vwin_first_child (windata_t *vwin);
 
-windata_t *view_model (PRN *prn, MODEL *pmod, int hsize, int vsize, 
-		       char *title);
+windata_t *view_model (PRN *prn, MODEL *pmod, char *title);
 
-void view_window_set_editable (windata_t *vwin);
+windata_t *view_script (const char *filename, int editable, 
+			int role);
+
+gint catch_viewer_key (GtkWidget *w, GdkEventKey *key, 
+		       windata_t *vwin);
+
+void set_model_save_state (windata_t *vwin, gboolean s);
+
+void viewer_set_editable (windata_t *vwin);
 
 int gui_validate_varname (const char *varname, GretlType t);
 
