@@ -22,12 +22,16 @@
 
 #include "gretl.h"
 #include "selector.h"
+#include "winstack.h"
 #include "textutil.h"
 #include "treeutils.h"
 #include "forecast.h"
 #include "texprint.h"
+#include "guiprint.h"
+#include "graph_page.h"
 
 #include "gretl_scalar.h"
+#include "libset.h"
 
 #ifdef G_OS_WIN32
 # include <windows.h>
@@ -177,7 +181,7 @@ void print_window_content (gchar *fullbuf, gchar *selbuf,
 	SetMapMode(dc, MM_TEXT);
 	/* make simple header */
 	if (hdrstart != NULL) {
-	    sprintf(hdr, I_("%s, page %d"), hdrstart, page++);
+	    sprintf(hdr, I_("%s, page %d"), hdrstart, page++); 
 	} else {
 	    sprintf(hdr, "%d", page++);
 	}
@@ -465,7 +469,8 @@ void print_window_content (gchar *fullbuf, gchar *selbuf,
     g_signal_connect(op, "draw-page", G_CALLBACK(draw_text_page), &pinfo);
 
     res = gtk_print_operation_run(op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
-				  GTK_WINDOW(vwin->main), &err);
+				  GTK_WINDOW(vwin_toplevel(vwin)), 
+				  &err);
 
     if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
 	errbox("Error printing:\n%s", err->message);
@@ -584,7 +589,7 @@ draw_image (GtkPrintOperation *op, GtkPrintContext *context,
     cairo_paint(cr);
 }
 
-void gtk_print_graph (const char *fname)
+void gtk_print_graph (const char *fname, GtkWidget *parent)
 {
     char pngname[FILENAME_MAX];
     GtkPrintOperation *op;
@@ -609,7 +614,7 @@ void gtk_print_graph (const char *fname)
     g_signal_connect(op, "draw-page", G_CALLBACK(draw_image), NULL);
 
     res = gtk_print_operation_run(op, GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
-				  GTK_WINDOW(mdata->main), &err);
+				  GTK_WINDOW(parent), &err);
 
     if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
 	errbox("Error printing:\n%s", err->message);
@@ -647,16 +652,16 @@ void rtf_print_obs_marker (int t, const DATASET *pdinfo, PRN *prn)
 #define STATS_ROW  "\\trowd \\trqc \\trgaph60\\trleft-30\\trrh262" \
                    "\\cellx2700\\cellx4000\\cellx6700\\cellx8000\n\\intbl"
 
-static void printfrtf (double zz, PRN *prn, int endrow)
+static void printf_rtf (double zz, PRN *prn, int endrow)
 {
     /* was using "qr", for right alignment */
 
     if (na(zz)) {
 	if (endrow) {
 	    pprintf(prn, "\\qc %s\\cell\\intbl \\row\n",
-		    I_("undefined"));
+		    A_("undefined"));
 	} else {
-	    pprintf(prn, "\\qc %s\\cell", I_("undefined"));
+	    pprintf(prn, "\\qc %s\\cell", A_("undefined"));
 	}
 	return;
     }
@@ -684,23 +689,23 @@ rtfprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
     ntodate(date1, pdinfo->t1, pdinfo);
     ntodate(date2, pdinfo->t2, pdinfo);
 
-    sprintf(tmp, I_("Summary Statistics, using the observations %s - %s"),
+    sprintf(tmp, A_("Summary Statistics, using the observations %s - %s"),
 	    date1, date2);
 
     pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n", tmp);
     
     if (summ->list[0] == 1) {
-	sprintf(tmp, I_("for the variable %s (%d valid observations)"), 
+	sprintf(tmp, A_("for the variable %s (%d valid observations)"), 
 		pdinfo->varname[summ->list[1]], summ->n);
 	pprintf(prn, "%s\\par\n\n", tmp);
 	pputs(prn, "{" VAR_SUMM_ROW "\\intbl ");
     } else {
 	if (summ->missing) {
-	    strcpy(tmp, I_("(missing values were skipped)"));
+	    strcpy(tmp, A_("(missing values were skipped)"));
 	    pprintf(prn, "%s\\par\n\n", tmp); /* FIXME */
 	}
 	pprintf(prn, "{" SUMM_ROW
-		"\\intbl \\qc %s\\cell", I_("Variable"));
+		"\\intbl \\qc %s\\cell", A_("Variable"));
     }
 
     pprintf(prn, 
@@ -709,21 +714,21 @@ rtfprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
 	    " \\qc %s\\cell"
 	    " \\qc %s\\cell"
 	    " \\intbl \\row\n",
-	    I_("Mean"), I_("Median"), I_("Minimum"), I_("Maximum"));
+	    A_("Mean"), A_("Median"), A_("Minimum"), A_("Maximum"));
 
     for (i=0; i<summ->list[0]; i++) {
 	vi = summ->list[i + 1];
 	if (summ->list[0] > 1) {
 	    pprintf(prn, "\\intbl \\qc %s\\cell ", pdinfo->varname[vi]);
 	}
-	printfrtf(summ->mean[i], prn, 0);
-	printfrtf(summ->median[i], prn, 0);
-	printfrtf(summ->low[i], prn, 0);
-	printfrtf(summ->high[i], prn, 1);
+	printf_rtf(summ->mean[i], prn, 0);
+	printf_rtf(summ->median[i], prn, 0);
+	printf_rtf(summ->low[i], prn, 0);
+	printf_rtf(summ->high[i], prn, 1);
     }
 
     if (summ->list[0] > 1) pprintf(prn, "\\intbl \\qc %s\\cell",
-				   I_("Variable"));
+				   A_("Variable"));
 
     pprintf(prn, 
 	    " \\qc %s\\cell"
@@ -731,17 +736,17 @@ rtfprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
 	    " \\qc %s\\cell"
 	    " \\qc %s\\cell"
 	    " \\intbl \\row\n",
-	    I_("Std. Dev."), I_("C.V."), I_("Skewness"), I_("Ex. kurtosis"));
+	    A_("Std. Dev."), A_("C.V."), A_("Skewness"), A_("Ex. kurtosis"));
 
     for (i=0; i<summ->list[0]; i++) {
 	vi = summ->list[i + 1];
 	if (summ->list[0] > 1) {
 	    pprintf(prn, "\\intbl \\qc %s\\cell ", pdinfo->varname[vi]);
 	}
-	printfrtf(summ->sd[i], prn, 0);
-	printfrtf(summ->cv[i], prn, 0);
-	printfrtf(summ->skew[i], prn, 0);
-	printfrtf(summ->xkurt[i], prn, 1);
+	printf_rtf(summ->sd[i], prn, 0);
+	printf_rtf(summ->cv[i], prn, 0);
+	printf_rtf(summ->skew[i], prn, 0);
+	printf_rtf(summ->xkurt[i], prn, 1);
     }
 
     pputs(prn, "}}\n");
@@ -838,10 +843,10 @@ int text_print_equation (const MODEL *pmod, const DATASET *pdinfo,
 	}
     } else {
 	if (!na(pmod->adjrsq)) {
-	    pprintf(prn, ", %s = %.3f ", I_("R-squared"), pmod->rsq);
+	    pprintf(prn, ", %s = %.3f ", _("R-squared"), pmod->rsq);
 	} else if (!na(pmod->lnL)) {
 	    eqn_numstr(pmod->lnL, xstr);
-	    pprintf(prn, ", %s = %s ", I_("loglikelihood"), xstr);
+	    pprintf(prn, ", %s = %s ", _("loglikelihood"), xstr);
 	}
 	x = gretl_model_get_double(pmod, "rho_in");
 	if (!na(x)) {
@@ -850,12 +855,85 @@ int text_print_equation (const MODEL *pmod, const DATASET *pdinfo,
     }
 
     pprintf(prn, "\n(%s)\n",
-	    (opt & OPT_T)? I_("t-ratios in parentheses") :
-	    I_("standard errors in parentheses"));
+	    (opt & OPT_T)? _("t-ratios in parentheses") :
+	    _("standard errors in parentheses"));
 
     return 0;
 }
 
+int text_print_x_y_fitted (int vx, int vy, const double *f, 
+			   const DATASET *dset, PRN *prn)
+{
+    char obs1[OBSLEN], obs2[OBSLEN];
+    char label[VNAMELEN];
+    const double *x = dset->Z[vx];
+    const double *y = dset->Z[vy];
+    int obslen = max_obs_marker_length(dset);
+    int t1 = dset->t1;
+    int t2 = dset->t2;
+    int t, pmax, err = 0;
+
+    for (t=t1; t<=dset->t2; t++) {
+	if (na(x[t])) {
+	    t1++;
+	} else {
+	    break;
+	}
+    }
+
+    for (t=t2; t>dset->t1; t--) {
+	if (na(x[t])) {
+	    t2--;
+	} else {
+	    break;
+	}
+    }
+
+    ntodate(obs1, t1, dset);
+    ntodate(obs2, t2, dset);
+    pprintf(prn, _("Model estimation range: %s - %s"), obs1, obs2);
+    pputc(prn, '\n');
+    bufspace(obslen, prn);
+
+    for (t=0; t<3; t++) {
+	if (t == 0) strcpy(label, dset->varname[vx]);
+	if (t == 1) strcpy(label, dset->varname[vy]);
+	if (t == 2) strcpy(label, _("fitted"));
+	pprintf(prn, "%*s", UTF_WIDTH(label, 13), label); 
+    }
+
+    pputs(prn, "\n\n");
+
+    pmax = get_precision(y, dset->n, 6);
+
+    for (t=t1; t<=t2; t++) {
+	print_obs_marker(t, dset, obslen, prn);
+	if (na(x[t])) {
+	    /* nothing to print */
+	    pputc(prn, '\n');
+	} else if (na(y[t])) {
+	    /* y missing but x and fitted should be OK */
+	    if (pmax != PMAX_NOT_AVAILABLE) {
+		pprintf(prn, "%13.*f%13s%13.*f\n", pmax, x[t], "NA", pmax, f[t]);
+	    } else {
+		pprintf(prn, "%13g%13s%13g\n", x[t], "NA", f[t]);
+	    }
+	} else {
+	    /* got all values */
+	    if (pmax != PMAX_NOT_AVAILABLE) {
+		pprintf(prn, "%13.*f%13.*f%13.*f\n", 
+			pmax, x[t], pmax, y[t], pmax, f[t]);
+	    } else {
+		pprintf(prn, "%13g%13g%13g\n", x[t], y[t], f[t]);
+	    }
+	}
+    }
+
+    pputc(prn, '\n');
+
+    return err;
+}
+
 /* print value in (non-correlation) matrix */
 
 static void tex_matnum (double x, PRN *prn)
@@ -866,13 +944,13 @@ static void tex_matnum (double x, PRN *prn)
     pprintf(prn, "%s & ", s);
 }
 
-static void printftex (double x, PRN *prn, int endrow)
+static void printf_tex (double x, PRN *prn, int endrow)
 {
     if (na(x)) {
 	if (endrow) {
-	    pprintf(prn, "\\multicolumn{2}{c}{%s}\\\\", I_("undefined"));
+	    pprintf(prn, "\\multicolumn{2}{c}{%s}\\\\", A_("undefined"));
 	} else {
-	    pprintf(prn, "\\multicolumn{2}{c}{%s} & ", I_("undefined"));
+	    pprintf(prn, "\\multicolumn{2}{c}{%s} & ", A_("undefined"));
 	}
     } else {
 	char s[32];
@@ -897,34 +975,34 @@ texprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
     ntodate(date1, pdinfo->t1, pdinfo);
     ntodate(date2, pdinfo->t2, pdinfo);
 
-    sprintf(tmp, I_("Summary Statistics, using the observations %s--%s"),
+    sprintf(tmp, A_("Summary Statistics, using the observations %s--%s"),
 	    date1, date2);
 
     pprintf(prn, "\\begin{center}\n%s\\\\\n", tmp);
     
     if (summ->list[0] == 1) {
 	tex_escape(vname, pdinfo->varname[summ->list[1]]);
-	sprintf(tmp, I_("for the variable %s (%d valid observations)"), 
+	sprintf(tmp, A_("for the variable %s (%d valid observations)"), 
 		vname, summ->n);
 	pprintf(prn, "%s\\\\[8pt]\n\n", tmp);
 	pprintf(prn, "\\begin{tabular}{r@{%c}lr@{%c}lr@{%c}lr@{%c}l}\n",
 		pt, pt, pt, pt);
     } else {
 	if (summ->missing) {
-	    pprintf(prn, "%s\\\\[8pt]\n\n", I_("(missing values were skipped)"));
+	    pprintf(prn, "%s\\\\[8pt]\n\n", A_("(missing values were skipped)"));
 	} else {
 	    pputs(prn, "\n\\vspace{8pt}\n\n");
 	}
 	pprintf(prn, "\\begin{tabular}{lr@{%c}lr@{%c}lr@{%c}lr@{%c}l}\n",
 		pt, pt, pt, pt);
-	pprintf(prn, "%s &", I_("Variable"));
+	pprintf(prn, "%s &", A_("Variable"));
     }
 
     pprintf(prn, " \\multicolumn{2}{c}{%s}%%\n"
 	    " & \\multicolumn{2}{c}{%s}%%\n"
 	    "  & \\multicolumn{2}{c}{%s}%%\n"
 	    "   & \\multicolumn{2}{c}{%s} \\\\[1ex]\n",
-	    I_("Mean"), I_("Median"), I_("Minimum"), I_("Maximum"));
+	    A_("Mean"), A_("Median"), A_("Minimum"), A_("Maximum"));
 
     for (i=0; i<summ->list[0]; i++) {
 	vi = summ->list[i + 1];
@@ -932,10 +1010,10 @@ texprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
 	    tex_escape(vname, pdinfo->varname[vi]);
 	    pprintf(prn, "%s & ", vname);
 	}
-	printftex(summ->mean[i], prn, 0);
-	printftex(summ->median[i], prn, 0);
-	printftex(summ->low[i], prn, 0);
-	printftex(summ->high[i], prn, 1);
+	printf_tex(summ->mean[i], prn, 0);
+	printf_tex(summ->median[i], prn, 0);
+	printf_tex(summ->low[i], prn, 0);
+	printf_tex(summ->high[i], prn, 1);
 	if (i == summ->list[0] - 1) {
 	    pputs(prn, "[10pt]\n\n");
 	} else {
@@ -944,14 +1022,14 @@ texprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
     }
 
     if (summ->list[0] > 1) {
-	pprintf(prn, "%s & ", I_("Variable"));
+	pprintf(prn, "%s & ", A_("Variable"));
     }
 
     pprintf(prn, " \\multicolumn{2}{c}{%s}%%\n"
 	    " & \\multicolumn{2}{c}{%s}%%\n"
 	    "  & \\multicolumn{2}{c}{%s}%%\n"
 	    "   & \\multicolumn{2}{c}{%s} \\\\[1ex]\n",
-	    I_("Std.\\ Dev."), I_("C.V."), I_("Skewness"), I_("Ex.\\ kurtosis"));
+	    A_("Std.\\ Dev."), A_("C.V."), A_("Skewness"), A_("Ex.\\ kurtosis"));
 
     for (i=0; i<summ->list[0]; i++) {
 	vi = summ->list[i + 1];
@@ -959,10 +1037,10 @@ texprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
 	    tex_escape(vname, pdinfo->varname[vi]);
 	    pprintf(prn, "%s & ", vname);
 	}
-	printftex(summ->sd[i], prn, 0);
-	printftex(summ->cv[i], prn, 0);
-	printftex(summ->skew[i], prn, 0);
-	printftex(summ->xkurt[i], prn, 1);
+	printf_tex(summ->sd[i], prn, 0);
+	printf_tex(summ->cv[i], prn, 0);
+	printf_tex(summ->skew[i], prn, 0);
+	printf_tex(summ->xkurt[i], prn, 1);
 	pputc(prn, '\n');
     }
 
@@ -972,6 +1050,8 @@ texprint_summary (const Summary *summ, const DATASET *pdinfo, PRN *prn)
 void special_print_summary (const Summary *summ, const DATASET *pdinfo,
 			    PRN *prn)
 {
+    set_alt_gettext_mode(prn);
+
     if (tex_format(prn)) {
 	texprint_summary(summ, pdinfo, prn);
     } else if (rtf_format(prn)) {
@@ -982,7 +1062,7 @@ void special_print_summary (const Summary *summ, const DATASET *pdinfo,
 static void tex_outxx (double xx, PRN *prn)
 {
     if (na(xx)) {
-	pprintf(prn, "%s & ", I_("undefined"));
+	pprintf(prn, "%s & ", A_("undefined"));
     } else {
 	pprintf(prn, "$%.4f$ & ", xx);
     }
@@ -991,7 +1071,7 @@ static void tex_outxx (double xx, PRN *prn)
 static void rtf_outxx (double xx, PRN *prn)
 {
     if (na(xx)) {
-	pprintf(prn, "\\qc %s\\cell ", I_("undefined"));
+	pprintf(prn, "\\qc %s\\cell ", A_("undefined"));
     } else {
 	pprintf(prn, "\\qc %.4f\\cell ", xx);	
     }
@@ -1040,18 +1120,18 @@ rtfprint_vmatrix (const VMatrix *vmat, const DATASET *pdinfo, PRN *prn)
 	ntodate(date1, vmat->t1, pdinfo);
 	ntodate(date2, vmat->t2, pdinfo);
 
-	sprintf(tmp, I_("Correlation coefficients, using the observations "
+	sprintf(tmp, A_("Correlation coefficients, using the observations "
 			"%s - %s"), date1, date2);
 	pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n", tmp);
 	if (vmat->missing) {
-	    pprintf(prn, "%s\\par\n", I_("(missing values were skipped)"));
+	    pprintf(prn, "%s\\par\n", A_("(missing values were skipped)"));
 	}
-	sprintf(tmp, I_("5%% critical value (two-tailed) = %.4f for n = %d"), 
+	sprintf(tmp, A_("5%% critical value (two-tailed) = %.4f for n = %d"), 
 		rhocrit95(n), n);
 	pprintf(prn, "%s\\par\n\\par\n{", tmp);
     } else {
 	pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n\\par\n{",
-		I_("Coefficient covariance matrix"));
+		A_("Coefficient covariance matrix"));
     }
     
     for (i=0; i<=blockmax; i++) {
@@ -1085,7 +1165,7 @@ rtfprint_vmatrix (const VMatrix *vmat, const DATASET *pdinfo, PRN *prn)
 		if (vmat->ci == CORR) {
 		    rtf_outxx(vmat->vec[idx], prn);
 		} else {
-		    printfrtf(vmat->vec[idx], prn, 0);
+		    printf_rtf(vmat->vec[idx], prn, 0);
 		}
 	    }
 	    pprintf(prn, "\\ql %s\\cell \\intbl \\row\n", vmat->names[j]);
@@ -1101,7 +1181,7 @@ rtfprint_vmatrix (const VMatrix *vmat, const DATASET *pdinfo, PRN *prn)
 		if (vmat->ci == CORR) {
 		    rtf_outxx(vmat->vec[idx], prn);
 		} else {
-		    printfrtf(vmat->vec[idx], prn, 0);
+		    printf_rtf(vmat->vec[idx], prn, 0);
 		}
 	    }
 	    pprintf(prn, "\\ql %s\\cell \\intbl \\row\n", vmat->names[ij2]);
@@ -1133,19 +1213,19 @@ texprint_vmatrix (const VMatrix *vmat, const DATASET *pdinfo, PRN *prn)
 	ntodate(date2, vmat->t2, pdinfo);
 
 	pputs(prn, "\\begin{center}\n");
-	pprintf(prn, I_("Correlation coefficients, using the observations "
+	pprintf(prn, A_("Correlation coefficients, using the observations "
 			"%s--%s"), date1, date2);
 	pputs(prn, "\\\\\n");
 	if (vmat->missing) {
-	    pputs(prn, I_("(missing values were skipped)"));
+	    pputs(prn, A_("(missing values were skipped)"));
 	    pputs(prn, "\\\\\n");
 	}
-	pprintf(prn, I_("5\\%% critical value (two-tailed) = %.4f for n = %d"), 
+	pprintf(prn, A_("5\\%% critical value (two-tailed) = %.4f for n = %d"), 
 		rhocrit95(n), n);
 	pputs(prn, "\\\\\n");
     } else {
 	pprintf(prn, "\\begin{center}\n%s\\\\\n", 
-		I_("Coefficient covariance matrix"));
+		A_("Coefficient covariance matrix"));
     }
 
     pputs(prn, "\\vspace{8pt}\n");
@@ -1216,6 +1296,8 @@ texprint_vmatrix (const VMatrix *vmat, const DATASET *pdinfo, PRN *prn)
 void special_print_vmatrix (const VMatrix *vmat, const DATASET *pdinfo, 
 			    PRN *prn)
 {
+    set_alt_gettext_mode(prn);
+
     if (tex_format(prn)) {
 	texprint_vmatrix(vmat, pdinfo, prn);
     } else if (rtf_format(prn)) {
@@ -1257,7 +1339,7 @@ static int texprint_fcast_stats (const FITRESID *fr,
 
     len = gretl_vector_get_length(m);
 
-    pputs(prn, _("Forecast evaluation statistics"));
+    pputs(prn, A_("Forecast evaluation statistics"));
     pputs(prn, "\\\\[1ex]\n\n");
 
     pputs(prn, "\\begin{tabular}{ll}\n");
@@ -1266,10 +1348,10 @@ static int texprint_fcast_stats (const FITRESID *fr,
     for (i=0; i<len; i++) {
 	x = gretl_vector_get(m, i);
 	if (!isnan(x)) {
-	    pprintf(prn, "%s & %s%.5g \\\\\n", I_(strs[j]), (x < 0)? "$-$" : "", 
+	    pprintf(prn, "%s & %s%.5g \\\\\n", A_(strs[j]), (x < 0)? "$-$" : "", 
 		    fabs(x));
 	    if (i == 1) {
-		pprintf(prn, "%s & %.5g \\\\\n", I_(strs[j+1]), sqrt(x));	
+		pprintf(prn, "%s & %.5g \\\\\n", A_(strs[j+1]), sqrt(x));	
 	    }
 	}
 	j += (i == 1)? 2 : 1;
@@ -1292,10 +1374,10 @@ void tex_fit_resid_head (const FITRESID *fr, const DATASET *pdinfo,
     ntodate(date2, fr->t2, pdinfo);
 
     pputs(prn, "\\begin{raggedright}\n");
-    pputs(prn, I_("Model estimation range:"));
+    pputs(prn, A_("Model estimation range:"));
     pprintf(prn, " %s--%s \\\\ \n", date1, date2);
 
-    pprintf(prn, I_("Standard error of residuals = %g"), fr->sigma);
+    pprintf(prn, A_("Standard error of residuals = %g"), fr->sigma);
     pputs(prn, "\n\\end{raggedright}\n");
 }
 
@@ -1310,10 +1392,10 @@ void rtf_fit_resid_head (const FITRESID *fr, const DATASET *pdinfo,
     ntodate(date2, fr->t2, pdinfo);
 
     pputs(prn, "{\\rtf1\\par\n\\qc ");
-    pputs(prn, I_("Model estimation range:")); 
+    pputs(prn, A_("Model estimation range:")); 
     pprintf(prn, " %s - %s\\par\n", date1, date2);
 
-    sprintf(tmp, I_("Standard error of residuals = %g"), 
+    sprintf(tmp, A_("Standard error of residuals = %g"), 
 	    fr->sigma);
     pprintf(prn, "\\qc %s\\par\n\\par\n", tmp);
 }
@@ -1353,7 +1435,7 @@ static void texprint_fit_resid (const FITRESID *fr,
 	    " \\multicolumn{1}{c}{%s} & \n"
 	    "  \\multicolumn{1}{c}{%s} & \n"
 	    "   \\multicolumn{1}{c}{%s}\\\\\n",
-	    vname, I_("fitted"), I_("residual"));
+	    vname, A_("fitted"), A_("residual"));
 
     for (t=fr->t1; t<=fr->t2; t++) {
 	tex_print_obs_marker(t, pdinfo, prn);
@@ -1382,7 +1464,7 @@ static void texprint_fit_resid (const FITRESID *fr,
     pputs(prn, "\\end{longtable}\n\n");
 
     if (anyast) {
-	pputs(prn, I_("\\textit{Note}: * denotes a residual "
+	pputs(prn, A_("\\textit{Note}: * denotes a residual "
 		      "in excess of 2.5 standard errors\n\n"));
     }
 
@@ -1413,7 +1495,7 @@ static void rtfprint_fit_resid (const FITRESID *fr,
 	    " \\qc %s\\cell"
 	    " \\ql \\cell"
 	    " \\intbl \\row\n",
-	    fr->depvar, I_("fitted"), I_("residual"));
+	    fr->depvar, A_("fitted"), A_("residual"));
 
     for (t=fr->t1; t<=fr->t2; t++) {
 	rtf_print_obs_marker(t, pdinfo, prn);
@@ -1421,7 +1503,7 @@ static void rtfprint_fit_resid (const FITRESID *fr,
 	    pputs(prn, "\\qc \\cell \\qc \\cell \\qc \\cell \\ql \\cell"
 		  " \\intbl \\row\n"); 
 	} else if (na(fr->fitted[t])) {	 
-	    printfrtf(fr->actual[t], prn, 0);
+	    printf_rtf(fr->actual[t], prn, 0);
 	    pputs(prn, "\\qc \\cell \\qc \\cell \\ql \\cell"
 		  " \\intbl \\row\n"); 
 	} else {
@@ -1432,9 +1514,9 @@ static void rtfprint_fit_resid (const FITRESID *fr,
 	    if (ast) {
 		anyast = 1;
 	    }
-	    printfrtf(fr->actual[t], prn, 0);
-	    printfrtf(fr->fitted[t], prn, 0);
-	    printfrtf(xx, prn, 0);
+	    printf_rtf(fr->actual[t], prn, 0);
+	    printf_rtf(fr->fitted[t], prn, 0);
+	    printf_rtf(xx, prn, 0);
 	    pprintf(prn, "\\ql %s\\cell \\intbl \\row\n", 
 		    (ast)? "*" : "");
 	}
@@ -1443,7 +1525,7 @@ static void rtfprint_fit_resid (const FITRESID *fr,
     pputs(prn, "}\n");
     if (anyast) {
 	pprintf(prn, "\\par\n\\qc %s \\par\n",
-		I_("Note: * denotes a residual in excess of 2.5 standard errors"));
+		A_("Note: * denotes a residual in excess of 2.5 standard errors"));
     }
     pputs(prn, "}\n");
 }
@@ -1452,6 +1534,8 @@ void special_print_fit_resid (const FITRESID *fr,
 			      const DATASET *pdinfo, 
 			      PRN *prn)
 {
+    set_alt_gettext_mode(prn);
+
     if (tex_format(prn)) {
 	texprint_fit_resid(fr, pdinfo, prn);
     } else if (rtf_format(prn)) {
@@ -1489,7 +1573,7 @@ static void texprint_fcast_without_errs (const FITRESID *fr,
     tex_escape(vname, fr->depvar);
 
     pprintf(prn, "%s & %s & \\multicolumn{1}{c}{%s} \\\\ [4pt] \n",
-	    I_("Obs"), vname, I_("prediction"));
+	    A_("Obs"), vname, A_("prediction"));
 
     for (t=fr->t1; t<=fr->t2; t++) {
 	texprint_fcast_x(fr->actual[t], fr->pmax, actual);
@@ -1521,11 +1605,11 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
 
     if (fr->asymp) {
 	tval = normal_critval(fr->alpha / 2);
-	pprintf(prn, I_("For %g\\%% confidence intervals, $z(%g) = %.2f$\n\n"), 
+	pprintf(prn, A_("For %g\\%% confidence intervals, $z(%g) = %.2f$\n\n"), 
 		conf, fr->alpha / 2, tval);
     } else {
 	tval = student_critval(fr->df, fr->alpha / 2);
-	pprintf(prn, I_("For %g\\%% confidence intervals, $t(%d, %g) = %.3f$\n\n"), 
+	pprintf(prn, A_("For %g\\%% confidence intervals, $t(%d, %g) = %.3f$\n\n"), 
 		conf, fr->df, fr->alpha / 2, tval);
     }
 
@@ -1545,14 +1629,14 @@ static void texprint_fcast_with_errs (const FITRESID *fr,
 	    pt, pt, pt, pt, pt);
 
     tex_escape(vname, fr->depvar);
-    sprintf(hi, I_("%g\\%% interval"), conf);
+    sprintf(hi, A_("%g\\%% interval"), conf);
 
     pprintf(prn, "%s & \\multicolumn{2}{c}{%s} "
 	    " & \\multicolumn{2}{c}{%s}\n"
 	    "  & \\multicolumn{2}{c}{%s}\n"
 	    "   & \\multicolumn{4}{c}{%s} \\\\[1ex]\n",
-	    I_("Obs"), vname,
-	    I_("prediction"), I_("std. error"),
+	    A_("Obs"), vname,
+	    A_("prediction"), A_("std. error"),
 	    hi);
 
     if (pmax < 4) {
@@ -1607,12 +1691,12 @@ static void rtfprint_fcast_without_errs (const FITRESID *fr,
 	    " \\qc %s\\cell"
 	    " \\qc %s\\cell"
 	    " \\intbl \\row\n", 
-	    I_("Obs"), fr->depvar, I_("prediction")); 
+	    A_("Obs"), fr->depvar, A_("prediction")); 
 
     for (t=fr->t1; t<=fr->t2; t++) {
 	rtf_print_obs_marker(t, pdinfo, prn);
-	printfrtf(fr->actual[t], prn, 0);
-	printfrtf(fr->fitted[t], prn, 0);
+	printf_rtf(fr->actual[t], prn, 0);
+	printf_rtf(fr->fitted[t], prn, 0);
     }
 
     pputs(prn, "}}\n");
@@ -1629,16 +1713,16 @@ static void rtfprint_fcast_with_errs (const FITRESID *fr,
 
     if (fr->asymp) {
 	tval = normal_critval(fr->alpha / 2);
-	sprintf(tmp, I_("For %g%% confidence intervals, z(%g) = %.2f"), 
+	sprintf(tmp, A_("For %g%% confidence intervals, z(%g) = %.2f"), 
 		conf, fr->alpha / 2, tval);
     } else {
 	tval = student_critval(fr->df, fr->alpha / 2);
-	sprintf(tmp, I_("For %g%% confidence intervals, t(%d, %g) = %.3f"), 
+	sprintf(tmp, A_("For %g%% confidence intervals, t(%d, %g) = %.3f"), 
 		conf, fr->df, fr->alpha / 2, tval);
     }
     pprintf(prn, "{\\rtf1\\par\n\\qc %s\\par\n\\par\n", tmp);
 
-    sprintf(tmp, I_("%g%% interval"), conf);
+    sprintf(tmp, A_("%g%% interval"), conf);
 
     pputs(prn, "{" FCE_ROW "\\intbl ");
     pprintf(prn, 
@@ -1648,16 +1732,16 @@ static void rtfprint_fcast_with_errs (const FITRESID *fr,
 	    " \\qc %s\\cell"
 	    " \\qc %s\\cell"
 	    " \\intbl \\row\n", 
-	    I_("Obs"), fr->depvar, I_("prediction"), 
-	    I_("std. error"),
+	    A_("Obs"), fr->depvar, A_("prediction"), 
+	    A_("std. error"),
 	    tmp);
 
     for (t=fr->t1; t<=fr->t2; t++) {
 	rtf_print_obs_marker(t, pdinfo, prn);
 	maxerr = tval * fr->sderr[t];
-	printfrtf(fr->actual[t], prn, 0);
-	printfrtf(fr->fitted[t], prn, 0);
-	printfrtf(fr->sderr[t], prn, 0);
+	printf_rtf(fr->actual[t], prn, 0);
+	printf_rtf(fr->fitted[t], prn, 0);
+	printf_rtf(fr->sderr[t], prn, 0);
 	if (na(fr->sderr[t])) {
 	    pputs(prn, "\\qc \\cell \\intbl \\row\n");
 	} else {
@@ -1675,6 +1759,8 @@ void special_print_forecast (const FITRESID *fr,
 			     const DATASET *pdinfo, 
 			     PRN *prn)
 {
+    set_alt_gettext_mode(prn);
+
     if (tex_format(prn)) {
 	if (fr->sderr != NULL) {
 	    texprint_fcast_with_errs(fr, pdinfo, prn);
@@ -1699,7 +1785,7 @@ texprint_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
     pprintf(prn, " %s & ", vname);
 
     if (isnan(cf->coeff[i])) {
-	pprintf(prn, "\\multicolumn{2}{c}{%s} & ", I_("undefined"));
+	pprintf(prn, "\\multicolumn{2}{c}{%s} & ", A_("undefined"));
     } else {
 	char coeff[32];
 
@@ -1708,7 +1794,7 @@ texprint_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
     }
 
     if (isnan(cf->maxerr[i])) {
-	pprintf(prn, "\\multicolumn{4}{c}{%s}", I_("undefined"));
+	pprintf(prn, "\\multicolumn{4}{c}{%s}", A_("undefined"));
     } else {
 	char lo[32], hi[32];
 
@@ -1733,12 +1819,12 @@ static void texprint_confints (const CoeffIntervals *cf, PRN *prn)
 	    "\\begin{tabular}{rr@{%c}lr@{%c}lr@{%c}l}\n",
 	    pt, pt, pt);
 
-    cstr = g_strdup_printf(I_("%g\\%% confidence interval"), 100 * (1 - cf->alpha));
+    cstr = g_strdup_printf(A_("%g\\%% confidence interval"), 100 * (1 - cf->alpha));
 
     pprintf(prn, " %s%%\n"
 	    " & \\multicolumn{2}{c}{%s}%%\n"
 	    "  & \\multicolumn{4}{c}{%s}\\\\[1ex]\n",
-	    I_("Variable"), I_("Coefficient"),
+	    A_("Variable"), A_("Coefficient"),
 	    cstr);
 
     g_free(cstr);
@@ -1756,10 +1842,10 @@ rtfprint_coeff_interval (const CoeffIntervals *cf, int i, PRN *prn)
 {
     pprintf(prn, "\\qc %s\\cell", cf->names[i]);
 
-    printfrtf(cf->coeff[i], prn, 0);
+    printf_rtf(cf->coeff[i], prn, 0);
 
     if (isnan(cf->maxerr[i])) {
-	pprintf(prn, "\\qc %s\\cell ", I_("undefined"));
+	pprintf(prn, "\\qc %s\\cell ", A_("undefined"));
     } else {
 	pprintf(prn, "\\qc (%#.*g, %#.*g)\\cell ", 
 		GRETL_DIGITS, cf->coeff[i] - cf->maxerr[i], 
@@ -1780,7 +1866,7 @@ static void rtfprint_confints (const CoeffIntervals *cf, PRN *prn)
     pprintf(prn, "{\\rtf1\\par\n\\qc t(%d, %g) = %.3f\\par\n\\par\n", 
 	    cf->df, tail, cf->t);
 
-    cstr = g_strdup_printf(I_("%g\\%% confidence interval"), 100 * (1 - cf->alpha));
+    cstr = g_strdup_printf(A_("%g\\%% confidence interval"), 100 * (1 - cf->alpha));
 
     pputs(prn, "{" CF_ROW "\\intbl ");
     pprintf(prn, 
@@ -1788,7 +1874,7 @@ static void rtfprint_confints (const CoeffIntervals *cf, PRN *prn)
 	    " \\qc %s\\cell"
 	    " \\qc %s\\cell"
 	    " \\intbl \\row\n", 
-	    I_("Variable"), I_("Coefficient"), 
+	    A_("Variable"), A_("Coefficient"), 
 	    cstr);
 
     g_free(cstr);
@@ -1802,6 +1888,8 @@ static void rtfprint_confints (const CoeffIntervals *cf, PRN *prn)
 
 void special_print_confints (const CoeffIntervals *cf, PRN *prn)
 {
+    set_alt_gettext_mode(prn);
+
     if (tex_format(prn)) {
 	texprint_confints(cf, prn);
     } else if (rtf_format(prn)) {
@@ -1811,32 +1899,33 @@ void special_print_confints (const CoeffIntervals *cf, PRN *prn)
 
 int scalars_to_prn (PRN *prn)
 {
+    char decpoint = get_data_export_decpoint();
+    char delim = get_data_export_delimiter();
     const char *sname;
     double sval;
     int i, n = n_saved_scalars();
 
-    if (dataset->delim == ',' && ',' == dataset->decpoint) {
+    if (delim == ',' && ',' == decpoint) {
 	errbox(_("You can't use the same character for "
 		 "the column delimiter and the decimal point"));
 	return 1;
     }
 
-    if (dataset->decpoint != ',') {
+    if (decpoint != ',') {
 	gretl_push_c_numeric_locale();
     }
 
-
     for (i=0; i<n; i++) {
 	sname = gretl_scalar_get_name(i);
 	sval = gretl_scalar_get_value_by_index(i);
 	if (na(sval)) {
-	    pprintf(prn, "%s%cNA\n", sname, dataset->delim);
+	    pprintf(prn, "%s%cNA\n", sname, delim);
 	} else {
-	    pprintf(prn, "%s%c%.15g\n", sname, dataset->delim, sval);
+	    pprintf(prn, "%s%c%.15g\n", sname, delim, sval);
 	}
     }
 
-    if (dataset->decpoint != ',') {
+    if (decpoint != ',') {
 	gretl_pop_c_numeric_locale();
     }
 
@@ -1848,29 +1937,35 @@ static int data_to_buf_as_rtf (const int *list, PRN *prn)
     int err;
 
     gretl_print_set_format(prn, GRETL_FORMAT_RTF);
-    err = print_data_in_columns(list, NULL, dataset, prn);
+    err = print_data_in_columns(list, NULL, dataset, OPT_NONE, prn);
     return err;
 }
 
-static int data_to_buf_as_csv (const int *list, PRN *prn)
+static int data_to_buf_as_csv (const int *list, gretlopt opt,
+			       PRN *prn)
 {
     int err;
 
     gretl_print_set_format(prn, GRETL_FORMAT_CSV);
-    err = print_data_in_columns(list, NULL, dataset, prn);
+    err = print_data_in_columns(list, NULL, dataset, opt, prn);
     return err;
 }
 
 static int real_csv_to_clipboard (const int *list)
 {
     PRN *prn = NULL;
+    gretlopt opt = OPT_NONE;
     int err = 0;
 
     if (bufopen(&prn)) {
 	return 1;
     }
 
-    err = data_to_buf_as_csv(list, prn);
+    if (get_csv_exclude_obs()) {
+	opt = OPT_X;
+    }
+
+    err = data_to_buf_as_csv(list, opt, prn);
     if (!err) {
 	err = prn_to_clipboard(prn, GRETL_FORMAT_CSV);
 	if (err) {
@@ -1891,10 +1986,11 @@ int csv_selected_to_clipboard (void)
     int err = 0;
 
     if (list != NULL) {
-	if (csv_options_dialog(NULL)) {
-	    return 0;
+	int resp = csv_options_dialog(COPY_CSV, GRETL_OBJ_DSET, NULL);
+
+	if (!canceled(resp)) {
+	    err = real_csv_to_clipboard(list);
 	}
-	err = real_csv_to_clipboard(list);
 	free(list);
     }
 
@@ -1903,7 +1999,7 @@ int csv_selected_to_clipboard (void)
 
 /* called from session.c: copy data to clipboard */
 
-int csv_to_clipboard (void)
+int csv_to_clipboard (GtkWidget *parent)
 {
     gchar *liststr;
     int cancel, err = 0;
@@ -1915,7 +2011,8 @@ int csv_to_clipboard (void)
 	int *list = gretl_list_from_string(liststr, &err);	
 
 	if (list != NULL) {
-	    cancel = csv_options_dialog(NULL);
+	    cancel = csv_options_dialog(COPY_CSV, GRETL_OBJ_DSET,
+					parent);
 	    if (!cancel) {
 		err = real_csv_to_clipboard(list);
 	    }
@@ -1929,6 +2026,8 @@ int csv_to_clipboard (void)
 
 static void matrix_print_as_csv (const gretl_matrix *m, PRN *prn)
 {
+    char decpoint = get_data_export_decpoint();
+    char delim = get_data_export_delimiter();
     char numstr[48];
     double x;
     int i, j;
@@ -1939,12 +2038,12 @@ static void matrix_print_as_csv (const gretl_matrix *m, PRN *prn)
 	for (j=0; j<m->cols; j++) {
 	    x = gretl_matrix_get(m, i, j);
 	    sprintf(numstr, "%.*g", DBL_DIG, x);
-	    if (dataset->decpoint != '.') {
-		charsub(numstr, '.', dataset->decpoint);
+	    if (decpoint != '.') {
+		charsub(numstr, '.', decpoint);
 	    }
 	    pputs(prn, numstr);
 	    if (j < m->cols - 1) {
-		pputc(prn, dataset->delim);
+		pputc(prn, delim);
 	    }
 	}
 	pputc(prn, '\n');
@@ -1953,68 +2052,67 @@ static void matrix_print_as_csv (const gretl_matrix *m, PRN *prn)
     gretl_pop_c_numeric_locale();
 }
 
-int matrix_to_clipboard_as_csv (const gretl_matrix *m)
+int matrix_to_clipboard_as_csv (const gretl_matrix *m,
+				GtkWidget *parent)
 {
-    PRN *prn = NULL;
-    gretlopt opt = OPT_M;
+    if (m != NULL) {
+	int resp = csv_options_dialog(COPY_CSV, GRETL_OBJ_MATRIX,
+				      parent);
+	PRN *prn = NULL;
 
-    if (m == NULL) {
-	return 0;
+	if (canceled(resp)) {
+	    return 0;
+	} else if (bufopen(&prn)) {
+	    return 1;
+	} else {
+	    matrix_print_as_csv(m, prn);
+	    prn_to_clipboard(prn, GRETL_FORMAT_CSV);
+	    gretl_print_destroy(prn);
+	}
     }
 
-    if (csv_options_dialog(&opt)) {
-	/* canceled */
-	return 0;
-    }
-
-    if (bufopen(&prn)) {
-	return 1;
-    }
-
-    matrix_print_as_csv(m, prn);
-    prn_to_clipboard(prn, GRETL_FORMAT_CSV);
-    gretl_print_destroy(prn);
-
     return 0;
 }
 
-int scalars_to_clipboard_as_csv (void)
+int scalars_to_clipboard_as_csv (GtkWidget *parent)
 {
-    PRN *prn = NULL;
     int err = 0;
 
     if (n_saved_scalars() == 0) {
 	warnbox(_("No scalar variables are currently defined"));
-	return 0;
-    }
+    } else {
+	int resp = csv_options_dialog(COPY_CSV, GRETL_OBJ_SCALARS,
+				      parent);
+	PRN *prn = NULL;
 
-    if (csv_options_dialog(NULL)) {
-	return 0;
+	if (canceled(resp)) {
+	    return 0;
+	} else if (bufopen(&prn)) {
+	    return 1;
+	} else {
+	    err = scalars_to_prn(prn);
+	    if (!err) {
+		prn_to_clipboard(prn, GRETL_FORMAT_CSV);
+	    }
+	    gretl_print_destroy(prn);
+	}
     }
 
-    if (bufopen(&prn)) {
-	return 1;
-    }
-
-    err = scalars_to_prn(prn);
-    if (!err) {
-	prn_to_clipboard(prn, GRETL_FORMAT_CSV);
-    }
-
-    gretl_print_destroy(prn);
-
     return err;
 }
 
 #include "series_view.h"
 #include "fileselect.h"
 
+/* callback from "series view" window, for use when
+   the delimited or RTF options are chosen
+*/
+
 int copy_vars_formatted (windata_t *vwin, int fmt, int action)
 {
+    char save_delim = get_data_export_delimiter();
     int *list = series_view_get_list(vwin);
     PRN *prn = NULL;
-    char save_delim = dataset->delim;
-    char save_decpoint = dataset->decpoint;
     int i, err = 0;
 
     if (list != NULL) {
@@ -2026,11 +2124,10 @@ int copy_vars_formatted (windata_t *vwin, int fmt, int action)
 	}
 
 	if (fmt == GRETL_FORMAT_CSV) {
-	    dataset->delim = ',';
-	    dataset->decpoint = '.';
+	    set_data_export_delimiter(',');
 	} else if (fmt == GRETL_FORMAT_TAB) {
-	    fmt |= GRETL_FORMAT_CSV;
-	    dataset->delim = '\t';
+	    fmt = GRETL_FORMAT_CSV;
+	    set_data_export_delimiter('\t');
 	}
 
 	if (series_view_is_sorted(vwin)) {
@@ -2040,13 +2137,12 @@ int copy_vars_formatted (windata_t *vwin, int fmt, int action)
 	    }
 	} else {
 	    err = bufopen(&prn);
-	}
-
-	if (!err) {
-	    if (fmt == GRETL_FORMAT_RTF) {
-		err = data_to_buf_as_rtf(list, prn);
-	    } else {
-		err = data_to_buf_as_csv(list, prn);
+	    if (!err) {
+		if (fmt == GRETL_FORMAT_RTF) {
+		    err = data_to_buf_as_rtf(list, prn);
+		} else {
+		    err = data_to_buf_as_csv(list, OPT_NONE, prn);
+		}
 	    }
 	}
 
@@ -2064,13 +2160,17 @@ int copy_vars_formatted (windata_t *vwin, int fmt, int action)
 	free(list);
     }
 
-    dataset->delim = save_delim;
-    dataset->decpoint = save_decpoint;
+    set_data_export_delimiter(save_delim);
 
     return err;
 }
 
-int font_has_minus (PangoFontDescription *desc)
+/* We mostly use this for checking whether the font described by @desc
+   has the Unicode minus sign (0x2212), which looks better than a
+   simple dash if it's available.  
+*/
+
+int font_has_symbol (PangoFontDescription *desc, int symbol)
 {
     GtkWidget *widget;
     PangoContext *context = NULL;
@@ -2081,7 +2181,7 @@ int font_has_minus (PangoFontDescription *desc)
     int ret = 0;
 
     if (desc == NULL) {
-	return -1;
+	return 0;
     }
 
     widget = gtk_label_new("");  
@@ -2100,8 +2200,7 @@ int font_has_minus (PangoFontDescription *desc)
 	if (pfont != NULL) {
 	    coverage = pango_font_get_coverage(pfont, lang);
 	    if (coverage != NULL) {
-		/* U+2212 = minus sign */
-		ret = (pango_coverage_get(coverage, 0x2212) == PANGO_COVERAGE_EXACT);
+		ret = (pango_coverage_get(coverage, symbol) == PANGO_COVERAGE_EXACT);
 		pango_coverage_unref(coverage);
 	    }
 	    g_object_unref(G_OBJECT(pfont));
@@ -2114,3 +2213,241 @@ int font_has_minus (PangoFontDescription *desc)
 
     return ret;
 }
+
+#ifdef G_OS_WIN32
+
+static int get_latex_path (char *latex_path)
+{
+    int ret;
+    char *p;
+
+    ret = SearchPath(NULL, latex, NULL, MAXLEN, latex_path, &p);
+
+    return (ret == 0);
+}
+
+#else
+
+static int spawn_latex (char *texsrc)
+{
+    GError *error = NULL;
+    gchar *errout = NULL, *sout = NULL;
+    gchar *argv[] = {
+	latex,
+	"\\batchmode",
+	"\\input",
+	texsrc,
+	NULL
+    };
+    int ok, status;
+    int ret = LATEX_OK;
+
+    signal(SIGCHLD, SIG_DFL);
+
+    ok = g_spawn_sync (gretl_dotdir(), /* working dir */
+		       argv,
+		       NULL,    /* envp */
+		       G_SPAWN_SEARCH_PATH,
+		       NULL,    /* child_setup */
+		       NULL,    /* user_data */
+		       &sout,   /* standard output */
+		       &errout, /* standard error */
+		       &status, /* exit status */
+		       &error);
+
+    if (!ok) {
+	errbox(error->message);
+	g_error_free(error);
+	ret = LATEX_EXEC_FAILED;
+    } else if (status != 0) {
+	if (errout && *errout) {
+	    errbox(errout);
+	} else {
+	    gchar *errmsg;
+
+	    errmsg = g_strdup_printf("%s\n%s", 
+				     _("Failed to process TeX file"),
+				     sout);
+	    errbox(errmsg);
+	    g_free(errmsg);
+	}
+	ret = LATEX_ERROR;
+    } else if (errout && *errout) {
+	fputs("spawn_latex: found stuff on stderr:\n", stderr);
+	fputs(errout, stderr);
+    }
+
+    /* change above, 2008-08-22: before we flagged a LATEX_ERROR
+       if we saw anything on standard error, regardless of the
+       exit status 
+    */
+
+    g_free(errout);
+    g_free(sout);
+
+    return ret;
+}
+
+#endif /* !G_OS_WIN32 */
+
+int latex_compile (char *texshort)
+{
+#ifdef G_OS_WIN32
+    static char latex_path[MAXLEN];
+    char tmp[MAXLEN];
+#endif
+    int err = LATEX_OK;
+
+#ifdef G_OS_WIN32
+    if (*latex_path == '\0' && get_latex_path(latex_path)) {
+	win_show_last_error();
+	return LATEX_EXEC_FAILED;
+    }
+
+    sprintf(tmp, "\"%s\" \\batchmode \\input %s", latex_path, texshort);
+    if (win_run_sync(tmp, gretl_dotdir())) {
+	return LATEX_EXEC_FAILED;
+    }
+#else
+    err = spawn_latex(texshort);
+#endif /* G_OS_WIN32 */
+
+    return err;
+}
+
+static int check_for_rerun (const char *texbase)
+{
+    char logfile[MAXLEN];
+    char lline[512];
+    FILE *fp;
+    int ret = 0;
+
+    sprintf(logfile, "%s.log", texbase);
+    fp = gretl_fopen(logfile, "r");
+
+    if (fp != NULL) {
+	while (fgets(lline, sizeof lline, fp)) {
+	    if (strstr(lline, "Rerun LaTeX")) {
+		ret = 1;
+		break;
+	    }
+	}
+	fclose(fp);
+    }
+
+    return ret;
+}
+
+static void view_or_save_latex (PRN *bprn, const char *fname, int saveit)
+{
+    char texfile[MAXLEN], texbase[MAXLEN], tmp[MAXLEN];
+    int use_pdf = get_tex_use_pdf();
+    int dot, err = LATEX_OK;
+    char *texshort = NULL;
+    const char *buf;
+    PRN *fprn;
+
+    *texfile = 0;
+
+    if (fname != NULL) {
+	strcpy(texfile, fname);
+    } else {
+	sprintf(texfile, "%swindow.tex", gretl_dotdir());
+    } 
+
+    /* ensure we don't get stale output */
+    remove(texfile);
+
+    fprn = gretl_print_new_with_filename(texfile, &err);
+    if (err) {
+	gui_errmsg(err);
+	return;
+    }
+
+    gretl_tex_preamble(fprn, prn_format(bprn));
+    buf = gretl_print_get_buffer(bprn);
+    pputs(fprn, buf);
+    pputs(fprn, "\n\\end{document}\n");
+
+    gretl_print_destroy(fprn);
+	
+    if (saveit) {
+	return;
+    }
+
+    dot = dotpos(texfile);
+    *texbase = 0;
+    strncat(texbase, texfile, dot);
+
+    texshort = strrchr(texbase, SLASH) + 1;
+    if (texshort == NULL) {
+	errbox(_("Failed to process TeX file"));
+	return;
+    } 
+
+    err = latex_compile(texshort);
+
+    /* now maybe re-run latex (e.g. for longtable) */
+    if (err == LATEX_OK) {
+	if (check_for_rerun(texbase)) {
+	    err = latex_compile(texshort);
+	}
+    }
+
+    if (err == LATEX_OK) {
+#if defined(G_OS_WIN32)
+	if (use_pdf) {
+	    sprintf(tmp, "%s.pdf", texbase);
+	    win32_open_file(tmp);
+	} else {
+	    sprintf(tmp, "\"%s\" \"%s.dvi\"", viewdvi, texbase);
+	    if (WinExec(tmp, SW_SHOWNORMAL) < 32) {
+		win_show_last_error();
+	    }
+	}
+#elif defined(OSX_BUILD)
+	if (use_pdf) {
+	    sprintf(tmp, "%s.pdf", texbase);
+	} else {
+	    sprintf(tmp, "%s.dvi", texbase);
+	}
+	if (osx_open_file(tmp)) {
+	    file_read_errbox(tmp);
+	}
+#else
+	if (use_pdf) {
+	    sprintf(tmp, "%s.pdf", texbase);
+	    gretl_fork("viewpdf", tmp);
+	} else {
+	    sprintf(tmp, "%s.dvi", texbase);
+	    gretl_fork("viewdvi", tmp);
+	}
+#endif
+    }
+
+    sprintf(tmp, "%s.log", texbase);
+    if (err == LATEX_ERROR) {
+	view_file(tmp, 0, 1, 78, 350, VIEW_FILE);
+    } else {
+	fprintf(stderr, "wrote '%s'\n", texfile);
+	/* gretl_remove(texfile); */
+	gretl_remove(tmp);
+    }
+
+    sprintf(tmp, "%s.aux", texbase);
+    gretl_remove(tmp);
+}
+
+void view_latex (PRN *prn)
+{
+    view_or_save_latex(prn, NULL, 0);
+}
+
+void save_latex (PRN *prn, const char *fname)
+{
+    if (prn != NULL) {
+	view_or_save_latex(prn, fname, 1);
+    } else {
+	save_graph_page(fname);
+    }
+}
