@@ -25,6 +25,8 @@
 #include "treeutils.h"
 #include "dlgutils.h"
 #include "menustate.h"
+#include "toolbar.h"
+#include "winstack.h"
 
 #ifdef G_OS_WIN32
 # include "gretlwin32.h"
@@ -42,9 +44,9 @@ static int translated_helpfile = -1;
 static char *en_gui_helpfile;
 static char *en_cli_helpfile;
 
-static void real_do_help (int idx, int pos, int role);
-static void en_help_callback (GtkAction *action, windata_t *hwin);
-static void delete_help_viewer (GtkAction *a, windata_t *hwin);
+static windata_t *real_do_help (int idx, int pos, int role);
+static void en_help_callback (GtkWidget *w, windata_t *hwin);
+static void delete_help_viewer (GtkWidget *w, windata_t *hwin);
 static char *funcs_helpfile (void);
 static void helpwin_set_topic_index (windata_t *hwin, int idx);
 
@@ -65,20 +67,17 @@ static GtkWidget *find_dialog = NULL;
 static GtkWidget *find_entry;
 static char *needle;
 
-const gchar *help_tools_ui = 
-    "<ui>"
-    "  <toolbar/>"
-    "</ui>";
+enum {
+    EN_ITEM = 1
+};
 
-GtkActionEntry help_tools[] = {
-    { "ZoomIn", GTK_STOCK_ZOOM_IN, NULL, NULL, N_("Larger"),
-      G_CALLBACK(text_zoom) },
-    { "ZoomOut", GTK_STOCK_ZOOM_OUT, NULL, NULL, N_("Smaller"),
-      G_CALLBACK(text_zoom) },
-    { "EnHelp", GRETL_STOCK_EN, NULL, NULL, N_("Show English help"), 
-      G_CALLBACK(en_help_callback) },
-    { "Close", GTK_STOCK_CLOSE, NULL, NULL, N_("Close"),
-      G_CALLBACK(delete_help_viewer) }
+static GretlToolItem help_tools[] = {
+    { N_("Larger"), GTK_STOCK_ZOOM_IN, G_CALLBACK(text_larger), 0},
+    { N_("Smaller"), GTK_STOCK_ZOOM_OUT, G_CALLBACK(text_smaller), 0},
+    { N_("Show English help"), GRETL_STOCK_EN, G_CALLBACK(en_help_callback), 
+      EN_ITEM },
+    { N_("Windows"), GRETL_STOCK_COMPASS, GNULL, 0 },
+    { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_help_viewer), 0 }
 };
 
 static gint n_help_tools = G_N_ELEMENTS(help_tools);
@@ -144,6 +143,9 @@ static struct gui_help_item gui_help_items[] = {
     { BWFILTER,       "bwfilter" },
     { POLYWEIGHTS,    "polyweights" },
     { X12AHELP,       "x12a" },
+    { LOESS,          "loess" },
+    { NADARWAT,       "nadarwat" },
+    { CLUSTER,        "cluster" },
     { -1,             NULL },
 };
 
@@ -705,7 +707,7 @@ static gboolean help_iter_from_index (int idx, int role,
     return FALSE;
 }
 
-static void en_help_callback (GtkAction *action, windata_t *hwin)
+static void en_help_callback (GtkWidget *w, windata_t *hwin)
 {
     int pos, idx = hwin->active_var;
     int role = (hwin->role == CLI_HELP)? CLI_HELP_EN :
@@ -721,7 +723,7 @@ static void en_help_callback (GtkAction *action, windata_t *hwin)
     real_do_help(idx, pos, role);
 }
 
-static void delete_help_viewer (GtkAction *a, windata_t *hwin) 
+static void delete_help_viewer (GtkWidget *w, windata_t *hwin) 
 {
     gtk_widget_destroy(hwin->main); 
 }
@@ -1007,81 +1009,40 @@ void vwin_add_finder (windata_t *vwin)
 		     vwin);
 }
 
-static void add_help_tool_by_name (windata_t *hwin, 
-				   const char *aname)
-{
-    GtkActionEntry *item = NULL;
-    int i;
-
-    for (i=0; i<n_help_tools; i++) {
-	if (!strcmp(help_tools[i].name, aname)) {
-	    item = &help_tools[i];
-	    break;
-	}
-    }
-
-    if (item != NULL) {
-	guint id = gtk_ui_manager_new_merge_id(hwin->ui);
-	GtkActionGroup *actions;
-
-	actions = gtk_action_group_new("AdHoc");
-	gtk_action_group_set_translation_domain(actions, "gretl");
-	gtk_action_group_add_actions(actions, item, 1, hwin);
-	gtk_ui_manager_add_ui(hwin->ui, id, "/toolbar", 
-			      item->name, item->name,
-			      GTK_UI_MANAGER_TOOLITEM, FALSE);
-	gtk_ui_manager_insert_action_group(hwin->ui, actions, 0);
-	g_object_unref(actions);
-    }
-}
-
 #define SHOW_FINDER(r)    (r != GUI_HELP && r != GUI_HELP_EN)
 #define SHOW_EN_BUTTON(r) (translated_helpfile && \
                            (r == CLI_HELP || r == GUI_HELP))
 
 void set_up_helpview_menu (windata_t *hwin) 
 {
-    GtkWidget *hbox, *tbar;
-    GError *err = NULL;
+    GretlToolItem *item = NULL;
+    GtkWidget *hbox;
+    int i;
 
-    hwin->ui = gtk_ui_manager_new();
-
-    /* add skeleton toolbar ui */
-    gtk_ui_manager_add_ui_from_string(hwin->ui, help_tools_ui, -1, &err);
-
-    if (err != NULL) {
-	fprintf(stderr, "building helpview ui failed: %s", err->message);
-	g_error_free(err);
-	return;
-    }
-
-    /* add buttons to toolbar depending on context */
-    add_help_tool_by_name(hwin, "ZoomIn");
-    add_help_tool_by_name(hwin, "ZoomOut");
-    if (SHOW_EN_BUTTON(hwin->role)) {
-	add_help_tool_by_name(hwin, "EnHelp");
-    }
-    add_help_tool_by_name(hwin, "Close");
-
-    /* hbox holding toolbar */
     hbox = gtk_hbox_new(FALSE, 0);
+    hwin->mbar = gretl_toolbar_new();
+
+    for (i=0; i<n_help_tools; i++) {
+	item = &help_tools[i];
+	if (!SHOW_EN_BUTTON(hwin->role) && item->flag == EN_ITEM) {
+	    continue;
+	}
+	if (winlist_item(item)) {
+	    vwin_toolbar_insert_winlist(hwin);
+	} else {
+	    gretl_toolbar_insert(hwin->mbar, item, item->func,
+				 hwin, -1);
+	}
+    }	
+
     gtk_box_pack_start(GTK_BOX(hwin->vbox), hbox, FALSE, FALSE, 0);
-    tbar = gtk_ui_manager_get_widget(hwin->ui, "/toolbar");
-    gtk_box_pack_start(GTK_BOX(hbox), tbar, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), hwin->mbar, FALSE, FALSE, 0);
 
-    gtk_toolbar_set_icon_size(GTK_TOOLBAR(tbar), GTK_ICON_SIZE_MENU);
-    gtk_toolbar_set_style(GTK_TOOLBAR(tbar), GTK_TOOLBAR_ICONS);
-    gtk_toolbar_set_show_arrow(GTK_TOOLBAR(tbar), FALSE);
-    hwin->mbar = tbar;
-
-    if (SHOW_FINDER(hwin->role)) {
+     if (SHOW_FINDER(hwin->role)) {
 	vwin_add_finder(hwin);
     }
 
     gtk_widget_show_all(hbox);
-
-    gtk_window_add_accel_group(GTK_WINDOW(hwin->main), 
-			       gtk_ui_manager_get_accel_group(hwin->ui));
 }
 
 static char *funcs_helpfile (void)
@@ -1095,12 +1056,10 @@ static char *funcs_helpfile (void)
     return fname;
 }
 
-void context_help (GtkWidget *widget, gpointer data)
+void show_gui_help (int helpcode)
 {
-    int pos, idx = GPOINTER_TO_INT(data);
-    int role = GUI_HELP;
-
-    idx = gui_ci_to_index(idx);
+    int idx = gui_ci_to_index(helpcode);
+    int pos, role = GUI_HELP;
 
     /* try for GUI help first */
     pos = help_pos_from_index(idx, role);
@@ -1123,17 +1082,40 @@ void context_help (GtkWidget *widget, gpointer data)
 	pos = help_pos_from_index(idx, role);
     }
 
-    if (pos <= 0) {
+    if (pos > 0) {
+	real_do_help(idx, pos, role);
+    } else {
 	warnbox(_("Sorry, no help is available"));
-	return;
+    }
+}
+
+static void context_help (GtkWidget *widget, gpointer data)
+{
+    int helpcode = GPOINTER_TO_INT(data);
+
+    show_gui_help(helpcode);
+}
+
+GtkWidget *context_help_button (GtkWidget *hbox, int helpcode)
+{
+    GtkWidget *button;
+
+    button = gtk_button_new_from_stock(GTK_STOCK_HELP);
+    gtk_widget_set_can_default(button, TRUE);
+    gtk_container_add(GTK_CONTAINER(hbox), button);
+    gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbox),
+				       button, TRUE);
+
+    if (helpcode >= 0) {
+	/* passing helpcode < 0 is a way of reserving
+	   the callback for something special
+	*/
+	g_signal_connect(G_OBJECT(button), "clicked", 
+			 G_CALLBACK(context_help), 
+			 GINT_TO_POINTER(helpcode));
     }
 
-#if HDEBUG
-    fprintf(stderr, "context_help: idx=%d, pos=%d, role=%d\n", 
-	    idx, pos, role);
-#endif
-
-    real_do_help(idx, pos, role);
+    return button;
 }
 
 static gboolean nullify_hwin (GtkWidget *w, windata_t **phwin)
@@ -1183,7 +1165,7 @@ static void helpwin_set_topic_index (windata_t *hwin, int idx)
     }
 }
 
-static void real_do_help (int idx, int pos, int role)
+static windata_t *real_do_help (int idx, int pos, int role)
 {
     static windata_t *gui_hwin;
     static windata_t *cli_hwin;
@@ -1196,7 +1178,7 @@ static void real_do_help (int idx, int pos, int role)
 
     if (pos < 0) {
 	dummy_call();
-	return;
+	return NULL;
     }
 
 #if HDEBUG
@@ -1270,6 +1252,8 @@ static void real_do_help (int idx, int pos, int role)
 	    helpwin_set_topic_index(hwin, idx);
 	}
     }
+
+    return hwin;
 }
 
 /* called from main menu in gretl.c; also used as callback from
@@ -1648,17 +1632,19 @@ get_tree_model_haystack (GtkTreeModel *mod, GtkTreeIter *iter, int col,
     }
 }
 
-static gboolean real_find_in_listbox (windata_t *vwin, const gchar *s, 
+static gboolean real_find_in_listbox (windata_t *vwin, 
+				      const gchar *s, 
 				      gboolean sensitive,
 				      gboolean vnames)
 {
+    int search_cols[4] = {0, 0, -1, -1};
     int minvar, wrapped = 0;
     char haystack[MAXLEN];
     char pstr[16];
     GtkTreeModel *model = NULL;
     GtkTreeIter iter;
     gboolean got_iter;
-    int pos = -1;
+    int i, pos = -1;
 
     /* first check that there's something to search */
     if (vwin->listbox != NULL) {
@@ -1689,25 +1675,33 @@ static gboolean real_find_in_listbox (windata_t *vwin, const gchar *s,
 	return FALSE;
     }
 
+    if (vnames) {
+	/* case-sensitive search for series names */
+	search_cols[0] = 1;  /* series name */
+	search_cols[1] = -1; /* invalid */
+    } else if (vwin == mdata) {
+	search_cols[0] = 1;  /* series name */
+	search_cols[1] = 2;  /* description */
+    } else if (vwin->role == FUNC_FILES) {
+	search_cols[0] = 0;  /* package name */
+	search_cols[1] = 2;  /* description (skip version #) */
+    } else if (vwin->role == REMOTE_FUNC_FILES) {
+	search_cols[0] = 0;  /* package name */
+	search_cols[1] = 3;  /* description */
+	search_cols[2] = 2;  /* author */
+    } else {
+	/* databases, datafiles */
+	search_cols[0] = 1; /* description */
+	search_cols[1] = 0; /* filename */
+    }
+
  search_wrap:
 
     while (pos < 0) {
-	/* try looking in column 1 first */
-	get_tree_model_haystack(model, &iter, 1, haystack);
-
-	pos = string_match_pos(haystack, needle, sensitive, 0);
-
-	if (pos < 0 && !vnames) {
-	    if (vwin == mdata) {
-		/* then column 2 */
-		get_tree_model_haystack(model, &iter, 2, haystack);
-	    } else {
-		/* then column 0 */
-		get_tree_model_haystack(model, &iter, 0, haystack);
-	    }
+	for (i=0; pos < 0 && search_cols[i] >= 0; i++) {
+	    get_tree_model_haystack(model, &iter, search_cols[i], haystack);
 	    pos = string_match_pos(haystack, needle, sensitive, 0);
 	}
-
 	if (pos >= 0 || !gtk_tree_model_iter_next(model, &iter)) {
 	    break;
 	}
@@ -1791,7 +1785,7 @@ static void cancel_find (GtkWidget *button, GtkWidget *dialog)
 
 static void parent_find (GtkWidget *finder, windata_t *caller)
 {
-    GtkWidget *w = caller->main;
+    GtkWidget *w = vwin_toplevel(caller);
 
     if (w != NULL) {
 	gtk_window_set_transient_for(GTK_WINDOW(finder), GTK_WINDOW(w));
@@ -1898,7 +1892,8 @@ enum {
     GRETL_GUIDE = 1,
     GRETL_REF,
     GNUPLOT_REF,
-    X12A_REF
+    X12A_REF,
+    GRETL_KEYS
 };
 
 static int get_writable_path (char *path, const char *fname, int code)
@@ -2000,6 +1995,10 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 	"gretl-ref-it.pdf",
 	"gretl-ref-es.pdf"  
     };
+    const char *kbd_files[] = {
+	"gretl-keys.pdf",
+	"gretl-keys-a4.pdf"
+    };
     const char *fname;
     char *tmp = NULL;
     FILE *fp;
@@ -2014,6 +2013,9 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 	fname = guide_files[i];
     } else if (code == GRETL_REF) {
 	fname = ref_files[i];
+    } else if (code == GRETL_KEYS) {
+	if (i > 1) i = 1; /* no translations */
+	fname = kbd_files[i];
     } else if (code == GNUPLOT_REF) {
 	fname = "gnuplot.pdf";
     } else if (code == X12A_REF) {
@@ -2094,12 +2096,15 @@ void display_pdf_help (GtkAction *action)
     int err, code = GRETL_GUIDE;
 
     if (action != NULL) {
-	if (strcmp(gtk_action_get_name(action), "UserGuide")) {
-	    /* PDF command ref wanted */
+	if (!strcmp(gtk_action_get_name(action), "PDFCmdRef")) {
 	    code = GRETL_REF;
-	} 
+	} else if (!strcmp(gtk_action_get_name(action), "KbdRef")) {
+	    code = GRETL_KEYS;
+	}
     }
 
+    fprintf(stderr, "display_pdf_help: code = %d\n", code);
+
     err = find_or_download_pdf(code, get_manpref(), fname);
 
     if (!err) {
