@@ -22,12 +22,6 @@
 #ifndef LIBRARY_H
 #define LIBRARY_H
 
-enum {
-    LATEX_OK,
-    LATEX_EXEC_FAILED,
-    LATEX_ERROR
-} tex_return_codes;
-
 /* general purpose functions */
 
 #ifdef G_OS_WIN32
@@ -51,15 +45,17 @@ void library_command_free (void);
 
 CMD *get_lib_cmd (void);
 
-int gretl_command_sprintf (const char *template, ...);
+int lib_command_sprintf (const char *template, ...);
 
-int gretl_command_strcpy (const char *s);
+int lib_command_strcpy (const char *s);
 
-int gretl_command_strcat (const char *s);
+int lib_command_strcat (const char *s);
 
-int record_command_line (const char *s);
+int record_command_verbatim (void);
 
-int record_command_verbatim (const char *s);
+int parse_lib_command (void);
+
+int record_lib_command (void);
 
 int user_fopen (const char *fname, char *fullname, PRN **pprn);
 
@@ -77,7 +73,9 @@ void gui_errmsg (int errcode);
 
 void gui_warnmsg (int errcode);
 
-void register_graph (PRN *prn);
+void errmsg_plus (int err, const char *plus);
+
+void gui_graph_handler (int err);
 
 /* sample-related functions */
 
@@ -163,10 +161,14 @@ void do_saved_eqn_system (GtkWidget *w, dialog_t *dlg);
 
 int do_model (selector *sr);
 
+int do_nonparam_model (selector *sr);
+
 int do_vector_model (selector *sr);
 
 void do_graph_model (const int *list, int fit);
 
+void do_nonparam_plot (windata_t *vwin);
+
 void do_gini (void);
 
 void do_qqplot (void);
@@ -245,18 +247,18 @@ void do_add_obs (void);
 
 void do_remove_obs (void);
 
-void add_logs_etc (int ci);
+void add_logs_etc (int ci, int varnum);
 
 void logs_etc_callback (GtkAction *action);
 
 void add_system_resid (GtkAction *action, gpointer p);
 
-int save_fit_resid (MODEL *pmod, int code);
+int save_fit_resid (windata_t *vwin, int code);
 
 int save_bundled_series (const double *x, const char *key,
-			 const char *note);
+			 const char *note, windata_t *vwin);
 
-void add_model_stat (MODEL *pmod, int which);
+void add_model_stat (MODEL *pmod, int which, windata_t *vwin);
 
 void resid_plot (GtkAction *action, gpointer p);
 
@@ -308,40 +310,36 @@ void do_new_script (int code);
 
 void new_script_callback (GtkAction *action);
 
-int do_store (char *mydatfile, int action);
+int do_store (char *filename, int action, gpointer data);
 
-void view_latex (PRN *prn);
+void set_csv_exclude_obs (gboolean s);
 
-void save_latex (PRN *prn, const char *fname);
+gboolean get_csv_exclude_obs (void);
 
 void do_save_text (char *fname, MODEL *pmod);
 
 int gui_exec_line (ExecState *s, DATASET *dset);
 
-int check_and_record_command (void);
-
-#if USE_GTK_SPINNER
-
-void start_wait_for_output (GtkWidget *w, int big);
+void start_wait_for_output (GtkWidget *w, gboolean big);
 
 void stop_wait_for_output (GtkWidget *w);
 
-#endif
-
 /* other */
 
+gchar *get_genr_string (GtkWidget *entry, dialog_t *dlg);
+
 int menu_op_wrapper (selector *sr);
 
 int max_untouchable_series_ID (void);
 
-int latex_compile (char *texshort);
-
 void add_mahalanobis_data (windata_t *vwin);
 
 void add_pca_data (windata_t *vwin);
 
 void add_fcast_data (windata_t *vwin);
 
+void add_nonparam_data (windata_t *vwin);
+
 void VECM_add_EC_data (GtkAction *action, gpointer p);
 
 void maybe_display_string_table (void);
