@@ -26,6 +26,7 @@
 #include "selector.h"
 #include "varinfo.h"
 #include "treeutils.h"
+#include "session.h"
 #include "menustate.h"
 
 void refresh_data (void)
@@ -103,8 +104,6 @@ void variable_menu_state (gboolean s)
 static void view_items_state (gboolean s)
 {
     const char *viewpaths[] = {
-	"IconView",
-	"EditScalars",
 	"GraphVars",
 	"MultiPlots",
 	"summary",
@@ -112,7 +111,6 @@ static void view_items_state (gboolean s)
 	"xtab",
 	"pca",
 	"mahal",
-	"xcorrgm",
 	NULL
     };
     char fullpath[32];
@@ -122,9 +120,14 @@ static void view_items_state (gboolean s)
 	sprintf(fullpath, "/menubar/View/%s", viewpaths[i]);
 	flip(mdata->ui, fullpath, s);
     }
+
+    flip(mdata->ui, "/menubar/View/IconView", have_session_objects());
+
+    flip(mdata->ui, "/menubar/View/xcorrgm",  
+	 data_status && dataset_is_time_series(dataset));
 }
 
-void main_menubar_state (gboolean s)
+void dataset_menubar_state (gboolean s)
 {
     if (mdata == NULL || mdata->ui == NULL) return;
 
@@ -142,7 +145,13 @@ void main_menubar_state (gboolean s)
 
     view_items_state(s);
 
-    if (s || get_n_listed_windows() == 0) {
+    if (s || !have_session_objects()) {
+	/* Either we're enabling dataset items, in which
+	   case we should also enable the /View menu, or
+	   we're disabling the dataset items and there are 
+	   no session objects, in which case /View should 
+	   be disabled.
+	*/
 	flip(mdata->ui, "/menubar/View", s);
     }
 
@@ -151,19 +160,17 @@ void main_menubar_state (gboolean s)
     set_main_colheads_clickable(s);
 }
 
-void window_list_state (gboolean s)
+void iconview_menubar_state (gboolean s)
 {
     if (s) {
 	GtkAction *a = gtk_ui_manager_get_action(mdata->ui, "/menubar/View");
 
 	if (a != NULL && !gtk_action_get_sensitive(a)) {
 	    gtk_action_set_sensitive(a, TRUE);
-	    view_items_state(FALSE);
 	}
-	flip(mdata->ui, "/menubar/View/Windows", s);
-    } else if (!data_status) {
-	flip(mdata->ui, "/menubar/View", s);
     }
+
+    flip(mdata->ui, "/menubar/View/IconView", s);
 }
 
 #define COMPACTABLE(d) (d->structure == TIME_SERIES && \
@@ -182,14 +189,19 @@ void time_series_menu_state (gboolean s)
     gboolean sx = extended_ts(dataset);
     gboolean panel = dataset_is_panel(dataset);
     gboolean realpan = multi_unit_panel_sample(dataset);
-    gboolean ur = s;
+    gboolean ur;
 
     if (mdata->ui == NULL) {
 	return;
     }
 
+    /* unit-root tests: require time-series or panel data,
+       and a time series length greater than 5
+    */
     if (panel) {
 	ur = dataset->pd > 5;
+    } else {
+	ur = s && sample_size(dataset) > 5;
     }
 
     /* Plots */
@@ -201,11 +213,10 @@ void time_series_menu_state (gboolean s)
     /* Variable menu */
     flip(mdata->ui, "/menubar/Variable/URTests", ur); 
     if (ur && !s) {
+	/* a time-series only "ur" option */
 	flip(mdata->ui, "/menubar/Variable/URTests/fractint", s);
     }
-    if (ur && !panel) {
-	flip(mdata->ui, "/menubar/Variable/URTests/levinlin", panel);
-    }
+    flip(mdata->ui, "/menubar/Variable/URTests/levinlin", ur && panel);
     flip(mdata->ui, "/menubar/Variable/corrgm", s);
     flip(mdata->ui, "/menubar/Variable/pergm", s);
     flip(mdata->ui, "/menubar/Variable/Filter", s);
@@ -216,8 +227,10 @@ void time_series_menu_state (gboolean s)
     flip(mdata->ui, "/menubar/Variable/Tramo", get_tramo_ok());
 #endif
     flip(mdata->ui, "/menubar/Variable/Hurst", s);
+
     /* Model menu */
     flip(mdata->ui, "/menubar/Model/TSModels", s);
+
     /* Sample menu */
     flip(mdata->ui, "/menubar/Data/DataCompact", 
 	 s && (COMPACTABLE(dataset) || dated_weekly_data(dataset)));
@@ -318,71 +331,127 @@ static gint var_popup_click (GtkWidget *w, gpointer p)
     gchar *item = (gchar *) p;
     int v = mdata_active_var();
 
-    if (!strcmp(item, _("Display values"))) 
+    if (!strcmp(item, _("Display values"))) { 
 	display_var();
-    if (!strcmp(item, _("Descriptive statistics"))) 
+    } else if (!strcmp(item, _("Summary statistics"))) { 
 	do_menu_op(VAR_SUMMARY, NULL, OPT_NONE);
-    else if (!strcmp(item, _("Time series plot"))) 
+    } else if (!strcmp(item, _("Time series plot"))) { 
 	do_graph_var(v);
-    else if (!strcmp(item, _("Panel plot..."))) 
+    } else if (!strcmp(item, _("Panel plot..."))) { 
 	do_graph_var(v);
-    else if (!strcmp(item, _("Frequency distribution"))) 
+    } else if (!strcmp(item, _("Frequency distribution"))) { 
 	do_freq_dist();
-    else if (!strcmp(item, _("Boxplot")))
+    } else if (!strcmp(item, _("Boxplot"))) {
 	menu_boxplot_callback(v);
-    else if (!strcmp(item, _("Gini coefficient")))
+    } else if (!strcmp(item, _("Gini coefficient"))) {
 	do_gini();
-    else if (!strcmp(item, _("Correlogram")))
+    } else if (!strcmp(item, _("Correlogram"))) {
 	do_corrgm();
-    else if (!strcmp(item, _("Periodogram"))) 
+    } else if (!strcmp(item, _("Periodogram"))) { 
 	do_pergm(NULL);
-    else if (!strcmp(item, _("ARIMA model"))) {
+    } else if (!strcmp(item, _("ARIMA model"))) {
 	selector_set_varnum(v);
 	modelspec_dialog(ARMA);
-    } else if (!strcmp(item, _("Dickey-Fuller test"))) 
+    } else if (!strcmp(item, _("Dickey-Fuller test"))) { 
 	unit_root_test(ADF);
-    else if (!strcmp(item, _("KPSS test"))) 
+    } else if (!strcmp(item, _("KPSS test"))) { 
 	unit_root_test(KPSS);
-    else if (!strcmp(item, _("Hurst exponent"))) 
-	do_hurst();
-    else if (!strcmp(item, _("Edit attributes")))  
+    } else if (!strcmp(item, _("Edit attributes"))) {  
 	varinfo_dialog(v);
-    else if (!strcmp(item, _("Edit values")))  
+    } else if (!strcmp(item, _("Edit values"))) {  
 	show_spreadsheet(SHEET_EDIT_VARLIST);
-    else if (!strcmp(item, _("Copy to clipboard"))) 
+    } else if (!strcmp(item, _("Copy to clipboard"))) { 
 	csv_selected_to_clipboard();
-    else if (!strcmp(item, _("Delete"))) 
+    } else if (!strcmp(item, _("Delete"))) { 
 	delete_single_var(v);
-    else if (!strcmp(item, _("Define new variable..."))) 
+    } else if (!strcmp(item, _("Add log"))) {
+	add_logs_etc(LOGS, v);
+    } else if (!strcmp(item, _("Add difference"))) {
+	add_logs_etc(DIFF, v);
+    } else if (!strcmp(item, _("Define new variable..."))) { 
 	genr_callback();
+    }
 
     gtk_widget_destroy(mdata->popup);
 
     return FALSE;
 }
 
+static int missvals_in_selection (void)
+{
+    int *list = main_window_selection_as_list();
+    int miss = 0;
+
+    if (list != NULL) {
+	int i, vi, t;
+
+	for (i=1; i<=list[0] && !miss; i++) {
+	    vi = list[i];
+	    for (t=dataset->t1; t<=dataset->t2; t++) {
+		if (na(dataset->Z[vi][t])) {
+		    miss = 1;
+		    break;
+		}
+	    }
+	}
+	free(list);
+    }
+
+    return miss;
+}
+
+static int uniform_corr_option (const gchar *title, gretlopt *popt)
+{
+    const char *opts[] = {
+	N_("Ensure uniform sample size"),
+	NULL
+    };
+    int uniform = 0;
+    int resp;
+
+    resp = checks_only_dialog(title, NULL, opts, 1, 
+			      &uniform, CORR, NULL);
+
+    if (!canceled(resp) && uniform) {
+	*popt = OPT_U;
+    }
+
+    return resp;
+}
+
 static gint selection_popup_click (GtkWidget *w, gpointer p)
 {
     gchar *item = (gchar *) p;
     int ci = 0;
 
-    if (!strcmp(item, _("Descriptive statistics"))) {
+    if (!strcmp(item, _("Summary statistics"))) {
 	ci = SUMMARY;
     } else if (!strcmp(item, _("Correlation matrix"))) {
 	ci = CORR;
     }
 
-    if (ci == CORR) {
-	gchar *title;
+    if (ci == CORR && missvals_in_selection()) {
+	gchar *title = g_strdup_printf("gretl: %s", _("correlation matrix"));
+	gretlopt opt = OPT_NONE;
+	int resp;
 
-	title = g_strdup_printf("gretl: %s", gretl_command_word(ci));
-	simple_selection(title, menu_op_wrapper, ci, NULL);
+	resp = uniform_corr_option(title, &opt);
+	if (!canceled(resp)) {
+	    char *buf = main_window_selection_as_string();
+
+	    if (buf != NULL) {
+		do_menu_op(ci, buf, opt);
+		free(buf);
+	    }
+	}	    
 	g_free(title);
     } else if (ci != 0) {
 	char *buf = main_window_selection_as_string();
 	
-	do_menu_op(ci, buf, OPT_NONE);
-	free(buf);
+	if (buf != NULL) {
+	    do_menu_op(ci, buf, OPT_NONE);
+	    free(buf);
+	}
     } else if (!strcmp(item, _("Display values"))) { 
 	display_selected(); 
     } else if (!strcmp(item, _("Cross-correlogram")))  {
@@ -398,9 +467,13 @@ static gint selection_popup_click (GtkWidget *w, gpointer p)
     } else if (!strcmp(item, _("Delete")))  {
 	delete_selected_vars();
     } else if (!strcmp(item, _("Add logs")))  {
-	add_logs_etc(LOGS);
+	add_logs_etc(LOGS, 0);
     } else if (!strcmp(item, _("Add differences")))  {
-	add_logs_etc(DIFF);
+	add_logs_etc(DIFF, 0);
+    } else if (!strcmp(item, _("Define list"))) { 
+	make_list_from_main();
+    } else if (!strcmp(item, _("Define new variable..."))) { 
+	genr_callback();
     }
 
     gtk_widget_destroy(mdata->popup);
@@ -412,7 +485,7 @@ GtkWidget *build_var_popup (void)
 {
     const char *items[] = {
 	N_("Display values"),
-	N_("Descriptive statistics"),
+	N_("Summary statistics"),
 	N_("Time series plot"),
 	N_("Panel plot..."),
 	N_("Frequency distribution"),
@@ -424,6 +497,8 @@ GtkWidget *build_var_popup (void)
 	N_("Copy to clipboard"),
 	N_("Delete"),
 	NULL,
+	N_("Add log"),
+	N_("Add difference"),
 	N_("Define new variable...")
     };
     GtkWidget *menu;
@@ -451,7 +526,11 @@ GtkWidget *build_var_popup (void)
 	if ((i == 6 || i == 7) && !dataset_is_time_series(dataset)) {
 	    continue;
 	}
-	if (i == 2 && !extended_ts(dataset)) {
+	if ((i == 2 || i == 14) && !extended_ts(dataset)) {
+	    continue;
+	}
+	if (i == 5 && dataset_is_time_series(dataset)) {
+	    /* skip boxplot option */
 	    continue;
 	}
 	item = gtk_menu_item_new_with_label(_(items[i]));
@@ -469,17 +548,20 @@ GtkWidget *build_selection_popup (void)
 {
     const char *items[] = {
 	N_("Display values"),
-	N_("Descriptive statistics"),
+	N_("Summary statistics"),
 	N_("Correlation matrix"),
 	N_("Cross-correlogram"),
 	N_("Time series plot"),
 	N_("XY scatterplot"),
-	N_("Copy to clipboard"),
 	N_("Edit values"),
+	N_("Copy to clipboard"),
 	N_("Delete"),
 	NULL,
 	N_("Add logs"),
-	N_("Add differences")
+	N_("Add differences"),
+	NULL,
+	N_("Define list"),
+	N_("Define new variable...")
     };
     GtkWidget *menu;
     GtkWidget *item;
@@ -595,6 +677,8 @@ void set_sample_label (DATASET *pdinfo)
 	return;
     }
 
+    *t1str = *t2str = '\0';
+
     /* set the sensitivity of various menu items */
 
     time_series_menu_state(dataset_is_time_series(pdinfo));
@@ -713,42 +797,129 @@ int vwin_add_ui (windata_t *vwin, GtkActionEntry *entries,
     return 0;
 }
 
-int vwin_menu_add_item (windata_t *vwin, const gchar *path, 
-			GtkActionEntry *entry)
+static GtkActionGroup *get_named_group (GtkUIManager *uim,
+					const char *name, 
+					int *newgroup)
 {
-    guint id = gtk_ui_manager_new_merge_id(vwin->ui);
-    GtkActionGroup *actions;
+    GList *list = gtk_ui_manager_get_action_groups(uim); 
+    GtkActionGroup *actions = NULL;
 
-    actions = gtk_action_group_new("AdHoc");
+    while (list != NULL) {
+	GtkActionGroup *group = list->data;
+
+	if (!strcmp(gtk_action_group_get_name(group), name)) {
+	    actions = group;
+	    break;
+	}
+	list = list->next;
+    } 
+
+    if (actions == NULL) {
+	actions = gtk_action_group_new(name);
+	gtk_action_group_set_translation_domain(actions, "gretl");
+	*newgroup = 1;
+    } else {
+	*newgroup = 0;
+    }
+
+    return actions;
+}
+
+int vwin_menu_add_item_unique (windata_t *vwin, 
+			       const gchar *aname, 
+			       const gchar *path, 
+			       GtkActionEntry *entry)
+{
+    GList *list = gtk_ui_manager_get_action_groups(vwin->ui);
+    GtkActionGroup *actions;
+    guint id;
+
+    while (list != NULL) {
+	GtkActionGroup *group = list->data;
+
+	if (!strcmp(aname, gtk_action_group_get_name(group))) {
+	    gtk_ui_manager_remove_action_group(vwin->ui, group);
+	    break;
+	}
+	list = list->next;
+    }
+
+    id = gtk_ui_manager_new_merge_id(vwin->ui);
+    actions = gtk_action_group_new(aname);
     gtk_action_group_set_translation_domain(actions, "gretl");
 
     gtk_action_group_add_actions(actions, entry, 1, vwin);
     gtk_ui_manager_add_ui(vwin->ui, id, path, entry->name, entry->name,
 			  GTK_UI_MANAGER_MENUITEM, FALSE);
+
     gtk_ui_manager_insert_action_group(vwin->ui, actions, 0);
     g_object_unref(actions);
+	
+    return id;
+}
+
+/* Retrieve existing "AdHoc" action group from @uim, or add a 
+   new group of this name to the UIManager and return it.
+*/
+
+GtkActionGroup *get_ad_hoc_group (GtkUIManager *uim,
+				  int *newgroup)
+{
+    return get_named_group(uim, "AdHoc", newgroup);
+}
+
+/* Adds the specified @entry to vwin->ui at @path; returns
+   the "merge_id", which can be used to remove the item
+*/
+
+int vwin_menu_add_item (windata_t *vwin, const gchar *path, 
+			GtkActionEntry *entry)
+{
+    GtkActionGroup *actions;
+    int newgroup = 1;
+    guint id;
+
+    actions = get_ad_hoc_group(vwin->ui, &newgroup);
+    gtk_action_group_add_actions(actions, entry, 1, vwin);
+    id = gtk_ui_manager_new_merge_id(vwin->ui); 
+
+    gtk_ui_manager_add_ui(vwin->ui, id, path, entry->name, entry->name,
+			  GTK_UI_MANAGER_MENUITEM, FALSE);
+
+    if (newgroup) {
+	gtk_ui_manager_insert_action_group(vwin->ui, actions, 0);
+	g_object_unref(actions);
+    }
 
     return id;
 }
 
+/* Adds the specified @entries to vwin->ui at @path; returns
+   the "merge_id", which can be used to remove the items
+*/
+
 int vwin_menu_add_items (windata_t *vwin, const gchar *path, 
 			 GtkActionEntry *entries, int n)
 {
-    guint id = gtk_ui_manager_new_merge_id(vwin->ui);
     GtkActionGroup *actions;
+    int newgroup = 1;
+    guint id;
     int i;
 
-    actions = gtk_action_group_new("AdHoc");
-    gtk_action_group_set_translation_domain(actions, "gretl");
-
+    actions = get_ad_hoc_group(vwin->ui, &newgroup);
     gtk_action_group_add_actions(actions, entries, n, vwin);
+    id = gtk_ui_manager_new_merge_id(vwin->ui);
+
     for (i=0; i<n; i++) {
 	gtk_ui_manager_add_ui(vwin->ui, id, path, 
 			      entries[i].name, entries[i].name,
 			      GTK_UI_MANAGER_MENUITEM, FALSE);
     }
-    gtk_ui_manager_insert_action_group(vwin->ui, actions, 0);
-    g_object_unref(actions);
+
+    if (newgroup) {
+	gtk_ui_manager_insert_action_group(vwin->ui, actions, 0);
+	g_object_unref(actions);
+    }
 
     return id;
 }
