@@ -9,7 +9,7 @@ void gretl_set_window_quasi_modal (GtkWidget *w);
 void flip (GtkUIManager *ui, const char *path, gboolean s);
 
 void variable_menu_state (gboolean s);
-void main_menubar_state (gboolean s);
+void dataset_menubar_state (gboolean s);
 void time_series_menu_state (gboolean s);
 void panel_menu_state (gboolean s);
 void ts_or_panel_menu_state (gboolean s);
@@ -17,7 +17,7 @@ void session_menu_state (gboolean s);
 void restore_sample_state (gboolean s);
 void compact_data_state (gboolean s);
 void drop_obs_state (gboolean s);
-void window_list_state (gboolean s);
+void iconview_menubar_state (gboolean s);
 
 void main_menus_enable (gboolean s);
 
@@ -35,6 +35,14 @@ void action_entry_init (GtkActionEntry *entry);
 int vwin_add_ui (windata_t *vwin, GtkActionEntry *entries,
 		 gint n_entries, const gchar *ui_info);
 
+GtkActionGroup *get_ad_hoc_group (GtkUIManager *uim,
+				  int *newgroup);
+
+int vwin_menu_add_item_unique (windata_t *vwin, 
+			       const gchar *aname, 
+			       const gchar *path, 
+			       GtkActionEntry *entry);
+
 int vwin_menu_add_item (windata_t *vwin, const gchar *path, 
 			GtkActionEntry *entry);
 
