@@ -20,9 +20,9 @@
 #ifndef OBS_BUTTON_H__
 #define OBS_BUTTON_H__
 
-GtkWidget *obs_button_new (GtkAdjustment *adjustment, DATAINFO *pdinfo);
+GtkWidget *obs_button_new (GtkAdjustment *adjustment, DATASET *pdinfo);
 
-GtkWidget *data_start_button (GtkAdjustment *adj, DATAINFO *pdinfo);
+GtkWidget *data_start_button (GtkAdjustment *adj, DATASET *pdinfo);
 
 int obs_button_get_value (GtkWidget *button);
 
