@@ -20,6 +20,13 @@
 #ifndef SELECTOR_H
 #define SELECTOR_H
 
+#define robust_conf(c) (c != LOGIT && c != PROBIT &&	\
+                        c != OLOGIT && c != OPROBIT &&	\
+                        c != QUANTREG && c != INTREG && \
+                        c != MLOGIT && c != COUNTMOD && \
+                        c != DURATION && c != HECKIT && \
+			c != BIPROBIT && c != TOBIT)
+
 typedef struct iterinfo_t iterinfo;
 
 struct iterinfo_t {
@@ -30,17 +37,24 @@ struct iterinfo_t {
 
 void clear_selector (void);
 
-selector *selection_dialog (const char *title, int (*callback)(), 
-			    guint cmdcode);
+selector *selection_dialog (int ci, const char *title, int (*callback)());
 
-selector *simple_selection (const char *title, int (*callback)(), 
-			    guint cmdcode, gpointer p);
+selector *simple_selection (int ci, const char *title, int (*callback)(), 
+			    GtkWidget *parent);
+
+selector *
+simple_selection_for_viewer (int ci, const char *title, int (*callback)(), 
+			     windata_t *vwin);
+
+selector *
+simple_selection_with_data (int ci, const char *title, int (*callback)(), 
+			    GtkWidget *parent, gpointer data);
 
 void modelspec_dialog (int ci);
 
 void selector_set_varnum (int v);
 
-void selector_from_model (void *ptr, int ci);
+void selector_from_model (windata_t *vwin);
 
 char *main_window_selection_as_string (void);
 
@@ -80,6 +94,8 @@ void set_selector_storelist (const char *s);
 
 void selector_register_genr (int newvars, gpointer p);
 
+void selector_register_hc_choice (void);
+
 void selector_cleanup (void);
 
 #endif /* SELECTOR_H */
