@@ -95,6 +95,10 @@ static int series_view_allocate (series_view *sview)
 {
     int err = 0;
 
+    if (sview == NULL) {
+	return E_DATA;
+    }
+
     if (sview->points != NULL) {
 	/* already allocated */
 	return 0;
@@ -117,40 +121,6 @@ static int series_view_allocate (series_view *sview)
     return err;
 }
 
-static PRN *single_series_view_print_formatted (windata_t *vwin, 
-						PrnFormat fmt)
-{
-    series_view *sview = (series_view *) vwin->data;
-    char obslabel[OBSLEN];
-    char dchar = dataset->delim;
-    double x;
-    PRN *prn;
-    int i, t;
-
-    if (bufopen(&prn)) {
-	return NULL;
-    }
-
-    pprintf(prn, "obs%c%s\n", dchar, dataset->varname[sview->varnum]);
-
-    for (i=0; i<sview->npoints; i++) {
-	t = sview->points[i].obsnum;
-	x = sview->points[i].val;
-	if (dataset_has_markers(dataset)) {
-	    strcpy(obslabel, dataset->S[t]);
-	} else {
-	    ntodate(obslabel, t, dataset);
-	}
-	if (na(x)) {
-	    pprintf(prn, "%s%cNA\n", obslabel, dchar);
-	} else {
-	    pprintf(prn, "\"%s\"%c%.8g\n", obslabel, dchar, x);
-	}
-    }
-
-    return prn;
-}
-
 /* for printing sorted or reformatted data, for a window displaying
    a single series */
 
@@ -158,11 +128,10 @@ static void single_series_view_print (windata_t *vwin)
 {
     series_view *sview = (series_view *) vwin->data;
     char num_format[32];
-    char obslabel[OBSLEN];
-    int thislen, obslen = 0;
     double x;
     PRN *prn;
-    int i, t, err = 0;
+    int i, t, obslen;
+    int err = 0;
 
     if (bufopen(&prn)) {
 	return;
@@ -183,34 +152,25 @@ static void single_series_view_print (windata_t *vwin)
 	goto finalize;
     }
 
-    obslen = max_obs_label_length(dataset);
-    if (obslen < 2) {
-	obslen = -2;
-    }
+    obslen = max_obs_marker_length(dataset);
 
     if (sview->format == 'g') {
-	sprintf(num_format, "%%*s %%#13.%dg\n", sview->digits);
+	sprintf(num_format, "%%#13.%dg\n", sview->digits);
     } else {
-	sprintf(num_format, "%%*s %%13.%df\n", sview->digits);
+	sprintf(num_format, "%%13.%df\n", sview->digits);
     }
 	
-    pprintf(prn, "\n%*s ", obslen, "");
+    pprintf(prn, "\n%*s ", obslen, " ");
     pprintf(prn, "%13s\n\n", dataset->varname[sview->varnum]);
 
     for (i=0; i<sview->npoints; i++) {
 	t = sview->points[i].obsnum;
 	x = sview->points[i].val;
-	if (dataset_has_markers(dataset)) {
-	    strcpy(obslabel, dataset->S[t]);
-	    thislen = get_utf_width(obslabel, obslen);
-	} else {
-	    ntodate(obslabel, t, dataset);
-	    thislen = obslen;
-	}
+	print_obs_marker(t, dataset, obslen, prn);
 	if (na(x)) {
-	    pprintf(prn, "%*s\n", thislen, obslabel);
+	    pputc(prn, '\n');
 	} else {
-	    pprintf(prn, num_format, thislen, obslabel, x);
+	    pprintf(prn, num_format, x);
 	} 
     }
 
@@ -256,11 +216,14 @@ static void multi_series_view_print_sorted (windata_t *vwin)
 	return;
     }
 
-    err = print_data_in_columns(sview->list, obsvec, dataset, prn);
+    err = print_data_in_columns(sview->list, obsvec, dataset, 
+				OPT_NONE, prn);
     if (err) {
 	gui_errmsg(err);
     } else {
-	textview_set_text(vwin->text, gretl_print_get_trimmed_buffer(prn));
+	const char *buf = gretl_print_get_trimmed_buffer(prn);
+
+	textview_set_text(vwin->text, buf);
     }
 
     free(obsvec);
@@ -307,18 +270,11 @@ int series_view_is_sorted (windata_t *vwin)
 
 PRN *vwin_print_sorted_with_format (windata_t *vwin, PrnFormat fmt)
 {
-    series_view *sview;
+    series_view *sview = (series_view *) vwin->data;
     int *obsvec;
     PRN *prn;
     int err = 0;
 
-    if (vwin->role == VIEW_SERIES) {
-	/* a single series */
-	return single_series_view_print_formatted(vwin, fmt);
-    }
-
-    sview = (series_view *) vwin->data;
-
     obsvec = make_obsvec(sview);
     if (obsvec == NULL) {
 	return NULL;
@@ -330,7 +286,18 @@ PRN *vwin_print_sorted_with_format (windata_t *vwin, PrnFormat fmt)
     }
 
     gretl_print_set_format(prn, fmt);
-    err = print_data_in_columns(sview->list, obsvec, dataset, prn);
+
+    if (sview->varnum > 0) {
+	/* single series, no list */
+	int list[2] = {1, sview->varnum};
+
+	err = print_data_in_columns(list, obsvec, dataset, 
+				    OPT_NONE, prn);
+    } else {
+	err = print_data_in_columns(sview->list, obsvec, dataset, 
+				    OPT_NONE, prn);
+    }
+
     if (err) {
 	gui_errmsg(err);
 	gretl_print_destroy(prn);
@@ -342,6 +309,29 @@ PRN *vwin_print_sorted_with_format (windata_t *vwin, PrnFormat fmt)
     return prn;
 }
 
+static void summary_print_formatted (windata_t *vwin, series_view *sview)
+{
+    Summary *summ = vwin->data;
+    int digits = 0, places = 0;
+    PRN *prn;
+
+    if (bufopen(&prn)) {
+	return;
+    }
+
+    if (sview->view != VIEW_STANDARD && sview->digits > 0) {
+	if (sview->format == 'g') {
+	    digits = sview->digits;
+	} else {
+	    places = sview->digits;
+	}
+    }
+
+    print_summary_single(summ, digits, places, dataset, prn);
+    textview_set_text(vwin->text, gretl_print_get_trimmed_buffer(prn));
+    gretl_print_destroy(prn);
+}    
+
 static int sort_points (const void *a, const void *b)
 {
     const data_point *pa = (const data_point *) a;
@@ -411,7 +401,8 @@ void multi_series_view_sort_by (GtkWidget *w, windata_t *vwin)
 	return;
     }
 
-    v = select_var_from_list(sview->list, _("Variable to sort by"));
+    v = select_var_from_list(sview->list, _("Variable to sort by"),
+			     vwin->main);
     if (v < 0) {
 	/* canceled */
 	return;
@@ -521,14 +512,32 @@ int can_format_data (windata_t *vwin)
 	series_view *sview = vwin->data;
 
 	return (sview->list != NULL);
+    } else if (vwin->role == SUMMARY && vwin->data != NULL) {
+	Summary *s = vwin->data;
+
+	return (s->list != NULL && s->list[0] == 1);
     } else {
 	return 0;
     }
 }
 
+static void series_view_init (series_view *sview)
+{
+    sview->varnum = 0;
+    sview->sortvar = 0;
+    sview->npoints = 0;
+    sview->view = VIEW_STANDARD;
+    sview->digits = 6;
+    sview->format = 'g';
+    sview->sorted = 0;
+    sview->points = NULL;
+    sview->list = NULL;
+}
+
 static series_view *series_view_new (int varnum, const int *list)
 {
     series_view *sview = NULL;
+    int *svlist = NULL;
 
     if (varnum == 0 && list == NULL) {
 	return NULL;
@@ -539,25 +548,18 @@ static series_view *series_view_new (int varnum, const int *list)
 	return NULL;
     }
 
-    if (list == NULL) {
-	sview->list = NULL;
-    } else {
-	sview->list = gretl_list_copy(list);
-	if (sview->list == NULL) {
+    if (list != NULL) {
+	svlist = gretl_list_copy(list);
+	if (svlist == NULL) {
 	    free(sview);
 	    sview = NULL;
 	}
-    } 
+    }	
 
     if (sview != NULL) {
+	series_view_init(sview);
 	sview->varnum = varnum;
-	sview->sortvar = 0;
-	sview->npoints = 0;
-	sview->view = VIEW_STANDARD;
-	sview->digits = 6;
-	sview->format = 'g';
-	sview->sorted = 0;
-	sview->points = NULL;
+	sview->list = svlist;
     }
 
     return sview;
@@ -576,23 +578,25 @@ series_view *multi_series_view_new (const int *list)
 }
 
 struct view_toggler {
-    char *view;
     GtkWidget *spin;
     GtkWidget *combo;
+    char view;
+    int digits;
+    char format;
+    windata_t *target_vwin;
+    series_view *sview;
 };
 
-/* toggle between standard and custom views */
+/* toggle for standard versus custom view */
 
 static void series_view_toggle_view (GtkWidget *w, struct view_toggler *vt)
 {
-    gint i = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(w), "view"));
+    gboolean std = button_is_active(w);
 
-    if (button_is_active(w)) {
-        *vt->view = i;
-	gtk_widget_set_sensitive(vt->spin, i == VIEW_CUSTOM);
-	if (vt->combo != NULL) {
-	    gtk_widget_set_sensitive(vt->combo, i == VIEW_CUSTOM);
-	}
+    vt->view = std ? VIEW_STANDARD : VIEW_CUSTOM;
+    gtk_widget_set_sensitive(vt->spin, !std);
+    if (vt->combo != NULL) {
+	gtk_widget_set_sensitive(vt->combo, !std);
     }
 }
 
@@ -608,8 +612,25 @@ static void series_view_set_fmt (GtkComboBox *cb, char *format)
     *format = (i == 0)? 'g' : 'f';
 }
 
-static void real_view_format_dialog (GtkWidget *src, windata_t *vwin,
-				     series_view *sview)
+static void reformat_callback (GtkButton *b, struct view_toggler *vt)
+{
+    windata_t *vwin = vt->target_vwin;
+    series_view *sview = vt->sview;
+
+    sview->digits = vt->digits;
+    sview->format = vt->format;
+    sview->view = vt->view;
+
+    if (vwin->role == SUMMARY) {
+	summary_print_formatted(vwin, sview);
+    } else if (sview->list != NULL) {
+	multi_series_view_print(vwin);
+    } else {
+	single_series_view_print(vwin);
+    }
+}
+
+static void real_view_format_dialog (windata_t *vwin, series_view *sview)
 {
     struct view_toggler vt;
     GtkWidget *dlg, *vbox;
@@ -618,13 +639,18 @@ static void real_view_format_dialog (GtkWidget *src, windata_t *vwin,
     GSList *group;
     int std;
 
-    dlg = gretl_dialog_new(_("gretl: data format"), NULL,
-			   GRETL_DLG_BLOCK | GRETL_DLG_MODAL);
+    dlg = gretl_dialog_new(_("gretl: data format"), vwin->main,
+			   GRETL_DLG_BLOCK);
 
     vbox = gtk_dialog_get_content_area(GTK_DIALOG(dlg));
 
     std = (sview->view == VIEW_STANDARD);
-    vt.view = &sview->view;
+
+    vt.view = sview->view;
+    vt.digits = sview->digits;
+    vt.format = sview->format;
+    vt.target_vwin = vwin;
+    vt.sview = sview;
 
     hbox = gtk_hbox_new(FALSE, 5);
     tmp = gtk_label_new(_("Select data format"));
@@ -634,7 +660,6 @@ static void real_view_format_dialog (GtkWidget *src, windata_t *vwin,
     /* standard format radio option */
     hbox = gtk_hbox_new(FALSE, 5);
     b1 = gtk_radio_button_new_with_label(NULL, _("Standard format"));
-    g_object_set_data(G_OBJECT(b1), "view", GINT_TO_POINTER(VIEW_STANDARD));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b1), std);
     gtk_box_pack_start(GTK_BOX(hbox), b1, TRUE, TRUE, 5);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
@@ -643,7 +668,6 @@ static void real_view_format_dialog (GtkWidget *src, windata_t *vwin,
     hbox = gtk_hbox_new(FALSE, 5);
     group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b1));
     b2 = gtk_radio_button_new_with_label(group, _("Show"));
-    g_object_set_data(G_OBJECT(b2), "view", GINT_TO_POINTER(VIEW_CUSTOM));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b2), !std);
     gtk_box_pack_start(GTK_BOX(hbox), b2, FALSE, FALSE, 5);
 
@@ -651,7 +675,7 @@ static void real_view_format_dialog (GtkWidget *src, windata_t *vwin,
     vt.spin = gtk_spin_button_new_with_range(1, 15, 1);
     gtk_spin_button_set_value(GTK_SPIN_BUTTON(vt.spin), sview->digits);
     g_signal_connect(G_OBJECT(vt.spin), "value-changed",
-		     G_CALLBACK(series_view_set_digits), &sview->digits);
+		     G_CALLBACK(series_view_set_digits), &vt.digits);
     gtk_widget_set_sensitive(vt.spin, !std);
     gtk_box_pack_start(GTK_BOX(hbox), vt.spin, FALSE, FALSE, 0);
 
@@ -665,15 +689,13 @@ static void real_view_format_dialog (GtkWidget *src, windata_t *vwin,
 	gtk_combo_box_set_active(GTK_COMBO_BOX(vt.combo), 1);
     }
     g_signal_connect(G_OBJECT(GTK_COMBO_BOX(vt.combo)), "changed",
-		     G_CALLBACK(series_view_set_fmt), &sview->format);
+		     G_CALLBACK(series_view_set_fmt), &vt.format);
     gtk_widget_set_sensitive(vt.combo, !std);
     gtk_box_pack_start(GTK_BOX(hbox), vt.combo, FALSE, FALSE, 5);
 
-    /* connect toggle signals */
+    /* connect toggle signal */
     g_signal_connect(G_OBJECT(b1), "toggled",
                      G_CALLBACK(series_view_toggle_view), &vt);
-    g_signal_connect(G_OBJECT(b2), "toggled",
-                     G_CALLBACK(series_view_toggle_view), &vt);
 
     /* pack the custom line */
     gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 5);
@@ -681,36 +703,40 @@ static void real_view_format_dialog (GtkWidget *src, windata_t *vwin,
     hbox = gtk_dialog_get_action_area(GTK_DIALOG(dlg));
 
     /* Cancel button */
-    cancel_options_button(hbox, dlg, &sview->digits);
+    cancel_delete_button(hbox, dlg);
    
     /* OK button */
     tmp = ok_button(hbox);
+    g_signal_connect(G_OBJECT(tmp), "clicked",
+		     G_CALLBACK(reformat_callback), &vt);
     g_signal_connect(G_OBJECT(tmp), "clicked",
 		     G_CALLBACK(delete_widget), dlg);
     gtk_widget_grab_default(tmp);
 
     gtk_widget_show_all(dlg);
-
-    if (sview->digits > 0) {
-	if (sview->list != NULL) {
-	    multi_series_view_print(vwin);
-	} else {
-	    single_series_view_print(vwin);
-	}
-    } else { 
-	/* canceled */
-	sview->digits = 6;
-    }
 }
 
-void series_view_format_dialog (GtkWidget *src, windata_t *vwin)
+void series_view_format_dialog (windata_t *vwin)
 {
-    series_view *sview = (series_view *) vwin->data;
+    static series_view static_sview;
+    series_view *sview;
 
-    if (vwin->role == VIEW_SERIES && series_view_allocate(sview)) {
-	return;
-    }
+    if (vwin->role == SUMMARY) {
+	static int initted;
 
-    real_view_format_dialog(src, vwin, sview);
+	if (!initted) {
+	    series_view_init(&static_sview);
+	    initted = 1;
+	}
+	sview = &static_sview;
+    } else {    
+	sview = (series_view *) vwin->data;
+
+	if (series_view_allocate(sview)) {
+	    return;
+	}
+    } 
+
+    real_view_format_dialog(vwin, sview);
 }
 
