@@ -49,7 +49,6 @@
 #include "matrix_extra.h"
 #include "cmd_private.h"
 #include "gretl_scalar.h"
-#include "gretl_bundle.h"
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -148,6 +147,7 @@ enum {
 static char *global_items[] = {
     N_("Save session"),
     N_("Arrange icons"),
+    N_("Windows"),
     N_("Close window")
 };
 
@@ -260,8 +260,8 @@ static gboolean session_view_click (GtkWidget *widget,
 static int real_delete_model_from_session (SESSION_MODEL *model);
 static void make_short_label_string (char *targ, const char *src);
 static gui_obj *get_gui_obj_by_data (void *finddata);
-static void add_user_matrix_callback (void);
-static void add_bundle_callback (void);
+static void add_user_matrix_callback (const char *mname);
+static void add_bundle_callback (gretl_bundle *bundle);
 
 static int session_graph_count;
 static int commands_recorded;
@@ -273,6 +273,7 @@ int session_is_modified (void)
 
 void mark_session_changed (void)
 {
+    iconview_menubar_state(TRUE);
     session.status = SESSION_CHANGED;
     if (save_item != NULL) {
 	gtk_widget_set_sensitive(save_item, TRUE);
@@ -303,6 +304,11 @@ int get_commands_recorded (void)
     return commands_recorded;
 }
 
+int widget_is_iconview (GtkWidget *w)
+{
+    return w == iconview;
+}
+
 /* constructors and destructors for session data-objects */
 
 static void free_session_text (SESSION_TEXT *text)
@@ -855,7 +861,7 @@ int gui_add_graph_to_session (char *fname, char *fullname, int type)
     } 
 
     gretl_remove(fname);
-    strcpy(fname, shortname);
+    strcpy(fname, fullname); /* was @shortname */
 
     err = real_add_graph_to_session(shortname, graphname, type);
     if (err == ADD_OBJECT_FAIL) {
@@ -1005,11 +1011,11 @@ int add_model_to_session_callback (void *ptr, GretlObjType type)
     targ = get_session_model_by_name(name); 
 
     if (targ != NULL) {
-	GtkWidget *w = match_window_by_data(targ->ptr);
+	windata_t *vwin = get_viewer_for_data(targ->ptr);
 
-	if (w != NULL) {
+	if (vwin != NULL) {
 	    /* current model window is "orphaned" */
-	    gtk_widget_destroy(w);
+	    gretl_viewer_destroy(vwin);
 	}
 
 	targ->ptr = ptr;
@@ -1059,7 +1065,12 @@ void model_add_as_icon (GtkAction *action, gpointer p)
 #endif
 
     if (get_session_model_by_data(ptr)) {
-	infobox(_("Model is already saved"));
+	/* "can't happen" */
+	if (close_on_add(action)) {
+	    gretl_viewer_destroy(vwin);
+	} else {
+	    infobox(_("Model is already saved"));
+	}
 	return;
     }
     
@@ -1083,8 +1094,10 @@ void model_add_as_icon (GtkAction *action, gpointer p)
 
     if (!err) {
 	mark_session_changed();
-	if (close_on_add(action) && !window_is_busy(vwin)) {
-	    gtk_widget_destroy(vwin->main);
+	if (close_on_add(action)) {
+	    gretl_viewer_destroy(vwin);
+	} else {
+	    set_model_save_state(vwin, FALSE);
 	}
     } 	
 }
@@ -1104,9 +1117,10 @@ void bundle_add_as_icon (GtkAction *action, gpointer p)
     free(defname);
     blurb = "Save bundle\nName (max. 15 characters):";
     pshow = (iconlist == NULL)? &show : NULL;
-    resp = object_name_entry_dialog(vname, GRETL_TYPE_BUNDLE, blurb, pshow);
+    resp = object_name_entry_dialog(vname, GRETL_TYPE_BUNDLE, 
+				    blurb, pshow, vwin->main);
 
-    if (resp >= 0) {    
+    if (!canceled(resp)) {    
 	int err = gretl_bundle_set_name(bundle, vname);
 	int flipit = 1;
 
@@ -1119,7 +1133,7 @@ void bundle_add_as_icon (GtkAction *action, gpointer p)
 		view_session();
 	    }
 	    mark_session_changed();
-	    if (close_on_add(action) && !window_is_busy(vwin)) {
+	    if (close_on_add(action)) {
 		gtk_widget_destroy(vwin->main);
 		flipit = 0;
 	    }
@@ -1294,6 +1308,7 @@ void do_open_session (void)
     char fname[MAXLEN];   /* multi-purpose temp variable */
     gchar *zdirname = NULL;
     FILE *fp;
+    int nodata = 0;
     int err = 0;
 
     sinfo_init(&sinfo);
@@ -1338,7 +1353,7 @@ void do_open_session (void)
     session_file_make_path(xmlname, "session.xml");
 
     /* try getting the name of the session data file first */
-    err = get_session_datafile_name(xmlname, &sinfo);
+    err = get_session_datafile_name(xmlname, &sinfo, &nodata);
 
     if (err) {
 	fprintf(stderr, "Failed on read_session_xml: err = %d\n", err);
@@ -1346,36 +1361,39 @@ void do_open_session (void)
 	goto bailout;
     }
 
-    /* construct path to session data file */
-    session_file_make_path(datafile, sinfo.datafile);
-    fp = gretl_fopen(datafile, "r");
+    if (!nodata) {
+	/* construct path to session data file */
+	session_file_make_path(datafile, sinfo.datafile);
+	fp = gretl_fopen(datafile, "r");
 
-    if (fp != NULL) {
-	/* OK, write good name into gdtname */
-	fprintf(stderr, "got datafile name '%s'\n", datafile);
-	strcpy(gdtname, datafile);
-	fclose(fp);
-    } else {
-	/* try remedial action, transform filename? */
-	fprintf(stderr, "'%s' : not found, trying to fix\n", datafile);
-	err = get_session_dataname(gdtname, &sinfo);
+	if (fp != NULL) {
+	    /* OK, write good name into gdtname */
+	    fprintf(stderr, "got datafile name '%s'\n", datafile);
+	    strcpy(gdtname, datafile);
+	    fclose(fp);
+	} else {
+	    /* try remedial action, transform filename? */
+	    fprintf(stderr, "'%s' : not found, trying to fix\n", datafile);
+	    err = get_session_dataname(gdtname, &sinfo);
+	}
+
+	if (!err) {
+	    err = gretl_read_gdt(gdtname, dataset, OPT_B, NULL);
+	}
+
+	if (err) {
+	    /* FIXME more explicit error message? */
+	    file_read_errbox(sinfo.datafile);
+	    goto bailout;
+	} else {
+	    fprintf(stderr, "Opened session datafile '%s'\n", gdtname);
+	    data_status = USER_DATA;
+	}
     }
 
-    if (!err) {
-	err = gretl_read_gdt(gdtname, dataset, OPT_B, NULL);
-    }
-
-    if (err) {
-	/* FIXME more explicit error message */
-	file_read_errbox(sinfo.datafile);
-	goto bailout;
-    } else {
-	fprintf(stderr, "Opened session datafile '%s'\n", gdtname);
-	data_status = USER_DATA;
-    }
-
-    /* having opened the data file, get the rest of the info from
-       session.xml */    
+    /* having opened the data file (or not, if there's none), get the 
+       rest of the info from session.xml 
+    */    
     err = read_session_xml(xmlname, &sinfo);
     if (err) {
 	fprintf(stderr, "Failed on read_session_xml: err = %d\n", err);
@@ -1401,34 +1419,36 @@ void do_open_session (void)
     session_file_make_path(fname, "settings.inp");
     err = maybe_read_settings_file(fname);
 
-    if (sinfo.resample_n > 0) {
-	err = dataset_resample(sinfo.resample_n, sinfo.seed, 
-			       dataset);
-    } else if (sinfo.mask != NULL) {
-	err = restrict_sample_from_mask(sinfo.mask, dataset,
-					OPT_NONE);
-	if (!err) {
-	    dataset->restriction = sinfo.restriction;
-	    sinfo.restriction = NULL;
+    if (!nodata) {
+	if (sinfo.resample_n > 0) {
+	    err = dataset_resample(sinfo.resample_n, sinfo.seed, 
+				   dataset);
+	} else if (sinfo.mask != NULL) {
+	    err = restrict_sample_from_mask(sinfo.mask, dataset,
+					    OPT_NONE);
+	    if (!err) {
+		dataset->restriction = sinfo.restriction;
+		sinfo.restriction = NULL;
+	    }
 	}
+
+	if (err) {
+	    errbox(_("Couldn't set sample"));
+	    goto bailout;
+	}
+
+	dataset->t1 = sinfo.t1;
+	dataset->t2 = sinfo.t2;
+
+	register_data(OPENED_VIA_SESSION);
+
+	if (sinfo.mask != NULL) {
+	    set_sample_label(dataset);
+	}
+
+	sinfo_free_data(&sinfo);
     }
 
-    if (err) {
-	errbox(_("Couldn't set sample"));
-	goto bailout;
-    }
-
-    dataset->t1 = sinfo.t1;
-    dataset->t2 = sinfo.t2;
-
-    register_data(OPENED_VIA_SESSION);
-
-    if (sinfo.mask != NULL) {
-	set_sample_label(dataset);
-    }
-
-    sinfo_free_data(&sinfo);
-
     set_main_window_title(session.name, FALSE);
 
  bailout:
@@ -1500,10 +1520,11 @@ void session_init (void)
     *session.name = '\0';
     *session.dirname = '\0';
 
-    winstack_init();
+    set_matrix_add_callback(add_user_matrix_callback);
+    set_matrix_delete_callback(session_matrix_destroy_by_name);
 
-    set_matrix_add_callback(add_user_matrix_callback); 
     set_bundle_add_callback(add_bundle_callback);
+    set_bundle_delete_callback(session_bundle_destroy_by_name);
 }
 
 void free_session (void)
@@ -1614,7 +1635,7 @@ void gui_clear_dataset (void)
 
     data_status = 0;
     orig_vars = 0;
-    main_menubar_state(FALSE);
+    dataset_menubar_state(FALSE);
 }
 
 static void session_clear_data (DATASET *pdinfo)
@@ -1662,8 +1683,7 @@ void close_session (gretlopt opt)
     session.show_notes = 0;
     commands_recorded = 0;
 
-    winstack_destroy();
-    close_plot_windows();
+    close_session_windows();
     selector_cleanup();
     function_call_cleanup();
     edit_dialog_special_get_text(NULL);
@@ -1726,8 +1746,6 @@ static int real_save_session_dataset (const char *dname)
     if (!err) {
 	session_file_make_path(tmpname, dname);
 	write_err = gretl_write_gdt(tmpname, NULL, dataset, OPT_NONE, 1);
-	fprintf(stderr, "Save session datafile as '%s', err = %d\n",
-		tmpname, write_err);
     }
 
     if (mask != NULL) {
@@ -1749,6 +1767,8 @@ static int real_save_session_dataset (const char *dname)
 	/* flag the fact that the data are saved */
 	data_status &= ~MODIFIED_DATA;
     }
+
+    fprintf(stderr, "real_save_session_dataset: err=%d\n", err);
     
     return err;
 }
@@ -1855,9 +1875,19 @@ int save_session (char *fname)
 	strcpy(session.dirname, dirname);
     }
 
-    make_session_dataname(datname);
+    *datname = '\0';
+
+    if (data_status) {
+	make_session_dataname(datname);
+    } else {
+	strcpy(datname, "none");
+    }
+
     write_session_xml(datname);
-    err = real_save_session_dataset(datname);
+
+    if (data_status) {
+	err = real_save_session_dataset(datname);
+    }
 
     if (!err) {
 	session_switch_log_location(LOG_SAVE);
@@ -1872,6 +1902,7 @@ int save_session (char *fname)
 
 	/* make zipfile containing session files */
 	err = (*gretl_make_zipfile)(fname, dirname, &gerr);
+	fprintf(stderr, "gretl_make_zipfile: err = %d\n", err);
 	close_plugin(handle);
     }
 
@@ -1883,17 +1914,20 @@ int save_session (char *fname)
 	if (fname != sessionfile) {
 	    session_name_from_session_file(session.name, fname);
 	    strcpy(sessionfile, fname);
-	    data_status |= SESSION_DATA;
+	    data_status |= SESSION_DATA; /* FIXME? */
 	    set_sample_label(dataset);
 	}
 	mark_session_saved();
     }
 
+    fprintf(stderr, "save_session; returning %d\n", err);
+
     return err;
 }
 
 void session_notes_callback (GtkWidget *w, gpointer p)
 {
+    windata_t *vwin = (windata_t *) p;
     const char *opts[] = {
 	N_("Display notes on opening session file")
     };
@@ -1903,7 +1937,7 @@ void session_notes_callback (GtkWidget *w, gpointer p)
     active[0] = session.show_notes;
 
     resp = checks_only_dialog("gretl", NULL, opts, 1,
-			      active, 0);
+			      active, 0, vwin->main);
 
     if (resp >= 0 && session.show_notes != active[0]) {
 	session.show_notes = active[0];
@@ -2029,17 +2063,16 @@ static gchar *graph_str (SESSION_GRAPH *graph)
     return buf;
 }
 
-static int maybe_raise_object_window (gpointer p)
+static int maybe_raise_object_window (gpointer data)
 {
-    GtkWidget *w = match_window_by_data(p);
-    int ret = 0;
+    windata_t *vwin = get_viewer_for_data(data);
 
-    if (w != NULL) {
-	gtk_window_present(GTK_WINDOW(w));
-	ret = 1;
+    if (vwin != NULL) {
+	gretl_viewer_present(vwin);
+	return 1;
+    } else {
+	return 0;
     }
-
-    return ret;
 }
 
 static int display_session_model (SESSION_MODEL *sm)
@@ -2058,7 +2091,7 @@ static int display_session_model (SESSION_MODEL *sm)
 	MODEL *pmod = (MODEL *) sm->ptr;
 
 	printmodel(pmod, dataset, OPT_NONE, prn);
-	view_model(prn, pmod, 78, 400, sm->name);
+	view_model(prn, pmod, sm->name);
     } else if (sm->type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) sm->ptr;
 
@@ -2066,10 +2099,10 @@ static int display_session_model (SESSION_MODEL *sm)
 	view_buffer(prn, 78, 450, sm->name, var->ci, var);
     } else if (sm->type == GRETL_OBJ_SYS) {
 	equation_system *sys = (equation_system *) sm->ptr;
-	int cancel = 0;
 
-	edit_dialog(sm->name, NULL, NULL, do_saved_eqn_system, sys, 
-		    ESTIMATE, VARCLICK_NONE, &cancel); 
+	edit_dialog(ESTIMATE, sm->name, NULL, NULL, 
+		    do_saved_eqn_system, sys, 
+		    VARCLICK_NONE, iconview); 
     }
 
     return 0;
@@ -2284,8 +2317,8 @@ static void maybe_delete_session_object (gui_obj *obj)
 	char fullname[MAXLEN];
 
 	session_file_make_path(fullname, graph->fname);
-	busy = plot_file_is_busy(fullname);
-	if (!busy && match_window_by_filename(fullname)) {
+	if (get_window_for_plot(fullname) || 
+	    get_editor_for_file(fullname)) {
 	    busy = 1;
 	}
     } else {
@@ -2300,11 +2333,12 @@ static void maybe_delete_session_object (gui_obj *obj)
 	    p = obj->data;
 	}
 
-	busy = (p != NULL && winstack_match_data(p));
+	busy = (p != NULL && get_viewer_for_data(p) != NULL);
     }
 
     if (busy) {
-	warnbox(_("Please close this object's window first"));
+	warnbox(_("%s: please close this object's window first"),
+		obj->name);
     } else {	    
 	gchar *msg = g_strdup_printf(_("Really delete %s?"), obj->name);
 
@@ -2315,6 +2349,23 @@ static void maybe_delete_session_object (gui_obj *obj)
     }
 }
 
+static gui_obj *get_gui_obj_by_name_and_sort (const char *name,
+					      int sort)
+{
+    GList *mylist = g_list_first(iconlist);
+    gui_obj *obj = NULL;
+
+    while (mylist != NULL) {
+	obj = (gui_obj *) mylist->data;
+	if (!strcmp(name, obj->name) && obj->sort == sort) {
+	    return obj;
+	}
+	mylist = mylist->next;
+    }
+
+    return NULL;
+}
+
 static gui_obj *get_gui_obj_by_data (void *targ)
 {
     GList *mylist = g_list_first(iconlist);
@@ -2331,20 +2382,24 @@ static gui_obj *get_gui_obj_by_data (void *targ)
     return NULL;
 }
 
-int session_matrix_destroy_by_name (const char *name, PRN *prn)
+/* This callback is invoked when a named matrix is deleted via script
+   or console.  Note that we call user_matrix_destroy() below, and NOT
+   user_matrix_destroy_by_name() -- calling the latter would lead to a
+   loop back to here.  
+*/
+
+int session_matrix_destroy_by_name (const char *name)
 {
     user_matrix *u = get_user_matrix_by_name(name);
     int err;
 
     if (u == NULL) {
 	err = E_UNKVAR;
-    } else if (winstack_match_data(u)) {
-	errbox(_("Please close this object's window first"));
-	return 0;
     } else {
-	gui_obj *obj = get_gui_obj_by_data(u);
+	maybe_close_window_for_data(u, GRETL_OBJ_MATRIX);
+	if (iconlist != NULL) {
+	    gui_obj *obj = get_gui_obj_by_data(u);
 
-	if (obj != NULL) {
 	    session_delete_icon(obj);
 	}
 	err = user_matrix_destroy(u);
@@ -2353,23 +2408,27 @@ int session_matrix_destroy_by_name (const char *name, PRN *prn)
     return err;
 }
 
-int session_bundle_destroy_by_name (const char *name, PRN *prn)
+/* This callback is invoked when a named bundle is deleted via script
+   or console. As with session_matrix_destroy_by_name() we invoke
+   user_bundle_destroy() rather than gretl_bundle_delete_by_name()
+   to avoid looping back.
+*/
+
+int session_bundle_destroy_by_name (const char *name)
 {
     gretl_bundle *b = get_gretl_bundle_by_name(name);
     int err;
 
     if (b == NULL) {
 	err = E_UNKVAR;
-    } else if (winstack_match_data(b)) {
-	errbox(_("Please close this object's window first"));
-	return 0;
     } else {
-	gui_obj *obj = get_gui_obj_by_data(b);
+	maybe_close_window_for_data(b, GRETL_OBJ_BUNDLE);
+	if (iconlist != NULL) {
+	    gui_obj *obj = get_gui_obj_by_data(b);
 
-	if (obj != NULL) {
 	    session_delete_icon(obj);
 	}
-	err = gretl_bundle_delete_by_name(name, NULL);
+	err = user_bundle_destroy(b);
     } 
     
     return err;
@@ -2390,12 +2449,12 @@ static void rename_session_graph (SESSION_GRAPH *graph, const char *newname)
 
 static void maybe_sync_model_window_name (SESSION_MODEL *sm)
 {
-    GtkWidget *w = match_window_by_data(sm->ptr);
+    windata_t *vwin = get_viewer_for_data(sm->ptr);
 
-    if (w != NULL) {
+    if (vwin != NULL) {
 	gchar *title = g_strdup_printf("gretl: %s", sm->name);
 
-	gtk_window_set_title(GTK_WINDOW(w), title);
+	gretl_viewer_set_title(vwin, title);
 	g_free(title);
     }
 }
@@ -2476,7 +2535,7 @@ static void rename_object_callback (GtkWidget *widget, dialog_t *dlg)
     }
 
     if (!err) {
-	close_dialog(dlg);
+	edit_dialog_close(dlg);
     }
 }
 
@@ -2491,9 +2550,9 @@ static void rename_object_dialog (gui_obj *obj)
 
     tmp = g_strdup_printf(_("Enter new name\n(max. %d characters)"),
 			  maxlen);
-    edit_dialog(_("gretl: rename object"), tmp,
-		obj->name, rename_object_callback, obj, 
-		0, VARCLICK_NONE, NULL);
+    edit_dialog(0, _("gretl: rename object"), tmp, obj->name, 
+		rename_object_callback, obj, 
+		VARCLICK_NONE, iconview);
     g_free(tmp);
 }
 
@@ -2686,14 +2745,13 @@ static void batch_pack_icons (void)
     }
 }
 
-static void add_user_matrix_callback (void)
+static void add_user_matrix_callback (const char *mname)
 {
     if (iconview != NULL) {
-	int n = n_user_matrices();
+	user_matrix *u = get_user_matrix_by_name(mname);
 
-	if (n > 0) {
-	    session_add_icon(get_user_matrix_by_index(n-1), GRETL_OBJ_MATRIX, 
-			     ICON_ADD_SINGLE);
+	if (u != NULL) {
+	    session_add_icon(u, GRETL_OBJ_MATRIX, ICON_ADD_SINGLE);
 	}
     } else if (autoicon_on()) {
 	view_session();
@@ -2702,14 +2760,17 @@ static void add_user_matrix_callback (void)
     mark_session_changed();
 }
 
-static void add_bundle_callback (void)
+static void add_bundle_callback (gretl_bundle *bundle)
 {
     if (iconview != NULL) {
-	int n = n_user_bundles();
+	/* first check for replacement of an existing bundle */
+	const char *name = gretl_bundle_get_name(bundle);
+	gui_obj *obj = get_gui_obj_by_name_and_sort(name, GRETL_OBJ_BUNDLE);
 
-	if (n > 0) {
-	    session_add_icon(get_gretl_bundle_by_index(n-1), GRETL_OBJ_BUNDLE, 
-			     ICON_ADD_SINGLE);
+	if (obj != NULL) {
+	    obj->data = bundle;
+	} else {
+	    session_add_icon(bundle, GRETL_OBJ_BUNDLE, ICON_ADD_SINGLE);
 	}
     } else if (autoicon_on()) {
 	view_session();
@@ -2718,6 +2779,25 @@ static void add_bundle_callback (void)
     mark_session_changed();
 }
 
+int have_session_objects (void)
+{
+    int n = data_status;
+
+    /* Note: the following lines enable the icon view even when
+       there's no dataset present.
+    */
+
+    if (n == 0) {
+	n = session.ngraphs + session.ntexts;
+    }
+
+    if (n == 0) {
+	n = n_user_matrices() + n_user_bundles() + n_saved_scalars();
+    }
+
+    return n > 0;
+}
+
 static void add_all_icons (void) 
 {
     int show_graph_page = check_for_prog(latex);
@@ -2728,14 +2808,16 @@ static void add_all_icons (void)
     if (data_status) {
 	session_add_icon(NULL, GRETL_OBJ_INFO,    ICON_ADD_BATCH);  /* data info */
 	session_add_icon(NULL, GRETL_OBJ_DSET,    ICON_ADD_BATCH);  /* data file */
-	session_add_icon(NULL, GRETL_OBJ_SCALARS, ICON_ADD_BATCH);  /* scalars */
-	session_add_icon(NULL, GRETL_OBJ_NOTES,   ICON_ADD_BATCH);  /* session notes */
 	session_add_icon(NULL, GRETL_OBJ_STATS,   ICON_ADD_BATCH);  /* summary stats */
 	session_add_icon(NULL, GRETL_OBJ_CORR,    ICON_ADD_BATCH);  /* correlation matrix */
 	session_add_icon(NULL, GRETL_OBJ_MODTAB,  ICON_ADD_BATCH);  /* model table */
-	if (show_graph_page) {
-	    session_add_icon(NULL, GRETL_OBJ_GPAGE, ICON_ADD_BATCH); /* graph page */
-	}
+    }
+
+    /* standard icons that don't really require a dataset in place */
+    session_add_icon(NULL, GRETL_OBJ_SCALARS, ICON_ADD_BATCH);  /* scalars */ 
+    session_add_icon(NULL, GRETL_OBJ_NOTES,   ICON_ADD_BATCH);  /* session notes */
+    if (show_graph_page) {
+	session_add_icon(NULL, GRETL_OBJ_GPAGE, ICON_ADD_BATCH); /* graph page */
     }
 
     for (i=0; i<session.nmodels; i++) {
@@ -2877,11 +2959,17 @@ static gboolean session_view_click (GtkWidget *widget,
 	return FALSE;
     }
 
-    if (!in_icon) { 
-	/* click on window background */
-	gtk_menu_popup(GTK_MENU(global_popup), NULL, NULL, NULL, NULL,
-		       event->button, event->time);
-	return TRUE;
+    mods = widget_get_pointer_mask(widget);
+
+    if (!in_icon) {
+	if (RIGHT_CLICK(mods)) {
+	    /* right-click on window background */
+	    gtk_menu_popup(GTK_MENU(global_popup), NULL, NULL, NULL, NULL,
+			   event->button, event->time);
+	    return TRUE;
+	} else {
+	    return FALSE;
+	}
     }
 
     obj = (gui_obj *) data;
@@ -2922,7 +3010,7 @@ static gboolean session_view_click (GtkWidget *widget,
 	    display_model_table_wrapper(); 
 	    break;
 	case GRETL_OBJ_GPAGE:
-	    display_graph_page(); 
+	    display_graph_page(iconview); 
 	    break;
 	case GRETL_OBJ_CORR:
 	    do_menu_op(ALL_CORR, NULL, OPT_NONE); 
@@ -2934,8 +3022,6 @@ static gboolean session_view_click (GtkWidget *widget,
 	return TRUE;
     }
 
-    mods = widget_get_pointer_mask(widget);
-
     if (RIGHT_CLICK(mods)) {
 	if (obj->sort == GRETL_OBJ_EQN  || obj->sort == GRETL_OBJ_GRAPH || 
 	    obj->sort == GRETL_OBJ_TEXT || obj->sort == GRETL_OBJ_DSET || 
@@ -2968,6 +3054,8 @@ static void global_popup_callback (GtkWidget *widget, gpointer data)
 	}
     } else if (!strcmp(item, _("Arrange icons"))) {
 	rearrange_icons();
+    } else if (!strcmp(item, _("Windows"))) {
+	window_list_popup(widget, NULL, iconview);
     } else if (!strcmp(item, _("Close window"))) {
 	gtk_widget_destroy(iconview);
     }
@@ -3001,7 +3089,7 @@ static void matrix_popup_callback (GtkWidget *widget, gpointer data)
 	view_matrix_properties(m, name);
     } else if (!strcmp(item, _("Copy as CSV..."))) {
 	m = user_matrix_get_matrix(u);
-	matrix_to_clipboard_as_csv(m);
+	matrix_to_clipboard_as_csv(m, iconview);
     } else if (!strcmp(item, _("Rename"))) {
 	rename_object_dialog(obj);
     } else if (!strcmp(item, _("Delete"))) {
@@ -3032,7 +3120,7 @@ static void data_popup_callback (GtkWidget *widget, gpointer data)
     } else if (!strcmp(item, _("Export as CSV..."))) {
 	file_save(mdata, EXPORT_CSV);
     } else if (!strcmp(item, _("Copy as CSV..."))) {
-	csv_to_clipboard();
+	csv_to_clipboard(iconview);
     }
 }
 
@@ -3043,7 +3131,7 @@ static void scalars_popup_callback (GtkWidget *widget, gpointer data)
     if (!strcmp(item, _("Edit"))) {
 	edit_scalars();
     } else if (!strcmp(item, _("Copy as CSV..."))) {
-	scalars_to_clipboard_as_csv();
+	scalars_to_clipboard_as_csv(iconview);
     }
 }
 
@@ -3073,7 +3161,7 @@ static void object_popup_callback (GtkWidget *widget, gpointer data)
 	} else if (obj->sort == GRETL_OBJ_MODTAB) {
 	    display_model_table_wrapper();
 	} else if (obj->sort == GRETL_OBJ_GPAGE) {
-	    display_graph_page();
+	    display_graph_page(iconview);
 	} else if (obj->sort == GRETL_OBJ_GRAPH || 
 		   obj->sort == GRETL_OBJ_PLOT) {
 	    open_gui_graph(obj);
@@ -3121,9 +3209,9 @@ static void object_popup_callback (GtkWidget *widget, gpointer data)
 	}
     } else if (!strcmp(item, _("Help"))) {
 	if (obj->sort == GRETL_OBJ_MODTAB) {
-	    context_help(NULL, GINT_TO_POINTER(MODELTAB));
+	    show_gui_help(MODELTAB);
 	} else if (obj->sort == GRETL_OBJ_GPAGE) {
-	    context_help(NULL, GINT_TO_POINTER(GRAPHPG));
+	    show_gui_help(GRAPHPG);
 	}
     } else if (!strcmp(item, _("Save as TeX..."))) {   
 	if (obj->sort == GRETL_OBJ_GPAGE) {
@@ -3551,7 +3639,7 @@ void view_session (void)
     gtk_widget_show(hbox);
     gtk_widget_show(iconview);
 
-    add_window_list_item(iconview, OPEN_SESSION);
+    window_list_add(iconview, OPEN_SESSION);
 
     gtk_container_foreach(GTK_CONTAINER(scroller), 
 			  (GtkCallback) white_bg_style, 
@@ -3830,7 +3918,7 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
 
  done:
 
-    title = gretl_window_title(name, ("properties"));
+    title = gretl_window_title(name);
     view_buffer(prn, 78, 400, title, PRINT, NULL);
     g_free(title);
 }
