@@ -57,6 +57,10 @@ int get_commands_recorded (void);
 
 void session_menu_state (gboolean s);
 
+int have_session_objects (void);
+
+int widget_is_iconview (GtkWidget *w);
+
 const char *get_session_dirname (void);
 
 int real_add_text_to_session (PRN *prn, const char *tname);
@@ -82,9 +86,9 @@ void bundle_add_as_icon (GtkAction *action, gpointer p);
 
 void *get_session_object_by_name (const char *name, GretlObjType *type);
 
-int session_matrix_destroy_by_name (const char *name, PRN *prn);
+int session_matrix_destroy_by_name (const char *name);
 
-int session_bundle_destroy_by_name (const char *name, PRN *prn);
+int session_bundle_destroy_by_name (const char *name);
 
 void delete_text_from_session (void *p);
 
