@@ -345,7 +345,8 @@ static int restore_session_models (xmlNodePtr node, xmlDocPtr doc)
 /* peek inside the session file and retrieve the name of the
    data file, only */
 
-static int get_session_datafile_name (const char *fname, struct sample_info *sinfo)
+static int get_session_datafile_name (const char *fname, struct sample_info *sinfo,
+				      int *nodata)
 {
     xmlDocPtr doc = NULL;
     xmlNodePtr cur = NULL;
@@ -361,8 +362,12 @@ static int get_session_datafile_name (const char *fname, struct sample_info *sin
     /* read datafile attribute, if present */
     tmp = xmlGetProp(cur, (XUC) "datafile");
     if (tmp != NULL) {
-	strcpy(sinfo->datafile, (char *) tmp);
-	my_filename_from_utf8(sinfo->datafile);
+	if (!strcmp((const char *) tmp, "none")) {
+	    *nodata = 1;
+	} else {
+	    strcpy(sinfo->datafile, (char *) tmp);
+	    my_filename_from_utf8(sinfo->datafile);
+	}
 	free(tmp);
     }
 
@@ -737,11 +742,13 @@ static int write_session_xml (const char *datname)
 		trname, print_today());
 	g_free(trname);
     } else {
-	fputs("<gretl-session>\n", fp);
+	fprintf(fp, "<gretl-session date=\"%s\">\n", print_today());
     }
 
-    fprintf(fp, " <sample t1=\"%d\" t2=\"%d\"/>\n", dataset->t1, dataset->t2);
-    write_datainfo_submask(dataset, fp);
+    if (data_status) {
+	fprintf(fp, " <sample t1=\"%d\" t2=\"%d\"/>\n", dataset->t1, dataset->t2);
+	write_datainfo_submask(dataset, fp);
+    }
 
     nmodels = session.nmodels;
     tabmodels = model_table_n_models();
@@ -845,6 +852,7 @@ static int write_session_xml (const char *datname)
     fputs(" </models>\n", fp);
 
     fprintf(fp, " <graphs count=\"%d\">\n", session.ngraphs);
+
     for (i=0; i<session.ngraphs; i++) {
 	objname = session.graphs[i]->name;
 	xmlname = get_xmlname(objname, &err);
@@ -865,6 +873,7 @@ static int write_session_xml (const char *datname)
     fputs(" </graphs>\n", fp);
 
     fprintf(fp, " <texts count=\"%d\">\n", session.ntexts);
+
     for (i=0; i<session.ntexts; i++) {
 	objname = session.texts[i]->name;
 	xmlname = get_xmlname(objname, &err);
