@@ -27,6 +27,7 @@
 #include "session.h"
 #include "textbuf.h"
 #include "ssheet.h"
+#include "selector.h"
 
 #include "libset.h"
 #include "texprint.h"
@@ -35,6 +36,12 @@
 #include <sys/types.h>
 #include <dirent.h>
 
+#if GTK_MAJOR_VERSION == 3 && GTK_MINOR_VERSION >= 2
+# define USE_GTK_FONT_CHOOSER 1
+#else
+# define USE_GTK_FONT_CHOOSER 0
+#endif
+
 #ifdef G_OS_WIN32
 # include <windows.h>
 # include "gretlwin32.h"
@@ -42,7 +49,11 @@
 # include <sys/stat.h>
 # include <fcntl.h>
 # include <errno.h>
-# include "gtkfontselhack.h"
+# if USE_GTK_FONT_CHOOSER
+#  include "fontfilter.h"
+# else
+#  include "gtkfontselhack.h"
+# endif
 #endif
 
 static char rcfile[FILENAME_MAX];
@@ -79,6 +90,8 @@ static int manpref;
 static int autoicon = 1;
 static int session_prompt = 1;
 static int keep_folder = 1;
+static int tabbed_editor;
+static int tabbed_models;
 char gpcolors[64];
 static char datapage[24] = "Gretl";
 static char scriptpage[24] = "Gretl";
@@ -159,6 +172,10 @@ RCVAR rc_vars[] = {
       BOOLSET, 0, TAB_MAIN, NULL },
     { "autoicon", N_("Show icon view automatically"), NULL, &autoicon, 
       BOOLSET, 0, TAB_MAIN, NULL },
+    { "tabedit", N_("Script editor uses tabs"), NULL, &tabbed_editor, 
+      BOOLSET, 0, TAB_MAIN, NULL },
+    { "tabmodels", N_("Model viewer uses tabs"), NULL, &tabbed_models, 
+      BOOLSET, 0, TAB_MAIN, NULL },
     { "session_prompt", N_("Prompt to save session"), NULL, &session_prompt, 
       BOOLSET, 0, TAB_MAIN, NULL },
     { "oxsupport", N_("Enable Ox support"), NULL, &ox_support, 
@@ -179,7 +196,7 @@ RCVAR rc_vars[] = {
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
 #ifdef G_OS_WIN32 
     { "Rbin", "Path to Rterm.exe", NULL, paths.rbinpath, 
-      MACHSET | INVISET, MAXSTR, TAB_NONE, NULL },
+      MACHSET | BROWSER, MAXSTR, TAB_NONE, NULL },
 #endif
     { "latex", N_("Command to compile TeX files"), NULL, latex, 
       MACHSET | BROWSER, MAXSTR, TAB_PROGS, NULL },
@@ -301,6 +318,16 @@ int autoicon_on (void)
     }
 }
 
+int use_tabbed_editor (void)
+{
+    return tabbed_editor;
+}
+
+int use_tabbed_model_viewer (void)
+{
+    return tabbed_models;
+}
+
 int session_prompt_on (void)
 {
     return session_prompt;
@@ -429,13 +456,7 @@ static int write_OK (gchar *dirname)
 	gchar  *target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
 
 	if (target != NULL) {
-	    FILE *fp = gretl_try_fopen(target, "w");
-
-	    if (fp != NULL) {
-		ok = 1;
-		fclose(fp);
-		gretl_remove(target);
-	    }
+	    ok = (gretl_test_fopen(target, "w") == 0);
 	    g_free(target);
 	}
     }
@@ -515,6 +536,8 @@ void get_default_dir (char *s, int action)
 
     if (action == SAVE_FUNCTIONS || action == SAVE_DATA_PKG) {
 	get_pkg_save_dir(s, action);
+    } else if (action == SAVE_REMOTE_DB) {
+	sprintf(s, "%sdb", gretl_home());
     } else if (action == OPEN_RATS_DB) {
 	strcpy(s, gretl_ratsbase());
     } else {
@@ -838,6 +861,14 @@ static void option_dialog_canceled (GtkWidget *w, int *c)
     *c = 1;
 }
 
+static void sensitize_prefs_help (GtkNotebook *book,
+				  gpointer arg1,
+				  guint newpg,
+				  GtkWidget *button)
+{
+    gtk_widget_set_sensitive(button, newpg == TAB_VCV - 1);
+} 
+
 int options_dialog (int page, const char *varname, GtkWidget *parent) 
 {
     static GtkWidget *dialog;
@@ -901,6 +932,14 @@ int options_dialog (int page, const char *varname, GtkWidget *parent)
 		     dialog);
     gtk_widget_show(button);
 
+    /* Help button */
+    button = context_help_button(hbox, HCCME);
+    gtk_widget_show(button);
+    gtk_widget_set_sensitive(button, page == TAB_VCV);
+    g_signal_connect(G_OBJECT(notebook), "switch-page",
+		     G_CALLBACK(sensitize_prefs_help),
+		     button);
+
     if (page > 1 && page < TAB_MAX) {
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), page - 1);
     }
@@ -1013,11 +1052,12 @@ static const char **get_radio_setting_strings (void *var, int *n)
     return strs;
 }
 
+static const char *hc_strs[] = {
+    "HC0", "HC1", "HC2", "HC3", "HC3a", "HAC"
+};
+
 static const char **get_list_setting_strings (void *var, int *n)
 {
-    static const char *hc_strs[] = {
-	"HC0", "HC1", "HC2", "HC3", "HC3a", "HAC"
-    };
     static const char *hc_panel_strs[] = {
 	"Arellano", "PCSE"
     };
@@ -1043,6 +1083,36 @@ static const char **get_list_setting_strings (void *var, int *n)
     return strs;
 }
 
+const char *get_default_hc_string (int ci)
+{
+    if (ci == GARCH) {
+	int k = libset_get_int(GARCH_ROBUST_VCV);
+
+	return (k == ML_BW)? "BW" : "QML";
+    } else if (!robust_conf(ci)) {
+	return "QML";
+    } else {
+	int xsect = dataset_is_cross_section(dataset);
+	int tseries = dataset_is_time_series(dataset);
+  
+	if (tseries && libset_get_bool(FORCE_HC)) {
+	    xsect = 1;
+	} else if (ci == VAR) {
+	    xsect = 1;
+	}
+
+	if (xsect) {
+	    return hc_strs[libset_get_int(HC_VERSION)];
+	} else if (tseries) {
+	    /* (and not forced to an HC variant) */
+	    return "HAC";
+	} else {
+	    /* panel */
+	    return libset_get_bool(PCSE) ? "PCSE" : "Arellano";
+	}
+    }
+}
+
 static void 
 get_table_sizes (int page, int *n_str, int *n_bool, int *n_browse,
 		 int *n_list)
@@ -1396,79 +1466,6 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    }
 	} 
     }
-
-#if 0 /* not for now */
-    if (tab == TAB_MAIN) {
-	/* font selector buttons, working directory */
-	int add_appfont = 1;
-	GtkWidget *hb, *im;
-
-#ifdef G_OS_WIN32
-	if (use_wimp) {
-	    add_appfont = 0;
-	}
-#endif
-
-	hb = gtk_hbox_new(FALSE, 0);
-	w = gtk_button_new();
-	gtk_button_set_label(GTK_BUTTON(w), _("Fixed font..."));
-	im = gtk_image_new_from_stock(GTK_STOCK_SELECT_FONT, 
-				      GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image(GTK_BUTTON(w), im);
-	gtk_box_pack_start(GTK_BOX(hb), w, FALSE, FALSE, 0);
-	g_signal_connect(G_OBJECT(w), "clicked", 
-			 G_CALLBACK(font_selector), 
-			 GINT_TO_POINTER(FIXED_FONT_SELECTION));
-	gtk_box_pack_start(GTK_BOX(box), hb, FALSE, FALSE, 10);
-	gtk_widget_show_all(hb);
-
-	if (add_appfont) {
-	    hb = gtk_hbox_new(FALSE, 0);
-	    w = gtk_button_new();
-	    gtk_button_set_label(GTK_BUTTON(w), _("Menu font..."));
-	    im = gtk_image_new_from_stock(GTK_STOCK_SELECT_FONT, 
-					  GTK_ICON_SIZE_BUTTON);
-	    gtk_button_set_image(GTK_BUTTON(w), im);
-	    gtk_box_pack_start(GTK_BOX(hb), w, FALSE, FALSE, 0);
-	    g_signal_connect(G_OBJECT(w), "clicked", 
-			     G_CALLBACK(font_selector), 
-			     GINT_TO_POINTER(APP_FONT_SELECTION));
-	    gtk_box_pack_start(GTK_BOX(box), hb, FALSE, FALSE, 0);
-	    gtk_widget_show_all(hb);
-	}
-
-	hb = gtk_hbox_new(FALSE, 0);
-	w = gtk_button_new();
-	gtk_button_set_label(GTK_BUTTON(w), _("Working directory..."));
-	im = gtk_image_new_from_stock(GTK_STOCK_DIRECTORY, 
-				      GTK_ICON_SIZE_BUTTON);
-	gtk_button_set_image(GTK_BUTTON(w), im);
-	gtk_box_pack_start(GTK_BOX(hb), w, FALSE, FALSE, 0);
-	g_signal_connect(G_OBJECT(w), "clicked", 
-			 G_CALLBACK(working_dir_dialog), NULL);
-	gtk_box_pack_start(GTK_BOX(box), hb, FALSE, FALSE, 10);
-	gtk_widget_show_all(hb);
-    }
-#endif	
-
-    if (tab == TAB_VCV) {
-	/* we need a help button */
-	GtkWidget *hb = gtk_hbox_new(FALSE, 0);
-
-	w = gtk_label_new("");
-	gtk_box_pack_start(GTK_BOX(hb), w, TRUE, TRUE, 0);
-	gtk_widget_show(w);
-
-	w = gtk_button_new_from_stock(GTK_STOCK_HELP);
-	gtk_box_pack_start(GTK_BOX(hb), w, FALSE, FALSE, 0);
-	g_signal_connect(G_OBJECT(w), "clicked", 
-			 G_CALLBACK(context_help), 
-			 GINT_TO_POINTER(HCCME));
-	gtk_widget_show(w);
-
-	gtk_box_pack_start(GTK_BOX(box), hb, FALSE, FALSE, 0);
-	gtk_widget_show(hb);
-    }
 }
 
 static void set_gp_colors (void)
@@ -1612,13 +1609,17 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 
     write_rc(); /* note: calls gretl_update_paths */
 
-    /* register these for session using libset apparatus */
+    /* register these settings for the current session using 
+       the "libset" apparatus 
+    */
     libset_set_bool(SHELL_OK, shellok);
     set_xsect_hccme(hc_xsect);
     set_tseries_hccme(hc_tseri);
     set_panel_hccme(hc_panel);
     set_garch_robust_vcv(hc_garch);
 
+    selector_register_hc_choice();
+
     gretl_www_init(paths.dbhost, dbproxy, use_proxy);
 }
 
@@ -2097,7 +2098,7 @@ static int fontsel_code (GtkAction *action)
 
 /* font selection: Windows version first */
 
-#ifdef G_OS_WIN32
+#if defined(G_OS_WIN32)
 
 void font_selector (GtkAction *action)
 {
@@ -2124,7 +2125,122 @@ void font_selector (GtkAction *action)
     }
 }
 
-#else /* not MS Windows */
+/* font selection via GtkFontChooser */
+
+#elif USE_GTK_FONT_CHOOSER
+
+gboolean latin_font_filter (PangoFontFamily *family,
+			    PangoFontFace *face,
+			    gpointer data)
+{
+    const char *facename = pango_font_face_get_face_name(face);
+
+    if (strstr(facename, "Italic") || strstr(facename, "Bold")) {
+	return FALSE;
+    } else {
+	return validate_single_font(family, FONT_FILTER_LATIN);
+    }
+}
+
+gboolean mono_font_filter (PangoFontFamily *family,
+			   PangoFontFace *face,
+			   gpointer data)
+{
+    const char *facename = pango_font_face_get_face_name(face);
+
+    if (strstr(facename, "Italic") || strstr(facename, "Bold") ||
+	strstr(facename, "Oblique")) {
+	return FALSE;
+    } else {
+	return validate_single_font(family, FONT_FILTER_LATIN_MONO);
+    }
+}
+
+static void close_font_chooser (GtkWidget *w, GtkFontChooser *fc)
+{
+    gretl_font_filter_cleanup();
+    gtk_widget_destroy(GTK_WIDGET(fc));
+}
+
+static void font_selection_ok (GtkWidget *w, GtkFontChooser *fc)
+{
+    gchar *fontname = gtk_font_chooser_get_font(fc);
+
+    if (fontname != NULL && *fontname != '\0') {
+	int mono = widget_get_int(fc, "mono");
+
+	if (mono) {
+	    strcpy(fixedfontname, fontname);
+	    set_fixed_font();
+	    write_rc();
+	} else {
+	    set_app_font(fontname);
+	    write_rc();
+	}
+    }
+
+    g_free(fontname);
+    gretl_font_filter_cleanup();
+    gtk_widget_destroy(GTK_WIDGET(fc));
+}
+
+void font_selector (GtkAction *action)
+{
+    static GtkWidget *fontsel = NULL;
+    GtkFontFilterFunc filter;
+    GtkWidget *button;
+    int which = fontsel_code(action);
+    char *title = NULL;
+    const char *fontname = NULL;
+ 
+    if (fontsel != NULL) {
+	gtk_window_present(GTK_WINDOW(fontsel));
+        return;
+    }
+
+    if (which == FIXED_FONT_SELECTION) {
+	title = _("Font for gretl output windows");
+	filter = (GtkFontFilterFunc) &mono_font_filter;
+	fontname = fixedfontname;
+    } else if (which == APP_FONT_SELECTION) {
+	title = _("Font for menus and labels");
+	filter = (GtkFontFilterFunc) &latin_font_filter;
+	fontname = appfontname;
+    }
+
+    gretl_font_filter_init();
+
+    fontsel = gtk_font_chooser_dialog_new(title, GTK_WINDOW(mdata->main));
+    gtk_font_chooser_set_font(GTK_FONT_CHOOSER(fontsel), 
+			      fontname); 
+    gtk_font_chooser_set_filter_func(GTK_FONT_CHOOSER(fontsel),
+				     filter, NULL, NULL);
+    gtk_window_set_position(GTK_WINDOW(fontsel), GTK_WIN_POS_MOUSE);
+
+    g_signal_connect(G_OBJECT(fontsel), "destroy",
+		     G_CALLBACK(gtk_widget_destroyed),
+		     &fontsel);
+
+    button = gtk_dialog_get_widget_for_response(GTK_DIALOG(fontsel),
+						GTK_RESPONSE_OK);
+    if (button != NULL) {
+	g_signal_connect(G_OBJECT(button), "clicked",
+			 G_CALLBACK(font_selection_ok),
+			 fontsel);
+    }
+
+    button = gtk_dialog_get_widget_for_response(GTK_DIALOG(fontsel),
+						GTK_RESPONSE_CANCEL);
+    if (button != NULL) {
+	g_signal_connect(G_OBJECT(button), "clicked",
+			 G_CALLBACK(close_font_chooser),
+			 fontsel);
+    }
+
+    gtk_widget_show(fontsel);
+}
+
+#else /* GTK, not using GtkFontChooser */
 
 static void font_selection_ok (GtkWidget *w, GtkFontselHackDialog *fs)
 {
@@ -2135,11 +2251,11 @@ static void font_selection_ok (GtkWidget *w, GtkFontselHackDialog *fs)
     if (fontname != NULL && *fontname != '\0') {
 	int filter = gtk_fontsel_hack_dialog_get_filter(fs); 
 
-	if (filter == GTK_FONT_HACK_LATIN_MONO) {
+	if (filter == FONT_HACK_LATIN_MONO) {
 	    strcpy(fixedfontname, fontname);
 	    set_fixed_font();
 	    write_rc();
-	} else if (filter == GTK_FONT_HACK_LATIN) {
+	} else if (filter == FONT_HACK_LATIN) {
 	    set_app_font(fontname);
 	    write_rc();
 	}
@@ -2163,11 +2279,11 @@ void font_selector (GtkAction *action)
 
     if (which == FIXED_FONT_SELECTION) {
 	title = _("Font for gretl output windows");
-	filter = GTK_FONT_HACK_LATIN_MONO;
+	filter = FONT_HACK_LATIN_MONO;
 	fontname = fixedfontname;
     } else if (which == APP_FONT_SELECTION) {
 	title = _("Font for menus and labels");
-	filter = GTK_FONT_HACK_LATIN;
+	filter = FONT_HACK_LATIN;
 	fontname = appfontname;
     }
 
