@@ -23,6 +23,10 @@ int get_manpref (void);
 
 int autoicon_on (void);
 
+int use_tabbed_editor (void);
+
+int use_tabbed_model_viewer (void);
+
 int session_prompt_on (void);
 
 void set_session_prompt (int val);
@@ -67,6 +71,8 @@ const char *get_datapage (void);
 
 const char *get_scriptpage (void);
 
+const char *get_default_hc_string (int ci);
+
 int check_for_prog (const char *prog);
 
 #endif /* SETTINGS_H */
