@@ -78,6 +78,7 @@ typedef struct {
     gretl_matrix *oldmat;
     char mname[VNAMELEN];
     const char **colnames;
+    const char **rownames;
     int *varlist;
     int datacols, datarows;
     int totcols;
@@ -89,9 +90,16 @@ typedef struct {
     SheetFlags flags;
     guint cid;
     guint point;
+    int *edits;
+    int *inserts;
 } Spreadsheet;
 
+#define editing_series(s) (s->cmd == SHEET_EDIT_VARLIST || \
+                           s->cmd == SHEET_EDIT_DATASET || \
+                           s->cmd == SHEET_NEW_DATASET)
+
 #define MATRIX_DIGITS DBL_DIG
+#define SERIES_DIGITS 8
 
 static void set_up_sheet_column (GtkTreeViewColumn *column, gint width, 
 				 gboolean expand);
@@ -110,6 +118,7 @@ static void sheet_show_popup (GtkWidget *w, Spreadsheet *sheet);
 static void get_data_from_sheet (GtkWidget *w, Spreadsheet *sheet);
 static gint maybe_exit_sheet (GtkWidget *w, Spreadsheet *sheet);
 
+static void add_scalar_callback (GtkWidget *w, Spreadsheet *sheet);
 static void update_scalars_from_sheet (Spreadsheet *sheet);
 static void scalars_changed_callback (void);
 
@@ -117,17 +126,25 @@ static void
 spreadsheet_scroll_to_foot (Spreadsheet *sheet, int row, int col);
 
 enum {
-    SERIES_ADD_BTN,
-    SERIES_APPLY_BTN
+    SHEET_ADD_BTN,
+    SHEET_APPLY_BTN
 };
 
 static GretlToolItem series_items[] = {
-    { N_("Add..."), GTK_STOCK_ADD,    G_CALLBACK(sheet_show_popup),    SERIES_ADD_BTN },
-    { N_("Apply"),  GTK_STOCK_APPLY,  G_CALLBACK(get_data_from_sheet), SERIES_APPLY_BTN },
+    { N_("Add..."), GTK_STOCK_ADD,    G_CALLBACK(sheet_show_popup),    SHEET_ADD_BTN },
+    { N_("Apply"),  GTK_STOCK_APPLY,  G_CALLBACK(get_data_from_sheet), SHEET_APPLY_BTN },
+    { N_("Windows"), GRETL_STOCK_COMPASS, GNULL, 0 },
     { N_("Close"),  GTK_STOCK_CLOSE,  G_CALLBACK(maybe_exit_sheet),    0 }
 };
 
+static GretlToolItem scalar_items[] = {
+    { N_("Add..."),  GTK_STOCK_ADD,    G_CALLBACK(add_scalar_callback), 0 },
+    { N_("Windows"), GRETL_STOCK_COMPASS, GNULL, 0 },
+    { N_("Close"),   GTK_STOCK_CLOSE,  G_CALLBACK(maybe_exit_sheet), 0 }
+};
+
 static int n_series_items = G_N_ELEMENTS(series_items);
+static int n_scalar_items = G_N_ELEMENTS(scalar_items);
 
 const gchar *matrix_ui = 
     "<ui>"
@@ -179,6 +196,10 @@ static void sheet_set_modified (Spreadsheet *sheet, gboolean s)
 	sheet->flags |= SHEET_MODIFIED;
     } else {
 	sheet->flags &= ~SHEET_MODIFIED;
+	free(sheet->edits);
+	sheet->edits = NULL;
+	free(sheet->inserts);
+	sheet->inserts = NULL;
     }
 
     if (sheet->save != NULL) {
@@ -188,6 +209,90 @@ static void sheet_set_modified (Spreadsheet *sheet, gboolean s)
     }
 }
 
+/* record the fact that the value at @col and @row as been
+   changed */
+
+static int sheet_push_edit (Spreadsheet *sheet, int col, int row)
+{
+    int n_edits = (sheet->edits != NULL)? sheet->edits[0] : 0;
+    int *ed = realloc(sheet->edits, (n_edits + 3) * sizeof *ed);
+
+    if (ed == NULL) {
+	nomem();
+	return E_ALLOC;
+    } else {
+	sheet->edits = ed;
+	n_edits += 2;
+	sheet->edits[0] = n_edits;
+	sheet->edits[n_edits-1] = col;
+	sheet->edits[n_edits] = row;
+	sheet_set_modified(sheet, TRUE);
+	return 0;
+    }
+}
+
+/* record the fact that a new row (observation) has been inserted
+   at @row */
+
+static int sheet_push_insert (Spreadsheet *sheet, int row)
+{
+    int *test = gretl_list_append_term(&sheet->inserts, row);
+
+    if (test == NULL) {
+	nomem();
+	sheet->inserts = NULL;
+	return E_ALLOC;
+    } else {
+	sheet_set_modified(sheet, TRUE);
+	return 0;
+    }
+}
+
+static void sheet_revise_edit_rows (Spreadsheet *sheet, int new_row)
+{
+    int n_edits = (sheet->edits != NULL)? sheet->edits[0] : 0;
+
+    if (n_edits > 0) {
+	int i;
+
+	for (i=2; i<=n_edits; i+=2) {
+	    if (sheet->edits[i] >= new_row) {
+		sheet->edits[i] += 1;
+	    }
+	}
+    }
+}
+
+static int data_column_edited (Spreadsheet *sheet, int col)
+{
+    if (sheet->edits != NULL) {
+	int j;
+
+	for (j=1; j<sheet->edits[0]; j+=2) {
+	    if (sheet->edits[j] == col) {
+		return 1;
+	    }
+	}
+    }
+
+    return 0;
+}
+
+static int data_cell_edited (Spreadsheet *sheet, int col, int row)
+{
+    if (sheet->edits != NULL) {
+	int j;
+
+	for (j=1; j<sheet->edits[0]; j+=2) {
+	    if (sheet->edits[j] == col && sheet->edits[j+1] == row) {
+		return 1;
+	    }
+	}
+    }
+
+    return 0;
+}
+
 static char *single_underscores (char *targ, const char *src)
 {
     char *p = targ;
@@ -210,25 +315,29 @@ static char *single_underscores (char *targ, const char *src)
 static void set_locator_label (Spreadsheet *sheet, GtkTreePath *path,
 			       GtkTreeViewColumn *column)
 {
-    GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(sheet->view));
-    GtkTreeIter iter;
-    gchar *rstr;
-    const gchar *cstr;
-    char tmp[VNAMELEN];
-
-    gtk_tree_model_get_iter(model, &iter, path);
-    gtk_tree_model_get(model, &iter, 0, &rstr, -1);
-    cstr = gtk_tree_view_column_get_title(column);
     if (sheet->matrix != NULL) {
-	sprintf(sheet->location, "%s, %s", rstr, cstr);
+	gchar *pstr = gtk_tree_path_to_string(path);
+	gint cnum = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(column), "colnum"));
+
+	sprintf(sheet->location, "%d, %d", atoi(pstr) + 1, cnum);
+	g_free(pstr);
     } else {
+	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(sheet->view));
+	const gchar *cstr = gtk_tree_view_column_get_title(column);
+	GtkTreeIter iter;
+	char tmp[VNAMELEN];
+	gchar *rstr;
+
 	single_underscores(tmp, cstr);
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_model_get(model, &iter, 0, &rstr, -1);
 	sprintf(sheet->location, "%s, %s", tmp, rstr);
+	g_free(rstr);
     }
+
     gtk_statusbar_pop(GTK_STATUSBAR(sheet->locator), sheet->cid);
     gtk_statusbar_push(GTK_STATUSBAR(sheet->locator), 
 		       sheet->cid, sheet->location);
-    g_free(rstr);
 }
 
 static void set_treeview_column_number (GtkTreeViewColumn *col, int j)
@@ -349,7 +458,7 @@ static void update_sheet_matrix (Spreadsheet *sheet)
 
     for (i=0; i<sheet->datarows; i++) {
 	for (j=0; j<sheet->datacols; j++) {
-	    gtk_tree_model_get(model, &iter, j+1, &numstr, -1);
+	    gtk_tree_model_get(model, &iter, j + 1, &numstr, -1);
 	    x = atof(numstr); 
 	    gretl_matrix_set(sheet->matrix, i, j, x);
 	    g_free(numstr);
@@ -362,36 +471,32 @@ static void maybe_update_column_names (Spreadsheet *sheet)
 {
     if (sheet->flags & SHEET_COLNAME_MOD) {
 	gretl_matrix *M = sheet->oldmat;
-	GtkTreeViewColumn *col;
-	const char *title;
-	char *cnames, tmp[13];
-	int j, err = 0;
+	char **cnames;
 
-	cnames = malloc(M->cols * 13 + 1);
+	cnames = strings_array_new_with_length(M->cols, 13);
 
 	if (cnames == NULL) {
-	    err = 1;
+	    nomem();
 	} else {
-	    *cnames = '\0';
+	    GtkTreeViewColumn *col;
+	    const char *title;
+	    int j, err = 0;
+
 	    for (j=0; j<M->cols && !err; j++) {
-		title = NULL;
-		col = gtk_tree_view_get_column(GTK_TREE_VIEW(sheet->view), j + 1);
+		col = gtk_tree_view_get_column(GTK_TREE_VIEW(sheet->view), 
+					       j + 1);
 		if (col != NULL) {
 		    title = gtk_tree_view_column_get_title(col);
-		}
-		if (title != NULL) {
-		    *tmp = '\0';
-		    single_underscores(tmp, title);
-		    strcat(cnames, tmp);
-		    strcat(cnames, " ");
-		} else {
-		    err = 1;
+		    if (title != NULL) {
+			single_underscores(cnames[j], title);
+		    } else {
+			err = 1;
+		    }
 		}
 	    }
 	    if (!err) {
-		umatrix_set_names_from_string(M, cnames, 0);
+		gretl_matrix_set_colnames(M, cnames);
 	    }
-	    free(cnames);
 	}
 
 	sheet->flags &= ~SHEET_COLNAME_MOD;
@@ -404,16 +509,18 @@ static void maybe_update_column_names (Spreadsheet *sheet)
 static void update_saved_matrix (Spreadsheet *sheet)
 {
     if (sheet->oldmat != NULL) {
-	if (sheet->oldmat->rows == sheet->matrix->rows &&
-	    sheet->oldmat->cols == sheet->matrix->cols) {
-	    gretl_matrix_copy_values(sheet->oldmat, sheet->matrix);
-	} else if (sheet->oldmat->rows * sheet->oldmat->cols ==
-		   sheet->matrix->rows * sheet->matrix->cols) {
-	    sheet->oldmat->rows = sheet->matrix->rows;
-	    sheet->oldmat->cols = sheet->matrix->cols;
-	    gretl_matrix_copy_values(sheet->oldmat, sheet->matrix);
+	gretl_matrix *targ = sheet->oldmat;
+	gretl_matrix *src = sheet->matrix;
+
+	if (targ->rows == src->rows && targ->cols == src->cols) {
+	    gretl_matrix_copy_values(targ, src);
+	} else if (targ->rows * targ->cols == src->rows * src->cols) {
+	    gretl_matrix_destroy_info(targ);
+	    targ->rows = src->rows;
+	    targ->cols = src->cols;
+	    gretl_matrix_copy_values(targ, src);
 	} else {
-	    gretl_matrix *m = gretl_matrix_copy(sheet->matrix);
+	    gretl_matrix *m = gretl_matrix_copy(src);
 
 	    if (m == NULL) {
 		nomem();
@@ -475,12 +582,11 @@ maybe_update_store (Spreadsheet *sheet, const gchar *new_text,
 
 	if (sheet->matrix != NULL) {
 	    update_sheet_matrix_element(sheet, new_text, path_string, colnum);
-	} 
-
-	if (editing_scalars(sheet)) {
+	    sheet_set_modified(sheet, TRUE);
+	} else if (editing_scalars(sheet)) {
 	    update_scalars_from_sheet(sheet);
 	} else {
-	    sheet_set_modified(sheet, TRUE);
+	    sheet_push_edit(sheet, colnum, atoi(path_string));
 	}
     }
 
@@ -773,14 +879,18 @@ real_add_new_obs (Spreadsheet *sheet, const char *obsname, int n)
     } else {
 	for (i=1; i<=sheet->datacols; i++) {
 	    gtk_list_store_set(store, &iter, i, "", -1);
+	    sheet_push_edit(sheet, i, rownum);
 	}
     }	
 
-    if (sheet->point == SHEET_AT_POINT && !dataset->markers) {
-	for (i=rownum; i<sheet->datarows; i++) {
-	    get_obs_string(rowlabel, i, dataset);
-	    gtk_list_store_set(store, &iter, 0, rowlabel, -1);
-	    gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+    if (sheet->point == SHEET_AT_POINT) {
+	sheet_revise_edit_rows(sheet, rownum);
+	if (!dataset->markers) {
+	    for (i=rownum; i<sheet->datarows; i++) {
+		get_obs_string(rowlabel, i, dataset);
+		gtk_list_store_set(store, &iter, 0, rowlabel, -1);
+		gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
+	    }
 	}
     } 
 
@@ -795,6 +905,7 @@ real_add_new_obs (Spreadsheet *sheet, const char *obsname, int n)
 	gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &insiter, path);
 	gtk_list_store_set(store, &insiter, 1, "", -1);
 	gtk_tree_path_free(path);
+	sheet_push_insert(sheet, rownum);
     }
 
     if (pstr != NULL) {
@@ -819,7 +930,7 @@ static void name_new_var (GtkWidget *widget, dialog_t *dlg)
     *varname = 0;
     strncat(varname, buf, VNAMELEN - 1);
 
-    close_dialog(dlg);
+    edit_dialog_close(dlg);
 
     if (real_add_new_series(sheet, varname)) {
 	nomem();
@@ -838,30 +949,26 @@ static void name_new_obs (GtkWidget *widget, dialog_t *dlg)
     *obsmarker = 0;
     strncat(obsmarker, buf, OBSLEN - 1);
 
-    close_dialog(dlg);
+    edit_dialog_close(dlg);
     real_add_new_obs(sheet, obsmarker, 1);
 }
 
 static void name_var_dialog (Spreadsheet *sheet) 
 {
-    int cancel = 0;
-    
-    edit_dialog(_("gretl: name variable"), 
+    edit_dialog(0, _("gretl: name variable"), 
 		_("Enter name for new variable\n"
-		  "(max. 15 characters)"),
-		NULL, name_new_var, sheet, 
-		0, VARCLICK_NONE, &cancel);
+		  "(max. 15 characters)"), NULL,
+		name_new_var, sheet, 
+		VARCLICK_NONE, sheet->win);
 }
 
 static void new_case_dialog (Spreadsheet *sheet) 
 {
-    int cancel = 0;
-
-    edit_dialog(_("gretl: case marker"), 
+    edit_dialog(0, _("gretl: case marker"), 
 		_("Enter case marker for new obs\n"
-		  "(max. 8 characters)"),
-		NULL, name_new_obs, sheet, 
-		0, VARCLICK_NONE, &cancel);
+		  "(max. 8 characters)"), NULL,
+		name_new_obs, sheet, 
+		VARCLICK_NONE, sheet->win);
 }
 
 static void name_matrix_col (GtkWidget *widget, dialog_t *dlg) 
@@ -879,7 +986,7 @@ static void name_matrix_col (GtkWidget *widget, dialog_t *dlg)
     strncat(tmp, buf, 12);
     double_underscores(colname, tmp);
 
-    close_dialog(dlg);
+    edit_dialog_close(dlg);
 
     old = gtk_tree_view_column_get_title(col);
 
@@ -892,16 +999,15 @@ static void name_matrix_col (GtkWidget *widget, dialog_t *dlg)
     }
 }
 
-static void name_column_dialog (GtkTreeViewColumn *col, gpointer p) 
+static void name_column_dialog (GtkTreeViewColumn *column, 
+				Spreadsheet *sheet) 
 {
-    int cancel = 0;
-
-    edit_dialog(_("gretl: name column"), 
+    edit_dialog(0, _("gretl: name column"), 
 		_("Enter name for column\n"
 		  "(max. 12 characters)"),
-		gtk_tree_view_column_get_title(col),
-		name_matrix_col, col, 
-		0, VARCLICK_NONE, &cancel);
+		gtk_tree_view_column_get_title(column),
+		name_matrix_col, column, 
+		VARCLICK_NONE, sheet->win);
 }
 
 static GtkListStore *make_sheet_liststore (Spreadsheet *sheet)
@@ -1002,16 +1108,18 @@ static int add_data_column (Spreadsheet *sheet)
 static void sheet_get_scalar (GtkWidget *w, dialog_t *dlg)
 {
     double x, *px = (double *) edit_dialog_get_data(dlg);
-    const gchar *buf;
+    const gchar *buf = edit_dialog_get_text(dlg);
     int err = 0;
 
-    buf = edit_dialog_get_text(dlg);
-    if (buf == NULL) return;
+    if (buf != NULL) {
+	x = gui_double_from_string(buf, &err);
+    }
 
-    x = gui_double_from_string(buf, &err);
-    if (!err) {
+    if (buf == NULL || err) {
+	edit_dialog_reset(dlg);
+    } else {
 	*px = x;
-	close_dialog(dlg);
+	edit_dialog_close(dlg);
     }
 }
 
@@ -1025,10 +1133,11 @@ static void matrix_edit_callback (GtkAction *action, gpointer data)
     if (!strcmp(s, "ScalarMult") || !strcmp(s, "ScalarDiv")) {
 	int cancel = 0;
 
-	edit_dialog(_("gretl: specify scalar"), 
-		    _("Enter a numerical value"),
-		    NULL, sheet_get_scalar, &x, 
-		    0, VARCLICK_NONE, &cancel);
+	blocking_edit_dialog(0, _("gretl: specify scalar"), 
+			     _("Enter a numerical value"),
+			     NULL, sheet_get_scalar, &x, 
+			     VARCLICK_NONE, sheet->win,
+			     &cancel);
 	if (cancel || na(x)) {
 	    return;
 	}
@@ -1086,7 +1195,7 @@ static void sheet_add_obs_direct (Spreadsheet *sheet)
     if (dataset->markers) {
 	new_case_dialog(sheet);
     } else if (sheet->point == SHEET_AT_END) {
-	int n = add_obs_dialog(NULL, 1);
+	int n = add_obs_dialog(NULL, 1, sheet->win);
 
 	if (n > 0) {
 	    real_add_new_obs(sheet, NULL, n);
@@ -1310,6 +1419,26 @@ static void update_scalars_from_sheet (Spreadsheet *sheet)
     } 
 }
 
+/* handle the case where observations have been inserted (not just
+   appended); we need to shift the original dataset observations
+   "down" by one place for each insertion
+*/
+
+static void process_inserts_for_series (Spreadsheet *sheet, int v)
+{
+    double *src;
+    int n = sheet->orig_nobs;
+    int i, t;
+
+    for (i=1; i<=sheet->inserts[0]; i++) {
+	/* the starting point for the values to be moved */
+	t = dataset->t1 + sheet->inserts[i];
+	src = dataset->Z[v] + t;
+	memmove(src + 1, src, (n - t) * sizeof *src);
+	n++;
+    }
+}
+
 /* pull modified values from the data-editing spreadsheet
    into the main dataset */
 
@@ -1323,7 +1452,7 @@ static void update_dataset_from_sheet (Spreadsheet *sheet)
     int newvars = sheet->added_vars;
     int newobs = sheet->datarows - sheet->orig_nobs;
     int missobs = 0;
-    int i, colnum, s, t;
+    int i, j, t;
 
 #if SSDEBUG
     fprintf(stderr, "update_dataset_from_sheet: oldv=%d, newvars=%d, newobs=%d\n",
@@ -1332,7 +1461,7 @@ static void update_dataset_from_sheet (Spreadsheet *sheet)
 
     model = gtk_tree_view_get_model(view);
 
-    /* first extend series length, if needed */
+    /* first extend the series length, if needed */
 
     if (newobs > 0) {
 	if (dataset_add_observations(newobs, dataset, OPT_A) ||
@@ -1342,10 +1471,10 @@ static void update_dataset_from_sheet (Spreadsheet *sheet)
 	}
     }
 
-    /* then add any new variables to data set */
+    /* then add any new series to the dataset */
 
     if (newvars > 0) {
-	int vi, v0 = sheet->varlist[0];
+	int vj, v0 = sheet->varlist[0];
 	const gchar *newname;
 
 	if (dataset_add_series(newvars, dataset)) {
@@ -1353,51 +1482,64 @@ static void update_dataset_from_sheet (Spreadsheet *sheet)
 	    return;
 	}
 
-	for (i=0; i<newvars; i++) { 
+	for (j=0; j<newvars; j++) { 
 	    newname = NULL;
-	    vi = oldv + i;
-	    gretl_list_append_term(&sheet->varlist, vi);
+	    vj = oldv + j;
+	    gretl_list_append_term(&sheet->varlist, vj);
 	    if (sheet->varlist == NULL) {
 		nomem();
 		return;
 	    }
-	    colnum = v0 + 1 + i;
-	    column = gtk_tree_view_get_column(view, colnum);
+	    column = gtk_tree_view_get_column(view, v0 + 1 + j);
 	    newname = gtk_tree_view_column_get_title(column);
-	    strcpy(dataset->varname[vi], newname); 
+	    strcpy(dataset->varname[vj], newname); 
+	    for (i=0; i<dataset->n; i++) {
+		dataset->Z[vj][i] = NADBL;
+	    }
 #if SSDEBUG
 	    fprintf(stderr, " added var %d (%s) from column %d\n",
-		    vi, newname, colnum);
+		    vj, newname, v0 + 1 + j);
 #endif
 	}
     }
 
-    /* copy data values from spreadsheet */
+    /* now copy data values from spreadsheet, as needed */
 
-    colnum = 1;
-    for (i=1; i<=sheet->varlist[0]; i++) {
-	int vi = sheet->varlist[i];
-	gchar *numstr;
-
-	gtk_tree_model_get_iter_first(model, &iter);
+    for (j=1; j<=sheet->varlist[0]; j++) {
+	int vj = sheet->varlist[j];
+	int var_added = (vj >= oldv);
 
 #if SSDEBUG
-	fprintf(stderr, " updating data for var %d (%s) from column %d\n",
-		vi, dataset->varname[vi], colnum);
+	fprintf(stderr, " update data for var %d (%s) from column %d?\n",
+		vj, dataset->varname[vj], j);
 #endif
-	t = dataset->t1;
-	for (s=0; s<sheet->datarows; s++) {
-	    gtk_tree_model_get(model, &iter, colnum, &numstr, -1);
-	    if (*numstr != '\0') {
-		dataset->Z[vi][t++] = atof(numstr); 
-	    } else {
-		dataset->Z[vi][t++] = NADBL;
-		missobs = 1;
+
+	if (!var_added && sheet->inserts != NULL) {
+	    process_inserts_for_series(sheet, vj);
+	}
+
+	if (var_added || data_column_edited(sheet, j)) {
+	    gchar *numstr;
+
+#if SSDEBUG
+	    fprintf(stderr, " yes, %s\n", var_added ? "added" : "modified");
+#endif
+	    gtk_tree_model_get_iter_first(model, &iter);
+	    for (i=0; i<sheet->datarows; i++) {
+		if (var_added || data_cell_edited(sheet, j, i)) {
+		    t = dataset->t1 + i;
+		    gtk_tree_model_get(model, &iter, j, &numstr, -1);
+		    if (*numstr != '\0') {
+			dataset->Z[vj][t] = atof(numstr); 
+		    } else {
+			dataset->Z[vj][t] = NADBL;
+			missobs = 1;
+		    }
+		    g_free(numstr);
+		}
+		gtk_tree_model_iter_next(model, &iter);
 	    }
-	    g_free(numstr);
-	    gtk_tree_model_iter_next(model, &iter);
 	}
-	colnum++;
     }
 
     /* copy observation markers, if relevant */
@@ -1407,7 +1549,7 @@ static void update_dataset_from_sheet (Spreadsheet *sheet)
 
 	gtk_tree_model_get_iter_first(model, &iter);
 	t = dataset->t1;
-	for (s=0; s<sheet->datarows; s++) {
+	for (i=0; i<sheet->datarows; i++) {
 	    gtk_tree_model_get(model, &iter, 0, &marker, -1);
 	    strcpy(dataset->S[t++], marker);
 	    g_free(marker);
@@ -1423,6 +1565,7 @@ static void update_dataset_from_sheet (Spreadsheet *sheet)
 
     sheet_set_modified(sheet, FALSE);
     sheet->added_vars -= newvars; /* record that these are handled */
+    sheet->orig_nobs += newobs;   /* and these too */
 }
 
 static void matrix_new_name (GtkWidget *w, dialog_t *dlg)
@@ -1431,13 +1574,12 @@ static void matrix_new_name (GtkWidget *w, dialog_t *dlg)
     const gchar *buf = edit_dialog_get_text(dlg);
 
     if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_MATRIX)) {
-	return;
+	edit_dialog_reset(dlg);
+    } else {
+	*newname = '\0';
+	strncat(newname, buf, VNAMELEN - 1);
+	edit_dialog_close(dlg);
     }
-
-    *newname = 0;
-    strncat(newname, buf, VNAMELEN - 1);
-
-    close_dialog(dlg);
 }
 
 static void matrix_save_as (GtkWidget *w, Spreadsheet *sheet)
@@ -1445,10 +1587,11 @@ static void matrix_save_as (GtkWidget *w, Spreadsheet *sheet)
     char newname[VNAMELEN];
     int cancel = 0;
 
-    edit_dialog(_("gretl: save matrix"), 
-		_("Enter a name"),
-		NULL, matrix_new_name, newname, 
-		0, VARCLICK_NONE, &cancel);
+    blocking_edit_dialog(0, _("gretl: save matrix"), 
+			 _("Enter a name"), NULL,
+			 matrix_new_name, newname, 
+			 VARCLICK_NONE, sheet->win,
+			 &cancel);
     
     if (!cancel) {
 	gretl_matrix *m;
@@ -1525,6 +1668,23 @@ static void set_ok_transforms (Spreadsheet *sheet)
 	 s < GRETL_MATRIX_SYMMETRIC);
 }
 
+static void maybe_rename_sheet_cols (Spreadsheet *sheet)
+{
+    GtkTreeViewColumn *col;
+    const char *cname;
+    char cstr[16];
+    int i;
+
+    for (i=1; i<=sheet->matrix->cols; i++) {
+	col = gtk_tree_view_get_column(GTK_TREE_VIEW(sheet->view), i);
+	cname = gtk_tree_view_column_get_title(col);
+	sprintf(cstr, "%d", i);
+	if (strcmp(cname, cstr)) {
+	    gtk_tree_view_column_set_title(col, cstr);
+	}
+    }
+}
+
 static int rejig_sheet_cols (Spreadsheet *sheet)
 {
     int n = sheet->matrix->cols - sheet->datacols;
@@ -1567,7 +1727,7 @@ static int rejig_sheet_cols (Spreadsheet *sheet)
 	gtk_tree_view_column_set_clickable(col, TRUE);
 	g_object_set_data(G_OBJECT(col), "sheet", sheet);
 	g_signal_connect(G_OBJECT(col), "clicked",
-			 G_CALLBACK(name_column_dialog), col);
+			 G_CALLBACK(name_column_dialog), sheet);
     }
 
     return 0;
@@ -1589,6 +1749,8 @@ static int update_sheet_from_matrix (Spreadsheet *sheet)
 	    return err;
 	}
 	resized = 1;
+    } else {
+	maybe_rename_sheet_cols(sheet);
     }
 
     store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
@@ -1601,7 +1763,11 @@ static int update_sheet_from_matrix (Spreadsheet *sheet)
 	    gtk_list_store_append(store, &iter);
 	    sprintf(tmpstr, "%d", i1);
 	    gtk_list_store_set(store, &iter, 0, tmpstr, -1);
+	} else if (sheet->rownames != NULL) {
+	    sprintf(tmpstr, "%d", i1);
+	    gtk_list_store_set(store, &iter, 0, tmpstr, -1);
 	}
+	    
 	for (j=0; j<sheet->matrix->cols; j++) {
 	    x = gretl_matrix_get(sheet->matrix, i, j);
 	    if (x == -0.0) {
@@ -1655,8 +1821,8 @@ static int add_matrix_data_to_sheet (Spreadsheet *sheet)
 {
     gchar tmpstr[32];
     GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
-    GtkTreeIter iter;
     GtkListStore *store;
+    GtkTreeIter iter;
     double x;
     int i, j;
 
@@ -1665,7 +1831,12 @@ static int add_matrix_data_to_sheet (Spreadsheet *sheet)
     /* row labels */
     gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
     for (i=0; i<sheet->datarows; i++) {
-	sprintf(tmpstr, "%d", i+1);
+	if (sheet->rownames != NULL) {
+	    *tmpstr = '\0';
+	    strncat(tmpstr, sheet->rownames[i], 31);
+	} else {
+	    sprintf(tmpstr, "%d", i+1);
+	}
 	gtk_list_store_append(store, &iter);
 	gtk_list_store_set(store, &iter, 0, tmpstr, -1);
     }
@@ -1728,6 +1899,52 @@ static int add_scalars_to_sheet (Spreadsheet *sheet)
     return 0;
 }
 
+static void maybe_update_scalars_sheet (Spreadsheet *sheet)
+{
+    GtkTreeView *view = GTK_TREE_VIEW(sheet->view);
+    int ns = n_saved_scalars();
+    int update = 0;
+
+    if (sheet->datarows != ns) {
+	sheet->datarows = ns;
+	update = 1;
+    } else {
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	gchar *rowname, *numstr;
+	const char *sname;
+	double sx, rx;
+	int i;
+
+	model = gtk_tree_view_get_model(view);
+	gtk_tree_model_get_iter_first(model, &iter);
+
+	for (i=0; i<sheet->datarows; i++) {
+	    gtk_tree_model_get(model, &iter, 0, &rowname, 1, &numstr, -1);
+	    rx = atof(numstr);
+	    sname = gretl_scalar_get_name(i);
+	    sx = gretl_scalar_get_value_by_index(i);
+	    if (strcmp(rowname, sname) || rx != sx) {
+		update = 1;
+	    }
+	    g_free(rowname);
+	    g_free(numstr);
+	    if (update) {
+		break;
+	    }
+	    gtk_tree_model_iter_next(model, &iter);
+	}
+    }
+
+    if (update) {
+	GtkListStore *store;
+
+	store = GTK_LIST_STORE(gtk_tree_view_get_model(view));
+	gtk_list_store_clear(store);
+	add_scalars_to_sheet(sheet);
+    }
+}
+
 static int add_data_to_sheet (Spreadsheet *sheet, SheetCmd c)
 {
     gchar rowlabel[OBSLEN];
@@ -1763,7 +1980,6 @@ static int add_data_to_sheet (Spreadsheet *sheet, SheetCmd c)
 	    gtk_list_store_set(store, &iter, 1, "", -1);
 	} else {
 	    char numstr[32];
-	    int colnum = 0;
 	    int vi;
 
 	    for (i=1; i<=sheet->varlist[0]; i++) {
@@ -1771,9 +1987,9 @@ static int add_data_to_sheet (Spreadsheet *sheet, SheetCmd c)
 		if (na(dataset->Z[vi][t])) {
 		    *numstr = '\0';
 		} else {
-		    sprintf(numstr, "%.*g", DBL_DIG, dataset->Z[vi][t]);
+		    sprintf(numstr, "%.*g", SERIES_DIGITS, dataset->Z[vi][t]);
 		}
-		gtk_list_store_set(store, &iter, ++colnum, numstr, -1);
+		gtk_list_store_set(store, &iter, i, numstr, -1);
 	    }
 	}
 	gtk_tree_model_iter_next(GTK_TREE_MODEL(store), &iter);
@@ -1810,12 +2026,26 @@ gint get_string_width (const gchar *str)
     return width;
 }
 
-static gint get_row_label_width (void)
+static gint get_row_label_width (Spreadsheet *sheet)
 {
     static gint width;
 
     if (width == 0) {
-	width = get_string_width("XXXX");
+	if (sheet != NULL && sheet->rownames != NULL) {
+	    const char *s = sheet->rownames[0];
+	    int i, len, maxlen = 0;
+
+	    for (i=0; i<sheet->datarows; i++) {
+		len = strlen(sheet->rownames[i]);
+		if (len > maxlen) {
+		    maxlen = len;
+		    s = sheet->rownames[i];
+		}
+	    }
+	    width = get_string_width(s);
+	} else {
+	    width = get_string_width("XXXX");
+	}
     }
     return width;
 }
@@ -2245,22 +2475,6 @@ static int build_sheet_view (Spreadsheet *sheet)
     gchar tmpstr[32];
     gint i, width, colnum;
 
-    if (sheet->varlist != NULL) {
-	sheet->datacols = 0;
-
-	for (i=sheet->varlist[0]; i>0; i--) {
-	    if (var_is_hidden(dataset, sheet->varlist[i])) {
-		gretl_list_delete_at_pos(sheet->varlist, i);
-	    } else {
-		sheet->datacols += 1;
-	    }
-	}  
-
-	if (sheet->varlist[0] < dataset->v - 1) {
-	    sheet->flags |= SHEET_SHORT_VARLIST;
-	}
-    }
-
     if (get_local_decpoint() == ',') {
 	sheet->flags |= SHEET_USE_COMMA;
     }
@@ -2279,7 +2493,7 @@ static int build_sheet_view (Spreadsheet *sheet)
 	col0str = _("Name");
 	width = get_name_col_width();
     } else if (sheet->matrix != NULL) {
-	width = get_row_label_width();
+	width = get_row_label_width(sheet);
     } else {
 	width = get_obs_col_width();
     }
@@ -2311,7 +2525,7 @@ static int build_sheet_view (Spreadsheet *sheet)
 	    g_object_set_data(G_OBJECT(column), "sheet", sheet);
 	    gtk_tree_view_column_set_clickable(column, TRUE);
 	    g_signal_connect(G_OBJECT(column), "clicked",
-			     G_CALLBACK(name_column_dialog), column);
+			     G_CALLBACK(name_column_dialog), sheet);
 	}
 	sheet->colnames = NULL;
     } else if (editing_scalars(sheet)) {
@@ -2340,6 +2554,7 @@ static int build_sheet_view (Spreadsheet *sheet)
 	set_treeview_column_number(column, 2);
 	g_object_set_data(G_OBJECT(column), "sheet", sheet);
     } else {
+	/* editing data series */
 	colnum = 0;
 	for (i=1; i<=sheet->varlist[0]; i++) {
 	    int vi = sheet->varlist[i];
@@ -2387,8 +2602,10 @@ static void free_spreadsheet (GtkWidget *widget, Spreadsheet **psheet)
     }
 
     free(sheet->varlist);
+    free(sheet->edits);
+    free(sheet->inserts);
 
-    if (sheet->matrix == NULL) {
+    if (editing_series(sheet)) {
 	set_dataset_locked(FALSE);
     }
 
@@ -2415,7 +2632,7 @@ static void free_matrix_sheet (GtkWidget *widget, Spreadsheet *sheet)
 
     free(sheet->varlist);
 
-    if (sheet->matrix == NULL) {
+    if (editing_series(sheet)) {
 	set_dataset_locked(FALSE);
     }
 
@@ -2476,21 +2693,24 @@ static Spreadsheet *spreadsheet_new (SheetCmd c, int varnum)
     sheet->matrix = NULL;
     sheet->oldmat = NULL;
     sheet->colnames = NULL;
+    sheet->rownames = NULL;
     sheet->cmd = c;
     sheet->flags = 0;
 
+    sheet->edits = NULL;
+    sheet->inserts = NULL;
+
     sheet->mname[0] = '\0';
 
     if (c == SHEET_EDIT_MATRIX || c == SHEET_EDIT_SCALARS) {
 	return sheet;
     }
 
-    if (dataset->t1 != 0 || dataset->t2 < dataset->n - 1) {
+    sheet->orig_nobs = sample_size(dataset);
+    if (sheet->orig_nobs < dataset->n) {
 	sheet->flags |= SHEET_SUBSAMPLED;
     }
 
-    sheet->orig_nobs = dataset->t2 - dataset->t1 + 1;
-
     if (sheet->cmd == SHEET_NEW_DATASET) {
 	sheet->varlist = gretl_list_new(1);
     } else {
@@ -2605,7 +2825,7 @@ static void size_matrix_window (Spreadsheet *sheet)
 	nc = 2;
     }
 
-    w = get_row_label_width() + nc * get_data_col_width() + 30;
+    w = get_row_label_width(sheet) + nc * get_data_col_width() + 30;
     if (w > 640) {
 	w = 640;
     }
@@ -2623,7 +2843,7 @@ static void size_scalars_window (Spreadsheet *sheet)
     int nc = 2;
     int w, h;
 
-    w = get_row_label_width() + nc * get_data_col_width() + 
+    w = get_row_label_width(sheet) + nc * get_data_col_width() + 
 	get_delete_col_width() + 30;
     if (w > 640) {
 	w = 640;
@@ -2637,20 +2857,24 @@ static void size_scalars_window (Spreadsheet *sheet)
     gtk_window_resize(GTK_WINDOW(sheet->win), w, h);    
 }
 
-static void size_data_window (Spreadsheet *sheet)
+static void size_data_window (Spreadsheet *sheet, int hscroll)
 {
-    int ocw = get_obs_col_width();
-    int dcw = get_data_col_width();
-    int extra = 40;
-    int nc, w, h = 400;
+    int w, h = 400;
 
-    nc = (sheet->varlist[0] > 4)? 4 : sheet->varlist[0];
-    if (nc < 2) {
-	extra += 40;
+    if (hscroll) {
+	int nc = (sheet->varlist[0] > 4)? 4 : sheet->varlist[0];
+	int ocw = get_obs_col_width();
+	int dcw = get_data_col_width();
+	int extra = 40;
+
+	if (nc < 2) {
+	    extra += 40;
+	}
+	w = ocw + nc * dcw + extra;
+    } else {
+	w = -1;
     }
 
-    w = ocw + nc * dcw + extra;
-
     gtk_window_set_default_size(GTK_WINDOW(sheet->win), w, h);
 }
 
@@ -2681,7 +2905,7 @@ button_entered (GtkWidget *w, GdkEventCrossing *e, Spreadsheet *sheet)
 
 static void adjust_add_menu_state (Spreadsheet *sheet)
 {
-    sheet->flags |= SHEET_ADD_OBS_OK | SHEET_INSERT_OBS_OK;
+    sheet->flags |= (SHEET_ADD_OBS_OK | SHEET_INSERT_OBS_OK);
 
     if (complex_subsampled() || dataset->t2 < dataset->n - 1) {
 	sheet->flags &= ~SHEET_ADD_OBS_OK;
@@ -2713,35 +2937,69 @@ static void series_sheet_add_locator (Spreadsheet *sheet,
     gtk_box_pack_end(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
 }
 
+static void sheet_toolbar_insert_winlist (Spreadsheet *sheet,
+					  GtkWidget *tbar)
+{
+    GtkWidget *img, *button = gtk_button_new();
+    GtkToolItem *item = gtk_tool_item_new();
+
+    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("Windows"));
+    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+    img = gtk_image_new_from_stock(GRETL_STOCK_COMPASS, GTK_ICON_SIZE_MENU);
+    gtk_container_add(GTK_CONTAINER(button), img);
+    gtk_container_add(GTK_CONTAINER(item), button);
+    g_signal_connect(button, "button-press-event", 
+		     G_CALLBACK(window_list_popup), sheet->win);
+    gtk_toolbar_insert(GTK_TOOLBAR(tbar), item, -1);
+}
+
 static void sheet_add_toolbar (Spreadsheet *sheet, GtkWidget *vbox)
 {
     GtkWidget *hbox, *tbar;
+    GretlToolItem *items;
     GretlToolItem *item;
-    GtkToolItem *button;
-    int i;
+    GtkWidget *button;
+    int i, n_items;
 
     hbox = gtk_hbox_new(FALSE, 0);
     tbar = gretl_toolbar_new();
 
-    for (i=0; i<n_series_items; i++) {
-	item = &series_items[i];
-	button = gretl_toolbar_insert(tbar, item, item->func, sheet, -1);
-	if (item->flag == SERIES_APPLY_BTN) {
-	    sheet->apply = GTK_WIDGET(button);
-	    gtk_widget_set_sensitive(sheet->apply, FALSE);
-	} 
-	if (item->flag != SERIES_ADD_BTN) {
-	    g_signal_connect(G_OBJECT(button), "enter-notify-event",
-			     G_CALLBACK(button_entered), sheet);
+    if (editing_scalars(sheet)) {
+	items = scalar_items;
+	n_items = n_scalar_items;
+    } else {
+	items = series_items;
+	n_items = n_series_items;
+    }
+
+    for (i=0; i<n_items; i++) {
+	item = &items[i];
+	if (winlist_item(item)) {
+	    sheet_toolbar_insert_winlist(sheet, tbar);
+	} else {
+	    button = gretl_toolbar_insert(tbar, item, item->func, sheet, -1);
+	    if (!editing_scalars(sheet)) {
+		if (item->flag == SHEET_APPLY_BTN) {
+		    sheet->apply = button;
+		    gtk_widget_set_sensitive(sheet->apply, FALSE);
+		} 
+		if (item->flag != SHEET_ADD_BTN) {
+		    g_signal_connect(G_OBJECT(button), "enter-notify-event",
+				     G_CALLBACK(button_entered), sheet);
+		}
+	    }
 	}
     }
 
     gtk_box_pack_start(GTK_BOX(hbox), tbar, FALSE, FALSE, 0);
-    series_sheet_add_locator(sheet, hbox);
-
+    if (!editing_scalars(sheet)) {
+	series_sheet_add_locator(sheet, hbox);
+    }
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-    adjust_add_menu_state(sheet);
+    if (!editing_scalars(sheet)) {
+	adjust_add_menu_state(sheet);
+    }
 }
 
 static void sheet_add_matrix_menu (Spreadsheet *sheet, GtkWidget *vbox)
@@ -2768,7 +3026,7 @@ static void sheet_add_matrix_menu (Spreadsheet *sheet, GtkWidget *vbox)
 static void sheet_add_matrix_locator (Spreadsheet *sheet, GtkWidget *vbox)
 {
     GtkWidget *status_box = gtk_hbox_new(FALSE, 1);
-    gint w = get_row_label_width();
+    gint w = get_row_label_width(NULL);
 
     gtk_container_set_border_width(GTK_CONTAINER(status_box), 0);
     gtk_box_pack_start(GTK_BOX(vbox), status_box, FALSE, FALSE, 0);
@@ -2793,6 +3051,7 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 {
     Spreadsheet *sheet = *psheet;
     GtkWidget *tmp, *scroller, *main_vbox;
+    int hscroll = 1;
     int err = 0;
 
     sheet->win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -2816,8 +3075,13 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 	size_matrix_window(sheet);
     } else if (c == SHEET_EDIT_SCALARS) {
 	size_scalars_window(sheet);
+	hscroll = 0;
     } else {
-	size_data_window(sheet);
+	if (sheet->varlist != NULL && sheet->varlist[0] < 3) {
+	    /* make columns full "natural" width */
+	    hscroll = 0;
+	}
+	size_data_window(sheet, hscroll);
     }
 
     if (block) {
@@ -2835,7 +3099,25 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
     if (sheet->matrix != NULL) {
 	sheet_add_matrix_menu(sheet, main_vbox);
 	sheet_add_matrix_locator(sheet, main_vbox);
-    } else if (c != SHEET_EDIT_SCALARS) {
+    } else {
+	if (c != SHEET_EDIT_SCALARS) {
+	    if (sheet->varlist != NULL) {
+		int i;
+
+		sheet->datacols = 0;
+
+		for (i=sheet->varlist[0]; i>0; i--) {
+		    if (var_is_hidden(dataset, sheet->varlist[i])) {
+			gretl_list_delete_at_pos(sheet->varlist, i);
+		    } else {
+			sheet->datacols += 1;
+		    }
+		} 
+		if (sheet->varlist[0] < dataset->v - 1) {
+		    sheet->flags |= SHEET_SHORT_VARLIST;
+		}
+	    }
+	}
 	sheet_add_toolbar(sheet, main_vbox);
     }
 
@@ -2843,9 +3125,8 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 
     scroller = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
-				   c == SHEET_EDIT_SCALARS ?
-				   GTK_POLICY_NEVER :
-				   GTK_POLICY_AUTOMATIC, 
+				   hscroll ? GTK_POLICY_AUTOMATIC :
+				   GTK_POLICY_NEVER,
 				   GTK_POLICY_AUTOMATIC);
     gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scroller),
 					GTK_SHADOW_IN);    
@@ -2868,8 +3149,8 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
 			 G_CALLBACK(free_spreadsheet), psheet);
     } 
 
-    if (sheet->matrix != NULL || c == SHEET_EDIT_SCALARS) {
-	/* control buttons (we don't use these for editing series) */
+    if (sheet->matrix != NULL) {
+	/* control buttons (for edting matrices only) */
 	GtkWidget *button_box;
 
 	button_box = gtk_hbutton_box_new();
@@ -2957,14 +3238,12 @@ static void real_show_spreadsheet (Spreadsheet **psheet, SheetCmd c,
     select_first_editable_cell(sheet);
 
     gtk_widget_show(sheet->win);
-    
-    if (c != SHEET_EDIT_SCALARS) {
-	add_window_list_item(sheet->win, 0);
-    }
+    window_list_add(sheet->win, SSHEET);
 
-    if (c != SHEET_EDIT_MATRIX && c != SHEET_EDIT_SCALARS) {
+    if (editing_series(sheet)) {
 	/* we can't have the user making confounding changes elsewhere,
-	   while editing the dataset here */
+	   while editing the dataset here 
+	*/
 	set_dataset_locked(TRUE);
     }
 
@@ -3020,6 +3299,7 @@ void edit_scalars (void)
     static Spreadsheet *sheet; 
 
     if (sheet != NULL) {
+	maybe_update_scalars_sheet(sheet);
 	gtk_window_present(GTK_WINDOW(sheet->win));
 	return;
     }
@@ -3037,6 +3317,13 @@ void edit_scalars (void)
     real_show_spreadsheet(&sheet, SHEET_EDIT_SCALARS, 0);
 }
 
+void sync_scalars_window (void)
+{
+    if (scalars_sheet != NULL) {
+	maybe_update_scalars_sheet(scalars_sheet);
+    }
+}
+
 struct gui_matrix_spec {
     char name[VNAMELEN];
     int rows;
@@ -3158,7 +3445,7 @@ static int new_matrix_dialog (struct gui_matrix_spec *spec)
     GtkWidget *rb;
     GtkWidget *w;
     int maxdim = 1000;
-    int canceled = 0;
+    int ret = GRETL_CANCEL;
 
     dlg = gretl_dialog_new("Matrix", mdata->main, 
 			   GRETL_DLG_BLOCK | GRETL_DLG_MODAL);
@@ -3275,14 +3562,14 @@ static int new_matrix_dialog (struct gui_matrix_spec *spec)
 
     /* control buttons */
     hbox = gtk_dialog_get_action_area(GTK_DIALOG(dlg));
-    w = cancel_delete_button(hbox, dlg, &canceled);
-    w = ok_button(hbox);
+    w = cancel_delete_button(hbox, dlg);
+    w = ok_validate_button(hbox, &ret, NULL);
     g_signal_connect(G_OBJECT(w), "clicked", G_CALLBACK(matrix_dialog_ok), &mdlg);
     gtk_widget_grab_default(w);
 
     gtk_widget_show_all(dlg);
 
-    return canceled;
+    return ret;
 }
 
 static int gui_matrix_from_list (selector *sr)
@@ -3428,7 +3715,8 @@ static void edit_matrix (gretl_matrix *m, const char *name,
 	free(sheet);
 	sheet = NULL;
     } else {
-	sheet->colnames = user_matrix_get_names(m, 0);
+	sheet->colnames = gretl_matrix_get_colnames(m);
+	sheet->rownames = gretl_matrix_get_rownames(m);
 	sheet->datarows = gretl_matrix_rows(sheet->matrix);
 	sheet->datacols = gretl_matrix_cols(sheet->matrix);
 	real_show_spreadsheet(&sheet, SHEET_EDIT_MATRIX, block);
@@ -3440,9 +3728,6 @@ static void edit_matrix (gretl_matrix *m, const char *name,
 
 	if (u != NULL) {
 	    g_object_set_data(G_OBJECT(sheet->win), "object", u);
-	    g_signal_connect(G_OBJECT(sheet->win), "destroy", 
-			     G_CALLBACK(winstack_remove), sheet->win);
-	    winstack_add(sheet->win);
 	}
     }
 }
@@ -3454,19 +3739,20 @@ static void real_gui_new_matrix (gretl_matrix *m, const char *name)
 {
     struct gui_matrix_spec spec;
     int block = (m == NULL);
-    int cancel = 0;
+    int resp;
 
     gui_matrix_spec_init(&spec, m, name);
 
-    cancel = new_matrix_dialog(&spec);
-    if (cancel) {
+    resp = new_matrix_dialog(&spec);
+    if (canceled(resp)) {
 	return;
     }
 
     if (spec.uselist) {
 	/* matrix from listed vars */
-	simple_selection(_("Define matrix"), gui_matrix_from_list, 
-			 DEFINE_MATRIX, &spec);
+	simple_selection_with_data(DEFINE_MATRIX, _("Define matrix"), 
+				   gui_matrix_from_list, 
+				   NULL, &spec);
     } else if (spec.formula != NULL) {
 	/* matrix from genr-style formula */
 	matrix_from_formula(&spec);
@@ -3496,7 +3782,7 @@ void edit_user_matrix_by_name (const char *name)
 
     /* do we already have a window open, editing this
        matrix? */
-    w = match_window_by_data(u);
+    w = get_window_for_data(u);
     if (w != NULL) {
 	gtk_window_present(GTK_WINDOW(w));
 	return;
@@ -3518,6 +3804,7 @@ static int locked;
 void set_dataset_locked (gboolean s)
 {
     locked = s;
+    flip(mdata->ui, "/menubar/Data", !s);
     flip(mdata->ui, "/menubar/Sample", !s);
 }
 
