@@ -21,6 +21,8 @@
 #include "textbuf.h"
 #include "toolbar.h"
 #include "dlgutils.h"
+#include "winstack.h"
+#include "tabwin.h"
 #include "gretl_func.h"
 
 #ifdef G_OS_WIN32
@@ -129,7 +131,7 @@ static void get_char_width_and_height (GtkWidget *widget,
     pl = pango_layout_new(pc);
     pango_layout_set_text(pl, "X", -1);
     pango_layout_get_pixel_size(pl, &w, &h);
-    g_object_unref(G_OBJECT(pl));
+    g_object_unref(pl);
 
     if (width != NULL) {
 	*width = w;
@@ -190,7 +192,7 @@ gchar *textview_get_trimmed_text (GtkWidget *view)
 }
 
 gchar *textview_get_selection_or_all (GtkWidget *view,
-				      int *sel)
+				      gboolean *selection)
 {
     GtkTextBuffer *tbuf;
     GtkTextIter start, end;
@@ -203,9 +205,9 @@ gchar *textview_get_selection_or_all (GtkWidget *view,
     }
 
     if (gtk_text_buffer_get_selection_bounds(tbuf, &start, &end)) {
-	*sel = 1;
+	*selection = TRUE;
     } else {
-	*sel = 0;
+	*selection = FALSE;
 	gtk_text_buffer_get_start_iter(tbuf, &start);
 	gtk_text_buffer_get_end_iter(tbuf, &end);
     }
@@ -465,6 +467,7 @@ void sourceview_print (windata_t *vwin)
     GtkSourcePrintCompositor *comp;
     GtkPrintOperation *print;
     GtkPrintOperationResult res;
+    GtkWidget *mainwin;
     GError *error = NULL;
 
     comp = gtk_source_print_compositor_new_from_view(view);
@@ -482,15 +485,17 @@ void sourceview_print (windata_t *vwin)
     g_signal_connect(print, "begin_print", G_CALLBACK(begin_print), comp);
     g_signal_connect(print, "draw_page", G_CALLBACK(draw_page), comp);
 
+    mainwin = vwin_toplevel(vwin);
+
     res = gtk_print_operation_run(print, 
 				  GTK_PRINT_OPERATION_ACTION_PRINT_DIALOG,
-				  GTK_WINDOW(vwin->main), 
+				  GTK_WINDOW(mainwin), 
 				  &error);
 
     if (res == GTK_PRINT_OPERATION_RESULT_ERROR) {
 	GtkWidget *dlg; 
 
-	dlg = gtk_message_dialog_new(GTK_WINDOW(vwin->main),
+	dlg = gtk_message_dialog_new(GTK_WINDOW(mainwin),
 				     GTK_DIALOG_DESTROY_WITH_PARENT,
 				     GTK_MESSAGE_ERROR,
 				     GTK_BUTTONS_CLOSE,
@@ -707,12 +712,22 @@ void create_source (windata_t *vwin, int hsize, int vsize,
     gtk_widget_modify_font(GTK_WIDGET(vwin->text), fixed_font);
 
     cw = get_char_width(vwin->text);
-    hsize *= cw;
-    hsize += 48;
-
     set_source_tabs(vwin->text, cw);
 
-    gtk_window_set_default_size(GTK_WINDOW(vwin->main), hsize, vsize); 
+    if (hsize > 0) {
+	hsize *= cw;
+	hsize += 48; /* ?? */
+    }
+
+    if (hsize > 0 && vsize > 0) {
+	GtkWidget *vmain = vwin_toplevel(vwin);
+
+	if (window_is_tab(vwin)) {
+	    vsize += 15;
+	}
+	gtk_window_set_default_size(GTK_WINDOW(vmain), hsize, vsize);
+    }
+
     gtk_text_view_set_editable(view, editable);
     gtk_text_view_set_cursor_visible(view, editable);
 
@@ -734,10 +749,8 @@ void create_source (windata_t *vwin, int hsize, int vsize,
     }	
 }
 
-void text_zoom (GtkAction *action, gpointer data)
+static void text_change_size (windata_t *vwin, int larger)
 {
-    const gchar *s = gtk_action_get_name(action);
-    windata_t *vwin = (windata_t *) data;
     static PangoFontDescription *hpf;
     static gint fsize;
 
@@ -746,9 +759,9 @@ void text_zoom (GtkAction *action, gpointer data)
 	fsize = pango_font_description_get_size(hpf) / PANGO_SCALE;
     }
 
-    if (!strcmp(s, "ZoomIn")) {
+    if (larger) {
 	fsize++;
-    } else if (!strcmp(s, "ZoomOut")) {
+    } else {
 	fsize--;
     } 
 
@@ -756,6 +769,16 @@ void text_zoom (GtkAction *action, gpointer data)
     gtk_widget_modify_font(vwin->text, hpf);
 }
 
+void text_larger (GtkWidget *w, gpointer data)
+{
+    text_change_size((windata_t *) data, 1);
+}
+
+void text_smaller (GtkWidget *w, gpointer data)
+{
+    text_change_size((windata_t *) data, 0);
+}
+
 #define helpfont "sans"
 
 static GtkTextTagTable *gretl_tags_new (void)
@@ -826,6 +849,11 @@ static GtkTextTagTable *gretl_tags_new (void)
     g_object_set(tag, "family", "monospace", NULL);
     gtk_text_tag_table_add(table, tag);
 
+    tag = gtk_text_tag_new("optflag");
+    g_object_set(tag, "family", "monospace", 
+		 "foreground", "#396d60", NULL);
+    gtk_text_tag_table_add(table, tag);
+
     tag = gtk_text_tag_new("text");
     g_object_set(tag, "family", helpfont, NULL);
     gtk_text_tag_table_add(table, tag);
@@ -1150,24 +1178,23 @@ static void open_script_link (GtkTextTag *tag)
 
     g_free(fname);
 
-    view_file(fullname, 0, 0, 78, 370, VIEW_SCRIPT);
+    view_script(fullname, 0, VIEW_SCRIPT);
 }
 
 static void make_bibitem_window (const char *buf,
 				 GtkWidget *tview)
 {
     windata_t *vwin;
-    GtkWidget *top;
+    GtkWidget *top, *vmain;
 
     vwin = view_formatted_text_buffer(NULL, buf, 64, 100);
+    vmain = vwin_toplevel(vwin);
     top = gtk_widget_get_toplevel(tview);
-    gtk_window_set_transient_for(GTK_WINDOW(vwin->main),
-				 GTK_WINDOW(top));
-    gtk_window_set_destroy_with_parent(GTK_WINDOW(vwin->main), 
-				       TRUE);
-    gtk_window_set_position(GTK_WINDOW(vwin->main), 
+    gtk_window_set_transient_for(GTK_WINDOW(vmain), GTK_WINDOW(top));
+    gtk_window_set_destroy_with_parent(GTK_WINDOW(vmain), TRUE);
+    gtk_window_set_position(GTK_WINDOW(vmain), 
 			    GTK_WIN_POS_CENTER_ON_PARENT);
-    gtk_widget_show(vwin->main);
+    gtk_widget_show(vmain);
 }
 
 static void open_bibitem_link (GtkTextTag *tag, GtkWidget *tview)
@@ -1227,7 +1254,8 @@ static void open_external_link (GtkTextTag *tag)
     }
 }
 
-static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
+static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, 
+			    gpointer en_ptr)
 {
     GSList *tags = NULL, *tagp = NULL;
 
@@ -1237,7 +1265,7 @@ static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
 	GtkTextTag *tag = tagp->data;
 	gint page = object_get_int(tag, "page");
 	gint xref = object_get_int(tag, "xref");
-	gint en = GPOINTER_TO_INT(p);
+	gint en = GPOINTER_TO_INT(en_ptr);
 
 	if (page != 0 || xref != 0) {
 	    if (page == GUIDE_PAGE) {
@@ -1278,7 +1306,7 @@ static void follow_if_link (GtkWidget *tview, GtkTextIter *iter, gpointer p)
 /* Help links can be activated by pressing Enter */
 
 static gboolean cmdref_key_press (GtkWidget *tview, GdkEventKey *ev,
-				  gpointer p)
+				  gpointer en_ptr)
 {
     GtkTextIter iter;
     GtkTextBuffer *tbuf;
@@ -1289,7 +1317,7 @@ static gboolean cmdref_key_press (GtkWidget *tview, GdkEventKey *ev,
 	tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(tview));
 	gtk_text_buffer_get_iter_at_mark(tbuf, &iter, 
 					 gtk_text_buffer_get_insert(tbuf));
-	follow_if_link(tview, &iter, p);
+	follow_if_link(tview, &iter, en_ptr);
 	break;
     default:
 	break;
@@ -1301,7 +1329,7 @@ static gboolean cmdref_key_press (GtkWidget *tview, GdkEventKey *ev,
 /* Help links can be activated by clicking */
 
 static gboolean cmdref_event_after (GtkWidget *w, GdkEvent *ev,
-				    gpointer p)
+				    gpointer en_ptr)
 {
     GtkTextIter start, end, iter;
     GtkTextView *view;
@@ -1332,7 +1360,7 @@ static gboolean cmdref_event_after (GtkWidget *w, GdkEvent *ev,
 
     gtk_text_view_get_iter_at_location(view, &iter, x, y);
 
-    follow_if_link(w, &iter, p);
+    follow_if_link(w, &iter, en_ptr);
 
     return FALSE;
 }
@@ -2026,7 +2054,8 @@ void script_tabs_dialog (GtkWidget *w, windata_t *vwin)
     resp = checks_dialog(title, NULL, 
 			 &opt, 1, &smt, 0, 0,
 			 0, NULL, /* no radio buttons */
-			 &tsp, spintxt, 2, 8, 0);
+			 &tsp, spintxt, 2, 8, 0,
+			 vwin_toplevel(vwin));
 
     if (resp != GRETL_CANCEL) {
 	tabwidth = tsp;
@@ -2606,15 +2635,11 @@ build_script_popup (windata_t *vwin, struct textbit **ptb)
 	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
     }
 
-    item = gtk_separator_menu_item_new();
-    gtk_widget_show(item);
-    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
-
-    item = gtk_menu_item_new_with_label(_("Main window"));
-    g_signal_connect(G_OBJECT(item), "activate",
-		     G_CALLBACK(raise_main_window), NULL);
-    gtk_widget_show(item);
-    gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), item);
+    if (window_is_undockable(vwin)) {
+	add_undock_popup_item(pmenu, vwin);
+    } else if (window_is_dockable(vwin)) {
+	add_dock_popup_item(pmenu, vwin);
+    }	
 
     return pmenu;
 }
@@ -2693,7 +2718,7 @@ static void insert_help_figure (GtkTextBuffer *tbuf, GtkTextIter *iter,
 
     if (pixbuf != NULL) {
 	gtk_text_buffer_insert_pixbuf(tbuf, iter, pixbuf);
-	g_object_unref(G_OBJECT(pixbuf));
+	g_object_unref(pixbuf);
     }
 }
 
@@ -2713,7 +2738,7 @@ static void insert_tagged_text (GtkTextBuffer *tbuf, GtkTextIter *iter,
 	ftag = "literal";
 	break;
     case INSERT_OPT:
-	ftag = "literal"; /* FIXME? */
+	ftag = "optflag";
 	break;
     case INSERT_SUP:
 	ftag = "superscript";
@@ -2732,6 +2757,23 @@ static void insert_tagged_text (GtkTextBuffer *tbuf, GtkTextIter *iter,
 	break;
     }
 
+#ifdef G_OS_WIN32
+    if (ins == INSERT_OPT) {
+	/* Unicode word joiner not supported? Try zero width 
+	   non breaking space instead */
+	char tmp[32];
+
+	strcpy(tmp, s);
+	tmp[2] = 0xEF;
+	tmp[3] = 0xBB;
+	tmp[4] = 0xBF;
+
+	gtk_text_buffer_insert_with_tags_by_name(tbuf, iter, tmp, -1,
+						 ftag, indent, NULL);
+	return;
+    }
+#endif
+
     if (ftag != NULL) {
 	gtk_text_buffer_insert_with_tags_by_name(tbuf, iter, s, -1,
 						 ftag, indent, NULL);
@@ -3069,7 +3111,15 @@ void create_text (windata_t *vwin, int hsize, int vsize,
 	}
     }
 
-    gtk_window_set_default_size(GTK_WINDOW(vwin->main), hsize, vsize); 
+    if (hsize > 0 && vsize > 0) {
+	GtkWidget *vmain = vwin_toplevel(vwin);
+
+	if (window_is_tab(vwin)) {
+	    vsize += 15;
+	}
+	gtk_window_set_default_size(GTK_WINDOW(vmain), hsize, vsize);
+    }
+
     gtk_text_view_set_editable(GTK_TEXT_VIEW(w), editable);
     gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(w), editable);
 }
@@ -3121,14 +3171,16 @@ void viewer_split_pane (windata_t *vwin, int vertical)
     GtkWidget *vbox = vwin->vbox;
     GtkWidget *view1 = vwin->text;
     GtkWidget *sw, *paned, *view2;
+    GtkWidget *vmain;
     GtkTextBuffer *tbuf;
     gint width, height;
 
     sw = g_object_get_data(G_OBJECT(vbox), "sw");
 
-    gtk_window_get_size(GTK_WINDOW(vwin->main), &width, &height);
+    vmain = vwin_toplevel(vwin);
+    gtk_window_get_size(GTK_WINDOW(vmain), &width, &height);
 
-    g_object_ref(G_OBJECT(sw));
+    g_object_ref(sw);
     gtk_container_remove(GTK_CONTAINER(vwin->vbox), sw);
 
     if (vertical) {
@@ -3151,7 +3203,7 @@ void viewer_split_pane (windata_t *vwin, int vertical)
 		     G_CALLBACK(text_popup_handler), vwin);
 
     gtk_paned_add1(GTK_PANED(paned), sw);
-    g_object_unref(G_OBJECT(sw));
+    g_object_unref(sw);
 
     sw = gtk_scrolled_window_new(NULL, NULL);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
@@ -3181,7 +3233,7 @@ void viewer_close_pane (windata_t *vwin)
 
     /* grab the first child and reference it */
     sw = gtk_paned_get_child1(GTK_PANED(paned));
-    g_object_ref(G_OBJECT(sw));
+    g_object_ref(sw);
     gtk_container_remove(GTK_CONTAINER(paned), sw);
 
     /* remove the "paned" widget */
@@ -3192,6 +3244,6 @@ void viewer_close_pane (windata_t *vwin)
     gtk_container_add(GTK_CONTAINER(vwin->vbox), sw);
     g_object_set_data(G_OBJECT(vwin->vbox), "sw", sw);
     gtk_widget_show(sw);
-    g_object_unref(G_OBJECT(sw));
+    g_object_unref(sw);
 }
 
