@@ -48,7 +48,7 @@ gchar *textview_get_text (GtkWidget *view);
 gchar *textview_get_trimmed_text (GtkWidget *view);
 
 gchar *textview_get_selection_or_all (GtkWidget *view,
-				      int *sel);
+				      gboolean *selection);
 
 gchar *textview_get_current_line (GtkWidget *view);
 
@@ -68,7 +68,9 @@ void text_undo (GtkWidget *w, windata_t *vwin);
 
 int text_can_undo (windata_t *vwin);
 
-void text_zoom (GtkAction *action, gpointer data);
+void text_larger (GtkWidget *w, gpointer data);
+
+void text_smaller (GtkWidget *w, gpointer data);
 
 void textview_set_text_colorized (GtkWidget *view, const char *buf);
 
