@@ -28,6 +28,7 @@
 #include "fileselect.h"
 #include "texprint.h"
 #include "system.h"
+#include "winstack.h"
 
 #if USE_GTKSOURCEVIEW_2
 # include <gtksourceview/gtksourceiter.h>
@@ -263,8 +264,7 @@ static void replace_string_dialog (windata_t *vwin)
     gtk_text_buffer_get_start_iter(s->buf, &s->iter);
 
     s->w = gtk_dialog_new();
-    gtk_window_set_transient_for(GTK_WINDOW(s->w), GTK_WINDOW(vwin->main));
-    gtk_window_set_destroy_with_parent(GTK_WINDOW(s->w), TRUE);
+    gretl_dialog_set_destruction(s->w, vwin_toplevel(vwin));
     g_signal_connect(G_OBJECT(s->w), "destroy",
 		     G_CALLBACK(destroy_replacer), s);
 
@@ -425,7 +425,7 @@ static int special_text_handler (windata_t *vwin, guint fmt, int what)
 	    int action = (fmt & GRETL_FORMAT_TEX)? SAVE_TEX : SAVE_RTF;
 
 	    file_selector_with_parent(action, FSEL_DATA_PRN, 
-				      prn, vwin->main);
+				      prn, vwin_toplevel(vwin));
 	}
     }
 
@@ -451,7 +451,8 @@ void window_tex_callback (GtkWidget *w, windata_t *vwin)
 	}
     }
 
-    opt = radio_dialog("gretl: LaTeX", NULL, opts, 3, 0, 0);
+    opt = radio_dialog("gretl: LaTeX", NULL, opts, 3, 0, 0,
+		       vwin_toplevel(vwin));
 
     if (opt >= 0) {
 	int fmt = GRETL_FORMAT_TEX;
@@ -601,8 +602,8 @@ static void window_copy_or_save (windata_t *vwin, guint fmt, int action)
 	    int fcode = (fmt == GRETL_FORMAT_RTF_TXT)? 
 		SAVE_RTF : SAVE_OUTPUT;
 
-	    file_selector_with_parent(fcode, FSEL_DATA_PRN, 
-				      textprn, vwin->main);
+	    file_selector_with_parent(fcode, FSEL_DATA_PRN, textprn, 
+				      vwin_toplevel(vwin));
 	}
 	gretl_print_destroy(textprn);
     }	
@@ -651,20 +652,20 @@ void window_print (GtkAction *action, windata_t *vwin)
     g_free(selbuf);
 }
 
-#define UTF_MINUS(u,i) (u[i] == 0xE2 && u[i+1] == 0x88 && u[i+2] == 0x92)
+#define U_MINUS(u,i) (u[i] == 0xE2 && u[i+1] == 0x88 && u[i+2] == 0x92)
 
-/* check @s for UTF-8 minus signs (U+2212), and if any are found do an
+/* check @s for Unicode minus signs (U+2212), and if any are found do an
    in-place replacement with ASCII dashes
 */
 
-char *strip_utf_minus (char *s)
+char *strip_unicode_minus (char *s)
 {
     unsigned char *u = (unsigned char *) s;
     int i, n = strlen(s);
     int got_minus = 0;
 
     for (i=0; i<n-3; i++) {
-	if (UTF_MINUS(u, i)) {
+	if (U_MINUS(u, i)) {
 	    got_minus = 1;
 	    break;
 	}
@@ -677,7 +678,7 @@ char *strip_utf_minus (char *s)
 	    int j = 0;
 
 	    for (i=0; i<n; i++) {
-		if (i < n - 3 && UTF_MINUS(u, i)) {
+		if (i < n - 3 && U_MINUS(u, i)) {
 		    tmp[j++] = '-';
 		    i += 2;
 		} else {
@@ -692,13 +693,13 @@ char *strip_utf_minus (char *s)
     return s;
 }
 
-int has_utf8_minus (const unsigned char *s)
+int has_unicode_minus (const unsigned char *s)
 {
     int i, n = strlen((const char *) s);
     int has_minus = 0;
 
     for (i=0; i<n-3; i++) {
-	if (UTF_MINUS(s, i)) {
+	if (U_MINUS(s, i)) {
 	    has_minus = 1;
 	    break;
 	}
