@@ -31,6 +31,8 @@
 #include "cmdstack.h"
 #include "dlgutils.h"
 #include "fileselect.h"
+#include "winstack.h"
+#include "tabwin.h"
 #include "toolbar.h"
 
 #include "usermat.h"
@@ -63,6 +65,12 @@
 #include "../pixmaps/mini.func.xpm"
 #include "../pixmaps/mini.db.xpm"
 
+/* for window-finder */
+#include "../pixmaps/mini.gretl.xpm"
+#include "../pixmaps/mini.table.xpm"
+#include "../pixmaps/mini.page.xpm"
+#include "../pixmaps/mini.tools.xpm"
+
 enum {
     SAVE_ITEM = 1,
     SAVE_AS_ITEM,
@@ -90,7 +98,8 @@ enum {
     SPLIT_ITEM,
     EDITOR_ITEM,
     NOTES_ITEM,
-    STOP_ITEM
+    STOP_ITEM,
+    NEW_ITEM
 } viewbar_flags;
 
 static GtkIconFactory *gretl_stock_ifac;
@@ -122,7 +131,11 @@ void gretl_stock_icons_init (void)
 	{ mini_split_v_xpm, GRETL_STOCK_SPLIT_V },
 	{ mini_compass_xpm, GRETL_STOCK_COMPASS },
 	{ mini_spreadsheet_xpm, GRETL_STOCK_SHEET },
-	{ mini_db_xpm, GRETL_STOCK_DB}
+	{ mini_db_xpm, GRETL_STOCK_DB},
+	{ mini_gretl_xpm, GRETL_STOCK_GRETL},
+	{ mini_table_xpm, GRETL_STOCK_TABLE},
+	{ mini_page_xpm, GRETL_STOCK_PAGE},
+	{ mini_tools_xpm, GRETL_STOCK_TOOLS}
     };
     int n = G_N_ELEMENTS(stocks);
 
@@ -155,9 +168,10 @@ void gretl_stock_icons_init (void)
 
 static void save_as_callback (GtkWidget *w, windata_t *vwin)
 {
+    GtkWidget *vmain = vwin_toplevel(vwin);
     guint u = 0;
 
-    if (g_object_get_data(G_OBJECT(vwin->main), "text_out")) {
+    if (g_object_get_data(G_OBJECT(vmain), "text_out")) {
 	const char *opts[] = {
 	    N_("Save to file"),
 	    N_("Save to session as icon")
@@ -165,7 +179,7 @@ static void save_as_callback (GtkWidget *w, windata_t *vwin)
 	int resp;
 
 	resp = radio_dialog(_("gretl: save text"), _("Save text"), 
-			    opts, 2, 0, 0);
+			    opts, 2, 0, 0, vmain);
 	if (resp < 0) {
 	    return;
 	} else if (resp == 1) {
@@ -212,7 +226,8 @@ static void mail_script_callback (GtkWidget *w, windata_t *vwin)
 }
 
 /* callback for the "Open" icon in a script editing window,
-   which enables the user to switch to a different script
+   which enables the user to switch to a different script,
+   or to open another tab if the editor is tab-enabled
 */
 
 static void file_open_callback (GtkWidget *w, windata_t *vwin)
@@ -240,6 +255,7 @@ static void toolbar_new_callback (GtkWidget *w, windata_t *vwin)
 static void window_print_callback (GtkWidget *w, windata_t *vwin)
 {
 #ifdef G_OS_WIN32
+    /* gtksourceview printing is screwed on Windows */
     window_print(NULL, vwin);
 #else
     if (textview_use_highlighting(vwin->role)) {
@@ -258,7 +274,7 @@ static void window_print_callback (GtkWidget *w, windata_t *vwin)
 
 static void window_help (GtkWidget *w, windata_t *vwin)
 {
-    context_help(NULL, GINT_TO_POINTER(vwin->role));
+    show_gui_help(vwin->role);
 }
 
 static void multi_save_as_callback (GtkWidget *w, windata_t *vwin)
@@ -301,41 +317,40 @@ static void toolbar_refresh (GtkWidget *w, windata_t *vwin)
     }
 }
 
-static void matrix_savename (GtkWidget *w, dialog_t *dlg)
+static void set_matrix_name (GtkWidget *widget, dialog_t *dlg)
 {
-    char *newname = (char *) edit_dialog_get_data(dlg);
-    const gchar *buf = edit_dialog_get_text(dlg);
+    char *vname = (char *) edit_dialog_get_data(dlg);
+    const gchar *s = edit_dialog_get_text(dlg);
 
-    if (buf == NULL || gui_validate_varname(buf, GRETL_TYPE_MATRIX)) {
-	return;
+    if (s == NULL || gui_validate_varname(s, GRETL_TYPE_MATRIX)) {
+	edit_dialog_reset(dlg);
+    } else {
+	strcpy(vname, s);
+	edit_dialog_close(dlg);
     }
-
-    *newname = 0;
-    strncat(newname, buf, VNAMELEN - 1);
-
-    close_dialog(dlg);
 }
 
 static void add_matrix_callback (GtkWidget *w, windata_t *vwin)
 {
-    gretl_matrix *m = NULL;
-    char mname[VNAMELEN];
-    int err, cancel = 0;
-
     if (vwin->role == XTAB) {
-	m = xtab_to_matrix(vwin->data);
-	if (m == NULL) {
-	    nomem();
-	} else {
-	    edit_dialog(_("gretl: save matrix"), 
-			_("Enter a name"),
-			NULL, matrix_savename, mname, 
-			0, VARCLICK_NONE, &cancel);
-	    if (cancel) {
-		gretl_matrix_free(m);
+	char mname[VNAMELEN];
+	gretl_matrix *m;
+	int err, cancel = 0;
+
+	blocking_edit_dialog(0, _("gretl: save matrix"), 
+			     _("Enter a name"), NULL,
+			     set_matrix_name, mname, 
+			     VARCLICK_NONE, 
+			     vwin_toplevel(vwin),
+			     &cancel);
+	if (!cancel) {
+	    m = xtab_to_matrix(vwin->data);
+	    if (m == NULL) {
+		nomem();
 	    } else {
 		err = add_or_replace_user_matrix(m, mname);
 		if (err) {
+		    gretl_matrix_free(m);
 		    gui_errmsg(err);
 		} else {
 		    infobox(_("Saved matrix as %s"), mname);
@@ -357,6 +372,8 @@ static void add_data_callback (GtkWidget *w, windata_t *vwin)
 	add_mahalanobis_data(vwin);
     } else if (vwin->role == FCAST) {
 	add_fcast_data(vwin);
+    } else if (vwin->role == LOESS || vwin->role == NADARWAT) {
+	add_nonparam_data(vwin);
     }
 
     if (dataset->v > oldv) {
@@ -409,7 +426,7 @@ static void reformat_callback (GtkWidget *w, windata_t *vwin)
     if (vwin->role == VIEW_MODELTABLE) {
 	format_model_table(vwin);
     } else {
-	series_view_format_dialog(w, vwin);
+	series_view_format_dialog(vwin);
     }
 }
 
@@ -461,7 +478,7 @@ static void coeffint_set_alpha (GtkWidget *w, windata_t *vwin)
     }
 
     dialog = gretl_dialog_new(_("gretl: coefficient confidence intervals"), 
-			      vwin->main, GRETL_DLG_BLOCK);
+			      vwin_toplevel(vwin), GRETL_DLG_BLOCK);
 
     hbox = gtk_hbox_new(FALSE, 5);
     hb2 = gtk_hbox_new(FALSE, 5);
@@ -524,7 +541,7 @@ static void coeffint_set_alpha (GtkWidget *w, windata_t *vwin)
     hbox = gtk_dialog_get_action_area(GTK_DIALOG(dialog));
 
     /* Cancel button */
-    cancel_options_button(hbox, dialog, NULL);
+    cancel_delete_button(hbox, dialog);
 
     g_object_set_data(G_OBJECT(dialog), "vwin", vwin);
     g_object_set_data(G_OBJECT(dialog), "xptr", &x);
@@ -548,7 +565,8 @@ static void set_output_sticky (GtkWidget *w, windata_t *vwin)
 
     deflt = (vwin->flags & VWIN_STICKY)? 1 : 0;
 
-    resp = radio_dialog(NULL, NULL, opts, 2, deflt, 0);
+    resp = radio_dialog(NULL, NULL, opts, 2, deflt, 0, 
+			vwin_toplevel(vwin));
 
     if (resp == 0) {
 	vwin->flags &= ~VWIN_STICKY;
@@ -557,9 +575,13 @@ static void set_output_sticky (GtkWidget *w, windata_t *vwin)
     }
 }
 
-static void compass_callback (GtkWidget *w, windata_t *vwin)
+static void toolbar_plot_callback (GtkWidget *w, windata_t *vwin)
 {
-    window_list_popup(vwin->main);
+    if (vwin->role == VIEW_SERIES) {
+	series_view_graph(w, vwin);
+    } else {
+	do_nonparam_plot(vwin);
+    }
 }
 
 static void activate_script_help (GtkWidget *widget, windata_t *vwin)
@@ -572,8 +594,8 @@ static void delete_file_viewer (GtkWidget *widget, windata_t *vwin)
 {
     gint resp = 0;
 
-    if (window_is_busy(vwin)) {
-	maybe_raise_dialog();
+    if (window_is_tab(vwin)) {
+	maybe_destroy_tabwin(vwin);
 	return;
     }
 
@@ -595,9 +617,11 @@ static int edit_script_popup_item (GretlToolItem *item)
 	!strcmp(item->icon, GTK_STOCK_FIND_AND_REPLACE);
 }
 
-static void set_plot_icon (GretlToolItem *item)
+static void set_plot_icon (GretlToolItem *item, int role)
 {
-    if (dataset_is_time_series(dataset)) {
+    if (role == LOESS || role == NADARWAT) {
+	item->icon = GRETL_STOCK_SCATTER;
+    } else if (dataset_is_time_series(dataset)) {
 	item->icon = GRETL_STOCK_TS;
     } else {
 	item->icon = GRETL_STOCK_BOX;
@@ -612,7 +636,7 @@ static void vwin_cut_callback (GtkWidget *w, windata_t *vwin)
 }
 
 static GretlToolItem viewbar_items[] = {
-    { N_("New window"), GTK_STOCK_NEW, G_CALLBACK(toolbar_new_callback), OPEN_ITEM },
+    { N_("New window"), GTK_STOCK_NEW, G_CALLBACK(toolbar_new_callback), NEW_ITEM },
     { N_("Open..."), GTK_STOCK_OPEN, G_CALLBACK(file_open_callback), OPEN_ITEM },
     { N_("Save"), GTK_STOCK_SAVE, G_CALLBACK(vwin_save_callback), SAVE_ITEM },
     { N_("Save as..."), GTK_STOCK_SAVE_AS, G_CALLBACK(save_as_callback), SAVE_AS_ITEM },
@@ -634,7 +658,7 @@ static GretlToolItem viewbar_items[] = {
     { N_("Scripts index"), GTK_STOCK_INDEX, G_CALLBACK(script_index), INDEX_ITEM },
     { N_("Confidence level..."), GRETL_STOCK_ALPHA, G_CALLBACK(coeffint_set_alpha), ALPHA_ITEM },
     { N_("LaTeX"), GRETL_STOCK_TEX, G_CALLBACK(window_tex_callback), TEX_ITEM },
-    { N_("Graph"), GRETL_STOCK_TS, G_CALLBACK(series_view_graph), PLOT_ITEM },
+    { N_("Graph"), GRETL_STOCK_TS, G_CALLBACK(toolbar_plot_callback), PLOT_ITEM },
     { N_("Reformat..."), GTK_STOCK_CONVERT, G_CALLBACK(reformat_callback), FORMAT_ITEM },
     { N_("Edit values..."), GRETL_STOCK_SHEET, G_CALLBACK(series_view_edit), EDITOR_ITEM },
     { N_("Refresh"), GTK_STOCK_REFRESH, G_CALLBACK(toolbar_refresh), REFRESH_ITEM },
@@ -647,7 +671,7 @@ static GretlToolItem viewbar_items[] = {
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(window_help), HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_gnuplot_help), GP_HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_x12a_help), X12A_HELP_ITEM },
-    { N_("Windows"), GRETL_STOCK_COMPASS, G_CALLBACK(compass_callback), 0 },
+    { N_("Windows"), GRETL_STOCK_COMPASS, GNULL, 0 },
     { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_file_viewer), 0 }
 };
 
@@ -663,6 +687,8 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 
 #define open_ok(r) (vwin_editing_script(r))
 
+#define new_ok(r) (vwin_editing_script(r))
+
 #define edit_ok(r) (vwin_editing_script(r) || \
 		    vwin_editing_buffer(r) || \
                     r == EDIT_PKG_CODE || \
@@ -686,11 +712,14 @@ static int n_viewbar_items = G_N_ELEMENTS(viewbar_items);
 			r == VIEW_SCRIPT || \
 			r == VIEW_LOG)
 
-#define sort_ok(r)    (r == VIEW_SERIES)
-#define plot_ok(r)    (r == VIEW_SERIES)
+#define sort_ok(r) (r == VIEW_SERIES)
+
+#define plot_ok(r) (r == VIEW_SERIES || \
+		    r == LOESS || r == NADARWAT)
 
 #define add_data_ok(r) (r == PCA || r == LEVERAGE || \
-                        r == MAHAL || r == FCAST)
+                        r == MAHAL || r == FCAST || \
+			r == LOESS || r == NADARWAT)
 
 #define split_ok(r) (r == SCRIPT_OUT)
 
@@ -713,6 +742,8 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (!open_ok(r) && f == OPEN_ITEM) {
 	return NULL;
+    } else if (!new_ok(r) && f == NEW_ITEM) {
+	return NULL;
     } else if (!exec_ok(r) && f == EXEC_ITEM) {
 	return NULL;
     } else if (!cmd_help_ok(r) && f == CMD_HELP_ITEM) {
@@ -771,9 +802,8 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 
 GtkWidget *gretl_toolbar_new (void)
 {
-    GtkWidget *tb;
+    GtkWidget *tb = gtk_toolbar_new();
 
-    tb = gtk_toolbar_new();
     gtk_toolbar_set_icon_size(GTK_TOOLBAR(tb), GTK_ICON_SIZE_MENU);
     gtk_toolbar_set_style(GTK_TOOLBAR(tb), GTK_TOOLBAR_ICONS);
     gtk_toolbar_set_show_arrow(GTK_TOOLBAR(tb), FALSE);
@@ -781,47 +811,60 @@ GtkWidget *gretl_toolbar_new (void)
     return tb;
 }
 
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 12)
-
-void gretl_tooltips_add (GtkWidget *w, const gchar *str)
-{
-    static GtkTooltips *gretl_tips;
-
-    if (gretl_tips == NULL) {
-	gretl_tips = gtk_tooltips_new();
-	gtk_tooltips_enable(gretl_tips); /* redundant? */
-    }
-
-    gtk_tooltips_set_tip(gretl_tips, w, str, NULL);
-}
-
-#else /* new tooltips API */
-
 void gretl_tooltips_add (GtkWidget *w, const gchar *str)
 {
     gtk_widget_set_tooltip_text(w, str);
 }
 
-#endif /* GTK variants */
-
-GtkToolItem *gretl_toolbar_insert (GtkWidget *tbar,
-				   GretlToolItem *item,
-				   GCallback func,
-				   gpointer data,
-				   gint pos)
+void vwin_toolbar_insert_winlist (windata_t *vwin)
 {
-    GtkToolItem *button;
+    GtkWidget *img, *button = gtk_button_new();
+    GtkToolItem *item = gtk_tool_item_new();
 
-    button = gtk_tool_button_new_from_stock(item->icon);
-#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION < 12)
-    gtk_tool_item_set_tooltip(button, gretl_tips, _(item->tip), NULL);
-#else
-    gtk_widget_set_tooltip_text(GTK_WIDGET(button), _(item->tip));
-#endif
-    g_signal_connect(button, "clicked", func, data);
-    gtk_toolbar_insert(GTK_TOOLBAR(tbar), button, pos);
+    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("Windows"));
+    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+    img = gtk_image_new_from_stock(GRETL_STOCK_COMPASS, 
+				   GTK_ICON_SIZE_MENU);
+    gtk_container_add(GTK_CONTAINER(button), img);
+    gtk_container_add(GTK_CONTAINER(item), button);
+    g_signal_connect(button, "button-press-event", 
+		     G_CALLBACK(vwin_winlist_popup), vwin);
+    gtk_toolbar_insert(GTK_TOOLBAR(vwin->mbar), item, -1);
+}
 
-    return button;
+GtkWidget *gretl_toolbar_insert (GtkWidget *tbar,
+				 GretlToolItem *tool,
+				 GCallback func,
+				 gpointer data,
+				 gint pos)
+{
+    GtkToolItem *item;
+
+    item = gtk_tool_button_new_from_stock(tool->icon);
+    gtk_widget_set_tooltip_text(GTK_WIDGET(item), _(tool->tip));
+    g_signal_connect(item, "clicked", func, data);
+    gtk_toolbar_insert(GTK_TOOLBAR(tbar), item, pos);
+
+    return GTK_WIDGET(item);
+}
+
+static GtkWidget *vwin_toolbar_insert (GretlToolItem *tool,
+				       GCallback func,
+				       windata_t *vwin,
+				       gint pos)
+{
+    GtkToolItem *item;
+
+    item = gtk_tool_button_new_from_stock(tool->icon);
+    if (tool->flag == NEW_ITEM && window_is_tab(vwin)) {
+	gtk_widget_set_tooltip_text(GTK_WIDGET(item), _("New tab"));
+    } else {
+	gtk_widget_set_tooltip_text(GTK_WIDGET(item), _(tool->tip));
+    }
+    g_signal_connect(item, "clicked", func, vwin);
+    gtk_toolbar_insert(GTK_TOOLBAR(vwin->mbar), item, pos);
+
+    return GTK_WIDGET(item);
 }
 
 static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
@@ -831,7 +874,7 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
     int latex_ok = latex_is_ok();
     int save_ok = (flags & VIEWBAR_EDITABLE);
     GtkWidget *hpane = NULL, *vpane = NULL;
-    GtkToolItem *button;
+    GtkWidget *button;
     GretlToolItem *item;
     GCallback func;
     int i;
@@ -839,6 +882,11 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
     for (i=0; i<n_viewbar_items; i++) {
 	item = &viewbar_items[i];
 
+	if (winlist_item(item)) {
+	    vwin_toolbar_insert_winlist(vwin);
+	    continue;
+	}
+
 	func = item_get_callback(item, vwin, latex_ok, sortby_ok,
 				 format_ok, save_ok);
 	if (func == NULL) {
@@ -846,16 +894,16 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
 	}
 
 	if (item->flag == PLOT_ITEM) {
-	    set_plot_icon(item);
+	    set_plot_icon(item, vwin->role);
 	}
 
-	button = gretl_toolbar_insert(vwin->mbar, item, func, vwin, -1);
+	button = vwin_toolbar_insert(item, func, vwin, -1);
 
 	if (func == (GCallback) split_pane_callback) {
 	    if (hpane == NULL) {
-		hpane = GTK_WIDGET(button);
+		hpane = button;
 	    } else {
-		vpane = GTK_WIDGET(button);
+		vpane = button;
 	    }
 	}
 
@@ -863,17 +911,17 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
 	    if (vwin->role != CONSOLE) {
 		/* nothing to save just yet */
 		g_object_set_data(G_OBJECT(vwin->mbar), "save_button", button); 
-		gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
+		gtk_widget_set_sensitive(button, FALSE);
 	    }
 	} else if (item->flag == SAVE_AS_ITEM) {
 	    g_object_set_data(G_OBJECT(vwin->mbar), "save_as_button", button);
 	    if (strstr(vwin->fname, "script_tmp")) {
-		gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
+		gtk_widget_set_sensitive(button, FALSE);
 	    }
 	} else if (item->flag == STOP_ITEM) {
+	    g_object_set_data(G_OBJECT(vwin->mbar), "stop_button", button);
 	    /* nothing to stop yet */
-	    g_object_set_data(G_OBJECT(vwin->mbar), "stop_button", button); 
-	    gtk_widget_set_sensitive(GTK_WIDGET(button), FALSE);
+	    gtk_widget_set_sensitive(button, FALSE);
 	}
     }
 
@@ -888,20 +936,15 @@ static void viewbar_add_items (windata_t *vwin, ViewbarFlags flags)
 
 void vwin_add_viewbar (windata_t *vwin, ViewbarFlags flags)
 {
-    GtkWidget *hbox;
-
     if ((flags & VIEWBAR_HAS_TEXT) || vwin->role == SCRIPT_OUT) {
-	g_object_set_data(G_OBJECT(vwin->main), "text_out", GINT_TO_POINTER(1));
+	g_object_set_data(G_OBJECT(vwin->main), "text_out", 
+			  GINT_TO_POINTER(1));
     }
 
-    hbox = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(vwin->vbox), hbox, FALSE, FALSE, 0);
-
     vwin->mbar = gretl_toolbar_new();
     viewbar_add_items(vwin, flags);
 
-    gtk_box_pack_start(GTK_BOX(hbox), vwin->mbar, FALSE, FALSE, 0);
-    gtk_widget_show_all(hbox);
+    vwin_pack_toolbar(vwin);
 }
 
 static void remove_child (GtkWidget *child, GtkWidget *cont)
@@ -915,6 +958,11 @@ void viewbar_add_edit_items (windata_t *vwin)
 			  (GtkCallback) remove_child, vwin->mbar);
     viewbar_add_items(vwin, VIEWBAR_EDITABLE);
     gtk_widget_show_all(vwin->mbar);
+
+    if (vwin->role == EDIT_SCRIPT && use_tabbed_editor() && 
+	!window_is_tab(vwin)) {
+	script_editor_show_new_open(vwin, FALSE);
+    }
 }
 
 GtkWidget *build_text_popup (windata_t *vwin)
@@ -950,6 +998,10 @@ GtkWidget *build_text_popup (windata_t *vwin)
 		}
 	    } else if (func == G_CALLBACK(text_undo) && !text_can_undo(vwin)) {
 		continue;
+	    } else if (func == G_CALLBACK(delete_file_viewer)) {
+		if (window_is_tab(vwin)) {
+		    continue;
+		}
 	    }
 	    w = gtk_menu_item_new_with_label(_(item->tip));
 	    g_signal_connect(G_OBJECT(w), "activate", func, vwin);
@@ -959,15 +1011,11 @@ GtkWidget *build_text_popup (windata_t *vwin)
     }
 
     if (vwin->role != EDIT_SCRIPT) {
-	w = gtk_separator_menu_item_new();
-	gtk_widget_show(w);
-	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), w);
-
-	w = gtk_menu_item_new_with_label(_("Main window"));
-	g_signal_connect(G_OBJECT(w), "activate", 
-			 G_CALLBACK(raise_main_window), NULL);
-	gtk_widget_show(w);
-	gtk_menu_shell_append(GTK_MENU_SHELL(pmenu), w);
+	if (window_is_undockable(vwin)) {
+	    add_undock_popup_item(pmenu, vwin);
+	} else if (window_is_dockable(vwin)) {
+	    add_dock_popup_item(pmenu, vwin);
+	}
     }
 
     return pmenu;
@@ -1007,9 +1055,8 @@ static void tbar_xy_graph (void)
 	} else if (mdata_selection_count() == 2) {
 	    plot_from_selection(GR_XY);
 	} else {
-	    selection_dialog(_("gretl: define graph"), 
-			     do_graph_from_selector,
-			     GR_XY);
+	    selection_dialog(GR_XY, _("gretl: define graph"), 
+			     do_graph_from_selector);
 	}
     } else {
 	warnbox(_("Please open a data file first"));
@@ -1019,7 +1066,7 @@ static void tbar_xy_graph (void)
 static void tbar_model (void)
 {
     if (data_status) {
-	selection_dialog(_("gretl: specify model"), do_model, OLS);
+	selection_dialog(OLS, _("gretl: specify model"), do_model);
     } else {
 	warnbox(_("Please open a data file first"));
     }
@@ -1051,6 +1098,7 @@ static GretlToolItem mainbar_items[] = {
     { N_("new script"),         GTK_STOCK_EDIT,      G_CALLBACK(tbar_new_script), 0 },
     { N_("open gretl console"), GRETL_STOCK_CONSOLE, G_CALLBACK(gretl_console), 0 },
     { N_("session icon view"),  GRETL_STOCK_ICONS,   G_CALLBACK(tbar_iconview), 0 },
+    { N_("Windows"),            GRETL_STOCK_COMPASS, GNULL, 0 },
     { N_("function packages"),  GRETL_STOCK_FUNC,    G_CALLBACK(tbar_show_funcs), 0 },
     { N_("user's guide"),       GRETL_STOCK_PDF,     G_CALLBACK(tbar_users_guide), 0 },
     { N_("command reference"),  GTK_STOCK_HELP,      G_CALLBACK(tbar_command_ref), 0 },
@@ -1062,18 +1110,23 @@ static GretlToolItem mainbar_items[] = {
 
 void add_mainwin_toolbar (GtkWidget *vbox)
 {
-    GtkWidget *hbox, *toolbar;
     GretlToolItem *item;
+    GtkWidget *hbox;
     int i, n = G_N_ELEMENTS(mainbar_items);
 
-    toolbar = gretl_toolbar_new();
+    mdata->mbar = gretl_toolbar_new();
 
     for (i=0; i<n; i++) {
 	item = &mainbar_items[i];
-	gretl_toolbar_insert(toolbar, item, item->func, mdata, -1);
+	if (winlist_item(item)) {
+	    vwin_toolbar_insert_winlist(mdata);
+	} else {
+	    gretl_toolbar_insert(mdata->mbar, item, item->func, mdata, -1);
+	}
     }
 
     hbox = gtk_hbox_new(FALSE, 0);
-    gtk_box_pack_start(GTK_BOX(hbox), toolbar, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(hbox), mdata->mbar, FALSE, FALSE, 0);
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 }
+
