@@ -25,6 +25,8 @@ typedef enum {
     VIEWBAR_HAS_TEXT = 1 << 1
 } ViewbarFlags;
 
+#define winlist_item(i) (strcmp(i->icon, GRETL_STOCK_COMPASS) == 0)
+
 void gretl_stock_icons_init (void);
 
 void add_mainwin_toolbar (GtkWidget *vbox);
@@ -43,10 +45,12 @@ void gretl_tooltips_add (GtkWidget *w, const gchar *str);
 
 GtkWidget *gretl_toolbar_new (void);
 
-GtkToolItem *gretl_toolbar_insert (GtkWidget *tbar,
-				   GretlToolItem *item,
-				   GCallback func,
-				   gpointer data,
-				   gint pos);
+GtkWidget *gretl_toolbar_insert (GtkWidget *tbar,
+				 GretlToolItem *item,
+				 GCallback func,
+				 gpointer data,
+				 gint pos);
+
+void vwin_toolbar_insert_winlist (windata_t *vwin);
 
 #endif /* TOOLBAR_H */
