@@ -23,6 +23,7 @@
 #include "dlgutils.h"
 #include "ssheet.h"
 #include "toolbar.h"
+#include "winstack.h"
 #include "varinfo.h"
 
 #define VSET_MAX_FIELDS 6
@@ -107,6 +108,7 @@ static void name_setter_init (name_setter *nset, char *vname,
     nset->descrip = descrip;
     nset->changed[0] = 0;
     nset->changed[1] = 0;
+    *cancel = 1;
 }
 
 /* see if anything has been changed via the varinfo dialog */
@@ -251,6 +253,7 @@ static void really_set_variable_info (GtkWidget *w, gui_varinfo *vset)
 
     if (vset->changed[VSET_VARNAME]) {
 	newstr = entry_get_trimmed_text(vset->name_entry);
+	/* note: do_rename_var() logs the corresponding command */
 	err = do_rename_variable(v, newstr);
 	g_free(newstr);
     }
@@ -290,6 +293,12 @@ static void really_set_variable_info (GtkWidget *w, gui_varinfo *vset)
     if (!err && vset->changed[VSET_DISCRETE]) {
 	ival = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vset->discrete_check));
 	set_var_discrete(dataset, v, ival);
+	if (ival) {
+	    lib_command_sprintf("setinfo %s --discrete", dataset->varname[v]);
+	} else {
+	    lib_command_sprintf("setinfo %s --continuous", dataset->varname[v]);
+	}
+	record_command_verbatim();
     }  
 
     if (!err) {
@@ -327,6 +336,8 @@ static void set_series_name_and_desc (GtkWidget *w, name_setter *nset)
     gchar *s = NULL;
     int err = 0;
 
+    *nset->cancel = 0;
+
     if (nset->changed[0]) {
 	/* series name: take care in allowing overwrite of existing 
 	   series */
@@ -395,6 +406,7 @@ static void name_setter_cancel (GtkWidget *w, name_setter *nset)
 
 static void free_name_setter (GtkWidget *w, name_setter *nset)
 {
+    set_dataset_locked(FALSE);
     free(nset);
 }
 
@@ -714,7 +726,8 @@ void varinfo_dialog (int varnum)
     }
 
     flags = GRETL_DLG_BLOCK | GRETL_DLG_RESIZE;
-    vset->dlg = gretl_dialog_new(_("gretl: variable attributes"), NULL, flags);
+    vset->dlg = gretl_dialog_new(_("gretl: variable attributes"), 
+				 NULL, flags);
     gui_varinfo_init(vset, varnum);
 
     is_parent = series_is_parent(dataset, varnum);
@@ -947,8 +960,8 @@ void varinfo_dialog (int varnum)
     gtk_widget_show(vset->dlg);
 }
 
-void name_new_variable_dialog (char *vname, char *descrip,
-			       int *cancel)
+void name_new_series_dialog (char *vname, char *descrip,
+			     windata_t *vwin, int *cancel)
 {
     GtkWidget *tmp, *vbox, *hbox;
     name_setter *nset;
@@ -963,8 +976,9 @@ void name_new_variable_dialog (char *vname, char *descrip,
 	return;
     }
 
-    flags = GRETL_DLG_MODAL | GRETL_DLG_BLOCK | GRETL_DLG_RESIZE;
-    nset->dlg = gretl_dialog_new(_("gretl: variable attributes"), NULL, flags);
+    flags = GRETL_DLG_BLOCK | GRETL_DLG_RESIZE;
+    nset->dlg = gretl_dialog_new(_("gretl: variable attributes"), 
+				 vwin_toplevel(vwin), flags);
 
     make_varname_unique(vname, 0, dataset);
     name_setter_init(nset, vname, descrip, cancel);
@@ -1036,6 +1050,7 @@ void name_new_variable_dialog (char *vname, char *descrip,
     gtk_widget_show(tmp);
 
     gtk_widget_show_all(hbox);
+    set_dataset_locked(TRUE);
     gtk_widget_show(nset->dlg);
 }
 
