@@ -32,19 +32,20 @@ enum {
     BOOT_NORMAL_U    = 1 << 3,  /* simulate normal residuals */
     BOOT_STUDENTIZE  = 1 << 4,  /* studentize, when doing confidence interval */
     BOOT_GRAPH       = 1 << 5,  /* graph the distribution */
-    BOOT_LDV         = 1 << 6,  /* model includes lagged dep var */
-    BOOT_RESTRICT    = 1 << 7,  /* called via "restrict" command */
-    BOOT_F_FORM      = 1 << 8,  /* compute F-statistics */
-    BOOT_FREE_RQ     = 1 << 9,  /* free restriction matrices */
-    BOOT_SAVE        = 1 << 10, /* save results vector */
-    BOOT_VERBOSE     = 1 << 11, /* for debugging */
-    BOOT_SILENT      = 1 << 12  /* suppress printed output */
+    BOOT_RESTRICT    = 1 << 6,  /* called via "restrict" command */
+    BOOT_F_FORM      = 1 << 7,  /* compute F-statistics */
+    BOOT_FREE_RQ     = 1 << 8,  /* free restriction matrices */
+    BOOT_SAVE        = 1 << 9,  /* save results vector */
+    BOOT_VERBOSE     = 1 << 10, /* verbose output */
+    BOOT_SILENT      = 1 << 11  /* suppress printed output */
 };
 
 #define resampling(b) (b->flags & BOOT_RESAMPLE_U)
 #define verbose(b) (b->flags & BOOT_VERBOSE)
+#define boot_Ftest(b) (b->flags & BOOT_F_FORM)
 
 typedef struct boot_ boot;
+typedef struct ldvinfo_ ldvinfo;
 
 struct boot_ {
     int flags;          /* option flags */
@@ -54,7 +55,6 @@ struct boot_ {
     int p;              /* index number of coeff to examine */
     int g;              /* number of restrictions */
     int mci;            /* model command index */
-    int ldvpos;         /* col. number of lagged dep var in X matrix */
     gretl_matrix *y;    /* holds original, then artificial, dep. var. */
     gretl_matrix *X;    /* independent variables */
     gretl_matrix *b0;   /* coefficients used to generate dep var */
@@ -70,13 +70,26 @@ struct boot_ {
     double a;           /* alpha, for confidence interval */
     double pval;        /* p-value for test */
     char vname[VNAMELEN]; /* name of variable analysed */
+    ldvinfo *ldv;       /* lagged depvar info, if applicable */
+};
+
+struct ldvinfo_ {
+    int n;      /* number of lagged dependent variable terms */
+    int *xcol;  /* location of such terms (0-based column of X) */
+    int *lag;   /* lag order of each such term */
 };
 
 static gretl_vector *bs_data;
 static char bs_vname[VNAMELEN];
 
+static void free_ldvinfo (ldvinfo *ldv);
+
 static void boot_destroy (boot *bs)
 {
+    if (bs == NULL) {
+	return;
+    }
+
     gretl_matrix_free(bs->y);
     gretl_matrix_free(bs->X);
     gretl_matrix_free(bs->b0);
@@ -88,6 +101,8 @@ static void boot_destroy (boot *bs)
 	gretl_matrix_free(bs->q);
     }
 
+    free_ldvinfo(bs->ldv);
+
     free(bs);
 }
 
@@ -184,11 +199,9 @@ static int make_bs_flags (gretlopt opt)
 
     if (opt & OPT_S) {
 	flags |= BOOT_SILENT;
-    }    
-
-#if BDEBUG > 1
-    flags |= BOOT_VERBOSE;
-#endif
+    } else if (opt & OPT_V) {
+	flags |= BOOT_VERBOSE;
+    }
 
     return flags;
 }
@@ -220,18 +233,108 @@ int maybe_adjust_B (int B, double a, int flags)
     return B;
 }
 
+static void free_ldvinfo (ldvinfo *ldv)
+{
+    if (ldv != NULL) {
+	free(ldv->xcol);
+	free(ldv->lag);
+	free(ldv);
+    }
+}
+
+/* add information on lagged dependent variables among the
+   regressors in a model to be subject to bootstrap
+   analysis
+*/
+
+static int add_ldvinfo (boot *b, int n)
+{
+    ldvinfo *ldv;
+
+    ldv = malloc(sizeof *ldv);
+    if (ldv == NULL) {
+	return E_ALLOC;
+    }
+
+    ldv->xcol = malloc(n * sizeof *ldv->xcol);
+    ldv->lag = malloc(n * sizeof *ldv->lag);
+
+    if (ldv->xcol == NULL || ldv->lag == NULL) {
+	free_ldvinfo(ldv);
+	return E_ALLOC;
+    }
+
+    ldv->n = n;
+    b->ldv = ldv;
+
+    return 0;
+}
+
+static int make_boot_ldvinfo (boot *b, const MODEL *pmod,
+			      const DATASET *dset)
+{
+    int xnum, ynum = pmod->list[1];
+    int i, p, nly = 0;
+    int leads = 0;
+    int err = 0;
+
+    for (i=0; i<pmod->ncoeff && !err; i++) {
+	xnum = pmod->list[i+2];
+	p = is_standard_lag_of(xnum, ynum, dset);
+	if (p > 0) {
+	    nly++;
+	} else if (p < 0) {
+	    leads++;
+	}
+    }
+
+    if (nly > 0 && leads > 0) {
+	gretl_errmsg_set("model contains leads of the dependent "
+			 "variable: not supported");
+	err = E_DATA;
+    }	
+
+    if (!err && nly > 0) {
+	int k = 0;
+
+	err = add_ldvinfo(b, nly);
+
+	if (!err) {
+	    for (i=0; i<pmod->ncoeff; i++) {
+		xnum = pmod->list[i+2];
+		p = is_standard_lag_of(xnum, ynum, dset);
+		if (p > 0) {
+		    b->ldv->xcol[k] = i;
+		    b->ldv->lag[k] = p;
+		    k++;
+		}
+	    }	    
+	}
+    }
+
+    return err;
+}
+
 static boot *boot_new (const MODEL *pmod,
 		       const DATASET *dset,
-		       int B, gretlopt opt)
+		       int B, gretlopt opt,
+		       int *err)
 {
     boot *bs;
-    int ldv;
 
     bs = malloc(sizeof *bs);
     if (bs == NULL) {
 	return NULL;
     }
 
+    bs->ldv = NULL;
+
+    *err = make_boot_ldvinfo(bs, pmod, dset);
+    if (*err) {
+	free(bs);
+	return NULL;
+    }
+
     bs->y = NULL;
     bs->X = NULL;
     bs->b0 = NULL;
@@ -255,14 +358,6 @@ static boot *boot_new (const MODEL *pmod,
     bs->p = 0;
     bs->g = 0;
 
-    ldv = gretl_model_get_int(pmod, "ldepvar");
-    if (ldv > 0) {
-	bs->flags |= BOOT_LDV;
-	bs->ldvpos = ldv - 2;
-    } else {
-	bs->ldvpos = -1;
-    }
-
     *bs->vname = '\0';
 
     bs->SE = NADBL;
@@ -278,10 +373,25 @@ static boot *boot_new (const MODEL *pmod,
     return bs;
 }
 
+static int ldv_lag (boot *bs, int k)
+{
+    if (bs->ldv != NULL) {
+	int i;
+
+	for (i=0; i<bs->ldv->n; i++) {
+	    if (bs->ldv->xcol[i] == k) {
+		return bs->ldv->lag[i];
+	    }
+	}
+    }
+
+    return 0;
+}
+
 static void make_normal_y (boot *bs)
 {
     double xti;
-    int i, t;
+    int i, t, p;
 
     /* generate scaled normal errors */
     gretl_matrix_random_fill(bs->y, D_NORMAL);
@@ -290,8 +400,9 @@ static void make_normal_y (boot *bs)
     /* construct y recursively */
     for (t=0; t<bs->X->rows; t++) {
 	for (i=0; i<bs->X->cols; i++) {
-	    if (t > 0 && i == bs->ldvpos) {
-		gretl_matrix_set(bs->X, t, i, bs->y->val[t-1]);
+	    p = ldv_lag(bs, i);
+	    if (p > 0 && t >= p) {
+		gretl_matrix_set(bs->X, t, i, bs->y->val[t-p]);
 	    } 
 	    xti = gretl_matrix_get(bs->X, t, i);
 	    bs->y->val[t] += bs->b0->val[i] * xti;
@@ -316,7 +427,7 @@ resample_vector (const gretl_matrix *u0, gretl_matrix *u, int *z)
 static void make_resampled_y (boot *bs, int *z)
 {
     double xti;
-    int i, t;
+    int i, t, p;
 
     /* resample the residuals, into y */
     resample_vector(bs->u0, bs->y, z);
@@ -324,8 +435,9 @@ static void make_resampled_y (boot *bs, int *z)
     /* construct y recursively */
     for (t=0; t<bs->X->rows; t++) {
 	for (i=0; i<bs->X->cols; i++) {
-	    if (t > 0 && i == bs->ldvpos) {
-		gretl_matrix_set(bs->X, t, i, bs->y->val[t-1]);
+	    p = ldv_lag(bs, i);
+	    if (p > 0 && t >= p) {
+		gretl_matrix_set(bs->X, t, i, bs->y->val[t-p]);
 	    }
 	    xti = gretl_matrix_get(bs->X, t, i);
 	    bs->y->val[t] += bs->b0->val[i] * xti;
@@ -354,7 +466,6 @@ static int do_restricted_ols (boot *bs)
 	    fprintf(stderr, "b[%d] = %g\n", i, bs->b0->val[i]);
 	}
 	fprintf(stderr, "s2 = %g\n", s2);
-	fprintf(stderr, "bs->ldvpos = %d\n", bs->ldvpos);
     }
 #endif
 
@@ -401,7 +512,9 @@ static int bs_store_result (boot *bs, double *xi)
 
 static void bs_print_result (boot *bs, double *xi, int tail, PRN *prn)
 {
-    if (bs->flags & BOOT_RESTRICT) {
+    if (bs->flags & BOOT_F_FORM) {
+	pprintf(prn, "\n%s: ", _("bootstrap F-test"));
+    } else if (bs->flags & BOOT_RESTRICT) {
 	pputs(prn, "\n  ");
     } else {
 	pprintf(prn, _("For the coefficient on %s (point estimate %g)"), 
@@ -448,7 +561,7 @@ static void bs_print_result (boot *bs, double *xi, int tail, PRN *prn)
     }
     pputc(prn, '\n');
 
-    if (bs->flags & BOOT_LDV) {
+    if (bs->ldv != NULL) {
 	pprintf(prn, "(%s)",  _("recognized lagged dependent variable"));
 	pputc(prn, '\n');
     }
@@ -680,6 +793,12 @@ static int hsk_transform_data (boot *bs, gretl_matrix *b,
 
 #endif
 
+static void print_test_round (boot *bs, int i, double test, PRN *prn)
+{
+    pprintf(prn, "round %d: test = %g%s\n", i+1, test,
+	    test > bs->test0 ? " *" : "");
+}
+
 /* do the actual bootstrap analysis: the objective is either to form a
    confidence interval or to compute a p-value; the methodology is
    either to resample the original residuals or to simulate normal
@@ -760,7 +879,11 @@ static int real_bootstrap (boot *bs, PRN *prn)
     }
 
     if (verbose(bs)) {
-	pprintf(prn, "%13s %13s\n", "b", "tval");
+	if (boot_Ftest(bs)) {
+	    pputc(prn, '\n');
+	} else {
+	    pprintf(prn, "%13s %13s\n", "b", "tval");
+	}
     }
 
     /* carry out B replications */
@@ -778,11 +901,18 @@ static int real_bootstrap (boot *bs, PRN *prn)
 	    make_resampled_y(bs, z); 
 	}
 
-	if (bs->ldvpos >= 0) {
-	    /* X matrix includes lagged dependent variable, so it has
+	if (bs->ldv != NULL) {
+	    /* X matrix includes lagged dependent variable(s), so it has
 	       to be modified */
-	    for (t=1; t<bs->T; t++) {
-		gretl_matrix_set(bs->X, t, bs->ldvpos, bs->y->val[t-1]);
+	    int j, p;
+
+	    for (j=0; j<bs->X->cols; j++) {
+		p = ldv_lag(bs, j);
+		if (p > 0) {
+		    for (t=p; t<bs->T; t++) {
+			gretl_matrix_set(bs->X, t, j, bs->y->val[t-p]);
+		    }
+		}
 	    }
 	    gretl_matrix_multiply_mod(bs->X, GRETL_MOD_TRANSPOSE,
 				      bs->X, GRETL_MOD_NONE,
@@ -831,6 +961,9 @@ static int real_bootstrap (boot *bs, PRN *prn)
 	    gretl_matrix_copy_values(V, XTXI);
 	    gretl_matrix_multiply_by_scalar(V, s2);
 	    test = bs_F_test(b, V, bs, &err);
+	    if (verbose(bs)) {
+		print_test_round(bs, i, test, prn);
+	    }
 	    if (test > bs->test0) {
 		tail++;
 	    }
@@ -947,10 +1080,7 @@ int bootstrap_analysis (MODEL *pmod, int p, int B,
 	return E_DATA;
     }
 
-    bs = boot_new(pmod, dset, B, opt);
-    if (bs == NULL) {
-	err = E_ALLOC;
-    }  
+    bs = boot_new(pmod, dset, B, opt, &err);
 
     if (!err && (bs->flags & BOOT_PVAL)) {
 	err = bs_add_restriction(bs, p);
@@ -996,9 +1126,9 @@ int bootstrap_analysis (MODEL *pmod, int p, int B,
  *
  * Calculates a bootstrap p-value for the restriction on the
  * coefficients of @pmod represented by the matrices @R and @q.
- * If the first lag of the dependent variable is present as a
- * regressor it is handled correctly but more complex lag
- * autoregressive schemes are not (yet) handled.
+ * If lags of the dependent variable are present as 
+ * regressors they should be handled correctly so long as
+ * the lagged terms were generated in the standard gretl manner.
  * 
  * Returns: 0 on success, non-zero code on error.
  */
@@ -1023,14 +1153,14 @@ int bootstrap_test_restriction (MODEL *pmod, gretl_matrix *R,
     if (opt & OPT_S) {
 	/* silent */
 	bopt |= OPT_S;
-    }    
+    } else if (opt & OPT_V) {
+	/* verbose */
+	bopt |= OPT_V;
+    }
 
     gretl_restriction_get_boot_params(&B, &bopt);
 
-    bs = boot_new(pmod, dset, B, bopt);
-    if (bs == NULL) {
-	err = E_ALLOC;
-    } 
+    bs = boot_new(pmod, dset, B, bopt, &err);
 
     if (!err) {
 	bs->R = R;
