@@ -688,7 +688,7 @@ static MODEL replicate_estimator (const MODEL *orig, int *list,
     int *full_list = NULL;
     char altparm[32] = {0};
     int mc = get_model_count();
-    int repci = orig->ci;
+    int cv, repci = orig->ci;
     int order = 0;
     int first = 1;
 
@@ -698,6 +698,12 @@ static MODEL replicate_estimator (const MODEL *orig, int *list,
 
     transcribe_option_flags(&myopt, orig->opt, OPT_D | OPT_J | OPT_R);
 
+    cv = gretl_model_get_cluster_var(orig);
+    if (cv > 0 && cv < dset->v) {
+	myopt |= OPT_C;
+	set_optval_string(orig->ci, OPT_C, dset->varname[cv]);
+    }    
+
     if (orig->ci == AR1) {
 	if (orig->opt & OPT_H) {
 	    myopt |= OPT_H;
@@ -1024,9 +1030,9 @@ int nonlinearity_test (MODEL *pmod, DATASET *dset, ModelAuxCode aux,
     impose_model_smpl(pmod, dset);
 
     /* add squares or logs */
-    tmplist = augment_regression_list(pmod->list, aux, dset);
-    if (tmplist == NULL) {
-	return E_ALLOC;
+    tmplist = augment_regression_list(pmod->list, aux, dset, &err);
+    if (err) {
+	return err;
     } else if (tmplist[0] == pmod->list[0]) {
 	/* no vars were added */
 	if (aux == AUX_SQ) {
@@ -1683,6 +1689,9 @@ double get_DW_pvalue_for_model (const MODEL *pmod, DATASET *dset,
 
     if (!*err) {
 	pv = gretl_model_get_double(&dwmod, "dw_pval");
+	if (na(pv)) {
+	    *err = E_DATA;
+	}
     }
 
     /* put back into dset what was there on input */
@@ -3444,10 +3453,11 @@ int add_leverage_values_to_dataset (DATASET *dset, gretl_matrix *m,
 }
 
 /**
- * leverage_test:
- * @pmod: pointer to model to be tested.
- * @dset: dataset struct.
- * @opt: if OPT_S, add calculated series to data set.
+ * leverage_test: 
+ * @pmod: pointer to model to be tested.  
+ * @dset: dataset struct.  
+ * @opt: if OPT_S, add calculated series to data set; operate 
+ * silently if OPT_Q.  
  * @prn: gretl printing struct.
  *
  * Tests the data used in the given model for points with
@@ -3474,7 +3484,7 @@ int leverage_test (MODEL *pmod, DATASET *dset,
 	return 1;
     }
 
-    m = (*model_leverage)(pmod, dset, OPT_NONE, prn, &err);
+    m = (*model_leverage)(pmod, dset, opt, prn, &err);
 
     if (!err && (opt & OPT_S)) {
 	err = add_leverage_values_to_dataset(dset, m, 
