@@ -39,8 +39,8 @@ enum {
     CSV_OBS1     = 1 << 5,
     CSV_TRAIL    = 1 << 6,
     CSV_AUTONAME = 1 << 7,
-    CSV_NONNUM   = 1 << 8,
-    CSV_REVERSED = 1 << 9
+    CSV_REVERSED = 1 << 8,
+    CSV_DOTSUB   = 1 << 9
 };
 
 typedef struct csvdata_ csvdata;
@@ -48,6 +48,7 @@ typedef struct csvdata_ csvdata;
 struct csvdata_ {
     int flags;
     char delim;
+    char decpoint;
     int markerpd;
     int maxlen;
     int real_n;
@@ -72,8 +73,8 @@ struct csvdata_ {
 #define csv_autoname(c)           (c->flags & CSV_AUTONAME)
 #define csv_skip_column(c)        (c->flags & (CSV_OBS1 | CSV_BLANK1))
 #define csv_have_data(c)          (c->flags & CSV_HAVEDATA)
-#define csv_force_nonnum(c)       (c->flags & CSV_NONNUM)
 #define csv_data_reversed(c)      (c->flags & CSV_REVERSED)
+#define csv_do_dotsub(c)          (c->flags & CSV_DOTSUB)
 
 #define csv_set_trailing_comma(c)   (c->flags |= CSV_TRAIL)
 #define csv_unset_trailing_comma(c) (c->flags &= ~CSV_TRAIL)
@@ -83,8 +84,8 @@ struct csvdata_ {
 #define csv_set_got_semi(c)         (c->flags |= CSV_GOTSEMI)
 #define csv_set_got_delim(c)        (c->flags |= CSV_GOTDELIM)
 #define csv_set_autoname(c)         (c->flags |= CSV_AUTONAME)
-#define csv_set_force_nonnum(c)     (c->flags |= CSV_NONNUM)
 #define csv_set_data_reversed(c)    (c->flags |= CSV_REVERSED)
+#define csv_set_dotsub(c)           (c->flags |= CSV_DOTSUB)
 
 #define csv_skipping(c)        (*c->skipstr != '\0')
 #define csv_has_non_numeric(c) (c->st != NULL)
@@ -124,7 +125,7 @@ static void csvdata_free (csvdata *c)
     free(c);
 }
 
-static csvdata *csvdata_new (DATASET *dset, gretlopt opt)
+static csvdata *csvdata_new (DATASET *dset)
 {
     csvdata *c = malloc(sizeof *c);
 
@@ -132,7 +133,7 @@ static csvdata *csvdata_new (DATASET *dset, gretlopt opt)
 	return NULL;
     }
 
-    c->flags = (opt & OPT_C)? CSV_NONNUM : 0;
+    c->flags = 0;
     c->delim = '\t';
     c->markerpd = -1;
     c->maxlen = 0;
@@ -154,10 +155,8 @@ static csvdata *csvdata_new (DATASET *dset, gretlopt opt)
 	free(c);
 	c = NULL;
     } else {
-	if (dset != NULL) {
-	    c->delim = c->dset->delim = dset->delim;
-	}
-	c->dset->delim = c->delim;
+	c->delim = get_data_export_delimiter();
+	c->decpoint = get_data_export_decpoint();
 	if (dset->Z != NULL) {
 	    c->flags |= CSV_HAVEDATA;
 	}
@@ -422,22 +421,22 @@ static int check_daily_dates (DATASET *dset, int *pd,
 	    err = 1;
 	} else {
 	    nmiss = fulln - T;
-	    pprintf(prn, M_("Observations: %d; days in sample: %d\n"), 
+	    pprintf(prn, A_("Observations: %d; days in sample: %d\n"), 
 		    T, fulln);
 	    if (nmiss > 300 * T) {
-		pprintf(prn, M_("Probably annual data\n"));
+		pprintf(prn, A_("Probably annual data\n"));
 		*pd = 1;
 	    } else if (nmiss > 50 * T) {
-		pprintf(prn, M_("Probably quarterly data\n"));
+		pprintf(prn, A_("Probably quarterly data\n"));
 		*pd = 4;
 	    } else if (nmiss > 20 * T) {
-		pprintf(prn, M_("Probably monthly data\n"));
+		pprintf(prn, A_("Probably monthly data\n"));
 		*pd = 12;
 	    } else if (nmiss > 3 * T) {
-		pprintf(prn, M_("Probably weekly data\n"));
+		pprintf(prn, A_("Probably weekly data\n"));
 		*pd = dset->pd = 52;
 	    } else {
-		pprintf(prn, M_("Missing daily observations: %d\n"), nmiss);
+		pprintf(prn, A_("Missing daily observations: %d\n"), nmiss);
 	    }
 	}
     }
@@ -689,7 +688,7 @@ void reverse_data (DATASET *dset, PRN *prn)
     int T = dset->n / 2;
     int i, t, s;
 
-    pprintf(prn, M_("reversing the data!\n"));
+    pprintf(prn, A_("reversing the data!\n"));
 
     for (t=0; t<T; t++) {
 	s = dset->n - 1 - t;
@@ -728,19 +727,19 @@ csv_daily_date_check (DATASET *dset, int *reversed,
     tryagain:
 
 	if (dorder == YYYYMMDD) {
-	    pputs(prn, M_("Trying date order YYYYMMDD\n"));
+	    pputs(prn, A_("Trying date order YYYYMMDD\n"));
 	    mon1 = d1[1];
 	    day1 = d1[2];
 	    mon2 = d2[1];
 	    day2 = d2[2];
 	} else if (dorder == DDMMYYYY) {
-	    pputs(prn, M_("Trying date order DDMMYYYY\n"));
+	    pputs(prn, A_("Trying date order DDMMYYYY\n"));
 	    day1 = d1[0];
 	    mon1 = d1[1];
 	    day2 = d2[0];
 	    mon2 = d2[1];
 	} else {
-	    pputs(prn, M_("Trying date order MMDDYYYY\n"));
+	    pputs(prn, A_("Trying date order MMDDYYYY\n"));
 	    mon1 = d1[0];
 	    day1 = d1[1];
 	    mon2 = d2[0];
@@ -757,7 +756,7 @@ csv_daily_date_check (DATASET *dset, int *reversed,
 		    return -1;
 		}
 	    }
-	    pprintf(prn, M_("Could be %s - %s\n"), lbl1, lbl2);
+	    pprintf(prn, A_("Could be %s - %s\n"), lbl1, lbl2);
 	    ret = check_daily_dates(dset, &pd, reversed, prn);
 	    if (ret >= 0 && pd > 0) {
 		if (pd == 52) {
@@ -782,7 +781,7 @@ csv_daily_date_check (DATASET *dset, int *reversed,
 	    return ret;
 	}
     } else {
-	pprintf(prn, M_("'%s' and '%s': couldn't get dates\n"), lbl1, lbl2);
+	pprintf(prn, A_("'%s' and '%s': couldn't get dates\n"), lbl1, lbl2);
     }
 
     return -1;
@@ -800,15 +799,15 @@ static int pd_from_date_label (const char *lbl, char *year, char *subp,
     try = atoi(year);
 
     if (try > 0 && try < 3000) {
-	pprintf(prn, M_("   %s: probably a year... "), year);
+	pprintf(prn, A_("   %s: probably a year... "), year);
     } else {
-	pprintf(prn, M_("   %s: probably not a year\n"), year);
+	pprintf(prn, A_("   %s: probably not a year\n"), year);
     }
 
     if (len == 5) {
-	pputs(prn, M_("   but I can't make sense of the extra bit\n"));
+	pputs(prn, A_("   but I can't make sense of the extra bit\n"));
     } else if (len == 4) {
-	pputs(prn, M_("and just a year\n"));
+	pputs(prn, A_("and just a year\n"));
 	pd = 1;
     } else {
 	char sep = lbl[4];
@@ -823,7 +822,7 @@ static int pd_from_date_label (const char *lbl, char *year, char *subp,
 		if (len == 7) s++;
 		p = atoi(s);
 		if (p > 0 && p < 5) {
-		    pprintf(prn, M_("quarter %s?\n"), s);
+		    pprintf(prn, A_("quarter %s?\n"), s);
 		    pd = 4;
 		} else {
 		    pprintf(prn, "quarter %d: not possible\n", p);
@@ -831,7 +830,7 @@ static int pd_from_date_label (const char *lbl, char *year, char *subp,
 	    } else if (len == 7) {
 		p = atoi(s);
 		if (p > 0 && p < 13) {
-		    pprintf(prn, M_("month %s?\n"), s);
+		    pprintf(prn, A_("month %s?\n"), s);
 		    pd = 12;
 		} else {
 		    pprintf(prn, "month %d: not possible\n", p);
@@ -866,7 +865,7 @@ static int time_series_label_check (DATASET *dset, int reversed,
 	    dset->sd0 = atof(dset->stobs);
 	    strcpy(dset->endobs, lbl2);
 	} else {
-	    pputs(prn, M_("   but the dates are not complete and consistent\n"));
+	    pputs(prn, A_("   but the dates are not complete and consistent\n"));
 	    pd = -1;
 	}
     } else if (pd == 4 || pd == 12) {
@@ -884,7 +883,7 @@ static int time_series_label_check (DATASET *dset, int reversed,
 	    dset->sd0 = obs_str_to_double(dset->stobs);
 	    ntodate(dset->endobs, dset->n - 1, dset);
 	} else {
-	    pputs(prn, M_("   but the dates are not complete and consistent\n"));
+	    pputs(prn, A_("   but the dates are not complete and consistent\n"));
 	    pd = -1;
 	}
     }
@@ -906,7 +905,7 @@ static int dates_maybe_reversed (const char *s1, const char *s2,
     ret = atoi(d1) > atoi(d2);
 
     if (ret) {
-	pputs(prn, M_("   dates are reversed?\n"));
+	pputs(prn, A_("   dates are reversed?\n"));
     }
     
     return ret;
@@ -1012,7 +1011,7 @@ int test_markers_for_dates (DATASET *dset, int *reversed,
 	return time_series_label_check(dset, *reversed, skipstr, prn);
     }
 
-    pprintf(prn, M_("   first row label \"%s\", last label \"%s\"\n"), 
+    pprintf(prn, A_("   first row label \"%s\", last label \"%s\"\n"), 
 	    lbl1, lbl2);
 
     /* are the labels (probably) just 1, 2, 3 etc.? */
@@ -1029,11 +1028,13 @@ int test_markers_for_dates (DATASET *dset, int *reversed,
 
     /* labels are of different lengths? */
     if (len1 != strlen(lbl2)) {
-	pputs(prn, M_("   label strings can't be consistent dates\n"));
+#if 0
+	pputs(prn, A_("   label strings can't be consistent dates\n"));
+#endif
 	return -1;
     }
 
-    pputs(prn, M_("trying to parse row labels as dates...\n"));
+    pputs(prn, A_("trying to parse row labels as dates...\n"));
 
     if (len1 == 8 || len1 == 10) {
 	/* daily data? */
@@ -1047,7 +1048,7 @@ int test_markers_for_dates (DATASET *dset, int *reversed,
 	    *reversed = dates_maybe_reversed(lbl1, lbl2, prn);
 	    pd = time_series_label_check(dset, *reversed, skipstr, prn);
 	} else {
-	    pputs(prn, M_("   definitely not a four-digit year\n"));
+	    pputs(prn, A_("   definitely not a four-digit year\n"));
 	}
     }
 
@@ -1132,7 +1133,7 @@ static int csv_max_line_length (FILE *fp, csvdata *cdata, PRN *prn)
 	cbak = c;
 	if (!isspace((unsigned char) c) && !isprint((unsigned char) c) &&
 	    !(c == CTRLZ) && !utf8_ok(fp, cc)) {
-	    pprintf(prn, M_("Binary data (%d) encountered (line %d:%d): "
+	    pprintf(prn, A_("Binary data (%d) encountered (line %d:%d): "
 			    "this is not a valid text file\n"), 
 		    c, lines, cc);
 	    return -1;
@@ -1155,9 +1156,9 @@ static int csv_max_line_length (FILE *fp, csvdata *cdata, PRN *prn)
     }
 
     if (maxlen == 0) {
-	pprintf(prn, M_("Data file is empty\n"));
+	pprintf(prn, A_("Data file is empty\n"));
     } else if (csv_has_trailing_comma(cdata)) {
-	pprintf(prn, M_("Data file has trailing commas\n"));
+	pprintf(prn, A_("Data file has trailing commas\n"));
     }
 
     if (maxlen > 0) {
@@ -1289,10 +1290,10 @@ static void check_first_field (const char *line, csvdata *c, PRN *prn)
 	field1[i] = '\0';
 	iso_to_ascii(field1);
 
-	pprintf(prn, M_("   first field: '%s'\n"), field1);
+	pprintf(prn, A_("   first field: '%s'\n"), field1);
 
 	if (import_obs_label(field1)) {
-	    pputs(prn, M_("   seems to be observation label\n"));
+	    pputs(prn, A_("   seems to be observation label\n"));
 	    csv_set_obs_column(c);
 	}
     }
@@ -1305,7 +1306,7 @@ static int csv_missval (const char *str, int i, int t,
 
     if (*str == '\0') {
 	if (t < 80 || *miss_shown < i) {
-	    pprintf(prn, M_("   the cell for variable %d, obs %d "
+	    pprintf(prn, A_("   the cell for variable %d, obs %d "
 			    "is empty: treating as missing value\n"), 
 		    i, t);
 	    *miss_shown += 1;
@@ -1315,7 +1316,7 @@ static int csv_missval (const char *str, int i, int t,
 
     if (import_na_string(str)) {
 	if (t < 80 || *miss_shown < i) {
-	    pprintf(prn, M_("   warning: missing value for variable "
+	    pprintf(prn, A_("   warning: missing value for variable "
 			    "%d, obs %d\n"), i, t);
 	    *miss_shown += 1;
 	}
@@ -1332,14 +1333,10 @@ static int non_numeric_check (csvdata *c, PRN *prn)
     int err = 0;
 
     for (i=1; i<c->dset->v; i++) {
-	if (is_codevar(c->dset->varname[i])) {
-	    nn++;
-	} else {
-	    for (t=0; t<c->dset->n; t++) {
-		if (c->dset->Z[i][t] == NON_NUMERIC) {
-		    nn++;
-		    break;
-		}
+	for (t=0; t<c->dset->n; t++) {
+	    if (c->dset->Z[i][t] == NON_NUMERIC) {
+		nn++;
+		break;
 	    }
 	}
     }
@@ -1354,14 +1351,10 @@ static int non_numeric_check (csvdata *c, PRN *prn)
     if (list != NULL) {
 	j = 1;
 	for (i=1; i<c->dset->v; i++) {
-	    if (is_codevar(c->dset->varname[i])) {
-		list[j++] = i;
-	    } else {
-		for (t=0; t<c->dset->n; t++) {
-		    if (c->dset->Z[i][t] == NON_NUMERIC) {
-			list[j++] = i;
-			break;
-		    }
+	    for (t=0; t<c->dset->n; t++) {
+		if (c->dset->Z[i][t] == NON_NUMERIC) {
+		    list[j++] = i;
+		    break;
 		}
 	    }
 	}
@@ -1369,36 +1362,32 @@ static int non_numeric_check (csvdata *c, PRN *prn)
 	printlist(list, "non-numeric vars list");
 
 	for (i=1; i<=list[0]; i++) {
+	    double nnfrac;
+	    int nnon = 0;
+	    int nok = 0;
+	    int tn = 0;
 	    int v = list[i];
-	    int cv = is_codevar(c->dset->varname[v]);
 
 	    series_set_flag(c->dset, v, VAR_DISCRETE);
 
-	    if (!cv) {
-		double nnfrac;
-		int nnon = 0;
-		int nok = 0;
-		int tn = 0;
-
-		for (t=0; t<c->dset->n; t++) {
-		    if (c->dset->Z[v][t] == NON_NUMERIC) {
-			if (!tn) tn = t + 1;
-			nnon++;
-		    } else if (!na(c->dset->Z[v][t])) {
-			nok++;
-		    }
+	    for (t=0; t<c->dset->n; t++) {
+		if (c->dset->Z[v][t] == NON_NUMERIC) {
+		    if (!tn) tn = t + 1;
+		    nnon++;
+		} else if (!na(c->dset->Z[v][t])) {
+		    nok++;
 		}
+	    }
 
-		nnfrac = (nok == 0)? 1.0 : (double) nnon / (nnon + nok);
-		pprintf(prn, "variable %d (%s): non-numeric values = %d "
-			"(%.2f percent)\n", v, c->dset->varname[v], 
-			nnon, 100 * nnfrac);
-		if (!csv_force_nonnum(c) && (nnon < 2 || nnfrac < 0.01)) {
-		    pprintf(prn, M_("ERROR: variable %d (%s), observation %d, "
-				    "non-numeric value\n"), 
-			    v, c->dset->varname[v], tn);
-		    err = E_DATA;
-		}
+	    nnfrac = (nok == 0)? 1.0 : (double) nnon / (nnon + nok);
+	    pprintf(prn, "variable %d (%s): non-numeric values = %d "
+		    "(%.2f percent)\n", v, c->dset->varname[v], 
+		    nnon, 100 * nnfrac);
+	    if (nnon < 2 || nnfrac < 0.01) {
+		pprintf(prn, A_("ERROR: variable %d (%s), observation %d, "
+				"non-numeric value\n"), 
+			v, c->dset->varname[v], tn);
+		err = E_DATA;
 	    }
 	}
 
@@ -1417,31 +1406,53 @@ static int non_numeric_check (csvdata *c, PRN *prn)
     return err;
 }
 
-/* remedial function: we may be trying to read a "CSV" file
-   that uses ',' as decimal separator, on a platform where
-   '.' is the decimal separator
-*/
-
-static double try_comma_atof (const char *orig)
+static double csv_atof (csvdata *c, const char *s)
 {
-    char *test, s[32];
-    double x;
-
-    if (strlen(orig) > 31) {
-	return NON_NUMERIC;
-    }
-
-    strcpy(s, orig);
-    charsub(s, ',', '.');
+    double x = NON_NUMERIC;
+    char *test;
 
     errno = 0;
-    x = strtod(s, &test);
 
-    if (*test == '\0' && errno != ERANGE) {
-	return x;
-    } else {
-	return NON_NUMERIC;
+    if (c->decpoint == '.' || !csv_do_dotsub(c) || strchr(s, ',') == NULL) {
+	/* we should currently be set to the correct locale,
+	   or there's no problematic decimal point in @s
+	*/
+	x = strtod(s, &test);
+	if (*test == '\0' && errno == 0) {
+	    return x;
+	} else {
+	    x = NON_NUMERIC;
+	}
+    } else if (csv_do_dotsub(c) && strlen(s) <= 31) {
+	/* substitute dot for comma */
+	char tmp[32];
+
+	strcpy(tmp, s);
+	charsub(tmp, ',', '.');
+
+	x = strtod(tmp, &test);
+	if (*test == '\0' && errno == 0) {
+	    return x;
+	} else {
+	    x = NON_NUMERIC;
+	}
     }
+
+    if (c->decpoint == '.' && strchr(s, ',') != NULL && strlen(s) <= 31) {
+	/* try remediation for decimal comma? */
+	char tmp[32];
+
+	strcpy(tmp, s);
+	charsub(tmp, ',', '.');
+	errno = 0;
+
+	x = strtod(tmp, &test);
+	if (*test != '\0' || errno != 0) {
+	    x = NON_NUMERIC;
+	} 
+    }
+
+    return x;
 }
 
 static int process_csv_obs (csvdata *c, int i, int t, int *miss_shown,
@@ -1460,16 +1471,9 @@ static int process_csv_obs (csvdata *c, int i, int t, int *miss_shown,
 	}
     } else if (csv_missval(c->str, i, t+1, miss_shown, prn)) {
 	c->dset->Z[i][t] = NADBL;
-    } else if (check_atof(c->str)) {
-	if (c->delim != ',' && get_local_decpoint() != ',' &&
-	    strchr(c->str, ',') != NULL) {
-	    c->dset->Z[i][t] = try_comma_atof(c->str);
-	} else {
-	    c->dset->Z[i][t] = NON_NUMERIC;
-	}
     } else {
-	c->dset->Z[i][t] = atof(c->str);
-    }
+	c->dset->Z[i][t] = csv_atof(c, c->str);
+    } 
 
     return err;
 }
@@ -1610,9 +1614,9 @@ static int csv_fields_check (FILE *fp, csvdata *c, PRN *prn)
 	chkcols = count_csv_fields(c->line, c->delim);
 	if (c->ncols == 0) {
 	    c->ncols = chkcols;
-	    pprintf(prn, M_("   number of columns = %d\n"), c->ncols);	    
+	    pprintf(prn, A_("   number of columns = %d\n"), c->ncols);	    
 	} else if (chkcols != c->ncols) {
-	    pprintf(prn, M_("   ...but row %d has %d fields: aborting\n"),
+	    pprintf(prn, A_("   ...but row %d has %d fields: aborting\n"),
 		    c->nrows, chkcols);
 	    err = E_DATA;
 	}
@@ -1660,7 +1664,7 @@ static int csv_varname_scan (csvdata *c, FILE *fp, PRN *prn, PRN *mprn)
     int i, k, numcount;
     int err = 0;
 
-    pputs(mprn, M_("scanning for variable names...\n"));
+    pputs(mprn, A_("scanning for variable names...\n"));
 
     while (csv_fgets(c->line, c->maxlen, fp)) {
 	if (*c->line == '#' || string_is_blank(c->line)) {
@@ -1675,7 +1679,7 @@ static int csv_varname_scan (csvdata *c, FILE *fp, PRN *prn, PRN *mprn)
     p = c->line;
     if (c->delim == ' ' && *p == ' ') p++;
     iso_to_ascii(p);
-    pprintf(mprn, M_("   line: %s\n"), p);
+    pprintf(mprn, A_("   line: %s\n"), p);
     
     numcount = 0;
 
@@ -1699,8 +1703,8 @@ static int csv_varname_scan (csvdata *c, FILE *fp, PRN *prn, PRN *mprn)
 	    nv = (csv_skip_column(c))? k : k + 1;
 
 	    if (*c->str == '\0') {
-		pprintf(prn, M_("   variable name %d is missing: aborting\n"), nv);
-		pputs(prn, M_(csv_msg));
+		pprintf(prn, A_("   variable name %d is missing: aborting\n"), nv);
+		pputs(prn, A_(csv_msg));
 		err = E_DATA;
 	    } else {
 		c->dset->varname[nv][0] = 0;
@@ -1726,7 +1730,7 @@ static int csv_varname_scan (csvdata *c, FILE *fp, PRN *prn, PRN *mprn)
 
     if (numcount == c->dset->v - 1 || 
 	obs_labels_no_varnames(obscol, c->dset, numcount)) {
-	pputs(prn, M_("it seems there are no variable names\n"));
+	pputs(prn, A_("it seems there are no variable names\n"));
 	/* then we undercounted the observations by one */
 	if (add_single_obs(c->dset)) {
 	    err = E_ALLOC;
@@ -1791,15 +1795,8 @@ static int fixed_format_read (csvdata *c, FILE *fp, PRN *prn)
 	    strncat(c->str, p, n);
 	    if (csv_missval(c->str, i, t+1, &miss_shown, prn)) {
 		c->dset->Z[i][t] = NADBL;
-	    } else if (check_atof(c->str)) {
-		if (c->delim != ',' && get_local_decpoint() != ',' &&
-		    strchr(c->str, ',') != NULL) {
-		    c->dset->Z[i][t] = try_comma_atof(c->str);
-		} else {
-		    err = E_DATA;
-		}
 	    } else {
-		c->dset->Z[i][t] = atof(c->str);
+		c->dset->Z[i][t] = csv_atof(c, c->str);
 	    }
 	}
 
@@ -1874,7 +1871,7 @@ real_read_labels_and_data (csvdata *c, FILE *fp, PRN *prn)
 		if (i < CSVSTRLEN - 1) {
 		    c->str[i++] = *p;
 		} else {
-		    pprintf(prn, M_("warning: truncating data at row %d, column %d\n"),
+		    pprintf(prn, A_("warning: truncating data at row %d, column %d\n"),
 			    t+1, k+1);
 		}
 		p++;
@@ -1922,7 +1919,7 @@ static int csv_read_data (csvdata *c, FILE *fp, PRN *prn, PRN *mprn)
     int reversed = csv_data_reversed(c);
     int err;
 
-    pputs(mprn, M_("scanning for row labels and data...\n"));
+    pputs(mprn, A_("scanning for row labels and data...\n"));
     err = real_read_labels_and_data(c, fp, prn);
 
     if (!err && csv_skip_column(c)) {
@@ -1941,10 +1938,10 @@ static void print_csv_parsing_header (const char *fname, PRN *prn)
     if (!g_utf8_validate(fname, -1, NULL)) {
 	gchar *trfname = g_locale_to_utf8(fname, -1, NULL, NULL, NULL);
 
-	pprintf(prn, "%s %s...\n", M_("parsing"), trfname);
+	pprintf(prn, "%s %s...\n", A_("parsing"), trfname);
 	g_free(trfname);
     } else {
-	pprintf(prn, "%s %s...\n", M_("parsing"), fname);
+	pprintf(prn, "%s %s...\n", A_("parsing"), fname);
     }
 }
 
@@ -1952,8 +1949,8 @@ static void print_csv_parsing_header (const char *fname, PRN *prn)
  * import_csv:
  * @fname: name of CSV file.
  * @dset: dataset struct.
- * @opt: use OPT_C to force interpretation of data colums containing
- * strings as coded values and not errors; for use of OPT_T see
+ * @opt: use OPT_N to force interpretation of data colums containing
+ * strings as coded (non-numeric) values and not errors; for use of OPT_T see
  * the help for "append".
  * @prn: gretl printing struct (or NULL).
  * 
@@ -1968,12 +1965,11 @@ int import_csv (const char *fname, DATASET *dset,
 {
     csvdata *c = NULL;
     const char *cols = NULL;
-    int popit = 0;
     FILE *fp = NULL;
     PRN *mprn = NULL;
     int newdata = (dset->Z == NULL);
-    char save_delim = dset->delim;
     int fixed_format = 0;
+    int popit = 0;
     long datapos;
     int i, err = 0;
 
@@ -1988,7 +1984,7 @@ int import_csv (const char *fname, DATASET *dset,
     }
 
     if (prn != NULL) {
-	check_for_console(prn);
+	set_alt_gettext_mode(prn);
     }
 
     if (gretl_messages_on()) {
@@ -1997,12 +1993,12 @@ int import_csv (const char *fname, DATASET *dset,
 
     fp = gretl_fopen(fname, "r");
     if (fp == NULL) {
-	pprintf(prn, M_("Couldn't open %s\n"), fname);
+	pprintf(prn, A_("Couldn't open %s\n"), fname);
 	err = E_FOPEN;
 	goto csv_bailout;
     }
 
-    c = csvdata_new(dset, opt);
+    c = csvdata_new(dset);
     if (c == NULL) {
 	err = E_ALLOC;
 	goto csv_bailout;
@@ -2010,7 +2006,7 @@ int import_csv (const char *fname, DATASET *dset,
 
     if (cols != NULL && *cols != '\0') {
 	fixed_format = 1;
-	pprintf(mprn, M_("using fixed column format\n"));
+	pprintf(mprn, A_("using fixed column format\n"));
 	err = csvdata_add_cols_list(c, cols);
 	if (err) {
 	    goto csv_bailout;
@@ -2031,44 +2027,44 @@ int import_csv (const char *fname, DATASET *dset,
     if (!fixed_format && !csv_got_delim(c)) {
 	/* set default delimiter */
 	if (csv_got_tab(c)) {
-	    c->delim = c->dset->delim = '\t';
+	    c->delim = '\t';
 	} else if (csv_got_semi(c)) {
-	    c->delim = c->dset->delim = ';';
+	    c->delim = ';';
 	} else {
-	    c->delim = c->dset->delim = ' ';
+	    c->delim = ' ';
 	}
     }
 
- alt_delim:
-
-    if (!fixed_format) {
-	pprintf(mprn, M_("using delimiter '%c'\n"), c->delim);
-    }
-
-    pprintf(mprn, M_("   longest line: %d characters\n"), c->maxlen - 1);
-
-    if (csv_has_trailing_comma(c) && c->delim != ',') {
-	csv_unset_trailing_comma(c);
-    }
-
     /* buffer to hold lines */
     c->line = malloc(c->maxlen);
     if (c->line == NULL) {
 	err = E_ALLOC;
 	goto csv_bailout;
     }  
-    
+
+ alt_delim:
+
+    if (!fixed_format) {
+	pprintf(mprn, A_("using delimiter '%c'\n"), c->delim);
+    }
+
+    pprintf(mprn, A_("   longest line: %d characters\n"), c->maxlen - 1);
+
+    if (csv_has_trailing_comma(c) && c->delim != ',') {
+	csv_unset_trailing_comma(c);
+    }
+
     rewind(fp);
 
     /* read lines, check for consistency in number of fields */
     err = csv_fields_check(fp, c, mprn);
     if (err && !fixed_format) {
 	if (c->delim != ';' && csv_got_semi(c)) {
-	    c->delim = c->dset->delim = ';';
+	    c->delim = ';';
 	    err = 0;
 	    goto alt_delim;
 	}
-	pputs(prn, M_(csv_msg));
+	pputs(prn, A_(csv_msg));
 	goto csv_bailout;
     }
 
@@ -2080,11 +2076,11 @@ int import_csv (const char *fname, DATASET *dset,
 	c->dset->v = (csv_skip_column(c))? c->ncols : c->ncols + 1;
     }
 
-    pprintf(mprn, M_("   number of variables: %d\n"), c->dset->v - 1);
-    pprintf(mprn, M_("   number of non-blank lines: %d\n"), c->nrows);
+    pprintf(mprn, A_("   number of variables: %d\n"), c->dset->v - 1);
+    pprintf(mprn, A_("   number of non-blank lines: %d\n"), c->nrows);
 
     if (c->dset->n == 0) {
-	pputs(prn, M_("Invalid data file\n"));
+	pputs(prn, A_("Invalid data file\n"));
 	err = E_DATA;
 	goto csv_bailout;
     }
@@ -2121,9 +2117,11 @@ int import_csv (const char *fname, DATASET *dset,
 	goto csv_bailout;
     }
 
-    if (dset != NULL && dset->decpoint != ',') {
+    if (c->decpoint == '.' && get_local_decpoint() == ',') {
 	gretl_push_c_numeric_locale();
 	popit = 1;
+    } else if (c->decpoint == ',' && get_local_decpoint() == '.') {
+	csv_set_dotsub(c);
     }
 
     datapos = ftell(fp);
@@ -2163,7 +2161,7 @@ int import_csv (const char *fname, DATASET *dset,
     c->dset->t2 = c->dset->n - 1;
 
     if (c->markerpd > 0) {
-	pputs(mprn, M_("taking date information from row labels\n\n"));
+	pputs(mprn, A_("taking date information from row labels\n\n"));
 	if (csv_skipping(c)) {
 	    pprintf(prn, "WARNING: Check your data! gretl has stripped out "
 		    "what appear to be\nextraneous lines in a %s dataset: " 
@@ -2171,7 +2169,7 @@ int import_csv (const char *fname, DATASET *dset,
 		    (c->dset->pd == 4)? "quarterly" : "monthly");
 	}
     } else {
-	pputs(mprn, M_("treating these as undated data\n\n"));
+	pputs(mprn, A_("treating these as undated data\n\n"));
 	dataset_obs_info_default(c->dset);
     }
 
@@ -2198,7 +2196,7 @@ int import_csv (const char *fname, DATASET *dset,
 	    sprintf(c->dset->varname[i], "v%d", i);
 	}
     } else if (fix_varname_duplicates(c->dset)) {
-	pputs(prn, M_("warning: some variable names were duplicated\n"));
+	pputs(prn, A_("warning: some variable names were duplicated\n"));
     }
 
     err = merge_or_replace_data(dset, &c->dset, opt, prn);
@@ -2221,12 +2219,8 @@ int import_csv (const char *fname, DATASET *dset,
     csvdata_free(c);
 
     if (err == E_ALLOC) {
-	pputs(prn, M_("Out of memory\n"));
+	pputs(prn, A_("Out of memory\n"));
     }    
 
-    console_off();
-
-    dset->delim = save_delim;
-
     return err;
 }
