@@ -147,6 +147,11 @@ void clear_datainfo (DATASET *dset, int code)
     if (dset->restriction != NULL) {
 	free(dset->restriction);
 	dset->restriction = NULL;
+    }
+
+    if (dset->padmask != NULL) {
+	free(dset->padmask);
+	dset->padmask = NULL;
     }	
 
     /* if this is not a sub-sample datainfo, free varnames, labels, etc. */
@@ -209,7 +214,6 @@ void copy_dataset_obs_info (DATASET *targ, const DATASET *src)
     targ->sd0 = src->sd0;
     targ->pd = src->pd;
     targ->structure = src->structure;
-    targ->decpoint = src->decpoint;
 }
 
 /**
@@ -228,7 +232,6 @@ void dataset_obs_info_default (DATASET *dset)
     dset->sd0 = 1.0;
     dset->pd = 1;
     dset->structure = CROSS_SECTION;
-    dset->decpoint = '.';
 }
 
 /**
@@ -369,6 +372,7 @@ void datainfo_init (DATASET *dset)
     dset->v = 0;
     dset->n = 0;
     dset->pd = 1;
+    dset->structure = CROSS_SECTION;
     dset->sd0 = 1.0;
     dset->t1 = 0;
     dset->t2 = 0;
@@ -380,15 +384,14 @@ void datainfo_init (DATASET *dset)
     dset->varinfo = NULL;    
 
     dset->markers = NO_MARKERS;  
-    dset->delim = ',';
-    dset->decpoint = '.';
 
     dset->S = NULL;
     dset->descrip = NULL;
     dset->submask = NULL;
     dset->restriction = NULL;
+    dset->padmask = NULL;
 
-    dset->structure = CROSS_SECTION;
+    dset->auxiliary = 0;
 }
 
 /**
@@ -426,9 +429,8 @@ DATASET *datainfo_new (void)
 
 DATASET *create_new_dataset (int nvar, int nobs, int markers)
 {
-    DATASET *dset;
+    DATASET *dset = datainfo_new();
 
-    dset = malloc(sizeof *dset);
     if (dset == NULL) return NULL;
 
     dset->v = nvar;
@@ -448,19 +450,16 @@ DATASET *create_new_dataset (int nvar, int nobs, int markers)
     } 
 
     dataset_obs_info_default(dset);
-    dset->descrip = NULL;
 
     return dset;
 }
 
 DATASET *create_auxiliary_dataset (int nvar, int nobs)
 {
-    DATASET *dset;
-
-    dset = create_new_dataset(nvar, nobs, 0);
+    DATASET *dset = create_new_dataset(nvar, nobs, 0);
 
     if (dset != NULL) {
-	dset->descrip = gretl_strdup("auxiliary");
+	dset->auxiliary = 1;
     }
 
     return dset;
@@ -534,10 +533,10 @@ int start_new_Z (DATASET *dset, int resample)
 
     dset->S = NULL;
     dset->markers = NO_MARKERS;
-    dset->delim = ',';
     dset->descrip = NULL;
     dset->submask = NULL;
     dset->restriction = NULL;
+    dset->padmask = NULL;
     
     return 0;
 }
@@ -1650,7 +1649,7 @@ int dataset_drop_listed_variables (int *list,
 				DROP_NORMAL, prn);
 
     if (dlist[0] > 0) {
-	if (!err) {
+	if (!err && !dset->auxiliary) {
 	    err = gretl_lists_revise(dlist, 0);
 	}
 
@@ -2094,6 +2093,14 @@ int dataset_drop_last_variables (int delvars, DATASET *dset)
     }
 #endif
 
+#if 0
+    fprintf(stderr, "dataset_drop_last_variables: origv=%d, newv=%d\n", 
+	    dset->v, newv);
+    for (i=1; i<dset->v; i++) {
+	fprintf(stderr, "before: var[%d] = '%s'\n", i, dset->varname[i]);
+    }
+#endif
+
     for (i=newv; i<dset->v; i++) {
 	free(dset->varname[i]);
 	free_varinfo(dset, i);
@@ -2103,7 +2110,13 @@ int dataset_drop_last_variables (int delvars, DATASET *dset)
 
     err = shrink_dataset_to_size(dset, newv, DROP_NORMAL);
 
-    if (!err) {
+#if 0
+    for (i=1; i<dset->v; i++) {
+	fprintf(stderr, "after: var[%d] = '%s'\n", i, dset->varname[i]);
+    }
+#endif
+
+    if (!err && !dset->auxiliary) {
 	err = gretl_lists_revise(NULL, newv);
     }
 
@@ -2264,6 +2277,29 @@ static int missing_tail (const double *x, int n)
     return nmiss;
 }
 
+static int *list_to_array (const int *list, int *err)
+{
+    int nv = list[0];
+    int *arr = NULL;
+
+    if (nv <= 0) {
+	*err = E_DATA;
+    } else {
+	arr = malloc(nv * sizeof *arr);
+	if (arr == NULL) {
+	    *err = E_ALLOC;
+	} else {
+	    int i;
+
+	    for (i=0; i<nv; i++) {
+		arr[i] = list[i+1];
+	    }
+	}
+    }
+
+    return arr;
+}
+
 /**
  * dataset_stack_variables:
  * @vname: name for new variable, to be produced by stacking.
@@ -2271,7 +2307,7 @@ static int missing_tail (const double *x, int n)
  * @dset: dataset struct.
  * @prn: printing apparatus.
  *
- * Really for internal use.  Don't worry about it.
+ * Really for internal use. Don't worry about it.
  *
  * Returns: 0 on success, non-zero code on error.
  */
@@ -2329,18 +2365,14 @@ int dataset_stack_variables (const char *vname, const char *line,
     stacklist = get_list_by_name(s);
     if (stacklist != NULL) {
 	nv = stacklist[0];
-	if (nv == 0) {
+	if (nv <= 0) {
 	    err = E_DATA;
 	    goto bailout;
 	}
-	vnum = malloc(nv * sizeof *vnum);
-	if (vnum == NULL) {
-	    err = E_ALLOC;
+	vnum = list_to_array(stacklist, &err);
+	if (err) {
 	    goto bailout;
 	}
-	for (i=0; i<nv; i++) {
-	    vnum[i] = stacklist[i+1];
-	}
 	done = 1;
     }
 
@@ -2365,8 +2397,20 @@ int dataset_stack_variables (const char *vname, const char *line,
 	}
     }
 
+    if (!done && strchr(s, '*') != NULL) {
+	/* or perhaps a wildcard thing? */
+	int *list = generate_list(s, dset, &err);
+
+	if (!err) {
+	    nv = list[0];
+	    vnum = list_to_array(list, &err);
+	    free(list);
+	}
+	done = 1;
+    }    
+
     if (!done) {
-	/* or a comma-separated list of vars? */
+	/* or (special) a comma-separated list of vars? */
 	char *p = s;
 
 	while (*p) {
@@ -2823,6 +2867,8 @@ static int dataset_int_param (const char **ps, int op,
 	    ok = 1;
 	} else if (dated_daily_data(dset) && (k == 52 || k == 12)) {
 	    ok = 1;
+	} else if (dataset_is_daily(dset) && k == 52) {
+	    ok = 1;
 	}
 
 	if (!ok) {
