@@ -50,21 +50,21 @@
 
 #define DB_DEBUG 0
 
-#define RECNUM long
+#define RECNUM gint32
 #define NAMELENGTH 16
 #define RATSCOMMENTLENGTH 80
 #define RATSCOMMENTS 2
 #define RATS_PARSE_ERROR -999
 
 typedef struct {
-    long daynumber;                /* Number of days from 1-1-90
+    gint32 daynumber;              /* Number of days from 1-1-90
 				      to year, month, day */
     short panel;                   /* 1 for panel set, 2 for intraday
 				      date set , 0 o.w. */
 #define LINEAR    0                /* Single time direction */
 #define RATSPANEL 1                /* panel:period */    
 #define INTRADAY  2                /* date:intraday period */
-    long panelrecord;              /* Size of panel or 
+    gint32 panelrecord;            /* Size of panel or 
 				      number of periods per day */
     short dclass;                  /* See definitions below */
 #define UNDATEDCLASS   0           /* No time series properties */
@@ -72,7 +72,7 @@ typedef struct {
 #define PERYEARCLASS   2           /* x periods / year */
 #define PERWEEKCLASS   3           /* x periods / week */
 #define DAILYCLASS     4           /* x days / period */
-    long info;                     /* Number of periods per year or
+    gint32 info;                   /* Number of periods per year or
 				      per week */
     short digits;                  /* Digits for representing panel
 				      or intraday period */
@@ -87,7 +87,7 @@ typedef struct {
     RECNUM first_data;             /* First data record */
     char series_name[NAMELENGTH];  /* Series name */
     DATEINFO date_info;            /* Dating scheme for this series */
-    long datapoints;               /* Number of data points */
+    gint32 datapoints;             /* Number of data points */
     short data_type;               /* real, char, complex.
                                       Reserved.  Should be 0 */
     short digits;                  /* . + digit count for representation
@@ -736,8 +736,8 @@ static int dinfo_sanity_check (const DATEINFO *dinfo)
     if (err) {
 	gretl_errmsg_set(_("This is not a valid RATS 4.0 database"));
 	fprintf(stderr, "rats database: failed dinfo_sanity_check:\n"
-		" info=%ld, year=%d, month=%d, day=%d\n",
-		dinfo->info, (int) dinfo->year, (int) dinfo->month, 
+		" info=%d, year=%d, month=%d, day=%d\n",
+		(int) dinfo->info, (int) dinfo->year, (int) dinfo->month, 
 		(int) dinfo->day);
     }
 
@@ -867,6 +867,10 @@ static RECNUM read_rats_directory (FILE *fp, const char *series_name,
     fprintf(stderr, "read_rats_directory: name='%s'\n", rdir.series_name);
 #endif
 
+    if (!isprint(rdir.series_name[0])) {
+	return RATS_PARSE_ERROR;
+    }
+
     if (series_name != NULL && strcmp(series_name, rdir.series_name)) {
 	/* specific series not found yet: keep going */
 	return rdir.forward_point;
@@ -876,13 +880,13 @@ static RECNUM read_rats_directory (FILE *fp, const char *series_name,
 
     /* skip long, short, long, short */
     fseek(fp, 12, SEEK_CUR);
-    fread(&dinfo.info, sizeof(long), 1, fp);
+    fread(&dinfo.info, sizeof(gint32), 1, fp);
     fread(&dinfo.digits, sizeof(short), 1, fp);
     fread(&dinfo.year, sizeof(short), 1, fp);
     fread(&dinfo.month, sizeof(short), 1, fp);
     fread(&dinfo.day, sizeof(short), 1, fp);
 
-    fread(&rdir.datapoints, sizeof(long), 1, fp);
+    fread(&rdir.datapoints, sizeof(gint32), 1, fp);
     fseek(fp, sizeof(short) * 4L, SEEK_CUR);  /* skip 4 shorts */
 
 #if DB_DEBUG
@@ -2013,7 +2017,8 @@ static int parse_odbc_format (char *fmt)
 
 static char *odbc_get_query (char *s, int *err)
 {
-    char *p, *query = NULL;
+    char *query = NULL;
+    const char *p;
 
     if (*s == '"') {
 	query = gretl_quoted_string_strdup(s, NULL);
@@ -2296,9 +2301,9 @@ int db_get_series (char *line, DATASET *dset,
     }
 
 #if DB_DEBUG
-    fprintf(stderr, "db_get_series: line='%s', pZ=%p, dset=%p\n", 
-	    line, (void *) pZ, (void *) dset);
-    fprintf(stderr, "db_name = '%s'\n", db_name);
+    fprintf(stderr, "db_get_series: line='%s', dset=%p\n", 
+	    line, (void *) dset);
+    fprintf(stderr, "db_name = '%s'\n", saved_db_name);
 #endif
 
     if (*saved_db_name == '\0') {
@@ -2398,27 +2403,13 @@ int db_get_series (char *line, DATASET *dset,
 
 static FILE *tempfile_open (char *fname, int *err)
 {
-    FILE *fp = NULL;
-    int fd;
+    FILE *fp;
 
     strcat(fname, ".XXXXXX");
-#ifdef WIN32
-    fd = gretl_mkstemp(fname);
-#else
-    fd = mkstemp(fname);
-#endif
-
-    if (fd < 0) {
+    fp = gretl_mktemp(fname, "w+");
+    if (fp == NULL) {
 	*err = E_FOPEN;
-    } else {
-	fp = fdopen(fd, "w+");
-	if (fp == NULL) {
-	    *err = E_FOPEN;
-	    gretl_errmsg_sprintf(_("Couldn't open %s"), fname);
-	    close(fd);
-	    gretl_remove(fname);
-	}
-    } 
+    }
 
     return fp;
 }
@@ -2897,6 +2888,8 @@ static double *extend_series (const double *z, int n)
     return x;
 }
 
+#define DMDEBUG 0
+
 static double *
 daily_series_to_monthly (DATASET *dset, int i, 
 			 int nm, int yr, int mon, int offset, 
@@ -2906,8 +2899,7 @@ daily_series_to_monthly (DATASET *dset, int i,
     const double *src = dset->Z[i];
     const double *z;
     double *tmp = NULL;
-    int t, mdbak;
-    int sopt, eopt;
+    int t, sop_t, eop_t;
 
     x = malloc(nm * sizeof *x);
     if (x == NULL) {
@@ -2926,49 +2918,68 @@ daily_series_to_monthly (DATASET *dset, int i,
 	z = src;
     }
 
-    /* note: we can't necessarily assume that the first obs
-       is the first day of a month  
+    /* Note: we can't necessarily assume that the first obs
+       is the first day of a month. The @offset value gives the
+       number of daily observations (allowing for the number of
+       observed days in the week) in the first month of the daily
+       data, prior to the data actually starting.
     */
 
-    /* the "one day's slack" business with start-of-period
-       and end-of-period compaction is designed to allow for
-       the possibility that the first (or last) day of the
-       month may not have been a trading day
+    /* The "one day's slack" business below, with start-of-period and
+       end-of-period compaction is designed to allow for the
+       possibility that the first (or last) day of the month may not
+       have been a trading day.
     */
 
-    mdbak = 0;
-    sopt = offset;
-    eopt = offset - 1;
+    /* first obs for start-of-period */
+    sop_t = offset;
 
-#if 0
-    printf("offset=%d, sopt=%d\n", offset, sopt);
+    /* first obs for end-of-period */
+    if (sop_t > 0) {
+	eop_t = offset - 1;
+    } else {
+	/* the first obs starts a month */
+	eop_t = get_days_in_month(mon, yr, dset->pd) - 1;
+    }
+
+#if DMDEBUG
+    fprintf(stderr, "starting: offset=%d, any_eop=%d, sop_t=%d, eop_t=%d\n", 
+	    offset, any_eop, sop_t, eop_t);
 #endif
 
     for (t=0; t<nm; t++) {
+	/* loop across the months in the compacted data */
 	int mdays = get_days_in_month(mon, yr, dset->pd);
 
-	sopt += mdbak;
-	eopt += mdays;
+	if (t > 0) {
+	    eop_t += mdays;
+	}
 
-	if (t == 0 && offset > 0 && any_eop &&
-	    method != COMPACT_EOP) {
+#if DMDEBUG
+	fprintf(stderr, "t=%d: mon=%d, mdays=%d, sop_t=%d, eop_t=%d\n", 
+		t, mon, mdays, sop_t, eop_t);
+#endif
+
+	if (t == 0 && offset > 0 && any_eop && method != COMPACT_EOP) {
+	    /* we started with an incomplete month: so any
+	       method other than EOP yields an NA */
 	    x[t] = NADBL;
 	} else if (method == COMPACT_SOP) {
 	    /* allow one days's slack */
-	    if (na(z[sopt]) && sopt < dset->n - 1) {
-		x[t] = z[sopt + 1];
+	    if (na(z[sop_t]) && sop_t < dset->n - 1) {
+		x[t] = z[sop_t + 1];
 	    } else {
-		x[t] = z[sopt];
+		x[t] = z[sop_t];
 	    }
 	} else if (method == COMPACT_EOP) {
-	    if (eopt >= dset->n) {
+	    if (eop_t >= dset->n) {
 		x[t] = NADBL;
 	    } else {
 		/* allow one days's slack */
-		if (na(z[eopt]) && eopt > 0) {
-		    x[t] = z[eopt - 1];
+		if (na(z[eop_t]) && eop_t > 0) {
+		    x[t] = z[eop_t - 1];
 		} else {
-		    x[t] = z[eopt];
+		    x[t] = z[eop_t];
 		}
 	    }
 	} else if (method == COMPACT_SUM ||
@@ -2979,7 +2990,7 @@ daily_series_to_monthly (DATASET *dset, int i,
 	    x[t] = 0.0;
 
 	    for (j=0; j<mdays; j++) {
-		dayt = sopt + j;
+		dayt = sop_t + j;
 		if (dayt >= dset->n) {
 		    x[t] = NADBL;
 		    break;
@@ -3007,7 +3018,7 @@ daily_series_to_monthly (DATASET *dset, int i,
 	    }
 	}
 
-	mdbak = mdays;
+	sop_t += mdays;
 
 	if (mon == 12) {
 	    mon = 1;
@@ -3163,7 +3174,7 @@ static int get_obs_maj_min (const char *obs, int *maj, int *min)
 
 static int get_daily_offset (const DATASET *dset,
 			     int y, int m, int d, 
-			     int skip)
+			     int skip, int any_eop)
 {
     int ret = 0;
 
@@ -3171,9 +3182,15 @@ static int get_daily_offset (const DATASET *dset,
 	/* moving to start of next month: offset = no. of
 	   observations in the first month */
 	ret = days_in_month_after(y, m, d, dset->pd) + 1;
+    } else if (any_eop && !day_starts_month(d, m, y, dset->pd, NULL)) {
+	ret = days_in_month_after(y, m, d, dset->pd) + 1;
     } else {
 	/* offset = no. of obs missing at start of first month */
 	ret = days_in_month_before(y, m, d, dset->pd);
+#if DMDEBUG
+	fprintf(stderr, "days_in_month_before %d-%02d-%02d = %d "
+		"for pd=%d\n", y, m, d, ret, dset->pd);
+#endif
     }
 
     return ret;
@@ -3217,6 +3234,13 @@ static int get_n_ok_months (const DATASET *dset,
     *endyr = y2;
     *endmon = m2;
 
+#if DMDEBUG
+    fprintf(stderr, "get_n_ok_months: any_sop=%d, any_eop=%d, "
+	    "all_same=%d\n", any_sop, any_eop, all_same);
+    fprintf(stderr, "y1=%d m1=%d d1=%d; y2=%d m2=%d d2=%d\n", 
+	    y1, m1, d1, y2, m2, d2);
+#endif
+
     if (!day_starts_month(d1, m1, y1, dset->pd, &pad) && !any_eop) {
 	if (*startmon == 12) {
 	    *startmon = 1;
@@ -3238,10 +3262,16 @@ static int get_n_ok_months (const DATASET *dset,
 	nm--;
     }
 
+#if DMDEBUG
+    fprintf(stderr, "after adjustment: range %d:%02d to %d:%02d, "
+	    "pad=%d, skip=%d\n", *startyr, *startmon, *endyr, *endmon, 
+	    pad, skip);
+#endif
+
     if (pad) {
 	*offset = -1;
     } else {
-	*offset = get_daily_offset(dset, y1, m1, d1, skip);
+	*offset = get_daily_offset(dset, y1, m1, d1, skip, any_eop);
     }
 
     *p_any_eop = any_eop;
@@ -3444,7 +3474,7 @@ static int shorten_the_constant (double **Z, int n)
 }
 
 /* conversion to weekly using a "representative day", e.g. use
-   each Wednesday value.  repday is 0-based on Sunday.
+   each Wednesday value: @repday is 0-based on Sunday.
 */
 
 static int daily_dataset_to_weekly (DATASET *dset, int repday)
@@ -3807,8 +3837,9 @@ int compact_data_set (DATASET *dset, int newpd,
     ntodate(dset->endobs, dset->t2, dset);
     
     if (oldpd >= 5 && oldpd <= 7 && dset->markers) {
-	/* remove any daily date strings */
+	/* remove any daily date strings; revise endobs */
 	dataset_destroy_obs_markers(dset);
+	ntodate(dset->endobs, dset->t2, dset);
     }
 
     err = shorten_the_constant(dset->Z, dset->n);
