@@ -147,6 +147,9 @@ FreqDist *get_discrete_freq (int v, const DATASET *dset,
 int freqdist (int varno, const DATASET *dset,
 	      int *graph, gretlopt opt, PRN *prn);
 
+gretl_matrix *freqdist_matrix (const double *x, int t1, int t2, 
+			       int *err);
+
 int crosstab (const int *list, const DATASET *dset, 
 	      gretlopt opt, PRN *prn);
 
@@ -189,6 +192,12 @@ Summary *get_summary (const int *list, const DATASET *dset,
 		      gretlopt opt, PRN *prn, 
 		      int *err);
 
+Summary *get_summary_restricted (const int *list, 
+				 const DATASET *dset, 
+				 const double *rv,
+				 gretlopt opt, PRN *prn, 
+				 int *err);
+
 int list_summary (const int *list, const DATASET *dset, 
 		  gretlopt opt, PRN *prn);
 
@@ -198,6 +207,11 @@ void print_summary (const Summary *summ,
 		    const DATASET *dset,
 		    PRN *prn); 
 
+void print_summary_single (const Summary *s,
+			   int digits, int places,
+			   const DATASET *dset,
+			   PRN *prn);
+
 void free_summary (Summary *summ);
 
 VMatrix *corrlist (int *list, const DATASET *dset,
