@@ -872,6 +872,8 @@ int seasonally_adjust_series (const double *x, double *y,
 	}
     }
 
+    gretl_error_clear();
+
     adjust_series = get_plugin_function("adjust_series", &handle);
     
     if (adjust_series == NULL) {
@@ -900,6 +902,7 @@ static int new_unit (const DATASET *dset, int t)
 }
 
 #define panel_obs_ok(x,t,m) ((m == NULL || m[t] != 0) && !na(x[t])) 
+#define panel_obs_masked(m,t) (m != NULL && m[t] == 0)
 
 /**
  * panel_statistic:
@@ -907,7 +910,7 @@ static int new_unit (const DATASET *dset, int t)
  * @y: target into which to write.
  * @dset: data set information.
  * @k: code representing the desired statistic: F_PNOBS,
- * F_PMIN, F_PMAX, F_PMEAN or F_PSD.
+ * F_PMIN, F_PMAX, F_PMEAN, F_PXSUM or F_PSD.
  * @mask: either NULL or a series with 0s for observations
  * to be excluded from the calculations, non-zero values
  * at other observations.
@@ -1086,6 +1089,36 @@ int panel_statistic (const double *x, double *y, const DATASET *dset,
 		}
 	    }
 	}
+    } else if (k == F_PXSUM) {
+	/* the sum of cross-sectional values for each period */
+	double *yt = malloc(dset->pd * sizeof *yt);
+	int i, N = dset->n / dset->pd;
+
+	if (yt == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    for (t=0; t<dset->pd; t++) {
+		yt[t] = 0.0;
+		for (i=0; i<N; i++) {
+		    s = t + i * dset->pd;
+		    if (panel_obs_masked(mask, s)) {
+			continue;
+		    } else if (na(x[s])) {
+			yt[t] = NADBL;
+			break;
+		    } else {
+			yt[t] += x[s];
+		    }
+		}
+	    }
+	    s = 0;
+	    for (i=0; i<N; i++) {
+		for (t=0; t<dset->pd; t++) {
+		    y[s++] = yt[t];
+		}
+	    }	    
+	    free(yt);
+	}
     } else {
 	/* unsupported option */
 	err = E_DATA;
@@ -3157,15 +3190,11 @@ int check_declarations (char ***pS, parser *p)
     }
 
     s = p->lh.substr;
-    for (i=0; i<n; i++) {
-	S[i] = gretl_word_strdup(s, &s);
-	if (S[i] == NULL) {
-	    p->err = E_DATA;
-	    break;
-	}
+    for (i=0; i<n && !p->err; i++) {
+	S[i] = gretl_word_strdup(s, &s, OPT_S, &p->err);
     }
 
-    if (*s != '\0') {
+    if (!p->err && *s != '\0') {
 	p->err = E_DATA;
     }
 
@@ -3550,12 +3579,12 @@ static double imhof_bound (const double *lambda, int k, int *err)
     }
 
     /* The key factor in the integrand is the product of 
-       (1+(lambda(i)*x)**2)**(1/4) across i. Since, for those 
+       (1+(lambda(i)*x)^2)^(1/4) across i. Since, for those 
        factors with small |lambda(i)|, this won't go to zero very
        quickly, we count only the terms on the bigger eigenvalues.
     */
 
-    nl = 0.5 * nl;
+    nl *= 0.5;
     sum = 0.5 * sum + log(M_PI * nl);
     bound = exp(-(sum + log(e1)) / nl);
     bound += 5.0 / nl;
@@ -3653,6 +3682,7 @@ static double imhof_integral (double arg, const double *lambda, int k,
 	    ret = 0.0;
 	} else if (ret < 0) {
 	    fprintf(stderr, "n = %d, Imhof integral gave negative value %g\n", n, ret);
+	    gretl_errmsg_set(_("Imhof integral gave negative value"));
 	    *err = E_DATA;
 	    ret = NADBL;
 	}
@@ -3794,6 +3824,9 @@ double dw_pval (const gretl_matrix *u, const gretl_matrix *X,
 
     if (!err) {
 	DW /= uu;
+#if 0
+	fprintf(stderr, "DW = %g\n", DW);
+#endif
 	E = gretl_general_matrix_eigenvals(MA, 0, &err);
     }
 
@@ -4561,36 +4594,37 @@ int nadaraya_watson (const double *y, const double *x, double h,
     double xt, xs, ys, yt, k;
     double ah = fabs(h);
     int LOO = (h < 0);  /* leave-one-out */
-
     double TRIM = libset_get_double(NADARWAT_TRIM) * ah;
-
     int n = t2 + 1;
-    double *num;
-    double *den;
+    double *num, *den;
 
-    num = malloc(n * sizeof *num);
-    den = malloc(n * sizeof *den);
+    num = malloc(2 * n * sizeof *num);
 
-    if (num == NULL || den == NULL) {
+    if (num == NULL) {
 	return E_ALLOC;
     }
 
+    den = num + n;
+
     /* 
        here we initialize numerator and denominator; we use the
        "diagonal" in the standard case and 0 in the leave-one-out
        case.
-     */
+    */
 
     if (LOO) {
 	for (t=t1; t<=t2; t++) {
 	    num[t] = den[t] = 0.0;
 	}
     } else {
-	k =  nw_kernel(0);
+	k = nw_kernel(0);
 	for (t=t1; t<=t2; t++) {
 	    if (!na(y[t])) {
 		num[t] = k * y[t];
 		den[t] = k;
+	    } else {
+		num[t] = 0;
+		den[t] = 0;
 	    }
 	}
     }
@@ -4621,7 +4655,130 @@ int nadaraya_watson (const double *y, const double *x, double h,
     }
 
     free(num);
-    free(den);
+
+    return err;
+}
+
+static int xy_get_sample (const double *y, const double *x,
+			  int *t1, int *t2, int *n)
+{
+    int t, nxy, err = 0;
+
+    for (t=*t1; t<=*t2; t++) {
+	if (xna(x[t])) {
+	    *t1 += 1;
+	} else {
+	    break;
+	}
+    }
+
+    for (t=*t2; t>=*t1; t--) {
+	if (xna(x[t])) {
+	    *t2 -= 1;
+	} else {
+	    break;
+	}
+    }
+
+    /* nxy = the number of points where we have valid values
+       for both x and y; n = the number of valid x-values
+    */
+
+    nxy = *n = 0;
+
+    for (t=*t1; t<=*t2; t++) {
+	if (!xna(x[t])) {
+	    *n += 1;
+	    if (!xna(y[t])) {
+		nxy++;
+	    } 
+	} 
+    }
+
+    if (nxy < 16) {
+	err = E_TOOFEW;
+    }
+
+    return err;
+}
+
+static int get_dataset_t (const double *x, int pos, int t1)
+{
+    int t, k = 0;
+
+    for (t=t1; k<=pos; t++) {
+	if (!xna(x[t])) {
+	    if (k == pos) {
+		return t;
+	    }
+	    k++;
+	}
+    }
+
+    return 0;
+}
+
+int gretl_loess (const double *y, const double *x, int poly_order,
+		 double bandwidth, gretlopt opt, DATASET *dset, 
+		 double *m)
+{
+    gretl_matrix *my, *mx;
+    gretl_matrix *yh = NULL;
+    int *s_order = NULL;
+    int t1 = dset->t1;
+    int t2 = dset->t2;
+    int s, t, n;
+    int err = 0;
+
+    if (poly_order < 0 || poly_order > 2 || 
+	bandwidth <= 0 || bandwidth >= 1) {
+	return E_DATA;
+    }
+
+    err = xy_get_sample(y, x, &t1, &t2, &n);
+    if (err) {
+	return err;
+    }
+
+    /* note: n holds the number of non-missing observations
+       on x; the associated y-value may or may not be
+       missing
+    */
+
+    my = gretl_column_vector_alloc(n);
+    mx = gretl_column_vector_alloc(n);
+ 
+    if (my == NULL || mx == NULL) {
+	err = E_ALLOC;
+    } else {
+	s = 0;
+	for (t=t1; t<=t2; t++) {
+	    if (!xna(x[t])) {
+		my->val[s] = y[t];
+		mx->val[s] = x[t];
+		s++;
+	    }
+	}
+	/* sort the points by the value of x */
+	err = sort_pairs_by_x(mx, my, &s_order, NULL);
+    }
+
+    if (!err) {
+	yh = loess_fit(mx, my, poly_order, bandwidth, opt, &err);
+    }
+
+    if (!err) {
+	/* put the yh values into dataset order */
+	for (s=0; s<n; s++) {
+	    t = get_dataset_t(x, s_order[s], t1);
+	    m[t] = yh->val[s];
+	}
+    }
+
+    gretl_matrix_free(my);
+    gretl_matrix_free(mx);
+    gretl_matrix_free(yh);
+    free(s_order);
 
     return err;
 }
