@@ -107,6 +107,8 @@ struct str_table mvars[] = {
     { M_DWPVAL,  "$dwpval" },
     { M_FSTT,    "$Fstat" },
     { M_CHISQ,   "$chisq" },
+    { M_DIAGTEST, "$diagtest" },
+    { M_DIAGPVAL, "$diagpval" },
     { M_UHAT,    "$uhat" },
     { M_YHAT,    "$yhat" },
     { M_LLT,     "$llt" },
@@ -143,6 +145,7 @@ struct str_table mvars[] = {
     { M_EHAT,    "$ehat" },
     { M_MNLPROBS, "$mnlprobs" },
     { M_XLIST,   "$xlist" },
+    { M_YLIST,   "$ylist" },
     { M_COMMAND, "$command" },
     { M_DEPVAR,  "$depvar" },
     { 0,         NULL }
@@ -196,6 +199,7 @@ struct str_table funcs[] = {
     { F_GINI,     "gini" },
     { F_ZEROMISS, "zeromiss" },
     { F_SUM,      "sum" },
+    { F_SUMALL,   "sumall" },
     { F_MEAN,     "mean" },
     { F_MIN,      "min" },
     { F_MAX,      "max" },
@@ -215,6 +219,7 @@ struct str_table funcs[] = {
     { F_PMIN,     "pmin" },      /* panel min */
     { F_PMAX,     "pmax" },      /* panel max */
     { F_PMEAN,    "pmean" },     /* panel mean */
+    { F_PXSUM,    "pxsum" },     /* panel x-sectional sum */
     { F_PSD,      "psd" },       /* panel std dev */
     { F_PSHRINK,  "pshrink" },
     { F_HPFILT,   "hpfilt" },    /* Hodrick-Prescott filter */
@@ -377,6 +382,9 @@ struct str_table funcs[] = {
     { F_COLNAME,  "colname" },
     { F_RANDINT,  "randint" },
     { F_NADARWAT, "nadarwat" },   /* Nadaraya-Watson */
+    { F_SIMANN,   "simann" },     /* simulated annealing */
+    { F_LOESS,    "loess" },
+    { F_FREQ,     "freq" },
     { 0,          NULL }
 };
 
@@ -693,6 +701,8 @@ void context_error (int c, parser *p)
 	    pputs(p->prn, _("(for logical AND, use '&&')\n"));
 	} else if (c == '|') {
 	    pputs(p->prn, _("(for logical OR, use '||')\n"));
+	} else if (c == ',') {
+	    p->err = E_BADCOMMA;
 	}
     } else {
 	const char *s = getsymb(p->sym, p);
@@ -836,6 +846,27 @@ static void look_up_string_variable (const char *s, parser *p)
     }
 }
 
+int is_gretl_accessor (const char *s)
+{
+    int i, n;
+
+    for (i=0; dvars[i].id != 0; i++) {
+	n = strlen(dvars[i].str);
+	if (!strncmp(s, dvars[i].str, n)) {
+	    return !isalpha(s[n]);
+	}
+    }
+
+    for (i=0; mvars[i].id != 0; i++) {
+	n = strlen(mvars[i].str);
+	if (!strncmp(s, mvars[i].str, n)) {
+	    return !isalpha(s[n]);
+	}
+    }
+
+    return 0;
+}
+
 static void look_up_dollar_word (const char *s, parser *p)
 {
     p->idnum = dvar_lookup(s);
@@ -1355,7 +1386,12 @@ void lex (parser *p)
 	    parser_getc(p);
 	    return;
         case ';': 
-	    p->sym = P_SEMI;
+	    if (p->targ == LIST) {
+		p->sym = B_JOIN;
+	    } else {
+		/* used in matrix definition */
+		p->sym = P_SEMI;
+	    }
 	    parser_getc(p);
 	    return;
         case ':': 
