@@ -67,9 +67,10 @@ enum {
 	      B_LCAT,     /* list concatentation */
 	      B_LDIV,     /* matrix left division */
 	      B_ELLIP,    /* list-generating ellipsis */
+	      B_JOIN,     /* list-joining with separator */
 	      OP_MAX,     /* SEPARATOR: end of binary operators */
-              G_LPR,      /* grouping: left paren */
-  /* 40 */    G_RPR,      /* right paren */
+  /* 40 */    G_LPR,      /* grouping: left paren */
+	      G_RPR,      /* right paren */
               G_LBR,      /* left bracket */
               G_RBR,      /* right bracket */
               G_LCB,      /* left curly bracket */
@@ -78,8 +79,8 @@ enum {
 	      P_DOT,	  /* period */
 	      P_SEMI,	  /* semi-colon */
 	      P_COL,	  /* colon */
-	      PUNCT_MAX,  /* SEPARATOR: end of grouping and punctuation marks */
-  /* 50 */    CON,	  /* named constant */
+  /* 50 */    PUNCT_MAX,  /* SEPARATOR: end of grouping and punctuation marks */
+	      CON,	  /* named constant */
 	      DUM,	  /* dummy variable */
               USCALAR,	  /* user variable, scalar */
               USERIES,    /* user variable, series */
@@ -88,8 +89,8 @@ enum {
 	      NUM,	  /* scalar, evaluated */
 	      VEC,	  /* series, evaluated */
 	      IVEC,	  /* vector of integers, evaluated */
-	      LVEC,       /* list, as array of ints */
-  /* 60 */    MAT,	  /* matrix, evaluated */
+  /* 60 */    LVEC,       /* list, as array of ints */
+	      MAT,	  /* matrix, evaluated */
 	      OBS,	  /* observation from a series */
               MSL,	  /* matrix plus subspec */
               DMSL,	  /* "dollar" matrix plus subspec */
@@ -98,8 +99,8 @@ enum {
 	      MSPEC,	  /* evaluated matrix subspec */
 	      SUBSL,	  /* row or column component of MSPEC */
 	      MDEF,	  /* explicit matrix definition {...} */
-	      LAG,        /* variable plus lag length */	  
-  /* 70 */    DVAR,	  /* $ "dataset" variable (mostly scalar or series) */
+  /* 70 */    LAG,        /* variable plus lag length */	  
+	      DVAR,	  /* $ "dataset" variable (mostly scalar or series) */
 	      MVAR,	  /* $ model var (scalar, series, or matrix) */
               OVAR,	  /* object variable: variable "under" an object */
               LIST,	  /* reference to named list */
@@ -108,8 +109,8 @@ enum {
 	      BUNDLE,     /* gretl bundle (hash table) */
 	      BOBJ,       /* object inside a bundle */
 	      FARGS,	  /* set of n function arguments */
-	      EMPTY,      /* "null" */
-  /* 80 */    ABSENT,
+  /* 80 */    EMPTY,      /* "null" */
+	      ABSENT,
 	      DTYPE_MAX,  /* SEPARATOR: end of "bare" types */
               EROOT,	  /* dummy root for (...) expression */
               UFUN,	  /* user-defined function */
@@ -118,7 +119,7 @@ enum {
               INC,   
 	      DEC,
 	      QUERY,
-	      EOT,	  /* end of transmission */
+  /* 90 */    EOT,	  /* end of transmission */
 	      UNK 
 };
 
@@ -165,7 +166,8 @@ enum {
     F_ZEROMISS,
     F_MEDIAN,
     F_GINI,
-    F_SUM,   
+    F_SUM, 
+    F_SUMALL,
     F_MEAN,
     F_MIN,
     F_MAX,
@@ -257,6 +259,7 @@ enum {
     F_GETENV,
     F_NGETENV,
     F_PSHRINK,
+    F_FREQ,
     F1_MAX,	  /* SEPARATOR: end of single-arg functions */
     F_COR,
     F_COV,
@@ -278,7 +281,6 @@ enum {
     F_EIGGEN,
     F_FDJAC,
     F_LRVAR,
-    F_PRINCOMP,
     F_QUANTILE,
     F_CMULT,	  /* complex multiplication */
     F_HDPROD,     /* horizontal direct product */
@@ -318,9 +320,11 @@ enum {
     F_PMIN,
     F_PMAX,
     F_PMEAN,
+    F_PXSUM,
     F_PSD,
     F_RANDINT,
     F2_MAX,	  /* SEPARATOR: end of two-arg functions */
+    F_PRINCOMP,
     F_BFGSMAX,
     F_MSHAPE,
     F_SVD,
@@ -345,6 +349,7 @@ enum {
     F_MLAG,
     F_EIGSOLVE,
     F_NADARWAT,
+    F_SIMANN,
     F3_MAX,       /* SEPARATOR: end of three-arg functions */
     F_BKFILT,
     F_MOLS,
@@ -356,6 +361,7 @@ enum {
     F_KSMOOTH,
     F_KSIMUL,
     F_NRMAX,
+    F_LOESS,
     FN_MAX,	  /* SEPARATOR: end of n-arg functions */
 };
 
@@ -397,7 +403,7 @@ enum {
                              s == F_NRMAX || s == F_DESEAS)
 
 /* functions taking string arg in middle position */
-#define string_mid_func(s) (s == F_BFGSMAX || s == F_NRMAX)
+#define string_mid_func(s) (s == F_BFGSMAX || s == F_NRMAX || s == F_SIMANN)
 
 #define unary_op(s)  (s >= 1 && s < U_MAX)
 #define binary_op(s) (s > U_MAX && s < OP_MAX)
@@ -595,6 +601,7 @@ const char *constname (int c);
 const char *dvarname (int t);
 const char *mvarname (int t);
 const char *dumname (int t);
+int is_gretl_accessor (const char *s);
 
 /* handling declarations of variables */
 int check_declarations (char ***pS, parser *p);
