@@ -24,7 +24,6 @@
 #include "qr_estimate.h"
 #include "gretl_bfgs.h"
 
-#include "gretl_f2c.h"
 #include "../../minpack/minpack.h"
 
 #include <errno.h>
@@ -58,6 +57,7 @@ struct ocset_ {
     int noc;              /* total number of orthogonality conds. */
     int step;             /* number of estimation steps */
     int userwts;          /* got weights matrix from user (1/0) */
+    double scale_wt;      /* auto-scaling for weights matrix  */
     hac_info hinfo;       /* HAC characteristics */
     char Wname[VNAMELEN]; /* name of weighting matrix */
     char **lnames;        /* names of LHS terms in O.C.s */
@@ -116,6 +116,7 @@ static ocset *oc_set_new (void)
 	oc->noc = 0;
 	oc->step = 0;
 	oc->userwts = 0;
+	oc->scale_wt = 1;
 	*oc->Wname = '\0';
 	oc->lnames = NULL;
 	oc->rnames = NULL;
@@ -131,8 +132,6 @@ static int gmm_unkvar (const char *s)
     return E_UNKVAR;
 }
 
-#define matrix_is_dated(m) (!(m->t1 == 0 && m->t2 == 0))
-
 /* Determine the type of an element in an "orthog" specification:
    series, matrix or list of series
 */
@@ -350,49 +349,86 @@ add_new_cols_to_Z (nlspec *s, const gretl_matrix *M, int oldecols)
     return err;
 }
 
-/* Record the source (ID number of variable, or name of matrix and
-   column within matrix) for a column on the left-hand side of the set
-   of GMM orthogonality conditions.  We need this later for updating
-   the values in the given column, after adjusting the parameters.
+/* expand the array containing info on the columns on the left
+   of a set of GMM orthogonality conditions. 
 */
 
-static int 
-push_column_source (nlspec *s, int v, const char *mname)
+static int add_oc_cols (nlspec *s, int current_cols, int add_cols)
 {
-    gretl_matrix *m;
-    colsrc *cols;
-    int j, k, n, p;
+    colsrc *cols;    
+    int err = 0;
 
-    m = (mname != NULL)? get_matrix_by_name(mname) : NULL;
-    n = (s->oc->e != NULL)? s->oc->e->cols : 0;
-    k = (m != NULL)? m->cols : 1;
+    cols = realloc(s->oc->ecols, (current_cols + add_cols) * sizeof *cols);
 
-    cols = realloc(s->oc->ecols, (n + k) * sizeof *cols);
     if (cols == NULL) {
-	return E_ALLOC;
+	err = E_ALLOC;
+    } else {
+	int j, k;
+
+	for (j=0; j<add_cols; j++) {
+	    k = current_cols + j;
+	    cols[k].v = 0;
+	    cols[k].j = j;
+	    cols[k].mname[0] = '\0';
+	} 
+
+	s->oc->ecols = cols;
+    }
+    
+    return err;
+}
+
+/* Record the source (ID number of series) for a column on 
+   the LHS of a set of GMM orthogonality conditions. 
+*/
+
+static int push_col_source_series (nlspec *s, int v)
+{
+    int n = (s->oc->e != NULL)? s->oc->e->cols : 0;
+    int err = add_oc_cols(s, n, 1);
+
+    if (!err) {
+	s->oc->ecols[n].v = v;
     }
 
-    s->oc->ecols = cols;
+    return err;
+}
 
-    for (j=0; j<k; j++) {
-	p = n + j;
-	cols[p].v = (v < 0)? 0 : v;
-	cols[p].j = j;
+/* Record the source (named matrix) for a set of columns on 
+   the LHS of a set of GMM orthogonality conditions. 
+*/
 
-	if (mname == NULL) {
-	    cols[p].mname[0] = '\0';
-	} else {
-	    strcpy(cols[p].mname, mname);
+static int push_col_source_matrix (nlspec *s, const char *mname)
+{
+    int j, n = (s->oc->e != NULL)? s->oc->e->cols : 0;
+    gretl_matrix *m = get_matrix_by_name(mname);
+    int err = add_oc_cols(s, n, m->cols);
+
+    if (!err) {
+	for (j=0; j<m->cols; j++) {
+	    strcpy(s->oc->ecols[n+j].mname, mname);
 	}
-
-#if GMM_DEBUG
-	fprintf(stderr, "push_column_source: added source %d: v=%d, "
-		"mname='%s', j=%d\n", p, cols[p].v, cols[p].mname, 
-		cols[p].j);
-#endif
     }
 
-    return 0;
+    return err;
+}
+
+/* Record the source (list of series) for a set of columns on 
+   the LHS of a set of GMM orthogonality conditions. 
+*/
+
+static int push_col_source_list (nlspec *s, const int *list)
+{
+    int j, n = (s->oc->e != NULL)? s->oc->e->cols : 0;
+    int err = add_oc_cols(s, n, list[0]);
+
+    if (!err) {
+	for (j=0; j<list[0]; j++) {
+	    s->oc->ecols[n+j].v = list[j+1];
+	}
+    }
+
+    return err;
 }
 
 int nlspec_add_ivreg_oc (nlspec *s, int lhv, const int *rlist,
@@ -415,13 +451,13 @@ int nlspec_add_ivreg_oc (nlspec *s, int lhv, const int *rlist,
     if (e == NULL) {
 	err = E_ALLOC;
     } else {
-	e->t1 = s->t1;
-	e->t2 = s->t2;
+	gretl_matrix_set_t1(e, s->t1);
+	gretl_matrix_set_t2(e, s->t2);
 	for (t=0; t<s->nobs; t++) {
 	    x = Z[lhv][t + s->t1];
 	    gretl_vector_set(e, t, x);
 	}
-	err = push_column_source(s, lhv, NULL);
+	err = push_col_source_series(s, lhv);
     }
 
     if (err) {
@@ -435,8 +471,8 @@ int nlspec_add_ivreg_oc (nlspec *s, int lhv, const int *rlist,
     if (M == NULL) {
 	err = E_ALLOC;
     } else {
-	M->t1 = s->t1;
-	M->t2 = s->t2;
+	gretl_matrix_set_t1(M, s->t1);
+	gretl_matrix_set_t2(M, s->t2);
 	for (i=0; i<k && !err; i++) {
 	    v = rlist[i + 1];
 	    for (t=0; t<s->nobs; t++) {
@@ -484,7 +520,7 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
     if (ltype == GRETL_TYPE_MATRIX) {
 	e = get_matrix_copy_by_name(lname, &err);
 	if (!err) {
-	    err = push_column_source(s, 0, lname);
+	    err = push_col_source_matrix(s, lname);
 	}
     } else if (ltype == GRETL_TYPE_LIST) {
 	int *list = get_list_by_name(lname);
@@ -494,16 +530,16 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 	if (e == NULL) {
 	    err = E_ALLOC;
 	} else {
-	    e->t1 = s->t1;
-	    e->t2 = s->t2;
-	    for (i=0; i<k && !err; i++) {
-		v = list[i + 1];
+	    gretl_matrix_set_t1(e, s->t1);
+	    gretl_matrix_set_t2(e, s->t2);
+	    for (i=0; i<k; i++) {
+		v = list[i+1];
 		for (t=0; t<s->nobs; t++) {
 		    x = dset->Z[v][t + s->t1];
 		    gretl_matrix_set(e, t, i, x);
 		}
-		err = push_column_source(s, v, NULL);
 	    }
+	    err = push_col_source_list(s, list);
 	}	
     } else if (ltype == GRETL_TYPE_SERIES) {
 	v = series_index(dset, lname);
@@ -511,13 +547,13 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 	if (e == NULL) {
 	    err = E_ALLOC;
 	} else {
-	    e->t1 = s->t1;
-	    e->t2 = s->t2;
+	    gretl_matrix_set_t1(e, s->t1);
+	    gretl_matrix_set_t2(e, s->t2);
 	    for (t=0; t<s->nobs; t++) {
 		x = dset->Z[v][t + s->t1];
 		gretl_vector_set(e, t, x);
 	    }
-	    err = push_column_source(s, v, NULL);
+	    err = push_col_source_series(s, v);
 	}
     } else {
 	err = gmm_unkvar(lname);
@@ -550,8 +586,8 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 	if (M == NULL) {
 	    err = E_ALLOC;
 	} else {
-	    M->t1 = s->t1;
-	    M->t2 = s->t2;
+	    gretl_matrix_set_t1(M, s->t1);
+	    gretl_matrix_set_t2(M, s->t2);
 	    for (i=0; i<k && !err; i++) {
 		v = list[i + 1];
 		for (t=0; t<s->nobs; t++) {
@@ -568,8 +604,8 @@ static int oc_add_matrices (nlspec *s, int ltype, const char *lname,
 	if (M == NULL) {
 	    err = E_ALLOC;
 	} else {
-	    M->t1 = s->t1;
-	    M->t2 = s->t2;
+	    gretl_matrix_set_t1(M, s->t1);
+	    gretl_matrix_set_t2(M, s->t2);
 	    for (t=0; t<s->nobs; t++) {
 		x = dset->Z[v][t + s->t1];
 		gretl_vector_set(M, t, x);
@@ -688,6 +724,8 @@ static int gmm_matrix_resize (gretl_matrix **pA, nlspec *s, int oldt1)
     gretl_matrix *A = *pA;
     gretl_matrix *B = NULL;
     int T = s->t2 - s->t1 + 1;
+    int At1 = gretl_matrix_get_t1(A);
+    int At2 = gretl_matrix_get_t2(A);
     int m = A->cols;
     double x;
     int j, t, offt;
@@ -697,7 +735,11 @@ static int gmm_matrix_resize (gretl_matrix **pA, nlspec *s, int oldt1)
 	return E_ALLOC;
     }
 
-    offt = s->t1 - matrix_t1(A, oldt1); /* ?? */
+    if (At1 == 0 && At2 == 0) {
+	offt = s->t1 - oldt1;
+    } else {
+	offt = s->t1 - At1;
+    }
 
     for (j=0; j<m; j++) {
 	for (t=0; t<T; t++) {
@@ -706,8 +748,8 @@ static int gmm_matrix_resize (gretl_matrix **pA, nlspec *s, int oldt1)
 	}
     }
 
-    B->t1 = s->t1;
-    B->t2 = s->t2;
+    gretl_matrix_set_t1(B, s->t1);
+    gretl_matrix_set_t2(B, s->t2);
 
     gretl_matrix_replace(pA, B);
 
@@ -720,10 +762,10 @@ static int gmm_matrix_resize (gretl_matrix **pA, nlspec *s, int oldt1)
 
 static int gmm_fix_datarows (nlspec *s)
 {
-    int et1 = s->oc->e->t1;
-    int et2 = s->oc->e->t2;
-    int Zt1 = s->oc->Z->t1;
-    int Zt2 = s->oc->Z->t2;
+    int et1 = gretl_matrix_get_t1(s->oc->e);
+    int et2 = gretl_matrix_get_t2(s->oc->e);
+    int Zt1 = gretl_matrix_get_t1(s->oc->Z);
+    int Zt2 = gretl_matrix_get_t2(s->oc->Z);
     int oldt1 = s->t1;
     int err = 0;
 
@@ -1058,11 +1100,8 @@ static double get_gmm_crit (const double *b, void *p)
     return s->crit;
 }
 
-/* GMM callback for Minpack's fdjac2 function */
-
-static int 
-gmm_jacobian_calc (integer *m, integer *n, double *x, double *f,
-		   integer *iflag, void *p)
+static int gmm_jacobian_calc (int m, int n, double *x, double *f,
+			      int *iflag, void *p)
 {
     nlspec *s = (nlspec *) p;
     double fac;
@@ -1088,7 +1127,7 @@ gmm_jacobian_calc (integer *m, integer *n, double *x, double *f,
     T = s->nobs;
     fac = sqrt((double) T) / T;
 
-    for (i=0; i<*m; i++) {
+    for (i=0; i<m; i++) {
 	f[i] = 0.0;
 	for (t=0; t<T; t++) {
 	    f[i] += gretl_matrix_get(s->oc->tmp, t, i);
@@ -1342,16 +1381,14 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
     gretl_matrix *m1, *m2, *m3;
     int i, j, k = s->ncoeff;
     int T = s->nobs;
-    doublereal *wa4;
-    doublereal epsfcn = 0.0; /* could be > 0.0 */
-    integer m, n, ldjac;
-    integer iflag = 0;
+    double *wa4;
+    int m, n;
+    int iflag = 0;
     double *f;
     int err = 0;
 
     m = gretl_matrix_cols(s->oc->tmp);
     n = s->ncoeff;
-    ldjac = m; /* leading dimension of jac array */
 
     wa4 = malloc(m * sizeof *wa4);
     if (wa4 == NULL) {
@@ -1392,8 +1429,8 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
 	    f[i] *= Tfac;
 	}
 
-	fdjac2_(gmm_jacobian_calc, &m, &n, s->coeff, f, 
-		J->val, &ldjac, &iflag, &epsfcn, wa4, s);
+	fdjac2_(gmm_jacobian_calc, m, n, s->coeff, f, 
+		J->val, m, &iflag, 0.0, wa4, s);
 
 	if (iflag != 0) {
 	    fprintf(stderr, "fdjac2_: iflag = %d\n", (int) iflag);
@@ -1427,7 +1464,7 @@ int gmm_add_vcv (MODEL *pmod, nlspec *s)
     }
 
     if (!err) {
-	err = gretl_model_write_vcv(pmod, V, -1);
+	err = gretl_model_write_vcv(pmod, V);
     }
 
     if (!err && using_HAC(s)) {
@@ -1607,6 +1644,7 @@ static int maybe_prescale_weights (nlspec *s)
 	    m = pow(10, -m);
 	    fprintf(stderr, "GMM weights matrix: scaling by %g\n", m);
 	    gretl_matrix_multiply_by_scalar(s->oc->W, m);
+	    s->oc->scale_wt = m;
 	}
     }
 
@@ -1706,6 +1744,15 @@ int gmm_calculate (nlspec *s, PRN *prn)
 #endif
     }
 
+#if 1
+    if (!err && !(s->opt & (OPT_T | OPT_I))) {
+	/* we automatically scaled the identity matrix for weighting
+	   purposes: undo the scaling here
+	*/
+	s->crit /= s->oc->scale_wt;
+    }
+#endif
+
     if (oldcoeff != NULL) {
 	free(oldcoeff);
     }
@@ -1728,8 +1775,11 @@ static int needs_rejigging (gretl_matrix *m, int t1, int t2)
 	return 1;
     }
 
-    if (matrix_is_dated(m)) {
-	if (m->t1 != t1 || m->t2 != t2) {
+    if (gretl_matrix_is_dated(m)) {
+	int mt1 = gretl_matrix_get_t1(m);
+	int mt2 = gretl_matrix_get_t2(m);
+
+	if (mt1 != t1 || mt2 != t2) {
 	    /* offset is wrong */
 	    return 1;
 	}
@@ -1816,7 +1866,8 @@ int gmm_missval_check_etc (nlspec *s)
 #if 0
     /* thought: allow matrix version of GMM for vectors longer
        then the dataset -- but not ready yet */
-    if (!matrix_is_dated(s->oc->e) && !matrix_is_dated(s->oc->Z)) {
+    if (!gretl_matrix_is_dated(s->oc->e) && 
+	!gretl_matrix_is_dated(s->oc->Z)) {
 	int nr1 = s->oc->e->rows;
 	int nr2 = s->oc->Z->rows;
 	
@@ -1827,21 +1878,27 @@ int gmm_missval_check_etc (nlspec *s)
     }
 #endif
 
-    if (matrix_is_dated(s->oc->e)) {
-	if (s->oc->e->t1 > s->t1) {
-	    s->t1 = s->oc->e->t1;
+    if (gretl_matrix_is_dated(s->oc->e)) {
+	int et1 = gretl_matrix_get_t1(s->oc->e);
+	int et2 = gretl_matrix_get_t2(s->oc->e);
+
+	if (et1 > s->t1) {
+	    s->t1 = et1;
 	}
-	if (s->oc->e->t2 < s->t2) {
-	    s->t2 = s->oc->e->t2;
+	if (et2 < s->t2) {
+	    s->t2 = et2;
 	}
     }
 
-    if (matrix_is_dated(s->oc->Z)) {
-	if (s->oc->Z->t1 > s->t1) {
-	    s->t1 = s->oc->Z->t1;
+    if (gretl_matrix_is_dated(s->oc->Z)) {
+	int Zt1 = gretl_matrix_get_t1(s->oc->Z);
+	int Zt2 = gretl_matrix_get_t2(s->oc->Z);
+
+	if (Zt1 > s->t1) {
+	    s->t1 = Zt1;
 	}
-	if (s->oc->Z->t2 < s->t2) {
-	    s->t2 = s->oc->Z->t2;
+	if (Zt2 < s->t2) {
+	    s->t2 = Zt2;
 	}
     }
 
