@@ -51,8 +51,11 @@ typedef enum {
     GPT_FILL_SWITCH    = 1 << 21, /* switching from errorbars to fill */
     GPT_ERR_SWITCH     = 1 << 22, /* switching from fill to errorbars */
     GPT_MONO           = 1 << 23, /* monochrome output */
-    GPT_GRID           = 1 << 24, /* display default gnuplot grid */
-    GPT_POLAR          = 1 << 25  /* plot is in polar mode */
+    GPT_GRID_Y         = 1 << 24, /* display horizontal grid lines */
+    GPT_GRID_X         = 1 << 25, /* display vertical grid lines */
+    GPT_POLAR          = 1 << 26, /* plot is in polar mode */
+    GPT_XL             = 1 << 27, /* large */
+    GPT_XXL            = 1 << 28  /* extra-large */
 } GptFlags; 
 
 typedef struct gretlRGB_ gretlRGB;
@@ -70,6 +73,16 @@ typedef struct GPT_SPEC_ GPT_SPEC;
 #define BOXCOLOR (N_GP_COLORS - 2)
 #define SHADECOLOR (N_GP_COLORS - 1)
 
+#define GP_WIDTH      640
+#define GP_HEIGHT     480
+#define GP_LB_WIDTH   680
+#define GP_LB_HEIGHT  400
+#define GP_XL_WIDTH   680
+#define GP_XL_HEIGHT  510
+#define GP_XXL_WIDTH  680
+#define GP_XXL_HEIGHT 680
+#define GP_SQ_SIZE    480
+
 typedef enum {
     PLOT_REGULAR = 0,
     PLOT_H_TEST,
@@ -111,6 +124,7 @@ typedef enum {
     PLOT_FIT_CUBIC,
     PLOT_FIT_INVERSE,
     PLOT_FIT_LOESS,
+    PLOT_FIT_LOGLIN,
     PLOT_FIT_NA       /* fit option not applicable */
 } FitType;
 
@@ -128,7 +142,6 @@ typedef enum {
 
 typedef enum {
     GP_PNG_NONE,  /* no PNG terminal available */
-    GP_PNG_OLD,   /* old driver from gnuplot 3.N */
     GP_PNG_GD1,   /* libgd driver, no truecolor option */
     GP_PNG_GD2,   /* libgd with truecolor option */
     GP_PNG_CAIRO  /* newer cairo-based driver */
@@ -140,6 +153,16 @@ typedef enum {
     GP_PDF_CAIRO
 } PDFTerm;
 
+typedef enum {
+    GP_EPS_NONE,
+    GP_EPS_PS,
+    GP_EPS_CAIRO
+} EPSTerm;
+
+#define maybe_big_multiplot(c) (c == PLOT_MULTI_IRF || \
+				c == PLOT_MULTI_SCATTER || \
+				c == PLOT_PANEL)
+
 #define frequency_plot_code(c) (c == PLOT_FREQ_SIMPLE || \
 				c == PLOT_FREQ_NORMAL || \
 				c == PLOT_FREQ_GAMMA)
@@ -153,6 +176,10 @@ const char *get_png_line_for_plotspec (const GPT_SPEC *spec);
 
 const char *get_gretl_emf_term_line (PlotType ptype, int color);
 
+const char *get_gretl_pdf_term_line (PlotType ptype, GptFlags flags);
+
+const char *get_gretl_eps_term_line (PlotType ptype, GptFlags flags);
+
 const char *gp_justification_string (int j);
 
 int split_graph_fontspec (const char *s, char *name, int *psz);
@@ -167,7 +194,7 @@ void gnuplot_cleanup (void);
 
 int specified_gp_output_format (void);
 
-int write_plot_type_string (PlotType ptype, FILE *fp);
+int write_plot_type_string (PlotType ptype, GptFlags flags, FILE *fp);
 
 void write_plot_line_styles (int ptype, FILE *fp);
 
@@ -192,6 +219,9 @@ int gnuplot (const int *plotlist, const char *literal,
 int multi_scatters (const int *list, const DATASET *dset, 
 		    gretlopt opt);
 
+int matrix_scatters (const gretl_matrix *m, const int *list, 
+		     const DATASET *dset, gretlopt opt);
+
 int gnuplot_3d (int *list, const char *literal,
 		DATASET *dset, gretlopt opt);
 
@@ -263,6 +293,8 @@ int gnuplot_has_ttf (int reset);
 
 int gnuplot_pdf_terminal (void);
 
+int gnuplot_eps_terminal (void);
+
 int gnuplot_png_terminal (void);
 
 int gnuplot_has_latin5 (void);
@@ -295,6 +327,7 @@ void gnuplot_png_set_use_aa (int s);
 
 #ifndef WIN32
 int gnuplot_has_wxt (void);
+double gnuplot_get_version (void);
 #endif
 
 #endif /* GRAPHING_H */
