@@ -26,8 +26,8 @@
 #include "usermat.h"
 #include "gretl_scalar.h"
 
-#include "gretl_f2c.h"
-#include "../../minpack/minpack.h"  
+#include "../../minpack/minpack.h"
+#include <float.h> 
 
 #define BFGS_DEBUG 0
 
@@ -460,12 +460,12 @@ static int NR_fallback_hessian (double *b, gretl_matrix *H,
 
 #define ALT_OPG 0
 
-/* build the G matrix, given a set of coefficient estimates, b, and a
-   function for calculating the per-observation contributions
-   to the loglikelihood, lltfun 
+/* build the T x k G matrix, given a set of coefficient estimates, 
+   @b, and a function for calculating the per-observation contributions
+   to the loglikelihood, @lltfun 
 */
 
-gretl_matrix *numerical_score_matrix (double *b, int k, int T,
+gretl_matrix *numerical_score_matrix (double *b, int T, int k,
 				      BFGS_LLT_FUNC lltfun,
 				      void *data, int *err)
 {
@@ -478,7 +478,7 @@ gretl_matrix *numerical_score_matrix (double *b, int k, int T,
     double bi0, x0;
     int i, t;
 
-    G = gretl_zero_matrix_new(k, T);
+    G = gretl_zero_matrix_new(T, k);
     if (G == NULL) {
 	*err = E_ALLOC;
 	return NULL;
@@ -496,7 +496,7 @@ gretl_matrix *numerical_score_matrix (double *b, int k, int T,
 	    goto bailout;
 	}
 	for (t=0; t<T; t++) {
-	    gretl_matrix_set(G, i, t, x[t]);
+	    gretl_matrix_set(G, t, i, x[t]);
 	}
 	b[i] = bi0 + h;
 	x = lltfun(b, i, data);
@@ -505,8 +505,8 @@ gretl_matrix *numerical_score_matrix (double *b, int k, int T,
 	    goto bailout;
 	}
 	for (t=0; t<T; t++) {
-	    x0 = gretl_matrix_get(G, i, t);
-	    gretl_matrix_set(G, i, t, (x[t] - x0) / (2.0 * h));
+	    x0 = gretl_matrix_get(G, t, i);
+	    gretl_matrix_set(G, t, i, (x[t] - x0) / (2.0 * h));
 	}
 	b[i] = bi0;
 #if NLS_DEBUG
@@ -1139,7 +1139,7 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 
     while (1) {
 	/* Call the L-BFGS-B code */
-	setulb_(&n, &m, b, l, u, nbd, &f, g, &reltol, &pgtol, wa, iwa, 
+	setulb_(n, m, b, l, u, nbd, &f, g, &reltol, &pgtol, wa, iwa, 
 		task, csave, lsave, isave, dsave);
 
 	iter = isave[29] + 1;
@@ -1530,12 +1530,11 @@ double user_BFGS (gretl_matrix *b,
 		  PRN *prn, int *err)
 {
     umax *u;
-    double ret = NADBL;
     gretlopt opt = OPT_NONE;
-    int verbose = libset_get_bool(MAX_VERBOSE);
     int maxit = BFGS_MAXITER_DEFAULT;
-    int fcount = 0, gcount = 0;
+    int verbose, fcount = 0, gcount = 0;
     double tol;
+    double ret = NADBL;
 
     u = umax_new(GRETL_TYPE_DOUBLE);
     if (u == NULL) {
@@ -1557,6 +1556,7 @@ double user_BFGS (gretl_matrix *b,
     }
 
     tol = libset_get_double(BFGS_TOLER);
+    verbose = libset_get_bool(MAX_VERBOSE);
 
     if (verbose) {
 	opt = OPT_V;
@@ -1593,13 +1593,12 @@ double user_NR (gretl_matrix *b,
 		PRN *prn, int *err)
 {
     umax *u;
-    double ret = NADBL;
     double crittol = 1.0e-7;
     double gradtol = 1.0e-7;
     gretlopt opt = OPT_NONE;
-    int verbose = libset_get_bool(MAX_VERBOSE);
     int maxit = 100;
     int iters = 0;
+    double ret = NADBL;
 
     u = umax_new(GRETL_TYPE_DOUBLE);
     if (u == NULL) {
@@ -1620,7 +1619,7 @@ double user_NR (gretl_matrix *b,
 	return NADBL;
     }
 
-    if (verbose) {
+    if (libset_get_bool(MAX_VERBOSE)) {
 	opt = OPT_V;
 	u->prn = prn;
     }
@@ -1644,16 +1643,65 @@ double user_NR (gretl_matrix *b,
     return ret;
 }
 
+double user_simann (gretl_matrix *b, 
+		    const char *fncall,
+		    int maxit,
+		    DATASET *dset,
+		    PRN *prn, int *err)
+{
+    umax *u;
+    gretlopt opt = OPT_NONE;
+    double ret = NADBL;
+
+    u = umax_new(GRETL_TYPE_DOUBLE);
+    if (u == NULL) {
+	*err = E_ALLOC;
+	return ret;
+    }
+
+    u->ncoeff = gretl_vector_get_length(b);
+    if (u->ncoeff == 0) {
+	*err = E_DATA;
+	goto bailout;
+    }
+
+    u->b = b;
+
+    *err = user_gen_setup(u, fncall, NULL, NULL, dset);
+    if (*err) {
+	return NADBL;
+    }
+
+    if (libset_get_bool(MAX_VERBOSE)) {
+	opt = OPT_V;
+	u->prn = prn;
+    }
+
+    *err = gretl_simann(b->val, u->ncoeff, maxit, 
+			user_get_criterion, C_OTHER,
+			u, opt, prn);
+
+    if (!*err) {
+	ret = user_get_criterion(b->val, u);	    
+    }
+
+ bailout:
+
+    umax_destroy(u);
+
+    return ret;
+}
+
 #define JAC_DEBUG 0
 
-static int user_calc_fvec (integer *m, integer *n, double *x, double *fvec,
-			   integer *iflag, void *p)
+static int user_calc_fvec (int m, int n, double *x, double *fvec,
+			   int *iflag, void *p)
 {
     umax *u = (umax *) p;
     gretl_matrix *v;
     int i, err;
 
-    for (i=0; i<*n; i++) {
+    for (i=0; i<n; i++) {
 	u->b->val[i] = x[i];
     }
 
@@ -1677,11 +1725,11 @@ static int user_calc_fvec (integer *m, integer *n, double *x, double *fvec,
     gretl_matrix_print(v, "matrix from u->f");
 #endif
 
-    if (v == NULL || gretl_vector_get_length(v) != *m) {
+    if (v == NULL || gretl_vector_get_length(v) != m) {
 	fprintf(stderr, "user_calc_fvec: got bad matrix\n"); 
 	*iflag = -1;
     } else {
-	for (i=0; i<*m; i++) {
+	for (i=0; i<m; i++) {
 	    fvec[i] = v->val[i];
 	}
     }
@@ -1689,7 +1737,7 @@ static int user_calc_fvec (integer *m, integer *n, double *x, double *fvec,
     return 0;
 }
 
-static int fdjac_allocate (integer m, integer n,
+static int fdjac_allocate (int m, int n,
 			   gretl_matrix **J, 
 			   double **w, double **f)
 {
@@ -1713,11 +1761,10 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
 {
     umax *u;
     gretl_matrix *J = NULL;
-    integer m, n;
-    integer iflag = 0;
+    int m, n;
+    int iflag = 0;
     double *wa = NULL;
     double *fvec = NULL;
-    double epsfcn = 0.0;
     int i;
 
     *err = 0;
@@ -1766,8 +1813,8 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
 	fvec[i] = u->fm_out->val[i];
     }
 
-    fdjac2_(user_calc_fvec, &m, &n, theta->val, fvec, J->val, 
-	    &m, &iflag, &epsfcn, wa, u);
+    fdjac2_(user_calc_fvec, m, n, theta->val, fvec, J->val, 
+	    m, &iflag, 0.0, wa, u);
 
  bailout:
 
@@ -1841,16 +1888,16 @@ static void print_NR_status (int status, double crittol, double gradtol,
 			     double sumgrad, PRN *prn)
 {
     if (status == GRADTOL_MET) {
-	pprintf(prn, "Gradient within tolerance (%g)\n", gradtol);
+	pprintf(prn, _("Gradient within tolerance (%g)\n"), gradtol);
     } else if (status == CRITTOL_MET) {
-	pprintf(prn, "Successive criterion values within tolerance (%g)\n",
+	pprintf(prn, _("Successive criterion values within tolerance (%g)\n"),
 		crittol);
     } else if (status == STEPMIN_MET) {
 	if (sumgrad > 0) {
-	    pprintf(prn, "Warning: couldn't improve criterion (gradient = %g)\n",
+	    pprintf(prn, _("Warning: couldn't improve criterion (gradient = %g)\n"),
 		    sumgrad);
 	} else {
-	    pprintf(prn, "Warning: couldn't improve criterion\n");
+	    pprintf(prn, _("Warning: couldn't improve criterion\n"));
 	}
     }
 }
@@ -2110,3 +2157,137 @@ int newton_raphson_max (double *b, int n, int maxit,
 
     return err;
 }
+
+static void set_up_matrix (gretl_matrix *m, double *val, 
+			   int rows, int cols)
+{
+    m->val = val;
+    m->rows = rows;
+    m->cols = cols;
+    m->info = NULL;
+}
+
+/**
+ * gretl_simann:
+ * @theta: parameter array.
+ * @n: length of @theta.
+ * @maxit: the maximum number of iterations to perform.
+ * @cfunc: the function to be maximized.
+ * @crittype: 
+ * @data: pointer to be passed to the @cfunc callback.
+ * @opt:
+ * @prn:
+ *
+ * Simulated annealing: can help to improve the initialization
+ * of @theta for numerical optimization. On exit the value of
+ * @theta is set to the func-best point in case of improvement,
+ * otherwise to the last point visited.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */ 
+
+int gretl_simann (double *theta, int n, int maxit,
+		  BFGS_CRIT_FUNC cfunc, int crittype,
+		  void *data, gretlopt opt, PRN *prn)
+{
+    gretl_matrix b;
+    gretl_matrix *b0 = NULL;
+    gretl_matrix *b1 = NULL;
+    gretl_matrix *bstar = NULL;
+    gretl_matrix *d = NULL;
+    double f0, f1;
+    double fbest, fworst;
+    double Temp = 1.0;
+    double radius = 1.0;
+    int improved = 0;
+    int i, err = 0;
+
+    if (maxit <= 0) {
+	maxit = 1024;
+    }
+
+    set_up_matrix(&b, theta, n, 1);
+
+    b0 = gretl_matrix_copy(&b);
+    b1 = gretl_matrix_copy(&b);
+    bstar = gretl_matrix_copy(&b);
+    d = gretl_column_vector_alloc(n);
+
+    if (b0 == NULL || b1 == NULL || bstar == NULL || d == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    f0 = fbest = fworst = cfunc(b.val, data);
+
+    if (opt & OPT_V) {
+	pprintf(prn, "\nSimulated annealing: initial function value = %.8g\n",
+		f0);
+    }
+
+    /* Question: should the initial radius be a function of the scale
+       of the initial parameter vector?
+    */
+
+    for (i=0; i<maxit; i++) {
+	gretl_matrix_random_fill(d, D_NORMAL);
+	gretl_matrix_multiply_by_scalar(d, radius);
+	gretl_matrix_add_to(b1, d);
+	f1 = cfunc(b1->val, data);
+
+	if (!na(f1) && (f1 > f0 || gretl_rand_01() < Temp)) {
+	    /* jump to the new point */
+	    f0 = f1;
+	    gretl_matrix_copy_values(b0, b1);
+	    if (f0 > fbest) {
+		fbest = f0;
+		gretl_matrix_copy_values(bstar, b0);
+		if (opt & OPT_V) {
+		    if (!improved) {
+			pprintf(prn, "\n%6s %12s %12s %12s\n",
+				"iter", "temp", "radius", "fbest");
+		    }
+		    pprintf(prn, "%6d %#12.6g %#12.6g %#12.6g\n", 
+			    i, Temp, radius, fbest);
+		}
+		improved = 1;
+	    } else if (f0 < fworst) {
+		fworst = f0;
+	    }
+	} else {
+	    /* revert to where we were */
+	    gretl_matrix_copy_values(b1, b0);
+	    f1 = f0;
+	}
+
+	Temp *= 0.999;
+	radius *= 0.9999;
+    }
+
+    if (improved) {
+	/* use the best point */
+	gretl_matrix_copy_values(&b, bstar);
+    } else {
+	/* use the last point */
+	gretl_matrix_copy_values(&b, b0);
+    }
+
+    if (improved) {
+	if (opt & OPT_V) pputc(prn, '\n');
+    } else {
+	pprintf(prn, "No improvement found in %d iterations\n\n", maxit);
+    }
+    
+    if (fbest - fworst < 1.0e-9) {
+	pprintf(prn, "*** warning: surface seems to be flat\n");
+    }
+
+ bailout:
+
+    gretl_matrix_free(b0);
+    gretl_matrix_free(b1);
+    gretl_matrix_free(bstar);
+    gretl_matrix_free(d);
+
+    return err;
+}
