@@ -629,6 +629,37 @@ gretl_matrix *gretl_bundle_get_matrix (gretl_bundle *bundle,
     return m;
 }
 
+/**
+ * gretl_bundle_get_series:
+ * @bundle: bundle to access.
+ * @key: name of key to access.
+ * @n: location to receive length of series.
+ * @err: location to receive error code.
+ *
+ * Returns: the series associated with @key in the
+ * specified @bundle, if any; otherwise NULL.
+ */
+
+double *gretl_bundle_get_series (gretl_bundle *bundle,
+				 const char *key,
+				 int *n, int *err)
+{
+    double *x = NULL;
+    GretlType type;
+    void *ptr;
+
+    ptr = gretl_bundle_get_data(bundle, key, &type, n, err);
+    if (!*err && type != GRETL_TYPE_SERIES) {
+	*err = E_TYPES;
+    }
+
+    if (!*err) {
+	x = (double *) ptr;
+    }
+
+    return x;
+}
+
 /**
  * gretl_bundle_get_scalar:
  * @bundle: bundle to access.
@@ -661,6 +692,36 @@ double gretl_bundle_get_scalar (gretl_bundle *bundle,
     return x;
 }
 
+/**
+ * gretl_bundle_get_string:
+ * @bundle: bundle to access.
+ * @key: name of key to access.
+ * @err: location to receive error code.
+ *
+ * Returns: the string value associated with @key in the
+ * specified @bundle, if any; otherwise #NADBL.
+ */
+
+const char *gretl_bundle_get_string (gretl_bundle *bundle,
+				     const char *key,
+				     int *err)
+{
+    const char *ret = NULL;
+    GretlType type;
+    void *ptr;
+
+    ptr = gretl_bundle_get_data(bundle, key, &type, NULL, err);
+    if (!*err && type != GRETL_TYPE_STRING) {
+	*err = E_TYPES;
+    }
+
+    if (!*err) {
+	ret = (const char *) ptr;
+    }
+
+    return ret;
+}
+
 /**
  * gretl_bundle_get_note:
  * @bundle: bundle to access.
@@ -799,6 +860,67 @@ int gretl_bundle_set_data (gretl_bundle *bundle, const char *key,
     return err;
 }
 
+/**
+ * gretl_bundle_set_string:
+ * @bundle: target bundle.
+ * @key: name of key to create or replace.
+ * @str: the string to set.
+ * 
+ * Sets @str as a member of @bundle under the name @key.
+ * If @key is already present in the bundle the original 
+ * value is replaced and destroyed.
+ *
+ * Returns: 0 on success, error code on error.
+ */
+
+int gretl_bundle_set_string (gretl_bundle *bundle, const char *key,
+			     const char *str)
+{
+    return gretl_bundle_set_data(bundle, key, (void *) str, 
+				 GRETL_TYPE_STRING, 0);
+}
+
+/**
+ * gretl_bundle_set_scalar:
+ * @bundle: target bundle.
+ * @key: name of key to create or replace.
+ * @val: the value to set.
+ * 
+ * Sets @val as a member of @bundle under the name @key.
+ * If @key is already present in the bundle the original 
+ * value is replaced and destroyed.
+ *
+ * Returns: 0 on success, error code on error.
+ */
+
+int gretl_bundle_set_scalar (gretl_bundle *bundle, const char *key,
+			     double val)
+{
+    return gretl_bundle_set_data(bundle, key, &val, 
+				 GRETL_TYPE_DOUBLE, 0);
+}
+
+/**
+ * gretl_bundle_set_series:
+ * @bundle: target bundle.
+ * @key: name of key to create or replace.
+ * @x: array of doubles.
+ * @n: the length of @x.
+ * 
+ * Sets @x as a member of @bundle under the name @key.
+ * If @key is already present in the bundle the original 
+ * value is replaced and destroyed.
+ *
+ * Returns: 0 on success, error code on error.
+ */
+
+int gretl_bundle_set_series (gretl_bundle *bundle, const char *key,
+			     const double *x, int n)
+{
+    return gretl_bundle_set_data(bundle, key, (void *) x, 
+				 GRETL_TYPE_SERIES, n);
+}
+
 /**
  * gretl_bundle_delete_data:
  * @bundle: target bundle.
@@ -893,6 +1015,42 @@ int gretl_bundle_set_creator (gretl_bundle *bundle, const char *name)
     return err;
 }
 
+/* callback for adding icons representing bundles to the GUI
+   session window */
+
+static BUNDLE_ADD_FUNC bundle_add_callback;
+static BUNDLE_DEL_FUNC bundle_delete_callback;
+
+/**
+ * set_bundle_add_callback:
+ * @callback: function function to put in place.
+ *
+ * Sets the callback function to be invoked when a
+ * bundle is added to the stack of saved objects.  Intended
+ * for synchronizing the GUI program with the saved object
+ * state.
+ */
+
+void set_bundle_add_callback (BUNDLE_ADD_FUNC callback)
+{
+    bundle_add_callback = callback; 
+}
+
+/**
+ * set_bundle_delete_callback:
+ * @callback: function function to put in place.
+ *
+ * Sets the callback function to be invoked when a
+ * bundle is to be removed from the stack of saved objects.  Intended
+ * for synchronizing the GUI program with the saved object
+ * state.
+ */
+
+void set_bundle_delete_callback (BUNDLE_DEL_FUNC callback)
+{
+    bundle_delete_callback = callback; 
+}
+
 /**
  * save_named_bundle:
  * @name: name to give the bundle.
@@ -920,32 +1078,18 @@ int save_named_bundle (const char *name)
 	    strcpy(b->name, name);
 	    b->level = level;
 	    err = gretl_bundle_push(b, 0);
+    
+	    if (!err && bundle_add_callback != NULL && 
+		!bundle_name_is_temp(b->name) &&
+		gretl_function_depth() == 0) {
+		(*bundle_add_callback)(b);
+	    }
 	}
     }
 
     return err;
 }
 
-/* callback for adding icons representing bundles to the GUI
-   session window */
-
-static void (*bundle_add_callback)(void);
-
-/**
- * set_bundle_add_callback:
- * @callback: function function to out in place.
- *
- * Sets the callback function to be invoked when a
- * bundle is added to the stack of saved objects.  Intended
- * for synchronizing the GUI program with the saved object
- * state.
- */
-
-void set_bundle_add_callback (void (*callback))
-{
-    bundle_add_callback = callback; 
-}
-
 /* For use by geneval: take @bundle, created on the fly, and stack
    it under @name.  If there's already a bundle with the given name
    destroy and replace it.
@@ -961,17 +1105,17 @@ int gretl_bundle_add_or_replace (gretl_bundle *bundle, const char *name)
     bundle->level = level;
 
     if (b0idx >= 0) {
-	/* just replace on stack */
+	/* replace on stack */
 	gretl_bundle_destroy(bundles[b0idx]);
 	bundles[b0idx] = bundle;
     } else {
 	err = gretl_bundle_push(bundle, 0);
+    }
 
-	if (bundle_add_callback != NULL && 
-	    !bundle_name_is_temp(bundle->name) &&
-	    gretl_function_depth() == 0) {
-	    (*bundle_add_callback)();
-	}
+    if (!err && bundle_add_callback != NULL && 
+	!bundle_name_is_temp(bundle->name) &&
+	gretl_function_depth() == 0) {
+	(*bundle_add_callback)(bundle);
     }
 
     return err;
@@ -1303,6 +1447,8 @@ int copy_bundle_arg_as (const gretl_bundle *b, const char *newname)
  * gretl_bundle_localize:
  * @origname: name of bundle at caller level.
  * @localname: name to be used within function.
+ * whether the bundle currently allows direct access to
+ * bundled matrices.
  *
  * On entry to a function, renames the named bundle (provided 
  * as an argument) and sets its level so that is is accessible
@@ -1335,7 +1481,7 @@ int gretl_bundle_localize (const char *origname,
  * @origname: name used at caller level.
  *
  * On exit from a function, restores the original name and
- * level of a bundle which has been made available as an argument. 
+ * level of a bundle which has been made available as an argument.
  * 
  * Returns: 0 on success, non-zero on error.
  */
@@ -1358,6 +1504,25 @@ int gretl_bundle_unlocalize (const char *localname,
     return err;
 }
 
+/* provide an alternative to gretl_bundle_delete_by_name(),
+   below, that does NOT invoke bundle_delete_callback,
+   so that it is usable by that callback
+*/
+
+int user_bundle_destroy (gretl_bundle *b)
+{
+    int i, err = E_UNKVAR;
+
+    for (i=0; i<n_saved_bundles; i++) {
+	if (bundles[i] == b) {
+	    err = real_unstack_bundle(i, UNSTACK_AND_FREE);
+	    break;
+	}
+    }
+
+    return err;
+}
+
 /**
  * gretl_bundle_delete_by_name:
  * @name: name of bundle to delete.
@@ -1374,6 +1539,13 @@ int gretl_bundle_delete_by_name (const char *name, PRN *prn)
     int level = gretl_function_depth();
     int i, err = E_UNKVAR;
 
+    if (bundle_delete_callback != NULL && 
+	!bundle_name_is_temp(name) &&
+	gretl_function_depth() == 0) {
+	/* run this through the GUI so things stay in sync */
+	return (*bundle_delete_callback)(name);
+    }
+
     for (i=0; i<n_saved_bundles; i++) {
 	if (bundles[i]->level == level && 
 	    !strcmp(name, bundles[i]->name)) {
