@@ -59,10 +59,18 @@ gretl_matrix *gretl_bundle_get_matrix (gretl_bundle *bundle,
 				       const char *key,
 				       int *err);
 
+double *gretl_bundle_get_series (gretl_bundle *bundle,
+				 const char *key,
+				 int *n, int *err);
+
 double gretl_bundle_get_scalar (gretl_bundle *bundle,
 				const char *key,
 				int *err);
 
+const char *gretl_bundle_get_string (gretl_bundle *bundle,
+				     const char *key,
+				     int *err);
+
 const char *gretl_bundle_get_note (gretl_bundle *bundle, const char *key);
 
 const char *gretl_bundle_get_creator (gretl_bundle *bundle);
@@ -75,12 +83,27 @@ const char *bundled_item_get_note (bundled_item *item);
 int gretl_bundle_set_data (gretl_bundle *bundle, const char *key,
 			   void *ptr, GretlType type, int size);
 
+int gretl_bundle_set_string (gretl_bundle *bundle, const char *key,
+			     const char *str);
+
+int gretl_bundle_set_scalar (gretl_bundle *bundle, const char *key,
+			     double val);
+
+int gretl_bundle_set_series (gretl_bundle *bundle, const char *key,
+			     const double *x, int n);
+
 int gretl_bundle_set_note (gretl_bundle *bundle, const char *key,
 			   const char *note);
 
 int gretl_bundle_delete_data (gretl_bundle *bundle, const char *key);
 
-void set_bundle_add_callback (void (*callback));
+typedef void (*BUNDLE_ADD_FUNC) (gretl_bundle *);
+typedef int (*BUNDLE_DEL_FUNC) (const char *);
+
+
+void set_bundle_add_callback (BUNDLE_ADD_FUNC callback);
+
+void set_bundle_delete_callback (BUNDLE_DEL_FUNC callback);
 
 int gretl_bundle_add_or_replace (gretl_bundle *bundle, const char *name);
 
@@ -94,6 +117,8 @@ int copy_bundle_arg_as (const gretl_bundle *b, const char *newname);
 
 int gretl_bundle_delete_by_name (const char *name, PRN *prn);
 
+int user_bundle_destroy (gretl_bundle *b);
+
 int gretl_bundle_set_name (gretl_bundle *b, const char *name);
 
 int gretl_bundle_set_creator (gretl_bundle *b, const char *name);
