@@ -69,9 +69,10 @@ typedef enum {
     E_TOOFEW,      /* 47 */
     E_FUNCERR,     /* 48 : error set by function writer */
     E_STOP,        /* 49 : user aborted execution */
-    E_DB_DUP,      /* 50 : duplicate vars found when saving to database */
-    E_OK,          /* 51 : not really an error */
-    E_MAX          /* 52 */
+    E_BADCOMMA,    /* 50 */
+    E_DB_DUP,      /* 51 : duplicate vars found when saving to database */
+    E_OK,          /* 52 : not really an error */
+    E_MAX          /* 53 */
 } GretlError; 
 
 enum gretl_warning_codes {
@@ -91,6 +92,8 @@ const char *gretl_errmsg_get (void);
 
 const char *gretl_warnmsg_get (void);
 
+char *maybe_save_gretl_errmsg (int err);
+
 void gretl_errmsg_set (const char *str);
 
 void gretl_warnmsg_set (const char *str);
