@@ -53,11 +53,13 @@ static gchar *gretl_Rout;
 static gchar *gretl_Rmsg;
 static gchar *gretl_Oxprog;
 static gchar *gretl_octave_prog;
+static gchar *gretl_stata_prog;
 
 enum {
     LANG_R = 1,
     LANG_OX,
-    LANG_OCTAVE
+    LANG_OCTAVE,
+    LANG_STATA
 };
 
 static void destroy_foreign (void)
@@ -78,7 +80,9 @@ static int set_foreign_lang (const char *lang, PRN *prn)
     } else if (g_ascii_strcasecmp(lang, "ox") == 0) {
 	foreign_lang = LANG_OX;
     } else if (g_ascii_strcasecmp(lang, "octave") == 0) {
-	foreign_lang = LANG_OCTAVE;	
+	foreign_lang = LANG_OCTAVE;
+    } else if (g_ascii_strcasecmp(lang, "stata") == 0) {
+	foreign_lang = LANG_STATA;
     } else {
 	pprintf(prn, "%s: unknown language\n", lang);
 	err = E_DATA;
@@ -109,6 +113,17 @@ static const gchar *gretl_octave_filename (void)
     return gretl_octave_prog;
 }
 
+static const gchar *gretl_stata_filename (void)
+{
+    if (gretl_stata_prog == NULL) {
+	const char *dotdir = gretl_dotdir();
+
+	gretl_stata_prog = g_strdup_printf("%sgretltmp.do", dotdir);
+    }
+
+    return gretl_stata_prog;
+}
+
 static void make_gretl_R_names (void)
 {
     static int done;
@@ -184,6 +199,8 @@ static int lib_run_other_sync (gretlopt opt, PRN *prn)
 	path = gretl_octave_path();
 	fname = gretl_octave_filename();
 	cmd = g_strdup_printf("\"%s\" --silent \"%s\"", path, fname);
+    } else if (foreign_lang == LANG_STATA) {
+	cmd = g_strdup_printf("stata /q /e gretltmp.do");
     } else {
 	return 1;
     }
@@ -210,6 +227,21 @@ static char *win32_dotpath (void)
 
 #else /* !G_OS_WIN32 */
 
+/* print from Stata's batch logfile */
+
+static void do_stata_printout (PRN *prn)
+{
+    gchar *buf = NULL;
+
+    /* we're located in dotdir at this point */
+
+    if (g_file_get_contents("gretltmp.log", &buf, NULL, NULL)) {
+	pputs(prn, buf);
+	g_free(buf);
+	pputc(prn, '\n');
+    }
+}
+
 static int lib_run_prog_sync (char **argv, gretlopt opt, PRN *prn)
 {
     gchar *sout = NULL;
@@ -242,8 +274,12 @@ static int lib_run_prog_sync (char **argv, gretlopt opt, PRN *prn)
     } else if (sout != NULL) {
 	if (!(opt & OPT_Q)) {
 	    /* with OPT_Q, don't print non-error output */
-	    pputs(prn, sout);
-	    pputc(prn, '\n');
+	    if (foreign_lang == LANG_STATA) {
+		do_stata_printout(prn);
+	    } else {
+		pputs(prn, sout);
+		pputc(prn, '\n');
+	    }
 	}
     } else {
 	pprintf(prn, "%s: %s\n", argv[0], "Got no output");
@@ -272,19 +308,29 @@ static int lib_run_R_sync (gretlopt opt, PRN *prn)
 
 static int lib_run_other_sync (gretlopt opt, PRN *prn)
 {
-    char *argv[4];
+    char *argv[5];
     int err;
 
     if (foreign_lang == LANG_OX) {
 	argv[0] = (char *) gretl_oxl_path();
 	argv[1] = (char *) gretl_ox_filename();
 	argv[2] = NULL;
-    } else {
+    } else if (foreign_lang == LANG_OCTAVE) {
 	argv[0] = (char *) gretl_octave_path();
 	argv[1] = "--silent";
 	argv[2] = (char *) gretl_octave_filename();
 	argv[3] = NULL;
-    }
+    } else if (foreign_lang == LANG_STATA) {
+	argv[0] = "stata";
+	argv[1] = "-b";
+	argv[2] = "-q";
+	argv[3] = "gretltmp.do";
+	argv[4] = NULL;
+	/* otherwise there's no way to control the location
+	   of the state output (gretltmp.log)
+	*/
+	gretl_chdir(gretl_dotdir());
+    }	
 
     err = lib_run_prog_sync(argv, opt, prn);
 
@@ -393,6 +439,49 @@ static int write_octave_io_file (void)
     return 0;
 }
 
+static int write_stata_io_file (void)
+{
+    static int written;
+
+    if (!written) {
+	const char *dotdir = gretl_dotdir();
+	gchar *fname;
+	FILE *fp;
+
+	fname = g_strdup_printf("%sgretl_export.ado", dotdir);
+	fp = gretl_fopen(fname, "w");
+	g_free(fname);
+
+	if (fp == NULL) {
+	    return E_FOPEN;
+	} else {
+	    fputs("program define gretl_export\n", fp);
+	    /* not sure about req'd version, but see mat2txt.ado */
+	    fputs("version 8.2\n", fp);
+	    fputs("local matrix `1'\n", fp);
+	    fputs("local fname `2'\n", fp);
+	    fputs("tempname myfile\n", fp);
+	    fputs("file open `myfile' using \"`fname'\", "
+		  "write text replace\n", fp);
+	    fputs("local nrows = rowsof(`matrix')\n", fp);
+	    fputs("local ncols = colsof(`matrix')\n", fp);
+	    fputs("file write `myfile' %8.0g (`nrows') %8.0g (`ncols') _n\n", fp);
+	    fputs("forvalues r=1/`nrows' {\n", fp);
+	    fputs("  forvalues c=1/`ncols' {\n", fp);
+	    fputs("    file write `myfile' %15.0e (`matrix'[`r',`c']) _n\n", fp);
+	    fputs("  }\n", fp);
+	    fputs("}\n", fp);
+	    fputs("file close `myfile'\n", fp);
+	    fputs("end\n", fp);
+
+	    fclose(fp);
+	    written = 1;
+	}
+    }
+
+    return 0;
+}
+
 static void add_gretl_include (FILE *fp)
 {
 #ifdef G_OS_WIN32
@@ -406,7 +495,9 @@ static void add_gretl_include (FILE *fp)
 	}
     } else if (foreign_lang == LANG_OCTAVE) {
 	fprintf(fp, "source(\"%sgretl_io.m\")\n", dotcpy);
-    } 
+    } else if (foreign_lang == LANG_STATA) {
+	fprintf(fp, "quietly adopath + \"%s\"\n", dotcpy);
+    }
     g_free(dotcpy);
 #else
     const char *dotdir = gretl_dotdir();
@@ -419,7 +510,9 @@ static void add_gretl_include (FILE *fp)
 	}
     } else if (foreign_lang == LANG_OCTAVE) {
 	fprintf(fp, "source(\"%sgretl_io.m\")\n", dotdir);
-    } 
+    } else if (foreign_lang == LANG_STATA) {
+	fprintf(fp, "quietly adopath + \"%s\"\n", dotdir);
+    }
 #endif
 }
 
@@ -476,6 +569,81 @@ int write_gretl_ox_file (const char *buf, gretlopt opt, const char **pfname)
     return 0;
 }
 
+static int write_data_for_stata (const DATASET *dset,
+				 FILE *fp)
+{
+    const char *save_na = get_csv_na_string();
+    gchar *sdata;
+    int err;
+
+    set_csv_na_string(".");
+    sdata = g_strdup_printf("%sstata.csv", gretl_dotdir());
+    err = write_data(sdata, NULL, dset, OPT_C, 0);
+    set_csv_na_string(save_na);
+ 
+    if (err) {
+	gretl_errmsg_sprintf("write_data_for_stata: failed with err = %d\n", err);
+    } else {
+	fputs("* load data from gretl\n", fp);
+	fputs("insheet using \"stata.csv\"\n", fp);
+    }
+
+    g_free(sdata);
+
+    return err;
+}
+
+int write_gretl_stata_file (const char *buf, gretlopt opt, 
+			    const DATASET *dset,
+			    const char **pfname)
+{
+    const gchar *fname = gretl_stata_filename();
+    FILE *fp = gretl_fopen(fname, "w");
+    int err = write_stata_io_file();
+
+    if (fp == NULL) {
+	return E_FOPEN;
+    } else {
+	/* source the I-O functions */
+	if (!err) {
+	    add_gretl_include(fp);
+	}
+
+	if (opt & OPT_D) {
+	    /* --send-data */
+	    err = write_data_for_stata(dset, fp);
+	    if (err) {
+		fclose(fp);
+		return err;
+	    }
+	}
+
+	if (buf != NULL) {
+	    /* pass on the material supplied in the 'buf' argument */
+	    char line[1024];
+
+	    bufgets_init(buf);
+	    while (bufgets(line, sizeof line, buf)) {
+		fputs(line, fp);
+	    }
+	    bufgets_finalize(buf);
+	} else {
+	    /* put out the stored 'foreign' lines */
+	    int i;
+
+	    for (i=0; i<foreign_n_lines; i++) { 
+		fprintf(fp, "%s\n", foreign_lines[i]);
+	    }
+	}
+	fclose(fp);
+	if (pfname != NULL) {
+	    *pfname = fname;
+	}
+    }
+
+    return 0;
+}
+
 /* write out current dataset as an octave matrix, and, if this succeeds,
    write appropriate octave commands to @fp to source the data
 */
@@ -487,7 +655,7 @@ static int write_data_for_octave (const DATASET *dset,
     int err;
 
     mdata = g_strdup_printf("%smdata.tmp", gretl_dotdir());
-    err = write_data(mdata, NULL, dset, OPT_M | OPT_F, 0);
+    err = write_data(mdata, NULL, dset, OPT_M, 0);
  
     if (err) {
 	gretl_errmsg_sprintf("write_data_for_octave: failed with err = %d\n", err);
@@ -519,7 +687,6 @@ int write_gretl_octave_file (const char *buf, gretlopt opt,
 
 	if (opt & OPT_D) {
 	    /* --send-data */
-	    fprintf(stderr, "*** writing data for octave\n");
 	    err = write_data_for_octave(dset, fp);
 	    if (err) {
 		fclose(fp);
@@ -847,20 +1014,27 @@ void delete_gretl_R_files (void)
     }
 }
 
-void delete_gretl_ox_file (void)
+static void delete_gretl_ox_file (void)
 {
     if (gretl_Oxprog != NULL) {
 	gretl_remove(gretl_Oxprog);
     }
 }
 
-void delete_gretl_octave_file (void)
+static void delete_gretl_octave_file (void)
 {
     if (gretl_octave_prog != NULL) {
 	gretl_remove(gretl_octave_prog);
     }
 }
 
+static void delete_gretl_stata_file (void)
+{
+    if (gretl_stata_prog != NULL) {
+	gretl_remove(gretl_stata_prog);
+    }
+}
+
 /* The following code block is used if we're implementing
    gretl's R support by dlopening the R shared library
    (as opposed to executing the R binary).
@@ -1087,6 +1261,9 @@ static int gretl_Rlib_init (void)
 	};
 	int ok, argc = 3;
 
+#if FDEBUG
+	fprintf(stderr, "calling R_initEmbeddedR\n");
+#endif
 	ok = R_initEmbeddedR(argc, argv);
 	if (ok) {
 	    VR_GlobalEnv = *PR_GlobalEnv;
@@ -1103,7 +1280,7 @@ static int gretl_Rlib_init (void)
  bailout:
 
 #if FDEBUG
-    printf("gretl_Rlib_init: returning %d\n", err);
+    fprintf(stderr, "gretl_Rlib_init: returning %d\n", err);
 #endif
 
     return err;
@@ -1556,6 +1733,14 @@ int foreign_execute (const DATASET *dset,
 	    delete_gretl_octave_file();
 	} else {
 	    err = lib_run_other_sync(foreign_opt, prn);
+	}
+    } else if (foreign_lang == LANG_STATA) {
+	err = write_gretl_stata_file(NULL, foreign_opt, 
+				     dset, NULL);
+	if (err) {
+	    delete_gretl_stata_file();
+	} else {
+	    err = lib_run_other_sync(foreign_opt, prn);
 	}	
     } else {
 	/* "can't happen" */
