@@ -65,6 +65,8 @@ const char *fn_param_descrip (const ufunc *fun, int i);
 const char **fn_param_value_labels (const ufunc *fun, int i, 
 				    int *n);
 
+int fn_param_has_default (const ufunc *fun, int i);
+
 double fn_param_default (const ufunc *fun, int i);
 
 double fn_param_minval (const ufunc *fun, int i);
@@ -103,9 +105,8 @@ int gretl_function_append_line (const char *line);
 int gretl_is_public_user_function (const char *name);
 
 int gretl_function_exec (ufunc *u, fnargs *args, int rtype,
-			 double ***pZ, DATAINFO *pdinfo,
-			 void *ret, char **descrip,
-			 PRN *prn);
+			 DATASET *dset, void *ret, 
+			 char **descrip, PRN *prn);
 
 char *gretl_func_get_arg_name (const char *argvar, int *err);
 
@@ -113,9 +114,7 @@ int object_is_const (const char *name);
 
 int object_is_function_arg (const char *name);
 
-void sample_range_get_extrema (const DATAINFO *pdinfo, int *t1, int *t2);
-
-const char *get_funcerr_message (void);
+void sample_range_get_extrema (const DATASET *dset, int *t1, int *t2);
 
 int function_return_type_from_string (const char *s);
 
@@ -138,9 +137,11 @@ const char *function_package_get_name (fnpkg *pkg);
 
 int function_package_write_file (fnpkg *pkg);
 
-int create_and_write_function_package (const char *fname, PRN *prn);
+int create_and_write_function_package (const char *fname, 
+				       gretlopt opt,
+				       PRN *prn);
 
-int check_function_needs (const DATAINFO *pdinfo, FuncDataReq dreq,
+int check_function_needs (const DATASET *dset, FuncDataReq dreq,
 			  int minver);
 
 int write_session_functions_file (const char *fname);
