@@ -19,6 +19,7 @@
 
 #include "libgretl.h"
 #include "texprint.h"
+#include "libset.h"
 
 #include <glib.h>
 
@@ -162,7 +163,7 @@ int get_local_decpoint (void)
 /* fudges for strings that should not be in utf-8 under some
    conditions: under GTK translations always come out in utf-8 in the
    GUI, but when we're sending stuff to stderr we may have to put it
-   into ISO-8859-N.
+   into ISO-8859-N or some MS Code Page.
 */
 
 static int gretl_cset_maj;
@@ -170,7 +171,8 @@ static int gretl_cset_min;
 # ifdef WIN32
 static int gretl_cpage;
 # endif
-static int using_utf8;
+static int native_utf8;
+
 
 /* Use g_get_charset() to determine the current local character set,
    and record this information.  If we get an "ISO-XXXX-Y" locale,
@@ -184,16 +186,15 @@ void set_gretl_charset (void)
     const char *charset = NULL;
     char gretl_charset[32];
 
-    using_utf8 = g_get_charset(&charset);
+    native_utf8 = g_get_charset(&charset);
 
-    if (using_utf8) {
-	set_tex_use_utf(1);
+    if (native_utf8) {
 	set_stdio_use_utf8();
     }
 
     *gretl_charset = '\0';
 
-    if (!using_utf8 && charset != NULL && *charset != '\0') {
+    if (!native_utf8 && charset != NULL && *charset != '\0') {
 	char *p;
 
 	strncat(gretl_charset, charset, 31);
@@ -272,7 +273,28 @@ static const char *get_gretl_charset (void)
     return NULL;
 }
 
-#endif
+void get_suitable_tex_encoding (char *encfile)
+{
+    int done = 0;
+
+# ifdef WIN32
+    if (gretl_cpage > 0) {
+	sprintf(encfile, "cp%d", gretl_cpage);
+	done = 1;
+    }
+# else
+    if (gretl_cset_maj == 8859 && gretl_cset_min > 0) {
+	sprintf(encfile, "latin%d", gretl_cset_min);
+	done = 1;
+    } 
+# endif
+
+    if (!done) {
+	strcpy(encfile, "latin1");
+    }
+}
+
+#endif /* ENABLE_NLS */
 
 int iso_latin_version (void)
 {
@@ -325,18 +347,6 @@ int chinese_locale (void)
 #endif
 }
 
-static int gui_native_printing;
-
-void set_gui_native_printing (void)
-{
-    gui_native_printing = 1;
-}
-
-void unset_gui_native_printing (void)
-{
-    gui_native_printing = 0;
-}
-
 #ifdef ENABLE_NLS
 
 char *iso_gettext (const char *msgid)
@@ -363,8 +373,9 @@ char *iso_gettext (const char *msgid)
     /* iso_switch: we'll reckon that if the system character set is
        not UTF-8, and is an ISO-8859-N or Windows CP12NN 8-bit set,
        then we should probably recode when printing translated strings
-       in the context of writing TeX, CSV and RTF files.  If
-       iso_switch is non-zero (once it's determinate) this makes the
+       in the context of writing CSV and RTF files.  
+
+       If iso_switch is non-zero (once it's determinate) this makes the
        I_() gettext macro use the system encoding, otherwise I_() is
        equivalent to plain _(), which always spits out UTF-8.  There's
        a possible gotcha here: I suspect that a UTF-8 encoded RTF file 
@@ -382,7 +393,7 @@ char *iso_gettext (const char *msgid)
 	iso_switch = (cset != NULL);
     }
 
-    if (iso_switch && !gui_native_printing) {
+    if (iso_switch) {
 	bind_textdomain_codeset(PACKAGE, cset);
 	ret = gettext(msgid);
 	bind_textdomain_codeset(PACKAGE, "UTF-8");
@@ -393,6 +404,92 @@ char *iso_gettext (const char *msgid)
     return ret;
 } 
 
+/* return translated @msgid in UTF-8, when this is not
+   the default operation for plain gettext()
+*/
+
+static char *force_utf8_gettext (const char *msgid)
+{
+    static const char *cset;
+    char *ret;
+
+    if (cset == NULL) {
+	cset = get_gretl_charset();
+    }
+
+    bind_textdomain_codeset(PACKAGE, "UTF-8");
+    ret = gettext(msgid);
+    bind_textdomain_codeset(PACKAGE, cset);
+
+    return ret;
+} 
+
+/* Cases to consider below:
+
+   (1) The system-native encoding is UTF-8: in that case all output
+   should be in UTF-8, with the possible exception of RTF; and all
+   output _will_ be in UTF-8 if we just call plain gettext().
+
+   (2) The system-native encoding is not UTF-8 (this means
+   MS Windows -- or possibly ancient Linux?). Then we have two
+   sub-cases.
+
+   (2a) We're in the GUI program: plain gettext() will have been
+   coerced to produce UTF-8 for the sake of GTK. But we should
+   probably arrange for RTF and CSV output to be in the locale
+   encoding, and also any output going to the console.
+
+   (2b) We're at the command line (gretlcli): gettext() output will
+   be in the system locale, but under the "new scheme" (December
+   2011) TeX output should be coerced to UTF-8. All other output
+   should use plain gettext().
+*/
+
+enum {
+    GETTEXT_DEFAULT,
+    GETTEXT_FORCE_UTF8,
+    GETTEXT_FORCE_LOCALE
+};
+
+static int gettext_mode;
+
+void set_alt_gettext_mode (PRN *prn)
+{
+    gettext_mode = GETTEXT_DEFAULT;
+
+    if (prn != NULL && !native_utf8) {
+	if (gretl_in_gui_mode()) {
+	    if (rtf_format(prn) || csv_format(prn) || 
+		printing_to_standard_stream(prn)) {
+		gettext_mode = GETTEXT_FORCE_LOCALE;
+	    }
+	} else if (tex_format(prn)) {
+	    /* CLI mode, writing TeX */
+	    gettext_mode = GETTEXT_FORCE_UTF8;
+	}
+    }
+}
+
+char *alt_gettext (const char *msgid)
+{
+    if (gettext_mode == GETTEXT_DEFAULT) {
+	return gettext(msgid);
+    } else if (gettext_mode == GETTEXT_FORCE_UTF8) {
+	return force_utf8_gettext(msgid);
+    } else if (gettext_mode == GETTEXT_FORCE_LOCALE) {
+	return iso_gettext(msgid);
+    } else {
+	return gettext(msgid);
+    }
+}
+
+#else /* !ENABLE_NLS */
+
+void set_alt_gettext_mode (PRN *prn)
+{
+    return;
+}
+
 #endif /* ENABLE_NLS */
 
 #ifdef WIN32
@@ -481,15 +578,13 @@ const char *lang_string_from_id (int langid)
 
 int lang_id_from_name (const char *s)
 {
-    int i;
+    if (s != NULL || *s != '\0') {
+	int i;
 
-    if (s == NULL || *s == '\0') {
-	return 0;
-    }
-
-    for (i=0; i<LANG_MAX; i++) {
-	if (!strcmp(s, langs[i].name)) {
-	    return langs[i].id;
+	for (i=0; i<LANG_MAX; i++) {
+	    if (!strcmp(s, langs[i].name)) {
+		return langs[i].id;
+	    }
 	}
     }
 
@@ -591,7 +686,8 @@ void set_lcnumeric (int langid, int lcnumeric)
 
 #ifdef ENABLE_NLS
 
-static int set_locale_with_workaround (const char *lcode)
+static int 
+set_locale_with_workaround (int langid, const char *lcode)
 {
     char *test = setlocale(LC_ALL, lcode);
 
@@ -606,6 +702,15 @@ static int set_locale_with_workaround (const char *lcode)
 
     if (test != NULL) {
 	fprintf(stderr, "setlocale: '%s' -> '%s'\n", lcode, test);
+	if (strcmp("_File", _("_File")) == 0) {
+	    const char *langstr;
+	    char tmp[64];
+	    
+	    langstr = lang_string_from_id(langid);
+	    sscanf(langstr, "%s", tmp);
+	    lower(tmp);
+	    gretl_setenv("LANGUAGE", tmp);
+	}
     }
 
     return test == NULL;
@@ -641,7 +746,7 @@ int test_locale (const char *langstr)
     *ocpy = '\0';
     strncat(ocpy, orig, 63);
 
-    err = set_locale_with_workaround(lcode);
+    err = set_locale_with_workaround(langid, lcode);
 
     if (err) {
 	gretl_errmsg_sprintf(_("%s: locale is not supported "
@@ -679,7 +784,7 @@ int force_language (int langid)
 		err = 1;
 	    }
 # else
-	    err = set_locale_with_workaround(lcode);
+	    err = set_locale_with_workaround(langid, lcode);
 # endif
 	}
     }
@@ -1094,32 +1199,3 @@ int get_translated_width (const char *str)
     return w;
 }
 
-static int printing_to_console;
-
-#ifdef ENABLE_NLS
-
-char *maybe_iso_gettext (const char *msgid)
-{
-   if (printing_to_console) {
-       return iso_gettext(msgid);
-   } else {
-       return gettext(msgid);
-   }
-} 
-
-#endif
-
-void check_for_console (PRN *prn)
-{
-    if (prn != NULL) {
-	printing_to_console = 
-	    printing_to_standard_stream(prn);
-    }
-}
-
-void console_off (void)
-{
-    printing_to_console = 0;
-}
-
-
