@@ -80,11 +80,18 @@ static saved_list *saved_list_new (const int *list, const char *name)
 	    free(sl);
 	    sl = NULL;
 	} else {
-	    *sl->name = 0;
+	    *sl->name = '\0';
 	    strncat(sl->name, name, VNAMELEN - 1);
 	}
     }
 
+#if LDEBUG
+    if (sl != NULL) {
+	fprintf(stderr, "saved_list_new: '%s' (level %d)\n", name, sl->level);
+	printlist(sl->list, "sl->list");
+    }
+#endif
+
     return sl;
 }
 
@@ -106,6 +113,11 @@ static saved_list *get_saved_list_by_name (const char *name)
 	if (!strcmp(name, list_stack[i]->name) && 
 	    fsd == list_stack[i]->level) {
 	    sl = list_stack[i];
+#if LDEBUG > 1
+	    fprintf(stderr, "get_saved_list_by_name: found '%s' (level %d)\n", 
+		    sl->name, fsd);
+	    printlist(sl->list, "sl->list");
+#endif
 	    break;
 	}
     }
@@ -205,6 +217,10 @@ int *get_list_by_name (const char *name)
     int *ret = NULL;
     saved_list *sl;
 
+#if LDEBUG > 1
+    fprintf(stderr, "get_list_by_name: '%s'\n", name);
+#endif
+
     if (name != NULL) {
 	sl = get_saved_list_by_name(name);
 	if (sl != NULL) {
@@ -231,6 +247,12 @@ int append_to_list_by_name (const char *targ, const int *add)
     saved_list *sl = get_saved_list_by_name(targ);
     int err = 0;
 
+#if LDEBUG
+    fprintf(stderr, "append_to_list_by_name: '%s'\n", targ);
+    printlist(sl->list, "original (target) list");
+    printlist(add, "list to add");
+#endif
+
     if (sl == NULL) {
 	err = E_UNKVAR;
     } else {
@@ -247,6 +269,10 @@ int append_to_list_by_name (const char *targ, const int *add)
 	}
     } 
 
+#if LDEBUG
+    printlist(sl->list, "final list");
+#endif
+
     return err;
 }
 
@@ -266,6 +292,10 @@ int subtract_from_list_by_name (const char *targ, const int *sub)
     saved_list *sl = get_saved_list_by_name(targ);
     int err = 0;
 
+#if LDEBUG
+    fprintf(stderr, "subtract_from_list_by_name: '%s'\n", targ);
+#endif
+
     if (sl == NULL) {
 	err = E_UNKVAR;
     } else {
@@ -296,6 +326,10 @@ int replace_list_by_name (const char *targ, const int *src)
     saved_list *sl = get_saved_list_by_name(targ);
     int err = 0;
 
+#if LDEBUG
+    fprintf(stderr, "replace_list_by_name: '%s'\n", targ);
+#endif
+
     if (sl == NULL) {
 	err = E_UNKVAR;
     } else {
@@ -312,8 +346,13 @@ int replace_list_by_name (const char *targ, const int *src)
     return err;
 }
 
+enum {
+    LIST_ADD_NORMAL,
+    LIST_FORCE_NEW
+};
+
 static int real_remember_list (const int *list, const char *name, 
-			       int force_new, PRN *prn)
+			       int flag, PRN *prn)
 {
     saved_list *orig = NULL;
     int err = 0;
@@ -323,18 +362,18 @@ static int real_remember_list (const int *list, const char *name,
     }
 
 #if LDEBUG
-    fprintf(stderr, "remember_list (in): name='%s', force_new=%d,"
-	    " n_lists=%d\n", name, force_new, n_lists);
+    fprintf(stderr, "remember_list (in): name='%s', flag=%d,"
+	    " n_lists=%d\n", name, flag, n_lists);
 #endif
 
-    /* Note: 'force_new' means that we'll add a new list even if it
+    /* Note: if @flag = LIST_FORCE_NEW we'll add a new list even if it
        has the same name as an existing one.  This makes sense only if
        we're copying a list in the context of running a user-defined
        function, since in that case the new list will exist at a
        different "stack level" from any prior list.
     */
 
-    if (!force_new) {
+    if (flag != LIST_FORCE_NEW) {
 	orig = get_saved_list_by_name(name);
     }
 
@@ -365,7 +404,7 @@ static int real_remember_list (const int *list, const char *name,
 	if (list_stack[n_lists] == NULL) {
 	    err = E_ALLOC;
 	} else {
-	    if (gretl_messages_on()) {
+	    if (prn != NULL && gretl_messages_on()) {
 		const char *realname = list_stack[n_lists]->name;
 
 		if (list[0] > 0) {
@@ -406,7 +445,28 @@ static int real_remember_list (const int *list, const char *name,
 
 int remember_list (const int *list, const char *name, PRN *prn)
 {
-    return real_remember_list(list, name, 0, prn);
+    return real_remember_list(list, name, LIST_ADD_NORMAL, prn);
+}
+
+/**
+ * declare_list:
+ * @name: name to be given to the list.
+ *
+ * Implements bare declaration of a list in the "genr" context:
+ * adds a null list under the given @name.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+int declare_list (const char *name)
+{
+    int tmp[] = {0};
+
+#if LDEBUG
+    fprintf(stderr, "declare_list: '%s'\n", name);
+#endif
+
+    return real_remember_list(tmp, name, LIST_ADD_NORMAL, NULL);
 }
 
 static int destroy_saved_list (saved_list *sl)
@@ -500,17 +560,16 @@ int rename_saved_list (const char *orig, const char *newname)
 
 int *copy_list_as (const int *list, const char *name)
 {
-    int err = real_remember_list(list, name, 1, NULL);
-    int *ret = NULL;
+    int err = real_remember_list(list, name, LIST_FORCE_NEW, NULL);
 
     if (!err) {
 	saved_list *sl = list_stack[n_lists - 1];
 
 	sl->level += 1;
-	ret = sl->list;
+	return sl->list;
+    } else {
+	return NULL;
     }
-
-    return ret;
 }
 
 /**
@@ -529,24 +588,19 @@ int *copy_list_as (const int *list, const char *name)
 
 int *create_named_null_list (const char *name)
 {
-    int *list = gretl_null_list();
+    int tmp[] = {0};
+    int err;
 
-    if (list != NULL) {
-	saved_list *sl; 
-	int err;
+    err = real_remember_list(tmp, name, LIST_FORCE_NEW, NULL);
 
-	err = real_remember_list(list, name, 1, NULL);
-	free(list);
-	list = NULL;
+    if (!err) {
+	saved_list *sl = list_stack[n_lists - 1];
 
-	if (!err) {
-	    sl = list_stack[n_lists - 1];
-	    sl->level += 1;
-	    list = sl->list;
-	} 
+	sl->level += 1;
+	return sl->list;
+    } else {
+	return NULL;
     }
-
-    return list;
 }
 
 /**
@@ -567,25 +621,19 @@ int *create_named_null_list (const char *name)
 
 int *create_named_singleton_list (int varnum, const char *name)
 {
-    int *list = gretl_list_new(1);
+    int tmp[] = {1, varnum};
+    int err;
+
+    err = real_remember_list(tmp, name, LIST_FORCE_NEW, NULL);
  
-    if (list != NULL) {
-	saved_list *sl;
-	int err;
-	
-	list[1] = varnum;
-	err = real_remember_list(list, name, 1, NULL);
-	free(list);
-	list = NULL;
+    if (!err) {
+	saved_list *sl = list_stack[n_lists - 1]; 
 
-	if (!err) {
-	    sl = list_stack[n_lists - 1];
-	    sl->level += 1;
-	    list = sl->list;
-	}
+	sl->level += 1;
+	return sl->list;
+    } else {
+	return NULL;
     }
-
-    return list;
 }
 
 /**
@@ -603,11 +651,12 @@ int *create_named_singleton_list (int varnum, const char *name)
 int destroy_saved_lists_at_level (int level)
 {
     saved_list **lstack;
-    int i, j, nl = 0;
-    int err = 0;
+    int nl = n_lists;
+    int i, j, err = 0;
 
     for (i=0; i<n_lists; i++) {
 	if (list_stack[i] == NULL) {
+	    /* we reached the end of the (occupied) stack */
 	    break;
 	}
 	if (list_stack[i]->level == level) {
@@ -615,11 +664,10 @@ int destroy_saved_lists_at_level (int level)
 	    for (j=i; j<n_lists - 1; j++) {
 		list_stack[j] = list_stack[j+1];
 	    }
-	    list_stack[n_lists - 1] = NULL;
+	    list_stack[n_lists-1] = NULL;
 	    i--;
-	} else {
-	    nl++;
-	}
+	    nl--;
+	} 
     }
 
     if (nl < n_lists) {
@@ -683,7 +731,7 @@ int gretl_lists_revise (const int *dlist, int dmin)
     int lmax = 0;
     int i, j, k;
 
-#if 0
+#if LDEBUG
     fprintf(stderr, "gretl_lists_revise: dlist = %p, dmin = %d\n", 
 	    (void *) dlist, dmin);
 #endif
@@ -735,6 +783,10 @@ int gretl_lists_revise (const int *dlist, int dmin)
     /* use mapping to revise saved lists */
     for (j=0; j<n_lists; j++) {
 	list = list_stack[j]->list;
+#if LDEBUG
+	fprintf(stderr, "maybe revising list '%s'\n", list_stack[j]->name);
+	printlist(list, "original list");
+#endif
 	if (list == NULL) {
 	    continue;
 	}
@@ -748,6 +800,9 @@ int gretl_lists_revise (const int *dlist, int dmin)
 		}
 	    }
 	}
+#if LDEBUG
+	printlist(list, "revised?");
+#endif
     }
 
     free(maplist);
@@ -1282,17 +1337,35 @@ int *gretl_list_from_vector (const gretl_vector *v, int *err)
     return list;
 }
 
+#define LN_10 2.30258509299404590
+
+static int integer_length (int k)
+{
+    double x = k;
+    int len = 0;
+
+    if (x < 0) {
+	x = fabs(x);
+	len = 1;
+    } 
+
+    if (x < 10) {
+	len += 1;
+    } else {
+	len += (int) ceil(log(x)/LN_10);
+	len += (k % 10 == 0);
+    }
+
+    return len;
+}
+
 /**
  * gretl_list_to_string:
  * @list: array of integers.
  * 
  * Prints the given @list of integers into a newly
  * allocated string, separated by single spaces and with
- * one leading space.  This function is designed to handle 
- * positive integers in a range that is sensible for ID 
- * numbers of variables, typically with three digits or less, 
- * and will fail if the list contains any numbers greater 
- * than 9998.
+ * one leading space.
  *
  * Returns: The string representation of the list on success,
  * or NULL on failure.
@@ -1301,35 +1374,37 @@ int *gretl_list_from_vector (const gretl_vector *v, int *err)
 char *gretl_list_to_string (const int *list)
 {
     char *buf;
-    char numstr[8];
-    int len, i, err = 0;
+    int i, len = 1;
+
+    for (i=1; i<=list[0]; i++) {
+	if (list[i] == LISTSEP) {
+	    len += 2;
+	} else {
+	    len += integer_length(list[i]) + 1;
+	}
+    }
 
-    len = 6 * (list[0] + 1);
     if (len > MAXLINE - 32) {
+	/* string would be too long for command line */
 	return NULL;
     }
 
     buf = malloc(len);
-    if (buf == NULL) {
-	return NULL;
-    }
 
-    *buf = '\0';
+    if (buf != NULL) {
+	char numstr[16];
 
-    for (i=1; i<=list[0]; i++) {
-	if (list[i] == LISTSEP) {
-	    strcat(buf, " ;");
-	} else {
-	    sprintf(numstr, " %d", list[i]);
-	    strcat(buf, numstr);
+	*buf = '\0';
+	for (i=1; i<=list[0]; i++) {
+	    if (list[i] == LISTSEP) {
+		strcat(buf, " ;");
+	    } else {
+		sprintf(numstr, " %d", list[i]);
+		strcat(buf, numstr);
+	    }
 	}
     }
 
-    if (err) {
-	free(buf);
-	buf = NULL;
-    }
-
     return buf;
 }
 
@@ -2644,25 +2719,23 @@ int gretl_list_duplicates (const int *list, GretlCmdIndex ci)
  * @list1: 
  * @list2: 
  *
- * Returns: 1 if there are any elements in common between @list1
- * and @list2, otherwise 0.
+ * Returns: the number of elements that are in common between 
+ * @list1 and @list2.
  */
 
 int gretl_lists_share_members (const int *list1, const int *list2)
 {
-    int i;
+    int i, n = 0;
 
-    if (list1 == NULL || list2 == NULL) {
-	return 0;
-    }
-
-    for (i=1; i<=list1[0]; i++) {
-	if (in_gretl_list(list2, list1[i])) {
-	    return 1;
+    if (list1 != NULL && list2 != NULL) {
+	for (i=1; i<=list1[0]; i++) {
+	    if (in_gretl_list(list2, list1[i])) {
+		n++;
+	    }
 	}
     }
 
-    return 0;
+    return n;
 }
 
 /**
