@@ -32,7 +32,6 @@
    as non-zero for the purpose of determining the rank of X */
 
 #define R_DIAG_MIN 1.0e-8
-#define BLOCKT -666
 
 typedef enum {
     GRETL_MOD_NONE = 0,
@@ -54,6 +53,8 @@ typedef enum {
 
 typedef struct gretl_matrix_ gretl_vector;
 
+typedef struct matrix_info_ matrix_info;
+
 /**
  * gretl_matrix:
  * @rows: number of rows in matrix
@@ -71,9 +72,8 @@ typedef struct gretl_matrix_ gretl_vector;
 typedef struct gretl_matrix_ {
     int rows;
     int cols;
-    int t1;
-    int t2;
     double *val;
+    matrix_info *info;
 } gretl_matrix;
 
 typedef struct gretl_matrix_block_ gretl_matrix_block;
@@ -216,6 +216,8 @@ gretl_matrix *gretl_matrix_reuse (gretl_matrix *m, int rows, int cols);
 
 int gretl_matrix_realloc (gretl_matrix *m, int rows, int cols);
 
+void gretl_matrix_init (gretl_matrix *m);
+
 gretl_matrix *gretl_matrix_replace (gretl_matrix **pa, 
 				    gretl_matrix *b);
 
@@ -321,6 +323,9 @@ int gretl_vector_copy_values (gretl_vector *targ,
 int gretl_matrix_copy_values (gretl_matrix *targ, 
 			      const gretl_matrix *src);
 
+int gretl_matrix_copy_data (gretl_matrix *targ, 
+			    const gretl_matrix *src);
+
 int gretl_matrix_copy_values_shaped (gretl_matrix *targ, 
 				     const gretl_matrix *src);
 
@@ -462,10 +467,14 @@ double gretl_matrix_infinity_norm (const gretl_matrix *m);
 
 int gretl_LU_solve (gretl_matrix *a, gretl_matrix *b);
 
+int gretl_LU_solve_invert (gretl_matrix *a, gretl_matrix *b);
+
 int gretl_cholesky_decomp_solve (gretl_matrix *a, gretl_matrix *b);
 
 int gretl_cholesky_solve (const gretl_matrix *a, gretl_vector *b);
 
+int gretl_cholesky_invert (gretl_matrix *a);
+
 gretl_vector *gretl_toeplitz_solve (const gretl_vector *c, 
 				    const gretl_vector *r, 
 				    const gretl_vector *b, 
@@ -507,9 +516,6 @@ double gretl_symmetric_matrix_rcond (const gretl_matrix *m, int *err);
 
 double gretl_matrix_rcond (const gretl_matrix *m, int *err);
 
-int gretl_general_eigen_sort (gretl_matrix *evals, gretl_matrix *evecs, 
-			      int rank);
-
 int gretl_symmetric_eigen_sort (gretl_matrix *evals, gretl_matrix *evecs, 
 				int rank);
 
@@ -523,15 +529,20 @@ gretl_symmetric_matrix_eigenvals (gretl_matrix *m,
 				  int eigenvecs, 
 				  int *err);
 
+gretl_matrix *
+gretl_symm_matrix_eigenvals_descending (gretl_matrix *m, 
+					int eigenvecs, 
+					int *err);
+
 gretl_matrix *
 gretl_gensymm_eigenvals (const gretl_matrix *A, 
 			 const gretl_matrix *B, 
 			 gretl_matrix *V, 
 			 int *err);
 
-double gretl_symm_matrix_lambda_min (const gretl_matrix *m);
+double gretl_symm_matrix_lambda_min (const gretl_matrix *m, int *err);
 
-double gretl_symm_matrix_lambda_max (const gretl_matrix *m);
+double gretl_symm_matrix_lambda_max (const gretl_matrix *m, int *err);
 
 gretl_matrix *gretl_matrix_right_nullspace (const gretl_matrix *M, 
 					    int *err);
@@ -649,14 +660,16 @@ int
 gretl_matrix_diagonal_sandwich (const gretl_vector *d, const gretl_matrix *X,
 				gretl_matrix *DXD);
 
-void gretl_matrix_set_t1 (gretl_matrix *m, int t);
+int gretl_matrix_set_t1 (gretl_matrix *m, int t);
 
-void gretl_matrix_set_t2 (gretl_matrix *m, int t);
+int gretl_matrix_set_t2 (gretl_matrix *m, int t);
 
 int gretl_matrix_get_t1 (const gretl_matrix *m);
 
 int gretl_matrix_get_t2 (const gretl_matrix *m);
 
+int gretl_matrix_is_dated (const gretl_matrix *m);
+
 int gretl_is_identity_matrix (const gretl_matrix *m);
 
 int gretl_is_zero_matrix (const gretl_matrix *m);
@@ -699,7 +712,8 @@ gretl_matrix *gretl_matrix_minmax (const gretl_matrix *A,
 				   int mm, int rc, int idx,
 				   int *err);
 
-gretl_matrix *gretl_matrix_pca (const gretl_matrix *X, int p, int *err);
+gretl_matrix *gretl_matrix_pca (const gretl_matrix *X, int p, 
+				gretlopt opt, int *err);
 
 gretl_matrix *gretl_matrix_xtab (int t1, int t2, const double *x, 
 				 const double *y, int *err);
@@ -723,6 +737,16 @@ gretl_matrix *gretl_matrix_covariogram (const gretl_matrix *X,
 void gretl_matrix_transcribe_obs_info (gretl_matrix *targ,
 				       const gretl_matrix *src);
 
+int gretl_matrix_set_colnames (gretl_matrix *m, char **S);
+
+int gretl_matrix_set_rownames (gretl_matrix *m, char **S);
+
+const char **gretl_matrix_get_colnames (const gretl_matrix *m);
+
+const char **gretl_matrix_get_rownames (const gretl_matrix *m);
+
+void gretl_matrix_destroy_info (gretl_matrix *m);
+
 void lapack_mem_free (void);
 
 void set_blas_nmk_min (int nmk);
