@@ -49,17 +49,18 @@ typedef enum {
     VCV_HAC,
     VCV_ML,
     VCV_PANEL,
-    VCV_RQ
+    VCV_RQ,
+    VCV_CLUSTER
 } VCVMajorType;
 
 typedef enum {
-    VCV_UNSET,
-    VCV_HESSIAN,
-    VCV_IM,
-    VCV_OP,
-    VCV_QML,
-    VCV_BW,
-    VCV_MAX
+    ML_UNSET,
+    ML_HESSIAN,
+    ML_IM,
+    ML_OP,
+    ML_QML,
+    ML_BW,
+    ML_VCVMAX
 } MLVCVType;
 
 typedef enum {
@@ -273,6 +274,10 @@ int gretl_model_set_full_vcv_info (MODEL *pmod, int vmaj, int vmin,
 
 int gretl_model_set_vcv_info (MODEL *pmod, int vmaj, int vmin);
 
+int gretl_model_get_vcv_type (const MODEL *pmod);
+
+int gretl_model_get_cluster_var (const MODEL *pmod);
+
 void *gretl_model_get_data (const MODEL *pmod, const char *key);
 
 void *gretl_model_get_data_full (const MODEL *pmod, const char *key,
@@ -307,6 +312,8 @@ const char *gretl_model_get_depvar_name (const MODEL *pmod,
 
 int *gretl_model_get_x_list (const MODEL *pmod);
 
+int *gretl_model_get_y_list (const MODEL *pmod);
+
 int *gretl_model_get_secondary_list (const MODEL *pmod);
 
 int arma_model_nonseasonal_AR_order (const MODEL *pmod);
@@ -339,10 +346,26 @@ int gretl_model_get_coeff_separator (const MODEL *pmod, const char **ps, int *pp
 
 int gretl_model_new_vcv (MODEL *pmod, int *nelem);
 
-int gretl_model_write_vcv (MODEL *pmod, const gretl_matrix *V, int k);
+int gretl_model_write_vcv (MODEL *pmod, const gretl_matrix *V);
+
+int gretl_model_add_QML_vcv (MODEL *pmod, int ci,
+			     const gretl_matrix *H, 
+			     const gretl_matrix *G,
+			     const DATASET *dset,
+			     gretlopt opt);
+
+int gretl_model_add_hessian_vcv (MODEL *pmod, 
+				 const gretl_matrix *H);
+
+int gretl_model_add_OPG_vcv (MODEL *pmod, 
+			     const gretl_matrix *G);
 
 VMatrix *gretl_model_get_vcv (MODEL *pmod, const DATASET *dset);
 
+double gretl_model_get_vcv_element (const MODEL *pmod,
+				    int i, int j, 
+				    int np);
+
 int gretl_model_write_coeffs (MODEL *pmod, double *b, int k);
 
 int gretl_model_add_arinfo (MODEL *pmod, int nterms);
