@@ -206,7 +206,7 @@ static int maybe_recode_path (const char *path, int want_utf8, char **pconv)
  * @fname: name of file to be opened.
  * @mode: mode in which to open the file.
  *
- * A wrapper for  the C library's fopen(), making allowance for
+ * A wrapper for the C library's fopen(), making allowance for
  * the possibility that @fname has to be converted from
  * UTF-8 to the locale encoding or vice versa.
  *
@@ -250,18 +250,136 @@ FILE *gretl_fopen (const char *fname, const char *mode)
     return fp;
 }
 
+#ifdef G_OS_WIN32
+
+/* We'll not attempt to use mkstemp(), which is not supported
+   by mingw
+*/
+
+static FILE *win32_mktemp (char *tmpl, const char *mode)
+{
+    char *fname = mktemp(tmpl);
+    FILE *fp = NULL;
+
+    if (fname != NULL) {
+	fp = gretl_fopen(fname, mode);
+    }
+
+    return fp;
+}
+
+#endif
+
 /**
- * gretl_try_fopen:
- * @fname: name of file to be opened.
- * @mode: mode in which to open the file.
+ * gretl_mktemp:
+ * @template: template for filename; must end with "XXXXXX".
+ * @mode: e.g. "w" for text use or "wb" for binary mode.
  *
- * Basically the same as gretl_fopen(), except that no
- * error message is logged if @fname cannot be opened.
+ * A wrapper for the combination of mkstemp() and fdopen(),
+ * making allowance for the possibility that @template has to 
+ * be converted from UTF-8 to the locale encoding or vice versa.
+ * On successful exit @template holds the name of the newly
+ * created file.
  *
  * Returns: file pointer, or %NULL on failure.
  */
 
-FILE *gretl_try_fopen (const char *fname, const char *mode)
+FILE *gretl_mktemp (char *template, const char *mode)
+{
+    FILE *fp = NULL;
+    int fd = -1;
+
+    gretl_error_clear();
+
+#ifdef G_OS_WIN32
+    fp = win32_mktemp(template, mode);
+#else
+    fd = mkstemp(template); 
+#endif
+
+    if (errno != 0) {
+	gretl_errmsg_set_from_errno(NULL);
+    } else if (fd != -1) {
+	fp = fdopen(fd, mode);
+    }
+
+#if 0
+    fprintf(stderr, "gretl_mktemp: name='%s', fd=%d, fp=%p\n",
+	    template, fd, (void *) fp);
+#endif
+
+    return fp;
+}
+
+/**
+ * gretl_test_fopen:
+ * @fname: name of file to be opened.
+ * @mode: mode as used with fopen().
+ *
+ * Attempts to open @fname in the given mode; if the opening
+ * is successful the stream is then closed.
+ *
+ * Returns: 0 on success, -1 on filename encoding
+ * failure, or the system errno on failure of fopen().
+ */
+
+int gretl_test_fopen (const char *fname, const char *mode)
+{
+    gchar *fconv = NULL;
+    FILE *fp = NULL;
+    int err;
+
+    gretl_error_clear();
+    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+
+    if (err) {
+	gretl_error_clear();
+	err = -1;
+    } else if (fconv != NULL) {
+	fp = fopen(fconv, mode);
+	if (fp != NULL) {
+	    fclose(fp);
+	    if (*mode == 'w') {
+		gretl_remove(fconv);
+	    }
+	} else {
+	    err = errno;
+	}
+	g_free(fconv);
+    } else {
+	fp = fopen(fname, mode);
+	if (fp != NULL) {
+	    fclose(fp);
+	    if (*mode == 'w') {
+		gretl_remove(fname);
+	    }
+	} else {
+	    err = errno;
+	}
+    }
+
+    return err;
+}
+
+/**
+ * gretl_fopen_with_recode:
+ * @fname: name of file to be opened.
+ * @mode: mode in which to open the file.
+ * @recoded_fname: location to receive recoded filename,
+ * if recoding is needed.
+ *
+ * A wrapper for  the C library's fopen(), making allowance for
+ * the possibility that @fname has to be converted from
+ * UTF-8 to the locale encoding or vice versa. If conversion
+ * is carried out, the newly allocated recoded filename is 
+ * returned via the last argument (otherwise that argument is 
+ * not touched).
+ *
+ * Returns: file pointer, or %NULL on failure.
+ */
+
+FILE *gretl_fopen_with_recode (const char *fname, const char *mode,
+			       char **recoded_fname)
 {
     gchar *fconv = NULL;
     FILE *fp = NULL;
@@ -269,17 +387,35 @@ FILE *gretl_try_fopen (const char *fname, const char *mode)
 
     gretl_error_clear();
 
+#if FDEBUG
+    fprintf(stderr, "gretl_fopen_with_recode: got '%s'\n", fname);
+#endif
+
     err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
 
     if (!err) {
 	if (fconv != NULL) {
 	    fp = fopen(fconv, mode);
+#if FDEBUG
+            fprintf(stderr, "using recoded name, fp = %p\n", (void *) fp);
+#endif
+	    if (fp != NULL && recoded_fname != NULL) {
+		*recoded_fname = gretl_strdup(fconv);
+	    }
 	    g_free(fconv);
 	} else {
 	    fp = fopen(fname, mode);
 	}
     }
 
+#if FDEBUG
+    fprintf(stderr, "after fopen, errno = %d\n", errno);
+#endif
+
+    if (errno != 0) {
+	gretl_errmsg_set_from_errno(fname);
+    }
+
     return fp;
 }
 
@@ -394,6 +530,11 @@ int gretl_rename (const char *oldpath, const char *newpath)
 	}
     }
 
+#if 0
+    fprintf(stderr, "oldpath='%s'\nnewpath='%s'\n"
+	    "err=%d\n", oldpath, newpath, err);
+#endif
+
     if (oldconv != NULL || newconv != NULL) {
 	g_free(oldconv);
 	g_free(newconv);
@@ -606,41 +747,6 @@ int gretl_mkdir (const char *path)
     return !done;
 }
 
-/**
- * gretl_mkstemp:
- * @tmpl: template filename.
- *
- * Returns: A file handle (as from open()) to the file opened for 
- * reading and writing, or -1 on failure.
- */
-
-int gretl_mkstemp (char *tmpl)
-{
-    int fd = -1;
-
-    if (!g_utf8_validate(tmpl, -1, NULL)) {
-	/* g_mkstemp requires UTF-8 input */
-	gchar *pconv;
-	gsize bytes;
-
-	pconv = g_locale_to_utf8(tmpl, -1, NULL, &bytes, NULL);
-	if (pconv != NULL) {
-	    strcpy(tmpl, pconv);
-	    fd = g_mkstemp(tmpl);
-	    g_free(pconv);
-	    pconv = g_locale_from_utf8(tmpl, -1, NULL, &bytes, NULL);
-	    if (pconv != NULL) {
-		strcpy(tmpl, pconv);
-		g_free(pconv);
-	    }
-	}	
-    } else {
-	fd = g_mkstemp(tmpl);
-    }
-
-    return fd;
-}
-
 #else /* !win32 */
 
 /**
@@ -903,10 +1009,9 @@ int gretl_path_prepend (char *file, const char *path)
 static int try_open_file (char *targ, const char *finddir, 
 			  WIN32_FIND_DATA *fdata, int code)
 {
-    FILE *fp = NULL;
     char tmp[MAXLEN];
     int n = strlen(finddir);
-    int found = 0;
+    int err, found = 0;
     
     strcpy(tmp, finddir);
     tmp[n-1] = '\0';
@@ -914,15 +1019,14 @@ static int try_open_file (char *targ, const char *finddir,
     strcat(tmp, "\\");
     strcat(tmp, targ);
 
-    fp = gretl_try_fopen(tmp, "r");
-    if (fp == NULL && code == DATA_SEARCH) {
+    err = gretl_test_fopen(tmp, "r");
+    if (err && code == DATA_SEARCH) {
 	if (add_suffix(tmp, ".gdt")) {
-	    fp = gretl_try_fopen(tmp, "r");
+	    err = gretl_test_fopen(tmp, "r");
 	}
     }
 
-    if (fp != NULL) {
-	fclose(fp);
+    if (!err) {
 	strcpy(targ, tmp);
 	found = 1;
     }	
@@ -997,24 +1101,22 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
 static int try_open_file (char *targ, const char *finddir, 
 			  struct dirent *dirent, int code)
 {
-    FILE *fp = NULL;
     char tmp[MAXLEN];
-    int found = 0;
+    int err, found = 0;
     
     strcpy(tmp, finddir);
     strcat(tmp, dirent->d_name);
     strcat(tmp, "/");
     strcat(tmp, targ);
 
-    fp = gretl_try_fopen(tmp, "r");
-    if (fp == NULL && code == DATA_SEARCH) {
+    err = gretl_test_fopen(tmp, "r");
+    if (err && code == DATA_SEARCH) {
 	if (add_suffix(tmp, ".gdt")) {
-	    fp = gretl_try_fopen(tmp, "r");
+	    err = gretl_test_fopen(tmp, "r");
 	}
     }
 
-    if (fp != NULL) {
-	fclose(fp);
+    if (!err) {
 	strcpy(targ, tmp);
 	found = 1;
     }	
@@ -1078,29 +1180,33 @@ static int find_in_subdir (const char *topdir, char *fname, int code)
 
 #endif /* win32 vs posix */
 
+#define SEARCH_DEBUG 0
+
 static char *search_dir (char *fname, const char *topdir, int code)
 {
-    FILE *fp;
     char orig[MAXLEN];
+    int err;
+
+#if SEARCH_DEBUG
+    fprintf(stderr, "search_dir: trying '%s' for '%s'\n",
+	    topdir, fname);
+#endif
 
     strcpy(orig, fname);
 
     if (gretl_path_prepend(fname, topdir) == 0) {
-	fp = gretl_try_fopen(fname, "r");
-	if (fp != NULL) {
-	    fclose(fp);
+	err = gretl_test_fopen(fname, "r");
+	if (!err) {
 	    return fname;
 	}
 	if (code == DATA_SEARCH && add_suffix(fname, ".gdt")) {
-	    fp = gretl_try_fopen(fname, "r");
-	    if (fp != NULL) {
-		fclose(fp);
+	    err = gretl_test_fopen(fname, "r");
+	    if (!err) {
 		return fname;
 	    }
 	} else if (code == FUNCS_SEARCH && add_suffix(fname, ".gfn")) {
-	    fp = gretl_try_fopen(fname, "r");
-	    if (fp != NULL) {
-		fclose(fp);
+	    err = gretl_test_fopen(fname, "r");
+	    if (!err) {
 		return fname;
 	    }
 	}	    
@@ -1247,6 +1353,12 @@ char **get_plausible_search_dirs (int type, int *n_dirs)
 	build_path(dirname, gretl_app_support_dir(), subdir, NULL);
 	err = strings_array_add(&dirs, n_dirs, dirname);
     }
+#else
+    if (!err) {
+	/* the user's dotdir */
+	build_path(dirname, gretl_dotdir(), subdir, NULL);
+	err = strings_array_add(&dirs, n_dirs, dirname);
+    }
 #endif
 
     if (!err) {
@@ -1304,7 +1416,7 @@ char *gretl_function_package_get_path (const char *name,
     char *ret = NULL;
     char path[FILENAME_MAX];
     char **dirs;
-    int found = 0;
+    int err, found = 0;
     int i, n_dirs;
 
     *path = '\0';
@@ -1328,13 +1440,10 @@ char *gretl_function_package_get_path (const char *name,
 	    while ((dirent = readdir(dir)) != NULL && !found) {
 		dname = dirent->d_name;
 		if (!strcmp(dname, name)) {
-		    FILE *fp;
-
 		    sprintf(path, "%s%c%s%c%s.gfn", fndir, SLASH, 
 			    dname, SLASH, dname);
-		    fp = gretl_try_fopen(path, "r");
-		    if (fp != NULL) {
-			fclose(fp);
+		    err = gretl_test_fopen(path, "r");
+		    if (!err) {
 			found = 1;
 		    } else {
 			*path = '\0';
@@ -1389,7 +1498,12 @@ char *gretl_addpath (char *fname, int script)
 {
     char orig[MAXLEN];
     char *tmp = fname;
-    FILE *test;
+    int err;
+
+#if SEARCH_DEBUG
+    fprintf(stderr, "gretl_addpath: fname='%s', script=%d\n",
+	    fname, script);
+#endif
 
     strcpy(orig, fname);
 
@@ -1398,11 +1512,10 @@ char *gretl_addpath (char *fname, int script)
     }  
 
     /* try opening filename as given */
-    test = gretl_try_fopen(fname, "r");
+    err = gretl_test_fopen(fname, "r");
 
-    if (test != NULL) { 
+    if (!err) { 
 	/* fine, got it */
-	fclose(test); 
 	if (!fname_has_path(fname)) {
 	    make_path_absolute(fname, orig);
 	}
@@ -1416,7 +1529,7 @@ char *gretl_addpath (char *fname, int script)
 
 	if (*gpath != '\0') {
 	    /* try looking where the last-opened script was found */
-	    if ((fname = search_dir(fname, gpath, CURRENT_DIR))) {
+	    if (search_dir(fname, gpath, CURRENT_DIR) != NULL) {
 		return fname;
 	    }
 	}
@@ -1429,24 +1542,55 @@ char *gretl_addpath (char *fname, int script)
 	    /* try searching some standard gretl paths */
 	    if (script) {
 		sprintf(trydir, "%sscripts", gpath);
-		if ((fname = search_dir(fname, trydir, SCRIPT_SEARCH))) { 
+		if (search_dir(fname, trydir, SCRIPT_SEARCH) != NULL) { 
 		    return fname;
 		} else {
 		    fname = tmp;
 		    strcpy(fname, orig);
 		    sprintf(trydir, "%sfunctions", gpath);
-		    if ((fname = search_dir(fname, trydir, FUNCS_SEARCH))) { 
+		    if (search_dir(fname, trydir, FUNCS_SEARCH) != NULL) { 
 			return fname;
 		    }
 		}
 	    } else {
 		/* data file */
 		sprintf(trydir, "%sdata", gpath);
-		if ((fname = search_dir(fname, trydir, DATA_SEARCH))) { 
+		if (search_dir(fname, trydir, DATA_SEARCH) != NULL) { 
 		    return fname;
 		}
 	    } 
 	}
+
+	fname = tmp;
+	strcpy(fname, orig);
+#ifdef OSX_BUILD
+	gpath = gretl_app_support_dir();
+#else
+	gpath = gretl_dotdir();
+#endif
+
+	if (*gpath != '\0') {
+	    /* try looking in ~/Library or dotdir */
+	    if (script) {
+		sprintf(trydir, "%sscripts", gpath);
+		if (search_dir(fname, trydir, SCRIPT_SEARCH) != NULL) { 
+		    return fname;
+		} else {
+		    fname = tmp;
+		    strcpy(fname, orig);
+		    sprintf(trydir, "%sfunctions", gpath);
+		    if (search_dir(fname, trydir, FUNCS_SEARCH) != NULL) { 
+			return fname;
+		    }
+		}
+	    } else {
+		/* data file */
+		sprintf(trydir, "%sdata", gpath);
+		if (search_dir(fname, trydir, DATA_SEARCH) != NULL) { 
+		    return fname;
+		}
+	    } 
+	}	
     
 	fname = tmp;
 	strcpy(fname, orig);
@@ -1454,28 +1598,21 @@ char *gretl_addpath (char *fname, int script)
 
 	if (*gpath != '\0') {
 	    /* try looking in user's dir (and subdirs) */
-	    if ((fname = search_dir(fname, gpath, USER_SEARCH))) { 
+	    if (search_dir(fname, gpath, USER_SEARCH) != NULL) { 
 		return fname;
 	    }
 	}
-    }
 
-    /* try looking in default workdir? */
-    if (1) {
-	const char *dwork = maybe_get_default_workdir();
+	fname = tmp;
+	strcpy(fname, orig);
+	gpath = maybe_get_default_workdir();
 
-	if (dwork != NULL) {
-	    int ok = 0;
-
-	    fname = tmp;
-	    strcpy(fname, orig);
-	    if ((fname = search_dir(fname, dwork, USER_SEARCH))) { 
-		ok = 1;
-	    }
-	    if (ok) {
+	if (gpath != NULL && *gpath != '\0') {
+	    /* try looking in default workdir? */
+	    if (search_dir(fname, gpath, USER_SEARCH) != NULL) { 
 		return fname;
 	    }
-	}
+	}	    
     }
 
 #ifdef WIN32
@@ -1623,6 +1760,9 @@ int getopenfile (const char *line, char *fname, gretlopt opt)
     }
 
     /* try a basic path search on this filename */
+#if SEARCH_DEBUG
+    fprintf(stderr, "getopenfile: calling addpath\n");
+#endif
     fullname = gretl_addpath(fname, script);
 
     if (fullname != NULL && (opt & OPT_S)) {
@@ -1812,9 +1952,26 @@ const char *gretl_lib_path (void)
 	char *epath = getenv("GRETL_PLUGIN_PATH");
 
 	if (epath != NULL) {
-	    paths.libpath[0] = '\0';
+	    *paths.libpath = '\0';
 	    strncat(paths.libpath, epath, MAXLEN - 1);
 	}
+
+#if defined(LIBDIR) || defined(GRETL_PREFIX)
+	/* if blank, try drawing on compiled-in values */
+	if (*paths.libpath == '\0') {
+# ifdef LIBDIR
+	    strcat(paths.libpath, LIBDIR);
+# else
+	    strcat(paths.libpath, GRETL_PREFIX);
+	    slash_terminate(paths.libpath);
+	    strcat(paths.libpath, "lib");
+# endif
+	    slash_terminate(paths.libpath);
+	    strcat(paths.libpath, PLUGIN_SFX);
+	    slash_terminate(paths.libpath);
+	}
+#endif /* LIBDIR or GRETL_PREFIX defined */
+
 	set = 1;
     }
 
@@ -2323,7 +2480,8 @@ static void initialize_gretldir (char *dirname, gretlopt opt)
 	/* environment setting, if any, takes precedence */
 	strcpy(paths.gretldir, ghome);
 	slash_terminate(paths.gretldir);
-    } else if (*dirname != '\0' && *paths.gretldir == '\0') {
+    } else if (dirname != NULL && *dirname != '\0' && 
+	       *paths.gretldir == '\0') {
 	/* use value from config/registry, unless we already got
 	   a value somehow */
 	strcpy(paths.gretldir, dirname);
@@ -2357,6 +2515,28 @@ static void initialize_gretldir (char *dirname, gretlopt opt)
     strcpy(dirname, paths.gretldir);
 }
 
+/**
+ * set_gretl_plugin_path:
+ * @path: path to the gretl plugins.
+ *
+ * For use by third-party code: the purpose of this function
+ * is to ensure that libgretl can find its plugins. 
+ *
+ * @prefix, if given, should be the path under which the plugins
+ * are installed. On a unix-type system this might be, for example,
+ * /usr/local/lib/gretl-gtk2; on MS Windows it might be
+ * c:\program files\gretl\plugins.
+ **/
+
+void set_gretl_plugin_path (const char *path)
+{
+    if (path != NULL) {
+	*paths.libpath = '\0';
+	strncat(paths.libpath, path, MAXLEN - 2);
+	slash_terminate(paths.libpath);
+    }
+}
+
 /* Called at start-up only: set the "hidden" working dir,
    which is not user-configurable.
 */
@@ -2524,7 +2704,7 @@ static void load_default_path (char *targ)
     } else if (targ == paths.rlibpath) {
 	R_path_from_registry(targ, RLIB);
     } else if (targ == paths.oxlpath) {
-	sprintf(targ, "%s\\OxMetrics5\\Ox\\bin\\oxl.exe", progfiles);
+	sprintf(targ, "%s\\OxMetrics6\\Ox\\bin\\oxl.exe", progfiles);
     } else if (targ == paths.octpath) {
 	strcpy(targ, "C:\\Octave\\3.2.3_gcc-4.0.0\\bin\\octave.exe");
     } else if (targ == paths.pngfont) {
@@ -2592,7 +2772,7 @@ static void load_default_path (char *targ)
 #endif /* RLIBPATH */
     } else if (targ == paths.oxlpath) {
 #ifdef OSX_BUILD
-	strcpy(paths.oxlpath, "/Applications/OxMetrics5/ox/bin/oxl");
+	strcpy(paths.oxlpath, "/Applications/OxMetrics6/ox/bin/oxl");
 #else
 	strcpy(paths.oxlpath, "oxl");
 #endif /* OSX_BUILD */
