@@ -85,10 +85,15 @@ int string_is_utf8 (const unsigned char *s);
 
 FILE *gretl_fopen (const char *fname, const char *mode);
 
-FILE *gretl_try_fopen (const char *fname, const char *mode);
+FILE *gretl_fopen_with_recode (const char *fname, const char *mode,
+			       char **recoded_fname);
+
+int gretl_test_fopen (const char *fname, const char *mode);
 
 FILE *gretl_read_user_file (const char *fname);
 
+FILE *gretl_mktemp (char *template, const char *mode);
+
 int gretl_open (const char *pathname, int flags);
 
 int gretl_rename (const char *oldpath, const char *newpath);
@@ -134,6 +139,8 @@ char **get_plausible_search_dirs (int type, int *n_dirs);
 char *gretl_function_package_get_path (const char *name,
 				       PkgType type);
 
+void set_gretl_plugin_path (const char *path);
+
 const char *helpfile_path (int id);
 
 const char *gretl_home (void);
@@ -194,8 +201,6 @@ void set_gretl_png_font (const char *s);
 
 void win32_set_gretldir (const char *progname);
 
-int gretl_mkstemp (char *tmpl);
-
 #else
 
 void get_gretl_rc_path (char *rcfile);
