@@ -64,7 +64,7 @@ struct PRN_ {
     char *fname;       /* file name, or NULL */
 };
 
-#undef PRN_DEBUG
+#define PRN_DEBUG 0
 
 /**
  * gretl_print_destroy:
@@ -84,10 +84,15 @@ void gretl_print_destroy (PRN *prn)
     fpdup = (prn->fp == prn->fpaux);
 
     if (prn->fp != NULL && prn->fp != stdout && prn->fp != stderr) {
+#if PRN_DEBUG
+  	fprintf(stderr, "gretl_print_destroy: prn=%p, closing fp at %p\n", 
+		(void *) prn, (void *) prn->fp); 
+#endif
 	fclose(prn->fp);
     }
 
     if (prn->fname != NULL) {
+	/* prn had a tempfile attached */
 	gretl_remove(prn->fname);
 	free(prn->fname);
     }
@@ -110,36 +115,28 @@ void gretl_print_destroy (PRN *prn)
 
 static int prn_add_tempfile (PRN *prn)
 {
-    char fname[MAXLEN];
-    int fd, err = 0;
+    const char *dotdir = gretl_dotdir();
+    int n = strlen(dotdir) + 16;
+    int err = 0;
 
-    errno = 0;
+    prn->fname = malloc(n);
+    if (prn->fname == NULL) {
+	return E_ALLOC;
+    }
 
-    sprintf(fname, "%sprntmp.XXXXXX", gretl_dotdir());
+    sprintf(prn->fname, "%sprntmp.XXXXXX", dotdir);
+    prn->fp = gretl_mktemp(prn->fname, "w");
 
-#ifdef WIN32
-    fd = gretl_mkstemp(fname);
-#else
-    fd = mkstemp(fname);
+#if PRN_DEBUG
+    fprintf(stderr, "prn_add_tempfile: '%s' at %p\n",
+	    prn->fname, (void *) prn->fp);
 #endif
 
-    if (fd == -1) {
+    if (prn->fp == NULL) {
+	free(prn->fname);
+	prn->fname = NULL;
 	err = E_FOPEN;
-    } else {
-	prn->fname = gretl_strdup(fname);
-	if (prn->fname == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    prn->fp = fdopen(fd, "w");
-	    if (prn->fp == NULL) {
-		err = E_FOPEN;
-	    }
-	}
-    }
-
-    if (errno != 0) {
-	gretl_errmsg_set_from_errno("prn_add_tempfile");
-    }
+    } 
 
     return err;
 }
@@ -160,11 +157,6 @@ static PRN *real_gretl_print_new (PrnType ptype,
 	return NULL;
     }
 
-#if PRN_DEBUG
-    fprintf(stderr, "real_gretl_print_new: type=%d, fname='%s', buf=%p, fp=%d\n",
-	    ptype, fname, (void *) buf, (void *) fp);
-#endif    
-
     prn->fp = NULL;
     prn->fpaux = NULL;
     prn->buf = NULL;
@@ -212,12 +204,15 @@ static PRN *real_gretl_print_new (PrnType ptype,
 		free(prn);
 		prn = NULL;
 	    }
-#if PRN_DEBUG
-	    fprintf(stderr, "new prn buffer, p = %d\n", p);
-#endif
 	}
     }
 
+#if PRN_DEBUG
+    fprintf(stderr, "real_gretl_print_new at %p: type=%d, fname='%s', "
+	    "buf=%p, fp=%p\n", (void *) prn, ptype, fname, 
+	    (void *) buf, (void *) fp);
+#endif    
+
     if (perr != NULL) {
 	*perr = err;
     }
@@ -412,7 +407,15 @@ int gretl_print_rename_file (PRN *prn, const char *oldpath,
 	return E_DATA;
     }
 
+#if PRN_DEBUG
+    fprintf(stderr, "gretl_print_rename_file, prn at %p:\n oldpath='%s'\n"
+	    " newpath='%s'\n prn->fp=%p (closing)\n", (void *) prn,
+	    oldpath, newpath, (void *) prn->fp);
+    fprintf(stderr, " (old prn->fname = '%s')\n", prn->fname);
+#endif
+
     fclose(prn->fp);
+    prn->fp = NULL;
 
     if (oldpath == NULL && prn->fname != NULL) {
 	/* renaming from tempfile */
@@ -421,8 +424,14 @@ int gretl_print_rename_file (PRN *prn, const char *oldpath,
 	err = gretl_rename(oldpath, newpath);
     }
 
-    if (!err) {
+    if (err) {
+	fprintf(stderr, "%s\n", gretl_errmsg_get());
+    } else {
+	/* re-open the stream under its new name */
 	prn->fp = gretl_fopen(newpath, "a");
+#if PRN_DEBUG
+	fprintf(stderr, "gretl_print_rename_file: new fp=%p\n", prn->fp);
+#endif
 	if (prn->fname != NULL) {
 	    /* @prn originally used a tempfile: the record of
 	       the temporary filename should be deleted 
@@ -522,29 +531,35 @@ const char *gretl_print_get_trimmed_buffer (PRN *prn)
  * 
  * If @prn has a non-null buffer attached, provides
  * the width and/or height of the buffer, the width in
- * characters and the height in lines.
+ * characters and the height in lines. This function is
+ * intended for use with text designed for printing in
+ * a GUI window, and with "reasonable" line lengths for
+ * that context; if the lines are too long (more than 
+ * 120 UTF-8 characters) the values written to @width
+ * and/or @height will be zero.
  */
 
 void gretl_print_get_size (PRN *prn, int *width, int *height)
 {
     int w = 0, h = 0;
 
-    if (prn != NULL && prn->buf != NULL) {
-	const char *s = prn->buf;
-	int lw = 0;
+    if (prn != NULL && prn->buf != NULL) { 
+	char line[128];
+	int lw;
 
-	while (*s) {
-	    if (*s == '\n') {
-		h++;
-		if (lw > w) {
-		    w = lw;
-		}
-		lw = 0;
-	    } else {
-		lw++;
+	bufgets_init(prn->buf);
+	while (bufgets(line, sizeof line, prn->buf)) {
+	    lw = g_utf8_strlen(line, -1) - 1;
+	    if (lw > 120) {
+		w = h = 0;
+		break;
 	    }
-	    s++;
+	    if (lw > w) {
+		w = lw;
+	    }
+	    h++;
 	}
+	bufgets_finalize(prn->buf);
     }
 
     if (width != NULL) {
@@ -747,30 +762,32 @@ void gretl_print_toggle_doc_flag (PRN *prn)
 }
 
 /**
- * gretl_print_set_utf_flag:
+ * gretl_print_set_has_minus:
  * @prn: printing struct.
  * 
- * Sets the %GRETL_FORMAT_UTF flag on @prn.
+ * Sets the %GRETL_FORMAT_HAS_MINUS flag on @prn,
+ * indicating that the Unicode minus sign is
+ * supported.
  */
 
-void gretl_print_set_utf_flag (PRN *prn)
+void gretl_print_set_has_minus (PRN *prn)
 {
     if (prn != NULL) {
-	prn->format |= GRETL_FORMAT_UTF;
+	prn->format |= GRETL_FORMAT_HAS_MINUS;
     }
 }
 
 /**
- * gretl_print_supports_utf:
+ * gretl_print_has_minus:
  * @prn: printing struct.
  * 
- * Returns: 1 if @prn supports UTF-8 characters, else 0.
+ * Returns: 1 if @prn supports Unicode minus sign, else 0.
  */
 
-int gretl_print_supports_utf (PRN *prn)
+int gretl_print_has_minus (PRN *prn)
 {
     if (prn != NULL) {
-	return (prn->format & GRETL_FORMAT_UTF)? 1 : 0;
+	return (prn->format & GRETL_FORMAT_HAS_MINUS)? 1 : 0;
     } else {
 	return 0;
     }
@@ -784,7 +801,8 @@ static int realloc_prn_buffer (PRN *prn)
 
 #if PRN_DEBUG
     fprintf(stderr, "%d bytes left\ndoing realloc(%p, %d)\n",
-	    prn->bufsize - prn->blen, prn->buf, 2 * prn->bufsize);
+	    (int) (prn->bufsize - prn->blen), prn->buf, 
+	    (int) (2 * prn->bufsize));
 #endif
 
     newlen = prn->bufsize * 2;
@@ -797,7 +815,7 @@ static int realloc_prn_buffer (PRN *prn)
 	prn->buf = tmp;
 #if PRN_DEBUG
 	fprintf(stderr, "realloc: prn->buf is %d bytes at %p\n",
-		prn->bufsize, (void *) prn->buf);
+		(int) prn->bufsize, (void *) prn->buf);
 #endif
     }
 
@@ -814,8 +832,8 @@ static int pprintf_init (PRN *prn)
     prn->buf = malloc(prn->bufsize);
 
 #if PRN_DEBUG
-    fprintf(stderr, "pprintf: malloc'd %d bytes at %p\n", prn->bufsize,  
-	    (void *) prn->buf); 
+    fprintf(stderr, "pprintf: malloc'd %d bytes at %p\n", 
+	    (int) prn->bufsize, (void *) prn->buf); 
 #endif
 
     if (prn->buf == NULL) {
@@ -874,7 +892,7 @@ int pprintf (PRN *prn, const char *format, ...)
 	}
     }
 
-#if PRN_DEBUG
+#if PRN_DEBUG > 1
     fprintf(stderr, "printing at %p\n", (void *) (prn->buf + prn->blen));
 #endif
 
@@ -931,9 +949,9 @@ int pputs (PRN *prn, const char *s)
 	bytesleft = prn->bufsize - prn->blen;
     }	
 
-#if PRN_DEBUG
+#if PRN_DEBUG > 1
     fprintf(stderr, "pputs: bufsize=%d, blen=%d, bytesleft=%d, copying %d bytes\n", 
-	    (int) prn->bufsize, (int) blen, bytesleft, slen);
+	    (int) prn->bufsize, (int) prn->blen, bytesleft, slen);
 #endif
 
     strcpy(prn->buf + prn->blen, s);
@@ -972,7 +990,7 @@ int pputc (PRN *prn, int c)
 	}
     }
 
-#if PRN_DEBUG
+#if PRN_DEBUG > 1
     fprintf(stderr, "pputc: adding char at %p\n", (void *) (prn->buf + prn->blen));
 #endif
 
@@ -1296,7 +1314,7 @@ int gretl_print_alloc (PRN *prn, size_t s)
 	    memset(prn->buf + prn->blen, 0, 1);
 #if PRN_DEBUG
 	    fprintf(stderr, "gretl_print_alloc: realloc'd prn->buf "
-		    "to %d bytes\n", newlen);
+		    "to %d bytes\n", (int) newlen);
 #endif
 	}
     }
