@@ -651,7 +651,8 @@ static int restriction_set_form_matrices (gretl_restriction *rset)
 	err = equation_form_matrices(rset);
     } else if (rset->otype == GRETL_OBJ_VAR) {
 	err = vecm_form_matrices(rset);
-    } else {
+    } else if (rset->g > 0) {
+	/* mutivariate system */
 	err = sys_form_matrices(rset);
     }
 
@@ -721,9 +722,23 @@ bnum_from_name (gretl_restriction *r, const DATASET *dset,
 {
     int k = -1;
 
-    if (dset == NULL || r->otype != GRETL_OBJ_EQN || r->obj == NULL) {
+    if (dset == NULL || r->obj == NULL) {
 	gretl_errmsg_set(_("Please give a coefficient number"));
-    } else {
+    } else if (r->otype == GRETL_OBJ_VAR) {
+	GRETL_VAR *var = r->obj;
+	int i, v;
+
+	for (i=1; i<=var->ylist[0]; i++) {
+	    v = var->ylist[i];
+	    if (v > 0 && v < dset->v && !strcmp(dset->varname[v], s)) {
+		k = i;
+		break;
+	    }
+	}
+	if (k < 0) {
+	    gretl_errmsg_set(_("Please give a coefficient number"));
+	}
+    } else if (r->otype == GRETL_OBJ_EQN) {
 	const MODEL *pmod = r->obj;
 
 	k = gretl_model_get_param_number(pmod, dset, s);
@@ -1646,12 +1661,27 @@ static int parse_restriction_row (gretl_restriction *rset,
     }
 
     if (!err) {
+	char rhstr[32];
 	double rhs = 0.0;
 
-	if (!sscanf(s, " = %lf", &rhs)) {
+	s += strspn(s, " ");
+
+	if (*s != '=') {
 	    err = E_PARSE;
 	} else {
-	    row->rhs += rhs;
+	    s++;
+	    if (!sscanf(s, "%31s", rhstr)) {
+		err = E_PARSE;
+	    } else if (sscanf(s, "%lf", &rhs) == 1) {
+		row->rhs += rhs;
+	    } else if (gretl_is_scalar(rhstr)) {
+		rhs = gretl_scalar_get_value(rhstr);
+		if (na(rhs)) {
+		    err = E_DATA;
+		} else {
+		    row->rhs += rhs;
+		}
+	    }
 	}
     }
 
@@ -1816,8 +1846,8 @@ restriction_set_start (const char *line, gretlopt opt, int *err)
     fprintf(stderr, "restriction_set_start: line='%s'\n", line);
 #endif
 
-    if (!strncmp(line, "restrict", 8)) {
-	name = get_system_name_from_line(line, SYSNAME_RST);
+    if (!strncmp(line, "restrict ", 9)) {
+	name = get_system_name_from_line(line + 9);
     }
 
     if (name != NULL) {
@@ -1827,8 +1857,8 @@ restriction_set_start (const char *line, gretlopt opt, int *err)
 	    gretl_errmsg_sprintf("'%s': unrecognized name", name);
 	}
     } else {
-	/* get pointer to last-created object */
-	ptr = get_last_model(&type);  
+	/* get pointer to last-estimated model */
+	ptr = get_last_model(&type);
     }
 
     if (ptr == NULL) {
@@ -2010,7 +2040,7 @@ static int save_restricted_model (ExecState *state,
 	for (i=0; i<rmod->ncoeff; i++) {
 	    rmod->coeff[i] = b->val[i];
 	}
-	err = gretl_model_write_vcv(rmod, S, -1);
+	err = gretl_model_write_vcv(rmod, S);
     }
 
     if (!err) {
