@@ -40,7 +40,7 @@ static gretl_scalar **scalars;
 static int n_scalars;
 static int scalar_imin;
 
-static void (*scalar_edit_callback)(const char *name, double val);
+static void (*scalar_edit_callback)(void);
 
 #if SDEBUG
 
@@ -211,6 +211,15 @@ const char *gretl_scalar_get_name (int i)
     }
 }
 
+int gretl_scalar_get_level (int i)
+{
+    if (i >= 0 && i < n_scalars) {
+	return scalars[i]->level;
+    } else {
+	return -1;
+    }
+}
+
 double gretl_scalar_get_value_by_index (int i)
 {
     if (i >= 0 && i < n_scalars) {
@@ -250,6 +259,10 @@ void gretl_scalar_set_value (const char *name, double val)
 	s->val = val;
     }
 
+    if (scalar_edit_callback != NULL) {
+	scalar_edit_callback();
+    }
+
 #if SDEBUG
     debug_print_scalars("gretl_scalar_set_value");
 #endif
@@ -258,11 +271,11 @@ void gretl_scalar_set_value (const char *name, double val)
 /* check that we're not colliding with a pre-existing object of
    different type */
 
-static int gretl_scalar_check_name (const char *s, const DATAINFO *pdinfo)
+static int gretl_scalar_check_name (const char *s, const DATASET *dset)
 {
     int v, err = 0;
 
-    if ((v = series_index(pdinfo, s)) < pdinfo->v) {
+    if ((v = series_index(dset, s)) < dset->v) {
 	err = E_TYPES;
     } else if (get_matrix_by_name(s)) {
 	err = E_TYPES;
@@ -309,16 +322,16 @@ int gretl_scalar_add (const char *name, double val)
 #endif
 
     if (!err && level == 0 && scalar_edit_callback != NULL) {
-	scalar_edit_callback(name, val);
+	scalar_edit_callback();
     }
 
     return err;
 }
 
 int gretl_scalar_add_with_check (const char *name, double val,
-				 const DATAINFO *pdinfo)
+				 const DATASET *dset)
 {
-    int err = gretl_scalar_check_name(name, pdinfo);
+    int err = gretl_scalar_check_name(name, dset);
 
     if (!err) {
 	err = gretl_scalar_add(name, val);
@@ -387,9 +400,14 @@ int gretl_scalar_delete (const char *name, PRN *prn)
 	}
     }
 
-    if (!err && prn != NULL && gretl_messages_on()) {
-	pprintf(prn, _("Deleted scalar %s"), name);
-	pputc(prn, '\n');
+    if (!err) {
+	if (prn != NULL && gretl_messages_on()) {
+	    pprintf(prn, _("Deleted scalar %s"), name);
+	    pputc(prn, '\n');
+	}
+	if (level == 0 && scalar_edit_callback != NULL) {
+	    scalar_edit_callback();
+	}
     }
 
 #if SDEBUG
