@@ -39,8 +39,6 @@ struct _gretl_string_table {
     char *extra;      /* extra information */
 };
 
-static void clean_up_codevars (void);
-
 static col_table *col_table_new (int colnum)
 {
     col_table *ct = malloc(sizeof *ct);
@@ -222,8 +220,8 @@ gretl_string_table_index (gretl_string_table *st, const char *s, int col,
 	/* no table for this column yet: start one now */
 	ct = gretl_string_table_add_column(st, col);
 	if (ct != NULL) {
-	    pprintf(prn, M_("variable %d: translating from strings to "
-			    "code numbers\n"), col);
+	    pprintf(prn, _("variable %d: translating from strings to "
+			   "code numbers\n"), col);
 	}
     }
 
@@ -343,18 +341,18 @@ int gretl_string_table_print (gretl_string_table *st, DATASET *dset,
 
     if (st->n_cols > 0) {
 	fputc('\n', fp);
-	fputs(M_("One or more non-numeric variables were found.\n"
-		 "Gretl cannot handle such variables directly, so they\n"
-		 "have been given numeric codes as follows.\n\n"), fp);
+	fputs(_("One or more non-numeric variables were found.\n"
+		"Gretl cannot handle such variables directly, so they\n"
+		"have been given numeric codes as follows.\n\n"), fp);
 	if (st->extra != NULL) {
-	    fputs(M_("In addition, some mappings from numerical values to string\n"
-		     "labels were found, and are printed below.\n\n"), fp);
+	    fputs(_("In addition, some mappings from numerical values to string\n"
+		    "labels were found, and are printed below.\n\n"), fp);
 	}
     }
     
     for (i=0; i<st->n_cols; i++) {
 	ct = st->cols[i];
-	fprintf(fp, M_("String code table for variable %d (%s):\n"), 
+	fprintf(fp, _("String code table for variable %d (%s):\n"), 
 		ct->idx, dset->varname[ct->idx]);
 	for (j=0; j<ct->n_strs; j++) {
 	    fprintf(fp, "%3d = '%s'\n", j+1, ct->strs[j]);
@@ -365,7 +363,7 @@ int gretl_string_table_print (gretl_string_table *st, DATASET *dset,
 	fputs(st->extra, fp);
     }
 
-    pprintf(prn, M_("String code table written to\n %s\n"), stname);
+    pprintf(prn, _("String code table written to\n %s\n"), stname);
 
     fclose(fp);
     set_string_table_written();
@@ -508,12 +506,14 @@ static saved_string *get_saved_string_by_name (const char *name,
  * if there is no such variable.
  */
 
-char *get_string_by_name (const char *name)
+const char *get_string_by_name (const char *name)
 {
     int i, n, d;
 
     if (!strcmp(name, "dirsep")) {
 	return dsep.s;
+    } else if (!strcmp(name, "vcvtype")) {
+	return last_model_get_vcv_type();
     }
 
     n = sizeof built_ins / sizeof built_ins[0];
@@ -674,8 +674,6 @@ void saved_strings_cleanup (void)
     n_saved_strings = 0;
 
     gretl_free_builtin_strings();
-
-    clean_up_codevars();
 }
 
 void destroy_user_strings (void)
@@ -689,8 +687,6 @@ void destroy_user_strings (void)
     free(saved_strings);
     saved_strings = NULL;
     n_saved_strings = 0;
-
-    clean_up_codevars();
 }
 
 /* called on exiting a user-defined function to clean
@@ -1144,60 +1140,3 @@ int substitute_named_strings (char *line, int *subst)
 
     return err;
 }
-
-/* setting of names of imported variables that should be treated
-   as string codes */
-
-static char **codevars;
-static int n_codevars;
-
-static void clean_up_codevars (void)
-{
-    if (codevars != NULL && n_codevars > 0) {
-	free_strings_array(codevars, n_codevars);
-	codevars = NULL;
-	n_codevars = 0;
-    }
-}
-
-int is_codevar (const char *s)
-{
-    int i;
-
-    for (i=0; i<n_codevars; i++) {
-	if (!strcmp(s, codevars[i])) {
-	    return 1;
-	}
-    }
-
-    return 0;
-}
-
-int set_codevars (const char *s)
-{
-    char chunk[32];
-    const char *p;
-    int err = 0;
-
-    p = strstr(s, "codevars");
-    if (p != NULL) {
-	s = p + 9;
-    }
-
-    *chunk = '\0';
-    sscanf(s, "%31s", chunk);
-
-    if (*chunk == '\0') {
-	err = E_DATA;
-    } else {
-	clean_up_codevars();
-	if (strcmp(chunk, "null")) {
-	    codevars = gretl_string_split(s, &n_codevars);
-	    if (codevars == NULL) {
-		err = E_ALLOC;
-	    } 
-	}
-    }
-
-    return err;
-}
