@@ -45,7 +45,7 @@ int save_named_string (const char *name, const char *s, PRN *prn);
 
 int add_string_as (const char *s, const char *name);
 
-char *get_string_by_name (const char *name);
+const char *get_string_by_name (const char *name);
 
 int process_string_command (const char *line, 
 			    double ***pZ, DATASET *dset,
@@ -65,10 +65,6 @@ void destroy_user_strings (void);
 
 int destroy_saved_strings_at_level (int d);
 
-int is_codevar (const char *s);
-
-int set_codevars (const char *s);
-
 char *gretl_getenv (const char *key, int *defined, int *err);
 
 char *retrieve_date_string (int t, const DATASET *dset, int *err);
