@@ -47,20 +47,20 @@ static void gretl_tests_cleanup (void);
 
 /**
  * date:
- * @nt: observation number (zero-based).
+ * @t: observation number (zero-based).
  * @pd: data periodicity or frequency.
  * @sd0: floating point representation of starting date.
  *
- * Returns: the date corresponding to @nt, as a double-precision number.
+ * Returns: the date corresponding to @t, as a double-precision number.
  */
 
-double date (int nt, int pd, const double sd0)
+double date (int t, int pd, double sd0)
 {
     int ysd = (int) sd0, yy, pp, yp;
     int p10 = 10;
 
     if (pd == 1) {
-	return (double) (ysd + nt);  
+	return (double) (ysd + t);  
     } 
 
     pp = pd;
@@ -68,12 +68,12 @@ double date (int nt, int pd, const double sd0)
 	p10 *= 10;
     }
 
-    pp = nt % pd + p10 * (sd0 - ysd) + .5;
+    pp = t % pd + p10 * (sd0 - ysd) + .5;
     if (pp != pd)  {
-        yy = ysd + nt/pd  + pp/pd + .5;
+        yy = ysd + t/pd  + pp/pd + .5;
         yp = pp % pd;
     }  else {
-        yy = ysd + nt/pd + .5;
+        yy = ysd + t/pd + .5;
         yp = pp;
     }
 
@@ -386,7 +386,7 @@ static int few_vals (int t1, int t2, const double *x, double *ratio)
 int gretl_isdiscrete (int t1, int t2, const double *x)
 {
     int t, n = 0, d = 1;
-    double r;
+    double r = 0;
 
     for (t=t1; t<=t2; t++) {
 	if (na(x[t])) {
@@ -427,6 +427,39 @@ int gretl_isdiscrete (int t1, int t2, const double *x)
     return d;
 }
 
+ /**
+ * gretl_ispositive:
+ * @x: data series to examine.
+ * @t1: starting observation.
+ * @t2: ending observation. 
+ * @strict: boolean, strict inequality. 
+ *
+ * Returns: 1 if all non-missing values in @x, over the range @t1 to
+ * @t2, are positive (if strict==1) or non-negative (if strict==0),
+ * otherwise 0.
+ */
+
+int gretl_ispositive (int t1, int t2, const double *x, int strict)
+{
+    int t;
+
+    if (strict) {
+	for (t=t1; t<=t2; t++) {
+	    if (x[t] <= 0) {
+		return 0;
+	    }
+	}
+    } else {
+	for (t=t1; t<=t2; t++) {
+	    if (x[t] < 0) {
+		return 0;
+	    }
+	}
+    }
+
+    return 1;
+}
+
  /**
  * gretl_is_oprobit_ok:
  * @x: data series to examine.
@@ -1695,6 +1728,14 @@ double gretl_stopwatch (void)
 
 /* library init and cleanup functions */
 
+/**
+ * libgretl_init:
+ *
+ * In a program that uses libgretl, this function should be 
+ * called once, before any other libgretl functions are
+ * used. See also libgretl_cleanup().
+ **/
+
 void libgretl_init (void)
 {
     libset_init();
@@ -1732,6 +1773,17 @@ void libgretl_session_cleanup (int mode)
     }
 }
 
+/**
+ * libgretl_cleanup:
+ *
+ * In a program that uses libgretl, this function may be 
+ * called to free various chunks of memory after the program 
+ * is finished with libgretl. Do not attempt to call any
+ * other libgretl functions after invoking this cleanup.
+ *
+ * See also libgretl_init().
+ **/
+
 void libgretl_cleanup (void)
 {
     libgretl_session_cleanup(SESSION_CLEAR_ALL);
@@ -1747,6 +1799,7 @@ void libgretl_cleanup (void)
     option_flags_cleanup();
     kalman_cleanup();
     gnuplot_cleanup();
+    bufgets_cleanup();
 
 #ifdef USE_RLIB
     gretl_R_cleanup();
