@@ -29,6 +29,7 @@
 #ifndef STANDALONE
 # include "libgretl.h"
 # include "libset.h"
+# include "build.h"
 #endif
 
 #include "version.h"
@@ -55,13 +56,18 @@
 # include <netdb.h>
 # include <netinet/in.h>
 # include <arpa/inet.h>
-#endif /* WIN32 */
+#endif /* WIN32 or not */
 
 #define DBHLEN 64
 
 static int wproxy;
+#if 0 /* local testing */
+static char dbhost[DBHLEN]       = "dev.ricardo.ecn.wfu.edu";
+static char gretlhost[DBHLEN]    = "dev.ricardo.ecn.wfu.edu";
+#else
 static char dbhost[DBHLEN]       = "ricardo.ecn.wfu.edu";
 static char gretlhost[DBHLEN]    = "ricardo.ecn.wfu.edu";
+#endif
 static char datacgi[DBHLEN]      = "/gretl/cgi-bin/gretldata.cgi";
 static char updatecgi[DBHLEN]    = "/gretl/cgi-bin/gretl_update.cgi";
 static char manual_path[DBHLEN]  = "/project/gretl/manual/";
@@ -923,7 +929,8 @@ static uerr_t real_get_http (urlinfo *u, struct http_stat *hs, int *dt)
     err = make_connection(&sock, conn->host, conn->port);
 
 #if WDEBUG
-    fprintf(stderr, "make_connection returned %d\n", err);
+    fprintf(stderr, "%s, port %d: make_connection returned %d\n", 
+	    conn->host, conn->port, err);
 #endif   
 
     switch (err) {
@@ -1390,12 +1397,19 @@ static void url_init (urlinfo *u)
     u->upsize = 0;
     u->fp = NULL;
 
+#ifdef BUILD_DATE
+    sprintf(u->agent, "gretl-%s-%s", GRETL_VERSION, BUILD_DATE);
+#else
     sprintf(u->agent, "gretl-%s", GRETL_VERSION);
+#endif
 
 #if defined(STANDALONE) || defined (WIN32)
     /* the linux test updater program pretends to be Windows */
     strcat(u->agent, "w");
 #endif
+#if 0
+    fprintf(stderr, "url_init: agent = '%s'\n", u->agent);
+#endif
 }
 
 static urlinfo *urlinfo_new (void)
