@@ -166,7 +166,7 @@ static char *simple_fname (char *dest, const char *src)
     return dest;
 }
 
-static int alt_puts (const char *s, FILE *fp, gzFile *fz)
+static int alt_puts (const char *s, FILE *fp, gzFile fz)
 {
     int ret = 0;
 
@@ -447,14 +447,22 @@ void gretl_xml_put_matrix (const gretl_matrix *m, const char *name,
     }
 
     if (name == NULL) {
-	fprintf(fp, "<gretl-matrix rows=\"%d\" cols=\"%d\">\n", 
+	fprintf(fp, "<gretl-matrix rows=\"%d\" cols=\"%d\"\n", 
 		m->rows, m->cols);
     } else {
-	fprintf(fp, "<gretl-matrix name=\"%s\" rows=\"%d\" cols=\"%d\" "
-		"t1=\"%d\" t2=\"%d\">\n", 
-		name, m->rows, m->cols, m->t1, m->t2);
+	fprintf(fp, "<gretl-matrix name=\"%s\" rows=\"%d\" cols=\"%d\"",
+		name, m->rows, m->cols);
     }
 
+    if (gretl_matrix_is_dated(m)) {
+	int mt1 = gretl_matrix_get_t1(m);
+	int mt2 = gretl_matrix_get_t2(m);
+
+	fprintf(fp, " t1=\"%d\" t2=\"%d\"", mt1, mt2);
+    }
+
+    fputs(">\n", fp);
+
     for (i=0; i<m->rows; i++) {
 	for (j=0; j<m->cols; j++) {
 	    fprintf(fp, "%.15g ", gretl_matrix_get(m, i, j));
@@ -1330,8 +1338,8 @@ static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc,
 	gretl_matrix_free(m);
 	m = NULL;
     } else {
-	m->t1 = t1;
-	m->t2 = t2;
+	gretl_matrix_set_t1(m, t1);
+	gretl_matrix_set_t2(m, t2);
     }
 
     return m;
@@ -1431,7 +1439,7 @@ int gretl_write_matrix_as_gdt (const char *fname,
 			       const char **varnames, 
 			       const char **labels)
 {
-    gzFile *fz = Z_NULL;
+    gzFile fz = Z_NULL;
     char datname[MAXLEN];
     void *handle = NULL;
     char *xmlbuf = NULL;
@@ -1548,7 +1556,7 @@ int gretl_write_gdt (const char *fname, const int *list,
 		     int progress)
 {
     FILE *fp = NULL;
-    gzFile *fz = Z_NULL;
+    gzFile fz = Z_NULL;
     int gz = (opt & OPT_Z);
     int tsamp = dset->t2 - dset->t1 + 1;
     int *pmax = NULL;
@@ -2352,8 +2360,8 @@ static void record_transform_info (DATASET *dset, double version)
 
 static void data_read_message (const char *fname, DATASET *dset, PRN *prn)
 {
-    pprintf(prn, M_("\nRead datafile %s\n"), fname);
-    pprintf(prn, M_("periodicity: %d, maxobs: %d\n"
+    pprintf(prn, A_("\nRead datafile %s\n"), fname);
+    pprintf(prn, A_("periodicity: %d, maxobs: %d\n"
 		    "observations range: %s-%s\n"), 
 	    (custom_time_series(dset))? 1 : dset->pd, 
 	    dset->n, dset->stobs, dset->endobs);
@@ -2432,7 +2440,7 @@ int gretl_read_gdt (const char *fname, DATASET *dset,
 	}
     }
 
-    check_for_console(prn);
+    set_alt_gettext_mode(prn);
 
     tmpset = datainfo_new();
     if (tmpset == NULL) {
@@ -2559,8 +2567,6 @@ int gretl_read_gdt (const char *fname, DATASET *dset,
 	destroy_dataset(tmpset);
     }
 
-    console_off();
-
 #if GDT_DEBUG
     fprintf(stderr, "gretl_read_gdt: returning %d\n", err);
 #endif
