@@ -334,13 +334,9 @@ MODEL logit_probit (int *list, DATASET *dset, int ci,
 */
 
 MODEL logistic_driver (const int *list, DATASET *dset,
-		       const char *param) 
+		       gretlopt opt) 
 {
-    double lmax;
-
-    if (param == NULL || sscanf(param, "ymax=%lf", &lmax) != 1) {
-	lmax = NADBL;
-    }
+    double lmax = get_optval_double(LOGISTIC, OPT_M);
 
     return logistic_model(list, lmax, dset);
 }
@@ -462,7 +458,7 @@ int do_modprint (const char *line, gretlopt opt, PRN *prn)
 		s += strspn(s, " ");
 	    }
 	} else if (sscanf(s, "%15s", name) == 1) {
-	    parnames = get_string_by_name(name);
+	    parnames = (char *) get_string_by_name(name);
 	    if (parnames == NULL) {
 		err = E_UNKVAR;
 	    } else {
@@ -521,3 +517,47 @@ int script_add_obs_markers (DATASET *dset)
 
     return err;
 }
+
+int matrix_plot_driver (const int *list, const char *literal,
+			gretlopt opt)
+{
+    gretl_matrix *m = NULL;
+    const char *mname;
+    int err = 0;
+
+    mname = get_optval_string(GNUPLOT, OPT_X);
+
+    if (mname != NULL) {
+	m = get_matrix_by_name(mname);
+    }
+
+    if (m == NULL) {
+	err = E_DATA;
+    } else {
+	err = matrix_plot(m, list, literal, opt);
+    }
+
+    return err;
+}
+
+int matrix_scatters_driver (const int *list, const DATASET *dset,
+			    gretlopt opt)
+{
+    gretl_matrix *m = NULL;
+    const char *mname;
+    int err = 0;
+
+    mname = get_optval_string(SCATTERS, OPT_X);
+
+    if (mname != NULL) {
+	m = get_matrix_by_name(mname);
+    }
+
+    if (m == NULL) {
+	err = E_DATA;
+    } else {
+	err = matrix_scatters(m, list, dset, opt);
+    }
+
+    return err;
+}
