@@ -36,11 +36,17 @@ MODEL logit_probit (int *list, DATASET *dset,
 		    int ci, gretlopt opt, PRN *prn);
 
 MODEL logistic_driver (const int *list, DATASET *dset,
-		       const char *param); 
+		       gretlopt opt); 
 
 MODEL tobit_driver (const int *list, DATASET *dset, 
 		    gretlopt opt, PRN *prn);
 
+int matrix_plot_driver (const int *list, const char *literal,
+			gretlopt opt);
+
+int matrix_scatters_driver (const int *list, const DATASET *dset,
+			    gretlopt opt);
+
 int do_modprint (const char *line, gretlopt opt, PRN *prn);
 
 int script_add_obs_markers (DATASET *dset);
