@@ -61,12 +61,12 @@
 # define _(String) gettext (String)
 # define N_(String) gettext_noop (String)
 # define I_(String) iso_gettext (String) 
-# define M_(String) maybe_iso_gettext (String)
+# define A_(String) alt_gettext (String)
 #else
 # define _(String)  ((char *) String)
 # define N_(String) String
 # define I_(String) ((char *) String)
-# define M_(String) ((char *) String)
+# define A_(String) ((char *) String)
 #endif /* ENABLE_NLS */
 
 #define MAXLINE 16384  /* maximum length of command line */
@@ -103,7 +103,6 @@ typedef enum {
     GRETL_TYPE_INT_ARRAY,
     GRETL_TYPE_DOUBLE_ARRAY,
     GRETL_TYPE_STRING,
-    GRETL_TYPE_CHAR_ARRAY,
     GRETL_TYPE_CMPLX_ARRAY,
     GRETL_TYPE_SERIES,
     GRETL_TYPE_MATRIX,
@@ -277,12 +276,12 @@ typedef struct DATASET_ {
     char **varname;     /* array of names of variables */
     VARINFO **varinfo;  /* array of specific info on vars */
     char markers;       /* NO_MARKERS (0), REGULAR MARKERS or DAILY_DATE_STRINGS */
-    char delim;         /* default delimiter for "CSV" files */
-    char decpoint;      /* character used to represent decimal point */
     char **S;           /* to hold observation markers */
     char *descrip;      /* to hold info on data sources etc. */
     char *submask;      /* subsampling mask */
     char *restriction;  /* record of sub-sampling restriction */
+    char *padmask;      /* record of padding to re-balance panel data */
+    int auxiliary;      /* = 0 for regular dataset, 1 for aux dataset */
 } DATASET;
 
 typedef struct VMatrix_ {
