@@ -85,7 +85,6 @@ struct set_vars_ {
     int bootrep;                /* bootstrap replications */
     double nls_toler;           /* NLS convergence criterion */
     int loop_maxiter;           /* max no. of iterations in non-for loops */
-    char delim;                 /* delimiter for CSV data export */
     int vecm_norm;              /* VECM beta normalization */
     int optim;                  /* code for preferred optimizer */
     int bfgs_maxiter;           /* max iterations, BFGS */         
@@ -164,6 +163,8 @@ static int user_mp_bits;
 static int R_functions;
 static int R_lib = 1;
 static int csv_digits;
+static char data_delim = ',';
+static char data_export_decpoint = '.';
 
 static int boolvar_get_flag (const char *s);
 static const char *hac_lag_string (void);
@@ -292,9 +293,9 @@ static void coded_var_show_opts (const char *s, PRN *prn)
 
 static const char *get_arma_vcv_str (int v)
 {
-    if (v == VCV_HESSIAN) {
+    if (v == ML_HESSIAN) {
 	return arma_vcv_strs[0];
-    } else if (v == VCV_OP) {
+    } else if (v == ML_OP) {
 	return arma_vcv_strs[1];
     } else {
 	return "unknown";
@@ -380,7 +381,6 @@ static void state_vars_copy (set_vars *sv)
     sv->rq_maxiter = state->rq_maxiter;
     sv->gmm_maxiter = state->gmm_maxiter;
     sv->nls_toler = state->nls_toler;
-    sv->delim = state->delim; 
     sv->vecm_norm = state->vecm_norm;
     sv->optim = state->optim;
     sv->bfgs_maxiter = state->bfgs_maxiter;
@@ -391,6 +391,7 @@ static void state_vars_copy (set_vars *sv)
     sv->bhhh_toler = state->bhhh_toler;
     sv->lbfgs_mem = state->lbfgs_mem;
     sv->garch_vcv = state->garch_vcv;
+    sv->arma_vcv = state->arma_vcv;
     sv->garch_robust_vcv = state->garch_robust_vcv;
     sv->nadarwat_trim = state->nadarwat_trim;
 
@@ -416,7 +417,6 @@ static void state_vars_init (set_vars *sv)
     sv->loop_maxiter = 250;
     sv->rq_maxiter = 1000;
     sv->gmm_maxiter = 250;
-    sv->delim = UNSET_INT;
     sv->vecm_norm = NORM_PHILLIPS;
     sv->optim = OPTIM_AUTO;
     sv->initvals = NULL;
@@ -429,9 +429,9 @@ static void state_vars_init (set_vars *sv)
     sv->bhhh_maxiter = 500;
     sv->bhhh_toler = NADBL;
     sv->lbfgs_mem = 8;
-    sv->garch_vcv = VCV_UNSET;
-    sv->arma_vcv = VCV_HESSIAN;
-    sv->garch_robust_vcv = VCV_UNSET;
+    sv->garch_vcv = ML_UNSET;
+    sv->arma_vcv = ML_HESSIAN;
+    sv->garch_robust_vcv = ML_UNSET;
     sv->nadarwat_trim = 4.0;
 
     *sv->shelldir = '\0';
@@ -535,16 +535,6 @@ int get_mp_bits (void)
     }
 }
 
-char get_csv_delim (const DATASET *dset)
-{
-    check_for_state();
-    if (state->delim > 0) {
-	return state->delim;
-    } else {
-	return dset->delim;
-    }
-}
-
 const gretl_matrix *get_init_vals (void)
 {
     check_for_state();
@@ -805,7 +795,7 @@ static int parse_libset_int_code (const char *key,
     } else if (!g_ascii_strcasecmp(key, HAC_LAG)) {
 	err = parse_hac_lag_variant(val);
     } else if (!g_ascii_strcasecmp(key, GARCH_VCV)) {
-	for (i=0; i<VCV_MAX; i++) {
+	for (i=0; i<ML_VCVMAX; i++) {
 	    if (!g_ascii_strcasecmp(val, garch_vcv_strs[i])) {
 		state->garch_vcv = i;
 		err = 0;
@@ -814,10 +804,10 @@ static int parse_libset_int_code (const char *key,
 	}
     } else if (!g_ascii_strcasecmp(key, ARMA_VCV)) {
 	if (!g_ascii_strcasecmp(val, "op")) {
-	    state->arma_vcv = VCV_OP;
+	    state->arma_vcv = ML_OP;
 	    err = 0;
 	} else if (!g_ascii_strcasecmp(val, "hessian")) {
-	    state->arma_vcv = VCV_HESSIAN;
+	    state->arma_vcv = ML_HESSIAN;
 	    err = 0;
 	}
     } else if (!g_ascii_strcasecmp(key, HAC_KERNEL)) {
@@ -925,9 +915,9 @@ void set_garch_robust_vcv (const char *s)
     if (scpy != NULL) {
 	lower(scpy);
 	if (!strcmp(s, "qml")) {
-	    state->garch_robust_vcv = VCV_QML;
+	    state->garch_robust_vcv = ML_QML;
 	} else if (!strcmp(s, "bw")) {
-	    state->garch_robust_vcv = VCV_BW;
+	    state->garch_robust_vcv = ML_BW;
 	}
 	free(scpy);
     }
@@ -1293,13 +1283,13 @@ static int print_settings (PRN *prn, gretlopt opt)
     libset_header(N_("Program interaction and behavior"), prn, opt);
 
     if (opt & OPT_D) {
-	pprintf(prn, " csv_delim = %s\n", arg_from_delim(state->delim));
+	pprintf(prn, " csv_delim = %s\n", arg_from_delim(data_delim));
 	pprintf(prn, " csv_na = %s\n", get_csv_na_string());
     } else {
-	const char *dl = arg_from_delim(state->delim);
+	const char *dl = arg_from_delim(data_delim);
 
 	if (strcmp(dl, "unset")) {
-	    pprintf(prn, "set csv_delim %s\n", arg_from_delim(state->delim));
+	    pprintf(prn, "set csv_delim %s\n", arg_from_delim(data_delim));
 	}
 	pprintf(prn, "set csv_na %s\n", get_csv_na_string());
     }	
@@ -1432,7 +1422,7 @@ static int libset_query_settings (const char *s, PRN *prn)
 		s, state->seed ? state->seed : gretl_rand_get_seed());
     } else if (!strcmp(s, "csv_delim")) {
 	pprintf(prn, "%s: named character, currently \"%s\"\n", s,
-		arg_from_delim(state->delim));
+		arg_from_delim(data_delim));
 	coded_var_show_opts(s, prn);
     } else if (!strcmp(s, "shelldir")) {
 	pprintf(prn, "%s: string, currently \"%s\"\n", s,
@@ -1518,8 +1508,6 @@ int execute_set_line (const char *line, DATASET *dset,
 	    return set_matmask(line, dset, prn);
 	} else if (!strcmp(setobj, "shelldir")) {
 	    return set_shelldir(line);
-	} else if (!strcmp(setobj, "codevars")) {
-	    return set_codevars(line);
 	} else if (!strcmp(setobj, "workdir")) {
 	    return set_workdir(line);
 	} 
@@ -1569,7 +1557,7 @@ int execute_set_line (const char *line, DATASET *dset,
 	    char c = delim_from_arg(setarg);
 
 	    if (c > 0) {
-		state->delim = c;
+		data_delim = c;
 		err = 0;
 	    }
 	} else if (!strcmp(setobj, "seed")) {
@@ -1979,6 +1967,34 @@ static void libset_set_decpoint (int on)
 #endif
 }
 
+void set_data_export_decimal_comma (int s)
+{
+    if (s) {
+	data_export_decpoint = ',';
+    } else {
+	data_export_decpoint = '.';
+    }
+}
+
+char get_data_export_decpoint (void)
+{
+    char c = data_export_decpoint;
+
+    /* revert to '.' on access */
+    data_export_decpoint = '.';
+    return c;
+}
+
+void set_data_export_delimiter (char c)
+{
+    data_delim = c;
+}
+
+char get_data_export_delimiter (void)
+{
+    return data_delim;
+}
+
 static int check_R_setting (int *var, int val, const char *key)
 {
     int err = 0;
