@@ -520,9 +520,9 @@ real_gretl_matrix_data_subset (const int *list,
 		gretl_matrix_set(M, s, j, x);
 	    }
 	    if (s == 0) {
-		M->t1 = t;
+		gretl_matrix_set_t1(M, t);
 	    } else if (s == T - 1) {
-		M->t2 = t;
+		gretl_matrix_set_t2(M, t);
 		break;
 	    }
 	    s++;
@@ -549,8 +549,8 @@ real_gretl_matrix_data_subset (const int *list,
 	gretl_matrix_free(M);
 	M = NULL;
     } else if (T == Tmax) {
-	M->t1 = t1;
-	M->t2 = t2;
+	gretl_matrix_set_t1(M, t1);
+	gretl_matrix_set_t2(M, t2);
     }
 
     return M;
@@ -688,9 +688,9 @@ gretl_matrix_data_subset_special (const int *list,
 			    xti = M_NA;
 			}
 			if (s == 0) {
-			    X->t1 = t;
+			    gretl_matrix_set_t1(X, t);
 			} else if (s == T - 1) {
-			    X->t2 = t;
+			    gretl_matrix_set_t2(X, t);
 			}
 			gretl_matrix_set(X, s++, i, xti);
 		    }
@@ -760,7 +760,7 @@ DATASET *gretl_dataset_from_matrix (const gretl_matrix *m,
 	return NULL;
     }
 
-    names = user_matrix_get_names(m, 0);
+    names = gretl_matrix_get_colnames(m);
 
     for (i=1; i<=nv; i++) {
 	col = (list != NULL)? list[i] - 1 : i - 1;
@@ -813,6 +813,11 @@ int gretl_plotfit_matrices (const double *yvar, const double *xvar,
 	return E_DATA;
     }
 
+    if (fit == PLOT_FIT_LOGLIN && !gretl_ispositive(t1, t2, yvar, 1)) {
+	gretl_errmsg_set(_("Non-positive values encountered"));
+	return E_DATA;
+    }
+
     mask = calloc(T, 1);
     if (mask == NULL) {
 	return E_ALLOC;
@@ -854,7 +859,11 @@ int gretl_plotfit_matrices (const double *yvar, const double *xvar,
 	t = s + t1;
 	if (!mask[s]) {
 	    j = 0;
-	    y->val[i] = yvar[t];
+	    if (fit == PLOT_FIT_LOGLIN) {
+		y->val[i] = log(yvar[t]);
+	    } else {
+		y->val[i] = yvar[t];
+	    }
 	    if (fit != PLOT_FIT_LOESS) {
 		gretl_matrix_set(X, i, j++, 1.0);
 	    }
@@ -1097,21 +1106,22 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 
     if (msg != NULL && *msg != '\0' && !plain) {
 	pprintf(prn, "%s (%d x %d)", msg, m->rows, m->cols);
-	if (m->t1 == BLOCKT && m->t2 == BLOCKT) {
-	    pprintf(prn, " (part of matrix block)\n\n");
-	} else if (m->t1 != 0 || m->t2 != 0) {
-	    pprintf(prn, " [t1 = %d, t2 = %d]\n\n", m->t1 + 1, m->t2 + 1);
+	if (gretl_matrix_is_dated(m)) {
+	    int mt1 = gretl_matrix_get_t1(m);
+	    int mt2 = gretl_matrix_get_t2(m);
+
+	    pprintf(prn, " [t1 = %d, t2 = %d]\n\n", mt1 + 1, mt2 + 1);
 	} else {
 	    pputs(prn, "\n\n");
 	}
     }
 
     if (heads == NULL) {
-	heads = user_matrix_get_names(m, 0);
+	heads = gretl_matrix_get_colnames(m);
     }
 
     if (!packed) {
-	rownames = user_matrix_get_names(m, 1);
+	rownames = gretl_matrix_get_rownames(m);
 	if (rownames != NULL) {
 	    llen = max_label_length(rownames, m->rows);
 	}
@@ -1219,7 +1229,7 @@ static void maybe_print_col_heads (const gretl_matrix *m,
 {
     const char **heads;
 
-    heads = user_matrix_get_names(m, 0);
+    heads = gretl_matrix_get_colnames(m);
     
     if (heads != NULL) {
 	char wtest[32];
@@ -1300,7 +1310,7 @@ void gretl_matrix_print_with_format (const gretl_matrix *m,
 	    intcast = 1;
 	}
 
-	rownames = user_matrix_get_names(m, 1);
+	rownames = gretl_matrix_get_rownames(m);
 	if (rownames != NULL) {
 	    llen = max_label_length(rownames, m->rows);
 	}
