@@ -30,6 +30,7 @@
 #include "gretl_scalar.h"
 #include "flow_control.h"
 #include "system.h"
+#include "genparse.h"
 
 #include <time.h>
 #include <unistd.h>
@@ -362,12 +363,10 @@ int ok_in_loop (int c)
 	c == OPEN ||
 	c == RMPLOT ||
 	c == RUN ||
-	c == SCATTERS ||
 	c == SETMISS ||
 	c == SETOBS ||
 	c == TABPRINT ||
-	c == VIF ||
-	c == XCORRGM) {
+	c == VIF)  {
 	return 0;
     }
 
@@ -2120,7 +2119,7 @@ static void loop_model_print (LOOP_MODEL *lmod, const DATASET *dset,
     pprintf(prn, _("%s estimates using the %d observations %s-%s\n"),
 	    _(estimator_string(lmod->model0, prn)), lmod->model0->nobs, 
 	    startdate, enddate);
-    print_model_vcv_info(lmod->model0, prn);
+    print_model_vcv_info(lmod->model0, dset, prn);
     pprintf(prn, _("Statistics for %d repetitions\n"), lmod->n); 
     pprintf(prn, _("Dependent variable: %s\n\n"), 
 	    gretl_model_get_depvar_name(lmod->model0, dset));
@@ -2203,9 +2202,7 @@ static int loop_store_save (LOOP_STORE *lstore, PRN *prn)
     }
 
     lstore->dset->t2 = lstore->n - 1;
-
     pprintf(prn, _("store: using filename %s\n"), lstore->fname);
-
     err = write_data(lstore->fname, list, lstore->dset, lstore->opt, 0);
 
     if (!err) {
@@ -2281,14 +2278,13 @@ static void print_loop_results (LOOPSET *loop, const DATASET *dset,
     }
 }
 
-static int 
-substitute_dollar_targ (char *str, int maxlen, 
-			const LOOPSET *loop,
-			const DATASET *dset,
-			int *subst)
+static int substitute_dollar_targ (char *str, int maxlen, 
+				   const LOOPSET *loop,
+				   const DATASET *dset,
+				   int *subst)
 {
     char insert[32], targ[VNAMELEN + 3] = {0};
-    char *p, *ins, *q;
+    char *p, *ins, *q, *s;
     int targlen, inslen, idx = 0;
     int incr, cumlen = 0;
     int err = 0;
@@ -2359,7 +2355,12 @@ substitute_dollar_targ (char *str, int maxlen,
 
     /* crawl along str, replacing targ with ins */
 
-    while ((p = strstr(str, targ)) != NULL && !err) {
+    s = str;
+    while ((p = strstr(s, targ)) != NULL && !err) {
+	if (is_gretl_accessor(p)) {
+	    s++;
+	    continue;
+	}
 	if (incr > 0) {
 	    cumlen += incr;
 	    if (cumlen >= maxlen) {
@@ -2374,6 +2375,7 @@ substitute_dollar_targ (char *str, int maxlen,
 	if (subst != NULL) {
 	    *subst = 1;
 	}
+	s++; /* += strlen(ins)? */
     }
 
     free(q);
@@ -2467,7 +2469,7 @@ static int top_of_loop (LOOPSET *loop, DATASET *dset)
 	    gretl_scalar_set_value(loop->idxname, loop->idxval);
 	}
 
-	/* initialization in case this loop is being run more than
+	/* initialization, in case this loop is being run more than
 	   once (i.e. it's embedded in an outer loop) */
 
 	if (loop_is_progressive(loop)) {
@@ -2946,8 +2948,6 @@ int gretl_loop_exec (ExecState *s, DATASET *dset)
 		break;
 	    } else if (cmd->ci == ENDLOOP) {
 		; /* implicit break */
-	    } else if (cmd->ci == FREQ) {
-		err = freqdist(cmd->list[1], dset, 0, cmd->opt, prn);
 	    } else if (plain_model_ci(cmd->ci)) {
 		/* model may need special handling */
 		if (loop_is_progressive(loop) && !(cmd->opt & OPT_Q)) {
