@@ -35,8 +35,8 @@
 #include "gretl_bfgs.h"
 #include "tsls.h"
 
-#include "gretl_f2c.h"
-#include "../../minpack/minpack.h"  
+#include "../../minpack/minpack.h" 
+#include <float.h> 
 
 /**
  * SECTION:nls
@@ -61,19 +61,20 @@ enum {
 struct parm_ {
     char name[VNAMELEN];  /* name of parameter */
     int type;             /* type of parameter (scalar or vector) */
+    int dtype;            /* type of derivative for parameter */
     char *deriv;          /* string representation of derivative of regression
 			     function with respect to param (or NULL) */
-    int dnum;             /* ID number of variable holding the derivative */
+    int dnum;             /* ID number of series holding the derivative, or 0 */
     int nc;               /* number of individual coefficients associated
 			     with the parameter */
     char dname[VNAMELEN]; /* name of variable holding the derivative */
+    GENERATOR *dgenr;     /* generator for derivative */
     gretl_matrix *vec;    /* pointer to vector parameter */
-    gretl_matrix *dmat;   /* pointer to matrix derivative */
 };
 
 #define scalar_param(s,i) (s->params[i].type == GRETL_TYPE_DOUBLE)
-#define matrix_deriv(s,i) (s->params[i].dmat != NULL)
-#define scalar_deriv(s,i) (gretl_is_scalar(s->params[i].dname))
+#define matrix_deriv(s,i) (s->params[i].dtype == GRETL_TYPE_MATRIX)
+#define scalar_deriv(s,i) (s->params[i].dtype == GRETL_TYPE_DOUBLE)
 
 #define numeric_mode(s) (!(s->flags & NL_ANALYTICAL))
 #define analytic_mode(s) (s->flags & NL_ANALYTICAL)
@@ -81,7 +82,6 @@ struct parm_ {
 /* file-scope global variables */
 
 static nlspec private_spec;
-static integer one = 1;
 
 static char *adjust_saved_nlfunc (char *s);
 
@@ -95,15 +95,17 @@ static void set_analytic_mode (nlspec *s)
     s->flags |= NL_ANALYTICAL;
 }
 
-static void destroy_genrs_array (GENERATOR **genrs, int n)
+static void destroy_genrs_array (nlspec *s)
 {
     int i;
 
-    for (i=0; i<n; i++) {
-	destroy_genr(genrs[i]);
+    for (i=0; i<s->ngenrs; i++) {
+	destroy_genr(s->genrs[i]);
     }
 
-    free(genrs);
+    free(s->genrs);
+    s->genrs = NULL;
+    s->ngenrs = 0;
 }
 
 static int check_lhs_vec (nlspec *s)
@@ -119,6 +121,17 @@ static int check_lhs_vec (nlspec *s)
     return 0;
 }
 
+static gretl_matrix *get_derivative_matrix (nlspec *s, int i, int *err)
+{
+    gretl_matrix *m = genr_get_output_matrix(s->params[i].dgenr);
+
+    if (m == NULL) {
+	*err = E_DATA;
+    }
+
+    return m;
+}
+
 static int check_derivative_matrix (int i, gretl_matrix *m,
 				    nlspec *s)
 {
@@ -181,58 +194,68 @@ static int nls_dynamic_check (nlspec *s, char *formula)
 static int scalar_acceptable (nlspec *s, int i, const char *dername)
 {
     if (i < s->naux) {
-	/* auxiliary variable */
+	/* an auxiliary variable: scalars OK */
 	return 1;
     } else if (i == s->naux) {
-	/* criterion value */
+	/* the criterion value: accept a scalar for loglikelihood */
 	return s->ci == MLE;
     } else {
-	/* i > s->naux: derivative */
+	/* i > s->naux: a derivative */
 	return dername != NULL && gretl_is_scalar(dername);
     }
 }
 
+static int allocate_generators (nlspec *s)
+{
+    int np = analytic_mode(s) ? s->nparam : 0;
+    int i, nlf = (s->nlfunc != NULL);
+    int err = 0;
+
+    s->ngenrs = s->naux + nlf + np;
+    s->genrs = malloc(s->ngenrs * sizeof *s->genrs);
+
+    if (s->genrs == NULL) {
+	s->ngenrs = 0;
+	err = E_ALLOC;
+    } else {
+	for (i=0; i<s->ngenrs; i++) {
+	    s->genrs[i] = NULL;
+	}
+    }
+
+    return err;
+}
+
 /* we "compile" the required equations first, so we can subsequently
    execute the compiled versions for maximum efficiency 
 */
 
 static int nls_genr_setup (nlspec *s)
 {
-    GENERATOR **genrs;
-    gretl_matrix *m;
     char formula[MAXLINE];
-    int i, j, ngen, np;
-    int v, err = 0;
+    gretl_matrix *m;
+    int i, j, v;
+    int err = 0;
 
-    s->ngenrs = 0;
-    np = (analytic_mode(s))? s->nparam : 0;
-    ngen = s->naux + np;
-    if (s->nlfunc != NULL) {
-	ngen++;
+    err = allocate_generators(s);
+    if (err) {
+	return err;
     }
 
-#if NLS_DEBUG
-    fprintf(stderr, "nls_genr_setup: current v = %d, n_gen = %d\n", 
-	    s->dset->v, ngen);
-#endif
+    /* We now loop across the "generators", setting them up
+       and checking them. We hook up any auxiliary genrs
+       first, then the criterion function, then the anaytical
+       derivatives, if any.
+    */
 
-    genrs = malloc(ngen * sizeof *genrs);
-    if (genrs == NULL) {
-	return E_ALLOC;
-    }
+    j = -1; /* index for derivatives */
 
-    for (i=0; i<ngen; i++) {
-	genrs[i] = NULL;
-    }
-
-    j = 0;
-
-    for (i=0; i<ngen && !err; i++) {
+    for (i=0; i<s->ngenrs && !err; i++) {
 	char *dname = NULL;
 	int gentype = 0;
 
 	if (i < s->naux) {
-	    /* auxiliary variables */
+	    /* auxiliary variables first */
 	    strcpy(formula, s->aux[i]);
 	} else if (i == s->naux) {
 	    /* residual or likelihood function */
@@ -245,7 +268,7 @@ static int nls_genr_setup (nlspec *s)
 		sprintf(formula, "$nl_y = %s", s->nlfunc);
 	    }
 	} else {
-	    /* derivatives/gradients */
+	    j++; /* increment derivative index */
 	    sprintf(s->params[j].dname, "$nl_x%d", i);
 	    if (scalar_param(s, j)) {
 		sprintf(formula, "%s = %s", s->params[j].dname, 
@@ -255,16 +278,15 @@ static int nls_genr_setup (nlspec *s)
 			s->params[j].deriv);
 	    }
 	    dname = s->params[j].dname;
-	    j++;
 	}
 
 	if (!err) {
-	    genrs[i] = genr_compile(formula, s->dset, OPT_P, &err);
+	    s->genrs[i] = genr_compile(formula, s->dset, OPT_P, &err);
 	}
 
 	if (err) {
 	    fprintf(stderr, "genr_compile: genrs[%d] = %p, err = %d\n", i, 
-		    (void *) genrs[i], err);
+		    (void *) s->genrs[i], err);
 	    fprintf(stderr, "formula: '%s'\n", formula);
 	    fprintf(stderr, "%s\n", gretl_errmsg_get());
 	    break;
@@ -273,26 +295,26 @@ static int nls_genr_setup (nlspec *s)
 	/* see if the formula actually works, and flush out NAs
 	   while we're at it
 	*/
-	genr_set_na_check(genrs[i]);
-	err = execute_genr(genrs[i], s->dset, s->prn);
-	genr_unset_na_check(genrs[i]);
+	genr_set_na_check(s->genrs[i]);
+	err = execute_genr(s->genrs[i], s->dset, s->prn);
+	genr_unset_na_check(s->genrs[i]);
 
 	if (err) {
-	    fprintf(stderr, "execute_genr: genrs[%d] = %p, err = %d\n", i, 
-		    (void *) genrs[i], err);
+	    fprintf(stderr, "execute_genr: s->genrs[%d] = %p, err = %d\n", i, 
+		    (void *) s->genrs[i], err);
 	    fprintf(stderr, "formula: '%s'\n", formula);
 	    break;
 	}
 
-	v = genr_get_output_varnum(genrs[i]);
-	m = genr_get_output_matrix(genrs[i]);
+	v = genr_get_output_varnum(s->genrs[i]);
+	m = genr_get_output_matrix(s->genrs[i]);
 
 	if (v == 0 && m == NULL) {
-	    if (genr_is_print(genrs[i])) {
+	    if (genr_is_print(s->genrs[i])) {
 		continue;
 	    }
 	    /* not a series, not a matrix: should be scalar */
-	    gentype = genr_get_output_type(genrs[i]);
+	    gentype = genr_get_output_type(s->genrs[i]);
 	    if (gentype != GRETL_TYPE_DOUBLE) {
 		fprintf(stderr, "got bad gentype %d\n", gentype);
 		fprintf(stderr, "formula: '%s'\n", formula);
@@ -307,6 +329,7 @@ static int nls_genr_setup (nlspec *s)
 	}
 
 	if (i == s->naux) {
+	    /* the criterion function */
 	    if (m != NULL) {
 		s->lvec = m;
 		s->lhtype = GRETL_TYPE_MATRIX;
@@ -319,14 +342,21 @@ static int nls_genr_setup (nlspec *s)
 	    } else {
 		err = E_TYPES;
 	    }
-	} else if (j > 0) {
-	    int k = j - 1;
+	} else if (j >= 0) {
+	    /* derivatives */
+	    if (v > 0) {
+		s->params[j].dtype = GRETL_TYPE_SERIES;
+	    } else if (m != NULL) {
+		s->params[j].dtype = GRETL_TYPE_MATRIX;
+	    } else {
+		s->params[j].dtype = GRETL_TYPE_DOUBLE;
+	    }
 
-	    s->params[k].dnum = v;
-	    s->params[k].dmat = m;
+	    s->params[j].dnum = v;
+	    s->params[j].dgenr = s->genrs[i];
 
-	    if (m != NULL || !scalar_param(s, k)) {
-		err = check_derivative_matrix(k, m, s);
+	    if (m != NULL || !scalar_param(s, j)) {
+		err = check_derivative_matrix(j, m, s);
 	    } 
 	}
 
@@ -349,10 +379,7 @@ static int nls_genr_setup (nlspec *s)
     }	
 
     if (err) {
-	destroy_genrs_array(genrs, ngen);
-    } else {
-	s->ngenrs = ngen;
-	s->genrs = genrs;
+	destroy_genrs_array(s);
     }
 
     return err;
@@ -523,12 +550,13 @@ static int nlspec_push_param (nlspec *s, const char *name, char *deriv)
     p->name[0] = '\0';
     strncat(p->name, name, VNAMELEN - 1);
     p->type = GRETL_TYPE_DOUBLE;
+    p->dtype = GRETL_TYPE_NONE;
     p->deriv = deriv;
     p->dnum = 0;
     p->nc = 1;
     p->dname[0] = '\0';
+    p->dgenr = NULL;
     p->vec = NULL;
-    p->dmat = NULL;
 
 #if NLS_DEBUG
     fprintf(stderr, "added param[%d] = '%s'\n", np, p->name);
@@ -640,11 +668,9 @@ nlspec_add_params_from_line (nlspec *s, const char *str)
 #endif
 
     for (i=0; i<nf && !err; i++) {
-	char *name = gretl_word_strdup(str, &str);
+	char *name = gretl_word_strdup(str, &str, OPT_S, &err);
 
-	if (name == NULL) {
-	    err = E_ALLOC;
-	} else {
+	if (!err) {
 	    err = check_param_name(name);
 	}
 
@@ -740,23 +766,22 @@ int update_coeff_values (const double *b, nlspec *s)
 
 static int nls_make_trimmed_dataset (nlspec *spec, int t1, int t2)
 {
-    DATASET *dinfo = NULL;
-    double ***pZ = NULL;
-    double **origZ = *spec->Z;
-    int nvar = spec->dset->v;
+    DATASET *dset = NULL;
+    DATASET *d0 = spec->dset;
+    int nvar = d0->v;
     int nobs = 0;
     int i, t, s;
 
-    spec->missmask = malloc(spec->dset->n + 1);
+    spec->missmask = malloc(d0->n + 1);
     if (spec->missmask == NULL) {
 	return E_ALLOC;
     }
 
-    spec->missmask[spec->dset->n] = '\0';
-    memset(spec->missmask, '0', spec->dset->n);
+    spec->missmask[d0->n] = '\0';
+    memset(spec->missmask, '0', d0->n);
 
     for (t=t1; t<=t2; t++) {
-	if (na(origZ[spec->lhv][t])) {
+	if (na(d0->Z[spec->lhv][t])) {
 	    spec->missmask[t] = '1';
 	} else {
 	    nobs++;
@@ -767,22 +792,17 @@ static int nls_make_trimmed_dataset (nlspec *spec, int t1, int t2)
 	return E_DF;
     }
 
-    pZ = malloc(sizeof *pZ);
-    if (pZ == NULL) {
-	return E_ALLOC;
-    }
-
-    dinfo = create_auxiliary_dataset(pZ, nvar, nobs);
-    if (dinfo == NULL) {
+    dset = create_auxiliary_dataset(nvar, nobs);
+    if (dset == NULL) {
 	return E_ALLOC;
     }
 
     for (i=1; i<nvar; i++) {
-	strcpy(dinfo->varname[i], spec->dset->varname[i]);
+	strcpy(dset->varname[i], d0->varname[i]);
 	s = 0;
 	for (t=t1; t<=t2; t++) {
 	    if (!na(origZ[spec->lhv][t])) {
-		(*pZ)[i][s++] = origZ[i][t];
+		dset->Z[i][s++] = d0->Z[i][t];
 	    }
 	}
     }
@@ -790,8 +810,7 @@ static int nls_make_trimmed_dataset (nlspec *spec, int t1, int t2)
     spec->real_t1 = t1;
     spec->real_t2 = t2;
 
-    spec->Z = pZ;
-    spec->dset = dinfo;
+    spec->dset = dset;
     spec->t1 = 0;
     spec->t2 = nobs - 1;
     spec->nobs = nobs;
@@ -809,7 +828,7 @@ static int nls_make_trimmed_dataset (nlspec *spec, int t1, int t2)
 /* Adjust starting and ending points of sample if need be, to avoid
    missing values; abort if there are missing values within the
    (possibly reduced) sample range.  For this purpose we generate the
-   nls residual variable.  
+   nls residual variable, or the loglikelihood in case of MLE. 
 */
 
 static int nl_missval_check (nlspec *s, const DATASET *dset)
@@ -829,7 +848,7 @@ static int nl_missval_check (nlspec *s, const DATASET *dset)
     }
 
     if (s->lvec != NULL || s->lhtype == GRETL_TYPE_DOUBLE) {
-	/* not a series result */
+	/* the calculation gives a matrix or scalar */
 	goto nl_miss_exit;
     }
 
@@ -892,7 +911,7 @@ static int nl_missval_check (nlspec *s, const DATASET *dset)
 }
 
 /* get_mle_ll: callback used by BFGS.  Note that this should return
-   NADBL in case of numerical problems.  This will signal to BFGS to
+   NADBL in case of numerical problems; that signals to BFGS to
    try a smaller step length.
 */
 
@@ -1039,9 +1058,13 @@ static int get_nls_derivs (int T, int offset, double *g, double **G,
 #endif
 
 	if (matrix_deriv(spec, j)) {
-	    gretl_matrix *m = spec->params[j].dmat;
+	    gretl_matrix *m = get_derivative_matrix(spec, j, &err);
 	    int i;
 
+	    if (err) {
+		return err;
+	    }
+
 #if NLS_DEBUG
 	    fprintf(stderr, "param[%d]: matrix at %p (%d x %d)\n", j, 
 		    (void *) m, m->rows, m->cols);
@@ -1148,10 +1171,13 @@ static int get_mle_gradient (double *b, double *g, int n,
 #endif
 
 	if (matrix_deriv(spec, j)) {
-	    m = spec->params[j].dmat;
+	    m = get_derivative_matrix(spec, j, &err);
 #if ML_DEBUG > 1
 	    gretl_matrix_print(m, "deriv matrix");
 #endif
+	    if (err) {
+		break;
+	    }
 	    for (k=0; k<m->cols; k++) {
 		g[i] = 0.0;
 		for (t=0; t<m->rows; t++) {
@@ -1290,26 +1316,6 @@ static void add_stats_to_model (MODEL *pmod, nlspec *spec,
     }
 }
 
-static int QML_vcv (nlspec *spec, gretl_matrix *V)
-{
-    gretl_matrix *tmp = NULL;
-    int k = V->rows;
-    
-    tmp = gretl_matrix_alloc(k, k);
-    if (tmp == NULL) {
-	return E_ALLOC;
-    }
-
-    /* form sandwich: V <- H^{-1} V H^{-1} */
-    gretl_matrix_copy_values(tmp, V);
-    gretl_matrix_qform(spec->Hinv, GRETL_MOD_NONE, tmp,
-		       V, GRETL_MOD_NONE);
-
-    gretl_matrix_free(tmp);
-
-    return 0;
-}
-
 /* returns the per-observation contributions to the log
    likelihood */
 
@@ -1336,128 +1342,91 @@ static const double *mle_llt_callback (const double *b, int i, void *p)
     }
 }
 
-/* add variance matrix based on OPG, (GG')^{-1}, or on QML sandwich,
-   H^{-1} GG' H^{-1}
-*/
-
-static int mle_build_vcv (MODEL *pmod, nlspec *spec, int *vcvopt)
+static gretl_matrix *ml_gradient_matrix (nlspec *spec, int *err)
 {
     gretl_matrix *G = NULL;
-    gretl_matrix *V = NULL;
-    gretl_matrix *m;
-    double x = 0.0;
     int k = spec->ncoeff;
     int T = spec->nobs;
-    int i, j, v, s, t;
-    int err = 0;
-
-    V = gretl_matrix_alloc(k, k);
-    if (V == NULL) {
-	return E_ALLOC;
-    }
 
     if (numeric_mode(spec)) {
-	G = numerical_score_matrix(spec->coeff, k, T, mle_llt_callback, 
-				   (void *) spec, &err);
-	if (err) {
-	    gretl_matrix_free(V);
-	    return err;
-	}
+	G = numerical_score_matrix(spec->coeff, T, k, mle_llt_callback, 
+				   (void *) spec, err);
     } else {
-	G = gretl_matrix_alloc(k, T);
+	/* using analytical derivatives */
+	gretl_matrix *m;
+	double x = 0.0;
+	int i, j, v, s, t;
+	
+	G = gretl_matrix_alloc(T, k);
 	if (G == NULL) {
-	    gretl_matrix_free(V);
-	    return E_ALLOC;
+	    *err = E_ALLOC;
+	    return NULL;
 	}
 	j = 0;
 	for (i=0; i<spec->nparam; i++) {
 	    if (matrix_deriv(spec, i)) {
-		m = spec->params[i].dmat;
+		m = get_derivative_matrix(spec, i, err);
+		if (*err != 0) {
+		    break;
+		}
 		for (s=0; s<m->cols; s++) {
 		    x = gretl_matrix_get(m, 0, s);
 		    for (t=0; t<T; t++) {
 			if (t > 0 && m->rows > 0) {
 			    x = gretl_matrix_get(m, t, s);
 			}
-			gretl_matrix_set(G, j, t, x);
+			gretl_matrix_set(G, t, j, x);
 		    }
 		    j++;
 		}
 	    } else if (scalar_deriv(spec, i)) {
 		x = gretl_scalar_get_value(spec->params[i].dname);
 		for (t=0; t<T; t++) {
-		    gretl_matrix_set(G, j, t, x);
+		    gretl_matrix_set(G, t, j, x);
 		}
 		j++;
 	    } else {
 		v = spec->params[i].dnum;
 		for (t=0; t<T; t++) {
 		    x = spec->dset->Z[v][t + spec->t1];
-		    gretl_matrix_set(G, j, t, x);
+		    gretl_matrix_set(G, t, j, x);
 		}
 		j++;
 	    } 
 	}		
     }
 
-    gretl_matrix_multiply_mod(G, GRETL_MOD_NONE,
-			      G, GRETL_MOD_TRANSPOSE,
-			      V, GRETL_MOD_NONE);
-
-    if ((spec->opt & OPT_R) && spec->Hinv != NULL) {
-	/* robust option -> QML */
-	err = QML_vcv(spec, V);
-	*vcvopt = VCV_QML;
-    } else {
-	/* plain OPG */
-	err = gretl_invert_symmetric_matrix(V);
-	if (err) {
-	    gretl_errmsg_set("failed to invert OPG matrix GG'");
-#if 0 /* experiment: show estimates even tho' VCV has failed */
-	    *vcvopt = VCV_MAX;
-	    k = k * k;
-	    for (i=0; i<k; i++) {
-		V->val[i] = 0.0/0.0;
-	    }
-	    err = 0;
-#endif
-	} else {
-	    *vcvopt = VCV_OP;
-	}
-    }
-
-    if (!err) {
-	err = gretl_model_write_vcv(pmod, V, -1);
-    }
-
-    gretl_matrix_free(G);
-    gretl_matrix_free(V);
-
-    return err;
+    return G;
 }
 
 static int mle_add_vcv (MODEL *pmod, nlspec *spec)
 {
-    int vcvopt = VCV_OP;
     int err = 0;
 
     if (spec->opt & OPT_A) {
+	/* auxiliary model */
 	int i;
 
 	for (i=0; i<pmod->ncoeff; i++) {
 	    pmod->sderr[i] = NADBL;
 	}
     } else if (spec->opt & OPT_H) {
-	vcvopt = VCV_HESSIAN;
-	err = gretl_model_write_vcv(pmod, spec->Hinv, -1);
+	err = gretl_model_add_hessian_vcv(pmod, spec->Hinv);
     } else {
-	/* either OPG or QML */
-	err = mle_build_vcv(pmod, spec, &vcvopt);
-    }
+	gretl_matrix *G = ml_gradient_matrix(spec, &err);
 
-    if (!err && pmod->vcv != NULL) {
-	gretl_model_set_vcv_info(pmod, VCV_ML, vcvopt);
-    }    
+	if (!err) {
+	    if ((spec->opt & OPT_R) && spec->Hinv != NULL) {
+		/* robust option -> QML */
+		err = gretl_model_add_QML_vcv(pmod, MLE, 
+					      spec->Hinv, G,
+					      NULL, OPT_R);
+	    } else {
+		err = gretl_model_add_OPG_vcv(pmod, G);
+	    }
+	}
+	gretl_matrix_free(G);
+    }
 
     return err;
 }
@@ -2063,8 +2032,6 @@ static void clear_nlspec (nlspec *spec)
     if (spec->params != NULL) {
 	for (i=0; i<spec->nparam; i++) {
 	    free(spec->params[i].deriv);
-	    spec->params[i].vec = NULL;
-	    spec->params[i].dmat = NULL;
 	}
 	free(spec->params);
 	spec->params = NULL;
@@ -2175,8 +2142,8 @@ static void clear_nlspec (nlspec *spec)
 
 /* callback for lm_calculate (below) to be used by minpack */
 
-static int nls_calc (integer *m, integer *n, double *x, double *fvec, 
-		     double *jac, integer *ldjac, integer *iflag,
+static int nls_calc (int m, int n, double *x, double *fvec, 
+		     double *jac, int ldjac, int *iflag,
 		     void *p)
 {
     nlspec *s = (nlspec *) p;
@@ -2196,7 +2163,7 @@ static int nls_calc (integer *m, integer *n, double *x, double *fvec,
 	} 
     } else if (*iflag == 2) {
 	/* calculate jacobian at x, results into jac */
-	if (get_nls_derivs(*m, 0, jac, NULL, p)) {
+	if (get_nls_derivs(m, 0, jac, NULL, p)) {
 	    *iflag = -1; 
 	}
     }
@@ -2215,11 +2182,11 @@ static int check_derivatives (nlspec *spec, PRN *prn)
     double *x = spec->coeff;
     double *fvec = spec->fvec;
     double *jac = spec->jac;
-    integer m = spec->nobs;
-    integer n = spec->ncoeff;
-    integer ldjac = m;
-    integer mode, iflag;
-    doublereal *xp, *xerr, *fvecp;
+    int m = spec->nobs;
+    int n = spec->ncoeff;
+    int ldjac = m;
+    int mode, iflag;
+    double *xp, *xerr, *fvecp;
     int i, badcount = 0, zerocount = 0;
 
     xp = malloc((n + m + m) * sizeof *xp);
@@ -2244,11 +2211,11 @@ static int check_derivatives (nlspec *spec, PRN *prn)
     /* mode 1: x contains the point of evaluation of the function; on
        output xp is set to a neighboring point. */
     mode = 1;
-    chkder_(&m, &n, x, fvec, jac, &ldjac, xp, fvecp, &mode, xerr);
+    chkder_(m, n, x, fvec, jac, ldjac, xp, fvecp, mode, xerr);
 
     /* calculate gradient */
     iflag = 2;
-    nls_calc(&m, &n, x, fvec, jac, &ldjac, &iflag, spec);
+    nls_calc(m, n, x, fvec, jac, ldjac, &iflag, spec);
     if (iflag == -1) goto chkderiv_abort;
 
 #if NLS_DEBUG > 1
@@ -2260,7 +2227,7 @@ static int check_derivatives (nlspec *spec, PRN *prn)
 
     /* calculate function, at neighboring point xp */
     iflag = 1;
-    nls_calc(&m, &n, xp, fvecp, jac, &ldjac, &iflag, spec);
+    nls_calc(m, n, xp, fvecp, jac, ldjac, &iflag, spec);
     if (iflag == -1) goto chkderiv_abort; 
 
     /* mode 2: on input, fvec must contain the functions, the rows of
@@ -2269,7 +2236,7 @@ static int check_derivatives (nlspec *spec, PRN *prn)
        measures of correctness of the respective gradients.
     */
     mode = 2;
-    chkder_(&m, &n, x, fvec, jac, &ldjac, xp, fvecp, &mode, xerr);
+    chkder_(m, n, x, fvec, jac, ldjac, xp, fvecp, mode, xerr);
 
 #if NLS_DEBUG > 1
     fprintf(stderr, "\nchkder, done mode 2:\n");
@@ -2367,6 +2334,8 @@ static int mle_calculate (nlspec *s, PRN *prn)
 	}
 
 	if (err) {
+	    pprintf(prn, _("\nError: Hessian non-negative definite? (err = %d); "
+			  "dropping back to OPG\n"), err);
 	    /* try dropping back to OPG */
 	    s->opt &= ~OPT_H;
 	    s->opt &= ~OPT_R;
@@ -2382,13 +2351,13 @@ static int mle_calculate (nlspec *s, PRN *prn)
 
 static int lm_calculate (nlspec *spec, PRN *prn)
 {
-    integer m = spec->nobs;
-    integer n = spec->ncoeff;
-    integer lwa = 5 * n + m; /* work array size */
-    integer ldjac = m;       /* leading dimension of jac array */
-    integer info = 0;
-    integer *ipvt;
-    doublereal *wa;
+    int m = spec->nobs;
+    int n = spec->ncoeff;
+    int lwa = 5 * n + m; /* work array size */
+    int ldjac = m;       /* leading dimension of jac array */
+    int info = 0;
+    int *ipvt;
+    double *wa;
     int err = 0;
 
     wa = malloc(lwa * sizeof *wa);
@@ -2407,8 +2376,8 @@ static int lm_calculate (nlspec *spec, PRN *prn)
     /* note: maxfev is automatically set to 100*(n + 1) */
 
     /* call minpack */
-    lmder1_(nls_calc, &m, &n, spec->coeff, spec->fvec, spec->jac, &ldjac, 
-	    &spec->tol, &info, ipvt, wa, &lwa, spec);
+    lmder1_(nls_calc, m, n, spec->coeff, spec->fvec, spec->jac, ldjac, 
+	    spec->tol, &info, ipvt, wa, lwa, spec);
 
     switch ((int) info) {
     case -1: 
@@ -2445,8 +2414,8 @@ static int lm_calculate (nlspec *spec, PRN *prn)
 /* callback for lm_approximate (below) to be used by minpack */
 
 static int 
-nls_calc_approx (integer *m, integer *n, double *x, double *fvec,
-		 integer *iflag, void *p)
+nls_calc_approx (int m, int n, double *x, double *fvec,
+		 int *iflag, void *p)
 {
     int erru, errc, err;
 
@@ -2474,18 +2443,18 @@ nls_calc_approx (integer *m, integer *n, double *x, double *fvec,
 
 static int lm_approximate (nlspec *spec, PRN *prn)
 {
-    integer m = spec->nobs;
-    integer n = spec->ncoeff;
-    integer ldjac = m;
-    integer info = 0;
-    integer maxfev = 200 * (n + 1); /* max iterations */
-    integer mode = 1, nprint = 0, nfev = 0;
-    integer iflag = 0;
-    integer *ipvt;
-    doublereal gtol = 0.0;
-    doublereal epsfcn = 0.0, factor = 100.;
-    doublereal *dspace, *diag, *qtf;
-    doublereal *wa1, *wa2, *wa3, *wa4;
+    int m = spec->nobs;
+    int n = spec->ncoeff;
+    int ldjac = m;
+    int info = 0;
+    int maxfev = 200 * (n + 1); /* max iterations */
+    int mode = 1, nprint = 0, nfev = 0;
+    int iflag = 0;
+    int *ipvt;
+    double gtol = 0.0;
+    double epsfcn = 0.0, factor = 100.;
+    double *dspace, *diag, *qtf;
+    double *wa1, *wa2, *wa3, *wa4;
     int err = 0;
     
     dspace = malloc((5 * n + m) * sizeof *dspace);
@@ -2504,9 +2473,9 @@ static int lm_approximate (nlspec *spec, PRN *prn)
     wa4 = wa3 + n;
 
     /* call minpack */
-    lmdif_(nls_calc_approx, &m, &n, spec->coeff, spec->fvec, 
-	   &spec->tol, &spec->tol, &gtol, &maxfev, &epsfcn, diag, 
-	   &mode, &factor, &nprint, &info, &nfev, spec->jac, &ldjac, 
+    lmdif_(nls_calc_approx, m, n, spec->coeff, spec->fvec, 
+	   spec->tol, spec->tol, gtol, maxfev, epsfcn, diag, 
+	   mode, factor, nprint, &info, &nfev, spec->jac, ldjac, 
 	   ipvt, qtf, wa1, wa2, wa3, wa4, spec);
 
     spec->iters = nfev;
@@ -2544,8 +2513,8 @@ static int lm_approximate (nlspec *spec, PRN *prn)
 	spec->opt = OPT_NONE;
 
 	/* call minpack again */
-	fdjac2_(nls_calc_approx, &m, &n, spec->coeff, spec->fvec, 
-		spec->jac, &ldjac, &iflag, &epsfcn, wa4, spec);
+	fdjac2_(nls_calc_approx, m, n, spec->coeff, spec->fvec, 
+		spec->jac, ldjac, &iflag, epsfcn, wa4, spec);
 	spec->crit = ess;
 	spec->iters = iters;
 	spec->opt = opt;
@@ -2917,7 +2886,7 @@ static double default_nls_toler;
 double get_default_nls_toler (void)
 {
     if (default_nls_toler == 0.0) {
-	default_nls_toler = pow(dpmpar_(&one), .75);
+	default_nls_toler = pow(DBL_EPSILON, .75);
     }
 
     return default_nls_toler;
@@ -2937,6 +2906,10 @@ static int check_spec_requirements (nlspec *spec)
     return err;
 }
 
+/* make any adjustments that may be needed in case the value that's
+   returned by the user's likelihood function is a scalar 
+*/
+
 static int mle_scalar_check (nlspec *spec)
 {
     int err = 0;
@@ -3200,7 +3173,9 @@ static MODEL real_nl_model (nlspec *spec, DATASET *dset,
 	    dset->v - origv);
 #endif
 
-    dataset_drop_last_variables(dset->v - origv, dset);
+    if (!dset->auxiliary) {
+	dataset_drop_last_variables(dset->v - origv, dset);
+    }
 
     if (!(opt & OPT_A) && !nlmod.errcode) {
 	set_model_id(&nlmod);
