@@ -70,6 +70,7 @@ static double spearman_signif (double rho, int n)
     if (rho > x[0]) return .01;
     if (rho > x[1]) return .05;
     if (rho > x[2]) return .10;
+
     return 1.0;
 }
 
@@ -252,20 +253,21 @@ static void print_raw_and_ranked (int vx, int vy,
 				  PRN *prn)
 {
     int T = dset->t2 - dset->t1 + 1;
+    int obslen = max_obs_marker_length(dset);
     int t, i = 0;
 
-    obs_marker_init(dset);
-    pprintf(prn, "\n     %s ", _("Obs"));
-    pprintf(prn, "%13s%13s%13s%13s\n\n", dset->varname[vx], _("rank"),
+    pputc(prn, '\n');
+    bufspace(obslen + 1, prn);
+    pprintf(prn, "%15s%8s%15s%8s\n\n", dset->varname[vx], _("rank"),
 	    dset->varname[vy], _("rank"));
 
     for (t=0; t<T; t++) {
-	print_obs_marker(t + dset->t1, dset, prn);
+	print_obs_marker(t + dset->t1, dset, obslen, prn);
 	if (!(na(x[t])) && !(na(y[t]))) {
-	    gretl_printxn(x[t], 15, prn);
-	    pprintf(prn, "%15g", rx[i]);
-	    gretl_printxn(y[t], 15, prn);
-	    pprintf(prn, "%15g", ry[i]);
+	    pprintf(prn, "%#15g", x[t]);
+	    pprintf(prn, "%8g", rx[i]);
+	    pprintf(prn, "%#15g", y[t]);
+	    pprintf(prn, "%8g", ry[i]);
 	    i++;
 	}
 	pputc(prn, '\n');
@@ -1272,7 +1274,7 @@ struct pair_sorter {
  * sort_pairs_by_x:
  * @x: data vector by which to sort.
  * @y: data vector. 
- * @order: location to recieve sort order, or %NULL.
+ * @order: location to receive sort order, or %NULL.
  * @labels: array of strings to be sorted along with
  * the data, or %NULL.
  *
@@ -1354,60 +1356,266 @@ static int data_pre_sorted (const gretl_matrix *x)
     return 1;
 }
 
-/* revised weights based on bisquare function (Cleveland) */
+#define LDEBUG 0
 
-static void robust_weights (gretl_matrix *ei, const gretl_matrix *X, 
-			    const gretl_matrix *y, const gretl_matrix *bj, 
-			    gretl_matrix *wt, const gretl_matrix *wi,
-			    int d)
+/* convenience struct for passing loess data */
+
+struct loess_info {
+    const gretl_matrix *y;
+    const gretl_matrix *x;
+    gretl_matrix *Xi;
+    gretl_matrix *yi;
+    gretl_matrix *wt;
+    int d;
+    int n;
+    int N;
+    int n_ok;
+    int loo;
+};
+
+/* Compute robustness weights for loess, if wanted: on input @rw
+   contains the N residuals; on return it contains the robustness
+   weights as a function of the residuals. For observations
+   where y is missing, the weight is NADBL.
+*/
+
+static int make_robustness_weights (gretl_matrix *rw, int N) 
 {
-    int i, n = gretl_vector_get_length(wt);
-    double s, es, xi, di;
+    double *tmp = malloc(N * sizeof *tmp);
+    double s, eis;
+    int i, k, err = 0;
 
-    for (i=0; i<n; i++) {
-	ei->val[i] = y->val[i] - bj->val[0];
-	if (d > 0) {
-	    xi = gretl_matrix_get(X, i, 1);
-	    ei->val[i] -= bj->val[1] * xi;
-	    if (d == 2) {
-		ei->val[i] -= bj->val[2] * xi * xi;
+    if (tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    /* pack the absolute values of the non-missing residuals
+       into tmp */
+    k = 0;
+    for (i=0; i<N; i++) {
+	if (!na(rw->val[i])) {
+	    tmp[k++] = fabs(rw->val[i]);
+	}
+    }
+
+    /* find the median absolute residual */
+    s = gretl_median(0, k-1, tmp);
+    free(tmp);
+
+    if (na(s)) {
+	err = E_DATA;
+    } else {
+	s *= 6;
+	for (i=0; i<N && !err; i++) {
+	    if (!na(rw->val[i])) {
+		eis = rw->val[i] / s;
+		if (fabs(eis) < 1.0) {
+		    /* apply bisquare */
+		    rw->val[i] = (1 - eis * eis) * (1 - eis * eis);
+		} else {
+		    rw->val[i] = 0.0;
+		}
 	    }
 	}
-	ei->val[i] = fabs(ei->val[i]);
     }
 
-    s = gretl_median(0, n - 1, ei->val);
-
-    for (i=0; i<n; i++) {
-	es = ei->val[i] / (6.0 * s);
-	if (es < 1.0) {
-	    di = (1.0 - es * es) * (1.0 - es * es);
-	} else {
-	    di = 0.0;
-	}
-	wi->val[i] = di * wt->val[i];
-    }
+    return err;
 }
 
-static void weight_x_y (const gretl_matrix *x, const gretl_matrix *y,
-			gretl_matrix *Xr, gretl_matrix *yr,
-			const gretl_matrix *w, int j, int d)
-{
-    int t, n = gretl_vector_get_length(w);
-    double xrt, wt;
+/* Multiply the robustness weights, @rw, into the regular
+   weights, @wt, taking care to register the two vectors
+   (rw is full-length while wt is local), and to skip any 
+   missing values in rw.
+*/
 
-    for (t=0; t<n; t++) {
-	wt = sqrt(w->val[t]);
-	gretl_matrix_set(Xr, t, 0, wt);
-	if (d > 0) {
-	    xrt = x->val[t+j];
-	    gretl_matrix_set(Xr, t, 1, xrt * wt);
-	    if (d == 2) {
-		gretl_matrix_set(Xr, t, 1, xrt * xrt * wt);
-	    }
-	}		
-	yr->val[t] = y->val[t+j] * wt;
+static void adjust_weights (const gretl_matrix *rw, 
+			    gretl_matrix *wt, int a)
+{
+    int k, n = gretl_vector_get_length(wt);
+    int t = a;
+
+    for (k=0; k<n; k++) {
+	wt->val[k] *= rw->val[t];
+	if (k < n - 1) {
+	    t++;
+	    while (na(rw->val[t])) t++;
+	}
     }
+} 
+
+/* Apply the weights in lo->wt to the local data matrices
+   lo->Xi and lo->yi. This is straightforward since all
+   the matrices have lo->n rows.
+*/
+
+static void weight_local_data (struct loess_info *lo)
+{
+    double xkj, wk;
+    int k, j;
+
+    for (k=0; k<lo->n; k++) {
+	wk = sqrt(lo->wt->val[k]);
+	for (j=0; j<lo->Xi->cols; j++) {
+	    xkj = gretl_matrix_get(lo->Xi, k, j);
+	    gretl_matrix_set(lo->Xi, k, j, xkj * wk);
+	}
+	lo->yi->val[k] *= wk;
+    }
+
+#if LDEBUG > 1
+    gretl_matrix_print(lo->Xi, "Xi, weighted");
+    gretl_matrix_print(lo->yi, "yi, weighted");
+#endif
+}
+
+static int next_ok_obs (const double *y, int i)
+{
+    i++;
+    while (xna(y[i])) i++;
+    return i;
+}
+
+static int loess_get_local_data (int i, int *pa,
+				 struct loess_info *lo,
+				 int *xconst)
+{
+    const double *x = lo->x->val;
+    const double *y = lo->y->val;
+    double xk, xk1, xds, h = 0;
+    int n = lo->n, n_ok = lo->n_ok;
+    int k_skip = -1;
+    int a, b, k, m, t;
+    int err = 0;
+
+    gretl_matrix_zero(lo->Xi);
+    gretl_matrix_zero(lo->yi);
+    gretl_matrix_zero(lo->wt);
+
+    a = *pa;
+
+#if LDEBUG
+    fprintf(stderr, "\ni=%d, a=%d, n_ok=%d\n", i, a, n_ok);
+#endif
+
+    /* First determine where we should start reading the 
+       neighbors of xi: search rightward from a, so far as
+       this is feasible.
+    */
+
+    while (n_ok > n) {
+	/* find b, the next point that would be included if
+	   we move the neighbor set one place to the right: 
+	   start at a+1 and proceed until we have n points 
+	   with valid y-values
+	*/
+	for (m=0, b=a+1; ; b++) {
+	    if (!xna(y[b]) && ++m == n) {
+		break;
+	    }
+	}
+	if (fabs(x[i] - x[a]) < fabs(x[i] - x[b])) {
+	    /* the max distance increased: get out */
+	    break;
+	}	    
+	/* shift one (valid) place to the right */
+	a = next_ok_obs(y, a);
+	n_ok--;
+    }
+
+    /* record status for next round */
+    *pa = a;
+    lo->n_ok = n_ok;
+
+    /* Having found the starting index for the n nearest neighbors
+       of xi, transcribe the relevant data into Xi and yi. As we 
+       go, check whether x is constant in this sub-sample.
+    */
+
+    *xconst = 1;
+    xk1 = 0;
+    t = a;
+
+    for (k=0; k<n; k++) {
+	if (lo->loo && t == i) {
+	    /* leave-one-out: mark this observation */
+	    k_skip = k;
+	}
+	lo->yi->val[k] = y[t];
+	xk = x[t];
+	gretl_matrix_set(lo->Xi, k, 0, 1.0);
+	gretl_matrix_set(lo->Xi, k, 1, xk);
+	if (lo->Xi->cols > 2) {
+	    gretl_matrix_set(lo->Xi, k, 2, xk * xk);
+	}
+	if (*xconst && k > 0 && xk > xk1) {
+	    *xconst = 0;
+	}
+	xk1 = xk;
+	if (k < n - 1) {
+	    t = next_ok_obs(y, t);
+	}
+    }
+
+#if LDEBUG
+    fprintf(stderr, " -> a=%d, n_ok=%d (xconst = %d)\n", 
+	    a, n_ok, *xconst);
+#endif
+
+    if (!err) {
+	/* find the max(abs) distance from xi */
+	double h0 = fabs(x[i] - gretl_matrix_get(lo->Xi, 0, 1));
+	double hn = fabs(x[i] - gretl_matrix_get(lo->Xi, n-1, 1));
+
+	h = (h0 > hn)? h0 : hn;
+    }
+
+    /* compute scaled distances and tricube weights */
+    for (k=0; k<n; k++) {
+	if (k == k_skip) {
+	    /* exclude this obs via a zero weight? */
+	    lo->wt->val[k] = 0.0;
+	    continue;
+	}
+	xk = gretl_matrix_get(lo->Xi, k, 1);
+	if (h == 0.0) {
+	    lo->wt->val[k] = 1.0;
+	} else {
+	    xds = fabs(x[i] - xk) / h;
+	    if (xds < 1.0) {
+		lo->wt->val[k] = pow(1.0 - pow(xds, 3.0), 3.0);
+	    }
+	}
+#if LDEBUG > 1
+	fprintf(stderr, "y=%10g, x=%10g, dist=%10g\n",
+		lo->yi->val[k], xk, fabs(x[i] - xk));
+#endif
+    }
+
+#if LDEBUG > 1
+    gretl_matrix_print(lo->Xi, "Xi");
+    gretl_matrix_print(lo->yi, "yi");
+    gretl_matrix_print(lo->wt, "wt");
+#endif
+
+    return err;
+}
+
+static int loess_count_usable_obs (const gretl_matrix *y,
+				   int N, int *amin)
+{
+    int i, n_ok = 0;
+
+    for (i=0; i<N; i++) {
+	if (!xna(y->val[i])) {
+	    if (n_ok == 0) {
+		/* record the first usable obs index */
+		*amin = i;
+	    }
+	    n_ok++;
+	}
+    }   
+
+    return n_ok;
 }
 
 /**
@@ -1415,40 +1623,39 @@ static void weight_x_y (const gretl_matrix *x, const gretl_matrix *y,
  * @x: x-axis variable (must be pre-sorted).
  * @y: response variable.
  * @d: order for polynomial fit (0 <= d <= 2).
- * @q: bandwidth (0 < q < 1).
+ * @q: bandwidth (0 < q <= 1).
  * @opt: give %OPT_R for robust variant (with re-weighting based on
  * the first-stage residuals).
- * @err: location to recieve error code.
+ * @err: location to receive error code.
  *
  * Computes loess estimates based on William Cleveland, "Robust Locally 
  * Weighted Regression and Smoothing Scatterplots", Journal of the 
  * American Statistical Association, Vol. 74 (1979), pp. 829-836.
- * Generally one expects that @d = 1 and @q is in the neighborhood
+ * Typically one expects that @d = 1 and @q is in the neighborhood
  * of 0.5.
  *
- * The x,y pairs must be pre-sorted by increasing value of x; an
+ * The x,y pairs must be pre-sorted by increasing value of @x; an
  * error is flagged if this is not the case.  See also
  * sort_pairs_by_x().
  * 
  * Returns: allocated vector containing the loess fitted values, or
- * %NULL on failure (in which case @err will contain a non-zero
- * error code).
+ * %NULL on failure.
  */
 
 gretl_matrix *loess_fit (const gretl_matrix *x, const gretl_matrix *y,
 			 int d, double q, gretlopt opt, int *err)
 {
+    struct loess_info lo;
     gretl_matrix_block *B;
-    gretl_matrix *Xr, *yr, *wt, *bj;
+    gretl_matrix *Xi, *yi, *wt, *b;
     gretl_matrix *yh = NULL;
-    gretl_matrix *ei = NULL;
-    gretl_matrix *wi = NULL;
-    int T = gretl_vector_get_length(y);
-    double xi, d1, d2, dmax;
-    int jmin, jmax, k, kmax;
-    int i, j, t, n;
+    gretl_matrix *rw = NULL;
+    int N = gretl_vector_get_length(y);
+    int k, iters, Xic, amin = 0;
+    int n_ok, robust = 0, loo = 0;
+    int i, n;
 
-    if (d < 0 || d > 2) {
+    if (d < 0 || d > 2 || q > 1.0) {
 	*err = E_DATA;
 	return NULL;
     }
@@ -1459,124 +1666,146 @@ gretl_matrix *loess_fit (const gretl_matrix *x, const gretl_matrix *y,
 	return NULL;
     }
 
-    /* check the supplied q value */
-    if (q > 1.0) {
-	q = 1;
-    } else if (q < (d + 1.0) / T) {
-	q = (d + 1.0) / T;
+    /* check for usable data points */
+    n_ok = loess_count_usable_obs(y, N, &amin);
+    if (n_ok < 4) {
+	*err = E_TOOFEW;
+	return NULL;
     }
 
-    n = (int) ceil(q * T);
+    if (opt & OPT_O) {
+	/* leave one out: experimental */
+	loo = 1;
+    }
 
-    B = gretl_matrix_block_new(&Xr, n, d+1,
-			       &yr, n, 1,
+    /* check for q too small */
+    if (q < (d + 1.0) / n_ok) {
+	q = (d + 1.0) / n_ok;
+    }
+
+    /* set the local sub-sample size */
+    n = (int) ceil(q * n_ok);
+
+    /* we need a minimum of two columns in Xi, in order
+       to compute the x-distance based weights */
+    Xic = (d == 0)? 2 : d + 1;
+
+    B = gretl_matrix_block_new(&Xi, n, Xic,
+			       &yi, n, 1,
 			       &wt, n, 1,
-			       &bj, d+1, 1,
+			       &b, d+1, 1,
 			       NULL);
     if (B == NULL) {
 	*err = E_ALLOC;
 	return NULL;
     }
 
-    yh = gretl_column_vector_alloc(T);
+    /* vector to hold the fitted values */
+    yh = gretl_column_vector_alloc(N);
     if (yh == NULL) {
 	*err = E_ALLOC;
 	goto bailout;
-    }    
+    } 
 
     if (opt & OPT_R) {
-	ei = gretl_column_vector_alloc(n);
-	wi = gretl_column_vector_alloc(n);
-	if (ei == NULL || wi == NULL) {
+	/* extra storage for residuals/robustness weights */
+	rw = gretl_column_vector_alloc(N);
+	if (rw == NULL) {
 	    *err = E_ALLOC;
 	    goto bailout;
 	}
-	kmax = 3;
+	iters = 2; /* maybe 3? */
+	robust = 1;
     } else {
-	kmax = 1;
+	iters = 1;
     }
 
-    for (i=0; i<T && !*err; i++) {
-	double xdt, xds;
+    /* fill out the convenience struct */
+    lo.y = y;
+    lo.x = x;
+    lo.Xi = Xi;
+    lo.yi = yi;
+    lo.wt = wt;
+    lo.d = d;
+    lo.n = n;
+    lo.N = N;
+    lo.loo = loo;
 
-	xi = x->val[i];
+    for (k=0; k<iters && !*err; k++) {
+	/* iterations for robustness, if wanted */
+	int a = amin;
 
-	/* get the search bounds */
-	jmin = i - n + 1;
-	if (jmin < 0) {
-	    jmin = 0;
-	}
-	jmax = i;
-	if (jmax + n > T) {
-	    jmax = T - n;
-	}
+	lo.n_ok = n_ok;
 
-	/* find the n x-values closest to xi */
-	for (j=jmin; j<=jmax; j++) {
-	    if (j + n >= T) {
+	for (i=0; i<N && !*err; i++) {
+	    /* iterate across points in full sample */
+	    double xi = x->val[i];
+	    int xconst = 0;
+
+	    /* a is the leftmost possible index for starting to
+	       read the n nearest neighbors of xi.
+	    */
+	    *err = loess_get_local_data(i, &a, &lo, &xconst);
+	    if (*err) {
 		break;
 	    }
-	    d1 = fabs(xi - x->val[j]);
-	    d2 = fabs(xi - x->val[j + n]);
-	    if (d1 <= d2) {
-		break;
+
+	    if (k > 0) {
+		/* We have robustness weights, rw, based on the residuals
+		   from the last round, which should be used to adjust 
+		   the wt as computed in loess_get_local_data().
+		*/
+		adjust_weights(rw, wt, a);
+	    } 
+
+	    /* apply weights to the local data */
+	    weight_local_data(&lo);
+
+	    if (d == 0 || xconst) {
+		/* not using x in the regressions: mask the x column(s) */
+		gretl_matrix_reuse(Xi, -1, 1);
+		if (b->rows > 1) {
+		    gretl_matrix_reuse(b, 1, 1);
+		}
 	    }
-	}
 
-	dmax = 0.0;
-	for (t=0; t<n; t++) {
-	    xdt = fabs(xi - x->val[t+j]);
-	    if (xdt > dmax) {
-		dmax = xdt;
-	    }
-	}
-
-	for (t=0; t<n; t++) {
-	    /* compute scaled distances */
-	    xdt = fabs(xi - x->val[t+j]);
-	    xds = xdt / dmax;
-	    /* form tricube weights */
-	    if (xds >= 1.0) {
-		wt->val[t] = 0.0;
-	    } else {
-		wt->val[t] = pow(1.0 - pow(xds, 3.0), 3.0);
-	    }
-	}
-
-	/* form weighted vars */
-	weight_x_y(x, y, Xr, yr, wt, j, d);
-
-	for (k=0; k<kmax && !*err; k++) {
 	    /* run local WLS */
-	    *err = gretl_matrix_ols(yr, Xr, bj, NULL, NULL, NULL);
+	    *err = gretl_matrix_SVD_ols(yi, Xi, b, NULL, NULL, NULL);
 
-	    if (!*err && k < kmax - 1) {
-		/* re-weight based on the previous residuals */
-		for (t=0; t<n; t++) {
-		    gretl_matrix_set(Xr, t, 1, x->val[t+j]);
-		    gretl_vector_set(yr, t, y->val[t+j]);
+	    if (!*err) {
+		/* yh: evaluate the polynomial at xi */
+		yh->val[i] = b->val[0];
+		if (b->rows > 1) {
+		    yh->val[i] += b->val[1] * xi;
+		    if (b->rows == 3) {
+			yh->val[i] += b->val[2] * xi * xi;
+		    }
 		}
-		robust_weights(ei, Xr, yr, bj, wt, wi, d);
-		weight_x_y(x, y, Xr, yr, wi, j, d);
-	    }
-	}	
-
-	if (!*err) {
-	    yh->val[i] = bj->val[0];
-	    if (d > 0) {
-		yh->val[i] += bj->val[1] * x->val[i];
-		if (d == 2) {
-		    yh->val[i] += bj->val[2] * x->val[i] * x->val[i];
+		if (robust && k < iters - 1) {
+		    /* save residual for robustness weights */
+		    if (xna(y->val[i])) {
+			rw->val[i] = NADBL;
+		    } else {
+			rw->val[i] = y->val[i] - yh->val[i];
+		    }
 		}
 	    }
+
+	    /* ensure matrices are at full size */
+	    gretl_matrix_reuse(Xi, -1, Xic);
+	    gretl_matrix_reuse(b, d+1, -1);
+
+	} /* end loop over sample */
+
+	if (!*err && robust && k < iters - 1) {
+	    *err = make_robustness_weights(rw, N);
 	}
-    }
+    } /* end robustness iterations */
 
  bailout:
 	
     gretl_matrix_block_destroy(B);
-    gretl_matrix_free(ei);
-    gretl_matrix_free(wi);
+    gretl_matrix_free(rw);
 
     if (*err) {
 	gretl_matrix_free(yh);
@@ -1585,4 +1814,3 @@ gretl_matrix *loess_fit (const gretl_matrix *x, const gretl_matrix *y,
 
     return yh;
 }
-
