@@ -559,8 +559,7 @@ void *gretl_get_object_by_name (const char *name)
 int 
 gretl_get_object_and_type (const char *name, void **pp, GretlObjType *type)
 {
-    const char *test;
-    int i, err = E_DATA;
+    int err = E_DATA;
 
     *pp = NULL;
     *type = 0;
@@ -569,13 +568,24 @@ gretl_get_object_and_type (const char *name, void **pp, GretlObjType *type)
 	return err;
     }
 
-    for (i=0; i<n_obj; i++) {
-	test = gretl_object_get_name(ostack[i].ptr, ostack[i].type);
-	if (!strcmp(name, test)) {
-	    *pp = ostack[i].ptr;
-	    *type = ostack[i].type;
+    if (!strcmp(name, "$system")) {
+	*pp = get_anonymous_equation_system();
+	if (*pp != NULL) {
+	    *type = GRETL_OBJ_SYS; 
 	    err = 0;
-	    break;
+	}
+    } else {
+	const char *test;
+	int i;
+
+	for (i=0; i<n_obj; i++) {
+	    test = gretl_object_get_name(ostack[i].ptr, ostack[i].type);
+	    if (!strcmp(name, test)) {
+		*pp = ostack[i].ptr;
+		*type = ostack[i].type;
+		err = 0;
+		break;
+	    }
 	}
     }
 
@@ -961,6 +971,10 @@ static double real_get_obj_scalar (void *p, GretlObjType type,
 	    x = sys->ess;
 	} else if (idx == M_DF) {
 	    x = sys->df;
+	} else if (idx == M_DIAGTEST) {
+	    err = system_diag_test(sys, &x, NULL);
+	} else if (idx == M_DIAGPVAL) {
+	    err = system_diag_test(sys, NULL, &x);
 	}
     } else if (type == GRETL_OBJ_VAR) {
 	GRETL_VAR *var = (GRETL_VAR *) p;
@@ -1042,14 +1056,38 @@ real_get_obj_list (void *p, GretlObjType type, int idx, int *err)
     int *list = NULL;
     
     if (idx <= 0) {
-	*err = 1;
-	return list;
+	*err = E_DATA;
+	return NULL;
     }
 
-    if (type == GRETL_OBJ_EQN && idx == M_XLIST) {
-	MODEL *pmod = (MODEL *) p;
+    if (type == GRETL_OBJ_EQN) {
+	if (idx == M_XLIST) {
+	    list = gretl_model_get_x_list((const MODEL *) p);
+	} else if (idx == M_YLIST) {
+	    list = gretl_model_get_y_list((const MODEL *) p);
+	} else {
+	    *err = E_BADSTAT;
+	}
+    } else if (type == GRETL_OBJ_VAR || type == GRETL_OBJ_SYS) {
+	if (idx == M_YLIST) {
+	    const int *ylist = NULL;
 
-	list = gretl_model_get_x_list(pmod);
+	    if (type == GRETL_OBJ_VAR) {
+		ylist = gretl_VAR_get_endo_list((const GRETL_VAR *) p);
+	    } else if (type == GRETL_OBJ_SYS) {
+		ylist = system_get_endog_vars((const equation_system *) p);
+	    }
+	    if (ylist == NULL) {
+		*err = E_BADSTAT;
+	    } else {
+		list = gretl_list_copy(ylist);
+		if (list == NULL) {
+		    *err = E_ALLOC;
+		}
+	    }
+	} else {
+	    *err = E_BADSTAT;
+	}
     } else {
 	*err = E_BADSTAT;
     }
@@ -1335,6 +1373,33 @@ void *last_model_get_data (const char *key, GretlType *type,
     return ret;
 }
 
+char *last_model_get_vcv_type (void)
+{
+    stacker *smatch = find_smatch(NULL);
+    static char ret[16];
+
+    *ret = '\0';
+
+    if (smatch != NULL && smatch->type == GRETL_OBJ_EQN) {
+	const MODEL *pmod = smatch->ptr;
+	VCVInfo *vi;
+
+	vi = gretl_model_get_data(pmod, "vcv_info");
+
+	if (vi != NULL && vi->vmaj == VCV_ML) {
+	    if (vi->vmin == ML_HESSIAN) {
+		strcpy(ret, "Hessian");
+	    } else if (vi->vmin == ML_OP) {
+		strcpy(ret, "OPG");
+	    } else if (vi->vmin == ML_QML) {
+		strcpy(ret, "Sandwich");
+	    }
+	}
+    }
+
+    return (*ret != '\0')? ret : NULL;
+}
+
 static int namechar_spn_with_space (const char *s)
 {
     const char *ok = "abcdefghijklmnopqrstuvwxyz"
