@@ -44,6 +44,7 @@
 			 c == FREQ ||		\
 			 c == KPSS ||		\
 			 c == MODTEST ||	\
+			 c == LEVERAGE ||	\
                          c == LEVINLIN ||       \
 			 c == LOOP ||		\
                          c == MAHAL ||          \
@@ -98,6 +99,9 @@ struct gretl_option gretl_opts[] = {
     { AR1,      OPT_H, "hilu", 0 },
     { AR1,      OPT_P, "pwe", 0 },
     { APPEND,   OPT_T, "time-series", 0 },
+    { APPEND,   OPT_R, "rowoffset", 2 },
+    { APPEND,   OPT_C, "coloffset", 2 },
+    { APPEND,   OPT_S, "sheet", 2 },
     { ARBOND,   OPT_A, "asymptotic", 0 },
     { ARBOND,   OPT_D, "time-dummies", 0 },
     { ARBOND,   OPT_H, "orthdev", 0 },
@@ -114,6 +118,7 @@ struct gretl_option gretl_opts[] = {
     { BIPROBIT, OPT_G, "opg", 0 },
     { BIPROBIT, OPT_R, "robust", 0 },
     { BIPROBIT, OPT_V, "verbose", 0 },
+    { BIPROBIT, OPT_C, "cluster", 2 },    
     { BIPROBIT, OPT_X, "save-xbeta", 0 },
     { BXPLOT,   OPT_O, "notches", 0 },
     { BXPLOT,   OPT_L, "outliers", 2 },
@@ -128,6 +133,7 @@ struct gretl_option gretl_opts[] = {
     { COINT,    OPT_R, "ctt", 0 },     
     { COINT,    OPT_S, "skip-df", 0 },
     { COINT,    OPT_T, "ct", 0 },
+    { COINT,    OPT_V, "verbose", 0 },    
     { COINT2,   OPT_A, "crt", 0 },
     { COINT2,   OPT_D, "seasonals", 0 },
     { COINT2,   OPT_N, "nc", 0 },
@@ -141,6 +147,7 @@ struct gretl_option gretl_opts[] = {
     { CORR,     OPT_S, "spearman", 0 },
     { CORR,     OPT_U, "uniform", 0 },
     { CORR,     OPT_V, "verbose", 0 },
+    { CORRGM,   OPT_G, "gui-graph", 0 },
     { CUSUM,    OPT_R, "squares", 0 },
     { DATA,     OPT_O, "odbc", 0 },
     { DATAMOD,  OPT_P, "preserve", 0 },
@@ -166,22 +173,18 @@ struct gretl_option gretl_opts[] = {
     { DURATION, OPT_M, "medians", 0 },    
     { DURATION, OPT_G, "opg", 0 },
     { DURATION, OPT_R, "robust", 0 },
+    { DURATION, OPT_C, "cluster", 2 },
     { DURATION, OPT_V, "verbose", 0 },
-#if 0 /* not yet */
-    { EQNPRINT, OPT_F, "filename", 0 }, /* backward compatibility */
-#endif
     { EQNPRINT, OPT_O, "complete", 0 },
     { EQNPRINT, OPT_T, "t-ratios", 0 },
     { TABPRINT, OPT_O, "complete", 0 },
     { TABPRINT, OPT_R, "rtf", 0 },
     { TABPRINT, OPT_T, "format", 2 },
-#if 0 /* not yet */
-    { TABPRINT, OPT_F, "filename", 0 }, /* backward compatibility */
-#endif
     { EQUATION, OPT_M, "multi", 0 },
     { ESTIMATE, OPT_I, "iterate", 0 },
     { ESTIMATE, OPT_M, "geomean", 0 },
     { ESTIMATE, OPT_N, "no-df-corr", 0 },
+    { ESTIMATE, OPT_U, "unrestrict-init", 0 },
     { ESTIMATE, OPT_V, "verbose", 0 },
     { FCAST,    OPT_D, "dynamic", 0 },
     { FCAST,    OPT_M, "mean-y", 0 },
@@ -216,13 +219,14 @@ struct gretl_option gretl_opts[] = {
     { GMM,      OPT_T, "two-step", 0 },
     { GMM,      OPT_V, "verbose", 0 },
     { GNUPLOT,  OPT_D, "input", 2 },
-    { GNUPLOT,  OPT_O, "with-lines", 0 },
+    { GNUPLOT,  OPT_O, "with-lines", 1 },
     { GNUPLOT,  OPT_I, "inverse-fit", 0 },
     { GNUPLOT,  OPT_L, "loess-fit", 0 },
     { GNUPLOT,  OPT_Q, "quadratic-fit", 0 },
     { GNUPLOT,  OPT_N, "linear-fit", 0 },
     { GNUPLOT,  OPT_B, "cubic-fit", 0 },
-    { GNUPLOT,  OPT_M, "with-impulses", 0 },
+    { GNUPLOT,  OPT_E, "semilog-fit", 0 },
+    { GNUPLOT,  OPT_M, "with-impulses", 1 },
     { GNUPLOT,  OPT_S, "suppress-fitted", 0 },
     { GNUPLOT,  OPT_T, "time-series", 0 },
     { GNUPLOT,  OPT_Z, "dummy", 0 },
@@ -230,12 +234,15 @@ struct gretl_option gretl_opts[] = {
     { GNUPLOT,  OPT_U, "output", 2 },
     { GNUPLOT,  OPT_Y, "single-yaxis", 0 },
     { GNUPLOT,  OPT_X, "matrix", 2 },
+    { GRAPHPG,  OPT_M, "monochrome", 0 },
+    { GRAPHPG,  OPT_O, "output", 2 },
     { HECKIT,   OPT_M, "ml", 0 },
     { HECKIT,   OPT_R, "robust", 0 },
     { HECKIT,   OPT_T, "two-step", 0 },
     { HECKIT,   OPT_V, "verbose", 0 },
     { HELP,     OPT_F, "func", 0 },
     { INTREG,   OPT_R, "robust", 0 },
+    { INTREG,   OPT_C, "cluster", 2 },    
     { INTREG,   OPT_V, "verbose", 0 },
     { IVREG,    OPT_G, "gmm", 0 },
     { IVREG,    OPT_I, "iterate", 0 },
@@ -245,6 +252,7 @@ struct gretl_option gretl_opts[] = {
     { IVREG,    OPT_T, "two-step", 0 },
     { IVREG,    OPT_W, "weights", 2 },
     { IVREG,    OPT_X, "no-tests", 0 },
+    { IVREG,    OPT_C, "cluster", 2 },
     { KALMAN,   OPT_C, "cross", 0 },
     { KALMAN,   OPT_D, "diffuse", 0 },
     { KPSS,     OPT_T, "trend", 0 },
@@ -268,10 +276,12 @@ struct gretl_option gretl_opts[] = {
     { MODTEST,  OPT_W, "white", 0 },
     { MODTEST,  OPT_X, "white-nocross", 0 },
     { LABELS,   OPT_F, "from-file", 2 },
-    { LABELS,   OPT_T, "to_file", 2 },
+    { LABELS,   OPT_T, "to-file", 2 },
+    { LOGISTIC, OPT_M, "ymax", 2 },
     { LOGIT,    OPT_M, "multinomial", 0 },
     { LOGIT,    OPT_P, "p-values", 0 },
     { LOGIT,    OPT_R, "robust", 0 },
+    { LOGIT,    OPT_C, "cluster", 2 },
     { LOGIT,    OPT_V, "verbose", 0 },
     { LOOP,     OPT_P, "progressive", 0 },
     { LOOP,     OPT_V, "verbose", 0 },
@@ -295,6 +305,7 @@ struct gretl_option gretl_opts[] = {
     { NEGBIN,   OPT_G, "opg", 0 },
     { NEGBIN,   OPT_M, "model1", 0 },
     { NEGBIN,   OPT_R, "robust", 0 },
+    { NEGBIN,   OPT_C, "cluster", 2 },
     { NEGBIN,   OPT_V, "verbose", 0 },
     { NLS,      OPT_N, "numerical", 0 },
     { NLS,      OPT_O, "vcv", 0 },
@@ -311,20 +322,23 @@ struct gretl_option gretl_opts[] = {
     { OLS,      OPT_N, "no-df-corr", 0 },
     { OLS,      OPT_O, "vcv", 0 }, 
     { OLS,      OPT_R, "robust", 0 },
-    { OLS,      OPT_Q, "quiet", 0 },
+    { OLS,      OPT_Q, "quiet", 0 }, /* note: for the sake of documentation */
     { OLS,      OPT_S, "simple-print", 0 },
     { OLS,      OPT_V, "anova", 0 },
+    { OLS,      OPT_C, "cluster", 2 },
     { OMIT,     OPT_A, "auto", 1 },
     { OMIT,     OPT_B, "both", 0 },
     { OMIT,     OPT_X, "chi-square", 0 },
     { OMIT,     OPT_I, "silent", 0 },
     { OMIT,     OPT_W, "test-only", 0 },
     { OPEN,     OPT_B, "progress-bar", 0 },
-    { OPEN,     OPT_C, "coded", 0 },
     { OPEN,     OPT_D, "drop-empty", 0 },
     { OPEN,     OPT_F, "cols", 2 },    
     { OPEN,     OPT_O, "odbc", 0 },
     { OPEN,     OPT_P, "preserve", 0 },
+    { OPEN,     OPT_R, "rowoffset", 2 },
+    { OPEN,     OPT_C, "coloffset", 2 },
+    { OPEN,     OPT_S, "sheet", 2 },
     { OPEN,     OPT_W, "www", 0 },
     { OUTFILE,  OPT_A, "append", 0 },
     { OUTFILE,  OPT_C, "close", 0 },
@@ -335,6 +349,7 @@ struct gretl_option gretl_opts[] = {
     { PANEL,    OPT_H, "hausman-reg", 0 }, /* backward compatibility */
     { PANEL,    OPT_I, "iterate", 0 },
     { PANEL,    OPT_M, "matrix-diff", 0 },
+    { PANEL,    OPT_N, "nerlove", 0 },
     { PANEL,    OPT_O, "vcv", 0 },
     { PANEL,    OPT_P, "pooled", 0 },
     { PANEL,    OPT_R, "robust", 0 },
@@ -342,11 +357,13 @@ struct gretl_option gretl_opts[] = {
     { PANEL,    OPT_U, "random-effects", 0 },
     { PANEL,    OPT_V, "verbose", 0 },
     { PANEL,    OPT_W, "unit-weights", 0 },
-    { POISSON,  OPT_R, "robust", 0 },    
+    { POISSON,  OPT_R, "robust", 0 },
+    { POISSON,  OPT_C, "cluster", 2 },
     { POISSON,  OPT_V, "verbose", 0 },
     { PCA,      OPT_C, "covariance", 0 },
     { PCA,      OPT_A, "save-all", 0 },
-    { PCA,      OPT_O, "save", 0 },
+    { PCA,      OPT_O, "save", 1 },
+    { PCA,      OPT_Q, "quiet", 0 },
     { PERGM,    OPT_O, "bartlett", 0 },
     { PERGM,    OPT_L, "log", 0 },
     { PERGM,    OPT_R, "radians", 0 },
@@ -358,6 +375,7 @@ struct gretl_option gretl_opts[] = {
     { PRINT,    OPT_N, "no-dates", 0 },
     { PROBIT,   OPT_P, "p-values", 0 },
     { PROBIT,   OPT_R, "robust", 0 },
+    { PROBIT,   OPT_C, "cluster", 2 },
     { PROBIT,   OPT_V, "verbose", 0 },
     { QQPLOT,   OPT_R, "raw", 0 },
     { QQPLOT,   OPT_Z, "z-scores", 0 },      
@@ -375,14 +393,19 @@ struct gretl_option gretl_opts[] = {
     { RESTRICT, OPT_N, "no-scaling", 0 },
     { RESTRICT, OPT_S, "silent", 0 },
     { RESTRICT, OPT_W, "wald", 0 },
+    { RMPLOT,   OPT_T, "trim", 0 },
     { RUNS,     OPT_D, "difference", 0 },
     { RUNS,     OPT_E, "equal", 0 },
     { SCATTERS, OPT_L, "with-lines", 0 },
+    { SCATTERS, OPT_T, "time-series", 0 },
     { SCATTERS, OPT_U, "output", 2 },
+    { SCATTERS, OPT_X, "matrix", 2 },
     { SET,      OPT_F, "from-file", 2 },
     { SET,      OPT_T, "to-file", 2 },
     { SETINFO,  OPT_C, "continuous", 0 },
     { SETINFO,  OPT_D, "discrete", 0 },
+    { SETINFO,  OPT_I, "description", 2 },
+    { SETINFO,  OPT_G, "graph-name", 2 },
     { SETOBS,   OPT_C, "stacked-cross-section", 0 },
     { SETOBS,   OPT_P, "panel-vars", 0 },
     { SETOBS,   OPT_R, "restructure", 0 },
@@ -413,17 +436,16 @@ struct gretl_option gretl_opts[] = {
     { STORE,    OPT_Z, "gzipped", 0 },
     { STORE,    OPT_X, "omit-obs", 0 },
     { STORE,    OPT_E, "comment", 2 },
+    { STORE,    OPT_I, "decimal-comma", 0 },
     { SUMMARY,  OPT_B, "by", 2 },
     { SUMMARY,  OPT_M, "matrix", 2 },
     { SUMMARY,  OPT_S, "simple", 0 },
     { SYSTEM,   OPT_I, "iterate", 0 },
     { SYSTEM,   OPT_V, "verbose", 0 },
-#if 0
-    { TABPRINT, OPT_F, "filename", 0 }, /* backward compatibility */
-#endif
     { TOBIT,    OPT_L, "llimit", 2 },
     { TOBIT,    OPT_M, "rlimit", 2 },
     { TOBIT,    OPT_R, "robust", 0 },
+    { TOBIT,    OPT_C, "cluster", 2 },
     { TOBIT,    OPT_V, "verbose", 0 },
     { VAR,      OPT_D, "seasonals", 0 },
     { VAR,      OPT_F, "variance-decomp", 0 },
@@ -440,9 +462,10 @@ struct gretl_option gretl_opts[] = {
     { VECM,     OPT_A, "crt", 0 },
     { VECM,     OPT_D, "seasonals", 0 },
     { VECM,     OPT_F, "variance-decomp", 0 },
-    { VECM,     OPT_I, "impulse-responses", 0 },
+    { VECM,     OPT_I, "impulse-responses", 1 },
     { VECM,     OPT_N, "nc", 0 },
     { VECM,     OPT_R, "rc", 0 },
+    { VECM,     OPT_C, "uc", 0 },    
     { VECM,     OPT_T, "ct", 0 },
     { VECM,     OPT_V, "verbose", 0 },
     { VECM,     OPT_X, "silent", 0 },
@@ -454,6 +477,25 @@ struct gretl_option gretl_opts[] = {
     { 0,        0L,    NULL, 0 }
 };
 
+int cluster_option_ok (int ci)
+{
+    int i, found = 0;
+
+    for (i=0; gretl_opts[i].ci; i++) {
+	if (gretl_opts[i].ci == ci) {
+	    if (gretl_opts[i].o == OPT_C &&
+		!strcmp(gretl_opts[i].longopt, "cluster")) {
+		return 1;
+	    }
+	    found = 1;
+	} else if (found) {
+	    break;
+	}
+    }
+
+    return 0;
+}
+
 /* this function is used in compiling the gretl reference
    manual */
 
@@ -752,7 +794,7 @@ static optparm *matching_optparm (int ci, gretlopt opt)
 }
 
 /* for a given @ci, @opt pair, determine its status with regard
-   to a parameter value (not allowed, allowed, or required)
+   to a parameter value: 0 = not allowed, 1 = allowed, 2 = required
 */
 
 static int option_parm_status (int ci, gretlopt opt)
@@ -869,6 +911,43 @@ double get_optval_double (int ci, gretlopt opt)
     return ret;
 }
 
+/**
+ * get_optval_int:
+ * @ci: gretl command index.
+ * @opt: gretl option value.
+ * @err: location to receive error code.
+ *
+ * Returns: the integer ancillary value currently 
+ * associated with option @opt for command @ci, if any,
+ * otherwise 0 (with a non-zero value written to @err).
+ */
+
+int get_optval_int (int ci, gretlopt opt, int *err)
+{
+    optparm *op = matching_optparm(ci, opt);
+    int status = option_parm_status(ci, opt);
+    int ret = 0;
+
+    if (op != NULL && op->val != NULL) {
+	if (integer_string(op->val)) {
+	    ret = atoi(op->val);
+	} else {
+	    double x = gretl_scalar_get_value(op->val);
+
+	    if (na(x)) {
+		*err = E_DATA;
+	    } else {
+		ret = (int) x;
+	    }
+	}
+    } else if (status == 2) {
+	/* parameter is required */
+	*err = E_DATA;
+    }
+
+    return ret;
+}
+
 /* called via GUI */
 
 /**
@@ -894,6 +973,23 @@ int set_optval_double (int ci, gretlopt opt, double x)
     return real_push_option_param(ci, opt, gretl_strdup(s), 1);
 }
 
+/**
+ * set_optval_string:
+ * @ci: gretl command index.
+ * @opt: gretl option value.
+ * @s: value to set.
+ *
+ * Sets a ancillary string value to be associated
+ * with option @opt for command @ci.
+ *
+ * Returns: 0 on success, non-zero on failure.
+ */
+
+int set_optval_string (int ci, gretlopt opt, const char *s)
+{
+    return real_push_option_param(ci, opt, gretl_strdup(s), 1);
+}
+
 gretlopt valid_long_opt (int ci, const char *lopt, OptStatus *status)
 {
     int opt = OPT_NONE;
@@ -949,10 +1045,6 @@ gretlopt valid_long_opt (int ci, const char *lopt, OptStatus *status)
     return opt;
 }
 
-#define data_open_special(s) (!strcmp(s, "sheet") || \
-                              !strcmp(s, "coloffset") || \
-			      !strcmp(s, "rowoffset"))
-
 /* extract an option parameter value following '=' */
 
 static int handle_optval (char *s, int ci, gretlopt opt, int status)
@@ -1032,9 +1124,6 @@ static gretlopt get_long_opts (char *line, int ci, int *err)
 	    if (match > 0) {
 		/* recognized an acceptable option flag */
 		ret |= match;
-	    } else if ((ci == OPEN || ci == APPEND) && 
-		       data_open_special(longopt)) {
-		; /* no-op here: handled elsewhere (FIXME) */
 	    } else {
 		/* not a valid flag, or not applicable in context */
 		gretl_errmsg_sprintf(_("Invalid option '--%s'"), longopt);
@@ -1155,7 +1244,7 @@ static int end_block_ci (const char *s)
 
 gretlopt get_gretl_options (char *line, int *err)
 {
-    gretlopt oflags = 0L;
+    gretlopt oflags = 0;
     gretlopt opt;
     char cmdword[9] = {0};
     int endblock = 0;
@@ -1166,11 +1255,16 @@ gretlopt get_gretl_options (char *line, int *err)
     }
 
     if (strlen(line) < 2 || *line == '#') {
-	return oflags;
+	return 0;
     }
 
     get_cmdword(line, cmdword);
 
+    if (!strcmp(cmdword, "catch")) {
+	*cmdword = '\0';
+	get_cmdword(line + 6, cmdword);
+    }
+
     if (!strcmp(cmdword, "end")) {
 	endblock = 1;
 	ci = end_block_ci(line);
@@ -1304,7 +1398,9 @@ const char *print_flags (gretlopt oflags, int ci)
 	    if (gretl_opts[i].parminfo) {
 		parm = get_optval_string(ci, opt);
 		if (parm != NULL && *parm != '\0') {
-		    if (quote_option_parm(ci, opt, parm)) {
+		    if (strchr(parm, ' ')) {
+			pprintf(flagprn, "=\"%s\"", parm);
+		    } else if (quote_option_parm(ci, opt, parm)) {
 			pprintf(flagprn, "=\"%s\"", parm);
 		    } else {
 			pprintf(flagprn, "=%s", parm);
