@@ -219,14 +219,14 @@ void print_centered (const char *s, int width, PRN *prn)
 }
 
 /**
- * max_obs_label_length:
+ * max_obs_marker_length:
  * @dset: dataset information.
  *
- * Returns: the length of the longest observation label
+ * Returns: the length of the longest observation marker
  * within the current sample range.
  */
 
-int max_obs_label_length (const DATASET *dset)
+int max_obs_marker_length (const DATASET *dset)
 {
     char s[OBSLEN];
     int t, n, nmax = 0;
@@ -707,14 +707,15 @@ char *gretl_fix_exponent (char *s)
     int n;
 
     if ((p = strstr(s, "+00")) || (p = strstr(s, "-00"))) {
-	if (*(p + 3)) {
+	if (*(p+3)) {
 	    memmove(p+1, p+2, strlen(p+1));
 	}
     }
 
     n = strlen(s);
     if (s[n-1] == '.' || s[n-1] == ',') {
-	s[n-1] = 0;
+	/* delete trailing junk */
+	s[n-1] = '\0';
     }
 
     return s;
@@ -824,6 +825,8 @@ void gretl_sprint_fullwidth_double (double x, int digits, char *targ,
     char decpoint;
     int n;
 
+    *targ = '\0';
+
     if (na(x)) {
 	strcpy(targ, "NA");
 	return;
@@ -850,12 +853,12 @@ void gretl_sprint_fullwidth_double (double x, int digits, char *targ,
 
     n = strlen(targ) - 1;
     if (targ[n] == decpoint) {
-	targ[n] = 0;
+	targ[n] = '\0';
     }
 
     cut_extra_zero(targ, digits);
 
-    if (*targ == '-' && gretl_print_supports_utf(prn)) {
+    if (*targ == '-' && gretl_print_has_minus(prn)) {
 	char tmp[36];
 
 	strcpy(tmp, targ + 1);
@@ -950,8 +953,8 @@ print_contemp_covariance_matrix (const gretl_matrix *m,
 
     if (tex) {
 	pputs(prn, "\\begin{center}\n");
-	pprintf(prn, "%s \\\\\n", I_("Cross-equation VCV for residuals"));
-	pprintf(prn, "(%s)\n\n", I_("correlations above the diagonal"));
+	pprintf(prn, "%s \\\\\n", A_("Cross-equation VCV for residuals"));
+	pprintf(prn, "(%s)\n\n", A_("correlations above the diagonal"));
 	pputs(prn, "\\[\n\\begin{array}{");
 	for (j=0; j<cols; j++) {
 	    pputc(prn, 'c');
@@ -996,10 +999,10 @@ print_contemp_covariance_matrix (const gretl_matrix *m,
     if (!na(ldet)) {
 	if (tex) {
 	    if (ldet < 0) {
-		pprintf(prn, "\n%s = ", I_("log determinant"));
+		pprintf(prn, "\n%s = ", A_("log determinant"));
 		pprintf(prn, "$-$%g\n", -ldet);
 	    } else {
-		pprintf(prn, "\n%s = %g\n", I_("log determinant"), ldet);
+		pprintf(prn, "\n%s = %g\n", A_("log determinant"), ldet);
 	    }
 	} else {
 	    pprintf(prn, "\n%s = %g\n", _("log determinant"), ldet);
@@ -1153,7 +1156,7 @@ void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 
 static void fit_resid_head (const FITRESID *fr, 
 			    const DATASET *dset, 
-			    PRN *prn)
+			    int obslen, PRN *prn)
 {
     char label[16];
     char obs1[OBSLEN], obs2[OBSLEN];
@@ -1184,8 +1187,9 @@ static void fit_resid_head (const FITRESID *fr,
 		    GRETL_DIGITS, fr->sigma);
 	}
     }
-    
-    pputs(prn, "\n         ");
+
+    pputc(prn, '\n');
+    bufspace(obslen, prn);
 
     for (i=1; i<4; i++) {
 	if (i == 1) strcpy(label, fr->depvar);
@@ -1206,12 +1210,14 @@ static void varheading (const int *list, int leader, int wid,
     int i, vi;
 
     if (delim) {
-	pprintf(prn, "obs%c", delim);
+	if (leader >= 0) {
+	    pprintf(prn, "obs%c", delim);
+	}
 	for (i=1; i<=list[0]; i++) { 
 	    vi = list[i];
 	    pputs(prn, dset->varname[vi]);
 	    if (i < list[0]) {
-		pputc(prn, dset->delim);
+		pputc(prn, delim);
 	    } 
 	}
 	pputc(prn, '\n');
@@ -1543,36 +1549,23 @@ static char *bufprintnum (char *buf, double x, int signif,
     return buf;
 }
 
-/**
- * obs_marker_init:
- * @dset: data information struct.
- *
- * Check the length to which observation markers should
- * be printed, in a tabular context.  (We don't want to
- * truncate 10-character date strings by chopping off the day.)
- */
-
-static int oprintlen = 8;
-
-void obs_marker_init (const DATASET *dset)
+static void real_print_obs_marker (int t, const DATASET *dset, 
+				   int len, int pad, PRN *prn)
 {
-    int t, datestrs = 0;
+    char tmp[OBSLEN] = {0};
+    int thislen = len;
 
-    if (dset->markers) {
-	for (t=0; t<dset->n; t++) {
-	    if (strlen(dset->S[t]) == 10 && 
-		isdigit(dset->S[t][0]) &&
-		strchr(dset->S[t], '/')) {
-		datestrs = 1;
-		break;
-	    }
-	}
-    } 
-
-    if (datestrs) {
-	oprintlen = 10;
+    if (dataset_has_markers(dset)) {
+	strcpy(tmp, dset->S[t]);
+	thislen = get_utf_width(tmp, len);
     } else {
-	oprintlen = 8;
+	ntodate(tmp, t, dset);
+    }
+
+    if (pad) {
+	pprintf(prn, "%*s ", thislen, tmp);
+    } else {
+	pprintf(prn, "%*s", thislen, tmp);
     }
 }
 
@@ -1580,22 +1573,15 @@ void obs_marker_init (const DATASET *dset)
  * print_obs_marker:
  * @t: observation number.
  * @dset: data information struct.
+ * @len: width to which to print.
  * @prn: gretl printing struct.
  *
  * Print a string (label, date or obs number) representing the given @t.
  */
 
-void print_obs_marker (int t, const DATASET *dset, PRN *prn)
+void print_obs_marker (int t, const DATASET *dset, int len, PRN *prn)
 {
-    char tmp[OBSLEN] = {0};
-
-    if (dset->markers) { 
-	strncat(tmp, dset->S[t], oprintlen);
-	pprintf(prn, "%*s ", oprintlen, tmp); 
-    } else {
-	ntodate(tmp, t, dset);
-	pprintf(prn, "%8s ", tmp);
-    }
+    real_print_obs_marker(t, dset, len, 1, prn);
 }
 
 /**
@@ -1707,8 +1693,9 @@ static void print_listed_objects (const char *s,
     const char *p;
     gretl_bundle *b;
     char *name;
+    int err = 0;
 
-    while ((name = gretl_word_strdup(s, &s)) != NULL) {
+    while ((name = gretl_word_strdup(s, &s, OPT_NONE, &err)) != NULL) {
 	if (gretl_is_scalar(name)) {
 	    print_scalar_by_name(name, prn);
 	} else if ((m = get_matrix_by_name(name)) != NULL) {
@@ -1720,7 +1707,6 @@ static void print_listed_objects (const char *s,
 	} else if ((p = get_string_by_name(name)) != NULL) {
 	    pprintf(prn, "%s\n", p);	
 	} 
-	    
 	free(name);
     }
 }
@@ -1817,9 +1803,8 @@ static int print_by_obs (int *list,
 			 PRN *prn)
 {
     int i, j, j0, k, t, nrem;
-    int colwidth, obslen;
+    int colwidth, obslen = 0;
     int *pmax = NULL;
-    char obslabel[OBSLEN];
     char buf[128];
     int blist[BMAX+1];
     int gprec = 6;
@@ -1834,7 +1819,7 @@ static int print_by_obs (int *list,
     if (opt & OPT_N) {
 	obslen = obslen_from_t(dset->t2);
     } else {
-	obslen = max_obs_label_length(dset);
+	obslen = max_obs_marker_length(dset);
     }
 
     colwidth = column_width_from_list(list, dset);
@@ -1855,24 +1840,15 @@ static int print_by_obs (int *list,
 	varheading(blist, obslen, colwidth, dset, 0, prn);
 	
 	for (t=dset->t1; t<=dset->t2; t++) {
-	    int thislen = obslen;
-
 	    if (screenvar && dset->Z[screenvar][t] == 0.0) {
 		/* screened out by boolean */
 		continue;
 	    }
-
 	    if (opt & OPT_N) {
-		sprintf(obslabel, "%d", t + 1);
-	    } else if (dataset_has_markers(dset)) {
-		strcpy(obslabel, dset->S[t]);
-		thislen = get_utf_width(obslabel, obslen);
+		pprintf(prn, "%*d", obslen, t + 1);
 	    } else {
-		ntodate(obslabel, t, dset);
+		real_print_obs_marker(t, dset, obslen, 0, prn);
 	    }
-
-	    pprintf(prn, "%*s", thislen, obslabel);
-
 	    for (i=1, j=j0; i<=blist[0]; i++, j++) {
 		x = dset->Z[blist[i]][t];
 		if (na(x)) {
@@ -1882,7 +1858,6 @@ static int print_by_obs (int *list,
 		    pputs(prn, buf);
 		}
 	    }
-
 	    pputc(prn, '\n');
 	} 
     } 
@@ -2067,7 +2042,7 @@ int print_series_with_format (const int *list,
 	sprintf(format, "%%#.%dg", digits);
     }
 
-    obslen = max_obs_label_length(dset);
+    obslen = max_obs_marker_length(dset);
 
     k = 1;
 
@@ -2143,13 +2118,13 @@ static void rtf_print_row_spec (int ncols, int type, PRN *prn)
     }    
 }
 
-#define prn_delimited(p) (prn_format(p) & (GRETL_FORMAT_CSV | GRETL_FORMAT_TAB))
-
 /**
  * print_data_in_columns:
  * @list: list of variables to print.
  * @obsvec: list of observation numbers (or %NULL)
  * @dset: dataset struct.
+ * @opt: may include OPT_X to exclude the observations
+ * column that is usually printed first.
  * @prn: gretl printing struct.
  *
  * Print the data for the variables in @list.  If @obsvec is not %NULL,
@@ -2164,10 +2139,13 @@ static void rtf_print_row_spec (int ncols, int type, PRN *prn)
  */
 
 int print_data_in_columns (const int *list, const int *obsvec, 
-			   const DATASET *dset, PRN *prn)
+			   const DATASET *dset, gretlopt opt,
+			   PRN *prn)
 {
-    int delimited = prn_delimited(prn);
+    int csv = csv_format(prn);
     int rtf = rtf_format(prn);
+    const char *csv_na = "";
+    int print_obs = 1;
     char delim = 0;
     int *pmax = NULL; 
     double xx;
@@ -2184,7 +2162,7 @@ int print_data_in_columns (const int *list, const int *obsvec,
 	T = sample_size(dset);
     }
 
-    /* must have a non-empty list of variables */
+    /* we must have a non-empty list of variables */
     if (list == NULL || list[0] < 1) {
 	return E_DATA;
     }
@@ -2206,19 +2184,23 @@ int print_data_in_columns (const int *list, const int *obsvec,
 	return E_ALLOC;
     }
 
-    if (delimited) {
-	if (csv_format(prn)) {
-	    gprec = 15;
-	}
-	delim = dset->delim;
+    if (csv) {
+	/* columns delimited by some character */
+	gprec = 15;
+	delim = get_data_export_delimiter();
 	if (get_local_decpoint() == ',' && delim == ',') {
 	    delim = ';';
 	}
+	csv_na = get_csv_na_string();
+	if (opt & OPT_X) {
+	    print_obs = 0;
+	    obslen = -1;
+	}
     } else if (rtf) {
 	ncols = list[0] + 1;
     } else {
 	colwidth = column_width_from_list(list, dset);
-	obslen = max_obs_label_length(dset);
+	obslen = max_obs_marker_length(dset);
     }
 
     if (rtf) {
@@ -2234,31 +2216,32 @@ int print_data_in_columns (const int *list, const int *obsvec,
 
     /* print data by observations */
     for (s=0; s<T; s++) {
-	if (obsvec != NULL) {
-	    t = obsvec[s+1];
-	} else {
-	    t = dset->t1 + s;
-	}
+	t = (obsvec != NULL)? obsvec[s+1] : (dset->t1 + s);
 	if (t >= dset->n) {
 	    continue;
 	}
+
 	if (rtf) {
 	    rtf_print_row_spec(ncols, RTF_HEADER, prn);
 	    pputc(prn, '{');
 	}
-	get_obs_string(obs_string, t, dset);
-	if (delimited) {
-	    pprintf(prn, "%s%c", obs_string, delim);
-	} else if (rtf) {
-	    pprintf(prn, "%s\\cell ", obs_string);
-	} else {
-	    pprintf(prn, "%*s", obslen, obs_string);
+
+	if (print_obs) {
+	    get_obs_string(obs_string, t, dset);
+	    if (csv) {
+		pprintf(prn, "%s%c", obs_string, delim);
+	    } else if (rtf) {
+		pprintf(prn, "%s\\cell ", obs_string);
+	    } else {
+		pprintf(prn, "%*s", obslen, obs_string);
+	    }
 	}
+
 	for (i=1; i<=list[0]; i++) {
 	    xx = dset->Z[list[i]][t];
 	    if (na(xx)) {
-		if (delimited) {
-		    pputs(prn, "NA");
+		if (csv) {
+		    pputs(prn, csv_na);
 		} else if (rtf) {
 		    pputs(prn, "\\qr NA\\cell ");
 		} else {
@@ -2273,7 +2256,7 @@ int print_data_in_columns (const int *list, const int *obsvec,
 		    pputs(prn, buf);
 		}
 	    }
-	    if (delimited && i < list[0]) {
+	    if (csv && i < list[0]) {
 		pputc(prn, delim);
 	    }
 	}
@@ -2379,14 +2362,14 @@ int text_print_fit_resid (const FITRESID *fr, const DATASET *dset,
     int kstep = fr->method == FC_KSTEP;
     int t, anyast = 0;
     double yt, yf, et;
+    int obslen;
     int err = 0;
 
-    fit_resid_head(fr, dset, prn); 
-
-    obs_marker_init(dset);
+    obslen = max_obs_marker_length(dset);
+    fit_resid_head(fr, dset, obslen, prn); 
 
     for (t=fr->t1; t<=fr->t2; t++) {
-	print_obs_marker(t, dset, prn);
+	real_print_obs_marker(t, dset, obslen, 0, prn);
 
 	yt = fr->actual[t];
 	yf = fr->fitted[t];
@@ -2415,7 +2398,6 @@ int text_print_fit_resid (const FITRESID *fr, const DATASET *dset,
 		    anyast = 1;
 		}
 	    }
-
 	    if (fr->pmax != PMAX_NOT_AVAILABLE) {
 		pprintf(prn, "%13.*f%13.*f%13.*f%s\n", 
 			fr->pmax, yt, fr->pmax, yf, fr->pmax, et,
@@ -2467,7 +2449,7 @@ int text_print_forecast (const FITRESID *fr, DATASET *dset,
 			 gretlopt opt, PRN *prn)
 {
     int do_errs = (fr->sderr != NULL);
-    int pmax = fr->pmax;
+    int obslen, pmax = fr->pmax;
     int errpmax = fr->pmax;
     int quiet = (opt & OPT_Q);
     double *maxerr = NULL;
@@ -2502,11 +2484,16 @@ int text_print_forecast (const FITRESID *fr, DATASET *dset,
 	}
     }
 
+    obslen = max_obs_marker_length(dset);
+    if (obslen < 8) {
+	obslen = 8;
+    }
+
     if (!quiet) {
 	pputc(prn, '\n');
     }
 
-    pprintf(prn, "     %s ", _("Obs"));
+    bufspace(obslen + 1, prn);
     pprintf(prn, "%12s", fr->depvar);
     pprintf(prn, "%*s", UTF_WIDTH(_("prediction"), 14), _("prediction"));
 
@@ -2528,10 +2515,8 @@ int text_print_forecast (const FITRESID *fr, DATASET *dset,
 	}
     }
 
-    obs_marker_init(dset);
-
     for (t=fr->t0; t<=fr->t2; t++) {
-	print_obs_marker(t, dset, prn);
+	print_obs_marker(t, dset, obslen, prn);
 	fcast_print_x(fr->actual[t], 15, pmax, prn);
 
 	if (na(fr->fitted[t])) {
@@ -2667,27 +2652,33 @@ print_iter_info (int iter, double crit, int type, int k,
     }	
 
     pputc(prn, '\n');
-	
-    pputs(prn, _("Parameters: "));
-    for (i=0; i<k; i++) {
-	print_iter_val(b[i], i, k, prn);
-    }
-    pputc(prn, '\n');
 
-    pputs(prn, _("Gradients:  "));
-    x = 0.0;
-    for (i=0; i<k; i++) {
-	x += fabs(b[i] * g[i]);
-	print_iter_val(g[i], i, k, prn);
-    }
-    pprintf(prn, " (norm %.2e)\n", sqrt(x/k));
-    if (iter >= 0) {
+    if (b != NULL) {
+	pputs(prn, _("Parameters: "));
+	for (i=0; i<k; i++) {
+	    print_iter_val(b[i], i, k, prn);
+	}
 	pputc(prn, '\n');
     }
 
-    if (iter < 0 || (iter % 20 == 0)) {
-	/* experimental */
-	iter_print_callback((iter < 0)? 0 : iter, prn);
+    if (g != NULL) {
+	pputs(prn, _("Gradients:  "));
+	x = 0.0;
+	for (i=0; i<k; i++) {
+	    x += fabs(b[i] * g[i]);
+	    print_iter_val(g[i], i, k, prn);
+	}
+	pprintf(prn, " (%s %.2e)\n", _("norm"), sqrt(x/k));
+	if (iter >= 0) {
+	    pputc(prn, '\n');
+	}
+    }
+
+    if (b != NULL && g != NULL) {
+	if (iter < 0 || (iter % 20 == 0)) {
+	    /* experimental */
+	    iter_print_callback((iter < 0)? 0 : iter, prn);
+	}
     }
 }
 
@@ -2738,32 +2729,42 @@ static readbuf *matching_buffer (const char *s)
     return NULL;
 }
 
-static int rbuf_push (const char *s)
+/**
+ * bufgets_init:
+ * @buf: source buffer.
+ *
+ * Initializes a text buffer for use with bufgets(). 
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int bufgets_init (const char *buf)
 {
     readbuf *tmp = NULL;
     int i, err = 0;
 
-    tmp = matching_buffer(s);
+    tmp = matching_buffer(buf);
     if (tmp != NULL) {
 	fprintf(stderr, "GRETL ERROR: buffer at %p is already "
-		"initialized\n", (void *) s);
+		"initialized\n", (void *) buf);
 	return 1;
     }
 
     for (i=0; i<n_bufs; i++) {
 	if (rbuf[i].start == NULL) {
-	    /* re-use existing slot */
-	    rbuf[i].start = rbuf[i].point = s;
+	    /* OK, re-use an existing slot */
+	    rbuf[i].start = rbuf[i].point = buf;
 	    return 0;
 	}
     }    
 
     tmp = realloc(rbuf, (n_bufs + 1) * sizeof *tmp);
+
     if (tmp == NULL) {
 	err = E_ALLOC;
     } else {
 	rbuf = tmp;
-	rbuf[n_bufs].start = rbuf[n_bufs].point = s;
+	rbuf[n_bufs].start = rbuf[n_bufs].point = buf;
 	n_bufs++;
     }
 
@@ -2786,9 +2787,16 @@ static void rbuf_set_point (const char *s, const char *p)
     }
 }
 
-static void rbuf_finalize (const char *s)
+/**
+ * bufgets_finalize:
+ * @buf: source buffer.
+ *
+ * Signals that we are done reading from @buf.
+ */
+
+void bufgets_finalize (const char *buf)
 {
-    readbuf *rbuf = matching_buffer(s);
+    readbuf *rbuf = matching_buffer(buf);
 
     if (rbuf != NULL) {
 	rbuf->start = rbuf->point = NULL;
@@ -2821,18 +2829,6 @@ char *bufgets (char *s, size_t size, const char *buf)
     int i, status = 0;
     const char *p;
 
-    if (s == NULL && size == 1) {
-	/* signal for end-of-read */
-	rbuf_finalize(buf);
-	return NULL;
-    }
-
-    if (s == NULL || size == 0) {
-	/* signal for initialization */
-	rbuf_push(buf);
-	return NULL;
-    }
-
     p = rbuf_get_point(buf);
     if (p == NULL) {
 	return NULL;
@@ -2944,28 +2940,13 @@ long buftell (const char *buf)
     return (rbuf == NULL)? 0 : rbuf->point - rbuf->start;
 }
 
-/**
- * bufgets_init:
- * @buf: source buffer.
- *
- * Initializes a text buffer for use with bufgets(). 
- */
+/* for internal use */
 
-void bufgets_init (const char *buf)
+void bufgets_cleanup (void)
 {
-    bufgets(NULL, 0, buf);
+    if (n_bufs > 0) {
+	free(rbuf);
+	rbuf = NULL;
+	n_bufs = 0;
+    }
 }
-
-/**
- * bufgets_finalize:
- * @buf: source buffer.
- *
- * Signals that we are done reading from @buf.
- */
-
-void bufgets_finalize (const char *buf)
-{
-    bufgets(NULL, 1, buf);
-}
-
-
