@@ -41,28 +41,23 @@ static void printf_series (int v, const DATASET *dset,
 			   int wstar, int pstar,
 			   PRN *prn)
 {
-    char label[OBSLEN];
+    int obslen = max_obs_marker_length(dset);
     double x;
-    int n, t;
-
-    n = max_obs_label_length(dset) + 1;
+    int t;
 
     for (t=dset->t1; t<=dset->t2; t++) {
-	get_obs_string(label, t, dset);
-	pprintf(prn, "%*s ", n, label);
+	print_obs_marker(t, dset, obslen, prn);
 	x = dset->Z[v][t];
-	if (na(x)) {
-	    pputc(prn, '\n');
-	    continue;
-	}
-	x *= (1 + 0x1p-52);
-	if (wstar && pstar) {
-	    pprintf(prn, fmt, wid, prec, x);
-	} else if (wstar || pstar) {
-	    wid = (wstar)? wid : prec;
-	    pprintf(prn, fmt, wid, x);
-	} else {
-	    pprintf(prn, fmt, x);
+	if (!na(x)) {
+	    x *= (1 + 0x1p-52);
+	    if (wstar && pstar) {
+		pprintf(prn, fmt, wid, prec, x);
+	    } else if (wstar || pstar) {
+		wid = (wstar)? wid : prec;
+		pprintf(prn, fmt, wid, x);
+	    } else {
+		pprintf(prn, fmt, x);
+	    }
 	}
 	pputc(prn, '\n');
     }
@@ -259,7 +254,7 @@ static char *get_next_arg (const char *s, int *len, int *err)
 	    *err = E_ALLOC;
 	}
     } else {
-	*err = E_PARSE;
+	*err = E_ARGS;
     }
 
 #if PSDEBUG
@@ -628,10 +623,12 @@ static int split_printf_line (const char *s, char *targ, int *sp,
     return 0;
 }
 
+/* supports both printf and sprintf */
+
 static int real_do_printf (const char *line, DATASET *dset, 
 			   PRN *inprn, int t)
 {
-    PRN *prn = inprn;
+    PRN *prn = NULL;
     char targ[VNAMELEN];
     char *format = NULL;
     char *args = NULL;
@@ -646,16 +643,18 @@ static int real_do_printf (const char *line, DATASET *dset,
 	return err;
     }
 
-    if (sp) {
-	/* "sprintf" */
-	if (*targ == '\0') {
-	    return E_PARSE;
-	}
-	prn = gretl_print_new(GRETL_PRINT_BUFFER, &err);
-	if (err) {
-	    return err;
-	}
-    } 
+    if (sp && *targ == '\0') {
+	/* sprintf: we need a target stringvar */
+	return E_PARSE;
+    }    
+
+    /* Even for printf we'll buffer the output locally in 
+       case there's an error part way through the printing.
+    */
+    prn = gretl_print_new(GRETL_PRINT_BUFFER, &err);
+    if (err) {
+	return err;
+    }    
 
 #if PSDEBUG
     fprintf(stderr, "do_printf: line = '%s'\n", line);
@@ -689,16 +688,19 @@ static int real_do_printf (const char *line, DATASET *dset,
 	}
     }
 
-    if (sp) {
-	if (!err) {
-	   err = save_named_string(targ, gretl_print_get_buffer(prn),
-				   inprn);
+    if (!err) {
+	const char *buf = gretl_print_get_buffer(prn);
+
+	if (sp) {
+	    err = save_named_string(targ, buf, inprn);
+	} else {
+	    pputs(inprn, buf);
 	}
-	gretl_print_destroy(prn);
-    } else if (err) {
-	pputc(prn, '\n');
+    } else if (!sp) {
+	pputc(inprn, '\n');
     }
 
+    gretl_print_destroy(prn);
     free(format);
     free(args);
 
@@ -1185,7 +1187,7 @@ static char *get_literal_or_stringvar (const char **src, int *err)
 	ret = tmp;
 	s++;
     } else if (svar) {
-	char *p = get_string_by_name(tmp);
+	const char *p = get_string_by_name(tmp);
 
 	if (p == NULL) {
 	    *err = E_UNKVAR;
