@@ -20,15 +20,12 @@
 #ifndef PRINTSCAN_H
 #define PRINTSCAN_H
 
-int do_printf (const char *line, double ***pZ, 
-	       DATAINFO *pdinfo, PRN *prn);
+int do_printf (const char *line, DATASET *dset, PRN *prn);
 
-int do_sscanf (const char *line, double ***pZ, 
-	       DATAINFO *pdinfo, PRN *prn);
+int do_sscanf (const char *line, DATASET *dset, PRN *prn);
 
 int n_scanned_items (void);
 
-int generate_obs_markers (const char *s, double ***pZ, 
-			  DATAINFO *pdinfo);
+int generate_obs_markers (const char *s, DATASET *dset);
 
 #endif /* PRINTSCAN_H */
