@@ -42,9 +42,9 @@ double normal_pvalue_2 (double x);
 
 double normal_pvalue_1 (double x);
 
-double student_pvalue_2 (int df, double x);
+double student_pvalue_2 (double df, double x);
 
-double student_pvalue_1 (int df, double x);
+double student_pvalue_1 (double df, double x);
 
 double chisq_cdf (int df, double x);
 
@@ -78,12 +78,16 @@ double gamma_cdf (double s1, double s2, double x, int control);
 
 double gamma_cdf_comp (double s1, double s2, double x, int control);
 
+double gamma_cdf_inverse (double shape, double scale, double p);
+
 double GED_pdf (double nu, double x);
 
 double GED_cdf (double nu, double x);
 
 double GED_cdf_comp (double nu, double x);
 
+double GED_cdf_inverse (double nu, double a);
+
 double tcrit95 (int df);
 
 double rhocrit95 (int n);
@@ -107,12 +111,14 @@ double gretl_get_critval (char st, const double *parm, double a);
 double *gretl_get_random_series (char st, const double *parm,
 				 const double *serp1, 
 				 const double *serp2, 
-				 const DATAINFO *pdinfo,
+				 const DATASET *dset,
 				 int *err);
 
-int batch_pvalue (const char *str, 
-		  double ***pZ, DATAINFO *pdinfo, 
-		  PRN *prn);
+gretl_matrix *gretl_get_random_matrix (char st, const double *parm,
+				       int rows, int cols, 
+				       int *err);
+
+int batch_pvalue (const char *str, DATASET *dset, PRN *prn);
 
 void print_pvalue (char st, const double *parm, double x, double pv, PRN *prn);
 
