@@ -36,22 +36,28 @@ enum {
     VCV_XPX
 };
 
+static int qr_make_cluster_vcv (MODEL *pmod, int ci,
+				const DATASET *dset,
+				gretl_matrix *XX,
+				gretlopt opt);
+
 /* General note: in fortran arrays, column entries are contiguous.
    Columns of data matrix X hold variables, rows hold observations.
    So in a fortran array, entries for a given variable are contiguous.
 */
 
-static double qr_get_tss (MODEL *pmod, const double *y, int *ifc,
-			  int *yconst)
+static double qr_get_tss (MODEL *pmod, const DATASET *dset,
+			  int *ifc, int *yconst)
 {
+    int yno = pmod->list[1];
     int pwe = (pmod->opt & OPT_P);
-    double y0 = 0.0, ymean = 0.0;
+    double yt, y0 = 0.0, ymean = 0.0;
     double x, tss = 0.0;
     double ctss = 0.0;
     int t;
 
     if (*ifc == 0) {
-	*ifc = check_for_effective_const(pmod, y);
+	*ifc = check_for_effective_const(pmod, dset);
     }
 
     *yconst = 1;
@@ -60,22 +66,22 @@ static double qr_get_tss (MODEL *pmod, const double *y, int *ifc,
 	double ry, d;
 
 	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    ry = y[t];
+	    ry = dset->Z[yno][t];
 	    if (t == pmod->t1 && pwe) {
 		ry *= sqrt(1.0 - pmod->rho * pmod->rho);
 	    } else {
-		ry -= pmod->rho * y[t-1];
+		ry -= pmod->rho * dset->Z[yno][t-1];
 	    }
 	    ymean += ry;
 	}
 	ymean /= pmod->nobs;
 
 	for (t=pmod->t1; t<=pmod->t2; t++) {
-	    ry = y[t];
+	    ry = dset->Z[yno][t];
 	    if (t == pmod->t1 && pwe) {
 		ry *= sqrt(1.0 - pmod->rho * pmod->rho);
 	    } else {
-		ry -= pmod->rho * y[t-1];
+		ry -= pmod->rho * dset->Z[yno][t-1];
 	    }
 	    if (t == pmod->t1) {
 		y0 = ry;
@@ -93,23 +99,24 @@ static double qr_get_tss (MODEL *pmod, const double *y, int *ifc,
     } else {
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    if (!na(pmod->yhat[t])) {
-		ymean += y[t];
+		ymean += dset->Z[yno][t];
 	    }
 	}
 	ymean /= pmod->nobs;
 
-	y0 = y[pmod->t1];
+	y0 = dset->Z[yno][pmod->t1];
 
 	for (t=pmod->t1; t<=pmod->t2; t++) {
 	    if (!na(pmod->yhat[t])) {
-		if (y[t] != y0) {
+		if (dset->Z[yno][t] != y0) {
 		    *yconst = 0;
 		}
-		x = y[t] - ymean;
+		x = dset->Z[yno][t] - ymean;
 		if (*ifc) {
 		    tss += x * x;
 		} else {
-		    tss += y[t] * y[t];
+		    yt = dset->Z[yno][t];
+		    tss += yt * yt;
 		    ctss += x * x;
 		}
 	    }
@@ -125,16 +132,17 @@ static double qr_get_tss (MODEL *pmod, const double *y, int *ifc,
     return tss;
 }
 
-static void qr_compute_stats (MODEL *pmod, const double *y, int n,
-			      gretlopt opt)
+static void qr_compute_stats (MODEL *pmod, const DATASET *dset,
+			      int n, gretlopt opt)
 {
     int yconst, ifc = pmod->ifc;
+    int yno = pmod->list[1];
 
-    pmod->tss = qr_get_tss(pmod, y, &ifc, &yconst);
+    pmod->tss = qr_get_tss(pmod, dset, &ifc, &yconst);
     pmod->chisq = NADBL;
 
     if (yconst && pmod->dfd > 0) {
-	double y0 = y[pmod->t1];
+	double y0 = dset->Z[yno][pmod->t1];
     
 	if (y0 > 0) {
 	    double tss = pmod->nobs * y0 * y0;
@@ -513,10 +521,10 @@ gretl_matrix *HAC_XOX (const gretl_matrix *uhat, const gretl_matrix *X,
     if (data_based_hac_bandwidth()) {
 	gretl_matrix umat;
 
+	gretl_matrix_init(&umat);
 	umat.rows = T;
 	umat.cols = 1;
 	umat.val = u;
-	umat.t1 = umat.t2 = 0;
 	*err = newey_west_bandwidth(&umat, kern, &p, &bt);
 	if (*err) {
 	    goto bailout;
@@ -612,9 +620,9 @@ static int qr_make_hac (MODEL *pmod, const DATASET *dset,
        into it, equal to the offset of the data on which the model
        is actually estimated.
     */
+    gretl_matrix_init(&umat);
     umat.rows = T;
     umat.cols = 1;
-    umat.t1 = umat.t2 = 0;
     umat.val = pmod->uhat + pmod->t1;
 
     XOX = HAC_XOX(&umat, X, &vi, &err);
@@ -969,8 +977,8 @@ int gretl_qr_regress (MODEL *pmod, DATASET *dset, gretlopt opt)
     gretl_matrix *V = NULL;
     int rank, warn = 0, err = 0;
 
-    T = pmod->nobs;               /* # of rows (observations) */
-    k = pmod->list[0] - 1;        /* # of cols (variables) */
+    T = pmod->nobs;        /* # of rows (observations) */
+    k = pmod->list[0] - 1; /* # of cols (variables) */
 
     Q = gretl_matrix_alloc(T, k);
     R = gretl_matrix_alloc(k, k);
@@ -1054,26 +1062,30 @@ int gretl_qr_regress (MODEL *pmod, DATASET *dset, gretlopt opt)
     /* VCV and standard errors */
     if (opt & OPT_R) { 
 	pmod->opt |= OPT_R;
-	if ((opt & OPT_T) && !libset_get_bool(FORCE_HC)) {
-	    qr_make_hac(pmod, dset, V);
+	if (opt & OPT_C) {
+	    err = qr_make_cluster_vcv(pmod, OLS, dset, V, opt);
+	} else if ((opt & OPT_T) && !libset_get_bool(FORCE_HC)) {
+	    err = qr_make_hac(pmod, dset, V);
 	} else {
-	    qr_make_hccme(pmod, dset, Q, V);
+	    err = qr_make_hccme(pmod, dset, Q, V);
 	}
     } else {
-	qr_make_regular_vcv(pmod, V, opt);
+	err = qr_make_regular_vcv(pmod, V, opt);
     }
 
-    /* get R^2, F */
-    qr_compute_stats(pmod, dset->Z[pmod->list[1]], T, opt);
+    if (!err) {
+	/* get R^2, F-stat */
+	qr_compute_stats(pmod, dset, T, opt);
 
-    /* D-W stat and p-value */
-    if ((opt & OPT_I) && pmod->missmask == NULL) {
-	qr_dw_stats(pmod, dset, Q, y);
-    }
+	/* D-W stat and p-value */
+	if ((opt & OPT_I) && pmod->missmask == NULL) {
+	    qr_dw_stats(pmod, dset, Q, y);
+	}
 
-    /* near-singularity? */
-    if (warn) {
-	gretl_model_set_int(pmod, "near-singular", 1);
+	/* near-singularity? */
+	if (warn) {
+	    gretl_model_set_int(pmod, "near-singular", 1);
+	}
     }
 
  qr_cleanup:
@@ -1098,7 +1110,7 @@ int qr_tsls_vcv (MODEL *pmod, const DATASET *dset, gretlopt opt)
     gretl_matrix *V = NULL;
     int k, err = 0;
 
-    k = pmod->list[0] - 1;        /* # of cols (variables) */
+    k = pmod->list[0] - 1;
 
     Q = make_data_X(pmod, dset);
     R = gretl_matrix_alloc(k, k);
@@ -1121,7 +1133,10 @@ int qr_tsls_vcv (MODEL *pmod, const DATASET *dset, gretlopt opt)
 
     /* VCV and standard errors */
     if (opt & OPT_R) {
-	if (dataset_is_panel(dset)) {
+	if (opt & OPT_C) {
+	    pmod->opt |= OPT_R;
+	    err = qr_make_cluster_vcv(pmod, IVREG, dset, V, opt);
+	} else if (dataset_is_panel(dset)) {
 	    err = qr_make_regular_vcv(pmod, V, OPT_X);
 	    if (!err) {
 		err = panel_tsls_robust_vcv(pmod, dset);
@@ -1149,3 +1164,239 @@ int qr_tsls_vcv (MODEL *pmod, const DATASET *dset, gretlopt opt)
     return err;    
 }
 
+static int cval_count (MODEL *pmod, double cvi, const double *cZ)
+{
+    int t, cc = 0;
+
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (!na(pmod->uhat[t]) && cZ[t] == cvi) {
+	    cc++;
+	}
+    }
+
+    return cc;
+}
+
+static int cval_count_max (MODEL *pmod, const gretl_matrix *cvals, 
+			   const double *cZ)
+{
+    int n = gretl_vector_get_length(cvals);
+    int i, cc, cmax = 0;
+
+    for (i=0; i<n; i++) {
+	cc = cval_count(pmod, cvals->val[i], cZ);
+	if (cc > cmax) {
+	    cmax = cc;
+	}
+    }
+
+    return cmax;
+}
+
+#define CDEBUG 0
+
+static gretl_matrix *cluster_vcv_calc (MODEL *pmod,
+				       int cvar,
+				       gretl_matrix *cvals, 
+				       gretl_matrix *XX,
+				       const DATASET *dset,
+				       int *err)
+
+{
+    gretl_matrix *V = NULL;
+    gretl_matrix *W = NULL;
+    gretl_vector *ei = NULL;
+    gretl_matrix *Xi = NULL;
+    gretl_vector *eXi = NULL;
+    const double *cZ;
+    int n_c, M, N, k = pmod->ncoeff;
+    int total_obs = 0;
+    int i, j, v, t;
+
+    cZ = dset->Z[cvar];    
+    N = cval_count_max(pmod, cvals, cZ);
+#if CDEBUG
+    fprintf(stderr, "max cval count = %d\n", N);
+#endif
+
+    V   = gretl_matrix_alloc(k, k);
+    W   = gretl_zero_matrix_new(k, k);
+    ei  = gretl_column_vector_alloc(N);
+    Xi  = gretl_matrix_alloc(N, k);
+    eXi = gretl_vector_alloc(k);
+
+    if (V == NULL || W == NULL || ei == NULL || 
+	Xi == NULL || eXi == NULL) {
+	*err = E_ALLOC;
+	goto bailout;
+    }
+
+    M = gretl_vector_get_length(cvals);
+    n_c = 0;
+
+    for (i=0; i<M; i++) {
+	double cvi = cvals->val[i];
+	int Ni = cval_count(pmod, cvi, cZ);
+	int s = 0;
+
+	if (Ni == 0) {
+	    continue;
+	}
+
+#if CDEBUG
+	fprintf(stderr, "i=%d, cvi=%g, Ni=%d\n", i, cvi, Ni);
+#endif
+	ei = gretl_matrix_reuse(ei, Ni, -1);
+	Xi = gretl_matrix_reuse(Xi, Ni, -1);
+
+	for (t=pmod->t1; t<=pmod->t2; t++) {
+	    if (!na(pmod->uhat[t]) && cZ[t] == cvi) {
+		gretl_vector_set(ei, s, pmod->uhat[t]);
+		for (j=0; j<k; j++) {
+		    v = pmod->list[j+2];
+		    gretl_matrix_set(Xi, s, j, dset->Z[v][t]);
+		}
+		s++;
+	    }
+	    if (s == Ni) {
+		/* we've filled this matrix */
+		break;
+	    }
+	}
+
+	gretl_matrix_multiply_mod(ei, GRETL_MOD_TRANSPOSE,
+				  Xi, GRETL_MOD_NONE,
+				  eXi, GRETL_MOD_NONE);
+	gretl_matrix_multiply_mod(eXi, GRETL_MOD_TRANSPOSE,
+				  eXi, GRETL_MOD_NONE,
+				  W, GRETL_MOD_CUMULATE);
+#if CDEBUG > 1
+	gretl_matrix_print(ei, "e(i)");
+	gretl_matrix_print(Xi, "X(i)");
+	gretl_matrix_print(W, "W");
+#endif
+	n_c++;
+	total_obs += s;
+    }
+
+    if (n_c < 2) {
+	gretl_errmsg_set("Invalid clustering variable");
+	*err = E_DATA;
+	goto bailout;
+    } else if (total_obs < pmod->nobs) {
+	*err = E_MISSDATA;
+	goto bailout;
+    }
+
+    /* form V(W) = (X'X)^{-1} W (X'X)^{-1} */
+    gretl_matrix_qform(XX, GRETL_MOD_NONE, W,
+		       V, GRETL_MOD_NONE);
+
+#if CDEBUG
+    gretl_matrix_print(XX, "X'X^{-1}");
+    gretl_matrix_print(W, "W");
+    gretl_matrix_print(V, "V");
+#endif
+
+    if (!(pmod->opt & OPT_N)) {
+	/* apply df adjustment a la Stata */
+	double dfadj;
+
+	N = pmod->nobs;
+	dfadj = (M/(M-1.0)) * (N-1.0)/(N-k);
+	gretl_matrix_multiply_by_scalar(V, dfadj);
+#if CDEBUG
+	gretl_matrix_print(V, "V(adjusted)");
+#endif
+
+    }
+
+ bailout:
+
+    gretl_matrix_free(W);
+    gretl_matrix_free(ei);
+    gretl_matrix_free(Xi);
+    gretl_matrix_free(eXi);
+
+    if (*err) {
+	gretl_matrix_free(V);
+	V = NULL;
+    }
+
+    return V;
+}
+
+/**
+ * qr_make_cluster_vcv:
+ * @pmod: pointer to model.
+ * @ci: command index (right now, OLS or IVREG).
+ * @dset: pointer to dataset.
+ * @XX: X'X matrix.
+ * 
+ * Compute and set on @pmod a variance matrix that is "clustered"
+ * by the value of a selected variable via the --cluster=foo 
+ * command-line option.
+ *
+ * Returns: 0 on success, non-zero code on error.
+ */
+
+static int qr_make_cluster_vcv (MODEL *pmod, int ci,
+				const DATASET *dset,
+				gretl_matrix *XX,
+				gretlopt opt)
+{
+    gretl_matrix *cvals = NULL;
+    gretl_matrix *V = NULL;
+    const char *cname;
+    int cvar, n_c = 0;
+    int err = 0;
+
+    if (pmod->ci != OLS && pmod->ci != IVREG) {
+	/* relax this? */
+	return E_NOTIMP;
+    }
+
+    cname = get_optval_string(ci, OPT_C); 
+    if (cname == NULL) {
+	return E_PARSE;
+    }
+
+    cvar = current_series_index(dset, cname);
+    if (cvar < 1 || cvar >= dset->v) {
+	err = E_UNKVAR;
+    }
+
+    if (!err) {
+	cvals = gretl_matrix_values(dset->Z[cvar] + pmod->t1,
+				    pmod->nobs, OPT_S, &err);
+	if (!err) {
+	    n_c = gretl_vector_get_length(cvals);
+	    if (n_c < 2) {
+		err = E_DATA;
+	    }
+	}
+    }
+
+#if CDEBUG
+    fprintf(stderr, "cluster var = %s (%d)\n", cname, cvar);
+    gretl_matrix_print(cvals, "cvals");
+#endif
+
+    if (!err) {
+	V = cluster_vcv_calc(pmod, cvar, cvals, XX, dset, &err);
+    }
+
+    if (!err) {
+	err = gretl_model_write_vcv(pmod, V);
+    }
+
+    if (!err) {
+	gretl_model_set_vcv_info(pmod, VCV_CLUSTER, cvar);
+	gretl_model_set_int(pmod, "n_clusters", n_c);
+    }
+
+    gretl_matrix_free(V);
+    gretl_matrix_free(cvals);
+
+    return err;
+}
