@@ -624,6 +624,10 @@ char *gretl_str_expand (char **orig, const char *add, const char *sep)
  * gretl_word_strdup:
  * @src: the source string.
  * @ptr: location to receive end of word pointer, or NULL.
+ * @opt: can include OPT_S for "strict" operation: in this
+ * case an error is flagged if @src contains any characters
+ * other than 'word' characters (see below), comma and space.
+ * @err: location to receive error code.
  *
  * Copies the first 'word' found in @src, where a word
  * is defined as consisting of alphanumeric characters
@@ -632,10 +636,11 @@ char *gretl_str_expand (char **orig, const char *add, const char *sep)
  * word.
  *
  * Returns: the allocated word or NULL in case no word is
- * found, or if allocation fails.
+ * found, or on error.
  */
 
-char *gretl_word_strdup (const char *src, const char **ptr)
+char *gretl_word_strdup (const char *src, const char **ptr,
+			 gretlopt opt, int *err)
 {
     char *targ = NULL;
 
@@ -651,8 +656,19 @@ char *gretl_word_strdup (const char *src, const char **ptr)
 	const char *p;
 	int len = 0;
 
-	while (*src && !is_word_char(*src)) {
-	    src++;
+	if (opt & OPT_S) {
+	    /* strict: don't allow any junk */
+	    while (*src && (*src == ' ' || *src == ',')) {
+		src++;
+	    }
+	    if (*src && !is_word_char(*src)) {
+		*err = E_PARSE;
+		return NULL;
+	    }
+	} else {
+	    while (*src && !is_word_char(*src)) {
+		src++;
+	    }
 	}
 
 	p = src;
@@ -668,6 +684,9 @@ char *gretl_word_strdup (const char *src, const char **ptr)
 
 	if (len > 0) {
 	    targ = gretl_strndup(p, len);
+	    if (targ == NULL) {
+		*err = E_ALLOC;
+	    }
 	}
     }
 
@@ -1067,15 +1086,15 @@ char *switch_ext_new (const char *src, const char *ext)
  * @maxlen, and @err is not NULL, then E_TOOLONG is written 
  * to @err.
  * 
- * Returns: 1 if a trailing backslash or comma was found, 
- * otherwise 0.
+ * Returns: 1 if a trailing backslash, comma or left parenthesis
+ * was found, otherwise 0.
  */
 
 int top_n_tail (char *str, size_t maxlen, int *err)
 {
     int i, n, cont = 0;
 
-    if (str == NULL || *str == 0 || *str == '\n' || *str == '\r') {
+    if (str == NULL || *str == '\0' || *str == '\n' || *str == '\r') {
 	return 0;
     }
 
@@ -1094,7 +1113,7 @@ int top_n_tail (char *str, size_t maxlen, int *err)
 	}
     }
 
-    if (*str != 0) {
+    if (*str != '\0') {
 	/* Drop any leading spaces, also possible questionmark.  Try
 	   to catch non-breaking spaces too -- ugh, Windows!
 	*/
@@ -1121,6 +1140,8 @@ int top_n_tail (char *str, size_t maxlen, int *err)
 		    cont = 1;
 		} else if (str[n] == ',') {
 		    cont = 1;
+		} else if (str[n] == '(') {
+		    cont = 1;
 		}
 	    }
 	}
@@ -1740,31 +1761,6 @@ void modify_date_for_csv (char *s, int pd)
     }
 }
 
-/**
- * csv_obs_string_to_prn:
- * @t: 0-based observation number.
- * @dset: data set information struct.
- * @prn: printing struct.
- *
- * Prints the observation string corresponding to obervation @t to
- * @prn, in a format suitable for a CSV file.
- */
-
-void csv_obs_to_prn (int t, const DATASET *dset, PRN *prn)
-{
-    if (dset->S != NULL) {
-	pprintf(prn, "%s%c", dset->S[t], dset->delim);
-    } else if (dset->structure != CROSS_SECTION) {
-	char tmp[OBSLEN];
-
-	ntodate(tmp, t, dset);
-	if (quarterly_or_monthly(dset)) {
-	    modify_date_for_csv(tmp, dset->pd);
-	}
-	pprintf(prn, "%s%c", tmp, dset->delim);
-    }
-}
-	
 /**
  * print_time:
  * @s: string into which to print: must be at least 48 bytes.
@@ -2195,3 +2191,33 @@ void get_column_widths (const char **strs, int *widths, int n)
 	}
     }
 }
+
+/**
+ * gretl_utf8_strncat:
+ * @dest: destination string.
+ * @src: source string.
+ * @n: maximum number of bytes to append.
+ *
+ * Works just like strncat(), except that it ensures that we
+ * don't end up with an incomplete UTF-8 character preceding
+ * the terminating NUL byte.
+ */
+
+char *gretl_utf8_strncat (char *dest, const char *src, size_t n)
+{
+    const char *p = src;
+    size_t b, b0 = 0;
+
+    while (p && *p) {
+	p = g_utf8_next_char(p);
+	if (p) {
+	    b = p - src;
+	    if (b > n) {
+		break;
+	    }
+	    b0 = b;
+	}
+    }
+
+    return strncat(dest, src, b0);
+}
