@@ -547,7 +547,12 @@ int restore_full_sample (DATASET *dset, ExecState *state)
 	return E_NODATA;
     }
 
+#if 0 
+    /* 2012-03-18: this can mask errors in functions in which
+       sub-sampling occurs, making script debugging difficult 
+    */
     gretl_error_clear();
+#endif
 
     if (!complex_subsampled()) {
 	int t1min, t2max;
@@ -585,8 +590,13 @@ int restore_full_sample (DATASET *dset, ExecState *state)
     if (dset->submask == RESAMPLED) {
 	err = resample_sync_dataset(dset);
     } else {
-	sync_datainfo_members(dset);
-	err = sync_data_to_full(dset);
+	if (dset->padmask != NULL) {
+	    err = undo_panel_padding(dset);
+	}
+	if (!err) {
+	    sync_datainfo_members(dset);
+	    err = sync_data_to_full(dset);
+	}
     }
 
     if (err == E_NOMERGE) {
@@ -695,12 +705,12 @@ static int copy_dummy_to_mask (char *mask, const double *x, int n)
 static int mask_from_temp_dummy (const char *s, DATASET *dset, 
 				 char *mask, PRN *prn)
 {
-    char formula[MAXLEN];
+    char formula[MAXLINE];
     double *x;
     int err = 0;
 
     *formula = '\0';
-    strncat(formula, s, MAXLEN - 1);
+    strncat(formula, s, MAXLINE - 1);
 
     x = generate_series(formula, dset, prn, &err);
 
