@@ -44,12 +44,6 @@ enum {
     SYSTEM_QUIET       = 1 << 8
 };
 
-enum {
-    SYSNAME_NEW,
-    SYSNAME_EST,
-    SYSNAME_RST
-};
-
 typedef struct id_atom_ id_atom;
 typedef struct identity_ identity;
 typedef struct predet_ predet;
@@ -58,6 +52,7 @@ typedef struct liml_data_ liml_data;
 struct equation_system_ {
     char *name;                 /* user-specified name for system, or NULL */
     int refcount;               /* for saving/deleting */
+    int fd;                     /* function execution depth */
     int t1;                     /* starting observation number */
     int t2;                     /* ending observation number */
     int T;                      /* number of observations per equation */
@@ -104,12 +99,14 @@ equation_system *equation_system_start (const char *line,
 					gretlopt opt,
 					int *err);
 
-char *get_system_name_from_line (const char *s, int context);
+char *get_system_name_from_line (const char *s);
+
+equation_system *get_anonymous_equation_system (void);
 
 int equation_system_append (equation_system *sys, const int *list);
 
 int equation_system_append_multi (equation_system *sys, 
-				  const char *mname, 
+				  const char *param, 
 				  const DATASET *dset);
 
 int system_parse_line (equation_system *sys,
@@ -131,11 +128,14 @@ int estimate_named_system (const char *line,
 
 void equation_system_destroy (equation_system *sys);
 
+void delete_anonymous_equation_system (int level);
+
 int system_want_df_corr (const equation_system *sys);
 
 int system_n_restrictions (const equation_system *sys);
 
 int system_max_indep_vars (const equation_system *sys);
+
 int system_n_indep_vars (const equation_system *sys);
 
 int *system_get_list (const equation_system *sys, int i);
@@ -193,6 +193,9 @@ int system_wald_test (const equation_system *sys,
 		      gretlopt opt,
 		      PRN *prn);
 
+int system_diag_test (const equation_system *sys, double *test,
+		      double *pval);
+
 double *system_get_resid_series (equation_system *sys, int eqnum,
 				 DATASET *dset, int *err);
 
@@ -231,6 +234,8 @@ gretl_matrix *sys_get_fitted_values (equation_system *sys,
 int system_adjust_t1t2 (equation_system *sys, 
 			const DATASET *dset);
 
+int system_supports_method (equation_system *sys, int method);
+
 #ifdef FULL_XML_HEADERS
 
 equation_system *equation_system_from_XML (xmlNodePtr node, 
