@@ -577,7 +577,7 @@ static int tex_print_coeff_custom (const model_coeff *mc, PRN *prn)
     if (colspec[0][0]) {
 	/* coefficient */
 	if (na(mc->b)) {
-	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", A_("undefined"));
 	} else {
 	    sprintf(fmt, "$%s$", colspec[0]);
 	    pprintf(prn, fmt, mc->b);
@@ -595,7 +595,7 @@ static int tex_print_coeff_custom (const model_coeff *mc, PRN *prn)
 	}
 	/* standard error */
 	if (na(mc->se)) {
-	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", A_("undefined"));
 	} else {
 	    pprintf(prn, colspec[1], mc->se);
 	}
@@ -612,7 +612,7 @@ static int tex_print_coeff_custom (const model_coeff *mc, PRN *prn)
 	}
 	/* t-ratio */
 	if (na(mc->tval)) {
-	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", A_("undefined"));
 	} else {
 	    sprintf(fmt, "$%s$", colspec[2]);
 	    pprintf(prn, fmt, mc->tval);
@@ -623,11 +623,17 @@ static int tex_print_coeff_custom (const model_coeff *mc, PRN *prn)
 	if (colspec[0][0] || colspec[1][0] || colspec[2][0]) {
 	    pputs(prn, " & ");
 	}
-	/* p-value */
-	if (na(mc->pval)) {
-	    pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", I_("undefined"));
+	/* p-value (or perhaps slope) */
+	if (mc->show_pval) {
+	    if (na(mc->pval)) {
+		pprintf(prn, "\\multicolumn{1}{c}{\\rm %s}", A_("undefined"));
+	    } else {
+		pprintf(prn, colspec[3], mc->pval);
+	    }
+	} else if (!na(mc->slope)) {
+	    pprintf(prn, colspec[3], mc->slope);
 	} else {
-	    pprintf(prn, colspec[3], mc->pval);
+	    pprintf(prn, "\\multicolumn{1}{c}{}");
 	}
     }  
 
@@ -704,7 +710,7 @@ void tex_print_coeff (const model_coeff *mc, PRN *prn)
     }
 
     if (na(mc->b)) {
-	sprintf(col1, "\\multicolumn{2}{c}{\\rm %s}", I_("undefined"));
+	sprintf(col1, "\\multicolumn{2}{c}{\\rm %s}", A_("undefined"));
     } else {
 	tex_rl_double(mc->b, col1);
     }
@@ -715,13 +721,13 @@ void tex_print_coeff (const model_coeff *mc, PRN *prn)
 	ncols = 3;
     } else {
 	if (na(mc->se)) {
-	    sprintf(col2, "\\multicolumn{2}{c}{\\rm %s}", I_("undefined"));
+	    sprintf(col2, "\\multicolumn{2}{c}{\\rm %s}", A_("undefined"));
 	} else {
 	    tex_rl_double(mc->se, col2);
 	}
 
 	if (na(mc->tval)) {
-	    sprintf(col3, "\\multicolumn{2}{c}{\\rm %s}", I_("undefined"));
+	    sprintf(col3, "\\multicolumn{2}{c}{\\rm %s}", A_("undefined"));
 	} else {
 	    tex_rl_float(mc->tval, col3, 4);
 	}
@@ -729,7 +735,9 @@ void tex_print_coeff (const model_coeff *mc, PRN *prn)
 
     *col4 = '\0';
 
-    if (!na(mc->slope)) {
+    if (!mc->show_pval && na(mc->slope)) {
+	strcpy(col4, "\\multicolumn{2}{c}{}");
+    } else if (!na(mc->slope)) {
 	tex_rl_double(mc->slope, col4);
     } else if (mc->show_pval) {
 	if (!na(mc->pval)) {
@@ -781,10 +789,10 @@ tex_custom_coeff_table_start (const char **cols, gretlopt opt, PRN *prn)
 
     pputs(prn, "}\n");
 
-    pprintf(prn, "\\multicolumn{1}{c}{%s} &\n", I_(cols[0]));
+    pprintf(prn, "\\multicolumn{1}{c}{%s} &\n", A_(cols[0]));
 
     if (colspec[0][0]) {
-	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(cols[1]));
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", A_(cols[1]));
     }
 
     if (!colspec[1][0] && !colspec[2][0] && !colspec[3][0]) {
@@ -796,7 +804,7 @@ tex_custom_coeff_table_start (const char **cols, gretlopt opt, PRN *prn)
 	if (colspec[0][0]) {
 	    pputs(prn, " &\n");
 	}
-	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(cols[2]));
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", A_(cols[2]));
     }
 
     if (!colspec[2][0] && !colspec[3][0]) {
@@ -808,14 +816,14 @@ tex_custom_coeff_table_start (const char **cols, gretlopt opt, PRN *prn)
 	if (colspec[0][0] || colspec[1][0]) {
 	    pputs(prn, " &\n");
 	}
-	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(cols[3]));
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", A_(cols[3]));
     }
 
     if (colspec[3][0]) {
 	if (colspec[0][0] || colspec[1][0] || colspec[2][0]) {
 	    pputs(prn, " &\n");
 	}
-	pprintf(prn, "\\multicolumn{1}{c}{%s}", I_(cols[4]));
+	pprintf(prn, "\\multicolumn{1}{c}{%s}", A_(cols[4]));
     }
 
     pputs(prn, " \\\\\n");
@@ -851,13 +859,13 @@ int tex_coeff_table_start (const char **cols, gretlopt opt, PRN *prn)
 	ncols += 2;
     }
 
-    pprintf(prn, "}\n%s &\n", I_(cols[0]));
+    pprintf(prn, "}\n%s &\n", A_(cols[0]));
 
     mcols = (opt & OPT_M)? 1 : 2;
 
     for (i=1; cols[i] != NULL; i++) {
 	bufspace(i, prn);
-	pprintf(prn, "\\multicolumn{%d}{c}{%s%s} %s\n", mcols, I_(cols[i]),
+	pprintf(prn, "\\multicolumn{%d}{c}{%s%s} %s\n", mcols, A_(cols[i]),
 		(cols[i+1] == NULL && binary)? "$^*$" : "",
 		(cols[i+1] == NULL)? "\\\\[1ex]" : "&");
     }
@@ -877,7 +885,7 @@ void tex_print_VECM_omega (GRETL_VAR *vecm, const DATASET *dset, PRN *prn)
     double x;
     int i, j;
 
-    pprintf(prn, "%s\n\n", I_("Cross-equation covariance matrix"));
+    pprintf(prn, "%s\n\n", A_("Cross-equation covariance matrix"));
     pputs(prn, "\\vspace{1em}\n");
 
     pputs(prn, "\\begin{tabular}{");
@@ -916,7 +924,7 @@ void tex_print_VECM_omega (GRETL_VAR *vecm, const DATASET *dset, PRN *prn)
     pputs(prn, "\\vspace{1em}\n");
 
     pputs(prn, "\\noindent\n");
-    pprintf(prn, "%s = ", I_("determinant"));
+    pprintf(prn, "%s = ", A_("determinant"));
     tex_print_double(exp(vecm->ldet), prn);
     pputs(prn, "\\\\\n");
 }
@@ -948,9 +956,9 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATASET *dset, PRN *prn)
     double x;
 
     pputs(prn, "\\noindent\n");
-    pputs(prn, I_("Cointegrating vectors"));
+    pputs(prn, A_("Cointegrating vectors"));
     if (jv->Bse != NULL) {
-	pprintf(prn, " (%s)\n", I_("standard errors in parentheses"));
+	pprintf(prn, " (%s)\n", A_("standard errors in parentheses"));
     } else {
 	pputc(prn, '\n');
     }
@@ -1004,9 +1012,9 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATASET *dset, PRN *prn)
     rows = gretl_matrix_rows(jv->Alpha);
 
     pputs(prn, "\\noindent\n");
-    pprintf(prn, I_("Adjustment vectors"));
+    pprintf(prn, A_("Adjustment vectors"));
     if (jv->Ase != NULL) {
-	pprintf(prn, " (%s)\n", I_("standard errors in parentheses"));
+	pprintf(prn, " (%s)\n", A_("standard errors in parentheses"));
     } else {
 	pputc(prn, '\n');
     }
@@ -1061,17 +1069,17 @@ void tex_print_VECM_coint_eqns (GRETL_VAR *vecm, const DATASET *dset, PRN *prn)
 
 void tex_print_VAR_ll_stats (GRETL_VAR *var, PRN *prn)
 {
-    pprintf(prn, "\\noindent\n%s = ", I_("Log-likelihood"));
+    pprintf(prn, "\\noindent\n%s = ", A_("Log-likelihood"));
     tex_print_double(var->ll, prn);
     pputs(prn, "\\par\n");
 
-    pprintf(prn, "\\noindent\n%s = ", I_("Determinant of covariance matrix"));
+    pprintf(prn, "\\noindent\n%s = ", A_("Determinant of covariance matrix"));
     tex_print_double(exp(var->ldet), prn);
     pputs(prn, "\\par\n");
 
-    pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", I_("AIC"), var->AIC);
-    pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", I_("BIC"), var->BIC);
-    pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", I_("HQC"), var->HQC);
+    pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", A_("AIC"), var->AIC);
+    pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", A_("BIC"), var->BIC);
+    pprintf(prn, "\\noindent\n%s $= %.4f$ \\par\n", A_("HQC"), var->HQC);
 }
 
 static PRN *make_tex_prn (int ID, char *fname,
@@ -1131,7 +1139,7 @@ static PRN *make_rtf_prn (int ID, char *fname, int *err)
 
 static char tex_preamble_file[MAXLEN];
 
-static const char *get_gretltex_local (char *local)
+static const char *set_tex_locale_filename (char *local)
 {
     char *lang = getenv("LANG");
 
@@ -1147,64 +1155,59 @@ static const char *get_gretltex_local (char *local)
     return local;
 }
 
+static int find_gretlpre (const char *path, const char *localname)
+{
+    char test[MAXLEN];
+    int err, gotit = 0;
+
+    if (*localname != '\0') {
+	/* localized preamble file? */
+	sprintf(test, "%s%s", path, localname);
+	err = gretl_test_fopen(test, "r");
+	if (!err) {
+	    strcpy(tex_preamble_file, test);
+	    gotit = 1;
+	}
+    }
+
+    if (!gotit) {
+	/* regular preamble file? */
+	sprintf(test, "%sgretlpre.tex", path);
+	err = gretl_test_fopen(test, "r");
+	if (!err) {
+	    strcpy(tex_preamble_file, test);
+	    gotit = 1;
+	}
+    }
+
+    return gotit;
+}
+
 void set_gretl_tex_preamble (void)
 {
-    const char *wdir = gretl_workdir();
-    char local[16], test[MAXLEN];
-    FILE *fp;
-    int gotit = 0;
+    const char *path = gretl_workdir();
+    char localname[16];
+    int gotit;
 
-    get_gretltex_local(local);
+    set_tex_locale_filename(localname);
 
-    /* localized preamble file in working dir? */
-    if (*local != '\0') {
-	sprintf(test, "%s%s", wdir, local);
-	fp = gretl_try_fopen(test, "r");
-	if (fp != NULL) {
-	    strcpy(tex_preamble_file, test);
-	    fclose(fp);
-	    gotit = 1;
-	}
-    }
+    gotit = find_gretlpre(path, localname);
 
     if (!gotit) {
-	/* regular preamble file in working dir? */
-	sprintf(test, "%sgretlpre.tex", wdir);
-	fp = gretl_try_fopen(test, "r");
-	if (fp != NULL) {
-	    strcpy(tex_preamble_file, test);
-	    fclose(fp);
-	    gotit = 1;
+	path = maybe_get_default_workdir();
+	if (path != NULL) {
+	    gotit = find_gretlpre(path, localname);
 	}
     }
-
+    
     if (!gotit) {
-	const char *ddir = maybe_get_default_workdir();
-
-	if (ddir != NULL) {
-	    /* localized preamble file in standard working dir? */
-	    if (*local != '\0') {
-		sprintf(test, "%s%s", ddir, local);
-		fp = gretl_try_fopen(test, "r");
-		if (fp != NULL) {
-		    strcpy(tex_preamble_file, test);
-		    fclose(fp);
-		    gotit = 1;
-		}
-	    }
-
-	    if (!gotit) {
-		/* regular preamble file in standard working dir */
-		sprintf(test, "%sgretlpre.tex", ddir);
-		fp = gretl_try_fopen(test, "r");
-		if (fp != NULL) {
-		    strcpy(tex_preamble_file, test);
-		    fclose(fp);
-		    gotit = 1;
-		}
-	    }
-	}
-    } 
+#ifdef OSX_BUILD
+	path = gretl_app_support_dir();
+#else
+	path = gretl_dotdir();
+#endif
+	gotit = find_gretlpre(path, localname);
+    }
 
     gretl_error_clear();
 }
@@ -1240,7 +1243,7 @@ static void landscape_modify_line (char *line)
     }
 }
 
-static int tex_use_utf;
+static int tex_use_utf = 1;
 
 void set_tex_use_utf (int s)
 {
@@ -1275,6 +1278,14 @@ void gretl_tex_preamble (PRN *prn, int fmt)
     }
 
     if (!userfile) {
+	const char *paper = in_usa()? "letterpaper" : "a4paper";
+	const char *driver = use_pdf ? "pdftex" : "dvips";
+	const char *margin = "";
+
+	if (fmt & GRETL_FORMAT_MODELTAB) {
+	    margin = "margin=2cm,";
+	}
+
 	pputs(prn, "\\documentclass");
 	
 	if (fmt & GRETL_FORMAT_MODELTAB) {
@@ -1289,30 +1300,32 @@ void gretl_tex_preamble (PRN *prn, int fmt)
 
 	pputs(prn, "{article}\n");
 
-	if (tex_use_utf) {
-	    pputs(prn, "\\usepackage{ucs}\n");
-	    pputs(prn, "\\usepackage[utf8x]{inputenc}\n");
+#ifdef ENABLE_NLS
+	if (!tex_use_utf) {
+	    /* not currently activated, but maybe allow a "set" variable? */
+	    char encfile[16];
+
+	    get_suitable_tex_encoding(encfile);
+	    pprintf(prn, "\\usepackage[%s]{inputenc}\n", encfile);
 	} else {
-	    pputs(prn, "\\usepackage[latin1]{inputenc}\n");
+	    pputs(prn, "\\usepackage[utf8]{inputenc}\n");
 	}
+#endif
+
 	if (lang != NULL && !strncmp(lang, "ru", 2)) {
 	    pputs(prn, "\\usepackage[russian]{babel}\n");
 	}
 
+	pprintf(prn, "\\usepackage[%s,%s%s]{geometry}\n", paper, margin,
+		driver);
+
 	if (fmt & GRETL_FORMAT_EQN) {
-	    pputs(prn, "\\usepackage{amsmath}\n\n");
-	} else if (fmt & GRETL_FORMAT_MODELTAB) {
-	    pputs(prn, "\\usepackage{longtable}\n");
-	    if (use_pdf) {
-		pputs(prn, "\\usepackage[margin=2cm,pdftex]{geometry}\n\n");
-	    } else {
-		pputs(prn, "\\usepackage[margin=2cm,dvips]{geometry}\n\n");
-	    }
+	    pputs(prn, "\\usepackage{amsmath}\n");
 	} else {
-	    pputs(prn, "\\usepackage{longtable}\n\n");
+	    pputs(prn, "\\usepackage{longtable}\n");
 	}
 
-	pputs(prn, "\\begin{document}\n\n"
+	pputs(prn, "\n\\begin{document}\n\n"
 	      "\\thispagestyle{empty}\n\n");
     }
 }
@@ -1357,6 +1370,8 @@ int tex_print_equation (const MODEL *pmod, const DATASET *dset,
 	return E_NOTIMP;
     }
 
+    set_alt_gettext_mode(prn);
+
     if (COUNT_MODEL(pmod->ci)) {
 	offvar = gretl_model_get_int(pmod, "offset_var");
 	if (offvar > 0) {
@@ -1552,8 +1567,8 @@ int tex_print_equation (const MODEL *pmod, const DATASET *dset,
 
     if (sderr_ok) {
 	pprintf(prn, "\\centerline{(%s)} \\notag\n",
-		(opt & OPT_T)? I_("$t$-statistics in parentheses") :
-		I_("standard errors in parentheses"));
+		(opt & OPT_T)? A_("$t$-statistics in parentheses") :
+		A_("standard errors in parentheses"));
     } else {
 	pputs(prn, "\\notag\n");
     }
