@@ -29,13 +29,24 @@
 
 #define TDEBUG 0
 
-static void tsls_omitzero (int *list, const DATASET *dset)
+static void tsls_omitzero (int *list, const DATASET *dset,
+			   const char *mask)
 {
-    int i, v;
+    int i, v, t, allzero;
 
     for (i=list[0]; i>1; i--) {
         v = list[i];
-        if (gretl_iszero(dset->t1, dset->t2, dset->Z[v])) {
+	allzero = 1;
+	for (t=dset->t1; t<=dset->t2; t++) {
+	    if (mask != NULL && mask[t - dset->t1]) {
+		continue;
+	    }
+	    if (dset->Z[v][t] != 0.0) {
+		allzero = 0;
+		break;
+	    }
+	}
+        if (allzero) {
 	    gretl_list_delete_at_pos(list, i);
 	}
     }
@@ -51,13 +62,13 @@ static void tsls_omitzero (int *list, const DATASET *dset)
 
 void tsls_free_data (const MODEL *pmod)
 {
-    const char *endog = gretl_model_get_data(pmod, "endog");
+    gretl_matrix *endog = gretl_model_get_data(pmod, "endog");
     double **X = gretl_model_get_data(pmod, "tslsX");
     int i, m = 0;
 
     if (endog != NULL && X != NULL) {
 	for (i=0; i<pmod->ncoeff; i++) {
-	    if (endog[i]) m++;
+	    if (endog->val[i] != 0) m++;
 	}
 	for (i=0; i<m; i++) {
 	    free(X[i]);
@@ -74,15 +85,13 @@ static int
 tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 		DATASET *dset)
 {
+    gretl_matrix *endog = NULL;
     double **X = NULL;
-    char *endog = NULL;
-    int i, v, err = 0;
-    size_t esize, Xsize;
-    size_t xs_old, es_old;
+    size_t Xsize, xs_old;
     int recycle_X = 0;
     int recycle_e = 0;
+    int i, v, err = 0;
 
-    esize = pmod->ncoeff * sizeof *endog;
     Xsize = hatlist[0] * sizeof *X;
 
     /* re-use old pointers if applicable */
@@ -106,19 +115,19 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 	}
     }
 
-    endog = gretl_model_get_data_full(pmod, "endog", NULL, &es_old);
+    endog = gretl_model_get_data(pmod, "endog");
     if (endog != NULL) {
-	if (esize == es_old) {
+	if (gretl_vector_get_length(endog) == pmod->ncoeff) {
 	    recycle_e = 1;
 	} else {
 	    gretl_model_detach_data_item(pmod, "endog");
-	    free(endog);
+	    gretl_matrix_free(endog);
 	    endog = NULL;
 	}
     }
 
-    if (!recycle_e && esize > 0) {
-	endog = malloc(esize);
+    if (!recycle_e) {
+	endog = gretl_matrix_alloc(pmod->ncoeff, 1);
 	if (endog == NULL) {
 	    free(X);
 	    return E_ALLOC;
@@ -134,7 +143,7 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 
     for (i=0; i<pmod->ncoeff; i++) {
 	v = pmod->list[i+2];
-	endog[i] = !in_gretl_list(exolist, v);
+	endog->val[i] = !in_gretl_list(exolist, v);
     }
 
      /* now attach X and endog to the model */
@@ -143,8 +152,7 @@ tsls_save_data (MODEL *pmod, const int *hatlist, const int *exolist,
 			     Xsize);
     }
     if (!recycle_e && endog != NULL) {
-	gretl_model_set_data(pmod, "endog", endog, GRETL_TYPE_CHAR_ARRAY,
-			     esize);
+	gretl_model_set_matrix_as_data(pmod, "endog", endog);
     }
 
     return err;
@@ -361,7 +369,8 @@ ivreg_list_add (const int *orig, const int *add, gretlopt opt, int *err)
  * but not in @instlist, then it is added to @instlist and this
  * is flagged by writing 1 into @addconst.
  *
- * Returns: allocated list of variables to be instrumented.
+ * Returns: allocated list of variables to be instrumented,
+ * or NULL if there are no such variables.
  */
 
 int *tsls_make_endolist (const int *reglist, int **instlist, 
@@ -832,32 +841,31 @@ static gretl_matrix *tsls_Q (int *instlist, int **pdlist,
     return Q;
 }
 
-static int tsls_form_xhat (gretl_matrix *Q, double *x, double *xhat,
-			   DATASET *dset, const char *mask)
+static int tsls_form_xhat (gretl_matrix *Q, gretl_matrix *r,
+			   DATASET *dset, int v0, int v1, 
+			   const char *mask)
 {
-    double *r;
+    const double *x = dset->Z[v0];
+    double *xhat = dset->Z[v1];
     int k = gretl_matrix_cols(Q);
+    int allzero = 1;
     int i, t, s = 0;
-
-    r = malloc(k * sizeof *r);
-    if (r == NULL) {
-	return E_ALLOC;
-    }
+    int err = 0;
 
 #if TDEBUG > 1
-    fprintf(stderr, "tsls_form_xhat: t1=%d, t2=%d, mask=%p\n",
-	    dset->t1, dset->t2, (void *) mask);
+    fprintf(stderr, "tsls_form_xhat: v0=%d, v1=%d, t1=%d, t2=%d, mask=%p\n",
+	    v0, v1, dset->t1, dset->t2, (void *) mask);
 #endif
 
     /* form r = Q'y */
     for (i=0; i<k; i++) {
-	r[i] = 0.0;
+	r->val[i] = 0.0;
 	s = 0;
 	for (t=dset->t1; t<=dset->t2; t++) {
 	    if (mask != NULL && mask[t - dset->t1]) {
 		continue;
 	    }
-	    r[i] += gretl_matrix_get(Q, s++, i) * x[t];
+	    r->val[i] += gretl_matrix_get(Q, s++, i) * x[t];
 	}
     }
 
@@ -870,14 +878,25 @@ static int tsls_form_xhat (gretl_matrix *Q, double *x, double *xhat,
 	}
 	xhat[t] = 0.0;
 	for (i=0; i<k; i++) {
-	    xhat[t] += gretl_matrix_get(Q, s, i) * r[i];
+	    xhat[t] += gretl_matrix_get(Q, s, i) * r->val[i];
+	}
+	if (xhat[t] != 0) {
+	    allzero = 0;
 	}
 	s++;
     }
 
-    free(r);
+    if (allzero) {
+	gretl_errmsg_sprintf(_("The first-stage fitted values for %s are all zero"),
+			     dset->varname[v0]);
+	err = E_DATA;
+    } else {
+	/* name the fitted series according to the original */
+	strcpy(dset->varname[v1], "h_");
+	strncat(dset->varname[v1], dset->varname[v0], VNAMELEN - 3);
+    }
 
-    return 0;
+    return err;
 }
 
 static void tsls_residuals (MODEL *pmod, const int *reglist, 
@@ -956,7 +975,7 @@ static void tsls_extra_stats (MODEL *pmod, int yno, int overid,
     if (dataset_is_time_series(dset) && !model_has_missing_obs(pmod)) {
 	/* time series, no missing obs within sample range */
 	pmod->rho = rhohat(1, pmod->t1, pmod->t2, pmod->uhat);
-	pmod->dw = dwstat(1, pmod, (const double **) dset->Z);
+	pmod->dw = dwstat(1, pmod, dset);
     } else {
 	pmod->rho = pmod->dw = NADBL;
     }
@@ -989,8 +1008,11 @@ static void replace_list_element (int *list, int targ, int repl)
     }
 }
 
-static int 
-reglist_remove_redundant_vars (const MODEL *tmod, int *s2list, int *reglist)
+static int reglist_remove_redundant_vars (const MODEL *tmod, 
+					  int *s2list, 
+					  int *reglist, 
+					  int *endolist,
+					  int *hatlist)
 {
     int *dlist = gretl_model_get_data(tmod, "droplist");
     int i, pos;
@@ -1004,6 +1026,23 @@ reglist_remove_redundant_vars (const MODEL *tmod, int *s2list, int *reglist)
 	if (pos > 1) {
 	    gretl_list_delete_at_pos(reglist, pos);
 	}
+	pos = in_gretl_list(hatlist, dlist[i]);
+	if (pos > 1) {
+	    /* First replace dlist[i] with the ID of the original
+	       regressor from @endolist, in place of the "hatlist"
+	       variable (which will be deleted when the model is
+	       returned), so that the printout of regressors that are
+	       dropped due to exact collinearity will show the
+	       appropriate names.
+
+	       Second, delete the redundant term from both endolist
+	       and hatlist, so that subsequent calculations will not
+	       get messed up.
+	    */	    
+	    dlist[i] = endolist[pos];
+	    gretl_list_delete_at_pos(endolist, pos);
+	    gretl_list_delete_at_pos(hatlist, pos);
+	}
     }
 
     return 0;
@@ -1016,7 +1055,7 @@ reglist_remove_redundant_vars (const MODEL *tmod, int *s2list, int *reglist)
    "Testing for Weak Instruments in Linear IV Regressions" (originally
    NBER Technical Working Paper 284; revised February 2003), at
    http://ksghome.harvard.edu/~JStock/pdf/rfa_6.pdf
- */
+*/
 
 static int 
 compute_stock_yogo (MODEL *pmod, const int *endolist, 
@@ -1028,7 +1067,7 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
     gretl_matrix_block *B2 = NULL;
     gretl_matrix *G, *S, *Y;
     gretl_matrix *Ya, *YPY;
-    gretl_matrix *X, *Z, *E;
+    gretl_matrix *X, *Z, *E = NULL;
     int T = pmod->nobs;
     int n = endolist[0];
     int K1 = pmod->ncoeff - n;
@@ -1046,7 +1085,14 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
 	return E_ALLOC;
     }
 
+#if TDEBUG
+    fprintf(stderr, "stock_yogo: pmod->ncoeff=%d, n=%d, K1=%d, K2=%d\n",
+	    pmod->ncoeff, n, K1, K2);
+    printlist(endolist, "endolist, in stock-yogo");
+#endif
+
     if (K1 > 0) {
+	/* we have some exogenous regressors */
 	gretl_matrix *M;
 	int j, ix = 0, iz = 0;
 
@@ -1205,19 +1251,9 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
     }
 
     if (!err) {
-	gretl_matrix *e;
-
-	e = gretl_symmetric_matrix_eigenvals(G, 0, &err); 
+	double gmin = gretl_symm_matrix_lambda_min(G, &err); 
 
 	if (!err) {
-	    double gmin = e->val[0];
-
-	    for (i=1; i<n; i++) {
-		if (e->val[i] < gmin) {
-		    gmin = e->val[i];
-		}
-	    }
-	    gretl_matrix_free(e);
 	    gretl_model_set_double(pmod, "gmin", gmin);
 	    gretl_model_set_int(pmod, "n", n);
 	    gretl_model_set_int(pmod, "K2", K2);
@@ -1242,26 +1278,29 @@ compute_stock_yogo (MODEL *pmod, const int *endolist,
    covariance estimator.
 */
 
-static int 
-compute_first_stage_F (MODEL *pmod, int v, const int *reglist, 
-		       const int *instlist, DATASET *dset, 
-		       gretlopt opt)
+static int compute_first_stage_F (MODEL *pmod, 
+				  const int *endolist, 
+				  const int *reglist, 
+				  const int *instlist, 
+				  DATASET *dset, 
+				  gretlopt opt)
 {
     MODEL mod1;
     int *list1 = NULL;
     int *flist = NULL;
     double F;
-    int i, vi;
+    int i, ev, vi;
     int err = 0;
 
     gretl_model_init(&mod1);
+    ev = endolist[1];
 
-    /* The regression list, list1, has endogenous regressor v as the
+    /* The regression list, list1, has endogenous regressor ev as the
        dependent variable and includes all instruments; the list for
        omission, flist, includes all the excluded instruments.
     */
 
-    list1 = gretl_list_append_term(&list1, v);
+    list1 = gretl_list_append_term(&list1, ev);
     if (list1 == NULL) {
 	return E_ALLOC;
     }
@@ -1283,9 +1322,9 @@ compute_first_stage_F (MODEL *pmod, int v, const int *reglist,
     }
 
     if (!err) {
-	gretlopt myopt = (opt & OPT_R)? (OPT_A | OPT_R) : OPT_A;
+	gretlopt myopt = (opt & OPT_R)? OPT_R : OPT_NONE;
 
-	mod1 = lsq(list1, dset, OLS, myopt);
+	mod1 = lsq(list1, dset, OLS, myopt | OPT_A);
 	err = mod1.errcode;
 	if (err) {
 	    fprintf(stderr, "compute_first_stage F: lsq failed\n");
@@ -1485,7 +1524,8 @@ int ivreg_process_lists (const int *list, int **reglist, int **instlist)
  * %OPT_A if this is an auxiliary reression, %OPT_E if we're
  * estimating one equation within a system, %OPT_H to
  * add "hatlist" to model even under %OPT_E, %OPT_X to skip
- * the set of tests that are usually calculated.
+ * the set of tests that are usually calculated. Also %OPT_C
+ * for clustered standard errors.
  *
  * Estimate the model given in @list by means of Two-Stage Least
  * Squares.  If %OPT_E is given, fitted values from the first-stage
@@ -1497,19 +1537,18 @@ int ivreg_process_lists (const int *list, int **reglist, int **instlist)
 MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
 {
     MODEL tsls;
-    gretl_matrix *Q = NULL;
+    gretl_matrix *Q = NULL, *r = NULL;
     char *missmask = NULL;
     int orig_t1 = dset->t1, orig_t2 = dset->t2;
     int *reglist = NULL, *instlist = NULL;
     int *hatlist = NULL, *s2list = NULL;
     int *exolist = NULL, *endolist = NULL;
-    int *droplist = NULL;
-    int addconst = 0, addvars = 0;
-    int ninst = 0, nreg = 0;
+    int *idroplist = NULL;
+    int addconst = 0;
+    int nendo, nreg = 0;
     int orig_nvar = dset->v;
     int sysest = (opt & OPT_E);
     int no_tests = (opt & OPT_X);
-    int nendo, ev = 0;
     int OverIdRank = 0;
     int i, err = 0;
 
@@ -1553,9 +1592,14 @@ MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
     /* drop any vars that are all zero, and reshuffle the constant
        into first position among the independent vars 
     */
-    tsls_omitzero(reglist, dset);
+    tsls_omitzero(reglist, dset, missmask);
     reglist_check_for_const(reglist, dset);
-    tsls_omitzero(instlist, dset);
+    tsls_omitzero(instlist, dset, missmask);
+
+#if TDEBUG
+    printlist(reglist, "reglist");
+    printlist(instlist, "instlist");
+#endif
 
     /* Determine the list of variables (endolist) for which we need to
        obtain fitted values in the first stage.  Note that we accept
@@ -1568,6 +1612,10 @@ MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
 	goto bailout;
     }
 
+#if TDEBUG
+    printlist(endolist, "endolist");
+#endif
+
     if (endolist != NULL) {
 	nendo = endolist[0];
 	hatlist = gretl_list_copy(endolist);
@@ -1580,7 +1628,7 @@ MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
     }
 
     if (!err) {
-	Q = tsls_Q(instlist, &droplist, dset, missmask, &err);
+	Q = tsls_Q(instlist, &idroplist, dset, missmask, &err);
     }
 
     if (err) {
@@ -1588,11 +1636,10 @@ MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
     } 
 
     /* check (again) for order condition: we do this after tsls_Q in case
-       any vars get dropped at that stage
+       any instruments get dropped at that stage
     */
-    ninst = instlist[0];
     nreg = reglist[0] - 1;
-    OverIdRank = ninst - nreg;
+    OverIdRank = instlist[0] - nreg;
     if (OverIdRank < 0) {
         gretl_errmsg_sprintf(_("The order condition for identification is not satisfied.\n"
 			       "At least %d more instruments are needed."), -OverIdRank);
@@ -1600,47 +1647,47 @@ MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
 	goto bailout;
     }
 
+#if TDEBUG
+    printlist(endolist, "endolist, after tsls_Q");
+    fprintf(stderr, "nreg = %d, OverIdRank = %d\n", nreg, OverIdRank);
+#endif
+
     /* allocate storage for fitted vars (etc.), if needed */
-    addvars = nendo;
-    if (addvars > 0) {
-	err = dataset_add_series(addvars, dset);
+    if (nendo > 0) {
+	err = dataset_add_series(nendo, dset);
+	if (!err) {
+	    r = gretl_vector_alloc(Q->cols);
+	    if (r == NULL) {
+		err = E_ALLOC;
+	    }
+	}
 	if (err) {
 	    goto bailout;
 	}
-    } 
-
-    /* prepare for first-stage F-test, if just one endogenous regressor */
-    if (nendo == 1) {
-	ev = endolist[1];
-    }      
+    }
 
     /* 
        Deal with the variables for which instruments are needed: loop
        across the list of variables to be instrumented (endolist),
        form the fitted values as QQ'x_i, and add these fitted values
-       into the data array Z.
+       into the data array, dset->Z.
     */
 
     for (i=0; i<nendo; i++) {
 	int v0 = endolist[i+1];
 	int v1 = orig_nvar + i;
-
+	
 	/* form xhat_i = QQ'x_i */
-	err = tsls_form_xhat(Q, dset->Z[v0], dset->Z[v1], dset, 
-			     missmask);
+	err = tsls_form_xhat(Q, r, dset, v0, v1, missmask);
 	if (err) {
 	    goto bailout;
 	}
 
-	/* name the fitted series according to the original */
-	strcpy(dset->varname[v1], "h_");
-	strncat(dset->varname[v1], dset->varname[v0], VNAMELEN - 3);
-
 	/* substitute v1 into the right place in the second-stage
 	   regression list */
 	replace_list_element(s2list, v0, v1);
 
-	/* update hatlist */
+	/* and update hatlist */
 	hatlist[i+1] = v1;
     }
 
@@ -1651,14 +1698,33 @@ MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
 	goto bailout;
     }
 
+#if TDEBUG > 1
+    PRN *prn = gretl_print_new(GRETL_PRINT_STDERR, &err);
+
+    printmodel(&tsls, dset, OPT_S, prn);
+    gretl_print_destroy(prn);
+#endif
+
+    if (tsls.list[0] < s2list[0]) {
+	/* Were collinear regressors dropped? If so, adjustments are needed */
+	OverIdRank += s2list[0] - tsls.list[0];
+	reglist_remove_redundant_vars(&tsls, s2list, reglist, endolist,
+				      hatlist);
+	nendo = endolist[0];
+#if TDEBUG
+	fprintf(stderr, "tsls: dropped collinear vars\n");
+#endif
+    }
+
     /* record the number of instruments used */
-    gretl_model_set_int(&tsls, "ninst", ninst);
+    gretl_model_set_int(&tsls, "ninst", instlist[0]);
 
     if (!no_tests) {
 	if (!sysest || (opt & OPT_H)) {
 	    if (nendo == 1) {
 		/* handles robust estimation, for single endogenous regressor */
-		compute_first_stage_F(&tsls, ev, reglist, instlist, dset, opt);
+		compute_first_stage_F(&tsls, endolist, reglist, instlist, 
+				      dset, opt);
 	    } else if (!(opt & OPT_R) && nendo > 0) {
 		/* at present, only handles case of i.i.d. errors */
 		compute_stock_yogo(&tsls, endolist, instlist, hatlist, dset);
@@ -1666,21 +1732,17 @@ MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
 	}
     } 
 
-    if (tsls.list[0] < s2list[0]) {
-	/* Were collinear regressors dropped? If so, adjustments are needed */
-	OverIdRank += s2list[0] - tsls.list[0];
-	reglist_remove_redundant_vars(&tsls, s2list, reglist);
-#if TDEBUG
-	fprintf(stderr, "tsls: dropped collinear vars\n");
-#endif
-    }
-
     if (nendo > 0) {
 	/* special: we need to use the original RHS vars to compute
 	   residuals and associated statistics */
 	tsls_residuals(&tsls, reglist, dset, opt);
     }
 
+    if (opt & OPT_C) {
+	/* clustered implies robust */
+	opt |= OPT_R;
+    }
+
     if (opt & OPT_R) {
 	/* robust standard errors called for */
 	qr_tsls_vcv(&tsls, dset, opt);
@@ -1714,8 +1776,8 @@ MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
     */
     tsls_recreate_full_list(&tsls, reglist, instlist);
 
-    if (droplist != NULL) {
-	gretl_model_set_list_as_data(&tsls, "inst_droplist", droplist); 
+    if (idroplist != NULL) {
+	gretl_model_set_list_as_data(&tsls, "inst_droplist", idroplist); 
     }
 
 #if 0
@@ -1727,6 +1789,7 @@ MODEL tsls (const int *list, DATASET *dset, gretlopt opt)
  bailout:
 
     gretl_matrix_free(Q);
+    gretl_matrix_free(r);
     free(missmask);
 
     if (err && !tsls.errcode) {
