@@ -42,8 +42,6 @@ struct user_matrix_ {
     int level;
     UMFlags flags;
     char name[VNAMELEN];
-    char **colnames;
-    char **rownames;
 };
 
 static user_matrix **matrices;
@@ -98,8 +96,6 @@ static user_matrix *user_matrix_new (gretl_matrix *M, const char *name)
     u->flags = 0;
     *u->name = '\0';
     strncat(u->name, name, VNAMELEN - 1);
-    u->colnames = NULL;
-    u->rownames = NULL;
 
     return u;
 }
@@ -121,27 +117,44 @@ static int matrix_is_saved (const gretl_matrix *m)
     return 0;
 }
 
-/* callback for adding icons representing matrices to the GUI
-   session window */
+/* callbacks for adding or deleting icons representing 
+   matrices in the GUI session window */
 
-static void (*matrix_add_callback)(void);
+static MATRIX_ADD_FUNC matrix_add_callback;
+static MATRIX_DEL_FUNC matrix_delete_callback;
 
 /**
  * set_matrix_add_callback:
- * @callback: function function to out in place.
+ * @callback: function function to put in place.
  *
  * Sets the callback function to be invoked when a user-defined
- * matrix is added to the stack of saved objects.  Intended
- * for synchronizing the GUI program with the saved object
+ * matrix is added to the stack of saved objects.  
+ * Intended for synchronizing the GUI program with the saved object
  * state.
  */
 
-void set_matrix_add_callback (void (*callback))
+void set_matrix_add_callback (MATRIX_ADD_FUNC callback)
 {
     matrix_add_callback = callback; 
 }
 
-static user_matrix *real_user_matrix_add (gretl_matrix *M, const char *name,
+/**
+ * set_matrix_delete_callback:
+ * @callback: function function to put in place.
+ *
+ * Sets the callback function to be invoked when a user-defined
+ * matrix is to be removed from the stack of saved objects.  
+ * Intended for synchronizing the GUI program with the saved object
+ * state.
+ */
+
+void set_matrix_delete_callback (MATRIX_DEL_FUNC callback)
+{
+    matrix_delete_callback = callback; 
+}
+
+static user_matrix *real_user_matrix_add (gretl_matrix *M, 
+					  const char *name,
 					  int callback_ok)
 {
     user_matrix **tmp;
@@ -185,7 +198,7 @@ static user_matrix *real_user_matrix_add (gretl_matrix *M, const char *name,
 
     if (callback_ok && matrix_add_callback != NULL && 
 	gretl_function_depth() == 0) {
-	(*matrix_add_callback)();
+	(*matrix_add_callback)(name);
     }
 
     return u;
@@ -275,45 +288,24 @@ user_matrix *get_user_matrix_by_index (int idx)
     }
 }
 
-static void usermat_destroy_names (user_matrix *u, int which)
-{
-    if (u->M != NULL) {
-	if ((which == 2 || which == 0) && u->colnames != NULL) {
-	    free_strings_array(u->colnames, u->M->cols);
-	    u->colnames = NULL;
-	}
-	if ((which == 2 || which == 1) && u->rownames != NULL) {
-	    free_strings_array(u->rownames, u->M->rows);
-	    u->rownames = NULL;
-	}	
-    }	    
-}
-
-static void usermat_destroy_matrix (user_matrix *u)
-{
-    if (!data_is_bundled(u->M)) {
-	gretl_matrix_free(u->M);
-    }
-}
-
 int user_matrix_replace_matrix (user_matrix *u, gretl_matrix *M)
 {
     if (u == NULL) {
 	return E_UNKVAR;
     }
 
-    if (M != u->M) {
 #if MDEBUG
-	fprintf(stderr, " freeing u->M at %p, replacing with "
-		"matrix at %p\n", u->M, M);
+    fprintf(stderr, "user_matrix_replace_matrix (%s)\n", u->name);
 #endif
-	if (u->colnames != NULL && M->cols != u->M->cols) {
-	    usermat_destroy_names(u, 0);
-	}
-	if (u->rownames != NULL && M->rows != u->M->rows) {
-	    usermat_destroy_names(u, 1);
-	}
-	usermat_destroy_matrix(u);
+
+    if (M != u->M) {
+	if (!data_is_bundled(u->M)) {
+#if MDEBUG
+	    fprintf(stderr, " %s: freeing matrix at %p, replacing with "
+		    "matrix at %p\n", u->name, u->M, M);
+#endif
+	    gretl_matrix_free(u->M);
+	}	
 	u->M = M;
     }
 
@@ -422,7 +414,6 @@ gretl_matrix *steal_matrix_by_name (const char *name)
 	user_matrix *u = get_user_matrix_by_name(name);
 
 	if (u != NULL) {
-	    usermat_destroy_names(u, 2);
 	    ret = u->M;
 	    u->M = NULL;
 	}
@@ -769,6 +760,51 @@ static int get_slices (matrix_subspec *spec,
     return err;
 }
 
+int assign_scalar_to_submatrix (gretl_matrix *M, double x,
+				matrix_subspec *spec)
+{
+    int mr = gretl_matrix_rows(M);
+    int mc = gretl_matrix_cols(M);
+    int i, err = 0;
+
+    if (spec == NULL) {
+	fprintf(stderr, "matrix_replace_submatrix: spec is NULL!\n");
+	return E_DATA;
+    }
+
+    if (spec->type[0] == SEL_DIAG) {
+	int n = (mr < mc)? mr : mc;
+
+	for (i=0; i<n; i++) {
+	    gretl_matrix_set(M, i, i, x);
+	}
+	return 0;
+    }
+
+    if (spec->rslice == NULL && spec->cslice == NULL) {
+	/* parse mspec into lists of affected rows and columns */
+	err = get_slices(spec, M);
+    }
+
+    if (!err) {
+	int sr = (spec->rslice == NULL)? mr : spec->rslice[0];
+	int sc = (spec->cslice == NULL)? mc : spec->cslice[0];
+	int j, l, k = 0;
+	int mi, mj;
+
+	for (i=0; i<sr; i++) {
+	    mi = (spec->rslice == NULL)? k++ : spec->rslice[i+1] - 1;
+	    l = 0;
+	    for (j=0; j<sc; j++) {
+		mj = (spec->cslice == NULL)? l++ : spec->cslice[j+1] - 1;
+		gretl_matrix_set(M, mi, mj, x);
+	    }
+	}
+    }
+
+    return err;
+}
+
 /* @M is the target for partial replacement, @S is the source to
    substitute, and @spec tells how/where to make the
    substitution.
@@ -921,11 +957,12 @@ gretl_matrix *matrix_get_submatrix (const gretl_matrix *M,
 	}
     }
 
-    if (S != NULL && S->rows == M->rows) {
-	if (M->t1 >= 0 && M->t2 > M->t1) {
-	    S->t1 = M->t1;
-	    S->t2 = M->t2;
-	}
+    if (S != NULL && S->rows == M->rows && gretl_matrix_is_dated(M)) {
+	int mt1 = gretl_matrix_get_t1(M);
+	int mt2 = gretl_matrix_get_t2(M);
+
+	gretl_matrix_set_t1(S, mt1);
+	gretl_matrix_set_t2(S, mt2);
     }
 
     return S;
@@ -946,7 +983,7 @@ double matrix_get_element (const gretl_matrix *M, int i, int j,
 	*err = E_DATA;
     } else if (i < 0 || i >= M->rows || j < 0 || j >= M->cols) {
 	gretl_errmsg_sprintf(_("Index value %d is out of bounds"), 
-			     (i < 0 || i >= M->rows)? (j+1) : (i+1));
+			     (i < 0 || i >= M->rows)? (i+1) : (j+1));
 	*err = 1;
     } else {
 	x = gretl_matrix_get(M, i, j);
@@ -1029,13 +1066,14 @@ static void destroy_user_matrix (user_matrix *u)
 	return;
     }
 
+    if (!data_is_bundled(u->M)) {
 #if MDEBUG
-    fprintf(stderr, "destroy_user_matrix: freeing matrix at %p...", 
-	    (void *) u->M);
+	fprintf(stderr, "destroy_user_matrix %s: freeing matrix at %p...\n", 
+		u->name, (void *) u->M);
 #endif
+	gretl_matrix_free(u->M);
+    }
 
-    usermat_destroy_names(u, 2);
-    usermat_destroy_matrix(u);
     free(u);
 
 #if MDEBUG
@@ -1205,9 +1243,16 @@ int user_matrix_destroy (user_matrix *u)
 
 int user_matrix_destroy_by_name (const char *name, PRN *prn)
 {
-    user_matrix *u = get_user_matrix_by_name(name);
+    user_matrix *u;
     int err;
 
+    if (matrix_delete_callback != NULL && gretl_function_depth() == 0) {
+	/* run this through the GUI program to ensure that
+	   things stay in sync */
+	return matrix_delete_callback(name);
+    }
+
+    u = get_user_matrix_by_name(name);
     err = user_matrix_destroy(u);
 
     if (!err && prn != NULL && gretl_messages_on()) {
@@ -1218,7 +1263,7 @@ int user_matrix_destroy_by_name (const char *name, PRN *prn)
     return err;
 }
 
-int umatrix_set_names_from_string (const gretl_matrix *M, 
+int umatrix_set_names_from_string (gretl_matrix *M, 
 				   const char *s,
 				   int byrow)
 {
@@ -1232,7 +1277,11 @@ int umatrix_set_names_from_string (const gretl_matrix *M,
     n = (byrow)? M->rows : M->cols;
 
     if (s == NULL || *s == '\0') {
-	usermat_destroy_names(u, byrow);
+	if (byrow) {
+	    gretl_matrix_set_rownames(M, NULL);
+	} else {
+	    gretl_matrix_set_colnames(M, NULL);
+	}
     } else {
 	char **S;
 	int ns;
@@ -1243,20 +1292,17 @@ int umatrix_set_names_from_string (const gretl_matrix *M,
 	} else if (ns != n) {
 	    err = E_NONCONF;
 	    free_strings_array(S, ns);
+	} else if (byrow) {
+	    gretl_matrix_set_rownames(M, S);
 	} else {
-	    usermat_destroy_names(u, byrow);
-	    if (byrow) {
-		u->rownames = S;
-	    } else {
-		u->colnames = S;
-	    }
-	}
+	    gretl_matrix_set_colnames(M, S);
+	} 
     }
 
     return err;
 }
 
-int umatrix_set_names_from_list (const gretl_matrix *M, 
+int umatrix_set_names_from_list (gretl_matrix *M, 
 				 const int *list,
 				 const DATASET *dset,
 				 int byrow)
@@ -1271,7 +1317,11 @@ int umatrix_set_names_from_list (const gretl_matrix *M,
     n = (byrow)? M->rows : M->cols;
 
     if (list == NULL || list[0] == 0) {
-	usermat_destroy_names(u, byrow);
+	if (byrow) {
+	    gretl_matrix_set_rownames(M, NULL);
+	} else {
+	    gretl_matrix_set_colnames(M, NULL);
+	}
     } else if (list[0] != n) {
 	err = E_NONCONF;
     } else {
@@ -1290,46 +1340,30 @@ int umatrix_set_names_from_list (const gretl_matrix *M,
 
 	if (err) {
 	    free_strings_array(S, n);
+	} else if (byrow) {
+	    gretl_matrix_set_rownames(M, S);
 	} else {
-	    usermat_destroy_names(u, byrow);
-	    if (byrow) {
-		u->rownames = S;
-	    } else {
-		u->colnames = S;
-	    }
+	    gretl_matrix_set_colnames(M, S);
 	}
     }
 
     return err;
 }
 
-const char **user_matrix_get_names (const gretl_matrix *M,
-				    int byrow)
-{
-    user_matrix *u = get_user_matrix_by_data(M);
-
-    if (u == NULL) {
-	return NULL;
-    } else if (byrow) {
-	return (const char **) u->rownames;
-    } else {
-	return (const char **) u->colnames;
-    }
-}
-
 char *user_matrix_get_column_name (const gretl_matrix *M, int col,
 				   int *err)
 {
-    user_matrix *u = get_user_matrix_by_data(M);
     char *ret = NULL;
 
-    if (u == NULL || col < 1 || col > M->cols) {
+    if (M == NULL || col < 1 || col > M->cols) {
 	*err = E_DATA;
     } else {
-	if (u->colnames == NULL) {
+	const char **S = gretl_matrix_get_colnames(M);
+
+	if (S == NULL) {
 	    ret = gretl_strdup("");
 	} else {
-	    ret = gretl_strdup(u->colnames[col-1]);
+	    ret = gretl_strdup(S[col-1]);
 	}
 	if (ret == NULL) {
 	    *err = E_ALLOC;
@@ -2038,6 +2072,7 @@ gretl_matrix *user_gensymm_eigenvals (const gretl_matrix *A,
 static void xml_put_user_matrix (user_matrix *u, FILE *fp)
 {
     gretl_matrix *M;
+    const char **S;
     int i, j;
 
     if (u == NULL || u->M == NULL) {
@@ -2049,18 +2084,22 @@ static void xml_put_user_matrix (user_matrix *u, FILE *fp)
     fprintf(fp, "<gretl-matrix name=\"%s\" rows=\"%d\" cols=\"%d\"", 
 	    u->name, M->rows, M->cols);
 
-    if (u->colnames != NULL) {
+    S = gretl_matrix_get_colnames(M);
+
+    if (S != NULL) {
 	fputs(" colnames=\"", fp);
 	for (j=0; j<M->cols; j++) {
-	    fputs(u->colnames[j], fp);
+	    fputs(S[j], fp);
 	    fputc((j < M->cols - 1)? ' ' : '"', fp);
 	}
     } 
 
-    if (u->rownames != NULL) {
+    S = gretl_matrix_get_rownames(M);
+
+    if (S != NULL) {
 	fputs(" rownames=\"", fp);
 	for (j=0; j<M->rows; j++) {
-	    fputs(u->rownames[j], fp);
+	    fputs(S[j], fp);
 	    fputc((j < M->rows - 1)? ' ' : '"', fp);
 	}
     }     
