@@ -98,6 +98,9 @@ int user_matrix_replace_matrix (user_matrix *u, gretl_matrix *M);
 int user_matrix_replace_matrix_by_name (const char *name, 
 					gretl_matrix *M);
 
+int assign_scalar_to_submatrix (gretl_matrix *M, double x,
+				matrix_subspec *spec);
+
 int user_matrix_replace_submatrix (const char *mname, 
 				   const gretl_matrix *S,
 				   matrix_subspec *spec);
@@ -108,18 +111,15 @@ int copy_named_matrix_as (const char *orig, const char *newname);
 
 int copy_matrix_as (const gretl_matrix *m, const char *newname);
 
-int umatrix_set_names_from_string (const gretl_matrix *M, 
+int umatrix_set_names_from_string (gretl_matrix *M, 
 				   const char *s,
 				   int byrow);
 
-int umatrix_set_names_from_list (const gretl_matrix *M, 
+int umatrix_set_names_from_list (gretl_matrix *M, 
 				 const int *list,
 				 const DATASET *dset,
 				 int byrow);
 
-const char **user_matrix_get_names (const gretl_matrix *M,
-				    int byrow);
-
 char *user_matrix_get_column_name (const gretl_matrix *M, int col,
 				   int *err);
 
@@ -196,6 +196,12 @@ int matrix_XTX_in_place (gretl_matrix *m);
 
 void write_matrices_to_file (FILE *fp);
 
-void set_matrix_add_callback (void (*callback));
+typedef void (*MATRIX_ADD_FUNC) (const char *);
+typedef int (*MATRIX_DEL_FUNC) (const char *);
+
+void set_matrix_add_callback (MATRIX_ADD_FUNC);
+
+void set_matrix_delete_callback (MATRIX_DEL_FUNC);
+
 
 #endif /* USERMAT_H_ */
