@@ -182,7 +182,6 @@ void gretl_VAR_clear (GRETL_VAR *var)
     var->detflags = 0;
     var->robust = 0;
     var->LBs = 0;
-    var->ycols = 0;
     var->xcols = 0;
 
     var->lags = NULL;
@@ -217,17 +216,18 @@ void gretl_VAR_clear (GRETL_VAR *var)
 #define lag_wanted(v, i) (v->lags == NULL || in_gretl_list(v->lags, i))
 
 /* Construct the common X matrix (composed of lags of the core
-   variables plus other terms if applicable)
+   variables plus other terms if applicable). This is in 
+   common between VARs and VECMs
 */
 
 void VAR_fill_X (GRETL_VAR *v, int p, const DATASET *dset)
 {
+    const double *x;
     int diff = (v->ci == VECM);
     int i, j, s, t, vi;
-    int k = 0; /* X column */
+    int k = 0; /* X column index */
 
     /* const first */
-
     if (v->detflags & DET_CONST) {
 	s = 0;
 	for (t=v->t1; t<=v->t2; t++) {
@@ -237,20 +237,19 @@ void VAR_fill_X (GRETL_VAR *v, int p, const DATASET *dset)
     }    
 
     /* add lagged Ys */
-
     for (i=0; i<v->neqns; i++) {
 	vi = v->ylist[i+1];
 	for (j=1; j<=p; j++) {
 	    if (!lag_wanted(v, j)) {
 		continue;
 	    }
+	    x = dset->Z[vi];
 	    s = 0;
 	    for (t=v->t1; t<=v->t2; t++) {
 		if (diff) {
-		    gretl_matrix_set(v->X, s++, k, 
-				     dset->Z[vi][t-j] - dset->Z[vi][t-j-1]);
+		    gretl_matrix_set(v->X, s++, k, x[t-j] - x[t-j-1]);
 		} else {
-		    gretl_matrix_set(v->X, s++, k, dset->Z[vi][t-j]);
+		    gretl_matrix_set(v->X, s++, k, x[t-j]);
 		}
 	    }
 	    k++;
@@ -258,7 +257,6 @@ void VAR_fill_X (GRETL_VAR *v, int p, const DATASET *dset)
     }
 
     /* add any exogenous vars */
-
     if (v->xlist != NULL) {
 	for (i=1; i<=v->xlist[0]; i++) {
 	    vi = v->xlist[i];
@@ -271,14 +269,18 @@ void VAR_fill_X (GRETL_VAR *v, int p, const DATASET *dset)
     }
 
     /* add other deterministic terms */
-
     if (v->detflags & DET_SEAS) {
 	int per = get_subperiod(v->t1, dset, NULL);
 	int pd1 = dset->pd - 1;
 	double s0, s1;
 
-	s1 = (v->ci == VECM)? 1.0 - 1.0 / dset->pd : 1.0;
-	s0 = (v->ci == VECM)? s1 - 1.0 : 0.0;
+	if (v->ci == VECM) {
+	    s1 = 1 - 1.0 / dset->pd;
+	    s0 = s1 - 1;
+	} else {
+	    s1 = 1;
+	    s0 = 0;
+	}	    
 	
 	for (t=0; t<v->T; t++) {
 	    for (i=0; i<pd1; i++) {
@@ -307,49 +309,20 @@ void VAR_fill_X (GRETL_VAR *v, int p, const DATASET *dset)
 #endif
 }
 
-/* construct the matrix of dependent variables for VAR or
-   VECM estimation */
+/* construct the matrix of dependent variables for a plain VAR */
 
-static void VAR_fill_Y (GRETL_VAR *v, int mod, const DATASET *dset)
+static void VAR_fill_Y (GRETL_VAR *v, const DATASET *dset)
 {
     int i, vi, s, t;
-    int k = 0;
 
     for (i=0; i<v->neqns; i++) {
 	vi = v->ylist[i+1];
 	s = 0;
 	for (t=v->t1; t<=v->t2; t++) {
-	    if (mod == DIFF) {
-		gretl_matrix_set(v->Y, s++, k, dset->Z[vi][t] - dset->Z[vi][t-1]);
-	    } else if (mod == LAGS) {
-		gretl_matrix_set(v->Y, s++, k, dset->Z[vi][t-1]);
-	    } else {
-		gretl_matrix_set(v->Y, s++, k, dset->Z[vi][t]);
-	    }
+	    gretl_matrix_set(v->Y, s++, i, dset->Z[vi][t]);
 	}
-	k++;
     }
 
-    if (mod == LAGS && auto_restr(v)) {
-	int trend = (v->jinfo->code == J_REST_TREND);
-
-	for (t=0; t<v->T; t++) {
-	    gretl_matrix_set(v->Y, t, k, (trend)? (v->t1 + t) : 1);
-	}
-	k++;
-    }
-
-    if (mod == LAGS && v->rlist != NULL) {
-	for (i=0; i<v->rlist[0]; i++) {
-	    vi = v->rlist[i+1];
-	    s = 0;
-	    for (t=v->t1; t<=v->t2; t++) {
-		gretl_matrix_set(v->Y, s++, k, dset->Z[vi][t-1]);
-	    }
-	    k++;
-	}
-    }  
-
     if (v->Y != NULL) {
 	gretl_matrix_set_t1(v->Y, v->t1);
 	gretl_matrix_set_t2(v->Y, v->t2);
@@ -360,23 +333,23 @@ static void VAR_fill_Y (GRETL_VAR *v, int mod, const DATASET *dset)
 #endif
 }
 
-#define VECM_PACK 0 /* not just yet */
-
-#if VECM_PACK
+/* construct the combined matrix of dependent variables for a VECM */
 
 static void VECM_fill_Y (GRETL_VAR *v, const DATASET *dset,
 			 gretl_matrix *Y)
 {
+    const double *yi;
     int i, vi, s, t;
     int k = 0;
 
     for (i=0; i<v->neqns; i++) {
 	vi = v->ylist[i+1];
+	yi = dset->Z[vi];
 	k = i + v->neqns;
 	s = 0;
 	for (t=v->t1; t<=v->t2; t++) {
-	    gretl_matrix_set(Y, s, i, dset->Z[vi][t] - dset->Z[vi][t-1]);
-	    gretl_matrix_set(Y, s, k, dset->Z[vi][t-1]);
+	    gretl_matrix_set(Y, s, i, yi[t] - yi[t-1]);
+	    gretl_matrix_set(Y, s, k, yi[t-1]);
 	    s++;
 	}
     }
@@ -386,23 +359,30 @@ static void VECM_fill_Y (GRETL_VAR *v, const DATASET *dset,
 
 	k++;
 	for (t=0; t<v->T; t++) {
-	    gretl_matrix_set(Y, t, k, (trend)? (v->t1 + t) : 1);
+	    gretl_matrix_set(Y, t, k, trend ? (v->t1 + t) : 1);
 	}
     }
 
     if (v->rlist != NULL) {
+	/* There's room for debate here (?) over whether we should
+	   use the current value or the first lag of restricted
+	   exogenous variables. But using the current value agrees
+	   with Ox. See also VAR_set_sample().
+	*/
 	for (i=0; i<v->rlist[0]; i++) {
 	    vi = v->rlist[i+1];
 	    k++;
 	    s = 0;
 	    for (t=v->t1; t<=v->t2; t++) {
-		gretl_matrix_set(Y, s++, k, dset->Z[vi][t-1]);
+		gretl_matrix_set(Y, s++, k, dset->Z[vi][t]); /* was t-1 */
 	    }
 	}
-    }  
-}
+    } 
 
-#endif
+#if VDEBUG
+    gretl_matrix_print(Y, "VECM Y");
+#endif 
+}
 
 /* Split the user-supplied list, if need be, and construct the lists
    of endogenous and (possibly) exogenous vars.  Note that
@@ -501,16 +481,16 @@ static int VAR_set_sample (GRETL_VAR *v, const DATASET *dset)
 	    }
 	}
 
-	/* Note that in estimating a VECM we need the first lag of the
-	   level of each restricted exogenous var to serve as
-	   dependent variable in one of the initial OLS equations.
+	/* In estimating a VECM we need the level (or first lag?)
+	   of each restricted exogenous var to serve as dependent
+	   variable in one of the initial OLS equations. See also
+	   VECM_fill_Y().
 	*/
 
-	s = t - 1;
 	if (v->rlist != NULL && !miss) {
 	    for (i=1; i<=v->rlist[0] && !miss; i++) {
 		vi = v->rlist[i];
-		if (na(dset->Z[vi][t]) || s < 0 || na(dset->Z[vi][s])) {
+		if (na(dset->Z[vi][t])) {
 		    v->t1 += 1;
 		    miss = 1;
 		}
@@ -647,45 +627,29 @@ static int VAR_check_df_etc (GRETL_VAR *v, const DATASET *dset,
     return err;
 }
 
-/* Note: if @v is a VECM, and it includes a restricted const
-   or trend, we make the Y and B matrices big enough to
-   accommodate the additional restricted term.
-*/
-
-static int VAR_add_basic_matrices (GRETL_VAR *v, gretlopt opt)
+static int VAR_add_basic_matrices (GRETL_VAR *v)
 {
-    int k = v->neqns;
+    int err = 0;
 
-    if (v->ci == VECM && (opt & (OPT_A | OPT_R))) {
-	/* allow for restricted const/trend */
-	k++;
-    }
-
-    /* restricted exog vars? */
-    if (v->rlist != NULL) {
-	k += v->rlist[0];
-    } 
-
-    v->Y = gretl_matrix_alloc(v->T, k);
+    v->Y = gretl_matrix_alloc(v->T, v->neqns);
     v->E = gretl_matrix_alloc(v->T, v->neqns);
-    if (v->Y == NULL || v->E == NULL) {
-	return E_ALLOC;
-    } 
-    
-    /* record max. cols of Y matrix */
-    v->ycols = k;
 
-    if (v->ncoeff > 0) {
-	v->X = gretl_matrix_alloc(v->T, v->ncoeff);
-	v->B = gretl_matrix_alloc(v->ncoeff, k);
-	if (v->X == NULL || v->B == NULL) { 
-	    return E_ALLOC;
-	}
-	/* record max. cols of X matrix */
-	v->xcols = v->ncoeff;
+    if (v->Y == NULL || v->E == NULL) {
+	err = E_ALLOC;
     }
 
-    return 0;
+    if (!err && v->ncoeff > 0) {
+	v->X = gretl_matrix_alloc(v->T, v->ncoeff);
+	v->B = gretl_matrix_alloc(v->ncoeff, v->neqns);
+	if (v->X == NULL || v->B == NULL) { 
+	    err = E_ALLOC;
+	} else {
+	    /* record the number of columns of X */
+	    v->xcols = v->X->cols;
+	}
+    }
+
+    return err;
 }
 
 static void set_to_NA (double *x, int n)
@@ -763,7 +727,7 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
     }
 
     if (!err) {
-	err = VAR_add_basic_matrices(var, opt);
+	err = VAR_add_basic_matrices(var);
     }
 
     if (!err && var->ci == VAR) {
@@ -810,7 +774,7 @@ static GRETL_VAR *gretl_VAR_new (int code, int order, int rank,
     if (!err) {
 	if (var->ci == VAR) {
 	    /* this is deferred for a VECM */
-	    VAR_fill_Y(var, 0, dset);
+	    VAR_fill_Y(var, dset);
 	}
 	VAR_fill_X(var, order, dset);
     }
@@ -845,6 +809,10 @@ static void johansen_info_free (JohansenInfo *jv)
     gretl_matrix_free(jv->Ra);
     gretl_matrix_free(jv->qa);
 
+    gretl_matrix_free(jv->YY);
+    gretl_matrix_free(jv->RR);
+    gretl_matrix_free(jv->BB);
+
     free(jv);
 }
 
@@ -1314,26 +1282,18 @@ static void VAR_dw_rho (MODEL *pmod)
     pmod->rho = ut1 / u11;
 }
 
-/* set basic model statistics when estimation has been
+/* set basic per-model statistics after estimation has been
    done by matrix methods */
 
 int set_VAR_model_stats (GRETL_VAR *var, int i)
 {
     MODEL *pmod = var->models[i];
-    double *y = NULL;
+    const double *y;
     double u, x, SSR = 0, TSS = 0;
     int t;
 
-    y = malloc(var->T * sizeof *y);
-    if (y == NULL) {
-	pmod->ybar = pmod->sdy = NADBL;
-	pmod->rsq = NADBL;
-	return E_ALLOC;
-    }
-
-    for (t=0; t<var->T; t++) {
-	y[t] = gretl_matrix_get(var->Y, t, i);
-    }
+    /* get the appropriate column of var->Y as a plain array */
+    y = var->Y->val + i * var->T;
 
     pmod->ybar = gretl_mean(0, var->T - 1, y);
     pmod->sdy = gretl_stddev(0, var->T - 1, y);
@@ -1362,8 +1322,6 @@ int set_VAR_model_stats (GRETL_VAR *var, int i)
 
     VAR_dw_rho(pmod);
 
-    free(y);
-
     return 0;
 }
 
@@ -1429,8 +1387,8 @@ const gretl_matrix *
 gretl_VAR_get_residual_matrix (const GRETL_VAR *var)
 {
     if (var->E != NULL) {
-	var->E->t1 = var->t1;
-	var->E->t2 = var->t2;
+	gretl_matrix_set_t1(var->E, var->t1);
+	gretl_matrix_set_t2(var->E, var->t2);
     }
 
     return var->E;
@@ -1447,8 +1405,8 @@ gretl_VAR_get_fitted_matrix (const GRETL_VAR *var)
 	Yh = gretl_matrix_copy(var->Y);
 	if (Yh != NULL) {
 	    gretl_matrix_subtract_from(Yh, var->E);
-	    Yh->t1 = var->t1;
-	    Yh->t2 = var->t2;
+	    gretl_matrix_set_t1(Yh, var->t1);
+	    gretl_matrix_set_t2(Yh, var->t2);
 	}
     }
 
@@ -2490,6 +2448,7 @@ static int *maybe_get_lags_list (int order, int *err)
  * @list: specification for the first model in the set.
  * @dset: dataset struct.
  * @opt: if includes %OPT_R, use robust VCV;
+ *       if includes %OPT_H, use HAC VCV;
  *       if includes %OPT_I, print impulse responses;
  *       if includes %OPT_F, print forecast variance decompositions;
  *       if includes %OPT_D, add seasonal dummies;
@@ -2686,25 +2645,34 @@ johansen_estimate_complete (GRETL_VAR *jvar, gretl_restriction *rset,
 /* N.B. we allow for the possibility that this allocation has
    already been done */
 
-static int allocate_johansen_residual_matrices (GRETL_VAR *v)
+static int allocate_johansen_extra_matrices (GRETL_VAR *v)
 {
-    if (v->jinfo->R0 != NULL && v->jinfo->S00 != NULL) {
+    if (v->jinfo->R0 != NULL && 
+	v->jinfo->S00 != NULL && 
+	v->jinfo->YY != NULL) {
 	return 0;
     } else {
-	int p = v->neqns;
-	int p1 = p + n_restricted_terms(v);
+	int p0 = v->neqns;
+	int p1 = p0 + n_restricted_terms(v);
+	int p = p0 + p1;
 
 	clear_gretl_matrix_err();
 
 	if (v->jinfo->R0 == NULL) {
-	    v->jinfo->R0 = gretl_matrix_alloc(v->T, p);
+	    v->jinfo->R0 = gretl_matrix_alloc(v->T, p0);
 	    v->jinfo->R1 = gretl_matrix_alloc(v->T, p1);
 	}
 
 	if (v->jinfo->S00 == NULL) {
-	    v->jinfo->S00 = gretl_matrix_alloc(p, p);
+	    v->jinfo->S00 = gretl_matrix_alloc(p0, p0);
 	    v->jinfo->S11 = gretl_matrix_alloc(p1, p1);
-	    v->jinfo->S01 = gretl_matrix_alloc(p, p1);
+	    v->jinfo->S01 = gretl_matrix_alloc(p0, p1);
+	}
+
+	if (v->ncoeff > 0 && v->jinfo->YY == NULL) {
+	    v->jinfo->YY = gretl_matrix_alloc(v->T, p);
+	    v->jinfo->RR = gretl_matrix_alloc(v->T, p);
+	    v->jinfo->BB = gretl_matrix_alloc(v->X->cols, p);
 	}
 
 	return get_gretl_matrix_err();
@@ -2728,13 +2696,23 @@ static void johansen_fill_S_matrices (GRETL_VAR *v)
     gretl_matrix_divide_by_scalar(v->jinfo->S01, v->T);
 }
 
+/* We come here if there are no regressors at stage 1 of
+   the Johansen procedure. This happens if the associated
+   VAR is of order 1 (becomes order 0 on differencing)
+   and there are no unrestricted exogenous variables.
+   The residuals are then just the values of the
+   variables themselves.
+*/
+
 static int johansen_degenerate_stage_1 (GRETL_VAR *v, 
 					const DATASET *dset)
 {
     const double **Z = (const double **) dset->Z;
     gretl_matrix *R0 = v->jinfo->R0;
     gretl_matrix *R1 = v->jinfo->R1;
-    int i, vi, s, t;
+    int i, j, vi, s, t;
+
+    fprintf(stderr, "degenerate stage 1\n");
 
     for (i=0; i<v->neqns; i++) {
 	vi = v->ylist[i+1];
@@ -2757,9 +2735,10 @@ static int johansen_degenerate_stage_1 (GRETL_VAR *v,
     if (v->rlist != NULL) {
 	for (i=0; i<v->rlist[0]; i++) {
 	    vi = v->rlist[i+1];
+	    j = v->neqns + i + 1;
 	    s = 0;
 	    for (t=v->t1; t<=v->t2; t++) {
-		gretl_matrix_set(R1, s++, i, Z[vi][t-1]);
+		gretl_matrix_set(R1, s++, j, Z[vi][t]); /* was t-1 */
 	    }
 	}
     } 
@@ -2769,45 +2748,60 @@ static int johansen_degenerate_stage_1 (GRETL_VAR *v,
     return 0;
 }
 
-static void VECM_maybe_shrink_matrices (GRETL_VAR *v)
+static void print_stage_1_coeffs (GRETL_VAR *v, gretl_matrix *B,
+				  PRN *prn)
 {
-    if (v->Y->cols > v->neqns) {
-	gretl_matrix_reuse(v->Y, -1, v->neqns);
-	gretl_matrix_reuse(v->B, -1, v->neqns);
-    }
+    gretl_matrix tmp;
+
+    gretl_matrix_init(&tmp);
+    
+    tmp.rows = B->rows;
+    tmp.cols = v->neqns;
+    tmp.val = B->val;
+
+    gretl_matrix_print_to_prn(&tmp, "\nCoefficients, VAR in differences", 
+			      prn);
+
+    tmp.cols += n_restricted_terms(v);
+    tmp.val += v->neqns * tmp.rows;
+
+    gretl_matrix_print_to_prn(&tmp, "Coefficients, eqns in lagged levels", 
+			      prn);
 }
 
 #define JVAR_USE_SVD 1
 
-/* For Johansen analysis: estimate VAR in differences along with the
-   other auxiliary regressions required to compute the relevant
-   matrices of residuals, for concentration of the log-likelihood.
-   Then compute S00, S11, S01. See, for example, James Hamilton,
-   "Time Series Analysis", section 20.2.
-*/
-
-#if VECM_PACK
-
-static void 
-johansen_partition_residuals (GRETL_VAR *v, const gretl_matrix *R)
+static void johansen_partition_residuals (GRETL_VAR *v)
 {
     int n = v->neqns * v->T;
     int m = n + n_restricted_terms(v) * v->T;
+    gretl_matrix *R = v->jinfo->RR;
 
     memcpy(v->jinfo->R0->val, R->val, n * sizeof(double));
     memcpy(v->jinfo->R1->val, R->val + n, m * sizeof(double));
 }
 
+/* For Johansen analysis: estimate VAR in differences along with the
+   other auxiliary regressions required to compute the relevant
+   matrices of residuals, for concentration of the log-likelihood.
+   Then compute S00, S11, S01. See for example James Hamilton,
+   "Time Series Analysis", section 20.2.
+*/
+
 int johansen_stage_1 (GRETL_VAR *v, const DATASET *dset, 
 		      gretlopt opt, PRN *prn)
 {
     int err;
 
-    err = allocate_johansen_residual_matrices(v); 
+    err = allocate_johansen_extra_matrices(v); 
     if (err) {
 	return err;
     }
 
+#if 0
+    fprintf(stderr, "johansen_stage_1: ncoeff = %d\n", v->ncoeff);
+#endif
+
     if (v->ncoeff == 0) {
 	/* nothing to concentrate out */
 	if (opt & OPT_V) {
@@ -2815,117 +2809,30 @@ int johansen_stage_1 (GRETL_VAR *v, const DATASET *dset,
 	}
 	johansen_degenerate_stage_1(v, dset);
     } else {
-	gretl_matrix_block *MB;
-	gretl_matrix *Y, *R, *B;
-	int p = v->neqns;
-	int p1 = p + n_restricted_terms(v);
-	
-	MB = gretl_matrix_block_new(&Y, v->T, p + p1,
-				    &R, v->T, p + p1,
-				    &B, v->X->cols, p + p1,
-				    NULL);
-	if (MB == NULL) {
-	    err = E_ALLOC;
-	} else {
-	    VECM_fill_Y(v, dset, Y);
-	    
-	    err = gretl_matrix_multi_SVD_ols(Y, v->X, B, R, NULL);
+	gretl_matrix *Y = v->jinfo->YY;
+	gretl_matrix *B = v->jinfo->BB;
+	gretl_matrix *R = v->jinfo->RR;
 
-	    if (!err) {
-		if (opt & OPT_V) {
-		    gretl_matrix_print_to_prn(B, "\nCoefficients, stage 1", 
-					      prn);
-		}
-		johansen_partition_residuals(v, R);
-		johansen_fill_S_matrices(v);
+	VECM_fill_Y(v, dset, Y);
+
+#if JVAR_USE_SVD	    
+	err = gretl_matrix_multi_SVD_ols(Y, v->X, B, R, NULL);
+#else
+	err = gretl_matrix_multi_ols(Y, v->X, B, R, NULL);
+#endif
+
+	if (!err) {
+	    if (opt & OPT_V) {
+		print_stage_1_coeffs(v, B, prn);
 	    }
-
-	    gretl_matrix_block_destroy(MB);
+	    johansen_partition_residuals(v);
+	    johansen_fill_S_matrices(v);
 	}
     }
 
     return err;
 }
 
-#else /* not VECM_PACK */
-
-int johansen_stage_1 (GRETL_VAR *v, const DATASET *dset, 
-		      gretlopt opt, PRN *prn)
-{
-    int err;
-
-    err = allocate_johansen_residual_matrices(v); 
-    if (err) {
-	return err;
-    }
-
-    if (v->ncoeff == 0) {
-	/* nothing to concentrate out */
-	if (opt & OPT_V) {
-	    pputs(prn, "\nNo initial VAR estimation is required\n\n");
-	}
-	return johansen_degenerate_stage_1(v, dset);
-    }
-
-    VECM_maybe_shrink_matrices(v);
-
-    VAR_fill_Y(v, DIFF, dset);
-
-    /* (1) VAR in first differences */
-
-#if JVAR_USE_SVD
-    err = gretl_matrix_multi_SVD_ols(v->Y, v->X, v->B, 
-				     v->jinfo->R0, NULL);
-#else
-    err = gretl_matrix_multi_ols(v->Y, v->X, v->B, 
-				 v->jinfo->R0, NULL);
-#endif
-
-    if (!err && (opt & OPT_V)) {
-	gretl_matrix_print_to_prn(v->B, "\nCoefficients, VAR in differences", 
-				  prn);
-    }
-
-    if (v->ycols > v->Y->cols) {
-	/* re-expand to full size */
-	gretl_matrix_reuse(v->Y, -1, v->ycols);
-	gretl_matrix_reuse(v->B, -1, v->ycols);
-    }
-
-    /* (2) System with lagged levels on LHS (may include
-       restricted terms in trailing columns)
-    */
-
-    VAR_fill_Y(v, LAGS, dset);
-
-#if JVAR_USE_SVD
-    err = gretl_matrix_multi_SVD_ols(v->Y, v->X, v->B, 
-				     v->jinfo->R1, NULL);
-#else
-    err = gretl_matrix_multi_ols(v->Y, v->X, v->B, 
-				 v->jinfo->R1, NULL);
-#endif
-
-    if (!err && (opt & OPT_V)) {
-	gretl_matrix_print_to_prn(v->B, "Coefficients, eqns in lagged levels", 
-				  prn);
-    }
-
-    VECM_maybe_shrink_matrices(v);
-
-    if (!err) {
-	johansen_fill_S_matrices(v);
-    }
-
-#if VDEBUG
-    fprintf(stderr, "johansen_stage_1: returning err = %d\n", err);
-#endif    
-
-    return err;
-}
-
-#endif /* VECM_PACK or not */
-
 static gretlopt opt_from_jcode (JohansenCode jc)
 {
     gretlopt opt = OPT_NONE;
@@ -2989,6 +2896,9 @@ johansen_info_new (GRETL_VAR *var, int rank, gretlopt opt)
     jv->q = NULL;
     jv->Ra = NULL;
     jv->qa = NULL;
+    jv->YY = NULL;
+    jv->RR = NULL;
+    jv->BB = NULL;
 
     jv->ll0 = jv->prior_ll = NADBL;
     jv->lrdf = jv->prior_df = 0;
@@ -3171,7 +3081,10 @@ int johansen_test_simple (int order, const int *list,
  * @list: list of endogenous variables, possibly plus
  * exogenous variables.
  * @dset: dataset struct.
- * @opt:
+ * @opt: may include OPT_N ("nc"), OPT_R ("rc"), OPT_A ("crt"), 
+ * OPT_T ("ct"), OPT_D (include seasonals), OPT_F (show variance
+ * decompositions), OPT_I (show impulse responses), OPT_V
+ * (verbose operation), OPT_Q (quiet), OPT_S (silent).
  * @prn: gretl printing struct.
  * @err: location to receive error code.
  *
@@ -3528,11 +3441,6 @@ int gretl_VECM_rank (const GRETL_VAR *vecm)
     return r;
 }
 
-const int *gretl_VECM_list (const GRETL_VAR *vecm)
-{
-    return vecm->ylist;
-}
-
 int beta_restricted_VECM (const GRETL_VAR *vecm)
 {
     if (vecm->jinfo != NULL && vecm->jinfo->R != NULL) {
@@ -3635,9 +3543,11 @@ VAR_matrix_from_models (const GRETL_VAR *var, int idx, int *err)
     double x;
     int i, j;
 
-    if (idx == M_VECG && var->ci != VECM) {
-	*err = E_BADSTAT;
-	return NULL;
+    if (idx == M_VECG) {
+	if (var->ci != VECM || var->order == 0) {
+	    *err = E_BADSTAT;
+	    return NULL;
+	}
     }
 
     if (idx == M_VECG) {
@@ -3887,10 +3797,6 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
 	}
     }	    
 
-    if (M != NULL && idx == M_UHAT) {
-	M->t1 = var->t1;
-    }
-
     return M;
 }
 
