@@ -90,32 +90,32 @@ static void VAR_info_header_block (int code, int v, int block,
     if (tex) {
 	pputs(prn, "\\vspace{1em}\n\n");
 	if (code == IRF) {
-	    pprintf(prn, I_("Responses to a one-standard error shock in %s"), 
+	    pprintf(prn, A_("Responses to a one-standard error shock in %s"), 
 		    tex_escape(vname, dset->varname[v]));
 	} else {
-	    pprintf(prn, I_("Decomposition of variance for %s"), 
+	    pprintf(prn, A_("Decomposition of variance for %s"), 
 		    tex_escape(vname, dset->varname[v]));
 	}
 	if (block == 0) {
 	    pputs(prn, "\n\n");
 	} else {
-	    pprintf(prn, " (%s)\n\n", I_("continued"));
+	    pprintf(prn, " (%s)\n\n", A_("continued"));
 	}
 	pprintf(prn, "\\vspace{1em}\n\n\\begin{longtable}{%s}\n",
 		(code == IRF)? "rrrrr" : "rrrrrr");
     } else if (rtf) {
 	pputs(prn, "\\par\n\n");
 	if (code == IRF) {
-	    pprintf(prn, I_("Responses to a one-standard error shock in %s"), 
+	    pprintf(prn, A_("Responses to a one-standard error shock in %s"), 
 		    dset->varname[v]);
 	} else {
-	    pprintf(prn, I_("Decomposition of variance for %s"), 
+	    pprintf(prn, A_("Decomposition of variance for %s"), 
 		    dset->varname[v]);
 	}
 	if (block == 0) {
 	    pputs(prn, "\\par\n\n");
 	} else {
-	    pprintf(prn, " (%s)\\par\n\n", I_("continued"));
+	    pprintf(prn, " (%s)\\par\n\n", A_("continued"));
 	}
 	/* FIXME */
 	VAR_RTF_row_spec((code == IRF)? IRF_ROW_MAX : VDC_ROW_MAX, prn);
@@ -136,9 +136,9 @@ static void VAR_info_header_block (int code, int v, int block,
 
     /* first column: period number header */
     if (tex) {
-	pprintf(prn, "%s & ", I_("period"));
+	pprintf(prn, "%s & ", A_("period"));
     } else if (rtf) {
-	pprintf(prn, "\\intbl \\qc %s\\cell ", I_("period"));
+	pprintf(prn, "\\intbl \\qc %s\\cell ", A_("period"));
     } else {
 	pputs(prn, _("period"));
     }
@@ -450,9 +450,9 @@ gretl_VAR_print_fcast_decomp (GRETL_VAR *var, int targ,
 	    k = VDC_ROW_MAX * block + i - 1;
 	    if (k < 0) {
 		if (tex) {
-		    pprintf(prn, " %s & ", I_("std. error"));
+		    pprintf(prn, " %s & ", A_("std. error"));
 		} else if (rtf) {
-		    pprintf(prn, " \\qc %s\\cell ", I_("std. error"));
+		    pprintf(prn, " \\qc %s\\cell ", A_("std. error"));
 		} else {
 		    pprintf(prn, " %14s", _("std. error"));
 		}
@@ -552,12 +552,14 @@ void print_Johansen_test_case (JohansenCode jcode, PRN *prn)
 	N_("Case 5: Unrestricted trend and constant")
     };
 
+    set_alt_gettext_mode(prn);
+
     if (jcode <= J_UNREST_TREND) {
 	if (plain_format(prn)) {
 	    pputs(prn, _(jcase[jcode]));
 	} else {
-	    pputs(prn, I_(jcase[jcode]));
-	}
+	    pputs(prn, A_(jcase[jcode]));
+	} 
     }
 }
 
@@ -603,7 +605,6 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
 		       PRN *prn)
 {
     JohansenInfo *jv = jvar->jinfo;
-    int utf = plain_format(prn);
     int rtf = rtf_format(prn);
     char namefmt[8];
     char s[16], vname[32];
@@ -612,11 +613,9 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
     int i, j;
     double x;
 
-    pprintf(prn, "beta (%s", (utf)? _("cointegrating vectors") :
-	    I_("cointegrating vectors"));
+    pprintf(prn, "beta (%s", A_("cointegrating vectors"));
     if (jv->Bse != NULL) {
-	pprintf(prn, ", %s)", (utf)? _("standard errors in parentheses") :
-		I_("standard errors in parentheses"));
+	pprintf(prn, ", %s)", A_("standard errors in parentheses"));
     } else {
 	pputc(prn, ')');
     }
@@ -670,11 +669,9 @@ print_VECM_coint_eqns (GRETL_VAR *jvar,
 
     rows = gretl_matrix_rows(jv->Alpha);
 
-    pprintf(prn, "alpha (%s", (utf)? _("adjustment vectors") :
-	    I_("adjustment vectors"));
+    pprintf(prn, "alpha (%s", A_("adjustment vectors"));
     if (jv->Ase != NULL) {
-	pprintf(prn, ", %s)", (utf)? _("standard errors in parentheses") :
-		I_("standard errors in parentheses"));
+	pprintf(prn, ", %s)", A_("standard errors in parentheses"));
     } else {
 	pputc(prn, ')');
     }
@@ -738,15 +735,13 @@ static int max_vlen (const int *list, const DATASET *dset)
 
 static void print_VECM_omega (GRETL_VAR *jvar, const DATASET *dset, PRN *prn)
 {
-    int utf = plain_format(prn);
     int rtf = rtf_format(prn);
     int *list = jvar->ylist;
     char s[32];
     int w0, wi = 12;
     int i, j;
 
-    pprintf(prn, "%s:\n", (utf)? _("Cross-equation covariance matrix") :
-	    I_("Cross-equation covariance matrix"));
+    pprintf(prn, "%s:\n", A_("Cross-equation covariance matrix"));
     gretl_prn_newline(prn);
 
     w0 = max_vlen(list, dset) + 1;
@@ -802,8 +797,7 @@ static void print_VECM_omega (GRETL_VAR *jvar, const DATASET *dset, PRN *prn)
 
     gretl_prn_newline(prn);
 
-    pprintf(prn, "%s = %g", (utf)? _("determinant") : I_("determinant"),
-	    exp(jvar->ldet));
+    pprintf(prn, "%s = %g", A_("determinant"), exp(jvar->ldet));
 
     gretl_prn_newline(prn);
 }
@@ -850,7 +844,7 @@ static void print_LR_stat (double x, int df, PRN *prn)
     } else if (rtf_format(prn)) {
 	pprintf(prn, "2 * (lu - lr) = %g", x);
 	gretl_prn_newline(prn);
-	pprintf(prn, "P(%s(%d) > %g) = %g", I_("Chi-square"), df, x, pv);
+	pprintf(prn, "P(%s(%d) > %g) = %g", A_("Chi-square"), df, x, pv);
     } else {
 	pprintf(prn, "2 * (lu - lr) = %g", x);
 	gretl_prn_newline(prn);
@@ -873,11 +867,7 @@ vecm_print_LR_test (GRETL_VAR *vecm, PRN *prn, int code)
 	ll0 = vecm->jinfo->prior_ll;
 	df = vecm->jinfo->lrdf - vecm->jinfo->prior_df;
 	gretl_prn_newline(prn);
-	if (tex_format(prn) || rtf_format(prn)) {
-	    pputs(prn, I_("Relative to prior restriction"));
-	} else {
-	    pputs(prn, _("Relative to prior restriction"));
-	} 
+	pputs(prn, A_("Relative to prior restriction"));
 	pputc(prn, ':');
 	gretl_prn_newline(prn);
     } else {
@@ -888,17 +878,13 @@ vecm_print_LR_test (GRETL_VAR *vecm, PRN *prn, int code)
     x = 2.0 * (ll0 - vecm->ll);
 
     if (tex_format(prn)) {
-	pprintf(prn, I_("Unrestricted loglikelihood $(l_u) = %.8g$"), ll0);
+	pprintf(prn, A_("Unrestricted loglikelihood $(l_u) = %.8g$"), ll0);
 	gretl_prn_newline(prn);
-	pprintf(prn, I_("Restricted loglikelihood $(l_r) = %.8g$"), vecm->ll);
-    } else if (rtf_format(prn)) {
-	pprintf(prn, I_("Unrestricted loglikelihood (lu) = %.8g"), ll0);
-	gretl_prn_newline(prn);
-	pprintf(prn, I_("Restricted loglikelihood (lr) = %.8g"), vecm->ll);
+	pprintf(prn, A_("Restricted loglikelihood $(l_r) = %.8g$"), vecm->ll);
     } else {
-	pprintf(prn, _("Unrestricted loglikelihood (lu) = %.8g"), ll0);
+	pprintf(prn, A_("Unrestricted loglikelihood (lu) = %.8g"), ll0);
 	gretl_prn_newline(prn);
-	pprintf(prn, _("Restricted loglikelihood (lr) = %.8g"), vecm->ll);
+	pprintf(prn, A_("Restricted loglikelihood (lr) = %.8g"), vecm->ll);
     }
 
     gretl_prn_newline(prn);
@@ -916,9 +902,7 @@ print_vecm_header_info (GRETL_VAR *vecm, int *lldone, PRN *prn)
 	return;
     }
     
-    pprintf(prn, "%s = %d", 
-	    (plain_format(prn))? _("Cointegration rank") : I_("Cointegration rank"),
-	    jrank(vecm));
+    pprintf(prn, "%s = %d", A_("Cointegration rank"), jrank(vecm));
     gretl_prn_newline(prn);
     print_Johansen_test_case(jcode(vecm), prn); 
 
@@ -952,12 +936,12 @@ static void VAR_print_LB_stat (const GRETL_VAR *var, PRN *prn)
 
     if (tex_format(prn)) {
 	pprintf(prn, "\\noindent\n%s: LB(%d) = %g, %s = %d [%.4f]\\par\n",
-		I_("Portmanteau test"), var->LBs, var->LB,
-		I_("df"), df, pv);	    
+		A_("Portmanteau test"), var->LBs, var->LB,
+		A_("df"), df, pv);	    
     } else if (rtf_format(prn)) {
 	pprintf(prn, "%s: LB(%d) = %g, %s = %d [%.4f]\\par\n",
-		I_("Portmanteau test"), var->LBs, var->LB,
-		I_("df"), df, pv);
+		A_("Portmanteau test"), var->LBs, var->LB,
+		A_("df"), df, pv);
     } else {
 	pprintf(prn, "%s: LB(%d) = %g, %s = %d [%.4f]\n",
 		_("Portmanteau test"), var->LBs, var->LB,
@@ -1032,6 +1016,8 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
 	return 0;
     }
 
+    set_alt_gettext_mode(prn);
+
     nlags = var_n_lags(var);
     maxlag = var_max_lag(var);
     
@@ -1049,24 +1035,16 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
     }
 
     if (vecm) {
-	if (tex || rtf) {
-	    sprintf(label, I_("VECM system, lag order %d"), var->order + 1);
-	} else {
-	    sprintf(label, _("VECM system, lag order %d"), var->order + 1);
-	}
+	sprintf(label, A_("VECM system, lag order %d"), var->order + 1);
     } else {
-	if (tex || rtf) {
-	    sprintf(label, I_("VAR system, lag order %d"), var->order);
-	} else {
-	    sprintf(label, _("VAR system, lag order %d"), var->order);
-	}
+	sprintf(label, A_("VAR system, lag order %d"), var->order);
     }
 
     if (tex) {
 	pputs(prn, "\\begin{center}");
 	pprintf(prn, "\n%s\\\\\n", label);
-	pprintf(prn, I_("%s estimates, observations %s--%s ($T=%d$)"),
-		(vecm)? I_("Maximum likelihood") : I_("OLS"), startdate, enddate, var->T);
+	pprintf(prn, A_("%s estimates, observations %s--%s ($T=%d$)"),
+		(vecm)? A_("Maximum likelihood") : A_("OLS"), startdate, enddate, var->T);
 	if (vecm) {
 	    print_vecm_header_info(var, &lldone, prn);
 	}
@@ -1074,8 +1052,8 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
     } else if (rtf) {
 	gretl_print_toggle_doc_flag(prn);
 	pprintf(prn, "\n%s\\par\n", label);
-	pprintf(prn, I_("%s estimates, observations %s-%s (T = %d)"),
-		(vecm)? I_("Maximum likelihood") : I_("OLS"), startdate, enddate, var->T);
+	pprintf(prn, A_("%s estimates, observations %s-%s (T = %d)"),
+		(vecm)? A_("Maximum likelihood") : A_("OLS"), startdate, enddate, var->T);
 	if (vecm) {
 	    print_vecm_header_info(var, &lldone, prn);
 	}	
@@ -1102,13 +1080,13 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
 	tex_print_VAR_ll_stats(var, prn);
     } else if (rtf) {
 	if (!lldone) {
-	    pprintf(prn, "%s = %.8g\\par\n", I_("Log-likelihood"), var->ll);
+	    pprintf(prn, "%s = %.8g\\par\n", A_("Log-likelihood"), var->ll);
 	}
-	pprintf(prn, "%s = %.8g\\par\n", I_("Determinant of covariance matrix"), 
+	pprintf(prn, "%s = %.8g\\par\n", A_("Determinant of covariance matrix"), 
 		exp(var->ldet));
-	pprintf(prn, "%s = %.4f\\par\n", I_("AIC"), var->AIC);
-	pprintf(prn, "%s = %.4f\\par\n", I_("BIC"), var->BIC);
-	pprintf(prn, "%s = %.4f\\par\n", I_("HQC"), var->HQC);
+	pprintf(prn, "%s = %.4f\\par\n", A_("AIC"), var->AIC);
+	pprintf(prn, "%s = %.4f\\par\n", A_("BIC"), var->BIC);
+	pprintf(prn, "%s = %.4f\\par\n", A_("HQC"), var->HQC);
     } else {
 	if (!lldone) {
 	    pprintf(prn, "%s = %.8g\n", _("Log-likelihood"), var->ll);
@@ -1130,6 +1108,7 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
     k = 0;
 
     for (i=0; i<var->neqns; i++) {
+	double Fval;
 	char Fstr[24];
 
 	if (!quiet) {
@@ -1139,14 +1118,14 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
 	        v = var->models[i]->list[1];
 		if (tex) {
 		    pputs(prn, "\n\\begin{center}\n");
-		    pprintf(prn, "%s\\\\[1em]\n", I_("Equation for "));
+		    pprintf(prn, "%s\\\\[1em]\n", A_("Equation for "));
 		    pprintf(prn, "%s\\\n", dset->varname[v]);
 		    pputs(prn, "\n\\end{center}\n");
 		} else if (rtf) {
-		    pprintf(prn, "\\par\n%s", I_("Equation for "));
+		    pprintf(prn, "\\par\n%s", A_("Equation for "));
 		    pprintf(prn, "%s:\\par\n\n", dset->varname[v]);
 		} else {
-		    pprintf(prn, "\n%s", I_("Equation for "));
+		    pprintf(prn, "\n%s", _("Equation for "));
 		    pprintf(prn, "%s:\n", dset->varname[v]);
 		}
 	    }
@@ -1158,10 +1137,10 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
 
 	if (tex) {
 	    pputs(prn, "\n\\begin{center}\n");
-	    pprintf(prn, "%s\\\\[1em]\n", I_("F-tests of zero restrictions"));
+	    pprintf(prn, "%s\\\\[1em]\n", A_("F-tests of zero restrictions"));
 	    pputs(prn, "\\begin{tabular}{lll}\n");
 	} else if (rtf) {
-	    pprintf(prn, "%s:\\par\n\n", I_("F-tests of zero restrictions"));
+	    pprintf(prn, "%s:\\par\n\n", A_("F-tests of zero restrictions"));
 	} else {
 	    pprintf(prn, "%s:\n\n", _("F-tests of zero restrictions"));
 	}
@@ -1171,53 +1150,59 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
 	}
 
 	for (j=0; j<var->neqns; j++) {
-	    pv = snedecor_cdf_comp(nlags, dfd, var->Fvals[k]);
-	    v = (var->models[j])->list[1];
-	    if (tex) {
-		pprintf(prn, I_("All lags of %s"), dset->varname[v]);
-		pputs(prn, " & ");
-		pprintf(prn, "$F(%d, %d) = %g$ & ", nlags, dfd, var->Fvals[k]);
-		pprintf(prn, "[%.4f]\\\\\n", pv);
-	    } else if (rtf) {
-		sprintf(label, I_("All lags of %s"), dset->varname[v]);
-		llen = strlen(label);
-		pputs(prn, label);
-		bufspace(fwidth - llen, prn);
-		pprintf(prn, "F(%d, %d) = %8.5g ", nlags, dfd, var->Fvals[k]);
-		pprintf(prn, "[%.4f]\\par\n", pv);
-	    } else {
-		sprintf(label, _("All lags of %s"), dset->varname[v]);
-		llen = g_utf8_strlen(label, -1);
-		pputs(prn, label);
-		bufspace(fwidth - llen, prn);
-		sprintf(Fstr, "F(%d, %d)", nlags, dfd);
-		pprintf(prn, "%12s = %#8.5g [%.4f]\n", Fstr, var->Fvals[k], pv);
+	    Fval = var->Fvals[k];
+	    if (!na(Fval)) {
+		pv = snedecor_cdf_comp(nlags, dfd, Fval);
+		v = (var->models[j])->list[1];
+		if (tex) {
+		    pprintf(prn, A_("All lags of %s"), dset->varname[v]);
+		    pputs(prn, " & ");
+		    pprintf(prn, "$F(%d, %d) = %g$ & ", nlags, dfd, Fval);
+		    pprintf(prn, "[%.4f]\\\\\n", pv);
+		} else if (rtf) {
+		    sprintf(label, A_("All lags of %s"), dset->varname[v]);
+		    llen = strlen(label);
+		    pputs(prn, label);
+		    bufspace(fwidth - llen, prn);
+		    pprintf(prn, "F(%d, %d) = %8.5g ", nlags, dfd, Fval);
+		    pprintf(prn, "[%.4f]\\par\n", pv);
+		} else {
+		    sprintf(label, _("All lags of %s"), dset->varname[v]);
+		    llen = g_utf8_strlen(label, -1);
+		    pputs(prn, label);
+		    bufspace(fwidth - llen, prn);
+		    sprintf(Fstr, "F(%d, %d)", nlags, dfd);
+		    pprintf(prn, "%12s = %#8.5g [%.4f]\n", Fstr, Fval, pv);
+		}
 	    }
 	    k++;
 	}
 
 	if (var->order > 1) {
-	    pv = snedecor_cdf_comp(var->neqns, dfd, var->Fvals[k]);
-	    if (tex) {
-		pprintf(prn, I_("All vars, lag %d"), maxlag);
-		pputs(prn, " & ");
-		pprintf(prn, "$F(%d, %d) = %g$ & ", var->neqns, dfd, var->Fvals[k]);
-		pprintf(prn, "[%.4f]\\\\\n", pv);
-	    } else if (rtf) {
-		sprintf(label, I_("All vars, lag %d"), maxlag);
-		llen = strlen(label);
-		pputs(prn, label);
-		bufspace(fwidth - llen, prn);
-		pprintf(prn, "F(%d, %d) = %8.5g ", var->neqns, dfd, var->Fvals[k]);
-		pprintf(prn, "[%.4f]\\par\n", pv);
-	    } else {
-		sprintf(label, _("All vars, lag %d"), maxlag);
-		llen = g_utf8_strlen(label, -1);
-		pputs(prn, label);
-		bufspace(fwidth - llen, prn);
-		sprintf(Fstr, "F(%d, %d)", var->neqns, dfd);
-		pprintf(prn, "%12s = %#8.5g [%.4f]\n", Fstr, var->Fvals[k], pv);
-	    } 
+	    Fval = var->Fvals[k];
+	    if (!na(Fval)) {
+		pv = snedecor_cdf_comp(var->neqns, dfd, Fval);
+		if (tex) {
+		    pprintf(prn, A_("All vars, lag %d"), maxlag);
+		    pputs(prn, " & ");
+		    pprintf(prn, "$F(%d, %d) = %g$ & ", var->neqns, dfd, Fval);
+		    pprintf(prn, "[%.4f]\\\\\n", pv);
+		} else if (rtf) {
+		    sprintf(label, A_("All vars, lag %d"), maxlag);
+		    llen = strlen(label);
+		    pputs(prn, label);
+		    bufspace(fwidth - llen, prn);
+		    pprintf(prn, "F(%d, %d) = %8.5g ", var->neqns, dfd, Fval);
+		    pprintf(prn, "[%.4f]\\par\n", pv);
+		} else {
+		    sprintf(label, _("All vars, lag %d"), maxlag);
+		    llen = g_utf8_strlen(label, -1);
+		    pputs(prn, label);
+		    bufspace(fwidth - llen, prn);
+		    sprintf(Fstr, "F(%d, %d)", var->neqns, dfd);
+		    pprintf(prn, "%12s = %#8.5g [%.4f]\n", Fstr, Fval, pv);
+		}
+	    }
 	    k++;
 	}
 
@@ -1238,28 +1223,23 @@ int gretl_VAR_print (GRETL_VAR *var, const DATASET *dset, gretlopt opt,
 
 	pputc(prn, '\n');
 
-	if (rtf || tex) {
-	    sprintf(h0str, I_("the longest lag is %d"), nextlag);
-	    sprintf(h1str, I_("the longest lag is %d"), maxlag);
-	} else {
-	    sprintf(h0str, _("the longest lag is %d"), nextlag);
-	    sprintf(h1str, _("the longest lag is %d"), maxlag);
-	}
+	sprintf(h0str, A_("the longest lag is %d"), nextlag);
+	sprintf(h1str, A_("the longest lag is %d"), maxlag);
 
 	pv = chisq_cdf_comp(df, var->LR);
 
 	if (tex) {
-	    pprintf(prn, "\\noindent %s ---\\par\n", I_("For the system as a whole"));
-	    pprintf(prn, "%s: %s\\par\n", I_("Null hypothesis"), h0str);
-	    pprintf(prn, "%s: %s\\par\n", I_("Alternative hypothesis"), h1str);
+	    pprintf(prn, "\\noindent %s ---\\par\n", A_("For the system as a whole"));
+	    pprintf(prn, "%s: %s\\par\n", A_("Null hypothesis"), h0str);
+	    pprintf(prn, "%s: %s\\par\n", A_("Alternative hypothesis"), h1str);
 	    pprintf(prn, "%s: $\\chi^2_{%d}$ = %.3f [%.4f]\\par\n",
-		    I_("Likelihood ratio test"), df, var->LR, pv);
+		    A_("Likelihood ratio test"), df, var->LR, pv);
 	} else if (rtf) {
-	    pprintf(prn, "\\par %s\n", I_("For the system as a whole"));
-	    pprintf(prn, "\\par %s: %s\n", I_("Null hypothesis"), h0str);
-	    pprintf(prn, "\\par %s: %s\n", I_("Alternative hypothesis"), h1str);
-	    pprintf(prn, "\\par %s: %s(%d) = %g [%.4f]\n", I_("Likelihood ratio test"), 
-		    I_("Chi-square"), df, var->LR, pv);
+	    pprintf(prn, "\\par %s\n", A_("For the system as a whole"));
+	    pprintf(prn, "\\par %s: %s\n", A_("Null hypothesis"), h0str);
+	    pprintf(prn, "\\par %s: %s\n", A_("Alternative hypothesis"), h1str);
+	    pprintf(prn, "\\par %s: %s(%d) = %g [%.4f]\n", A_("Likelihood ratio test"), 
+		    A_("Chi-square"), df, var->LR, pv);
 	} else {
 	    int ordlen = (var->order > 10)? 2 : 1;
 
