@@ -17,6 +17,7 @@
  */
 
 #include "libgretl.h"
+#include "version.h"
 #include "libset.h"
 #include "kalman.h"
 #include "matrix_extra.h"
@@ -174,7 +175,7 @@ int ma_out_of_bounds (arma_info *ainfo, const double *theta,
 	im = b->roots[i].i;
 	rt = re * re + im * im;
 	if (rt > DBL_EPSILON && rt <= 1.0) {
-	    pprintf(ainfo->prn, "MA root %d = %g\n", i, rt);
+	    pprintf(ainfo->prn, _("MA root %d = %g\n"), i, rt);
 	    err = 1;
 	    break;
 	}
@@ -825,29 +826,25 @@ static int arma_OPG_vcv (MODEL *pmod, kalman *K, double *b,
 {
     gretl_matrix *G = NULL;
     gretl_matrix *V = NULL;
-    double rcond;
     int err = 0;
 
-    G = numerical_score_matrix(b, k, T, kalman_arma_llt_callback, 
+    G = numerical_score_matrix(b, T, k, kalman_arma_llt_callback, 
 			       K, &err);
-    if (err) {
-	goto bailout;
+
+    if (!err) {
+	V = gretl_matrix_XTX_new(G);
+	if (V == NULL) {
+	    err = E_ALLOC;
+	}
     }
 
-    V = gretl_matrix_alloc(k, k);
-    if (V == NULL) {
-	err = E_ALLOC;
-	goto bailout;
-    }
+    if (!err) {
+	double rcond = gretl_symmetric_matrix_rcond(V, &err);
 
-    gretl_matrix_multiply_mod(G, GRETL_MOD_NONE,
-			      G, GRETL_MOD_TRANSPOSE,
-			      V, GRETL_MOD_NONE);
-
-    rcond = gretl_symmetric_matrix_rcond(V, &err);
-    if (!err && rcond < 1.0E-10) {
-	pprintf(prn, "OPG: rcond = %g; will try Hessian\n", rcond);
-	err = 1;
+	if (!err && rcond < 1.0E-10) {
+	    pprintf(prn, "OPG: rcond = %g; will try Hessian\n", rcond);
+	    err = 1;
+	}
     }
 
     if (!err) {
@@ -856,11 +853,9 @@ static int arma_OPG_vcv (MODEL *pmod, kalman *K, double *b,
 
     if (!err) {
 	gretl_matrix_multiply_by_scalar(V, s2);
-	err = gretl_model_write_vcv(pmod, V, -1);
+	err = gretl_model_write_vcv(pmod, V);
     }
 
- bailout:
-
     gretl_matrix_free(G);
     gretl_matrix_free(V);
     
@@ -908,7 +903,7 @@ static double kalman_arma_ll (const double *b, void *data)
 #endif
 
     if (kalman_do_ma_check && ma_out_of_bounds(ainfo, theta, Theta)) {
-	pputs(kalman_get_printer(K), "arma: MA estimate(s) out of bounds\n");
+	pputs(kalman_get_printer(K), _("MA estimate(s) out of bounds\n"));
 	return NADBL;
     }
 
@@ -943,7 +938,7 @@ static int arma_use_opg (gretlopt opt)
 
     if (opt & OPT_G) {
 	ret = 1;
-    } else if (libset_get_int(ARMA_VCV) == VCV_OP) {
+    } else if (libset_get_int(ARMA_VCV) == ML_OP) {
 	ret = 1;
     }
 
@@ -964,8 +959,8 @@ static void arma_add_ehat (MODEL *pmod, arma_info *ainfo,
 	gretl_matrix *ehat = gretl_matrix_copy(kh->E);
 
 	if (ehat != NULL) {
-	    ehat->t1 = pmod->t1;
-	    ehat->t2 = pmod->t2;
+	    gretl_matrix_set_t1(ehat, pmod->t1);
+	    gretl_matrix_set_t2(ehat, pmod->t2);
 	    gretl_model_set_matrix_as_data(pmod, "ehat", ehat);
 	}
     }
@@ -1038,9 +1033,9 @@ static int kalman_arma_finish (MODEL *pmod, arma_info *ainfo,
 	    if (kopt & KALMAN_AVG_LL) {
 		gretl_matrix_divide_by_scalar(Hinv, ainfo->T);
 	    }
-	    err = gretl_model_write_vcv(pmod, Hinv, -1);
+	    err = gretl_model_write_vcv(pmod, Hinv);
 	    if (!err) {
-		gretl_model_set_vcv_info(pmod, VCV_ML, VCV_HESSIAN);
+		gretl_model_set_vcv_info(pmod, VCV_ML, ML_HESSIAN);
 	    }
 	} else if (err == E_NOTPD && !(opt & OPT_H)) {
 	    /* try falling back to OPG, if use of the Hessian has not
@@ -1057,7 +1052,7 @@ static int kalman_arma_finish (MODEL *pmod, arma_info *ainfo,
     if (do_opg) {
 	err = arma_OPG_vcv(pmod, K, b, s2, k, ainfo->T, prn);
 	if (!err) {
-	    gretl_model_set_vcv_info(pmod, VCV_ML, VCV_OP);
+	    gretl_model_set_vcv_info(pmod, VCV_ML, ML_OP);
 	    pmod->opt |= OPT_G;
 	}
     }	
@@ -1549,11 +1544,12 @@ static int arima_by_ls (const DATASET *dset, arma_info *ainfo,
     } else {
 	gretl_vector y;
 
+	gretl_matrix_init(&y);
 	y.rows = ainfo->T;
 	y.cols = 1;
 	y.val = ainfo->y + ainfo->t1;
-	y.t1 = ainfo->t1;
-	y.t2 = ainfo->t2;
+	gretl_matrix_set_t1(&y, ainfo->t1);
+	gretl_matrix_set_t2(&y, ainfo->t2);
 
 	err = gretl_matrix_ols(&y, X, b, V, u, &s2);
     }
@@ -1571,7 +1567,7 @@ static int arima_by_ls (const DATASET *dset, arma_info *ainfo,
 	for (t=0; t<ainfo->T; t++) {
 	    pmod->uhat[t + ainfo->t1] = u->val[t];
 	}	
-	err = gretl_model_write_vcv(pmod, V, -1);
+	err = gretl_model_write_vcv(pmod, V);
     }
 
     if (!err) {
