@@ -35,8 +35,28 @@
 #include "biff.h"
 #include "build.h"
 
-static void free_sheet (void);
-static int allocate_row_col (int row, int col, wbook *book);
+typedef struct xls_info_ xls_info;
+
+struct sheetrow {
+    int last, end;
+    gchar **cells;
+};
+
+struct xls_info_ {
+    int codepage;
+    gchar **sst;
+    int sstsize;
+    int sstnext;
+    int datacols;
+    int totcols;
+    int nrows;
+    struct sheetrow *rows;
+    char *blank_col;
+};
+
+static void free_xls_info (xls_info *xi);
+static int allocate_row_col (int row, int col, wbook *book,
+			     xls_info *xi);
 
 int debug_print;
 
@@ -54,11 +74,6 @@ static void make_debug_fname (void);
                         r == BIFF_FORMULA || \
                         r == BIFF_LABELSST)
 
-struct sheetrow {
-    int last, end;
-    char **cells;
-};	
-
 enum {
     VARNAMES_OK = 0,
     VARNAMES_NULL,
@@ -67,11 +82,6 @@ enum {
     VARNAMES_NONE
 } varname_errors;
 
-char **sst = NULL;
-int sstsize = 0, sstnext = 0;
-struct sheetrow *rows = NULL;
-int nrows = 0;
-
 #define EXCEL_IMPORTER
 #include "import_common.c"
 
@@ -124,7 +134,8 @@ static int dbprintf (const char *format, ...)
 
 static void print_version (void)
 {
-    dbprintf("gretl, version %s, %s\n", GRETL_VERSION, BUILD_DATE);
+    dbprintf("gretl, version %s, %s %s\n", GRETL_VERSION, 
+	     _("build date"), BUILD_DATE);
 }
 
 static double get_le_double (const unsigned char *rec) 
@@ -194,11 +205,13 @@ static double biff_get_rk (const unsigned char *ptr)
     return NADBL;
 }
 
-static char *convert8to7 (const char *s, int count) 
+static gchar *convert8to7 (const char *s, int count) 
 {
-    char *dest;
-    int n = strspn(s, " \t");
+    gchar *dest;
+    int n;
 
+    /* we'll skip any leading space */
+    n = strspn(s, " \t");
     count -= n;
 
     if (count <= 0) {
@@ -207,7 +220,7 @@ static char *convert8to7 (const char *s, int count)
 	if (count > VNAMELEN - 1) {
 	    count = VNAMELEN - 1;
 	}
-	dest = malloc(VNAMELEN);
+	dest = g_malloc(VNAMELEN);
 	*dest = '\0';
 	s += n;
 	strncat(dest, s, count);
@@ -220,12 +233,12 @@ static char *convert8to7 (const char *s, int count)
     return dest;
 }
 
-static char *convert16to7 (const unsigned char *s, int count) 
+static gchar *convert16to7 (const unsigned char *s, int count) 
 {
     char *dest;
     int i, u, j = 0;
 
-    dest = malloc(VNAMELEN);
+    dest = g_malloc(VNAMELEN);
     if (dest == NULL) {
 	return NULL;
     }
@@ -245,14 +258,15 @@ static char *convert16to7 (const unsigned char *s, int count)
     return dest;    
 }
 
-static char *
-copy_unicode_string (unsigned char *src, int remlen, 
+static gchar *
+copy_unicode_string (xls_info *xi, unsigned char *src, int remlen, 
 		     int *skip, int *slop) 
 {
     int count = MS_OLE_GET_GUINT16(src);
     unsigned char flags = *(src + 2);
     int this_skip = 3, skip_to_next = 3;
     int csize = (flags & 0x01)? 2 : 1;
+    gchar *ret = NULL;
 
     dbprintf("copy_unicode_string: count = %d, csize = %d\n",
 	    count, csize);
@@ -302,22 +316,45 @@ copy_unicode_string (unsigned char *src, int remlen,
 
     if (count > 64) {
 	/* let's not mess with excessive strings */
-	return g_strdup("bigstr");
+	ret = g_strdup("bigstr");
     } else if (csize == 1) {
-	dbprintf("original string = '%s'\n", src + this_skip);
-	return convert8to7((char *) src + this_skip, count);
+	char show[68];
+
+	*show = '\0';
+	strncat(show, (char *) src + this_skip, count);
+	dbprintf("original string = '%s'\n", show);
+	ret = convert8to7((char *) src + this_skip, count);
     } else { 
-	return convert16to7(src + this_skip, count);
+	if (xi->codepage == 1200) {
+	    GError *gerr = NULL;
+	    gsize written;
+
+	    ret = g_convert((const gchar *) src + this_skip, 2*count, 
+			    "UTF-8", "UTF-16", NULL, &written, &gerr);    
+	    if (gerr != NULL) {
+		fprintf(stderr, "%s\n", gerr->message);
+		g_error_free(gerr);
+		g_free(ret);
+		ret = NULL;
+	    }
+	}
+
+	if (ret == NULL) {
+	    /* fallback */
+	    ret = convert16to7(src + this_skip, count);
+	}
     }
+
+    return ret;
 }
 
-static char *make_string (char *str) 
+static gchar *make_string (gchar *str) 
 {
-    char *ret = NULL;
+    gchar *ret = NULL;
 
     if (str != NULL) {
 	ret = g_strdup_printf("\"%s", str);
-	free(str);
+	g_free(str);
     } else {
 	ret = g_strdup("\"");
     }
@@ -387,7 +424,7 @@ static int check_copy_string (struct sheetrow *prow, int row, int col,
 
 	if (len > 0 && digits > 0) {
 	    /* may be numerical? */
-	    char *p, *q = malloc(len + 1);
+	    char *p, *q = g_malloc(len + 1);
 
 	    if (q == NULL) return 1;
 
@@ -418,7 +455,7 @@ static int check_copy_string (struct sheetrow *prow, int row, int col,
 		prow->cells[col] = q;
 		return 0;
 	    } else {
-		free(q);
+		g_free(q);
 	    }
 	} 
     }
@@ -520,7 +557,8 @@ static void check_for_date_formula (BiffQuery *q, wbook *book)
 
 #undef FORMAT_INFO
 
-static int process_item (BiffQuery *q, wbook *book, PRN *prn) 
+static int process_item (BiffQuery *q, wbook *book, xls_info *xi,
+			 PRN *prn) 
 {
     struct sheetrow *prow = NULL;
     static char **string_targ;
@@ -556,53 +594,53 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
     switch (q->ls_op) {
 
     case BIFF_SST: {
-	int k, skip, remlen, oldsz = sstsize;
+	int k, skip, remlen, oldsz = xi->sstsize;
 	guint16 sz;
 
-	if (sst != NULL) {
+	if (xi->sst != NULL) {
 	    fprintf(stderr, "Got a second string table: this is nonsense\n");
 	    return 1;
 	}
 
 	sz = MS_OLE_GET_GUINT16(q->data + 4);
-	sstsize += sz;
-	sst = realloc(sst, sstsize * sizeof *sst);
-	if (sst == NULL) {
+	xi->sstsize += sz;
+	xi->sst = realloc(xi->sst, xi->sstsize * sizeof *xi->sst);
+	if (xi->sst == NULL) {
 	    return 1;
 	}
 
 	dbprintf("Got SST: allocated for %d strings (%d bytes), %p\n", 
-		 sstsize, sstsize * sizeof *sst, (void *) sst);
+		 xi->sstsize, xi->sstsize * sizeof *xi->sst, (void *) xi->sst);
 
-	for (k=oldsz; k<sstsize; k++) {
+	for (k=oldsz; k<xi->sstsize; k++) {
 	    /* careful: initialize all pointers to NULL */
-	    sst[k] = NULL;
+	    xi->sst[k] = NULL;
 	}
 
 	ptr = q->data + 8;
 
-	for (k=oldsz; k<sstsize; k++) {
+	for (k=oldsz; k<xi->sstsize; k++) {
 	    remlen = q->length - (ptr - q->data);
 	    dbprintf("Working on sst[%d], data offset=%d, remlen=%d\n", 
 		     k, (int) (ptr - q->data), remlen);
 	    if (remlen <= 0) {
 		break;
 	    }
-	    sst[k] = copy_unicode_string(ptr, remlen, &skip, &slop);
+	    xi->sst[k] = copy_unicode_string(xi, ptr, remlen, &skip, &slop);
 	    ptr += skip;
 	}
 
-	if (k < sstsize) {
-	    sstnext = k;
+	if (k < xi->sstsize) {
+	    xi->sstnext = k;
 	}
 
 	break;
     }	
 
     case BIFF_CONTINUE: 
-	dbprintf("Got CONTINUE, sstnext = %d, len = %d\n", 
-		 sstnext, (int) q->length);
-	if (sstnext > 0) {
+	dbprintf("Got CONTINUE, xi->sstnext = %d, len = %d\n", 
+		 xi->sstnext, (int) q->length);
+	if (xi->sstnext > 0) {
 	    int k, skip, remlen;
 
 	    ptr = q->data;
@@ -614,29 +652,32 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 			 (int) csize);
 		ptr += 1 + csize * slop;
 	    }
-	    for (k=sstnext; k<sstsize; k++) {
+	    for (k=xi->sstnext; k<xi->sstsize; k++) {
 		remlen = q->length - (ptr - q->data);
 		if (remlen <= 0) {
 		    break;
 		}
 		dbprintf("Working on sst[%d], remlen = %d\n", k, remlen);
-		sst[k] = copy_unicode_string(ptr, remlen, &skip, &slop);
+		if (xi->sst[k] != NULL) {
+		    g_free(xi->sst[k]);
+		}
+		xi->sst[k] = copy_unicode_string(xi, ptr, remlen, &skip, &slop);
 		ptr += skip;
 	    }
-	    if (k < sstsize) {
-		sstnext = k;
+	    if (k < xi->sstsize) {
+		xi->sstnext = k;
 	    }
 	}
 	break;
 			   
     case BIFF_LABEL: 
 	dbprintf("Got LABEL, row=%d, col=%d\n", i, j);
-	if (allocate_row_col(i, j, book)) {
-	    return 1;
+	if (allocate_row_col(i, j, book, xi)) {
+	    return E_ALLOC;
 	} else {
 	    unsigned int len = MS_OLE_GET_GUINT16(q->data + 6);
 	
-	    prow = rows + i;
+	    prow = xi->rows + i;
 	    ptr = q->data + 8;
 	    fprintf(stderr, "BIFF_LABEL: calling convert8to7\n");
 	    prow->cells[j] = make_string(convert8to7((char *) ptr, len));
@@ -645,20 +686,20 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
   
     case BIFF_LABELSST:
 	dbprintf("Got LABELSST, row=%d, col=%d\n", i, j);
-	if (allocate_row_col(i, j, book)) {
-	    return 1;
+	if (allocate_row_col(i, j, book, xi)) {
+	    return E_ALLOC;
 	} else {
 	    unsigned int sidx = MS_OLE_GET_GUINT16(q->data + 6);
 
-	    prow = rows + i;
-	    if (sidx >= sstsize) {
+	    prow = xi->rows + i;
+	    if (sidx >= xi->sstsize) {
 		pprintf(prn, _("String index too large"));
 		pputc(prn, '\n');
-	    } else if (sst[sidx] != NULL) {
-		check_copy_string(prow, i, j, sidx, sst[sidx]);
+	    } else if (xi->sst[sidx] != NULL) {
+		check_copy_string(prow, i, j, sidx, xi->sst[sidx]);
 	    } else {
 		dbprintf("sst[%d] seems to be NULL, leaving string blank\n", (int) sidx);
-		prow->cells[j] = malloc(2);
+		prow->cells[j] = g_malloc(2);
 		if (prow->cells[j] != NULL) {
 		    prow->cells[j][0] = '\0';
 		}
@@ -667,22 +708,22 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	break;
 
     case BIFF_NUMBER: 
-	if (allocate_row_col(i, j, book)) {
-	    return 1;
+	if (allocate_row_col(i, j, book, xi)) {
+	    return E_ALLOC;
 	} else {
 	    val = get_le_double(q->data + 6);
-	    prow = rows + i;
+	    prow = xi->rows + i;
 	    prow->cells[j] = g_strdup_printf("%.15g", val);
 	    dbprintf("Got NUMBER (%g), row=%d, col=%d\n", val, i, j);
 	}
 	break;
 
     case BIFF_RK: 
-	if (allocate_row_col(i, j, book)) {
-	    return 1;
+	if (allocate_row_col(i, j, book, xi)) {
+	    return E_ALLOC;
 	} else {
 	    val = biff_get_rk(q->data + 6);
-	    prow = rows + i;
+	    prow = xi->rows + i;
 	    prow->cells[j] = g_strdup_printf("%.15g", val);
 	    dbprintf("Got RK (%g), row=%d, col=%d\n", val, i, j);
 	}
@@ -693,11 +734,11 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 
 	dbprintf("Got MULRK, row=%d, first_col=%d, ncols=%d\n", i, j, ncols);
 	for (k=0; k<ncols; k++) {
-	    if (allocate_row_col(i, j, book)) {
-		return 1;
+	    if (allocate_row_col(i, j, book, xi)) {
+		return E_ALLOC;
 	    }
 	    val = biff_get_rk(q->data + 6 + 6 * k);
-	    prow = rows + i; /* might have moved */
+	    prow = xi->rows + i; /* might have moved */
 	    prow->cells[j] = g_strdup_printf("%.15g", val);
 	    dbprintf(" MULRK[col=%d] = %g\n", j, val);
 	    j++;
@@ -707,16 +748,16 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 
     case BIFF_FORMULA:  
 	dbprintf("Got FORMULA, row=%d, col=%d\n", i, j);
-	if (allocate_row_col(i, j, book)) {
-	    return 1;
+	if (allocate_row_col(i, j, book, xi)) {
+	    return E_ALLOC;
 	} else {
 	    ptr = q->data + 6;
-	    prow = rows + i;
+	    prow = xi->rows + i;
 	    if (ptr[6] == 0xff && ptr[7] == 0xff) {
 		unsigned char fcode = ptr[0];
 
 		if (fcode == 0x0) {
-		    /* string formula: record target for following 
+		    /* string formula: record the target for following 
 		       STRING record */
 		    string_targ = prow->cells + j;
 		} else if (fcode == 0x1) {
@@ -752,16 +793,16 @@ static int process_item (BiffQuery *q, wbook *book, PRN *prn)
 	if (string_targ == NULL) {
 	    dbprintf("String record without preceding string formula\n");
 	} else {
-	    char *tmp = copy_unicode_string(q->data, 0, NULL, NULL);
+	    gchar *tmp = copy_unicode_string(xi, q->data, 0, NULL, NULL);
 
 	    *string_targ = make_string(tmp);
 	    dbprintf("Filled out string formula with '%s'\n", *string_targ);	
-	    string_targ = NULL;
+	    string_targ = NULL; /* handled */
 	}
 	break;
 
     case BIFF_BOF: 
-	if (rows != NULL) {
+	if (xi->rows != NULL) {
 	    fprintf(stderr, "BOF when current sheet is not flushed\n");
 	    return 1;
 	}
@@ -850,11 +891,11 @@ static int handled_record (BiffQuery *q)
     return 0;
 }
 
-static int process_sheet (const char *filename, wbook *book, PRN *prn) 
+static int process_sheet (const char *filename, wbook *book, xls_info *xi,
+			  PRN *prn) 
 {    
     int err = 0, gotbof = 0, eofcount = 0;
     long offset = book->byte_offsets[book->selected];
-
     MsOleStream *stream;
     MsOleErr result;
     BiffQuery *q;
@@ -917,11 +958,18 @@ static int process_sheet (const char *filename, wbook *book, PRN *prn)
 	} 
 
 	if (handled_record(q)) {
-	    err = process_item(q, book, prn);
+	    err = process_item(q, book, xi, prn);
 	} else if (q->ms_op == 0x02 && q->ls_op == BIFF_ROW) {
 	    dbprintf("Got BIFF_ROW\n");
 	} else if (q->opcode == BIFF_DBCELL) {
 	    dbprintf("Got BIFF_DBCELL\n");
+	} else if (q->opcode == 0x42) {
+	    if (q->length == 2 && q->data != NULL) {
+		int cp = MS_OLE_GET_GUINT16(q->data);
+
+		fprintf(stderr, "CODEPAGE: got %d\n", cp);
+		xi->codepage = cp;
+	    }
 	} else {
 	    dbprintf("skipping unhandled opcode 0x%02x\n", q->opcode);
 	}
@@ -941,12 +989,11 @@ static void row_init (struct sheetrow *row)
     row->cells = NULL;
 }
 
-static int allocate_row_col (int i, int j, wbook *book) 
+static int allocate_row_col (int i, int j, wbook *book,
+			     xls_info *xi) 
 {
-    struct sheetrow *myrows = NULL;
-    char **cells = NULL;
-    int new_nrows, newcol, k;
     static int started;
+    int k;
 
     if (!started && i > book->row_offset) {
 	book->row_offset = i;
@@ -955,89 +1002,103 @@ static int allocate_row_col (int i, int j, wbook *book)
 
     started = 1;
 
-    dbprintf("allocate: row=%d, col=%d, nrows=%d\n", i, j, nrows);
+    dbprintf("allocate: row=%d, col=%d, nrows=%d\n", i, j, xi->nrows);
 
-    if (i >= nrows) {
-	new_nrows = (i / 16 + 1) * 16;
-
-	myrows = realloc(rows, new_nrows * sizeof *myrows);
+    if (i >= xi->nrows) {
+	int new_nrows = (i / 16 + 1) * 16;
+	struct sheetrow *myrows;
 
+	myrows = realloc(xi->rows, new_nrows * sizeof *myrows);
 	if (myrows == NULL) {
 	    return 1;
 	}
 
-	rows = myrows;
+	xi->rows = myrows;
 
-	for (k=nrows; k<new_nrows; k++) {
+	for (k=xi->nrows; k<new_nrows; k++) {
 	    dbprintf("allocate: initing rows[%d]\n", k);
-	    row_init(&rows[k]);
-	    dbprintf("rows[%d].end=%d\n", i, rows[k].end);
+	    row_init(&xi->rows[k]);
+	    dbprintf("rows[%d].end=%d\n", i, xi->rows[k].end);
 	}
-	nrows = new_nrows;
+	xi->nrows = new_nrows;
     }
 
-    dbprintf("allocate: col=%d and rows[%d].end = %d\n", j, i, rows[i].end);
+    dbprintf("allocate: col=%d and rows[%d].end = %d\n", j, i, xi->rows[i].end);
+
+    if (j >= xi->rows[i].end) {
+	int newcol = (j / 16 + 1) * 16;
+	gchar **cells;
 
-    if (j >= rows[i].end) {
-	newcol = (j / 16 + 1) * 16;
 	dbprintf("allocate: reallocing rows[%d].cells to size %d\n", i, newcol);
-	cells = realloc(rows[i].cells, newcol * sizeof *cells);
+	cells = realloc(xi->rows[i].cells, newcol * sizeof *cells);
 
 	if (cells == NULL) {
 	    return 1;
 	}
 
-	rows[i].cells = cells;
+	xi->rows[i].cells = cells;
 
-	for (k=rows[i].end; k<newcol; k++) {
-	    rows[i].cells[k] = NULL;
+	for (k=xi->rows[i].end; k<newcol; k++) {
+	    xi->rows[i].cells[k] = NULL;
 	}
-	rows[i].end = newcol;
+	xi->rows[i].end = newcol;
     } 
  
-    if (j > rows[i].last) {
-	rows[i].last = j;
+    if (j > xi->rows[i].last) {
+	xi->rows[i].last = j;
     }
 
     return 0;
 }
 
-static void free_sheet (void) 
+static void xls_info_init (xls_info *xi)
+{
+    xi->codepage = 0;
+    xi->sst = NULL;
+    xi->sstsize = 0;
+    xi->datacols = 0;
+    xi->totcols = 0;
+    xi->nrows = 0;
+    xi->rows = NULL;
+    xi->blank_col = NULL;
+}
+
+static void free_xls_info (xls_info *xi) 
 {
     int i, j;
 
-    dbprintf("free_sheet(), nrows=%d\n", nrows);
+    dbprintf("free_xls_info(), nrows=%d\n", xi->nrows);
 
     /* free shared string table */
-    if (sst != NULL) {
-	for (i=0; i<sstsize; i++) {
-	    if (sst[i] != NULL) free(sst[i]);
+    if (xi->sst != NULL) {
+	for (i=0; i<xi->sstsize; i++) {
+	    g_free(xi->sst[i]);
 	}
-	free(sst);
+	free(xi->sst);
+	xi->sst = NULL;
     }
 
     /* free cells */
-    if (rows != NULL) {
-	for (i=0; i<nrows; i++) {
-	    if (rows[i].cells == NULL) {
+    if (xi->rows != NULL) {
+	for (i=0; i<xi->nrows; i++) {
+	    if (xi->rows[i].cells == NULL) {
 		dbprintf("rows[%d].cells = NULL, skipping free\n", i);
 		continue;
 	    }
-	    for (j=0; j<rows[i].end; j++) {
-		if (rows[i].cells[j] != NULL) {
+	    for (j=0; j<xi->rows[i].end; j++) {
+		if (xi->rows[i].cells[j] != NULL) {
 		    dbprintf("Freeing rows[%d].cells[%d] at %p\n",
-			     i, j, (void *) rows[i].cells[j]);
-		    free(rows[i].cells[j]); 
+			     i, j, (void *) xi->rows[i].cells[j]);
+		    g_free(xi->rows[i].cells[j]); 
 		}
 	    }
-	    dbprintf("Freeing rows[%d].cells at %p\n", i, (void *) rows[i].cells);
-	    free(rows[i].cells);
+	    dbprintf("Freeing rows[%d].cells at %p\n", i, (void *) xi->rows[i].cells);
+	    free(xi->rows[i].cells);
 	}
-	free(rows);
-	rows = NULL;
+	free(xi->rows);
     }
 
-    nrows = 0;
+    free(xi->blank_col);
 }
 
 #define IS_STRING(v) ((v[0] == '"'))
@@ -1046,7 +1107,7 @@ static void free_sheet (void)
    be at an offset into the worksheet)
  */
 
-static int first_col_strings (wbook *book)
+static int first_col_strings (wbook *book, xls_info *xi)
 {
     int i, j = book->col_offset;
     int startrow = book->row_offset + 1;
@@ -1054,18 +1115,18 @@ static int first_col_strings (wbook *book)
 
     dbprintf("checking for first column strings...\n");
 
-    for (i=startrow; i<nrows; i++) {
+    for (i=startrow; i<xi->nrows; i++) {
 	dbprintf("book->row_offset=%d, i=%d\n", book->row_offset, i);
-	dbprintf("rows = %p\n", (void *) rows);
-	if (rows == NULL || rows[i].cells == NULL || 
-	    rows[i].cells[j] == NULL ||
-	    !IS_STRING(rows[i].cells[j])) {
+	dbprintf("rows = %p\n", (void *) xi->rows);
+	if (xi->rows == NULL || xi->rows[i].cells == NULL || 
+	    xi->rows[i].cells[j] == NULL ||
+	    !IS_STRING(xi->rows[i].cells[j])) {
 	    dbprintf("no: not a string at row %d\n", i);
 	    ret = 0;
 	    break;
 	}
 	dbprintf("first_col_strings: rows[%d].cells[%d]: '%s'\n", i, j,
-		 rows[i].cells[j]);
+		 xi->rows[i].cells[j]);
     }
 
     if (ret) {
@@ -1077,25 +1138,7 @@ static int first_col_strings (wbook *book)
 
 #define obs_string(s) (!strcmp(s, "obs") || !strcmp(s, "id"))
 
-static int fix_varname (char *vname)
-{
-    int i, n = strlen(vname);
-    int bad = 0;
-
-    for (i=1; i<n; i++) {
-        if (!(isalpha((unsigned char) vname[i]))  
-            && !(isdigit((unsigned char) vname[i]))
-            && vname[i] != '_') {
-	    vname[i] = '_';
-	    bad++;
-	}
-    }
-
-    return (bad == n);
-}   
-
-static int 
-check_all_varnames (wbook *book, int totcols, const char *blank_col)
+static int check_all_varnames (wbook *book, xls_info *xi, PRN *prn)
 {
     int j, i = book->row_offset;
     int startcol = book->col_offset;
@@ -1109,21 +1152,21 @@ check_all_varnames (wbook *book, int totcols, const char *blank_col)
 	gotcols = 1;
     }
 
-    if (rows[i].cells == NULL) {
+    if (xi->rows[i].cells == NULL) {
 	fprintf(stderr, "Row %d is empty, trying lower...\n", i);
-	while (i < nrows - 1 && rows[i].cells == NULL) {
+	while (i < xi->nrows - 1 && xi->rows[i].cells == NULL) {
 	    book->row_offset += 1;
 	    i++;
 	}
     }
 
-    for (j=startcol; j<totcols; j++) { 
-	if (blank_col[j]) {
+    for (j=startcol; j<xi->totcols; j++) { 
+	if (xi->blank_col[j]) {
 	    gotcols++;
 	    continue;
 	}
 
-	if (rows[i].cells[j] == NULL) {
+	if (xi->rows[i].cells[j] == NULL) {
 	    dbprintf("got_varnames: rows[%d].cells[%d] is NULL\n", i, j);
 	    break;
 	}
@@ -1131,11 +1174,11 @@ check_all_varnames (wbook *book, int totcols, const char *blank_col)
 	gotcols++;
 
 	dbprintf("got_varnames: rows[%d].cells[%d] is '%s'\n", i, j, 
-		 rows[i].cells[j]);
+		 xi->rows[i].cells[j]);
 
-	if (IS_STRING(rows[i].cells[j])) {
+	if (IS_STRING(xi->rows[i].cells[j])) {
 	    /* skip beyond the quote */
-	    char *test = rows[i].cells[j] + 1;
+	    char *test = xi->rows[i].cells[j] + 1;
 
 	    /* "obs" or "id" is OK in the first col of the selection, 
 	       but not thereafter */
@@ -1143,14 +1186,10 @@ check_all_varnames (wbook *book, int totcols, const char *blank_col)
 		/* pass along */
 		;
 	    } else {
-		int verr = check_varname(test);
+		int verr = check_imported_varname(test, i, j, prn);
 
-		if (verr == VARNAME_BADCHAR) {
-		    verr = fix_varname(test);
-		}
-	    
 		if (verr) {
-		    return VARNAMES_INVALID;
+		    return verr;
 		}
 	    }
 	    vnames++;
@@ -1180,11 +1219,12 @@ struct string_err {
     char *str;
 };
 
+#define xls_cell(x,i,j) (x->rows[i].cells[j])
+
 /* check for invalid data in the selected data block */
 
 static int 
-check_data_block (wbook *book, int totcols, const char *blank_col,
-		  struct string_err *err)
+check_data_block (wbook *book, xls_info *xi, struct string_err *err)
 {
     int startcol = book->col_offset;
     int startrow = book->row_offset + 1;
@@ -1198,31 +1238,32 @@ check_data_block (wbook *book, int totcols, const char *blank_col,
     err->column = 0;
     err->str = NULL;
 
-    for (j=startcol; j<totcols; j++) {
-	if (blank_col[j]) {
+    for (j=startcol; j<xi->totcols; j++) {
+	if (xi->blank_col[j]) {
 	    continue;
 	}
-	for (i=startrow; i<nrows; i++) {
-	    dbprintf("data_block: looking at rows[%d], end = %d\n", i, rows[i].end);
-	    if (rows[i].cells  == NULL) {
+	for (i=startrow; i<xi->nrows; i++) {
+	    dbprintf("data_block: looking at rows[%d], end = %d\n", i, 
+		     xi->rows[i].end);
+	    if (xi->rows[i].cells  == NULL) {
 		dbprintf("data_block: rows[%d].cells = NULL\n", i);
 		ret = -1;
-	    } else if (j >= rows[i].end) {
+	    } else if (j >= xi->rows[i].end) {
 		dbprintf("data_block: short row, fell off the end\n");
 		ret = -1;
-	    } else if (rows[i].cells[j] == NULL) {
+	    } else if (xls_cell(xi, i, j) == NULL) {
 		dbprintf("data_block: rows[%d].cells[%d] = NULL\n", i, j);
-		rows[i].cells[j] = g_strdup("-999");
+		xi->rows[i].cells[j] = g_strdup("-999");
 		ret = -1;
-	    } else if (IS_STRING(rows[i].cells[j])) {
-		if (missval_string(rows[i].cells[j])) {
-		    free(rows[i].cells[j]);
-		    rows[i].cells[j] = g_strdup("-999");
+	    } else if (IS_STRING(xls_cell(xi, i, j))) {
+		if (missval_string(xls_cell(xi, i, j))) {
+		    g_free(xi->rows[i].cells[j]);
+		    xi->rows[i].cells[j] = g_strdup("-999");
 		    ret = -1;
 		} else {
 		    err->row = i + 1;
 		    err->column = j + 1;
-		    err->str = g_strdup(rows[i].cells[j]);
+		    err->str = g_strdup(xls_cell(xi, i, j));
 		    return 1;
 		}
 	    }
@@ -1258,9 +1299,9 @@ n_vars_from_col (wbook *book, int totcols, char *blank_col)
     return nv;
 }
 
-static int transcribe_data (wbook *book, DATASET *dset, 
-			    int totcols, char *blank_col, 
-			    PRN *prn)
+static int 
+transcribe_data (wbook *book, xls_info *xi, DATASET *dset, 
+		 PRN *prn)
 {
     int startcol = book->col_offset;
     int roff = book->row_offset;
@@ -1271,10 +1312,10 @@ static int transcribe_data (wbook *book, DATASET *dset,
 	startcol++;
     } 
 
-    for (i=startcol; i<totcols; i++) {
+    for (i=startcol; i<xi->totcols; i++) {
 	int ts;
 
-	if (blank_col[i]) {
+	if (xi->blank_col[i]) {
 	    continue;
 	}
 
@@ -1285,15 +1326,15 @@ static int transcribe_data (wbook *book, DATASET *dset,
 	dset->varname[j][0] = 0;
 	if (book_auto_varnames(book)) {
 	    sprintf(dset->varname[j], "v%d", j);
-	} else if (rows[roff].cells[i] == NULL) {
+	} else if (xi->rows[roff].cells[i] == NULL) {
 	    sprintf(dset->varname[j], "v%d", j);
-	} else if (i >= rows[roff].end) {
+	} else if (i >= xi->rows[roff].end) {
 	    sprintf(dset->varname[j], "v%d", j);
 	} else {
-	    strncat(dset->varname[j], rows[roff].cells[i] + 1, 
+	    strncat(dset->varname[j], xi->rows[roff].cells[i] + 1, 
 		    VNAMELEN - 1);
 	    dbprintf("accessing rows[%d].cells[%d] at %p\n",
-		     roff, i, (void *) rows[roff].cells[i]);
+		     roff, i, (void *) xi->rows[roff].cells[i]);
 	}
 
 	/* remedial: replace space with underscore */
@@ -1309,17 +1350,17 @@ static int transcribe_data (wbook *book, DATASET *dset,
 
 	for (t=0; t<dset->n; t++) {
 	    ts = t + 1 + roff;
-	    if (rows[ts].cells == NULL || i >= rows[ts].end ||
-		rows[ts].cells[i] == NULL) {
+	    if (xi->rows[ts].cells == NULL || i >= xi->rows[ts].end ||
+		xi->rows[ts].cells[i] == NULL) {
 		continue;
 	    }
 
 	    dbprintf("accessing rows[%d].cells[%d] at %p\n", ts, i,
-		     (void *) rows[ts].cells[i]);
+		     (void *) xi->rows[ts].cells[i]);
 	    dbprintf("setting Z[%d][%d] = rows[%d].cells[%d] "
-		     "= '%s'\n", j, t, i, ts, rows[ts].cells[i]);
+		     "= '%s'\n", j, t, i, ts, xi->rows[ts].cells[i]);
 
-	    dset->Z[j][t] = atof(rows[ts].cells[i]);
+	    dset->Z[j][t] = atof(xls_cell(xi, ts, i));
 	    if (dset->Z[j][t] == -999 || dset->Z[j][t] == -9999) {
 		dset->Z[j][t] = NADBL;
 	    }
@@ -1331,53 +1372,47 @@ static int transcribe_data (wbook *book, DATASET *dset,
     return err;
 }
 
-static int 
-get_sheet_dimensions (wbook *book, int *totcols, int *datacols, 
-		      char **blank_col, PRN *prn)
+static int get_sheet_dimensions (wbook *book, xls_info *xi, PRN *prn)
 {
     char *blanks = NULL;
     int i, j;
 
-    *totcols = 0;
-    *datacols = 0;
-    *blank_col = NULL;
-
     /* trim any trailing blank rows */
-    for (i=nrows-1; i>=0; i--) {
-	if (rows[i].cells == NULL) {
-	    nrows--;
+    for (i=xi->nrows-1; i>=0; i--) {
+	if (xi->rows[i].cells == NULL) {
+	    xi->nrows -= 1;
 	} else {
 	    break;
 	}
     }
 	
-    for (i=0; i<nrows; i++) {
-	if (rows[i].cells != NULL) {
-	    if (rows[i].last + 1 > *totcols) {
-		*totcols = rows[i].last + 1;
+    for (i=0; i<xi->nrows; i++) {
+	if (xi->rows[i].cells != NULL) {
+	    if (xi->rows[i].last + 1 > xi->totcols) {
+		xi->totcols = xi->rows[i].last + 1;
 	    }
 	}
     }
 
-    if (*totcols <= 0 || nrows < 1) {
+    if (xi->totcols <= 0 || xi->nrows < 1) {
 	pputs(prn, _("No data found.\n"));
 	pputs(prn, _(adjust_rc));
 	return 1;
     }
 
-    blanks = malloc(*totcols);
+    blanks = malloc(xi->totcols);
     if (blanks == NULL) {
 	return E_ALLOC;
     }
 
-    memset(blanks, 1, *totcols);
+    memset(blanks, 1, xi->totcols);
 
-    for (i=0; i<nrows; i++) {
-	if (rows[i].cells == NULL) {
+    for (i=0; i<xi->nrows; i++) {
+	if (xi->rows[i].cells == NULL) {
 	    continue;
 	}
-	for (j=0; j<=rows[i].last; j++) {
-	    if (rows[i].cells[j] != NULL) {
+	for (j=0; j<=xi->rows[i].last; j++) {
+	    if (xi->rows[i].cells[j] != NULL) {
 		if (blanks[j]) {
 		    blanks[j] = 0;
 		}
@@ -1385,31 +1420,31 @@ get_sheet_dimensions (wbook *book, int *totcols, int *datacols,
 	}
     }
 
-    for (i=0; i<*totcols; i++) {
+    for (i=0; i<xi->totcols; i++) {
 	if (!blanks[i]) {
-	    *datacols += 1;
+	    xi->datacols += 1;
 	}
     }
 
     if (book_numeric_dates(book)) {
-	*datacols -= 1;
+	xi->datacols -= 1;
     }
 
-    printf("rows=%d, total cols=%d, data cols=%d\n", nrows, 
-	   *totcols, *datacols);
+    printf("rows=%d, total cols=%d, data cols=%d\n", xi->nrows, 
+	   xi->totcols, xi->datacols);
 
-    if (*datacols < 1) {
+    if (xi->datacols < 1) {
 	pputs(prn, _("No data found.\n"));
 	pputs(prn, _(adjust_rc));
 	return 1;
     }
 
-    *blank_col = blanks;
+    xi->blank_col = blanks;
 
     return 0;
 }
 
-static int col0_is_numeric (int nrows, int row_offset, int col_offset)
+static int col0_is_numeric (xls_info *xi, int row_offset, int col_offset)
 {
     int t, tstart = 1 + row_offset;
     int nx = 0;
@@ -1418,8 +1453,8 @@ static int col0_is_numeric (int nrows, int row_offset, int col_offset)
     fprintf(stderr, "testing for all numerical values in col %d\n", 
 	    col_offset);
 
-    for (t=tstart; t<nrows; t++) {
-	test = rows[t].cells[col_offset];
+    for (t=tstart; t<xi->nrows; t++) {
+	test = xi->rows[t].cells[col_offset];
 	if (!numeric_string(test)) {
 	    fprintf(stderr, " no: non-numeric cell at row %d\n", t + 1);
 	    return 0;
@@ -1454,16 +1489,50 @@ static void book_time_series_setup (wbook *book, DATASET *newinfo, int pd)
     book_unset_obs_labels(book);
 }
 
+/* Make a contiguous array of observation labels for the
+   purpose of checking for dated observations. All we need
+   here is a "shell"; the actual strings are already
+   allocated.
+*/
+
+static char **labels_array (xls_info *xi, int row_offset, int j,
+			    DATASET *newset)
+{
+    char *s, **labels = NULL;
+    int i, t, ok = 1;
+
+    for (t=0; t<newset->n; t++) {
+	i = t + row_offset;
+	s = xi->rows[i].cells != NULL ? xi->rows[i].cells[j] : NULL;
+	if (s == NULL || *s == '\0') {
+	    ok = 0;
+	    break;
+	}
+    }
+
+    if (ok) {
+	labels = malloc(newset->n * sizeof *labels);
+	if (labels != NULL) {
+	    for (t=0; t<newset->n; t++) {
+		i = t + row_offset;
+		labels[t] = xi->rows[i].cells[j];
+	    }
+	}
+    }
+
+    return labels;
+}
+
 int xls_get_data (const char *fname, int *list, char *sheetname,
 		  DATASET *dset, gretlopt opt, PRN *prn)
 {
     int gui = (opt & OPT_G);
     wbook xbook;
     wbook *book = &xbook;
+    xls_info xlsi;
+    xls_info *xi = &xlsi;
     DATASET *newset;
-    int datacols, totcols;
     struct string_err strerr;
-    char *blank_col = NULL;
     int ts_markers = 0;
     char **ts_S = NULL;
     int r0, c0;
@@ -1484,6 +1553,7 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
     gretl_push_c_numeric_locale();
 
     wbook_init(book, list, sheetname);
+    xls_info_init(xi);
 
     if (excel_book_get_info(fname, book)) {
 	pputs(prn, _("Failed to get workbook info"));
@@ -1521,7 +1591,7 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
     if (err) goto getout;
 
     /* processing for specific worksheet */
-    err = process_sheet(fname, book, prn);
+    err = process_sheet(fname, book, xi, prn);
 
     if (err) {
 	const char *buf = gretl_print_get_buffer(prn);
@@ -1535,22 +1605,22 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
     }
 
     /* get sizes and locate any blank columns */
-    err = get_sheet_dimensions(book, &totcols, &datacols, &blank_col, prn);
+    err = get_sheet_dimensions(book, xi, prn);
 
     if (err) goto getout;
 
     /* check feasibility of offsets */
-    if (book->row_offset >= nrows) {
+    if (book->row_offset >= xi->nrows) {
 	pputs(prn, _("Starting row is out of bounds.\n"));
 	err = 1;
-    } else if (book->col_offset >= totcols) {
+    } else if (book->col_offset >= xi->totcols) {
 	pputs(prn, _("Starting column is out of bounds.\n"));
 	err = 1;
     }
 
     if (err) goto getout;
 
-    if (first_col_strings(book)) {
+    if (first_col_strings(book, xi)) {
 	puts("found label strings in first imported column");
     } else if (book_numeric_dates(book)) {
 	puts("found calendar dates in first imported column");
@@ -1559,13 +1629,11 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
     }
 
     /* any bad or missing variable names? */
-    err = check_all_varnames(book, totcols, blank_col);
+    err = check_all_varnames(book, xi, prn);
 
     if (err == VARNAMES_NULL || err == VARNAMES_NOTSTR) {
 	pputs(prn, _("One or more variable names are missing.\n"));
 	pputs(prn, _(adjust_rc));
-    } else if (err == VARNAMES_INVALID) {
-	invalid_varname(prn);
     } else if (err == VARNAMES_NONE) {
 	pputs(prn, _("it seems there are no variable names\n"));
 	book_set_auto_varnames(book);
@@ -1576,7 +1644,7 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
     if (err) goto getout; 
 
     /* any bad data? */
-    err = check_data_block(book, totcols, blank_col, &strerr);
+    err = check_data_block(book, xi, &strerr);
 
     if (err == 1) {
 	pprintf(prn, _("Expected numeric data, found string:\n"
@@ -1592,25 +1660,31 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
 
     r0 = book->row_offset;
     c0 = book->col_offset;
-    newset->n = nrows - 1 - book->row_offset;
+    newset->n = xi->nrows - 1 - r0;
 
     if (book_numeric_dates(book) || 
-	(!book_auto_varnames(book) && import_obs_label(rows[r0].cells[c0]))) {
-	pd = importer_dates_check(book->row_offset + 1, book->col_offset, 
-				  &book->flags, NULL, newset, prn, &err);
+	(!book_auto_varnames(book) && import_obs_label(xls_cell(xi, r0, c0)))) {
+	char **labels = labels_array(xi, r0 + 1, c0, newset);
+
+	if (labels != NULL) {
+	    pd = importer_dates_check(labels, &book->flags, newset, prn, &err);
+	    free(labels);
+	}
+
 	if (pd > 0) {
 	    /* got time-series info from dates/labels */
 	    book_time_series_setup(book, newset, pd);
 	    ts_markers = newset->markers;
 	    ts_S = newset->S;
-	} else if (!book_numeric_dates(book) && alpha_cell(rows[r0].cells[c0]) && 
-	    col0_is_numeric(nrows, r0, c0)) {
+	} else if (!book_numeric_dates(book) && 
+		   alpha_cell(xls_cell(xi, r0, c0)) && 
+		   col0_is_numeric(xi, r0, c0)) {
 	    book_unset_obs_labels(book);
 	}
     }
 
     /* dimensions of the dataset */
-    newset->v = n_vars_from_col(book, totcols, blank_col);
+    newset->v = n_vars_from_col(book, xi->totcols, xi->blank_col);
     fprintf(stderr, "newset->v = %d, newset->n = %d\n",
 	    newset->v, newset->n);
 
@@ -1628,7 +1702,7 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
     } 
 
     /* OK: actually populate the dataset */
-    err = transcribe_data(book, newset, totcols, blank_col, prn);
+    err = transcribe_data(book, xi, newset, prn);
     if (err) {
 	goto getout;
     }
@@ -1643,10 +1717,10 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
 	    i = book->col_offset;
 	    for (t=0; t<newset->n; t++) {
 		int ts = t + 1 + book->row_offset;
-		char *src = rows[ts].cells[i];
+		char *src = xls_cell(xi, ts, i);
 
 		if (src != NULL) {
-		    strncat(newset->S[t], src + 1, OBSLEN - 1);
+		    gretl_utf8_strncat(newset->S[t], src + 1, OBSLEN - 1);
 		}
 	    }
 	}
@@ -1668,9 +1742,8 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
 
  getout:
     
-    free(blank_col);
+    free_xls_info(xi);
     wbook_free(book);
-    free_sheet();
 
     gretl_pop_c_numeric_locale();
 
@@ -1686,6 +1759,3 @@ int xls_get_data (const char *fname, int *list, char *sheetname,
 
     return err;
 }  
-
-
-
