@@ -22,6 +22,7 @@
 */
 
 #include "libgretl.h"
+#include "version.h"
 #include "libset.h"
 #include "var.h"
 
@@ -110,7 +111,7 @@ write_garch_stats (MODEL *pmod, const int *list, const DATASET *dset,
     err = gretl_model_write_coeffs(pmod, theta, npar);
 
     if (!err) {
-	gretl_model_write_vcv(pmod, V, -1);
+	gretl_model_write_vcv(pmod, V);
     }
 
     if (err) {
@@ -245,15 +246,15 @@ static int get_vopt (int robust)
 
     /* The defaults: QML if "robust" option is in force,
        otherwise negative Hessian */
-    if (vopt == VCV_UNSET) {
+    if (vopt == ML_UNSET) {
 	if (robust) {
-	    if (ropt == VCV_UNSET) {
-		vopt = VCV_QML;
+	    if (ropt == ML_UNSET) {
+		vopt = ML_QML;
 	    } else {
 		vopt = ropt;
 	    }
 	} else {
-	    vopt = VCV_HESSIAN;
+	    vopt = ML_HESSIAN;
 	}
     }
 
