@@ -22,6 +22,7 @@
 #define FULL_XML_HEADERS
 
 #include "libgretl.h"
+#include "version.h"
 #include "gretl_xml.h"
 #include "importer.h"
 #include "csvdata.h"
@@ -304,7 +305,7 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 		if (VTYPE_IS_NUMERIC(vtype) || vtype == VALUE_STRING) {
 		    if (i == 0) {
 			/* first column: write content to labels */
-			strncat(sheet->label[t], tmp, OBSLEN - 1);
+			gretl_utf8_strncat(sheet->label[t], tmp, OBSLEN - 1);
 		    }
 		}
 
@@ -330,11 +331,8 @@ static int wsheet_parse_cells (xmlNodePtr node, wsheet *sheet, PRN *prn)
 			if (i == 0 && import_obs_label(tmp)) {
 			    ; /* keep going */
 			} else {
-			    charsub(sheet->varname[i], ' ', '_');
-			    if (check_varname(sheet->varname[i])) {
-				invalid_varname(prn);
-				err = 1;
-			    }
+			    err = check_imported_varname(sheet->varname[i],
+							 r, c, prn);
 			}
 		    } else if (i == 0) {
 			/* first column, not first row */
@@ -576,6 +574,15 @@ static int labels_numeric (char **S, int n)
     return 1;
 }
 
+#if 0 /* not yet */
+static const char *get_label (int i, int j, void *p)
+{
+    char **labels = p;
+
+    return labels[i];
+}
+#endif
+
 int gnumeric_get_data (const char *fname, int *list, char *sheetname,
 		       DATASET *dset, gretlopt opt, PRN *prn)
 {
@@ -688,7 +695,7 @@ int gnumeric_get_data (const char *fname, int *list, char *sheetname,
 
 	if (book_numeric_dates(book) || 
 	    (sheet->colheads > 0 && import_obs_label(sheet->label[0]))) {
-	    pd = importer_dates_check(r0, 0, &book->flags, sheet->label,
+	    pd = importer_dates_check(sheet->label + r0, &book->flags,
 				      newset, prn, &err);
 	    if (pd > 0) {
 		/* got time-series info from dates/labels */
