@@ -18,6 +18,7 @@
  */
 
 #include "libgretl.h"
+#include "version.h"
 #include "matrix_extra.h"
 #include "libset.h"
 #include "missing_private.h"
@@ -1318,6 +1319,30 @@ static gretl_matrix *heckit_init_H (double *theta,
 
 #endif
 
+static void heckit_trim_vcv (MODEL *pmod, int k,
+			     gretl_matrix *V)
+{
+    double vij;
+    int n = V->rows;
+    int i, j;
+
+    gretl_matrix_reuse(V, k, k);
+
+    for (i=0; i<k; i++) {
+	for (j=0; j<=i; j++) {
+	    vij = pmod->vcv[ijton(i, j, n)];
+	    gretl_matrix_set(V, i, j, vij);
+	}
+    }
+
+    for (i=0; i<k; i++) {
+	for (j=0; j<=i; j++) {
+	    vij = gretl_matrix_get(V, i, j);
+	    pmod->vcv[ijton(i, j, k)] = vij;
+	}
+    }    
+}
+
 int heckit_ml (MODEL *hm, h_container *HC, gretlopt opt, PRN *prn)
 {
     gretl_matrix *H = NULL;
@@ -1440,8 +1465,11 @@ int heckit_ml (MODEL *hm, h_container *HC, gretlopt opt, PRN *prn)
     }
 
     if (!err) {
-	np = HC->vcv->rows - 2;
-	err = gretl_model_write_vcv(hm, HC->vcv, np);
+	err = gretl_model_write_vcv(hm, HC->vcv);
+	if (!err) {
+	    np = HC->vcv->rows - 2;
+	    heckit_trim_vcv(hm, np, HC->vcv);
+	}
     }
 
     free(hess);
@@ -1607,7 +1635,7 @@ MODEL heckit_estimate (const int *list, DATASET *dset,
     }
 
     if (opt & OPT_R) {
-	gretl_model_set_vcv_info(&hm, VCV_ML, VCV_QML);
+	gretl_model_set_vcv_info(&hm, VCV_ML, ML_QML);
 	hm.opt |= OPT_R;
     }
 
