@@ -21,10 +21,42 @@
 # include "gtk_compat.h"
 #endif
 
-static void invalid_varname (PRN *prn)
+static int check_imported_varname (char *vname, int row, int col,
+				   PRN *prn)
 {
-    pputs(prn, gretl_errmsg_get());
-    pputs(prn, _("\nPlease rename this variable and try again"));
+    int err;
+
+    charsub(vname, ' ', '_');
+
+    err = check_varname(vname);
+
+    if (err == VARNAME_BADCHAR) {
+	/* the first byte was OK: try overwriting illegal 
+	   characters with '_' 
+	*/
+	char *s = vname + 1;
+
+	while (*s) {
+	    if (!(isalpha((unsigned char) *s))  
+		&& !(isdigit((unsigned char) *s))
+		&& *s != '_') {
+		*s = '_';
+	    }
+	    s++;
+	}
+
+	err = 0;
+    }
+
+    if (err) {
+	if (row >= 0 && col >= 0) {
+	    pprintf(prn, _("At row %d, column %d:\n"), row, col);
+	}
+	pputs(prn, gretl_errmsg_get());
+	pputs(prn, _("\nPlease rename this variable and try again"));
+    }
+
+    return (err)? E_DATA : 0;
 }
 
 #if defined(ODS_IMPORTER) || defined(XLSX_IMPORTER)
@@ -92,10 +124,10 @@ static int gretl_make_tempdir (char *dname)
 
 # endif /* G_OS_WIN32 or not */
 
-/* for ODS and XLSX: unzip the target file in the user's
+/* For ODS and XLSX: unzip the target file in the user's
    "dotdir". On successful completion @dname holds the
-   name of the temporary directory, in the dotdir, holding
-   the contents of the zipfile
+   name of the temporary subdirectory, in the dotdir, 
+   holding the contents of the zipfile.
 */
 
 static int open_import_zipfile (const char *fname, char *dname,
@@ -103,29 +135,45 @@ static int open_import_zipfile (const char *fname, char *dname,
 {
     int (*gretl_unzip_file)(const char *, GError **);
     const char *udir = gretl_dotdir();
+    const char *real_fname = fname;
+    char *recoded_fname = NULL;
     char *abspath = NULL;
     void *handle = NULL;
-    FILE *fp;
     GError *gerr = NULL;
+    FILE *fp;
     int err = 0;
 
     errno = 0;
     *dname = '\0';
 
-    fp = gretl_fopen(fname, "r");
+    /* In case @fname is in UTF-8 but we're on MS Windows (or
+       conceivably on an old non-UTF-8 *nix system), grab the
+       appropriately recoded filename to pass into the
+       zipfile apparatus, since in that context a filename
+       that works with plain system stdio is expected.
+    */
+
+    fp = gretl_fopen_with_recode(fname, "r", &recoded_fname);
     if (fp == NULL) {
 	return E_FOPEN;
     }
 
     fclose(fp);
 
-    /* by doing chdir, we may lose track of the file if 
-       its path is relative */
-    if (!g_path_is_absolute(fname)) {
-	abspath = get_absolute_path(fname);
+    if (recoded_fname != NULL) {
+	real_fname = recoded_fname;
     }
 
-    /* cd to user dir */
+    /* by doing chdir, we may lose track of the file if 
+       its path is relative */
+    if (!g_path_is_absolute(real_fname)) {
+	abspath = get_absolute_path(real_fname);
+	if (abspath != NULL) {
+	    real_fname = abspath;
+	}
+    }
+
+    /* cd to user dir and make temporary dir */
     if (gretl_chdir(udir)) {
 	gretl_errmsg_set_from_errno(udir);
 	err = E_FOPEN;
@@ -149,11 +197,7 @@ static int open_import_zipfile (const char *fname, char *dname,
     }
 
     if (!err) {
-	if (abspath != NULL) {
-	    err = (*gretl_unzip_file)(abspath, &gerr);
-	} else {
-	    err = (*gretl_unzip_file)(fname, &gerr);
-	}
+	err = (*gretl_unzip_file)(real_fname, &gerr);
 	if (gerr != NULL) {
 	    pprintf(prn, "gretl_unzip_file: '%s'\n", gerr->message);
 	    g_error_free(gerr);
@@ -163,10 +207,8 @@ static int open_import_zipfile (const char *fname, char *dname,
     }
 
     free(abspath);
-
-    if (handle != NULL) {
-	close_plugin(handle);
-    }
+    free(recoded_fname);
+    close_plugin(handle);
 
     return err;
 }
@@ -242,37 +284,31 @@ static int worksheet_start_dataset (DATASET *newinfo)
     }
 }
 
-#ifdef EXCEL_IMPORTER
-# define cell_string(i,j) ((rows[i].cells != NULL)? rows[i].cells[j] : NULL)
-#else
-# define cell_string(i,j) (labels[i])
-#endif
-
 static int 
-importer_dates_check (int row_offset, int col_offset, 
-		      BookFlag *pflags, char **labels, 
-		      DATASET *newinfo, PRN *prn, int *err)
+importer_dates_check (char **labels, BookFlag *pflags,
+		      DATASET *newset, PRN *prn, 
+		      int *err)
 {
     int d, t;
     char dstr[12];
     char *s;
     int ret = 0;
 
-    for (t=0; t<newinfo->n; t++) {
-	s = cell_string(t + row_offset, col_offset);
+    for (t=0; t<newset->n; t++) {
+	s = labels[t];
 	if (s == NULL || *s == '\0') {
 	    fprintf(stderr, "importer_dates_check: got blank label\n");
 	    return 0;
 	}
     }
 
-    *err = dataset_allocate_obs_markers(newinfo);
+    *err = dataset_allocate_obs_markers(newset);
     if (*err) {
 	return 0;
     }
 
-    for (t=0; t<newinfo->n && !*err; t++) {
-	s = cell_string(t + row_offset, col_offset);
+    for (t=0; t<newset->n && !*err; t++) {
+	s = labels[t];
 	if (*s == '"' || *s == '\'') s++;
 	if (*pflags & BOOK_NUMERIC_DATES) {
 	    if (sscanf(s, "%d", &d)) {
@@ -283,20 +319,20 @@ importer_dates_check (int row_offset, int col_offset,
 		*err = E_DATA;
 	    }
 	}
-	strncat(newinfo->S[t], s, OBSLEN - 1);
+	strncat(newset->S[t], s, OBSLEN - 1);
     }
 
     if (!*err) {
 	int reversed = 0;
 
-	ret = test_markers_for_dates(newinfo, &reversed, NULL, prn);
+	ret = test_markers_for_dates(newset, &reversed, NULL, prn);
 	if (reversed) {
 	    *pflags |= BOOK_DATA_REVERSED;
 	}
     }
 
-    if (newinfo->markers != DAILY_DATE_STRINGS) {
-	dataset_destroy_obs_markers(newinfo);
+    if (newset->markers != DAILY_DATE_STRINGS) {
+	dataset_destroy_obs_markers(newset);
     }
 
     return ret;
@@ -369,6 +405,11 @@ static void wbook_record_params (wbook *book, int *list)
 
 #endif /* !ODS_IMPORTER */
 
+/* @list may contain sheet number, row and/or column offset;
+   @sheetname may contain the name of a specific sheet; but
+   both may be NULL
+*/
+
 static void wbook_init (wbook *book, const int *list, char *sheetname)
 {
     book->version = 0;
