@@ -18,6 +18,7 @@
  */
 
 #include "libgretl.h"
+#include "version.h"
 #include "libset.h"
 #include "gretl_bfgs.h"
 
@@ -40,7 +41,7 @@ struct int_container_ {
     double *hi, *lo;  /* limit series for dependent variable */
     int *obstype;     /* dependent variable classifier */
     int typecount[4]; /* no. of obs by category */
-    double **X;       /* regressors */
+    gretl_matrix *X;  /* regressors */
     int *list;        /* dependent vars plus regular regressors */
     int t1;           /* beginning of sample */
     int t2;           /* end of sample */
@@ -53,7 +54,7 @@ struct int_container_ {
     double *dP;       /* probabilities */
     double *f0;       /* normalized density at min */
     double *f1;       /* normalized density at max */
-    double **G;       /* score matrix by observation */
+    gretl_matrix *G;  /* score matrix by observation */
     double *g;        /* total score vector */
 };
 
@@ -67,8 +68,8 @@ static void int_container_destroy (int_container *IC)
     free(IC->obstype);
     free(IC->list);
 
-    doubles_array_free(IC->X, IC->nx);
-    doubles_array_free(IC->G, IC->k);
+    gretl_matrix_free(IC->X);
+    gretl_matrix_free(IC->G);
 
     free(IC);
 }
@@ -78,7 +79,7 @@ static int_container *int_container_new (int *list, DATASET *dset,
 {
     int_container *IC;
     int nobs, i, s, t, vlo = list[1], vhi = list[2];
-    double x0, x1;
+    double x0, x1, xti;
 
     IC = malloc(sizeof *IC);
     if (IC == NULL) {
@@ -113,9 +114,9 @@ static int_container *int_container_new (int *list, DATASET *dset,
     IC->theta = malloc(IC->k * sizeof *IC->theta);
     IC->g = malloc(IC->k * sizeof *IC->g);
 
-    /* two-dimensional doubles arrays */
-    IC->X = doubles_array_new(IC->nx, nobs);
-    IC->G = doubles_array_new(IC->k, nobs);
+    /* big matrices */
+    IC->X = gretl_matrix_alloc(nobs, IC->nx);
+    IC->G = gretl_matrix_alloc(nobs, IC->k);
 
     /* int arrays */
     IC->obstype = malloc(nobs * sizeof *IC->obstype);
@@ -140,7 +141,6 @@ static int_container *int_container_new (int *list, DATASET *dset,
     s = 0;
     for (t=mod->t1; t<=mod->t2; t++) {
 	if (!na(mod->uhat[t])) {
-
 	    x0 = dset->Z[vlo][t];
 	    x1 = dset->Z[vhi][t];
 	    IC->lo[s] = x0;
@@ -157,9 +157,9 @@ static int_container *int_container_new (int *list, DATASET *dset,
 	    }
 
 	    for (i=0; i<IC->nx; i++) {
-		IC->X[i][s] = dset->Z[list[i+3]][t];
+		xti = dset->Z[list[i+3]][t];
+		gretl_matrix_set(IC->X, s, i, xti);
 	    }
-
 	    s++;
 	}
     }
@@ -250,13 +250,14 @@ static int create_midpoint_y (int *list, DATASET *dset,
 static void loglik_prelim (const double *theta, int_container *IC)
 {
     int i, t, k = IC->k;
-    double ndxt, z0, z1, x0, x1;
+    double ndxt, z0, z1, x0, x1, xti;
     double sigma = exp(theta[k-1]);
 
     for (t=0; t<IC->nobs; t++) {
 	ndxt = 0.0;
 	for (i=0; i<IC->nx; i++) {
-	    ndxt += theta[i] * IC->X[i][t];
+	    xti = gretl_matrix_get(IC->X, t, i);
+	    ndxt += theta[i] * xti;
 	}
 
 	IC->ndx[t] = ndxt;
@@ -292,10 +293,10 @@ static void loglik_prelim (const double *theta, int_container *IC)
     }
 }
 
-static double int_loglik (const double *theta, void *ptr)
+static double interval_loglik (const double *theta, void *ptr)
 {
     int_container *IC = (int_container *) ptr;
-    double x0, x1, z0, z1;
+    double x0, x1, z0, z1, gti;
     double derivs = 0.0, derivb = 0.0;
     double sigma, ndxt, ll = 0.0;
     int i, t, k = IC->k;
@@ -339,11 +340,12 @@ static double int_loglik (const double *theta, void *ptr)
 	ll += log(IC->dP[t]);
 
 	for (i=0; i<IC->nx; i++) {
-	    IC->G[i][t] = derivb * IC->X[i][t];
-	    IC->g[i] += IC->G[i][t]; 
+	    gti = derivb * gretl_matrix_get(IC->X, t, i);
+	    gretl_matrix_set(IC->G, t, i, gti);
+	    IC->g[i] += gti; 
 	}
 
-	IC->G[k-1][t] = derivs;
+	gretl_matrix_set(IC->G, t, k-1, derivs);
 	IC->g[k-1] += derivs;
     }
     
@@ -359,8 +361,8 @@ static double int_loglik (const double *theta, void *ptr)
     return ll;
 }
 
-static int int_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll, 
-		      void *ptr)
+static int interval_score (double *theta, double *s, int npar, 
+			   BFGS_CRIT_FUNC ll, void *ptr)
 {
     int_container *IC = (int_container *) ptr;
     int i;
@@ -372,7 +374,7 @@ static int int_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll,
     return 0;
 }
 
-int int_ahess (double *theta, gretl_matrix *V, void *ptr)
+int interval_hessian (double *theta, gretl_matrix *V, void *ptr)
 {
     int_container *IC = (int_container *) ptr;
     double z0 = 0, z1 = 0, mu = 0;
@@ -393,7 +395,7 @@ int int_ahess (double *theta, gretl_matrix *V, void *ptr)
     }
 
 #if INTDEBUG > 1
-    fprintf(stderr, "int_ahess:\n");
+    fprintf(stderr, "interval_hessian:\n");
     for (i=0; i<k; i++) {
 	fprintf(stderr, "theta[%2d] = %20.16f\n", i, theta[i]);
     }
@@ -442,9 +444,9 @@ int int_ahess (double *theta, gretl_matrix *V, void *ptr)
 	}
 
 	for (i=0; i<IC->nx; i++) {
-	    x0 = IC->X[i][t];
+	    x0 = gretl_matrix_get(IC->X, t, i);
 	    for (j=i; j<IC->nx; j++) {
-		x1 = IC->X[j][t];
+		x1 = gretl_matrix_get(IC->X, t, j);
 		vij = gretl_matrix_get(V, i, j);
 		vij += x * x0 * x1;
 		gretl_matrix_set(V, i, j, vij);
@@ -458,7 +460,7 @@ int int_ahess (double *theta, gretl_matrix *V, void *ptr)
 	}
 
 	for (i=0; i<IC->nx; i++) {
-	    x0 = IC->X[i][t];
+	    x0 = gretl_matrix_get(IC->X, t, i);
 	    vij = gretl_matrix_get(V, i, k-1);
 	    vij += x * x0;
 	    gretl_matrix_set(V, i, k-1, vij);
@@ -489,82 +491,82 @@ int int_ahess (double *theta, gretl_matrix *V, void *ptr)
     return err;
 }
 
-static int 
-int_ahess_inverse (double *theta, gretl_matrix *V, void *ptr)
+static gretl_matrix *interval_hessian_inverse (double *theta, 
+					       void *ptr,
+					       int *err)
 {
-    int err = int_ahess(theta, V, ptr);
+    int_container *IC = ptr;
+    gretl_matrix *H;
 
-    if (!err) {
-	err = gretl_invert_symmetric_matrix(V);
-    }
-
-    return err;
-}
-
-static gretl_matrix *intreg_sandwich (int_container *IC,
-				      gretl_matrix *H,
-				      int *err)
-{
-    gretl_matrix *G, *S, *V;
-    double x;
-    int k = IC->k;
-    int i, j, jj = 0;
-
-    G = gretl_matrix_alloc(IC->nobs, k);
-    S = gretl_matrix_alloc(k, k);
-    V = gretl_matrix_alloc(k, k);
-
-    if (G == NULL || S == NULL || V == NULL) {
-	gretl_matrix_free(V);
-	V = NULL;
-	*err = E_ALLOC;
-	goto bailout;
-    }
-
-    /* form the G matrix */
-    for (j=0; j<k; j++) {
-	for (i=0; i<IC->nobs; i++) {
-	    x = IC->G[j][i];
-	    G->val[jj++] = x;
-	}
-    }
-
-    /* form S = G'G */
-    *err = gretl_matrix_multiply_mod(G, GRETL_MOD_TRANSPOSE,
-				     G, GRETL_MOD_NONE,
-				     S, GRETL_MOD_NONE);
-
-    if (!*err) {
-	/* form sandwich: V = H^{-1} S H^{-1} */
-	*err = gretl_matrix_qform(H, GRETL_MOD_NONE, S,
-				  V, GRETL_MOD_NONE);
-    }
-
- bailout:
-    
-    gretl_matrix_free(G);
-    gretl_matrix_free(S);
-    gretl_matrix_free(H);
-
-    return V;
-}
-
-static gretl_matrix *intreg_VCV (int_container *IC, gretlopt opt,
-				 int *err)
-{
-    gretl_matrix *H = gretl_zero_matrix_new(IC->k, IC->k);
+    H = gretl_zero_matrix_new(IC->k, IC->k);
 
     if (H == NULL) {
 	*err = E_ALLOC;
     } else {
-	*err = int_ahess_inverse(IC->theta, H, IC);
+	*err = interval_hessian(theta, H, ptr);
     }
 
-    if (!*err && (opt & OPT_R)) {
-	return intreg_sandwich(IC, H, err);
-    } else {
-	return H;
+    if (!*err) {
+	*err = gretl_invert_symmetric_matrix(H);
     }
+
+    return H;
+}
+
+static void interval_trim_vcv (MODEL *pmod, int k,
+			       gretl_matrix *V)
+{
+    double vij;
+    int n = V->rows;
+    int i, j;
+
+    gretl_matrix_reuse(V, k, k);
+
+    for (i=0; i<k; i++) {
+	for (j=0; j<=i; j++) {
+	    vij = pmod->vcv[ijton(i, j, n)];
+	    gretl_matrix_set(V, i, j, vij);
+	}
+    }
+
+    for (i=0; i<k; i++) {
+	for (j=0; j<=i; j++) {
+	    vij = gretl_matrix_get(V, i, j);
+	    pmod->vcv[ijton(i, j, k)] = vij;
+	}
+    }    
+}
+
+static int intreg_model_add_vcv (MODEL *pmod,
+				 int_container *IC, 
+				 const DATASET *dset,
+				 gretlopt opt,
+				 double *x)
+{
+    gretl_matrix *H = NULL;
+    int err = 0;
+
+    H = interval_hessian_inverse(IC->theta, IC, &err);
+
+    if (!err) {
+	if (opt & OPT_R) {
+	    err = gretl_model_add_QML_vcv(pmod, pmod->ci, H, IC->G,
+					  dset, opt);
+	} else {
+	    err = gretl_model_add_hessian_vcv(pmod, H);
+	}
+    }
+
+    if (!err) {
+	int k = IC->k - 1;
+
+	*x = gretl_model_get_vcv_element(pmod, k, k, H->rows);
+	interval_trim_vcv(pmod, IC->nx, H);
+    }
+
+    gretl_matrix_free(H);
+
+    return err;
 }
 
 static void int_compute_gresids (int_container *IC)
@@ -660,7 +662,7 @@ static gretl_matrix *cond_moments (int_container *IC, double *sm3,
     int k = IC->k;  
     int noc = k + 2;  
     double a, b, phi0, phi1, m1, m2, u, x, y;
-    double m3 = 0.0, m4 = 0.0;
+    double gti, m3 = 0.0, m4 = 0.0;
     double sigma = exp(IC->theta[k - 1]);
     int i, t;
 
@@ -710,7 +712,8 @@ static gretl_matrix *cond_moments (int_container *IC, double *sm3,
 	*sm4 += m4;
 
 	for (i=0; i<k; i++) {
-	    gretl_matrix_set(ret, t, i, IC->G[i][t]);
+	    gti = gretl_matrix_get(IC->G, t, i);
+	    gretl_matrix_set(ret, t, i, gti);
 	}
 
 	gretl_matrix_set(ret, t, k, m3);
@@ -770,7 +773,7 @@ static int intreg_normtest (int_container *IC, double *teststat)
     return err;
 }
 
-static int fill_intreg_model (int_container *IC, gretl_matrix *V,
+static int fill_intreg_model (int_container *IC, 
 			      int fncount, int grcount,
 			      const DATASET *dset,
 			      gretlopt opt)
@@ -778,7 +781,7 @@ static int fill_intreg_model (int_container *IC, gretl_matrix *V,
     MODEL *pmod = IC->pmod;
     double x, ndx, u;
     int i, j, k = IC->k;
-    int obstype, vtype;
+    int obstype;
     int err = 0;
 
     pmod->ci = (opt & OPT_T)? TOBIT : INTREG;
@@ -790,23 +793,20 @@ static int fill_intreg_model (int_container *IC, gretl_matrix *V,
 	pmod->coeff[i] = IC->theta[i];
     }
 
-    err = gretl_model_write_vcv(pmod, V, IC->nx);
+    err = intreg_model_add_vcv(pmod, IC, dset, opt, &x);
     if (err) {
-	return err;
+	goto bailout;
     }
 
-    vtype = (opt & OPT_R)? VCV_QML : VCV_HESSIAN;
-    gretl_model_set_vcv_info(pmod, VCV_ML, vtype);
-
-    /* get the s.e. of sigma via the delta method */
-    x = gretl_matrix_get(V, k-1, k-1);
-    x *= pmod->sigma * pmod->sigma;
-    gretl_model_set_double(pmod, "se_sigma", sqrt(x));
+     if (!na(x)) {
+	/* get the s.e. of sigma via the delta method */
+	 x *= pmod->sigma * pmod->sigma;
+	 gretl_model_set_double(pmod, "se_sigma", sqrt(x));
+    }
 
     int_compute_gresids(IC);
 
     j = 0;
-
     for (i=IC->t1; i<=IC->t2; i++) {
 	if (!na(pmod->uhat[i])) {
 	    obstype = IC->obstype[j];
@@ -844,10 +844,6 @@ static int fill_intreg_model (int_container *IC, gretl_matrix *V,
 	gretl_model_set_int(pmod, "n_point", IC->typecount[3]);
     }
 
-    if (opt & OPT_R) {
-	pmod->opt |= OPT_R;
-    }
-
     if (IC->pmod->ifc && IC->nx > 1) {
 	pmod->chisq = chisq_overall_test(IC);
     } else {
@@ -859,14 +855,19 @@ static int fill_intreg_model (int_container *IC, gretl_matrix *V,
 	pmod->list[1] = 0;
     }
 
-    return 0;
+ bailout:
+
+    if (err && pmod->errcode == 0) {
+	pmod->errcode = err;
+    }
+
+    return err;
 }
 
 static int do_interval (int *list, DATASET *dset, MODEL *mod, 
 			gretlopt opt, PRN *prn) 
 {
     int_container *IC;
-    gretl_matrix *V = NULL;
     int maxit, fncount, grcount = 0;
     double toler, normtest = NADBL;
     gretlopt maxopt = opt & OPT_V;
@@ -886,27 +887,24 @@ static int do_interval (int *list, DATASET *dset, MODEL *mod,
 
     if (use_bfgs) {
 	err = BFGS_max(IC->theta, IC->k, maxit, toler, 
-		       &fncount, &grcount, int_loglik, C_LOGLIK,
-		       int_score, IC, NULL, maxopt, prn);
+		       &fncount, &grcount, interval_loglik, C_LOGLIK,
+		       interval_score, IC, NULL, maxopt, prn);
     } else {
 	double crittol = 1.0e-07;
 	double gradtol = 1.0e-07;
 	
 	err = newton_raphson_max(IC->theta, IC->k, maxit, crittol, gradtol,
-				 &fncount, C_LOGLIK, int_loglik, 
-				 int_score, int_ahess, IC, maxopt, prn);
+				 &fncount, C_LOGLIK, interval_loglik, 
+				 interval_score, interval_hessian, 
+				 IC, maxopt, prn);
     }
 
     if (!err) {
-	IC->ll = int_loglik(IC->theta, IC);
+	IC->ll = interval_loglik(IC->theta, IC);
     }
 
-    if (!err) {
-	V = intreg_VCV(IC, opt, &err);
-    }
-
-    if (!err) {
-	err = fill_intreg_model(IC, V, fncount, grcount, dset, opt);
+     if (!err) {
+	err = fill_intreg_model(IC, fncount, grcount, dset, opt);
     }
 
     if (!err) {
@@ -918,7 +916,6 @@ static int do_interval (int *list, DATASET *dset, MODEL *mod,
 	err = 0;
     }
 
-    gretl_matrix_free(V);
     int_container_destroy(IC);
 
     return err;
@@ -977,6 +974,10 @@ MODEL interval_estimate (int *list, DATASET *dset,
     dataset_drop_last_variables(1, dset);
     free(initlist);
 
+    if (opt & OPT_C) {
+	opt |= OPT_R;
+    }
+
     /* do the actual analysis */
     model.errcode = do_interval(list, dset, &model, opt, prn);
 
@@ -1058,6 +1059,10 @@ MODEL tobit_via_intreg (int *list, double llim, double rlim,
     model.errcode = tobit_add_lo_hi(&model, llim, rlim,
 				    dset, &ilist);
 
+    if (opt & OPT_C) {
+	opt |= OPT_R;
+    }
+
     if (!model.errcode) {
 	/* do the actual analysis */
 	model.errcode = do_interval(ilist, dset, &model, 
