@@ -18,6 +18,7 @@
  */
 
 #include "libgretl.h"
+#include "version.h"
 #include "pvalues.h"
 #include "gretl_matrix.h"
 #include "matrix_extra.h"
@@ -204,9 +205,12 @@ gamma_LR_T_pval (double trace, JohansenCode det, int n, int T)
   @T:  sample size minus number of parameters, or
        0 for the asymptotic result
 
-  The Doornik gamma-approximation approach with 
-  correction for the case of a "partial system" as
-  discussed in Harbo, Johansen, Nielsen and Rahbek.
+  The Doornik gamma-approximation approach with correction for the
+  case of a "partial system" as discussed in Harbo, Johansen, Nielsen
+  and Rahbek, "Asymptotic Inference on Cointegrating Rank in Partial
+  Systems" (Journal of Business and Economic Statistics 16/4, October
+  1998). The critical values were re-simulated in gretl with 50,000
+  replications.  
 */
 
 static double
@@ -663,8 +667,7 @@ static int vecm_check_size (GRETL_VAR *v, int flags)
 	    v->X->cols = v->ncoeff;
 	    v->B->rows = v->ncoeff;
 	}
-	v->Y->cols = v->neqns;
-	v->B->cols = v->neqns;
+
 	return 0;
     }
 
@@ -698,35 +701,23 @@ static int vecm_check_size (GRETL_VAR *v, int flags)
 	} else {
 	    err = gretl_matrix_realloc(v->X, v->T, xc);
 	    if (!err) {
+		/* record revised full size */
 		v->xcols = xc;
 	    }
 	}
     } 
 
-    if (err) {
-	return err;
-    } 
-
-    if (v->B == NULL) {
-	v->B = gretl_matrix_alloc(xc, v->neqns);
+    if (!err) {
 	if (v->B == NULL) {
-	    err = E_ALLOC;
-	}	
-    } else if (v->B->rows < xc) {
-	/* B may have extra cols for restricted terms; the
-	   true number of allocated columns is recorded
-	   in v->ycols
-	*/
-	err = gretl_matrix_realloc(v->B, xc, v->ycols);
-	if (!err && v->ycols > v->neqns) {
-	    v->B->cols = v->neqns;
+	    v->B = gretl_matrix_alloc(xc, v->neqns);
+	    if (v->B == NULL) {
+		err = E_ALLOC;
+	    }	
+	} else if (v->B->rows < xc) {
+	    err = gretl_matrix_realloc(v->B, xc, v->neqns);
 	}
     }
 
-    if (!err && v->Y->cols > v->neqns) {
-	gretl_matrix_reuse(v->Y, -1, v->neqns);
-    }
-
     return err;
 }
 
@@ -767,7 +758,7 @@ static int add_EC_terms_to_X (GRETL_VAR *v, gretl_matrix *X,
 	    /* restricted exog vars */
 	    if (v->rlist != NULL) {
 		for (i=0; i<v->rlist[0]; i++) {
-		    xt = dset->Z[v->rlist[i+1]][t-1];
+		    xt = dset->Z[v->rlist[i+1]][t]; /* was t-1 */
 		    bij = gretl_matrix_get(B, ii++, j);
 		    bxt += bij * xt;
 		}
@@ -781,7 +772,8 @@ static int add_EC_terms_to_X (GRETL_VAR *v, gretl_matrix *X,
 }
 
 /* preparing for OLS conditional on beta: construct the
-   appropriate dependent variable matrix, Y */
+   appropriate dependent variable matrix, Y 
+*/
 
 static int make_vecm_Y (GRETL_VAR *v, const DATASET *dset,
 			const gretl_matrix *Pi)
@@ -824,7 +816,7 @@ static int make_vecm_Y (GRETL_VAR *v, const DATASET *dset,
 			} else {
 			    k = j - v->ylist[0] - auto_restr(v) + 1;
 			    vj = v->rlist[k];
-			    xti = dset->Z[vj][t-1];
+			    xti = dset->Z[vj][t]; /* was t-1 */
 			} 
 			yti -= pij * xti;
 		    }
@@ -1609,9 +1601,8 @@ compute_coint_test (GRETL_VAR *jvar, const DATASET *dset,
 	    pv = gamma_harbo_trace_pval(trace, det, n, nrexo, i, jvar->df);
 	    pprintf(prn, " [%6.4f]\n", pv);
 	}
-    } 
-
-    if (!partial) {
+    } else {
+	/* !partial */
 	double pv;
 
 	pputc(prn, '\n');
@@ -1690,54 +1681,6 @@ static int johansen_get_eigenvalues (gretl_matrix *S00,
     return err;
 }
 
-#define OLD_STYLE 0
-
-#if OLD_STYLE
-
-int johansen_coint_test (GRETL_VAR *jvar, const DATASET *dset, 
-			 gretlopt opt, PRN *prn)
-{
-    gretl_matrix *S00 = NULL;
-    gretl_matrix *evals = NULL;
-    int n = jvar->neqns;
-    int err = 0;
-
-    S00 = gretl_matrix_copy(jvar->jinfo->S00);
-    if (S00 == NULL) {
-	err = E_ALLOC;
-    }
-
-    if (!err) {
-	err = johansen_get_eigenvalues(S00, 
-				       jvar->jinfo->S01, 
-				       jvar->jinfo->S11,
-				       &jvar->jinfo->Beta,
-				       &evals, 0);
-    }
-
-    if (err) {
-	pputs(prn, _("Failed to find eigenvalues\n"));
-    } else {
-	johansen_ll_calc(jvar, evals);
-	compute_coint_test(jvar, evals, dset, prn);
-
-	if (!(opt & OPT_Q)) {
-	    err = compute_alpha(jvar->jinfo);
-	    if (!err) {
-		print_beta_and_alpha(jvar, evals, n, dset, prn);
-		print_long_run_matrix(jvar, dset, prn);
-	    }
-	}
-    }
-
-    gretl_matrix_free(S00);
-    gretl_matrix_free(evals);
-
-    return err;
-}
-
-#else
-
 /* Public entry point for cointegration test */
 
 int johansen_coint_test (GRETL_VAR *jvar, const DATASET *dset, 
@@ -1781,8 +1724,6 @@ int johansen_coint_test (GRETL_VAR *jvar, const DATASET *dset,
     return err;
 }
 
-#endif
-
 static void set_beta_test_df (GRETL_VAR *jvar, const gretl_matrix *H)
 {
     int r = jrank(jvar);
@@ -2599,7 +2540,8 @@ static int vecm_beta_test (GRETL_VAR *jvar,
 	goto bailout;
     }
 
-    pputs(prn, "\nTest of restrictions on cointegrating relations\n\n");
+    pprintf(prn, "\n%s\n\n",
+	    _("Test of restrictions on cointegrating relations"));
 
     if (verbose) {
 	gretl_matrix_print_to_prn(H, "Restriction matrix, H", prn);
@@ -2666,6 +2608,7 @@ int vecm_test_restriction (GRETL_VAR *jvar,
 {
     gretl_matrix *B0 = NULL;
     gretl_matrix *A0 = NULL;
+    PRN *vprn;
     int err = 0;
 
     B0 = gretl_matrix_copy(jvar->jinfo->Beta);
@@ -2675,12 +2618,14 @@ int vecm_test_restriction (GRETL_VAR *jvar,
 	return E_ALLOC;
     }
 
+    vprn = (opt & OPT_S)? NULL : prn;
+
     if (simple_beta_restriction(jvar, rset)) {
-	err = vecm_beta_test(jvar, rset, dset, opt, prn);
+	err = vecm_beta_test(jvar, rset, dset, opt, vprn);
     } else if (simple_alpha_restriction(jvar, rset)) {
-	err = vecm_alpha_test(jvar, rset, dset, opt, prn);
+	err = vecm_alpha_test(jvar, rset, dset, opt, vprn);
     } else {
-	err = general_vecm_analysis(jvar, rset, dset, prn);
+	err = general_vecm_analysis(jvar, rset, dset, vprn);
     }
 
     if (!err) {
