@@ -413,10 +413,9 @@ static int solve_for_beta (Jwrap *J,
 			   const gretl_matrix *R,
 			   const gretl_matrix *q)
 {
-    gretl_matrix *b;
+    gretl_matrix *b = gretl_matrix_copy(q);
     int err = 0;
 
-    b = gretl_matrix_copy(q);
     if (b == NULL) {
 	return E_ALLOC;
     }
@@ -730,29 +729,11 @@ static int update_phi (Jwrap *J, switcher *s)
 	gretl_matrix_copy_values(s->TmpL, s->K2);
     }
 
-#if 0 /* just testing */
-    gretl_matrix *Tmp1 = gretl_matrix_copy(J->I11);
-    gretl_matrix *Tmp2 = gretl_matrix_copy(s->TmpL);
-
-    err = gretl_cholesky_decomp_solve(J->I11, s->TmpL);
-    if (err) {
-	fprintf(stderr, "cholesky decomp failed in update_phi\n");
-	gretl_matrix_copy_values(J->I11, Tmp1);
-	gretl_matrix_copy_values(s->TmpL, Tmp2);
-	err = gretl_LU_solve(J->I11, s->TmpL);
-	if (err) {
-	    fprintf(stderr, " and LU_solve failed too\n");
-	}
-    } 
-    gretl_matrix_free(Tmp1);
-    gretl_matrix_free(Tmp2);
-#else
     /* combine first and second chunks */
     err = gretl_cholesky_decomp_solve(J->I11, s->TmpL);
     if (err) {
 	fprintf(stderr, "cholesky decomp failed in update_phi\n");
     }
-#endif
 
     if (!err) {
 	/* right-hand chunk */
@@ -2188,10 +2169,8 @@ static gretl_matrix *replicate_q (int r, const gretl_matrix *q0,
 
 static int set_up_H (Jwrap *J, const gretl_restriction *rset)
 {
-    const gretl_matrix *R0 = rset_get_R_matrix(rset);
-    const gretl_matrix *q0 = rset_get_q_matrix(rset);
-    const gretl_matrix *R = R0;
-    const gretl_matrix *q = q0;
+    const gretl_matrix *R = rset_get_R_matrix(rset);
+    const gretl_matrix *q = rset_get_q_matrix(rset);
     int qzero, err = 0;
 
 #if JDEBUG
@@ -2270,11 +2249,7 @@ static int phi_init_nonhomog (Jwrap *J)
 			      J->beta, GRETL_MOD_TRANSPOSE,
 			      BB, GRETL_MOD_NONE);
 
-    E = gretl_symmetric_matrix_eigenvals(BB, 1, &err);
-
-    if (!err) {
-	err = gretl_symmetric_eigen_sort(E, BB, 0);
-    } 
+    E = gretl_symm_matrix_eigenvals_descending(BB, 1, &err);
 
     if (!err) {
 	err = gretl_matrix_extract_matrix(BP, BB, 0, J->r, GRETL_MOD_NONE);
@@ -2892,126 +2867,6 @@ static int printres (Jwrap *J, GRETL_VAR *jvar,
     return 0;
 }
 
-static void set_up_matrix (gretl_matrix *m, double *val, 
-			   int rows, int cols)
-{
-    m->val = val;
-    m->rows = rows;
-    m->cols = cols;
-    m->t1 = m->t2 = 0;
-}
-
-/* Simulated annealing: can help when the standard deterministic
-   initialization (theta_0) leads to a local maximum.
-   We use the loglik-best point in case of improvement, else 
-   the last point visited.
-
-   This should be moved into gretl_bfgs.c (with some general-
-   ization possible) and exposed for users.
-*/
-
-int gretl_simann (double *theta, int n, int maxit,
-		  BFGS_CRIT_FUNC cfunc, int crittype,
-		  void *data, gretlopt opt, PRN *prn)
-{
-    gretl_matrix b;
-    gretl_matrix *b0 = NULL;
-    gretl_matrix *b1 = NULL;
-    gretl_matrix *bstar = NULL;
-    gretl_matrix *d = NULL;
-    double f0, f1;
-    double fbest, fworst;
-    double Temp = 1.0;
-    double radius = 1.0;
-    int improved = 0;
-    int i, err = 0;
-
-    set_up_matrix(&b, theta, n, 1);
-
-    b0 = gretl_matrix_copy(&b);
-    b1 = gretl_matrix_copy(&b);
-    bstar = gretl_matrix_copy(&b);
-    d = gretl_column_vector_alloc(n);
-
-    if (b0 == NULL || b1 == NULL || bstar == NULL || d == NULL) {
-	err = E_ALLOC;
-	goto bailout;
-    }
-
-    f0 = fbest = fworst = cfunc(b.val, data);
-
-    if (opt & OPT_V) {
-	pprintf(prn, "\nSimulated annealing: initial function value = %.8g\n",
-		f0);
-    }
-
-    /* Question: should the initial radius be a function of the scale
-       of the initial parameter vector?
-    */
-
-    for (i=0; i<maxit; i++) {
-	gretl_matrix_random_fill(d, D_NORMAL);
-	gretl_matrix_multiply_by_scalar(d, radius);
-	gretl_matrix_add_to(b1, d);
-	f1 = cfunc(b1->val, data);
-
-	if (f1 > f0 || gretl_rand_01() < Temp) {
-	    /* jump to the new point */
-	    f0 = f1;
-	    gretl_matrix_copy_values(b0, b1);
-	    if (f0 > fbest) {
-		fbest = f0;
-		gretl_matrix_copy_values(bstar, b0);
-		if (opt & OPT_V) {
-		    if (!improved) {
-			pprintf(prn, "\n%6s %12s %12s %12s\n",
-				"iter", "temp", "radius", "fbest");
-		    }
-		    pprintf(prn, "%6d %#12.6g %#12.6g %#12.6g\n", 
-			    i, Temp, radius, fbest);
-		}
-		improved = 1;
-	    } else if (f0 < fworst) {
-		fworst = f0;
-	    }
-	} else {
-	    /* revert to where we were */
-	    gretl_matrix_copy_values(b1, b0);
-	    f1 = f0;
-	}
-
-	Temp *= 0.999;
-	radius *= 0.9999;
-    }
-
-    if (improved) {
-	/* use the best point */
-	gretl_matrix_copy_values(&b, bstar);
-    } else {
-	/* use the last point */
-	gretl_matrix_copy_values(&b, b0);
-    }
-
-    if (improved) {
-	if (opt & OPT_V) pputc(prn, '\n');
-    } else {
-	pprintf(prn, "No improvement found in %d iterations\n\n", maxit);
-    }
-    
-    if (fbest - fworst < 1.0e-9) {
-	pprintf(prn, "*** warning: surface seems to be flat\n");
-    }
-
- bailout:
-
-    gretl_matrix_free(b0);
-    gretl_matrix_free(b1);
-    gretl_matrix_free(bstar);
-    gretl_matrix_free(d);
-
-    return err;
-}
-
 /* solve for unrestricted alpha conditional on beta */
 
 static int J_compute_alpha (Jwrap *J)
