@@ -22,6 +22,7 @@
 #define FULL_XML_HEADERS
 
 #include "libgretl.h"
+#include "version.h"
 #include "gretl_xml.h"
 #include "csvdata.h"
 #include "importer.h"
@@ -457,13 +458,9 @@ static int real_read_cell (xmlNodePtr cur,
 	    val = get_ods_string_value(cur);
 	    if (val != NULL) {
 		*sheet->dset->varname[v] = '\0';
-		strncat(sheet->dset->varname[v],
-			val, VNAMELEN - 1);
-		fprintf(stderr, " varname: '%s'\n", val);
-		if (check_varname(sheet->dset->varname[v])) {
-		    invalid_varname(prn);
-		    err = 1;
-		}
+		strncat(sheet->dset->varname[v], val, VNAMELEN - 1);
+		err = check_imported_varname(sheet->dset->varname[v],
+					     iread, jread, prn);
 		free(val);
 	    } else {
 		err = ods_error(sheet, iread, jread, ODS_STRING, 
@@ -509,7 +506,7 @@ static int real_read_cell (xmlNodePtr cur,
 	}
 
 	if (!err) {
-	    strncat(sheet->dset->S[t], val, OBSLEN - 1);
+	    gretl_utf8_strncat(sheet->dset->S[t], val, OBSLEN - 1);
 	}
 
 	free(val);
@@ -1095,11 +1092,13 @@ static int ods_sheet_dialog (ods_sheet *sheet, int *err)
 }
 
 static int finalize_ods_import (DATASET *dset,
-				ods_sheet *sheet, 
+				ods_sheet *sheet,
+				const char *fname, 
 				gretlopt opt,
 				PRN *prn)
 {
     int err = import_prune_columns(sheet->dset);
+    int merge = (dset->Z != NULL);
 
     if (!err && sheet->dset->S != NULL) {
 	import_ts_check(sheet->dset);
@@ -1109,6 +1108,10 @@ static int finalize_ods_import (DATASET *dset,
 	err = merge_or_replace_data(dset, &sheet->dset, opt, prn);
     }  
 
+    if (!err && !merge) {
+	dataset_add_import_info(dset, fname, GRETL_ODS);
+    }    
+
     return err;
 }
 
@@ -1163,7 +1166,7 @@ int ods_get_data (const char *fname, int *list, char *sheetname,
     }
 
     if (!err) {
-	err = finalize_ods_import(dset, sheet, opt, prn); 
+	err = finalize_ods_import(dset, sheet, fname, opt, prn); 
 	if (!err && gui) {
 	    record_ods_params(sheet, list);
 	}
