@@ -20,6 +20,7 @@
 /* models for count data, Poisson and Negative Binomial */
 
 #include "libgretl.h"
+#include "version.h"
 #include "matrix_extra.h"
 #include "libset.h"
 #include "gretl_bfgs.h"
@@ -54,7 +55,6 @@ struct negbin_info_ {
     gretl_matrix *mu;      /* exp(X\beta) */
     gretl_matrix *llt;     /* per-observation likelihood */
     gretl_matrix *G;       /* score matrix */
-    gretl_matrix *V;       /* covariance matrix */
     gretl_vector *offset;  /* offset/exposure vector */
     PRN *prn;              /* verbose printer */
 };
@@ -69,7 +69,6 @@ static void negbin_free (negbin_info *nbinfo)
 {
     gretl_matrix_block_destroy(nbinfo->B);
     free(nbinfo->theta);
-    gretl_matrix_free(nbinfo->V);
     gretl_matrix_free(nbinfo->offset);
 }
 
@@ -87,13 +86,11 @@ static int negbin_init (negbin_info *nbinfo, MODEL *pmod,
     nbinfo->flags = 0;
 
     nbinfo->B = NULL;
-    nbinfo->V = NULL;
     nbinfo->offset = NULL;
 
     nbinfo->theta = malloc(np * sizeof *nbinfo->theta);
-    nbinfo->V = gretl_matrix_alloc(np, np);
     
-    if (nbinfo->theta == NULL || nbinfo->V == NULL) {
+    if (nbinfo->theta == NULL) {
 	return E_ALLOC;
     }
 
@@ -301,7 +298,8 @@ static gretl_matrix *negbin_init_H (negbin_info *nbinfo)
     return H;
 }
 
-static gretl_matrix *negbin_nhessian (negbin_info *nbinfo, int *err)
+static gretl_matrix *negbin_hessian_inverse (negbin_info *nbinfo, 
+					     int *err)
 {
     gretl_matrix *H;
     int np = nbinfo->k + 1;
@@ -315,58 +313,28 @@ static gretl_matrix *negbin_nhessian (negbin_info *nbinfo, int *err)
     return H;
 }
 
-/* OPG vcv matrix */
-
-static int negbin_OPG_vcv (MODEL *pmod, negbin_info *nbinfo)
-{
-    gretl_matrix_multiply_mod(nbinfo->G, GRETL_MOD_TRANSPOSE,
-			      nbinfo->G, GRETL_MOD_NONE,
-			      nbinfo->V, GRETL_MOD_NONE);
-
-    return gretl_invert_symmetric_matrix(nbinfo->V);
-}
-
-/* QML sandwich VCV */
-
-static int negbin_robust_vcv (MODEL *pmod, negbin_info *nbinfo)
+static int negbin_model_add_vcv (MODEL *pmod, negbin_info *nbinfo,
+				 const DATASET *dset, gretlopt opt)
 {
     gretl_matrix *H = NULL;
-    gretl_matrix *GG = NULL;
-    int np = nbinfo->k + 1;
     int err = 0;
 
-    GG = gretl_matrix_alloc(np, np);
-    if (GG == NULL) {
-	return E_ALLOC;
+    if (opt & OPT_G) {
+	err = gretl_model_add_OPG_vcv(pmod, nbinfo->G);
+    } else {
+	H = negbin_hessian_inverse(nbinfo, &err);
+	if (!err) {
+	    if (opt & OPT_R) {
+		err = gretl_model_add_QML_vcv(pmod, pmod->ci, 
+					      H, nbinfo->G,
+					      dset, opt);
+	    } else {
+		err = gretl_model_add_hessian_vcv(pmod, H);
+	    }
+	}
     }
 
-    gretl_matrix_multiply_mod(nbinfo->G, GRETL_MOD_TRANSPOSE,
-			      nbinfo->G, GRETL_MOD_NONE,
-			      GG, GRETL_MOD_NONE);
-
-    H = negbin_nhessian(nbinfo, &err);
-
-    if (!err) {
-	err = gretl_matrix_qform(H, GRETL_MOD_NONE,
-				 GG, nbinfo->V, GRETL_MOD_NONE);
-    }	
-
     gretl_matrix_free(H);
-    gretl_matrix_free(GG);
-
-    return err;
-}
-
-static int negbin_hessian_vcv (MODEL *pmod, negbin_info *nbinfo)
-{
-    gretl_matrix *H;
-    int err = 0;
-
-    H = negbin_nhessian(nbinfo, &err);
-
-    if (!err) {
-	gretl_matrix_replace(&nbinfo->V, H);
-    }
 
     return err;
 }
@@ -421,23 +389,7 @@ transcribe_negbin_results (MODEL *pmod, negbin_info *nbinfo,
     err = gretl_model_write_coeffs(pmod, nbinfo->theta, nc);
     
     if (!err) {
-	if (opt & OPT_R) {
-	    pmod->opt |= OPT_R;
-	    err = negbin_robust_vcv(pmod, nbinfo);
-	} else if (opt & OPT_G) {
-	    pmod->opt |= OPT_G;
-	    err = negbin_OPG_vcv(pmod, nbinfo);
-	} else {
-	    err = negbin_hessian_vcv(pmod, nbinfo);
-	} 
-
-	if (!err) {
-	    int vtype = (opt & OPT_G) ? VCV_OP :
-		(opt & OPT_R)? VCV_QML : VCV_HESSIAN;
-
-	    gretl_model_set_vcv_info(pmod, VCV_ML, vtype);
-	    err = gretl_model_write_vcv(pmod, nbinfo->V, -1);
-	}
+	err = negbin_model_add_vcv(pmod, nbinfo, dset, opt);
     }
 
     if (!err) {
@@ -517,46 +469,23 @@ static int do_negbin (MODEL *pmod, offset_info *oinfo,
     return pmod->errcode;
 }
 
-/* sandwich of hessian and OPG */
+/* sandwich of hessian and OPG, or clustered */
 
-static int poisson_robust_vcv (MODEL *pmod, gretl_matrix *G)
+static int poisson_robust_vcv (MODEL *pmod, 
+			       const gretl_matrix *G,
+			       const DATASET *dset, 
+			       gretlopt opt)
 {
-    gretl_matrix *H = NULL;
-    gretl_matrix *GG = NULL;
-    gretl_matrix *V = NULL;
-    int k = G->cols;
+    gretl_matrix *H;
     int err = 0;
 
     H = gretl_vcv_matrix_from_model(pmod, NULL, &err);
 
     if (!err) {
-	GG = gretl_matrix_alloc(k, k);
-	V = gretl_matrix_alloc(k, k);
-	if (GG == NULL || V == NULL) {
-	    err = E_ALLOC;
-	}
+	err = gretl_model_add_QML_vcv(pmod, POISSON, 
+				      H, G, dset, opt);
     }
 
-    if (!err) {
-	gretl_matrix_multiply_mod(G, GRETL_MOD_TRANSPOSE,
-				  G, GRETL_MOD_NONE,
-				  GG, GRETL_MOD_NONE);    
-	/* form sandwich: V = H^{-1} GG' H^{-1} */
-	err = gretl_matrix_qform(H, GRETL_MOD_NONE,
-				 GG, V, GRETL_MOD_NONE);
-    }
-
-    if (!err) {
-	err = gretl_model_write_vcv(pmod, V, -1);
-    }
-
-    if (!err) {
-	gretl_model_set_vcv_info(pmod, VCV_ML, VCV_QML);
-	pmod->opt |= OPT_R;
-    }
-
-    gretl_matrix_free(V);
-    gretl_matrix_free(GG);
     gretl_matrix_free(H);
 
     return err;
@@ -611,7 +540,7 @@ static int overdispersion_test (MODEL *pmod, const DATASET *dset,
 
     if (opt & OPT_R) {
 	gretl_matrix_reuse(G, n, k);
-	err = poisson_robust_vcv(pmod, G);
+	err = poisson_robust_vcv(pmod, G, dset, opt);
 	gretl_matrix_reuse(G, n, k+1);
     }
 
@@ -865,7 +794,12 @@ static int do_poisson (MODEL *pmod, offset_info *oinfo,
 
 	crit = tmpmod.nobs * tmpmod.rsq;
 
-	pprintf(prn, "%s %3d\tcrit = %g\n", _("iteration"), iter, crit);
+	if (prn != NULL) {
+	    double ll = poisson_ll(y, pmod->yhat, pmod->t1, pmod->t2);
+
+	    print_iter_info(iter, ll, C_LOGLIK, 0, 
+			    NULL, NULL, NADBL, prn);
+	}
 
 	for (i=0; i<tmpmod.ncoeff; i++) { 
 	    pmod->coeff[i] += tmpmod.coeff[i];
@@ -889,8 +823,6 @@ static int do_poisson (MODEL *pmod, offset_info *oinfo,
 	}
     }
 
-    pputc(prn, '\n');
-
     if (crit > POISSON_TOL) {
 	pmod->errcode = E_NOCONV;
     } 
@@ -967,6 +899,11 @@ int count_data_estimate (MODEL *pmod, int ci, int offvar,
 	}
     } 
 
+    if (opt & OPT_C) {
+	/* cluster implies robust */
+	opt |= OPT_R;
+    }
+
     if (opt & OPT_V) {
 	vprn = prn;
     }
