@@ -23,6 +23,7 @@
 */
 
 #include "libgretl.h"
+#include "version.h"
 #include "gretl_f2c.h"
 #include "usermat.h"
 #include "matrix_extra.h"
@@ -808,7 +809,7 @@ static int rq_write_variance (const gretl_matrix *V,
 	    }
 	}
     } else {
-	err = gretl_model_write_vcv(pmod, V, -1);
+	err = gretl_model_write_vcv(pmod, V);
     }
 
     return err;
