@@ -4,7 +4,7 @@
    Based on stataread.c from the GNU R "foreign" package with the 
    following original info:
 
-     * $Id: stata_import.c,v 1.46 2011/07/11 15:35:10 allin Exp $
+     * $Id: stata_import.c,v 1.48 2012/01/06 14:52:20 allin Exp $
   
      (c) 1999, 2000, 2001, 2002 Thomas Lumley. 
      2000 Saikat DebRoy
@@ -33,6 +33,7 @@
 #include <glib.h>
 
 #include "libgretl.h"
+#include "version.h"
 #include "gretl_string_table.h"
 #include "swap_bytes.h"
 
@@ -653,7 +654,7 @@ static int read_dta_data (FILE *fp, DATASET *dset,
 		gchar *tr = recode_stata_string(label);
 
 		if (tr != NULL) {
-		    strncat(VARLABEL(dset, i+1), tr, MAXLABEL - 1);
+		    gretl_utf8_strncat(VARLABEL(dset, i+1), tr, MAXLABEL - 1);
 		    g_free(tr);
 		}
 	    } 
