@@ -18,37 +18,51 @@
  */
 
 #include "libgretl.h"
+#include "version.h"
 #include "gretl_matrix.h"
+#include "libset.h"
 #include "system.h"
 #include "tsls.h"
 #include "sysml.h"
 
 #define SDEBUG 0
-#define SYS_USE_SVD 0
 
 #define sys_ols_ok(s) (s->method == SYS_METHOD_SUR || \
                        s->method == SYS_METHOD_OLS || \
                        s->method == SYS_METHOD_WLS)
 
-/* insert the elements of sub-matrix @M, multiplied by @scale, in the
-   appropriate position within the big matrix @X */
+/* Insert the elements of sub-matrix @M, multiplied by @scale, in the
+   appropriate position within the big matrix @X. We're building
+   a symmetric matrix here so we insert off-diagonal elements both
+   above and below the diagonal.
+*/
 
 static void 
-kronecker_place (gretl_matrix *X, const gretl_matrix *M,
-		 int startrow, int startcol, double scale)
+insert_sys_X_block (gretl_matrix *X, const gretl_matrix *M,
+		    int startrow, int startcol, double scale)
 {
-    int i, j;
-    int row, col;
-    double x;
+    int r, c, i, j;
+    double mij;
     
     for (i=0; i<M->rows; i++) {
-	row = startrow + i;
+	r = startrow + i;
 	for (j=0; j<M->cols; j++) {
-	    col = startcol + j;
-	    x = gretl_matrix_get(M, i, j);
-	    gretl_matrix_set(X, row, col, x * scale);
+	    c = startcol + j;
+	    mij = gretl_matrix_get(M, i, j);
+	    gretl_matrix_set(X, r, c, mij * scale);
 	}
     }
+    
+    if (startrow != startcol) {
+	for (i=0; i<M->rows; i++) {
+	    c = startrow + i;
+	    for (j=0; j<M->cols; j++) {
+		r = startcol + j;
+		mij = gretl_matrix_get(M, i, j);
+		gretl_matrix_set(X, r, c, mij * scale);
+	    }
+	}
+    }	
 }
 
 /* Retrieve the data placed on @pmod in the course of 2sls estimation:
@@ -61,25 +75,24 @@ kronecker_place (gretl_matrix *X, const gretl_matrix *M,
 
 double *model_get_Xi (const MODEL *pmod, DATASET *dset, int i)
 {
-    const char *endog = gretl_model_get_data(pmod, "endog");
-    double **X;
+    gretl_matrix *endog = gretl_model_get_data(pmod, "endog");
     double *xi = NULL;
 
-    if (endog == NULL || !endog[i]) {
-	/* return the original data */
-	return dset->Z[pmod->list[i+2]];
-    }
+    if (endog == NULL || endog->val[i] == 0) {
+	/* use the original data */
+	xi = dset->Z[pmod->list[i+2]];
+    } else {
+	double **X = gretl_model_get_data(pmod, "tslsX");
 
-    X = gretl_model_get_data(pmod, "tslsX");
+	if (X != NULL) {
+	    /* find and return the correct column of X */
+	    int j, k = 0;
 
-    if (X != NULL) {
-	/* find and return the correct column */
-	int j, k = 0;
-
-	for (j=0; j<i; j++) {
-	    if (endog[j]) k++;
+	    for (j=0; j<i; j++) {
+		if (endog->val[j] != 0) k++;
+	    }
+	    xi = X[k];
 	}
-	xi = X[k];
     }
 
     return xi;
@@ -93,38 +106,40 @@ double *model_get_Xi (const MODEL *pmod, DATASET *dset, int i)
 static int make_liml_X_block (gretl_matrix *X, const MODEL *pmod,
 			      DATASET *dset, int t1)
 {
-    int i, t;
     const double *Xi;
+    int i, t, err = 0;
 
     X->cols = pmod->ncoeff;
 
-    for (i=0; i<X->cols; i++) {
+    for (i=0; i<X->cols && !err; i++) {
 	Xi = model_get_Xi(pmod, dset, i);
 	if (Xi == NULL) {
-	    return 1;
-	}
-	for (t=0; t<X->rows; t++) {
-	    gretl_matrix_set(X, t, i, Xi[t+t1]);
+	    err = E_DATA;
+	} else {
+	    for (t=0; t<X->rows; t++) {
+		gretl_matrix_set(X, t, i, Xi[t+t1]);
+	    }
 	}
     }
 
-    return 0;
+    return err;
 }
 
 /* construct the X data block pertaining to a specific equation, using
    either the original data or fitted values from regression on a set
-   of instruments */
+   of instruments 
+*/
 
 static int 
 make_sys_X_block (gretl_matrix *X, const MODEL *pmod,
 		  DATASET *dset, int t1, int method)
 {
-    int i, t;
     const double *Xi;
+    int i, t, err = 0;
 
     X->cols = pmod->ncoeff;
 
-    for (i=0; i<X->cols; i++) {
+    for (i=0; i<X->cols && !err; i++) {
 	if (method == SYS_METHOD_3SLS || 
 	    method == SYS_METHOD_FIML || 
 	    method == SYS_METHOD_TSLS) {
@@ -133,71 +148,72 @@ make_sys_X_block (gretl_matrix *X, const MODEL *pmod,
 	    Xi = dset->Z[pmod->list[i+2]];
 	}
 	if (Xi == NULL) {
-	    return E_DATA;
-	}
-	for (t=0; t<X->rows; t++) {
-	    gretl_matrix_set(X, t, i, Xi[t+t1]);
+	    err = E_DATA;
+	} else {
+	    for (t=0; t<X->rows; t++) {
+		gretl_matrix_set(X, t, i, Xi[t+t1]);
+	    }
 	}
     }
 
-    return 0;
+    return err;
 }
 
-/* populate the cross-equation covariance matrix based on the
-   per-equation residuals */
+/* populate the cross-equation covariance matrix, @S, based on
+   the per-equation residuals: if @do_diag is non-zero we also
+   want to compute the Breusch-Pagan test for diagonality
+   of the matrix
+*/
 
 static int
-gls_sigma_from_uhat (equation_system *sys, gretl_matrix *sigma)
+gls_sigma_from_uhat (equation_system *sys, gretl_matrix *S,
+		     int do_diag)
 {
-    const gretl_matrix *e = sys->E;
-    int m = sys->neqns;
-    int T = sys->T;
     int geomean = system_vcv_geomean(sys);
+    double eti, etj, sij;
+    int m = sys->neqns;
     int i, j, t;
-    double xx;
 
     for (i=0; i<m; i++) {
 	for (j=i; j<m; j++) {
-	    xx = 0.0;
-	    for (t=0; t<T; t++) {
-		xx += gretl_matrix_get(e, t, i) * gretl_matrix_get(e, t, j);
+	    sij = 0.0;
+	    for (t=0; t<sys->T; t++) {
+		eti = gretl_matrix_get(sys->E, t, i);
+		etj = gretl_matrix_get(sys->E, t, j);
+		sij += eti * etj;
 	    }
 	    if (geomean) {
-		xx /= system_vcv_denom(sys, i, j);
+		sij /= system_vcv_denom(sys, i, j);
 	    } else {
-		xx /= T;
+		sij /= sys->T;
 	    }
-	    gretl_matrix_set(sigma, i, j, xx);
+	    gretl_matrix_set(S, i, j, sij);
 	    if (j != i) {
-		gretl_matrix_set(sigma, j, i, xx);
+		gretl_matrix_set(S, j, i, sij);
 	    }
 	}
     }
 
-    if (sys->method == SYS_METHOD_OLS && sys->diag == 0.0) {
-	double sii, sij, sjj;
+    if (do_diag) {
+	/* B-P test statistic */
+	double sii, sjj;
+
+	sys->diag = 0.0;
 
 	for (i=1; i<m; i++) {
-	    sii = gretl_matrix_get(sigma, i, i);
+	    sii = gretl_matrix_get(S, i, i);
 	    for (j=0; j<i; j++) {
-		sij = gretl_matrix_get(sigma, i, j);
-		sjj = gretl_matrix_get(sigma, j, j);
+		sij = gretl_matrix_get(S, i, j);
+		sjj = gretl_matrix_get(S, j, j);
 		sys->diag += (sij * sij) / (sii * sjj);
 	    }
 	}
-	sys->diag *= T;
+	sys->diag *= sys->T;
     }
 
     return 0;
 }
 
-/* Note: this was changed in Revision 1.86 of Fri Apr 30 2010. Up till
-   then we were reporting the regular R^2 for models estimated as part
-   of a system, which seems wrong.
-*/
-
-#define SYS_CORR_RSQ 1
-
 /* compute residuals, for all cases other than FIML */
 
 static void 
@@ -230,15 +246,11 @@ sys_resids (equation_system *sys, int eq, const DATASET *dset)
     }
 
     if (pmod->ifc && pmod->tss > 0) {
-	/* R-squared */
+	/* R-squared based on actual-fitted correlation */
 	double r;
 
-#if SYS_CORR_RSQ
 	pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, dset->Z[yno], 
 				   pmod->yhat);
-#else
-	pmod->rsq = 1 - (pmod->ess / pmod->tss);
-#endif
 	r = 1 - pmod->rsq;
 	pmod->adjrsq = 1.0 - (r * (pmod->nobs - 1) / pmod->dfd);
     } else {
@@ -246,147 +258,138 @@ sys_resids (equation_system *sys, int eq, const DATASET *dset)
     }
 }
 
-static void
-liml_scale_vcv (equation_system *sys, gretl_matrix *vcv)
-{
-    double s2, vij;
-    int vmin = 0;
-    int vi, vj;
-    int i, j, k;
-
-    for (i=0; i<sys->neqns; i++) {
-	s2 = sys->models[i]->sigma * sys->models[i]->sigma;
-	for (j=0; j<sys->models[i]->ncoeff; j++) {
-	    for (k=j; k<sys->models[i]->ncoeff; k++) {
-		vi = j + vmin;
-		vj = k + vmin;
-		vij = gretl_matrix_get(vcv, vi, vj);
-		vij *= s2;
-		gretl_matrix_set(vcv, vi, vj, vij);
-		gretl_matrix_set(vcv, vj, vi, vij);
-	    }
-	}
-	vmin += sys->models[i]->ncoeff;
-    }		
-}
-
 /* use the per-equation residual standard deviations for scaling
    the covariance matrix, block by diagonal block
 */
 
-static void 
-single_eq_scale_vcv (equation_system *sys, gretl_matrix *vcv)
+static void
+single_eqn_scale_vcv (MODEL *pmod, gretl_matrix *V, int offset)
 {
-    MODEL *pmod;
-    double s2, vij;
-    int ioff = 0, joff = 0;
-    int i, k, ii, jj;
+    double vij, s2 = pmod->sigma * pmod->sigma;
+    int i, j, ii, jj;
 
-    for (i=0; i<sys->neqns; i++) {
-	pmod = sys->models[i];
-	s2 = pmod->sigma * pmod->sigma;
-	k = pmod->ncoeff;
-	for (ii=0; ii<k; ii++) {
-	    for (jj=0; jj<k; jj++) {
-		vij = gretl_matrix_get(vcv, ioff + ii, joff + jj);
-		gretl_matrix_set(vcv, ioff + ii, joff + jj, vij * s2);
-	    }
+    for (i=0; i<pmod->ncoeff; i++) {
+	ii = i + offset;
+	for (j=i; j<pmod->ncoeff; j++) {
+	    jj = j + offset;
+	    vij = gretl_matrix_get(V, ii, jj);
+	    vij *= s2;
+	    gretl_matrix_set(V, ii, jj, vij);
+	    gretl_matrix_set(V, jj, ii, vij);
 	}
-	ioff += k;
-	joff += k;
-    }    
+    }
 }
 
-/* compute SUR, 3SLS or LIML parameter estimates (or restricted OLS,
-   TSLS, WLS) */
+/* write results from system estimation into the individual
+   model structs 
+*/
 
-static int 
-calculate_sys_coeffs (equation_system *sys,
-		      const DATASET *dset,
-		      gretl_matrix *X, gretl_matrix *y, 
-		      int mk, int nr, int do_iteration)
+static void transcribe_sys_results (equation_system *sys,
+				    const DATASET *dset,
+				    int do_iters)
 {
-    int do_bdiff = ((sys->method == SYS_METHOD_3SLS) && do_iteration);
-    double bij, oldb, bnum = 0.0, bden = 0.0;
-    gretl_matrix *vcv = NULL;
-    gretl_matrix *b = NULL;
-    int i, j, k, j0;
-    int err = 0;
+    int do_bdiff = (sys->method == SYS_METHOD_3SLS && do_iters);
+    double bij, oldb, bnum = 0, bden = 0;
+    int offset = 0;
+    int i, j, k;
 
-    vcv = gretl_matrix_copy(X);
-    if (vcv == NULL) {
-	return E_ALLOC;
-    }
-
-    err = gretl_LU_solve(X, y);
-    if (err) {
-	return err;
-    }
-
-#if SDEBUG > 1
-    gretl_matrix_print(y, "in calc_coeffs, betahat");
-#endif
-
-#if SYS_USE_SVD
-    /* memory-intensive for big matrices */
-    err = gretl_SVD_invert_matrix(vcv);
-#else    
-    err = gretl_invert_general_matrix(vcv);
-#endif
-
-#if SDEBUG
-    fprintf(stderr, "calculate_sys_coeffs: invert, err=%d\n", err);
-#endif
-
-    if (err) {
-	return err;
-    }
-
-#if SDEBUG > 1
-    gretl_matrix_print(vcv, "in calc_coeffs, vcv");
-#endif
-
-    j0 = 0;
     for (i=0; i<sys->neqns; i++) {
-	for (j=0; j<sys->models[i]->ncoeff; j++) {
-	    k = j0 + j;
-	    bij = gretl_vector_get(y, k);
+	MODEL *pmod = sys->models[i];
+
+	/* update the coefficients */
+	for (j=0; j<pmod->ncoeff; j++) {
+	    k = j + offset;
+	    bij = gretl_vector_get(sys->b, k);
 	    if (do_bdiff) {
-		oldb = sys->models[i]->coeff[j];
+		oldb = pmod->coeff[j];
 		bnum += (bij - oldb) * (bij - oldb);
 		bden += oldb * oldb;
 	    }
-	    sys->models[i]->coeff[j] = bij;
+	    pmod->coeff[j] = bij;
 	}
+	/* update residuals (and sigma-hat) */
 	sys_resids(sys, i, dset);
-	j0 += sys->models[i]->ncoeff;
+	/* for single-equation methods, vcv needs scaling by
+	   an estimate of the residual variance
+	*/
+	if (sys->method == SYS_METHOD_OLS || 
+	    sys->method == SYS_METHOD_TSLS ||
+	    sys->method == SYS_METHOD_LIML) {
+	    single_eqn_scale_vcv(pmod, sys->vcv, offset);
+	}
+	/* update standard errors */
+	for (j=0; j<pmod->ncoeff; j++) {
+	    k = j + offset;
+	    pmod->sderr[j] = sqrt(gretl_matrix_get(sys->vcv, k, k));
+	}
+	offset += pmod->ncoeff;
     }
 
     if (do_bdiff) {
 	sys->bdiff = sqrt(bnum / bden);
     }
+}
 
-    if (sys->method == SYS_METHOD_OLS || 
-	sys->method == SYS_METHOD_TSLS) {
-	single_eq_scale_vcv(sys, vcv);
-    } else if (sys->method == SYS_METHOD_LIML) {
-	liml_scale_vcv(sys, vcv);
+/* compute SUR, 3SLS or LIML parameter estimates (or restricted OLS,
+   TSLS, WLS) 
+*/
+
+static int 
+calculate_sys_coeffs (equation_system *sys, const DATASET *dset,
+		      gretl_matrix *X, gretl_matrix *y, 
+		      int mk, int nr, int do_iters)
+{
+    gretl_matrix *V;
+    int posdef = 0;
+    int err = 0;
+
+    V = gretl_matrix_copy(X);
+    if (V == NULL) {
+	return E_ALLOC;
     }
 
-    /* now set the model standard errors */
-    j0 = 0;
-    for (i=0; i<sys->neqns; i++) {
-	for (j=0; j<sys->models[i]->ncoeff; j++) {
-	    k = j0 + j;
-	    sys->models[i]->sderr[j] = sqrt(gretl_matrix_get(vcv, k, k));
+#if SDEBUG
+    gretl_matrix_print(X, "sys X");
+    gretl_matrix_print(y, "sys y");
+#endif  
+
+    /* calculate the coefficients */
+
+    if (sys->R == NULL) {
+	/* hopefully X may be positive definite */
+	err = gretl_cholesky_decomp_solve(X, y);
+	if (err) {
+	    /* nope; we'll fall back to the LU solver */
+	    gretl_matrix_copy_values(X, V);
+	    err = 0;
+	} else {
+	    posdef = 1;
+	    err = gretl_cholesky_invert(X);
 	}
-	j0 += sys->models[i]->ncoeff;
     }
 
-    /* save the coefficient vector and covariance matrix */
-    b = gretl_matrix_copy(y);
-    system_attach_coeffs(sys, b);
-    system_attach_vcv(sys, vcv);
+    if (!posdef) {
+	err = gretl_LU_solve_invert(X, y);
+    }
+
+    if (!err) {
+	/* save the coefficient vector and covariance matrix */
+	gretl_matrix *b = gretl_matrix_copy(y);
+
+	if (b == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    system_attach_coeffs(sys, b);
+	    gretl_matrix_copy_values(V, X);
+	    system_attach_vcv(sys, V);
+	    /* transcribe stuff to the included models */
+	    transcribe_sys_results(sys, dset, do_iters);
+	}
+    }
+
+    if (err) {
+	gretl_matrix_free(V);
+    }
 
     return err;
 }
@@ -445,25 +448,25 @@ static int hansen_sargan_test (equation_system *sys,
     int m = sys->neqns;
     int T = sys->T;
     int df = system_get_overid_df(sys);
-    gretl_matrix *WTW = NULL;
-    gretl_matrix *eW = NULL;
-    gretl_matrix *tmp = NULL;
+    gretl_matrix_block *B;
+    gretl_matrix *WTW, *eW, *tmp;
     double x, X2;
     int i, j, t;
     int err = 0;
 
-    if (df <= 0) return 1;
-
-    WTW = gretl_matrix_alloc(nx, nx);
-    eW = gretl_matrix_alloc(m, nx);
-    tmp = gretl_matrix_alloc(m, nx);
-
-    if (WTW == NULL || eW == NULL || tmp == NULL) {
-	err = E_ALLOC;
-	goto bailout;
+    if (df <= 0) {
+	return 1;
     }
 
-    /* construct W-transpose W */
+    B = gretl_matrix_block_new(&WTW, nx, nx,
+			       &eW, m, nx,
+			       &tmp, m, nx,
+			       NULL);
+    if (B == NULL) {
+	return E_ALLOC;
+    }
+
+    /* construct W-transpose W in WTW */
     for (i=0; i<nx; i++) {
 	Wi = dset->Z[exlist[i+1]] + sys->t1;
 	for (j=i; j<nx; j++) {
@@ -488,8 +491,8 @@ static int hansen_sargan_test (equation_system *sys,
 	goto bailout;
     }
 
-    /* set up vectors of SUR or 3SLS residuals, transposed, times W:
-       these are stacked in an m * nx matrix
+    /* set up vectors of SUR or 3SLS residuals, transposed, 
+       times W; these are stacked in the m * nx matrix eW
     */
     for (i=0; i<m; i++) {
 	for (j=0; j<nx; j++) {
@@ -535,9 +538,7 @@ static int hansen_sargan_test (equation_system *sys,
 
  bailout:
 
-    gretl_matrix_free(WTW);
-    gretl_matrix_free(eW);
-    gretl_matrix_free(tmp);
+    gretl_matrix_block_destroy(B);
 
     return err;
 }
@@ -582,7 +583,7 @@ static int basic_system_allocate (equation_system *sys,
 
 /* compute log-likelihood for iterated SUR estimator */
 
-double sur_ll (equation_system *sys)
+double sur_loglik (equation_system *sys)
 {
     int m = sys->neqns;
     int T = sys->T;
@@ -594,7 +595,7 @@ double sur_ll (equation_system *sys)
 	return NADBL;
     }
 
-    gls_sigma_from_uhat(sys, tmp);
+    gls_sigma_from_uhat(sys, tmp, 0);
     ldet = gretl_vcv_log_determinant(tmp);
 
     if (na(ldet)) {
@@ -612,29 +613,14 @@ double sur_ll (equation_system *sys)
 /* if we're estimating with a specified set of linear restrictions,
    Rb = q, augment the X matrix with R and R-transpose */
 
-static int 
+static void
 augment_X_with_restrictions (gretl_matrix *X, int mk, 
 			     equation_system *sys)
 {
-    double rij;
-    int nr, nc;
-    int i, j;
-
-    if (sys->R == NULL) return 1;
-
-    nr = sys->R->rows;
-    nc = sys->R->cols;
+    int i, j, nr = sys->R->rows;
 
     /* place the R matrix */
-    kronecker_place(X, sys->R, mk, 0, 1.0);
-
-    /* place R-transpose */
-    for (i=0; i<nr; i++) {
-	for (j=0; j<nc; j++) {
-	    rij = gretl_matrix_get(sys->R, i, j);
-	    gretl_matrix_set(X, j, i + mk, rij);
-	}
-    }
+    insert_sys_X_block(X, sys->R, mk, 0, 1.0);
 
     /* zero the bottom right-hand block */
     for (i=mk; i<mk+nr; i++) {
@@ -642,8 +628,6 @@ augment_X_with_restrictions (gretl_matrix *X, int mk,
 	    gretl_matrix_set(X, i, j, 0.0);
 	}
     }
-
-    return 0;
 }
 
 /* when estimating with a specified set of linear restrictions,
@@ -673,19 +657,18 @@ augment_y_with_restrictions (gretl_matrix *y, int mk, int nr,
 /* check for convergence of iteration: we use the change in the
    log-likelihood when iterating SUR to the ML solution, or
    a measure of the change in the coefficients when iterating
-   three-stage least squares
+   3SLS
 */
 
-static int sys_converged (equation_system *sys, 
-			  double *llbak, int *err,
-			  gretlopt opt, PRN *prn)
+static int sys_converged (equation_system *sys, double *llbak,
+			  gretlopt opt, PRN *prn, int *err)
 {
     double crit, tol = 0.0;
     int met = 0;
 
     if (sys->method == SYS_METHOD_SUR || 
 	sys->method == SYS_METHOD_WLS) {
-	double ll = sur_ll(sys);
+	double ll = sur_loglik(sys);
 
 	tol = SYS_LL_TOL;
 	crit = ll - *llbak;
@@ -755,13 +738,23 @@ static void clean_up_models (equation_system *sys)
     }
 }
 
-static int drop_redundant_instruments (equation_system *sys,
-				       const int *droplist, int i)
-{
-    int j, k, pos, err = 0;
+/* Given a record of the redundant instruments dropped at the tsls
+   stage, namely @idroplist, purge these series IDs from both the
+   global system instrument list (sys->ilist) and the per-equation
+   regression specification list (if the latter is given in tsls
+   form).  
+*/
 
-    for (j=1; j<=droplist[0]; j++) {
-	pos = in_gretl_list(sys->ilist, droplist[j]);
+static int drop_redundant_instruments (equation_system *sys,
+				       const int *idroplist, 
+				       int eqn)
+{
+    int i, j, di, pos;
+    int err = 0;
+
+    for (i=1; i<=idroplist[0]; i++) {
+	di = idroplist[i];
+	pos = in_gretl_list(sys->ilist, di);
 	if (pos > 0) {
 	    gretl_list_delete_at_pos(sys->ilist, pos);
 	} else {
@@ -769,12 +762,16 @@ static int drop_redundant_instruments (equation_system *sys,
 	}
     }
 
-    pos = gretl_list_separator_position(sys->lists[i]);
+    pos = gretl_list_separator_position(sys->lists[eqn]);
+
     if (pos > 0) {
-	for (j=1; j<=droplist[0]; j++) {
-	    for (k=pos+1; k<=sys->lists[i][0]; k++) {
-		if (sys->lists[i][k] == droplist[j]) {
-		    gretl_list_delete_at_pos(sys->lists[i], k);
+	int *eqnlist = sys->lists[eqn];
+
+	for (i=1; i<=idroplist[0]; i++) {
+	    di = idroplist[i];
+	    for (j=pos+1; j<=eqnlist[0]; j++) {
+		if (eqnlist[j] == di) {
+		    gretl_list_delete_at_pos(eqnlist, j);
 		    break;
 		}
 	    }
@@ -801,6 +798,22 @@ static gretlopt sys_tsls_opt (const equation_system *sys)
     return opt;
 }
 
+static int allocate_Xi_etc (gretl_matrix **Xi,
+			    gretl_matrix **Xj,
+			    gretl_matrix **M,
+			    int T, int k)
+{
+    *Xi = gretl_matrix_alloc(T, k);
+    *Xj = gretl_matrix_alloc(T, k);
+    *M = gretl_matrix_alloc(k, k);
+
+    if (*Xi == NULL || *Xj == NULL || *M == NULL) {
+	return E_ALLOC;
+    } else {
+	return 0;
+    }
+}
+
 /* general function that forms the basis for all specific system
    estimators */
 
@@ -821,7 +834,8 @@ int system_estimate (equation_system *sys, DATASET *dset,
     double llbak = -1.0e9;
     int single_equation = 0;
     int do_iteration = 0;
-    int rtsls = 0;
+    int rsingle = 0;
+    int do_diag = 0;
     int err = 0;
 
     sys->iters = 0;
@@ -837,11 +851,19 @@ int system_estimate (equation_system *sys, DATASET *dset,
 	single_equation = 1;
     }
 
-    if (nr > 0 && method == SYS_METHOD_3SLS) {
-	/* doing 3SLS with restrictions: we want to obtain
-	   restricted TSLS estimates as a starting point */
-	rtsls = 1;
-    }
+    if (nr > 0 && !(opt & OPT_U)) {
+	if (method == SYS_METHOD_3SLS) {
+	    /* doing 3SLS with restrictions: we want to obtain
+	       restricted TSLS estimates as a starting point 
+	    */
+	    rsingle = 1;
+	} else if (method == SYS_METHOD_SUR) {
+	    /* doing SUR with restrictions: we want to obtain
+	       restricted OLS estimates as a starting point 
+	    */
+	    rsingle = 1;
+	}	    
+    } 
 
     /* get uniform sample starting and ending points and check for
        missing data */
@@ -891,7 +913,7 @@ int system_estimate (equation_system *sys, DATASET *dset,
     for (i=0; i<sys->neqns; i++) {
 	int freeit = 0;
 	int *list = system_model_list(sys, i, &freeit);
-	const int *droplist = NULL;
+	const int *idroplist = NULL;
 	MODEL *pmod = models[i];
 
 	if (list == NULL) {
@@ -915,9 +937,9 @@ int system_estimate (equation_system *sys, DATASET *dset,
 	    break;
 	} 
 
-	droplist = gretl_model_get_data(pmod, "inst_droplist");
-	if (droplist != NULL) {
-	    drop_redundant_instruments(sys, droplist, i);
+	idroplist = gretl_model_get_data(pmod, "inst_droplist");
+	if (idroplist != NULL) {
+	    drop_redundant_instruments(sys, idroplist, i);
 	}
 
 	pmod->ID = i;
@@ -955,7 +977,7 @@ int system_estimate (equation_system *sys, DATASET *dset,
     */
  iteration_start:
 
-    gls_sigma_from_uhat(sys, sys->S);
+    gls_sigma_from_uhat(sys, sys->S, 0);
 
 #if SDEBUG > 1
     gretl_matrix_print(sys->S, "gls_sigma_from_uhat");
@@ -964,7 +986,7 @@ int system_estimate (equation_system *sys, DATASET *dset,
     if (method == SYS_METHOD_WLS) {
 	gretl_matrix_zero(X);
 	err = gretl_invert_diagonal_matrix(sys->S);
-    } else if (single_equation || rtsls) {
+    } else if (single_equation || rsingle) {
 	gretl_matrix_zero(X);
     } else {
 	err = gretl_invert_symmetric_matrix(sys->S);    
@@ -974,26 +996,15 @@ int system_estimate (equation_system *sys, DATASET *dset,
     fprintf(stderr, "system_estimate: on invert, err=%d\n", err);
 #endif
 
+    if (!err && Xi == NULL) {
+	/* the test against NULL here allows for the possibility
+	   that we're iterating 
+	*/
+	err = allocate_Xi_etc(&Xi, &Xj, &M, T, k);
+    }
+
     if (err) goto cleanup;
 
-    /* the initial tests against NULL here allow for the possibility
-       that we're iterating */
-
-    if (Xi == NULL) {
-	Xi = gretl_matrix_alloc(T, k);
-    }
-    if (Xj == NULL) {
-	Xj = gretl_matrix_alloc(T, k);
-    } 
-    if (M == NULL) {
-	M = gretl_matrix_alloc(k, k);
-    }
-
-    if (Xi == NULL || Xj == NULL || M == NULL) {
-	err = E_ALLOC;
-	goto cleanup;
-    }
-
     /* form the big stacked X matrix: Xi = data matrix for equation i, 
        specified in lists[i] 
     */
@@ -1004,12 +1015,12 @@ int system_estimate (equation_system *sys, DATASET *dset,
 
 	err = make_sys_X_block(Xi, models[i], dset, sys->t1, method);
 
-	for (j=0; j<sys->neqns && !err; j++) { 
+	for (j=0; j<=i && !err; j++) { 
 	    const gretl_matrix *Xk;
 	    double sij;
 
 	    if (i != j) {
-		if (single_equation || rtsls) {
+		if (single_equation || rsingle) {
 		    kcol += models[j]->ncoeff;
 		    continue;
 		}
@@ -1029,13 +1040,13 @@ int system_estimate (equation_system *sys, DATASET *dset,
 					    Xk, GRETL_MOD_NONE, 
 					    M, GRETL_MOD_NONE);
 
-	    if (rtsls || (single_equation && method != SYS_METHOD_WLS)) {
+	    if (rsingle || (single_equation && method != SYS_METHOD_WLS)) {
 		sij = 1.0;
 	    } else {
 		sij = gretl_matrix_get(sys->S, i, j);
 	    }
 
-	    kronecker_place(X, M, krow, kcol, sij);
+	    insert_sys_X_block(X, M, krow, kcol, sij);
 	    kcol += models[j]->ncoeff;
 	}
 
@@ -1052,7 +1063,7 @@ int system_estimate (equation_system *sys, DATASET *dset,
 	augment_X_with_restrictions(X, mk, sys);
     }
 
-    if (!do_iteration && !rtsls) {
+    if (!do_iteration && !rsingle) {
 	/* we're not coming back this way, so free some storage */
 	gretl_matrix_free(Xj);
 	Xj = NULL;
@@ -1074,7 +1085,7 @@ int system_estimate (equation_system *sys, DATASET *dset,
 	    double yv = 0.0;
 	    int lmin = 0, lmax = sys->neqns;
 
-	    if (single_equation || rtsls) {
+	    if (single_equation || rsingle) {
 		/* no cross terms wanted */
 		lmin = i;
 		lmax = i + 1;
@@ -1097,7 +1108,7 @@ int system_estimate (equation_system *sys, DATASET *dset,
 		    xx += gretl_matrix_get(Xi, t, j) * yl[t + sys->t1];
 		}
 
-		if (rtsls || (single_equation && method != SYS_METHOD_WLS)) {
+		if (rsingle || (single_equation && method != SYS_METHOD_WLS)) {
 		    sil = 1.0;
 		} else {
 		    sil = gretl_matrix_get(sys->S, i, l);
@@ -1114,11 +1125,6 @@ int system_estimate (equation_system *sys, DATASET *dset,
 	augment_y_with_restrictions(y, mk, nr, sys);
     }
 
-#if SDEBUG
-    gretl_matrix_print(X, "sys X");
-    gretl_matrix_print(y, "sys y");
-#endif    
-
     /* The estimates calculated below will be SUR, 3SLS or LIML,
        depending on how the data matrices above were constructed --
        unless, that is, we're just doing restricted OLS, WLS or TSLS
@@ -1126,33 +1132,34 @@ int system_estimate (equation_system *sys, DATASET *dset,
     */
     err = calculate_sys_coeffs(sys, dset, X, y, mk, nr, 
 			       do_iteration);
-    if (err) {
-	goto cleanup;
-    }
 
-    if (rtsls) {
-	rtsls = 0;
+    if (!err && rsingle) {
+	/* take one more pass */
+	rsingle = 0;
 	goto iteration_start;
     }
 
-    if (do_iteration) {
-	if (!sys_converged(sys, &llbak, &err, opt, prn)) {
-	    if (err) {
-		goto cleanup;
-	    } else {
+    if (!err && do_iteration) {
+	/* check for convergence */
+	if (!sys_converged(sys, &llbak, opt, prn, &err)) {
+	    if (!err) {
 		sys->iters += 1;
 		goto iteration_start;
 	    }
 	}
     }
 
+    if (err) goto cleanup;
+
     if (nr == 0 && (method == SYS_METHOD_3SLS || method == SYS_METHOD_SUR)) {
 	/* compute this test while we have sigma-inverse available */
 	hansen_sargan_test(sys, dset);
     }
 
+    do_diag = !(method == SYS_METHOD_SUR && do_iteration);
+
     /* refresh sigma (non-inverted) */
-    gls_sigma_from_uhat(sys, sys->S);
+    gls_sigma_from_uhat(sys, sys->S, do_diag);
 
     if (method == SYS_METHOD_FIML) {
 	/* compute FIML estimates */
