@@ -20,6 +20,7 @@
 /* TRAMO/SEATS, X-12-ARIMA plugin for gretl */
 
 #include "libgretl.h"
+#include "version.h"
 #include "estim_private.h"
 
 #include <gtk/gtk.h>
@@ -653,7 +654,7 @@ static int graph_series (const DATASET *dset, tx_request *req)
 	fputs("e\n", fp);
     }
 
-    fputs("set nomultiplot\n", fp);
+    fputs("unset multiplot\n", fp);
 
     gretl_pop_c_numeric_locale();
 
