@@ -22,6 +22,7 @@
 #define FULL_XML_HEADERS
 
 #include "libgretl.h"
+#include "version.h"
 #include "gretl_xml.h"
 #include "csvdata.h"
 #include "importer.h"
@@ -37,6 +38,7 @@
 #include "import_common.c"
 
 #define XDEBUG 0
+#define DATE_DEBUG 0
 
 struct xlsx_info_ {
     int flags;
@@ -49,6 +51,7 @@ struct xlsx_info_ {
     char stringsfile[FILENAME_MAX];
     int n_sheets;
     char **sheetnames;
+    char **filenames;
     int selsheet;
     int n_strings;
     char **strings;
@@ -69,6 +72,7 @@ static void xlsx_info_init (xlsx_info *xinfo)
     xinfo->stringsfile[0] = '\0';
     xinfo->n_sheets = 0;
     xinfo->sheetnames = NULL;
+    xinfo->filenames = NULL;
     xinfo->selsheet = 0;
     xinfo->n_strings = 0;
     xinfo->strings = NULL;
@@ -79,6 +83,7 @@ static void xlsx_info_free (xlsx_info *xinfo)
 {
     if (xinfo != NULL) {
 	free_strings_array(xinfo->sheetnames, xinfo->n_sheets);
+	free_strings_array(xinfo->filenames, xinfo->n_sheets);
 	free_strings_array(xinfo->strings, xinfo->n_strings);
 	destroy_dataset(xinfo->dset);
     }
@@ -92,7 +97,7 @@ static int xlsx_read_shared_strings (xlsx_info *xinfo, PRN *prn)
     xmlNodePtr cur = NULL;
     xmlNodePtr val;
     char *tmp;
-    int n = 0, i = 0;
+    int i, n = 0;
     int err = 0;
 
     err = gretl_xml_open_doc_root(xinfo->stringsfile, "sst", 
@@ -130,6 +135,21 @@ static int xlsx_read_shared_strings (xlsx_info *xinfo, PRN *prn)
 
     cur = cur->xmlChildrenNode;
 
+    /* The strings in an <sst> are mostly set up as 
+
+       <si><t>XXX</t></si>
+       <si><t>YYY</t></si> ...
+
+       But there are also weird cases where junk is interposed
+       and the structure becomes
+
+       <si><r>...<t>XXX</t></r><r>...<t>YYY</t></r></si> ...
+
+       That is, an <si> element may contain more than one <r>
+       element, which embeds a <t> along with formatting crap.
+    */
+
+    i = 0;
     while (cur != NULL && !err) {
 	if (!xmlStrcmp(cur->name, (XUC) "si")) {
 	    int gotstr = 0;
@@ -137,6 +157,7 @@ static int xlsx_read_shared_strings (xlsx_info *xinfo, PRN *prn)
 	    val = cur->xmlChildrenNode;
 	    while (val != NULL && !err && !gotstr) {
 		if (!xmlStrcmp(val->name, (XUC) "t")) {
+		    /* got a regular <t> element */
 		    tmp = (char *) xmlNodeGetContent(val);
 		    if (tmp == NULL) {
 			pprintf(prn, "failed reading string %d\n", i);
@@ -145,6 +166,23 @@ static int xlsx_read_shared_strings (xlsx_info *xinfo, PRN *prn)
 			xinfo->strings[i++] = tmp;
 			gotstr = 1;
 		    }
+		} else if (!xmlStrcmp(val->name, (XUC) "r")) {
+		    /* hunt for <t> inside an <r> element */
+		    xmlNodePtr sub = val->xmlChildrenNode;
+
+		    while (sub != NULL && !err && i < n) {
+			if (!xmlStrcmp(sub->name, (XUC) "t")) {
+			    tmp = (char *) xmlNodeGetContent(sub);
+			    if (tmp == NULL) {
+				pprintf(prn, "failed reading string %d\n", i);
+				err = E_DATA;
+			    } else {
+				xinfo->strings[i++] = tmp;
+				gotstr = 1;
+			    }
+			}
+			sub = sub->next;
+		    }
 		}
 		val = val->next;
 	    }
@@ -175,7 +213,7 @@ static int xlsx_read_shared_strings (xlsx_info *xinfo, PRN *prn)
 
 /* Given the string representation of the shared-string index for 
    a cell with a string value, look up the target string. If the
-   shared strings XML file has not yet been read, reading is 
+   shared strings XML file has not yet been read, its reading is 
    triggered.
 */
 
@@ -194,7 +232,7 @@ static const char *xlsx_string_value (const char *idx, xlsx_info *xinfo,
 
 	if (i >= 0 && i < xinfo->n_strings) {
 	    ret = xinfo->strings[i];
-	}
+	} 
     }
 
     return ret;
@@ -215,14 +253,12 @@ static int letter_val (char c)
 }
 
 /* Given an Excel cell reference such as "AB65" split it out
-   into 1-based row and column indices. If @xinfo is not
-   NULL we keep a running record of the maxima of these 
-   indices in xinfo->maxrow and xinfo->maxcol.
+   into 1-based row and column indices.
 */
 
 static int xlsx_cell_get_coordinates (const char *s, 
-				      xlsx_info *xinfo,
-				      int *row, int *col)
+				      int *row, 
+				      int *col)
 {
     char colref[8];
     int i, k, v;
@@ -252,21 +288,12 @@ static int xlsx_cell_get_coordinates (const char *s,
 	}
 	i--;
     }
-
-    if (!err && xinfo != NULL) {
-	if (*row > xinfo->maxrow) {
-	    xinfo->maxrow = *row;
-	}
-	if (*col > xinfo->maxcol) {
-	    xinfo->maxcol = *col;
-	} 
-    }   
 	
     return err;
 }
 
 static int xlsx_set_varname (xlsx_info *xinfo, int i, const char *s,
-			     PRN *prn)
+			     int row, int col, PRN *prn)
 {
     int err = 0;
 
@@ -283,11 +310,8 @@ static int xlsx_set_varname (xlsx_info *xinfo, int i, const char *s,
     } else {
 	*xinfo->dset->varname[i] = '\0';
 	strncat(xinfo->dset->varname[i], s, VNAMELEN - 1);
-	charsub(xinfo->dset->varname[i], ' ', '_');
-	if (check_varname(xinfo->dset->varname[i])) {
-	    invalid_varname(prn);
-	    err = 1;
-	}
+	err = check_imported_varname(xinfo->dset->varname[i], 
+				     row, col, prn);
     }
 
     return err;
@@ -297,7 +321,7 @@ static void
 xlsx_real_set_obs_string (xlsx_info *xinfo, int t, const char *s)
 {
     *xinfo->dset->S[t] = '\0';
-    strncat(xinfo->dset->S[t], s, OBSLEN - 1);
+    gretl_utf8_strncat(xinfo->dset->S[t], s, OBSLEN - 1);
 }
 
 static int xlsx_set_obs_string (xlsx_info *xinfo, int t, const char *s)
@@ -363,7 +387,7 @@ static int xlsx_handle_stringval (const char *s, int r, int c,
 	return 0; /* OK */
     } else {
 	pprintf(prn, _("Expected numeric data, found string:\n"
-		       "%s\" at row %d, column %d\n"), s, r, c);
+		       "'%s' at row %d, column %d\n"), s, r, c);
 	return E_DATA;
     }
 }
@@ -378,7 +402,7 @@ static int xlsx_var_index (xlsx_info *xinfo, int col)
 	i++; /* skip the constant in position 0 */
     }
 
-#if XDEBUG
+#if XDEBUG > 2
     fprintf(stderr, "xlsx_var_index: labels = %d, col = %d, i = %d\n", 
 	    (xinfo->flags & BOOK_OBS_LABELS)? 1 : 0, col, i);
 #endif
@@ -394,7 +418,7 @@ static int xlsx_obs_index (xlsx_info *xinfo, int row)
 	t--; /* the first row holds varnames */
     }
 
-#if XDEBUG
+#if XDEBUG > 2
     fprintf(stderr, "xlsx_obs_index: varnames = %d, row = %d, t = %d\n", 
 	    (xinfo->flags & BOOK_AUTO_VARNAMES)? 0 : 1, row, t);
 #endif
@@ -412,6 +436,10 @@ static void xlsx_check_top_left (xlsx_info *xinfo, int r, int c,
 	   varname, could hold "obs" or similar, or could
 	   be the first numerical value.
 	*/
+#if XDEBUG
+	fprintf(stderr, "xlsx_check_top_left: r=%d, c=%d, x=%g, stringcell=%d, "
+		"s='%s'\n", r, c, x, stringcell, s);
+#endif
 	if (!na(x)) {
 	    /* numerical value */
 	    xinfo->flags |= BOOK_AUTO_VARNAMES;
@@ -419,8 +447,10 @@ static void xlsx_check_top_left (xlsx_info *xinfo, int r, int c,
 	    /* blank or "obs" or similar */
 	    xinfo->flags |= BOOK_OBS_LABELS;
 	}
-	/* record the fact that the top-left corner is not empty */
-	xinfo->flags &= ~BOOK_TOP_LEFT_EMPTY;
+	if (!na(x) || stringcell) {
+	    /* record the fact that the top-left corner is not empty */
+	    xinfo->flags &= ~BOOK_TOP_LEFT_EMPTY;
+	}
     } else if (r == xinfo->yoffset + 1 && c == xinfo->xoffset + 2) {
 	/* first row, second column */
 	if (!na(x)) {
@@ -454,7 +484,7 @@ static void xlsx_maybe_handle_formula (xlsx_info *xinfo,
 		return;
 	    } 
 
-	    err = xlsx_cell_get_coordinates(cref, NULL, &st, &col);
+	    err = xlsx_cell_get_coordinates(cref, &st, &col);
 	    if (err || col != xinfo->xoffset + 1) {
 		return;
 	    }
@@ -478,6 +508,17 @@ static void xlsx_maybe_handle_formula (xlsx_info *xinfo,
     }
 }
 
+static void xlsx_set_dims (xlsx_info *xinfo, int r, int c)
+{
+    if (r > xinfo->maxrow) {
+	xinfo->maxrow = r;
+    }
+
+    if (c > xinfo->maxcol) {
+	xinfo->maxcol = c;
+    } 
+}
+
 /* Read the cells in a given row: the basic info we want from
    each cell is its reference ("r"), e.g. "A2"; its type
    ("t"), e.g. "s", if present; and its value, which is
@@ -489,12 +530,15 @@ static int xlsx_read_row (xmlNodePtr cur, xlsx_info *xinfo, PRN *prn)
     PRN *myprn = NULL;
     xmlNodePtr val;
     char *tmp;
-    int row, col;
+    int row = -1, col = -1;
+    int pass, empty = 1;
     int err = 0;
 
+    pass = xinfo->dset == NULL ? 1 : 2;
+
 #if XDEBUG
     myprn = prn;
-    pprintf(myprn, "*** Reading row...\n");    
+    pprintf(myprn, "*** Reading row (pass %d)...\n", pass);
 #endif
 
     cur = cur->xmlChildrenNode;
@@ -518,13 +562,17 @@ static int xlsx_read_row (xmlNodePtr cur, xlsx_info *xinfo, PRN *prn)
 		break;
 	    } 
 
-	    err = xlsx_cell_get_coordinates(cref, xinfo, &row, &col);
+	    err = xlsx_cell_get_coordinates(cref, &row, &col);
 	    if (err) {
 		pprintf(myprn, ": couldn't find coordinates\n", row, col);
 	    } else {
 		pprintf(myprn, "(%d, %d)", row, col);
 	    }
 
+	    if (pass == 2 && row > xinfo->maxrow) {
+		goto skipit;
+	    }
+
 	    tmp = (char *) xmlGetProp(cur, (XUC) "t");
 	    if (tmp != NULL) {
 		if (!strcmp(tmp, "s")) {
@@ -574,7 +622,7 @@ static int xlsx_read_row (xmlNodePtr cur, xlsx_info *xinfo, PRN *prn)
 	    if (err) {
 		pprintf(myprn, ": (%s) error", cref);
 	    } else if (!gotv) {
-		pprintf(myprn, ": (%s) no data", cref);
+		pprintf(myprn, ": (%s) no data value", cref);
 		if (gotf) {
 		    pprintf(myprn, "; formula = '%s'\n", formula);
 		} else {
@@ -590,7 +638,7 @@ static int xlsx_read_row (xmlNodePtr cur, xlsx_info *xinfo, PRN *prn)
 
 		if (stringcell) {
 		    if (row == xinfo->yoffset + 1) {
-			err = xlsx_set_varname(xinfo, i, strval, prn);
+			err = xlsx_set_varname(xinfo, i, strval, row, col, prn);
 		    } else if (col == xinfo->xoffset + 1) {
 			err = xlsx_set_obs_string(xinfo, t, strval);
 		    } else if (strval != NULL) {
@@ -603,12 +651,26 @@ static int xlsx_read_row (xmlNodePtr cur, xlsx_info *xinfo, PRN *prn)
 		}
 	    }
 
+	    if (gotv || gotf) {
+		empty = 0;
+	    }
+
+	skipit:
+
 	    free(cref);
 	    free(formula);
 	}
 	cur = cur->next;
     }
 
+    if (!err) {
+	if (empty) {
+	    pputs(myprn, " xlsx_read_row: empty row!\n");
+	} else if (pass == 1) {
+	    xlsx_set_dims(xinfo, row, col);
+	}
+    }
+
     if (err) {
 	fprintf(stderr, "xlsx_read_row: returning %d\n", err);
     }
@@ -636,10 +698,13 @@ static int xlsx_check_dimensions (xlsx_info *xinfo, PRN *prn)
 	n--;
     }
 
+    if (v < 0) v = 0;
+    if (n < 0) n = 0;
+
     pprintf(prn, "Found %d variables and %d observations\n", v, n);
 
-    if (v <= 0 || n <= 0) {
-	pputs(prn, "File contains no data");
+    if (v == 0 || n == 0) {
+	pputs(prn, "File contains no data\n");
 	err = E_DATA;
     } else {
 	int labels = (xinfo->flags & BOOK_OBS_LABELS);
@@ -669,8 +734,13 @@ static int xlsx_read_worksheet (xlsx_info *xinfo, PRN *prn)
     int gotdata = 0;
     int err = 0;
 
-    sprintf(xinfo->sheetfile, "xl%cworksheets%c%s.xml", 
-	    SLASH, SLASH, xinfo->sheetnames[xinfo->selsheet]);
+    sprintf(xinfo->sheetfile, "xl%c%s", SLASH, 
+	    xinfo->filenames[xinfo->selsheet]);
+
+#if XDEBUG
+    fprintf(stderr, "xlsx_read_worksheet: sheetnum=%d, name='%s'\n",
+	    xinfo->selsheet, xinfo->filenames[xinfo->selsheet]);
+#endif
 
     sprintf(xinfo->stringsfile, "xl%csharedStrings.xml", SLASH);
 
@@ -699,11 +769,13 @@ static int xlsx_read_worksheet (xlsx_info *xinfo, PRN *prn)
 	cur = cur->next;
     }
 
+#if XDEBUG
     if (!err) {
-	pprintf(prn, "\nMax row = %d, max col = %d\n", xinfo->maxrow,
+	pprintf(prn, "Max row = %d, max col = %d\n", xinfo->maxrow,
 		xinfo->maxcol);
 	pprintf(prn, "Accessed %d shared strings\n", xinfo->n_strings);
     }
+#endif
 
     if (!err && xinfo->dset == NULL) {
 	err = xlsx_check_dimensions(xinfo, prn);
@@ -725,6 +797,9 @@ static int xlsx_read_worksheet (xlsx_info *xinfo, PRN *prn)
     return err;
 }
 
+/* A quick check to see if a worksheet XML file contains
+   any actual data. */
+
 static int xlsx_sheet_has_data (const char *fname)
 {
     xmlDocPtr doc = NULL;
@@ -732,8 +807,7 @@ static int xlsx_sheet_has_data (const char *fname)
     gchar *fullname;
     int err, ret = 0;
 
-    fullname = g_strdup_printf("xl%cworksheets%c%s",
-			       SLASH, SLASH, fname);
+    fullname = g_strdup_printf("xl%c%s", SLASH, fname);
 
     err = gretl_xml_open_doc_root(fullname, "worksheet", 
 				  &doc, &cur);
@@ -754,51 +828,289 @@ static int xlsx_sheet_has_data (const char *fname)
 	xmlFreeDoc(doc);
     }
 
+    if (!ret) {
+	fprintf(stderr, "%s: contains no data\n", fname);
+    }
+
     g_free(fullname);
 
     return ret;
 }
 
-static int xlsx_gather_sheet_names (xlsx_info *xinfo, PRN *prn)
+static int xlsx_workbook_get_sheetnames (xlsx_info *xinfo,
+					 const char *fname)
 {
-    gchar *dname = g_strdup_printf("xl%cworksheets", SLASH);
-    DIR *dir = gretl_opendir(dname);
-    int i, err = 0;
+    xmlDocPtr doc = NULL;
+    xmlNodePtr c1, cur = NULL;
+    char *ID, *sheetname;
+    int ns = 0, found = 0;
+    int err;
 
-    if (dir == NULL) {
-	err = E_FOPEN;
-    } else {
-	struct dirent *dirent;
-
-	while ((dirent = readdir(dir)) != NULL) {
-	    const char *basename = dirent->d_name;
-	    
-	    if (has_suffix(basename, ".xml") && 
-		xlsx_sheet_has_data(basename)) {
-		gchar *tmp = g_strdup(basename);
-		gchar *p = strstr(tmp, ".xml");
-
-		*p = '\0';
-		strings_array_add(&xinfo->sheetnames, &xinfo->n_sheets,
-				  tmp);
-		g_free(tmp);
+    err = gretl_xml_open_doc_root(fname, "workbook", 
+				  &doc, &cur);
+    if (!err) {
+	cur = cur->xmlChildrenNode;
+	while (cur != NULL && !found) {
+	    if (!xmlStrcmp(cur->name, (XUC) "sheets")) {
+		c1 = cur->xmlChildrenNode;
+		while (c1 != NULL) {
+		    if (!xmlStrcmp(c1->name, (XUC) "sheet")) {
+			ID = (char *) xmlGetProp(c1, (XUC) "id");
+			sheetname = (char *) xmlGetProp(c1, (XUC) "name");
+			if (ID != NULL && sheetname != NULL) {
+			    strings_array_add(&xinfo->sheetnames, 
+					      &xinfo->n_sheets,
+					      sheetname);
+			    strings_array_add(&xinfo->filenames, 
+					      &ns, ID);
+			}
+			free(ID);
+			free(sheetname);
+		    }
+		    c1 = c1->next;
+		}
+		found = 1;
 	    }
+	    cur = cur->next;
 	}
-	closedir(dir);
+	xmlFreeDoc(doc);
     }
 
-    g_free(dname);
+    return err;
+}
 
-    if (xinfo->n_sheets == 0) {
+static void xlsx_expunge_sheet (xlsx_info *xinfo, int i)
+{
+    int j;
+
+    free(xinfo->sheetnames[i]);
+    free(xinfo->filenames[i]);
+
+    for (j=i; j<xinfo->n_sheets-1; j++) {
+	xinfo->sheetnames[j] = xinfo->sheetnames[j+1];
+	xinfo->filenames[j] = xinfo->filenames[j+1];
+    };
+
+    xinfo->n_sheets -= 1;
+}
+
+static int xlsx_match_sheet_id (xlsx_info *xinfo, 
+				const char *ID)
+{
+    int i;
+
+    if (ID == NULL) {
+	return -1;
+    }
+
+    for (i=0; i<xinfo->n_sheets; i++) {
+	if (!strcmp(ID, xinfo->filenames[i])) {
+	    return i;
+	}
+    }
+
+    return -1;
+}
+
+/* For the sheet names we got from the main workbook file:
+   check that (1) we can track down the corresponding
+   worksheet filename and (2) the worksheet file contains
+   some data.
+*/
+
+static int xlsx_verify_sheets (xlsx_info *xinfo, PRN *prn)
+{
+    xmlDocPtr doc = NULL;
+    xmlNodePtr cur = NULL;
+    char *checker;
+    char *ID, *fname;
+    int i, err;
+
+    checker = calloc(xinfo->n_sheets, 1);
+    if (checker == NULL) {
+	return E_ALLOC;
+    }
+
+    err = gretl_xml_open_doc_root("xl/_rels/workbook.xml.rels",
+				  "Relationships",
+				  &doc, &cur);
+    if (!err) {
+	cur = cur->xmlChildrenNode;
+	while (cur != NULL) {
+	    if (!xmlStrcmp(cur->name, (XUC) "Relationship")) {
+		ID = (char *) xmlGetProp(cur, (XUC) "Id");
+		if ((i = xlsx_match_sheet_id(xinfo, ID)) >= 0) {
+		    fname = (char *) xmlGetProp(cur, (XUC) "Target");
+		    if (fname != NULL) {
+			if (xlsx_sheet_has_data(fname)) {
+			    checker[i] = 1;
+			    free(xinfo->filenames[i]);
+			    xinfo->filenames[i] = fname;
+			} else {
+			    free(fname);
+			}
+		    }
+		}
+		free(ID);
+	    }
+	    cur = cur->next;
+	}
+	xmlFreeDoc(doc);
+    }
+    
+    if (!err) {
+	int j = 0;
+
+	for (i=0; i<xinfo->n_sheets; i++) {
+	    if (checker[j++] == 0) {
+		fprintf(stderr, "dropping sheet '%s'\n", 
+			xinfo->sheetnames[i]);
+		xlsx_expunge_sheet(xinfo, i--);
+	    }
+	}
+    }
+
+    free(checker);
+
+    return err;
+}
+
+static int xlsx_verify_specific_sheet (xlsx_info *xinfo, 
+				       int idx, PRN *prn)
+{
+    xmlDocPtr doc = NULL;
+    xmlNodePtr cur = NULL;
+    char *ID, *fname;
+    int err;
+
+    err = gretl_xml_open_doc_root("xl/_rels/workbook.xml.rels",
+				  "Relationships",
+				  &doc, &cur);
+    if (!err) {
+	const char *sname = xinfo->sheetnames[idx];
+	int found = 0;
+
+	cur = cur->xmlChildrenNode;
+	while (cur != NULL && !found) {
+	    if (!xmlStrcmp(cur->name, (XUC) "Relationship")) {
+		ID = (char *) xmlGetProp(cur, (XUC) "Id");
+		if (xlsx_match_sheet_id(xinfo, ID) == idx) {
+		    found = 1;
+		    fname = (char *) xmlGetProp(cur, (XUC) "Target");
+		    if (fname == NULL) {
+			pprintf(prn, "'%s': couldn't find filename\n", sname);
+			err = E_DATA;
+		    } else if (xlsx_sheet_has_data(fname)) {
+			pprintf(prn, "'%s' -> %s\n", sname, fname);
+			free(xinfo->filenames[idx]);
+			xinfo->filenames[idx] = fname;
+		    } else {
+			pprintf(prn, "'%s': contains no data\n", sname);
+			err = E_DATA;
+			free(fname);
+		    }
+		}
+		free(ID);
+	    }
+	    cur = cur->next;
+	}
+
+	xmlFreeDoc(doc);
+
+	if (!found) {
+	    pprintf(prn, "'%s': couldn't find file Id\n", sname);
+	    err = E_DATA;
+	}
+
+	if (!err) {
+	    /* record the pre-checked sheet selection */
+	    xinfo->selsheet = idx;
+	}
+    }
+
+    return err;
+}
+
+static int xlsx_gather_sheet_names (xlsx_info *xinfo, 
+				    const char *insheet,
+				    int *list, 
+				    PRN *prn)
+{
+    gchar *wb_name = g_strdup_printf("xl%cworkbook.xml", SLASH); 
+    int i, err;
+
+    err = xlsx_workbook_get_sheetnames(xinfo, wb_name);
+    g_free(wb_name);
+
+    if (!err && xinfo->n_sheets == 0) {
+	pputs(prn, "Found no sheets\n");
 	err = E_DATA;
-    } else if (xinfo->n_sheets > 1) {
-	strings_array_sort(&xinfo->sheetnames, &xinfo->n_sheets,
-			   OPT_NONE);
     }
 
     if (!err) {
+	const char *sname;
+	int have_insheet_name = 0;
+	int target_sheet_number = 0;
+	int insheet_idx = 0;
+
+	if (insheet != NULL && *insheet != '\0') {
+	    /* try looking for a worksheet specified by name */
+	    have_insheet_name = 1;
+	    insheet_idx = -1; /* invalidate index */
+	} else if (list != NULL && list[0] >= 1 && list[1] > 0) {
+	    /* or sheet specified by (1-based) sequence number */
+	    target_sheet_number = list[1];
+	    insheet_idx = -1; /* invalidate index */
+	}
+
+	fprintf(stderr, "Found these worksheets:\n");
 	for (i=0; i<xinfo->n_sheets; i++) {
-	    fprintf(stderr, "%d: %s\n", i, xinfo->sheetnames[i]);
+	    sname = xinfo->sheetnames[i];
+	    fprintf(stderr, "%d: '%s' (%s)\n", i+1, sname, xinfo->filenames[i]);
+	    if (have_insheet_name) {
+		if (insheet_idx < 0 && !strcmp(insheet, sname)) {
+		    /* found the name we were looking for */
+		    insheet_idx = i;
+		}
+	    } else if (target_sheet_number == i + 1) {
+		/* found the number we were looking for */
+		insheet_idx = i;
+	    }
+	}
+
+	if (insheet_idx < 0) {
+	    /* a sheet was pre-specified but was not found */
+	    if (have_insheet_name && integer_string(insheet)) {
+		/* try interpreting as plain integer index? */
+		insheet_idx = atoi(insheet) - 1;
+		if (insheet_idx < 0 || insheet_idx >= xinfo->n_sheets) {
+		    /* no, it doesn't work */
+		    insheet_idx = -1;
+		} 
+	    }
+	    if (insheet_idx < 0) {
+		if (have_insheet_name) {
+		    gretl_errmsg_sprintf(_("'%s': no such worksheet"), insheet);
+		} else {
+		    gretl_errmsg_set(_("Invalid argument for worksheet import"));
+		}
+		err = E_DATA;
+	    }
+	}
+
+	if (!err) {
+	    if (have_insheet_name || target_sheet_number > 0) {
+		/* avoid the expense of checking all sheets */
+		err = xlsx_verify_specific_sheet(xinfo, insheet_idx, prn);
+	    } else {
+		err = xlsx_verify_sheets(xinfo, prn);
+		if (xinfo->n_sheets == 0) {
+		    pputs(prn, "\nFound no valid sheets\n");
+		    err = E_DATA;
+		} else {
+		    pprintf(prn, "\nFound %d valid sheet(s)\n", xinfo->n_sheets);
+		}
+	    }
 	}
     }
 
@@ -823,118 +1135,30 @@ static void record_xlsx_params (xlsx_info *xinfo, int *list)
     }
 }
 
-static void xlxs_seek_selsheet_by_name (xlsx_info *xinfo, 
-					const char *name)
+/* When we're called from the command-line, the @list
+   argument may contain a sheet number followed by
+   x and y offsets. If a sheet number was given, it is
+   already handled by this point: here we check and
+   record the offsets, if present.
+*/
+
+static int set_xlsx_offsets_from_cli (xlsx_info *xinfo, 
+				      const int *list)
 {
-    xmlDocPtr doc = NULL;
-    xmlNodePtr cur = NULL;
-    xmlNodePtr val;
-    gchar *fname;
-    char *xname = NULL;
-    char *xid = NULL;
-    int gotit = 0;
-    int err;
+    int err = 0;
 
-    fname = g_strdup_printf("xl%cworkbook.xml", SLASH);
-    err = gretl_xml_open_doc_root(fname, "workbook", 
-				  &doc, &cur);
-    g_free(fname);
-    if (err) {
-	fprintf(stderr, "couldn't open workbook.xml\n");
-	return;
-    }
+    if (list != NULL && list[0] == 3) {
+	int xoff = list[2];
+	int yoff = list[3];
 
-    cur = cur->xmlChildrenNode;
-
-    while (cur != NULL && !gotit) {
-	if (!xmlStrcmp(cur->name, (XUC) "sheets")) {
-	    val = cur->xmlChildrenNode;
-	    while (val != NULL && !gotit) {
-		if (!xmlStrcmp(val->name, (XUC) "sheet")) {
-		    xname = (char *) xmlGetProp(val, (XUC) "name");
-		    if (xname != NULL) {
-			if (!strcmp(xname, name)) {
-			    xid = (char *) xmlGetProp(val, (XUC) "sheetId");
-			    gotit = 1;
-			} else {
-			    free(xname);
-			}
-		    }
-		}
-		val = val->next;
-	    }
+	if (xoff < 0 || yoff < 0) {
+	    gretl_errmsg_set(_("Invalid argument for worksheet import"));
+	    err = E_DATA;
+	} else{
+	    xinfo->xoffset = xoff;
+	    xinfo->yoffset = yoff;
 	}
-	cur = cur->next;
-    }
-
-    xmlFreeDoc(doc);
-
-    if (xname != NULL && xid != NULL) {
-	int i, j, k = atoi(xid);
-
-	for (i=0; i<xinfo->n_sheets; i++) {
-	    sscanf(xinfo->sheetnames[i], "sheet%d", &j);
-	    if (k == j) {
-		xinfo->selsheet = i;
-		break;
-	    }
-	}
-    }
-
-    free(xname);
-    free(xid);
-}
-
-static int set_xlsx_params_from_cli (xlsx_info *xinfo, 
-				     const int *list,
-				     char *sheetname)
-{
-    int gotname = (sheetname != NULL && *sheetname != '\0');
-    int gotlist = (list != NULL && list[0] == 3);
-    int i, err = 0;
-
-    if (!gotname && !gotlist) {
-	xinfo->selsheet = 0;
-	xinfo->xoffset = 0;
-	xinfo->yoffset = 0;
-	return 0;
-    }
-
-    /* invalidate this */
-    xinfo->selsheet = -1;
-
-    if (gotname) {
-	for (i=0; i<xinfo->n_sheets; i++) {
-	    if (!strcmp(sheetname, xinfo->sheetnames[i])) {
-		xinfo->selsheet = i;
-		break;
-	    }
-	}
-	if (xinfo->selsheet < 0 && integer_string(sheetname)) {
-	    i = atoi(sheetname);
-	    if (i >= 1 && i <= xinfo->n_sheets) {
-		xinfo->selsheet = i - 1;
-	    }
-	}
-	if (xinfo->selsheet < 0) {
-	    xlxs_seek_selsheet_by_name(xinfo, sheetname);
-	}
-    }
-
-    if (gotlist) {
-	if (!gotname) {
-	    /* convert to zero-based */
-	    xinfo->selsheet = list[1] - 1;
-	}
-	xinfo->xoffset = list[2];
-	xinfo->yoffset = list[3];
-    }
-
-    if (xinfo->selsheet < 0 || xinfo->selsheet >= xinfo->n_sheets ||
-	xinfo->xoffset < 0 || xinfo->yoffset < 0) {
-	gretl_errmsg_set(_("Invalid argument for worksheet import"));
-	err = E_DATA;
-    }
+    } 
 
     return err;
 }
@@ -956,6 +1180,11 @@ static int xlsx_sheet_dialog (xlsx_info *xinfo)
 	xinfo->selsheet = 0;
     }
 
+#if XDEBUG
+    fprintf(stderr, "xlsx_sheet_dialog: selected=%d, xoff=%d, yoff=%d\n",
+	    book.selected, book.col_offset, book.row_offset);
+#endif    
+
     xinfo->xoffset = book.col_offset;
     xinfo->yoffset = book.row_offset;
 
@@ -967,6 +1196,10 @@ static void xlsx_dates_check (DATASET *dset)
     int t, maybe_dates = 1;
     int d, dmin = 0, dmax = 0;
 
+#if DATE_DEBUG
+    fprintf(stderr, "xlsx_dates_check: starting\n");
+#endif
+
     /* We're dealing here with the case where our prior heuristics
        suggest we got an "observations" column, yet the values we
        found there were numeric (and we converted them to strings).
@@ -976,12 +1209,14 @@ static void xlsx_dates_check (DATASET *dset)
        For this purpose we'll require that all the obs labels are 
        integer strings, and that the gap between successive values
        should be constant, or variable to a degree that's consistent
-       with the data being daily (where the mininum gap is 1 day
-       but weekends and holidays might be skipped).
+       with a sane time-series frequency.
     */
 
     for (t=0; t<dset->n && maybe_dates; t++) {
 	if (!integer_string(dset->S[t])) {
+#if DATE_DEBUG
+	    fprintf(stderr, "S[%d] = '%s', giving up\n", t, dset->S[t]);
+#endif
 	    maybe_dates = 0;
 	} else if (t == 0) {
 	    if (!strcmp(dset->S[0], "1")) {
@@ -990,8 +1225,15 @@ static void xlsx_dates_check (DATASET *dset)
 	} else {
 	    d = atoi(dset->S[t]) - atoi(dset->S[t-1]);
 	    if (t == 1) {
+#if DATE_DEBUG
+		fprintf(stderr, " t=1, setting dmin = dmax = %d\n", d);
+#endif		
 		dmin = dmax = d;
 	    } else if (d < dmin) {
+#if DATE_DEBUG
+		fprintf(stderr, " at t=%d, dmin = %d - %d = %d\n",
+			t, atoi(dset->S[t]), atoi(dset->S[t-1]), d);
+#endif
 		dmin = d;
 	    } else if (d > dmax) {
 		dmax = d;
@@ -999,24 +1241,45 @@ static void xlsx_dates_check (DATASET *dset)
 	}
     }
 
-    if (maybe_dates) {
-	if (dmin == 1 || dmax == -1) {
-	    /* daily (may be in reverse order)? We may have some gaps
-	       but they shouldn't be too big */
-	    if (dmin == 1 && dmax > 5) {
-		maybe_dates = 0;
-	    } else if (dmax == -1 && dmin < -5) {
-		maybe_dates = 0;
-	    }
-	} else if (dmax != dmin) {
-	    /* non-daily: the gaps should not be variable */
-	    maybe_dates = 0;
-	} else if (dmax > 365 || dmax < -365) {
-	    /* gap greater than a year */
-	    maybe_dates = 0;
-	}
+#if DATE_DEBUG
+    fprintf(stderr, "after obs loop, maybe_dates = %d (dmax = %d)\n", 
+	    maybe_dates, dmax);
+#endif
+
+    if (maybe_dates && dmax < 0) {
+	/* allow for the possibility that time runs backwards */
+	int tmp = dmin;
+
+	dmin = -dmax;
+	dmax = -tmp;
+	fprintf(stderr, "xlsx_dates_check: diffmin=%d, diffmax=%d\n", 
+		dmin, dmax);
     }
 
+    if (maybe_dates) {
+	if (dmin >= 364 && dmax <= 365) {
+	    ; /* annual? */
+	} else if (dmin >= 90 && dmax <= 92) {
+	    ; /* quarterly? */
+	} else if (dmin >= 28 && dmax <= 31) {
+	    ; /* monthly? */
+	} else if (dmin == 7 && dmax == 7) {
+	    ; /* weekly? */
+	} else if (dmin == 1 && dmax <= 5) {
+	    ; /* daily? */
+	} else {
+	    /* unsupported frequency or nonsensical */
+#if DATE_DEBUG
+	    fprintf(stderr, "dmax = %d, dmin = %d, unsupported\n", dmax, dmin);
+#endif
+	    maybe_dates = 0;
+	} 
+    }
+
+#if DATE_DEBUG
+    fprintf(stderr, "xlsx_dates_check: maybe_dates = %d\n", maybe_dates);
+#endif
+
     if (maybe_dates) {
 	char datestr[OBSLEN];
 
@@ -1050,6 +1313,10 @@ static int finalize_xlsx_import (DATASET *dset,
 	}
     }
 
+    if (!err && fix_varname_duplicates(xinfo->dset)) {
+	pputs(prn, _("warning: some variable names were duplicated\n"));
+    }
+
     if (!err && xinfo->trydates) {
 	xlsx_dates_check(xinfo->dset);
     }
@@ -1069,10 +1336,23 @@ static int finalize_xlsx_import (DATASET *dset,
     return err;
 }
 
+/* Public driver function for retrieving data from OOXML workbook
+   file.
+
+   If we're coming from the command line @sheetname may be non-NULL
+   and/or @list may contain specification of sheet number (1-based),
+   row offset and/or column offset.
+
+   If we're coming from the GUI @sheetname will be NULL but @list will
+   be non-NULL, and is used to record choices made here via the
+   function xlsx_sheet_dialog().
+*/
+
 int xlsx_get_data (const char *fname, int *list, char *sheetname,
 		   DATASET *dset, gretlopt opt, PRN *prn)
 {
     int gui = (opt & OPT_G);
+    char *save_errmsg = NULL;
     xlsx_info xinfo;
     char dname[32];
     int err;
@@ -1082,11 +1362,14 @@ int xlsx_get_data (const char *fname, int *list, char *sheetname,
 	return err;
     }
 
+#if XDEBUG
+    fprintf(stderr, "xlsx_get_data: sheet='%s'\n", sheetname);
+    printlist(list, "xlsx list");
+#endif
+
     xlsx_info_init(&xinfo);
 
-    if (!err) {
-	err = xlsx_gather_sheet_names(&xinfo, prn);
-    }
+    err = xlsx_gather_sheet_names(&xinfo, sheetname, list, prn);
 
     if (!err) {
 	if (gui) {
@@ -1098,7 +1381,7 @@ int xlsx_get_data (const char *fname, int *list, char *sheetname,
 		goto bailout;
 	    } 
 	} else {
-	    err = set_xlsx_params_from_cli(&xinfo, list, sheetname);
+	    err = set_xlsx_offsets_from_cli(&xinfo, list);
 	} 
     }
 
@@ -1106,8 +1389,6 @@ int xlsx_get_data (const char *fname, int *list, char *sheetname,
 	err = xlsx_read_worksheet(&xinfo, prn);
     }
 
- bailout:
-
     if (!err && xinfo.dset != NULL) {
 	err = finalize_xlsx_import(dset, &xinfo, fname, opt, prn); 
 	if (!err && gui) {
@@ -1115,11 +1396,24 @@ int xlsx_get_data (const char *fname, int *list, char *sheetname,
 	}
     }
 
-    gretl_print_flush_stream(prn);
+ bailout:
 
+    gretl_print_flush_stream(prn);
     xlsx_info_free(&xinfo);
 
+    if (err > 0) {
+	/* preserve any error message so it's not overwritten
+	   by remove_temp_dir();
+	*/
+	save_errmsg = maybe_save_gretl_errmsg(err);
+    }
+
     remove_temp_dir(dname);
 
+    if (save_errmsg != NULL) {
+	gretl_errmsg_set(save_errmsg);
+	free(save_errmsg);
+    }
+
     return err;
 }
